;$Id: ac2gt.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;      AC2GT
;
; PURPOSE:
;      Returns single precision AC (aircraft) to EARTH or EARTH to AC 
;      coordinate transformation matrix. AC coordinates (x',y',z'): along plane
;      axis, right wing, down.  Earth(x,y,z): East, North, Up; positive angles
;      are counterclockwise. 
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;      res=ac2gt(attitude)
;      res=ac2gt(roll,pitch,heading)
;
; INPUTS:
;      Attitude: fltarr(3,m)
;                attitude[0,m], AC roll in radians
;                attitude[1,m], AC pitch in radians
;                attitude[2,m], AC heading in radians
;
;      Instead of attitude you can enter roll, pitch and heading as
;      separate arrays:
;     
;      Roll:    fltarr(m), AC roll in radians 
;      Pitch:   fltarr(m), AC pitch in radians 
;      Heading: fltarr(m), AC heading in radians 
;
; KEYWORD PARAMETERS:
;      DEG:  when set the attitude angles are given in degrees
;      G2A:  when set ac2gt returns ground to aircraft trans. matrix
;            otherwise returns aircraft to ground trans. matrix (default)
;      HELP: show this text
;
; OUTPUTS:
;      Res:  fltarr(3,3,m), array of transform. matrices for m sets of angles
;
; PROCEDURE:
;      The transformation matrix is given by:
;      AC to Earth:  Ta2g=Tz##(Th##(Tp##Tr)) and since T is a orthonormal matrix
;      Earth to AC transformation matrix is Tg2a=transpose(Ta2g)
;      Where Tr is the AC roll(phi) matrix, Tp - pitch(theta) matrix,
;      Th - heading(psi) matrix, and Tz - down to up (z) rotation :
;
;      Tr=[[1,0,0],[0,cos(phi),-sin(phi)],[0,sin(phi),cos(phi)]]
;      Tp=[[cos(theta),0,sin(theta)],[0,1,0],[-sin(theta),0,cos(theta)]]
;      Th=[[sin(psi),cos(psi),0],[-cos(psi),sin(psi),0],[0,0,1]]
;      Tz=[[1,0,0],[0,-1,0],[0,0,-1]] 
;
; NOTES:
;      AC coordinate system:  x' -> AC axis toward the nose
;                             y' -> right wing
;                             z' -> AC down
;      AC heading, psi,   (North=0, N->E positive)      -> rotate z' 
;      AC pitch,   theta, (AC nose up is positive)      -> rotate y'
;      AC roll,    phi,   (right wing down is positive) -> rotate x'  
;
; SIDE EFFECTS:
;      The function returns -1 if the usage is incorrect
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov, December 1999
;      S.H., Apr 00 -- added deg keyword (optimized for memory)
;      S.H., Aug 00 -- stupid bug fixed
;      S.H., Mar 01 -- minor change (speeded up the calculations)
;      S.H., Feb 03 -- changed routine name from kaaggat to ac2gt
;      S.H., Jul 03 -- few minor changes that don't not matter much
;      S.H., Oct 03 -- minor changes in the naming of attitude angles
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function ac2gt, roll,pitch,heading, deg=deg,g2a=g2a,help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'ac2gt'
  return,''
endif

; Check the input and prepare triginometric quantities

if n_params() eq 1 then begin
  if keyword_set(deg) then begin 
    sr=reform(sin(roll[0,*]*!DTOR)) & cr=reform(cos(roll[0,*]*!DTOR)) ; roll 
    sp=reform(sin(roll[1,*]*!DTOR)) & cp=reform(cos(roll[1,*]*!DTOR)) ; pitch
    sh=reform(sin(roll[2,*]*!DTOR)) & ch=reform(cos(roll[2,*]*!DTOR)) ; heading
  endif else begin
    sr=reform(sin(roll[0,*])) & cr=reform(cos(roll[0,*])) ; roll 
    sp=reform(sin(roll[1,*])) & cp=reform(cos(roll[1,*])) ; pitch
    sh=reform(sin(roll[2,*])) & ch=reform(cos(roll[2,*])) ; heading
  endelse
endif else if n_params() eq 3 then begin
  if keyword_set(deg) then begin
     sr=sin(roll*!DTOR)    & cr=cos(roll*!DTOR)
     sp=sin(pitch*!DTOR)   & cp=cos(pitch*!DTOR)
     sh=sin(heading*!DTOR) & ch=cos(heading*!DTOR)
  endif else begin
     sr=sin(roll)    & cr=cos(roll)
     sp=sin(pitch)   & cp=cos(pitch)
     sh=sin(heading) & ch=cos(heading)
  endelse
endif else begin
   message,'Usage: res=ac2gt(attitude) or res=ac2gt(roll,pitch,heading)',/info
   return,-1
endelse 

; The result of analytical calculation of a2g transformation matrix is:

  res=[[ sh*cp,  ch*cr+sh*sp*sr, -ch*sr+sh*sp*cr], $
       [ ch*cp, -sh*cr+ch*sp*sr,  sh*sr+ch*sp*cr], $
       [ sp,             -cp*sr,          -cp*cr]]

; For vector inputs reform/transpose res so that the input dimension n 
; becomes last. If g2a is set transpose the transformation matrix (matrices)
  
  n=n_elements(sr)
  if n gt 1 then begin
     res=reform(res,n,3,3)
     if keyword_set(g2a) then $
        return, transpose(res,[2,1,0]) $
     else return, transpose(res,[1,2,0])
  endif else if keyword_set(g2a) then return, transpose(res) else return, res

end
