;$Id: ac2wcr.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       AC2WCR
;
; PURPOSE:
;       Returns a KingAir(KA) or NCAR C130 variable with the time dimension 
;       matching WCR time dimension (profiles) or vice versa.
;       KA or WCR variables could be 1-D, 2-D or 3-D arrays with the last
;       dimension being the time(profile). The match is achieved by linearly 
;       interpolating KA (ac2wcr) or WCR (wcr2ac) data variable time dimension.
;
;       There is no lowpass filtering of the result and thus frequency
;       aliasing is possible in cases where the resampled data is also
;       downsampled.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       result=ac2wcr(var,actime,wcrtime,sps,timehack=timehack  $
;                     errorm=errorm,wcr2ac=wcr2ac,help=help)
;
; INPUTS:
;       Var:      numerical array, AC variable (or WCR var if wcr2ac is set)
;                 AC var must have one time dim only (sps dim should be merged
;                 with time dim - see acloadnc function)  
;       Actime:   long/double array, AC time vector (UNIX seconds)
;       Wcrtime:  double array, WCR time vector (UNIX seconds)
;       Sps:      input, AC variable sampling rate (default 1, see NOTES)
;
; KEYWORD PARAMETERS:
;       ERRORM:   when set prints error messages
;       TIMEHACK: input,float, WCR time correction in seconds  (default 0.0)
;       WCR2AC:   when set match WCR variable (var) to AC sps 
;       HELP:     shows this text
;
; OUTPUTS:
;      
; SIDE EFFECTS:
;       The function returns an empty string if the usage is incorrect, 
;       -1 if var is not defined or has more than 3 dimensions, 
;       -2 if wcrtime is not subset of actime, or
;       -3 if actime is not subset of wcrtime when wcr2ac is set     
;
; NOTES:
;       AC vars are equaly spaced in time.  WCR vars are non-equaly spaced
;       (varying number of profiles per second and a small variation in the
;       time interval between the profiles).  Thus ac2wcr returns 
;       irregularly gridded data.  If wcr2ac is set the returned WCR var is
;       regularly gridded matching AC netcdf sps dimension.
;
;       The timehack, if given, serves to sync WCR and AC time.
;       The time hack is applied to the WCR time (it should be small if any).
;       The input wcrtime is not changed.
;
;       If actime is a double precision array and includes the fractional 
;       seconds corresponding to the AC var sampling rate (sps), the sps input
;       parameter could be omitted or must be set to 1.
;
;       actime and wcrtime must match the AC and WCR data vars
;
;       ATTN: This function has not been tested for 3-D arrays yet
;        
; EXAMPLES:
;
; ******* 1) Interpolate KA data to sync with WCR time *************************
; 
; .run commonwcrset                     ; Install WCR data common blocks
; openwcrf,'Wpp00-04-13-18-51-54',data  ; Open WCR raw data file
; wcrtime=1                             ; Initilize wcrtime as scalar to force
;                                       ; loading WCR raw time by readwcrdata 
; readwcrdata,0,2000,rawtime=wcrtime    ; Load segment of WCR data and full raw
; wcrtime=wcrtime-86400.                ; time (UNIX format). WCR time for this
;                                       ; file was 24 hours ahead; so correct it
; timehack=0.0                          ; Assume no time sync problem between
;                                       ; the AC and WCR (GPS to UTC correction
;                                       ; is applied within readwcrdata)
;
; Extract the indices for desired UTC interval from the loaded wcr data segment.
; In this case readwcrdata loaded data by given start/stop record numbers.
; To check what times corresponds to this numbers one can print the following:
; print,sec2time([time[0],time[1999]), which will print start and stop time
; as hhmmss.ss double precision numbers.
; 
; ind=where(sec2time(time+timehack) gt 185416 and  $
;           sec2time(time+timehack) lt 185500)
;
; Load the AC UNIX time variable (time).  If there is no such variable in the
; AC netcdf data file one can use acloadnc routine with TIME keyword to construct
; AC UNIX time vector (this assume that netcdf file is NCAR/RAF Nimbus format
; compliant).  
;
; katime=acloadnc(ncid,'time','20000413.c25.nc',sps=sps,start=185400,stopt=185510)
;
; 'time' vector represents whole seconds. Thus expand it to sps (in this case
; sps=25). Use rebin but take care to expand last second
;
; katime25=rebin(double(katime),25*n_elements(katime))
; katime25[n_elements(katime25)-25:*]=katime25[n_elements(katime25)-25:*]+ $
;                                     findgen(25)/25.  
;
; Load the AC var to be interpolated. Make sure it is for the same time
; interval as katime.  Also make sure this time interval encloses the WCR
; time interval, which allows interpolation (no extrapolation) for all data
; points of the output.
;
; karoll=acloadnc(ncid,'hroll',sps=sps,start=185400,stopt=185510)
;
; Interpolate the KA roll to match WCR sampling times

; wcrroll=ac2wcr(karoll,katime25,wcrtime[ind],1,timehack=timehack)
;
; Plot the results

; plot,katime25-katime25[0],karoll,lin=1
; oplot,wcrtime[ind]+timehack-katime25[0],wcrroll
;
; aximage,10.*alog10(hh[*,ind]),time[ind],radrng,/tr,/timeax,setw=[-1,41]
; plot,time[ind],wcrroll,/noerase,xsty=5,ysty=5
;
; ******* 2) Interpolate WCR data to sync with KA time *************************
;
; .run commonwcrset
; openwcrf,'Wpp00-04-13-18-51-54',data
; wcrtime=1 & readwcrdata,0,2000,rawtime=wcrtime
; wcrtime=wcrtime-86400. ; WCR unix time for this file was 24 hours ahead
; timehack=0.0
;
; katime=acloadnc(ncid,'time','20000413.c25.nc',sps=sps,start=185417,stopt=185500)
; if sec2time(katime[0] mod 86400) ne 185417 or $
;    sec2time(katime[n_elements(katime)-1] mod 86400) ne 185510 then $
;    print, 'Requested time interval is not valid.'
; rebin but take care to expand last second
; katime25=rebin(double(katime),25*n_elements(katime))
; katime25[n_elements(katime25)-25:*]=katime25[n_elements(katime25)-25:*]+ $
;                                     findgen(25)/25.  
; 
; ; Interpolate the noise channel
; kahhnoise=ac2wcr(hhnoise,katime25,wcrtime,1,timehack=timehack,/wcr2ac)
;
; ; Interpolate the H-reflectivity
; kahh=ac2wcr(hh,katime25,wcrtime,1,timehack=timehack,/wcr2ac)
;
; ; Plot the results
; plot,wcrtime+timehack-katime25[0],10.*alog10(hhnoise),lin=1
; oplot,katime25-katime25[0],10.*alog10(kahhnoise)
;
; aximage,10.*alog10(kahh),(katime25 mod 86400),radrng,/tr,/timeax,setw=[-1,41]
; aximage,10.*alog10(hh),time,radrng,/tr,/timeax,setw=[-1,41]
;
; ******************************************************************************
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, August 2000
;       S.H., Aug 2000 -- added errorm keyword
;       S.H., Feb 2001 -- minor change (when checking for timehack param.)
;       S.H., Sep 2001 -- minor comment changes
;       S.H., Nov 2001 -- minor help text changes
;       S.H., Feb 2003 -- changed routine name from ka2wcr to ac2wcr, replaced
;                         ka prefix to ac where applicable
;       S.H., Apr 2003 -- changed acload to acloadnc
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function ac2wcr, var,actime,wcrtime,sps,timehack=timehack,errorm=errorm, $
                 wcr2ac=wcr2ac,help=help

  on_error,2
 
  if keyword_set(help) then begin
    doc_library,'ac2wcr'
    return,''
  endif

  if n_params() lt 3 then begin
    message,"Usage: res=ac2wcr(var,actime,wcrtime,sps, $",/info
    print,"          timehack=timehack,wcr2ac=wcr2ac,help=help)"
    return,''
  endif else if n_params() eq 3 then sps=1


; Check tne number of dimensions for the input

  ndim=size(var,/n_dim)
  n=size(var,/dim)
  nkt=n_elements(actime)-1
  nrt=n_elements(wcrtime)-1

  if ndim gt 3  or ndim lt 1 then begin
    if keyword_set(errorm) then $
       message,'VAR is not defined or has more than 3 dimensions.',/info
    return, -1
  endif

; Check input times and correct radar time if needed

  if sps gt 1 then begin
    ktime=rebin(double(actime),sps*n_elements(actime))
    ; take care of the last second (it is not interpolated by rebin)
    ktime[n_elements(ktime)-sps:*]=ktime[n_elements(ktime)-sps:*]+ $
                                 findgen(sps)/sps
  endif else ktime=actime 
  if n_elements(timehack) eq 0 then rtime=wcrtime else $
     if timehack eq 0. then rtime=wcrtime else rtime=wcrtime+timehack

  if keyword_set(wcr2ac) then begin
     if ktime[0] lt rtime[0] or ktime[nkt] gt rtime[nrt] then begin
       if keyword_set(errorm) then $
          message,'ACTIME is not subset of WCRTIME',/info
       return,-3
     endif
  endif else begin
     if rtime[0] lt ktime[0] or rtime[nrt] gt ktime[nkt] then begin
       if keyword_set(errorm) then $
          message,'WCRTIME is not subset of ACTIME',/info
       return,-2
     endif
  endelse

; Interpolate var

  if ndim eq 1 then begin
    if keyword_set(wcr2ac) then outvar=interpol(var,rtime,ktime) $
    else outvar=interpol(var,ktime,rtime)
  endif else if ndim eq 2 then begin
    if keyword_set(wcr2ac) then $
         outvar=interpolate(/grid,miss=-999.,var,findgen(n[0]),  $
                            interpol(findgen(n[1]),rtime,ktime)) $
    else outvar=interpolate(/grid,miss=-999.,var,findgen(n[0]),  $
                            interpol(findgen(n[1]),ktime,rtime))
  endif else begin
    if keyword_set(wcr2ac) then $
       outvar=interpolate(/grid,miss=-999.,var,findgen(n[0]),findgen(n[1]), $
                          interpol(findgen(n[2]),rtime,ktime)) $
    else $
       outvar=interpolate(/grid,miss=-999.,var,findgen(n[0]),findgen(n[1]), $
                          interpol(findgen(n[2]),ktime,rtime))
  endelse

  return,fix(outvar,type=size(var,/type))

end  
       
