;$Id: acloadins.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       ACLOADINS
;
; PURPOSE:
;       Returns UW_KingAir/NCAR_C130 selected INS and other AC data variables
;       from a netCDF file. There is only one time dimension in the loaded
;       variables. See the notes.  Time variable is resampled to match data
;       sampling rate (sps).
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       acdata = acloadins(cdfid,startt=startt,stopt=stopt,close=close,$
;                deg=deg,filename=filename,ncnames=ncnames,sps=sps,    $
;                wcrdvc=wcrdvc,help=help)
;
; INPUTS:
;       Cdfid:     netCDF file ID (if defined), returned from a previous call
;                  to ncdf_open, ncdf_create, ncdf_varload or ncdf_fcopy or
;                  other routine
;
; KEYWORD PARAMETERS:
;       CLOSE:     when set AC netcdf file will be closed.
;       DEG:       input, when set the attitude angles are given in degrees
;       FILENAME:  input, string, KA(N2UW) or NCAR 130 (N130AR) netCDF file
;                  name; if given will open netCDF file first. In this case
;                  Cdfid is an output parameter
;       NCNAMES:   input, string or string array, AC netcdf varnames to be used
;                  (if not defined N2UW or N130AR default var names are used)
;                  ncnames[0] - AC true air speed in m/s
;                  ncnames[1] - AC roll [deg]
;                  ncnames[2] - AC pitch [deg]
;                  ncnames[3] - AC true heading [deg]
;                  ncnames[4] - AC east-west(EW) ground velocity [m/s]
;                  ncnames[5] - AC north-south(NS) ground velocity [m/s]
;                  ncnames[6] - AC vertical velocity [m/s]
;                  ncnames[7] - AC East component of the wind [m/s]
;                  ncnames[8] - AC North component of the wind [m/s]
;                  ncnames[9] - AC vertical wind [m/s]
;                  ncnames[10]- AC roll rate [rad/s] (use '' if N/A)
;                  ncnames[11]- AC pitch rate [rad/s](use '' if N/A)
;                  ncnames[12]- AC heading rate [rad/s] (use '' if N/A)
;                  ncnames[13]- AC EW ground vel. GPS correction(use '' if N/A)
;                  ncnames[14]- AC NS ground vel. GPS correction(use '' if N/A)
;                  ncnames[15]- AC latitude [deg] (use '' if N/A)
;                  ncnames[16]- AC longitude [deg](use '' if N/A)
;                  ncnames[17]- AC pressure altitude [m](use '' if N/A)
;                  ncnames[18]- AC geometric altitude [m](use '' if N/A)
;                  ncnames[19]- AC sideslip angle [rad]
;
;                  Note: define first 10 names only if the last 10 vars are
;                        not needed; see Notes for default variable names;
;                        if wcrdvc is set the last 5 vars are not loaded
;       SPS:       output, integer; samples per second (1,10,25; 25 is expected)
;       STARTT:    input, long, start time in hhmmss
;       STOPT:     input, long, stop time in hhmmss including last second record
;       WCRDVC:    when set acdata contains hte first 18 fields only (see,acdata
;                  structure description below; needed for WCR Doppler velocity
;                  correction)
;       HELP:      shows this text
;
; OUTPUTS:
;       Cdfid:  netCDF file ID, returned if Filename is given
;       Acdata: structure, return AC IRS/INS data matching AC time
;               acdata.time   -> double(n), AC unix time in sec (see Notes)
;             ; Following angles are in radians or degrees if deg keyword set
;               acdata.roll   -> fltarr(n), AC roll angle
;               acdata.pitch  -> fltarr(n), AC pitch angle
;               acdata.heading-> fltarr(n), AC true heading from North
;             ; AC ground velocities in [m/s]
;               acdata.ug     -> fltarr(n), East-West AC ground velocity
;               acdata.vg     -> fltarr(n), North-South AC ground velocity
;               acdata.wg     -> fltarr(n), AC vertical velocity
;             ; UWKA GPS corrections of ins AC velocities (ignore for C130)
;               acdata.uerr   -> fltarr(n), E-W vel correction in m/s
;               acdata.verr   -> fltarr(n), N-S vel correction in m/s
;             ; AC measured winds in [m/s]
;               acdata.uw     -> fltarr(n), East wind velocity component
;               acdata.vw     -> fltarr(n), North wind velocity component
;               acdata.ww     -> fltarr(n), vertical wind velocity component
;             ; AC attitude rates in rad/sec or deg/sec (if deg keyword is set)
;               acdata.rollr  -> fltarr(n,3), roll rate
;               acdata.pitchr -> fltarr(n,3), pitch rate
;               acdata.yawr   -> fltarr(n,3), yaw rate
;             ;
;               acdata.tas    -> fltarr(n), true air speed in m/s
;
;             ; AC data info: aircraft ID (N2UW or N130AR)
;               acdata.info   -> string, aircraft ID
;
;             ; UWKA radar wing mirror position
;               acdata.wcrmirror -> intarr(n), KA radar wing mirror position
;                                   (0-side, 1-up, for side/up antenna)
;
;             ; Fields used for Dual-Doppler analysis only
;               acdata.lat    -> fltarr(n), latitude position of the AC [deg]
;               acdata.lon    -> fltarr(n), longitude position of the AC [deg]
;               acdata.palt   -> AC pressure altitude in meters
;               acdata.ralt   -> AC radar altimeter altitude in meters
;               acdata.beta   -> AC sideslip angle
;
;             Note: If any of the non-critical data (e.g., attitude rate) is
;                   not available, -1 is recorded
;
; SIDE EFFECTS:
;
; NOTES:
;       KingAir netCDF files follow the NCAR-RAF/nimbus convention.  This
;       convention requires that the time dimension, 'time' (unlimited or not),
;       is the first (last for idl, which uses fortran order) and the next
;       dimension, if exists, is always samples per second, 'sps'.
;
;       Acdata.time time vector is assembled from two variables:
;       'base_time' - seconds since 00:00:00 Jan 1 1970
;       'time_offset' - seconds since base_time, a sequence w/ 1 sec step
;       The result is a vector representing seconds since 00:00:00 Jan 1 1970.
;       If sps is greater than 1 actime is resampled to match sps.
;
;       This routine loads the requested variables with the 'sps' dimension,
;       if exists, merged with the 'time' dimension.  Thus the output always
;       has only one time dimension.
;
;       If startt and stopt are given netcdf variables are extracted within the
;       specified time. If stopt is not defined the variables are read untill
;       the end of the file.  And if both, startt and stopt are not defined or
;       are invalid the complete data set for the variables are read.
;
;       ________________________________________
;
;       Default KA (N2UW) netcdf variable names:
;       ________________________________________
;
;       True Air Sppeed
;
;         'TASX' or 'tas'
;
;       Attitude angles and their rates
;
;         roll -> 'hroll',     roll rate  ->'hrollr'
;         pitch -> 'hpitch',   pitch rate ->'hpitchr'
;         heading -> 'hthead', yaw rate   ->'hyawr'
;
;       Attitude vars have attribute 'units' with values 'deg', 'rad', 'deg/sec'
;       or 'rad/sec'.  All variables are converted according to deg keyword
;
;       AC ground velocity and its GPS correction (not applied to acvel)
;
;         E-W vel ->  'hewvel', E-W GPS vel correction ->  'uerr'
;         N-S vel ->  'hnsvel', N-S GPS vel correction ->  'verr'
;
;       Vertical velocity
;
;         'hwp3'   - use baro-loop output
;                    other possible vars are
;                         'hivs' - use direct ins output or
;                         'wnb'  - use UW adaptive filter
;
;       AC derived wind
;
;         'hu'  ; East component
;         'hv'  ; North component
;         'hw'  ; vertical component
;
;       AC LAT and LON position
;
;         'LAT', GPS-Corrected Inertial Latitude
;                    other possible variable:
;                       'GLAT', GPS Latitude
;         'LON', GPS-Corrected Inertial Longitude
;                    other possible variable:
;                       'GLON', GPS Longitude
;
;       AC pressure altitude
;         'PALT' - hydrostatic corrected pressure altitude in meters
;
;       AC geometric altitude
;         'ralt2' - radar altimeter altitude in meters
;
;       AC side slip angle
;
;         'beta' - side slip angle in deg or rad

;       Radar mirror position is extracted from 25th bit of lo_events
;
;       __________________________________________________
;
;       Default NCAR C-130 (N130AR) netcdf variable names:
;       __________________________________________________
;
;       True Air Sppeed
;
;        'TASX' or 'TASHC'
;
;       Attitude angles and their rates
;
;         roll -> 'ROLL',     roll rate  -> don't know at this point
;         pitch -> 'PITCH',   pitch rate -> don't know at this point
;         heading -> 'THDG',  yaw rate   -> don't know at this point
;
;       Attitude vars have attribute 'units' with values 'deg', 'rad', 'deg/sec'
;       or 'rad/sec'.  All variables are converted according to deg keyword
;
;       AC ground velocity
;
;         'VEWC', East ground velocity (GPS corrected)
;                 other possible variables:
;                 'GVEW', East ground velocity (GPS) or
;                 'VEW',  East ground velocity (INS) or
;
;         'VNSC', North ground velocity (GPS corrected)
;                 other possible variables:
;                 'GVNS', North ground velocity (GPS) or
;                 'VNS',  North ground velocity (INS) or
;
;       Vertical velocity
;
;         'WP3'   - use baro-loop output
;
;       AC wind
;
;         'UIC', East component (GPS corrected)
;                other possible variable:
;                'UI',  East component
;
;         'VIC', North component (GPS corrected)
;                other possible variable:
;                'VI',  North component
;
;         'WIC', vertical component (GPS corrected)
;                other possible variable:
;                'WI',  vertical component
;
;       AC LAT and LON position
;
;         'LATC', GPS-Corrected Inertial Latitude
;                 other possible variable:
;                 'GLAT', GPS Latitude
;         'LONC', GPS-Corrected Inertial Longitude
;                 other possible variable:
;                 'GLON', GPS Longitude
;
;       AC pressure altitude
;
;         'PALT'
;
;       AC geometric altitude
;
;         'HGM232' - radar altimeter altitude in meters
;
;       AC side slip angle
;
;         'SSLIP' - side slip angle in deg or rad
;       _________________________________
;
;       All variable units are defined in attribute 'units' and should be
;       'm/s' for velocity vars, and 'rad', 'deg', /rad/sec', or 'deg/sec' for
;       attitude variables.
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, December 2000
;       S.H., Jan 01 -- minor changes
;       S.H., Sep 01 -- replaced wtype keyword with ncnames
;       S.H., Feb 02 -- added close keyword
;       S.H., Mar 02 -- added tas keyword and modified ncnames
;       S.H., Oct 02 -- modified to reflect the changes in kaload;
;                       added acwind keyword and modified ncnames;
;                       modified the check for acvele vars existance
;       S.H., Jan 03 -- added keyword latlon and corresponding nc names
;       S.H., Feb 03 -- rename routine from kaloadins to acloadins, change
;                       prefix ka to ac where appropriate
;       S.H., Apr 03 -- changed kamirror to kawcrmirror, acload to acloadnc
;       S.H., May 03 -- major change in the formal parameters - change from
;                       subroutine to function and introduced acdata
;                       structure as output
;       S.H., Aug 03 -- added AC radar altimeter and AC sideslip to acdata;
;                       added wcrdvc keyword
;       S.H., Sep 03 -- changed ztrue with PALT for KingAir; replaced ztrue
;                       field in acdata with palt firle
;       S.H., Apr 04 -- removed unnecessary calculation of actime
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function acloadins, Cdfid,startt=startt,stopt=stopt,close=close,deg=deg,$
         filename=filename,ncnames=ncnames,sps=sps,wcrdvc=wcrdvc,help=help

  on_error,2

; Check some of the inputs

  if keyword_set(help) then begin
    doc_library,'acloadins'
    return,''
  endif

  if n_params() eq 0 then begin
    print,"Usage: acdata=acloadins(Cdfid,startt=startt,stopt=stopt,close=close,$"
    print,"                 deg=deg,filename=filename,ncnames=ncnames,$"
    print,"                 sps=sps,wcrdvc=wcrdvc,help=help)"
    return,''
  endif

; If filename is given open the file first.  Close the old one if exists.

  if n_elements(filename) gt 0 then begin
    if n_elements(Cdfid) eq 1 then ncdf_close,Cdfid
    Cdfid=ncdf_open(filename)
  endif

; Get the AC name(id)

  ncdf_attget,Cdfid,'Aircraft',acname, /global
  acname=strupcase(string(acname))

; Define ins(IRS) var names

  nncn=n_elements(ncnames)
  if nncn le 1 then begin
     if acname eq 'N2UW' then begin
       ; True Air Speed

       vntas = ['tas', 'TASX']

       ; Attitude angles and their rates

       vnroll    ='hroll'   &  vnrollr ='hrollr'
       vnpitch   ='hpitch'  &  vnpitchr='hpitchr'
       vnheading ='hthead'  &  vnyawr  ='hyawr'

       ; AC ground velocity and their errors

       vnewvel ='hewvel'  &   vnnsvel ='hnsvel'
       vnewvele='uerr'    &   vnnsvele='verr'

       ; Vertical velocity

       vnwvel = 'hwp3'

       ; AC wind

       vnewind = 'hu'  &  vnnwind = 'hv'  &  vnvwind = 'hw'

       ; AC LAT and LON position

       vnlat =  'LAT' & vnlon = 'LON'

       ; AC pressure altitude

       vnpalt = 'PALT'

       ; AC radar altimeter

       vnralt = 'ralt2'

       ; AC sideslip angle

       vnbeta = 'beta'

     endif else if acname eq 'N130AR' then begin
       ; True Air Speed

       vntas = ['TASX', 'TASHC']

       ; Attitude angles and their rates

       vnroll    ='ROLL'   &  vnrollr =''
       vnpitch   ='PITCH'  &  vnpitchr=''
       vnheading ='THDG'   &  vnyawr  =''

       ; AC ground velocity  (GPS corrected)

       vnewvel ='VEWC'  &   vnnsvel ='VNSC'
       vnewvele=''      &   vnnsvele=''

       ; Vertical velocity

       vnwvel = 'WP3'

       ; AC wind

       vnewind = 'UIC'  &  vnnwind = 'VIC'  &  vnvwind = 'WIC'

       ; AC LAT and LON position

       vnlat =  'LATC' & vnlon = 'LONC'

       ; AC pressure altitude

       vnpalt = 'PALT'

       ; AC radar altimeter

       vnralt = 'HGM232'

       ; AC sideslip angle

       vnbeta = 'SSLIP'

     endif else message,'Wrong AC identification(N2UW or N130AR)'
  endif else begin
     if nncn lt 10 then message,'NCNAMES must have 10, 13, 15 or 18 elements.'
     vntas    =ncnames[0]
     vnroll   =ncnames[1]
     vnpitch  =ncnames[2]
     vnheading=ncnames[3]
     vnewvel  =ncnames[4]
     vnnsvel  =ncnames[5]
     vnwvel   =ncnames[6]
     vnewind  =ncnames[7]
     vnnwind  =ncnames[8]
     vnvwind  =ncnames[9]
     if nncn gt 10 then begin
       vnrollr =ncnames[10]
       vnpitchr=ncnames[11]
       vnyawr  =ncnames[12]
       if nncn gt 13 then begin
         vnewvele=ncnames[13]
         vnnsvele=ncnames[14]
         if nncn gt 15 then begin
           vnlat  =ncnames[15]
           vnlon  =ncnames[16]
           vnpalt=ncnames[17]
           vnralt =ncnames[18]
           vnbeta =ncnames[19]
         endif
       endif
     endif
  endelse

  ; Units atribute name and valid values

  anunits ='units'
  avangle ='deg'
  avangler='rad/sec'
  avvel   ='m/s'

; Get all AC varnames and check which tas variable to use

  varnames=ncdf_varnames(Cdfid)

; Check tas names

  if n_elements(vntas) eq 2 then begin
    ind=where(vntas[0] eq varnames)
    if ind[0] ne -1 then vntas=vntas[0] $
    else begin
      ind=where(vntas[1] eq varnames)
      if ind[0] ne -1 then vntas=vntas[1] $
      else begin
        ncdf_close,Cdfid
        message, 'TAS variable('+vntas+') not found'
      endelse
    endelse
  endif else begin
    ind=where(vntas eq varnames)
    if ind[0] eq -1 then begin
      ncdf_close,Cdfid
      message, 'TAS variable('+vntas+') not found'
    endif
  endelse

; Assemble full time vector from 'base_time' and 'time_offset' if not given
; This is not needed. Calculated inside acloadnc.

;  base_time=ncdf_varload(Cdfid,'base_time')
;  actime     =long(ncdf_varload(Cdfid,'time_offset'))
;  if base_time eq -1 or actime[0] eq -1 then begin
;    ncdf_close, Cdfid
;    message,'base_time and/or time_offset do not exist.'
;  endif else $
;     actime=actime+base_time

; Load ins vars.  Assume all velocity vars are in m/s.

  ; Attitude

  roll=acloadnc(Cdfid,vnroll,startt=startt,stopt=stopt,time=actime,sps=sps)
  n=n_elements(roll)
  if n eq 1 then begin
    ncdf_close, Cdfid
    message,"This can't be happening!"
  endif

  pitch=acloadnc(Cdfid,vnpitch,startt=startt,stopt=stopt)
  heading=acloadnc(Cdfid,vnheading,startt=startt,stopt=stopt)

  ; Check attitude units

  atvalue=strlowcase(ncdf_attload(Cdfid,vnroll,anunits))
  if atvalue eq avangle and keyword_set(deg) eq 0 then begin
     roll   =!dtor*roll
     pitch  =!dtor*pitch
     heading=!dtor*heading
  endif

  ; AC ground velocity

  acvel=fltarr(n,3)

  acvel[*,0]=acloadnc(Cdfid,vnewvel,startt=startt,stopt=stopt)
  acvel[*,1]=acloadnc(Cdfid,vnnsvel,startt=startt,stopt=stopt)
  acvel[*,2]=acloadnc(Cdfid,vnwvel,startt=startt,stopt=stopt)

  ; AC ground velocity errors

  if vnewvele ne '' then begin
    acvele=acloadnc(Cdfid,vnewvele,startt=startt,stopt=stopt)
    if acvele[0] eq -1 then begin
      message,vnewvele+' and/or '+vnnsvele+' do not exist. Ignore.',/info
      acvele=[[-1],[-1]]
    endif else $
      acvele=[[acvele],[acloadnc(Cdfid,vnnsvele,startt=startt,stopt=stopt)]]
  endif else acvele=[[-1],[-1]]

  ; Attitude rate

  if vnrollr ne '' then begin
    attr=fltarr(n,3)

    attr[*,0]=acloadnc(Cdfid,vnrollr,startt=startt,stopt=stopt)

    ; Check if attitude rates exist

    if total(attr[*,0]+1) ne 0 then begin
      attr[*,1]=acloadnc(Cdfid,vnpitchr,startt=startt,stopt=stopt)
      attr[*,2]=acloadnc(Cdfid,vnyawr,startt=startt,stopt=stopt)

      ; Check attitude rate units

      atvalue=strlowcase(ncdf_attload(Cdfid,vnrollr,anunits))
      if strmid(atvalue,0,3) eq avangler and keyword_set(deg) then $
        attr=!radeg*attr
    endif else attr=[[-1],[-1],[-1]]
  endif else attr=[[-1],[-1],[-1]]

  ; AC wind

  acwind=fltarr(n,3)

  acwind[*,0]=acloadnc(Cdfid,vnewind,startt=startt,stopt=stopt)
  acwind[*,1]=acloadnc(Cdfid,vnnwind,startt=startt,stopt=stopt)
  acwind[*,2]=acloadnc(Cdfid,vnvwind,startt=startt,stopt=stopt)

  ; KA Radar mirror position is extracted from 25th bit of lo_events

  if acname eq 'N2UW' then $
     wcrmirror=kawcrmirror(Cdfid,sps=sps,startt=startt,stopt=stopt) $
  else wcrmirror=-1

  ; True air speed

  tas=acloadnc(Cdfid,vntas,startt=startt,stopt=stopt)
 
  ; If WCRDVC is not set get the rest of the data

  if keyword_set(wcrdvc) eq 0 then begin
    ; AC LAT and LON position

    if vnlat ne '' then begin
      latlon=fltarr(n,2)

      latlon[*,0]=acloadnc(Cdfid,vnlat,startt=startt,stopt=stopt)
      latlon[*,1]=acloadnc(Cdfid,vnlon,startt=startt,stopt=stopt)
    endif else latlon=[[-1],[-1]]

    ; Pressure altitude

    if vnpalt ne '' then $
      palt=acloadnc(Cdfid,vnpalt,startt=startt,stopt=stopt) $
    else palt=-1

    ; Radar altimeter altitude

    if vnralt ne '' then $
      ralt=acloadnc(Cdfid,vnralt,startt=startt,stopt=stopt) $
    else ralt=-1

    ; Sideslip angle

    if vnbeta ne '' then begin
      beta=acloadnc(Cdfid,vnbeta,startt=startt,stopt=stopt)
      ; Check attitude rate units
      atvalue=strlowcase(ncdf_attload(Cdfid,vnbeta,anunits))
      if strmid(atvalue,0,3) ne avangle and keyword_set(deg) then $
        beta=!radeg*beta
    endif else beta=-1

    ; Prepare output structure acdata

    acdata={                                                                   $
      roll:roll               ,pitch:pitch             ,heading:heading,       $
      rollr:reform(attr[*,0]) ,pitchr:reform(attr[*,1]),yawr:reform(attr[*,2]),$
      ug:reform(acvel[*,0])   ,vg:reform(acvel[*,1])   ,wg:reform(acvel[*,2]), $
      uw:reform(acwind[*,0])  ,vw:reform(acwind[*,1])  ,ww:reform(acwind[*,2]),$
      uerr:reform(acvele[*,0]),verr:reform(acvele[*,1]),info:acname,           $
      time:actime             ,tas:tas                 ,wcrmirror:wcrmirror,   $
      lat:reform(latlon[*,0]) ,lon:reform(latlon[*,1]) ,palt:palt,           $
      ralt:ralt               ,beta:beta }

  endif else begin
    acdata={                                                                   $
      roll:roll               ,pitch:pitch             ,heading:heading,       $
      rollr:reform(attr[*,0]) ,pitchr:reform(attr[*,1]),yawr:reform(attr[*,2]),$
      ug:reform(acvel[*,0])   ,vg:reform(acvel[*,1])   ,wg:reform(acvel[*,2]), $
      uw:reform(acwind[*,0])  ,vw:reform(acwind[*,1])  ,ww:reform(acwind[*,2]),$
      uerr:reform(acvele[*,0]),verr:reform(acvele[*,1]),info:acname,           $
      time:actime             ,tas:tas                 ,wcrmirror:wcrmirror}
  endelse
 
  if keyword_set(close) then ncdf_close, Cdfid

  return,acdata

end
