;$Id: acloadnc.pro,v 1.2 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       ACLOADNC
;
; PURPOSE:
;       Returns KingAir or NCAR C130 aircraft data variable from a netCDF file.
;       There is only one time dimension in the returned variable.  
;       See the notes.  
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       result=acloadnc(Cdfid,varname [,filename], $
;              sps=sps,startt=startt,stopt=stopt,time=time,help=help)
;
; INPUTS:
;       Cdfid:    netCDF file ID, returned from a previous call to 
;                 ncdf_open, ncdf_create, ncdf_varload or ncdf_fcopy
;       Varname:  string, netCDF variable name
;       Filename: string, optional netCDF file name; if given will open netCDF 
;                 file first. In this case Cdfid is an output parameter
;
; KEYWORD PARAMETERS:
;       SPS:    output, integer; samples per second
;       STARTT: input, long, start time in hhmmss
;       STOPT:  input, long, stop time in hhmmss including last second record
;       TIME:   output, dblarr, netcdf file time vector (unix sec), see NOTES
;       HELP:   shows this text
;
; OUTPUTS:
;       Cdfid:  netCDF file ID, returned if Filename is given
;
; SIDE EFFECTS:
;       The function returns -1 if the variable is not found or an empty string
;       if the usage is incorrect.
;       Variables must have maximum of 4 dimensions.
;
; NOTES:
;       KingAir netCDF files follow the NCAR-RAF/nimbus convention.  This
;       convention requires that the time dimension, 'time' (unlimited or not), 
;       is the first (last for idl, which uses fortran order) and the next 
;       dimension, if exists, is always samples per second, 'sps'.
;
;       Time vector (keyword TIME), is assembled from two variables:
;         'base_time'   - seconds since 00:00:00 Jan 1 1970
;         'time_offset' - seconds since base_time, a sequence with 1 sec step
;       If sps > 1 the time vector is resampled with the sps rate. The result
;       is a dblarr representing seconds [and fractional seconds] since
;       00:00:00.00 Jan 1 1970.
;
;       This function returns the requested variable with the 'sps' dimension,
;       if exists, merged with the 'time' dimension.  Thus the output always
;       has only one time dimension.
;
;       If startt and stopt are given netcdf variable is extracted within the
;       specified time. If stopt is not defined the variable is read untill
;       the end of the file.  And if both, startt and stopt are not defined or
;       are invalid the complete data set for the variable is read.
;
;       ATTN:  This function is not suitable to load WCR variables when present
;              in a merged AC/WCR netCDF file (Dave Leon's processing). WCR vars
;              do not have sps dimension. Use kawcrreform from this library.
;        
; EXAMPLES:
;
;      The following example assumes that a netcdf file was previously opened,
;      hroll variable exists and 121915/122252 is a legitimate time (hhmmss)
;      subinterval. In addition, acloadnc returns actime wich is the the full time
;      vector for this file in unix seconds (see notes above). 
;
;      roll=acloadnc(ncid,'hroll',sps=sps,start=121915,stop=122252,time=actime)
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, June 2000
;       S.H., Oct 00 -- added warning message for non sps dimensions
;                       and a note in the help text
;       S.H., Mar 01 -- use offset and count to speed up reading partial data
;       S.H., Aug 02 -- modified time (to expand ac time according to sps)
;                       and startt keywords
;       S.H., Feb 03 -- rename routine from kaload to acload, change
;                       prefix ka to ac where appropriate
;       S.H., Apr 03 -- renamed acload to acloadnc
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function acloadnc, Cdfid,varname,filename, $
                 sps=sps,startt=startt,stopt=stopt,time=time,help=help

  on_error,2
 
  if keyword_set(help) then begin
    doc_library,'acloadnc'
    return,''
  endif

  if n_params() eq 0 then begin
    message,"Usage: res=acloadnc(Cdfid,varname [,filename], $",/info
    print,"          sps=sps,startt=startt,stopt=stopt,time=time,help=help)"
    return,''
  endif else if n_params() eq 3 then begin
    if n_elements(Cdfid) eq 1 then ncdf_close,Cdfid
    Cdfid=ncdf_open(filename)
  endif

; Assemble full time vector from 'base_time' and 'time_offset'

  base_time=ncdf_varload(Cdfid,'base_time')
  time=long(ncdf_varload(Cdfid,'time_offset'))
  if base_time eq -1 or time[0] eq -1 then $
    message,'base_time and/or time_offset do not exist.' $
  else $
    time=time+base_time

; Inquire about varname var.  If does not exist return -1

  varinq=ncdf_varinq1(Cdfid,varname)
  if varinq.id eq -1 then return,-1

; If varname var is a scalar load it and exit

  if varinq.ndims eq 0 then begin
     ncdf_varget,Cdfid,varinq.id,vardata
     sps=0
     return,vardata
  endif

; Determine the time range for the variable

  if n_elements(startt) gt 0 or n_elements(stopt) gt 0 then begin
     timeh=long(sec2time(time mod 86400))
     if n_elements(startt) eq 0 then ind1=0 else begin
        ind1=where(startt eq timeh)
        if ind1[0] eq -1 then begin
           ind1=0
           message,'Start time not valid. Load from the beginning.',/info 
        endif else ind1=ind1[0]
     endelse
     if n_elements(stopt) gt 0 then begin
         ind2=where(stopt eq timeh)
         if ind2[0] eq -1 then begin
           ind2=n_elements(timeh)-1 
           message,'Stop time not valid. Load to the end of the file.',/info 
         endif else ind2=ind2[n_elements(ind2)-1]
     endif else ind2=n_elements(timeh)-1
     n=ind2-ind1+1
     if n lt 1 then message,'Incorrect start/stop times'
  endif else begin 
     ind1=0 
     n=varinq.dimsize[varinq.ndims-1]
     ind2=n-1
  endelse

; Determine offset and count

  offset1=ind1
  count1=n 
  sps=1
  if varinq.ndims eq 1 then begin
    offset=offset1
    count=count1
    sps=1
  endif else begin
    if strmid(varinq.dimname[varinq.ndims-2],0,3) ne 'sps' then begin
       message,'WARNING: '+varname+' 2nd dimension is not sps',/info
       print,'          Dimension name: ',varinq.dimname[varinq.ndims-2]
    endif
    sps=varinq.dimsize[varinq.ndims-2]
 
    if varinq.ndims eq 2 then begin
      offset=[0,offset1]
      count=[varinq.dimsize[0],count1]
    endif else if varinq.ndims eq 3 then begin
      offset=[0,0,offset1]
      count=[varinq.dimsize[0:1],count1]  
    endif else begin
      offset=[0,0,0,offset1]
      count=[varinq.dimsize[0:2],count1]  
    endelse
  endelse

; Load the variable

  ncdf_varget,Cdfid,varinq.id,vardata,offset=offset,count=count
        
; Reform

  if varinq.ndims gt 1 then begin
    if varinq.ndims eq 2 then  $                                      ; 2-D var
      vardata=reform(vardata,sps*n) $
    else if varinq.ndims eq 3 then  $                                 ; 3-D var
      vardata=reform(vardata,varinq.dimsize[0],sps*n) $
    else if varinq.ndims eq 4 then  $                                 ; 4-D var
      vardata=reform(vardata,varinq.dimsize[0],varinq.dimsize[1],sps*n)
  endif

  if n_elements(vardata) gt 1 then vardata=reform(vardata) $
  else vardata=vardata[0]

; If sps > 1 recalculate time vector to include fractional seconds if requested

  if sps gt 1 then begin
    timeh=time[ind1:ind2]
    time=rebin(double(timeh),sps*n_elements(timeh))
    ; take care of the last second (it is not interpolated by rebin)
    time[n_elements(time)-sps:*]=time[n_elements(time)-sps:*]+findgen(sps)/sps
  endif else time=double(time[ind1:ind2])
 
return,vardata

end  
       
