;$Id: kawcrmirror.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;      KAWCRMIRROR
;
; PURPOSE:
;      Returns time array for the KA (N2UW) radar wing mirror position 
;      (0-side, 1-up, single antenna configuration)
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;   res=kawcrmirror(ncf)
;   res=kawcrmirror(ncf,timem=timem,sps=sps,startt=startt,stopt=stopt, $
;                    date=fdate,leapsec=leapsec)
;
; INPUTS:
;      Ncf:  string, KA processed data netcdf file name or
;            long, netcdf file id
;
; KEYWORD PARAMETERS:
;      SPS:     input, KA variable sampling rate (default 1); use with timem=0
;               (e.g., sps=25 will force 25 Hz sampling of the function return) 
;      STARTT:  input, start time in hhmmss (default, the beginning of the file)
;               (can be coded as hhmmss.1, see timem)
;      STOPT:   input, stop time in hhmmss (default, the end of the file)
;      TIMEM:   1, kawcrmirror returns fltarr(n,3): n - number of mirror changes
;                  res[*,0] - start times, hhmmss
;                  res[*,1] - stop times, hhmmss
;                  res[*,2] - mirror position (0-side, 1-up)
;               0, kawcrmirror returns intarr(ntime*sps) w/ 
;                  radar position(default) (ntime - number of seconds)
;               fltarr(ntime), netcdf var TIME (previously read)
;                  will skip reading TIME netcdf var and is equivalent to
;                  input 1 if startt=hhmmss or input 0 if startt=hhmmss.1  
;      DATE:    output, string, KA flight date YYYYMMDD
;      LEAPSEC: output, long, loads leap seconds from KA data file
;      HELP:    show this text
;
; OUTPUT
;      Ncf:  long, netcdf file id  (if the input Ncf was a netcdf file name)
;
; SIDE EFFECTS:
;      Require netcdf file for the KA data.
;      Mirror position must be the 25th bit in var 'lo_events' (1Hz rate only)
;      Time in hhmmss is read from netcdf var 'TIME'
;      Flight date is read from var 'FlightDate'
;      The function returns -1 if startt and/or stopt are not within TIME
;      and an empty string if the usage is incorrect
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov, August 1999
;      S.H., May 2000 -- Added leapsec keyword, modified ncf
;      S.H., Aug 2000 -- modified timem keyword and added startt, stopt, sps
;      S.H., Nov 2000 -- modified the extraction of mirror pos. form lo_events
;                        due to interference in the '0' (low) level
;      S.H., Apr 2003 -- renamed kamirror to kawcrmirror
;      S.H., Aug 2003 -- minor changes in the help
;      S.H., Apr 2004 -- fixed midnight crossing problem when stopt is given
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function kawcrmirror,ncf,timem=timem,sps=sps,startt=startt,stopt=stopt,  $
                      date=fdate,leapsec=leapsec, help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'kawcrmirror'
  return,''
endif

if n_params() eq 0 then begin
   message,'Usage: res=kawcrmirror(ncf,/time,date=fdate)',/info
   return,-1
endif

; mirror events are recorded in the 24th bit of lo_events (1-Hz rate)
; 1 (16777216) - up; 0 - side

; read leap seconds, low volts events, time (hhmmss) and flight date (m/dd/yyyy)

  if (size(ncf))[1] eq 7 then begin
     leapsec=ncdf_varload(ncid,'leap_sec',ncf)
     ncf=ncid
  endif else begin 
     ncid=ncf
     leapsec=ncdf_varload(ncid,'leap_sec') 
  endelse

  loev=ncdf_varload(ncid,'lo_events')

  ncdf_attget,ncid,'FlightDate',val,/global
  junk=str_sep(val,'/')
  if strlen(junk[0]) eq 1 then junk[0]='0'+junk[0]
  if strlen(junk[1]) eq 1 then junk[1]='0'+junk[1]
  fdate=junk[2]+junk[0]+junk[1]

; find mirror positions

; ind=where((loev and 16777216) eq 0)     ; find side position; not good; there
                                          ; is interference at low level;
  ind=where((loev and 33554432) eq 0)     ; so use 25 bit to find up position

  strt=0                                  ; first record number
  stp=n_elements(loev)-1                  ; last record number

; mir=intarr(stp+1)+1                     ; initialize all to up pos.
; if ind[0] ne -1 then mir[ind]=0         ; assign side pos.

  mir=intarr(stp+1)                       ; initialize all to side pos.
  if ind[0] ne -1 then mir[ind]=1         ; assign up pos.

; check if time sub-interval is requested and if we are done 

  exitflag=0B
 
  if n_elements(startt) eq 0 and n_elements(stopt) eq 0 then begin
     if n_elements(timem) eq 0 then exitflag=1B $
     else if timem[0] eq 0 and n_elements(timem) eq 1 then exitflag=1B $
     else if timem[0] eq 1 and n_elements(timem) eq 1 then $
        time=ncdf_varload(ncid,'TIME') $
     else if n_elements(timem) gt 1 then time=timem
  endif else begin

     if n_elements(timem) le 1 then time=ncdf_varload(ncid,'TIME') $
     else if n_elements(timem) gt 1 then time=timem 

     if n_elements(startt) eq 0 then ind1=0 else $
        ind1=(where(long(time) eq long(startt)))[0]
     if ind1 eq -1 then begin
        message,'Invalid start time (startt): '+time2timestr(long(startt)),/info
        return,-1
     endif 

     if n_elements(stopt) eq 0 then ind2=stp else $
       ind2=(where(long(time) eq long(stopt)))[0]       
     if ind2 eq -1 then begin
        message,'Invalid stop time (stopt): '+time2timestr(stopt),/info
        return,-1
     endif

     time=time[ind1:ind2]
     stp=n_elements(time)-1
     mir=mir[ind1:ind2]

     if n_elements(timem) eq 0 then exitflag=1B $
     else if n_elements(timem) eq 1 then begin
       if timem eq 0 then exitflag=1B
     endif else if startt-long(startt) gt 0 then exitflag=1B
  endelse

  ; exit if we are done

  if exitflag then begin
     if n_elements(sps) eq 0 then return,mir $
     else if sps eq 1 then return,mir $
     else return, rebin(mir,sps*(stp+1))
  endif 

; find mirror change times

  ; find changes in the mirror position and the corresponding times
                
  junk=(shift(mir,1))[1:stp]

  if mir[0] eq 1 then begin
     ind1=where((mir[1:stp]-junk) eq 1)+1    ; find ups (first occurences)
     ind2=where((mir[1:stp]-junk) eq -1)+1   ; find sides (first occurences)
  endif else begin
     ind1=where((mir[1:stp]-junk) eq -1)+1   ; find sides
     ind2=where((mir[1:stp]-junk) eq 1)+1    ; find ups
  endelse

  ; add first index and quit if no change in position

  if ind1[0] gt 0 then ind1=[0,ind1]

  if ind2[0] eq 0 then begin
     mirrort=fltarr(1,3)
     mirrort[0,*]=[time[0],time[stp],mir[0]]

  endif else begin

  ; prepare the output: start/stop times and mirror position codes

    n=n_elements(ind1)
    mirrort=fltarr(n+n_elements(ind2),3)
    ind=lindgen(n)*2

    if n eq n_elements(ind2) then begin
       mirrort[ind,0]=time[ind1]
       mirrort[ind,1]=time[ind2-1]
       mirrort[ind,2]=mir[ind1]
       ind=ind+1
       mirrort[ind,0]=time[ind2]
       if n eq 1 then mirrort[ind,1]=time(stp) $
       else mirrort[ind,1]=time[[ind1[1:*]-1,stp]]
       mirrort[ind,2]=mir[ind2]
    endif else begin
       mirrort[ind,0]=time[ind1]
       mirrort[ind,1]=time[[ind2-1,stp]]
       mirrort[ind,2]=mir[ind1]
       ind=(ind+1)[0:n-2]
       mirrort[ind,0]=time[ind2]
       mirrort[ind,1]=time[ind1[1:*]-1]
       mirrort[ind,2]=mir[ind2]
    endelse 
  endelse
 
return,mirrort

end




