;$Id: kawcrreform.pro,v 1.10 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       KAWCRREFORM
;
; PURPOSE:
;       Reforms a WCR variable from a merged KA/WCR netcdf file by merging
;       'profile' and 'time' dimensions.
;       The resultant array has one less dimension and the 'time' is a new 
;       dimension which represents all profiles. This new time dimension is
;       not regularly spaced. 
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       result=kawcrreform(ncid,varname [,filename],startt=startt,stopt=stopt)
;
; INPUTS:
;       ncid:     netCDF variable ID, returned from a previous call to 
;                 ncdf_open, ncdf_create, ncdf_varload or ncdf_fcopy
;       Varname:  string, netCDF variable name
;       Filename: string, optional netCDF file name; if given will open netCDF 
;                 file first. In this case ncid is an output var
;
; KEYWORD PARAMETERS:
;       STARTT:   input, long, start time in hhmmss; (default is base_time,
;                   see Notes); if beyond the time interval of the file, starts
;                   from the biginning
;       STOPT:    input, long, stop time in hhmmss (see Notes); (default is the
;                   end if the file time, see Notes); if beyond the time interval 
;                   of the file, use the default end of the file time
;       HELP:     show this text ( use print,kawcrreform(/help) )
;
; OUTPUTS:
;       ncid:     netCDF variable ID, returned if Filename is given
;
; SIDE EFFECTS:
;       The function returns an empty string if the usage is incorrect or 
;       -1 if the variable is not found or
;       -2 if var 'profiles' does not exist or
;       -3 if 'base_time' and/or 'time_offset' do not exist.
;       -4 if STARTT and STOPT are not valid for this file
;       -5 if varname has more than 3 dimensions
;
; NOTES:
;       This routine assumes that there is a dimension called 'profile'
;       (representing the max. profiles in a second) in the netCDF file as well
;       as variable 'profiles' with the actual number of radar profiles
;       for each second.
;
;       varname netcdf variable can have no more than 3 dimesnions
;       and time dimension is always the last one
;
;       The data are read from KA/WCR merged netcdf file without any checkings
;       and modifications.
;
;       The use of STARTT and STOPT requires the existence of netcdf variables
;       'base_time' and 'time_offset', where base_time is the start time of the
;       data in the file given in seconds after 00:00:00 Jan 1, 1970 and 
;       time_offset is offset in seconds from base_time.
;
; EXAMPLES:
;
;       hh=kawcrreform(ncid,'hh')
;
;       This will load hh(range,profile,time) and reform to hh(range,new_time)
;       where new_time_size=n_elements(time)*total(profiles)
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, December 1999
;       S.H., 02/02 - added STARTT and STOPT keywords
;       S.H., 03/02 - resolved cases when there are no valid radar profiles
;                     in a given second
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function kawcrreform, ncid,varname,filename,startt=startt,stopt=stopt,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'kawcrreform'
    return,''
  endif

  seldim='profile'    ;  name of dimension to be reduced
  selname='profiles'  ;  var name with the number of valid profiles per second
  maxndims=3          ;  maximum number of dims in the varname var 


  if n_params() lt 2 then begin
    message,"Usage: res=kawcrreform(ncid,varname [,filename])",/info
    return,''
  endif else if n_params() eq 3 then ncid=ncdf_open(filename)

; Inquire about the varname and selname

  vari=ncdf_varinq1(ncid,varname)
  if vari.id eq -1 then return,-1              ; varname doesn't exist
  if vari.ndims gt maxndims then return,-5     ; more than 3 dimensions

  junk1=ncdf_varinq1(ncid,selname)
  if junk1.id eq -1 then return,-2             ; selname doesn't exist

; Check if startt and stopt are given and is possible to use them

  if n_elements(startt) eq 0 then strt=-1 else strt=startt
  if n_elements(stopt) eq 0 then stpt=-1 else stpt=stopt

  if strt ne -1 or stpt ne -1 then begin
    junk1=ncdf_varinq1(ncid,'base_time')
    if junk1.id eq -1 then return,-3     ; base-time doesn't exist
    junk1=ncdf_varinq1(ncid,'time_offset')
    if junk1.id eq -1 then return,-3     ; time_offset doesn't exist

    ; Load base_time and time_offset
    basetime=ncdf_varload(ncid,'base_time') 
    baset=basetime mod 86400
    timeoffset=ncdf_varload(ncid,'time_offset')
    n=n_elements(timeoffset)

    if strt eq -1 then strt=long(timeoffset[0]) else begin
       strt=long(time2sec(strt))
       if strt lt baset then strt=strt+86400-baset else strt=strt-baset
       if strt lt 0 or strt gt n-1 then strt=0L
    endelse

    if stpt eq -1 then stpt=long(timeoffset[n-1]) else begin
       stpt=long(time2sec(stpt))
       if stpt lt baset then stpt=stpt+86400-baset else stpt=stpt-baset
       if stpt lt 0 or stpt gt n-1 then stpt=n-1
    endelse

    if stpt le strt then return,-4   ; invalid start/stop times

    ; Define offset and count

    offset1=strt
    count1=stpt-offset1+1
    if vari.ndims eq 1 then begin
      offset=offset1
      count=count1
    endif else if vari.ndims eq 2 then begin
      offset=[0,offset1]
      count=[vari.dimsize[0],count1]
    endif else begin
      offset=[0,0,offset1]
      count=[vari.dimsize[0:1],count1]  
    endelse

    ; Load varname and seldim

    ncdf_varget,ncid,vari.id,tmp,offset=offset,count=count   
    ncdf_varget,ncid,selname,profs,offset=offset1,count=count1

  endif else begin
    ncdf_varget,ncid,vari.id,tmp   
    ncdf_varget,ncid,selname,profs
  endelse

; Check if the variable can be reformed and proceed with reforming   

  if vari.ndims eq 1 then return,tmp  ; nothing to integrate
   
  ind=where(vari.dimname eq seldim)   ; look for seldim and if doesn't exist
  if ind[0] eq -1 then return,tmp     ; return the varname unaltered

  vari.dimsize=size(tmp,/dimensions)

  n=long(total(profs))                ; size of the new time dimension
  j=0L

  if vari.ndims eq 2 then begin       ; apparently seldim is the first dimension
     res=reform(tmp,vari.dimsize[0]*vari.dimsize[1])
     for k=0L,vari.dimsize[1]-1 do $
       if profs[k] gt 0 then begin
         ind=lindgen(profs[k])     
         res[j+ind]=res[k*vari.dimsize[0]+ind]
         j=j+profs[k]
       endif
     res=res[0:n-1]
  endif else if vari.ndims eq 3 then begin
     if ind[0] eq 0 then begin              ; seldim is the first dimension
       res=reform(tmp(0:vari.dimsize[1]*n-1),vari.dimsize[1],n)
       for k=0L,vari.dimsize[2]-1 do $
         if profs[k] gt 0 then begin
           ind=lindgen(profs[k])     
           res[*,j+ind]=tmp[ind,*,k]
           j=j+profs[k]
         endif
     endif else begin                       ; seldim is the second dimension
       res=reform(tmp(0:vari.dimsize[0]*n-1),vari.dimsize[0],n)
       for k=0L,vari.dimsize[2]-1 do $
         if profs[k] gt 0 then begin
           ind=lindgen(profs[k])     
           res[*,j+ind]=tmp[*,ind,k]
           j=j+profs[k]
         endif
     endelse
  endif

  return,res 

end
