;$Id: openwcrf.pro,v 1.11 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       OPENWCRF
;
; PURPOSE:
;       Open SPPMAG, PPMAG4, PPMAG6 or FFT WCR file and load several
;       radar/data parameters (see showwcrprms)
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       OPENWCRF, Fn, Data
;
; INPUTS:
;       Fn:      string, wcr file name
;
; KEYWORD PARAMETERS:
;       ACSPEED:       input, float; average aircraft speed in m/s (default 90.)
;       WCRLUN:        lun, default is 31
;       TIMECOR:       if defined gives UTC time correction of the wcr time
;                      After June 1998, IRIG card is used to set the time.
;                      Normal use of the card is to get the time from a master
;                      IRIG card with GPS time.  Thus the default value is 
;                      -13 s, which are the current leap seconds (Jan 1999)
;       BIG_ENDIAN:    when set, assumes that the file data are in big endian
;                      order.  The default is LITTLE_ENDIAN (after Mar 2001) and
;                      For data before Mar 2002 it is ignored.  
;       HELP:  Show this text
;
; OUTPUTS:
;       Data:    wcr file structure
;       WcrFlag:  wcrflag[0] - file size doesn't match 
;                              (data file did not close properly)
;                 wcrflag[1] - # of bad profiles at the file end
;                 wcrflag[2] - WCR computer date and timestamps date don't match
;                              or IRIG connection/data was lost/corrupted
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       radar parameters loaded in wcrparams common block (see showwcrprms)
;       radrng[rangegates] and velo[fftbins] are loaded in wcrdata by this 
;       routine
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Sep 1995
;       S.H., 11/98  --  got rid of whattime and start time from wcr file name
;       S.H., 04/99  --  discard last profile(s) if header record(s) is/are no 
;                        good
;       S.H., 05/99  --  modified header and modes to work with the new VXI 
;                        software (limited modification; not all modes are 
;                        available)                    
;       S.H., 07/99  --  removed pulsewidth calculation using IF filter 
;                        settings; it is now recorded in the header properly
;       S.H., 08/99  --  added PPMAG6 mode, significant changes in the common 
;                        blocks and variable names
;                        added timecor keyword
;       S.H., 10/99  --  modified wcrdate calculation
;                        fixed typos in pulsesampledelay and help text
;       S.H., 12/99  --  corrected starttime and stoptime for tcorrection
;                        restored Tx leakage vars in wcrdata common 
;                        block; added noise samples option, added wcrflag;
;                        bypass file association when data param is not given
;       S.H., Feb 00 --  added help keyword, fft frequences adjusted (velo)
;                        to reflect that 1st bin is the DC in the fft mode
;       S.H., Apr 00 --  small change in the caldat call and allow 1 day 
;                        discrepancy in the date
;       S.H., Nov 00 --  added DPFFT mode
;       S.H., Jan 01 --  changed sign for the velo (positive vel toward the wcr)
;                        changed readu routine with my readse that takes care
;                        of byte swaping if needed; added little_endian keyword
;       S.H., Feb 01 --  changed routine name from openwcrf openwcrf
;       S.H., Mar 01 --  minor change in the determination of start/stop times
;       S.H., Jun 01 --  added one more element to wcrflag and warning for time
;                        stamp problem
;       S.H., Jul 01 --  minor comment added
;       S.H., Feb 02 --  added acspeed keyword
;       S.H., Apr 02 --  modified to work with a new data format; changed
;                        LITTLE_ENDIAN keyword to BIG_ENDIAN
;       S.H., May 02 --  small changes, fixed bug in starttime/stoptime decimal
;       S.H., Dec 02 --  changed definition of timeinterval from mean to median
;                        value after readwcrdata reads the data (openwcrf still
;                        calculates mean value); expanded dspsync to 6 elements
;       S.H., Jan 03 --  modified acqtime again to represent theoretical (true)
;                        time that is needed to collect the data for the given
;                        number of pulses to be averaged by the DSPs
;       S.H., Feb 03 --  modified prf for SPPMAG to allow pulse group spacing
;                        (added to wcr DAQ software in the spring of 2003)
;       S.H., Mar 03 --  change decoding IRIG field in the header according
;                        to the latest format
;       S.H., Nov 03 --  added noiseflag initialization
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro openwcrf, fn, data, WCRLUN=lun1,TIMECOR=timecor,HELP=help,ACSPEED=acspeed, $
                        BIG_ENDIAN=bgnd

@commonwcrinclude

on_error,2 

 if keyword_set(help) then begin
   doc_library,'openwcrf'
   return
 endif

 if (keyword_set(lun1) eq 0) then lun1 = 31

 if n_elements(acspeed) eq 0 then kaspeed=90. else kaspeed=acspeed

; Some WCR file constant parameters
  badf       =  -999.     ; float bad value
  badi       =  -999      ; integer bad value
  irigflag   =  -1        ; available after 3/02; 0-computer time, 1-IRIG card
  fileidstr  =  'W20'     ; it is updated to the first 3 chars of the filename
  filenamel  =   20       ; this is fixed and used to update fileIDstr
  filehdrsz  =   206L     ; bytes; after March 2002
  headersize =   20L      ; bytes; after March 2002
                          ; 92 before October 1996
                          ; 100 after October 1996 
                          ; 160 after May 1999
                          ; 20 after March 2002
  headervars =   5        ; number of 4 byte fields in the data header
                          ; 23 before October 1996   
                          ; 25 after October 1996
                          ; 40 after May 1999
                          ; 4 after March 2002
  bind       =   0        ; blocksize var position in data header, after 3/02
  tind       = [1,2]      ; timestamp var positions in data header, after 3/02  
  
  noisesamples=   1       ; # noise samples (always 1 before May 1999)
                          ; it is updated from the header info 
  txsamples  =    1       ; # of Tx pulse sample (int.cal. before 1997,
                          ;                       leakage after 1996) 
  sppalg     =    9
  pp4alg     =   14       ; was 7
  pp6alg     =   15 
  fftalg     =   11
  dftalg     =   13
  polh       =   104      ; was 7
  polv       =   118      ; was 15
  sppvars    =    4
  ppmvars    =   14
  sppmaxsamp =53376       ; bytes; averages*npulse*(rangegates+noisesamples+1)
  ppmmaxsamp =35584       ; bytes
  fftmaxsamp =53376       ; bytes; averages*fftbins*(rangegates+noisesamples+1)
  npulse     =    4       ; Number of pulses in a burst it is updated for pp6alg
                          ; or fft.
  lambda     =    0.00316 ; MW length in [m]
  hcorrection=intarr(8)   ; no correction is applied to any H measurements
  vcorrection=intarr(8)   ; no correction is applied to any V measurements
  timemode   =0           ; initialize timemode of the time vector 
  interrupts =0           ; no interrupts
  dspsync   =[badf,0,0,0,0,0]   ; initilize dspsync
  wcrflag   =[0,0,0]      ; wcrflag[0]= 1, file size doesn't match
                          ; wcrflag[1]= # of bad profiles at the file end 
                          ; wcrflag[2]=1, WCR computer date and timestamps date
                          ; don't match or IRIG connection/data was lost/corrupted
  noiseflag =0L           ; indicates how noise ch. data is loaded in readwcrdata;
                          ; it is determined in readwcrdata

  if n_elements(timecor) eq 0 then $
     tcorrection=-13. $   ; GPS leap seconds correction 
                          ; After Aug 1999 the radar records GPS time
                          ; After Mar 2002 if irigflag[0]=0 default is 0.
  else tcorrection=timecor 

; Get WCR file name and WCR date

  fileidstr=strmid(fn,strlen(fn)-filenameL,3)        ; allows renaming 
  wcrfile=strmid(fn,strpos(fn,fileidstr,/reverse_search),filenamel)
  if strmid(wcrfile,3,2) lt '90' then wcrdate='20'+strmid(wcrfile,3,2) $
  else wcrdate='19'+strmid(wcrfile,3,2)
  wcrdate=long(wcrdate+strmid(wcrfile,6,2)+strmid(wcrfile,9,2))
  if wcrdate lt 19900101 or wcrdate gt 20200101 then begin
     message,'WCR file naming problem warning: '+wcrfile,/info
  endif

; ------------------------------ Open the file ---------------------------------
 closewcrf, wcrlun=lun1   ; close wcrfile if it was not close previously
 openr,lun1,fn
 fs = fstat(lun1) 

; ---------------------------  Begin header data -------------------------------
if wcrdate gt 20020331 then begin

  ; Read file header

  version         =long(1)     & readse,lun1,big_end=bgnd, version,swap=swapbytes
  prf             =lonarr(6)   & readse,lun1,big_end=bgnd, prf
  junk            =lonarr(6)   & readse,lun1,big_end=bgnd, junk
  txsequence      =bytarr(6)   & readse,lun1,big_end=bgnd, txsequence
  rxsequence      =bytarr(6,2) & readse,lun1,big_end=bgnd, rxsequence
  pulselength     =long(1)     & readse,lun1,big_end=bgnd, pulselength 
  txsamples       =long(1)     & readse,lun1,big_end=bgnd, txsamples
  noisesamples    =long(1)     & readse,lun1,big_end=bgnd, noisesamples
  rangegates      =long(1)     & readse,lun1,big_end=bgnd, rangegates
  rangegatedelay  =long(1)     & readse,lun1,big_end=bgnd, rangegatedelay 
  rangegatespacing=long(1)     & readse,lun1,big_end=bgnd, rangegatespacing 
  junk            =lonarr(2)   & readse,lun1,big_end=bgnd, junk
  iffilter        =long(1)     & readse,lun1,big_end=bgnd, iffilter
  averages        =long(1)     & readse,lun1,big_end=bgnd, averages
  alg             =long(1)     & readse,lun1,big_end=bgnd, alg
  fftbins         =long(1)     & readse,lun1,big_end=bgnd, fftbins
  fftwindow       =long(1)     & readse,lun1,big_end=bgnd, fftwindow
  junk            =long(1)     & readse,lun1,big_end=bgnd, junk
  irigflag        =long(1)     & readse,lun1,big_end=bgnd, irigflag
  junk            =lonarr(18)  & readse,lun1,big_end=bgnd, junk
  datablocksize   =long(1)     & readse,lun1,big_end=bgnd, datablocksize

  ; Read first data header  

  fstblocknum     =long(1) & readse,lun1,big_end=bgnd, fstblocknum
  startt          =lonarr(2) & readse,lun1,big_end=bgnd, startt
  
  ; Tide up some of the header variables

  if alg eq pp6alg then npulse=6 $                        ; PPMAG6
  else if alg eq pp4alg or alg eq sppalg then npulse=4 $  ; SPPMAG/PPMAG 
  else npulse=1                                           ; FFT, DFFT
  prf             =float(prf(0:npulse-1))

  ; decode irigflag: bit 0   : IRIG OFF/ON (0-host time / 1-TFP IRIG input time)
  ;                  bit 1   : Forced TFP jamsynch (1)
  ;                  bit 4-7 : TFP status bits
  ; irigflag = -2 could be set by writewcrraw if timestamps are corrupted

  if irigflag gt 0 then irigflag=[(irigflag and 1),((irigflag and 2) eq 2),    $
                        ((irigflag and 16) eq 16)+2*((irigflag and 32) eq 32)+ $
                        4*((irigflag and 64) eq 64)]

  pulselength     =pulselength*25 
  rangegatedelay  =float(rangegatedelay)*3.75  
  rangegatespacing=float(rangegatespacing)*3.75
  txsequence      =txsequence[0:npulse-1]
  if alg eq sppalg or alg eq fftalg or alg eq dftalg then begin  
    rxsequence    =txsequence[0]
    if alg ne sppalg then prf=prf[0]
  endif else rxsequence =rxsequence[0:npulse-2>0,*]
  case iffilter of
    1: iffilter=5.01 ; MHz (old 5 MHz Chebychev filter,5.01 for identif.)
    2: iffilter=2.   ; MHz (2 MHz Bessel filter)
    3: iffilter=5.   ; MHz (5 MHz Bessel filter)
    5: iffilter=10.  ; MHz (10 MHz Bessel filter)
    else: iffilter=badf   ; unknown or not allowed  
  endcase

  ; Check if time stamps match wcrdate

   caldat, startt[0]/86400d0+2440587.5d0, mm, dd, yyyy
   k=yyyy*10000L+mm*100+dd

  if wcrdate ne k then begin
     wcrflag[2]=1
     if wcrdate lt k-1 then begin
        message,'Using time stamps to define data date: '+num2str(k),/info
        wcrdate=k
     endif else $
        message,'Using file name to define data date: '+num2str(wcrdate),/info
  endif

  ; Modify default tcorrection according to irigflag

  if n_elements(timecor) eq 0 and irigflag[0] ne 1 then tcorrection=0.

endif else begin
 ; --------------- header variables from old data formats --------------------

  bgnd       =   1 
  filehdrsz  =   0
  headervars =   23
  headersize =   92
  txsamples  =    1
  polh       =    7
  polv       =   15
  bind       =   1
  tind       = [3,4]
                                           ; 0; file type ? 
  version         =long(1) & readse,lun1,big_end=bgnd, version,swap=swapbytes
                                           ; 1; data block #
  fstblocknum     =long(1) & readse,lun1,big_end=bgnd, fstblocknum
                                           ; 2; in bytes
  datablocksize   =long(1) & readse,lun1,big_end=bgnd, datablocksize
                                           ; 3; unix time [s, us]
  startt          =lonarr(2) & readse,lun1,big_end=bgnd, startt
                                           ; 5; radar azimuth
  spare           =long(1) & readse,lun1,big_end=bgnd, spare
                                           ; 6; radar elevation
  spare           =long(1) & readse,lun1,big_end=bgnd, spare 
                                           ; 7; proc.alg.flag ?
  spare           =long(1) & readse,lun1,big_end=bgnd, spare

  ; Check if time stamps match wcrdate

   caldat, startt[0]/86400d0+2440587.5d0, mm, dd, yyyy
   k=yyyy*10000L+mm*100+dd

  if wcrdate ne k then begin
     wcrflag[2]=1
     if wcrdate lt k-1 then begin
        message,'Using time stamps to define data date: '+num2str(k),/info
        wcrdate=k
     endif else $
        message,'Using file name to define data date: '+num2str(wcrdate),/info
  endif

  ; Check if before May 18 1999 (new WCR software) and process accordingly

  if (wcrdate lt 19990518) then begin 

    pp4alg=7
                                            ; 8; [n,n,n,n] Hz
    prf             =lonarr(4) & readse,lun1,big_end=bgnd, prf
                                              ; 12; n 
    pulselength     =long(1)   & readse,lun1,big_end=bgnd, pulselength 
    pulselength     =pulselength*25           ;     n*50 [ns], N/A
                                              ; 13;[n,n,n,n],H/V:7/15
    txsequence      =bytarr(4) & readse,lun1,big_end=bgnd, txsequence
    txsequence      =reverse(txsequence)
                                              ; 14; n
    pulsesampledelay=long(1) & readse,lun1,big_end=bgnd, pulsesampledelay
    pulsesampledelay=(pulsesampledelay+2)*100 ; (2+n)*100 [ns]
                                              ; 15;
    rangegates      =long(1) & readse,lun1,big_end=bgnd, rangegates
                                              ; 16; n  
    rangegatedelay  =long(1) & readse,lun1,big_end=bgnd, rangegatedelay 
    rangegatedelay  =float(rangegatedelay)*3.75   ;     3.75*n [m]
                                              ; 17; n   
    rangegatespacing=long(1) & readse,lun1,big_end=bgnd, rangegatespacing 
    rangegatespacing=float(rangegatespacing)*3.75 ;     3.75*n [m]
                                              ; 18; n (n+1)*2 [us]
    spare           =long(1) & readse,lun1,big_end=bgnd, spare
                                              ; 19; 1:rx atten. on
    spare           =long(1) & readse,lun1,big_end=bgnd, spare
                                              ; 20; rx IF filt., N/A
    iffilter        =long(1) & readse,lun1,big_end=bgnd, iffilter
                                              ; 21;
    averages        =long(1) & readse,lun1,big_end=bgnd, averages
                                              ; 22;    9,7,11
    alg             =long(1) & readse,lun1,big_end=bgnd, alg

    ; Add fftbins and fftwindow for data taken after Oct 22, 1996 

    if (wcrdate gt 19961022) then begin 

       headersize =   headersize+8
       headervars =   headervars+2

                                             ; 23; number of bins for FFT files
       fftbins  =long(1) & readse,lun1,big_end=bgnd,fftbins
                                             ; 24; fft window: 0,1,2,3,4,5
                                             ; rect.,hann,taper,blackman,
                                             ; gauss,gaussian top
       fftwindow=long(1) & readse,lun1,big_end=bgnd,fftwindow
    endif else begin
       fftbins=averages*(alg eq fftalg)
       fftwindow=0
    endelse

    ; Pulse width was set manually; set it to -999
      pulselength=badi

    tcorrection =badf ; no time correction is available at this time

    rxsequence=txsequence
    if alg eq sppalg or alg eq fftalg then begin
       if alg eq fftalg then txsequence=txsequence[0]
       rxsequence=rxsequence[0]
    endif

  endif else begin   ; for data/header after May 18 1999

    headersize      =160       ; bytes
    headervars      =40        ; number of 4 byte fields 
                                            ; 8; [n,n,n,...] Hz
    prf             =lonarr(12) & readse,lun1,big_end=bgnd, prf
                                            ; 20;[s,s,s,s],'h','v'
    txsequence   =bytarr(8) & readse,lun1,big_end=bgnd, txsequence

    ; make it compatible with the previous versions
    for k=0,7 do $
        if txsequence(k) eq '104' then txsequence(k)=polh $
        else if txsequence(k) eq '118' then txsequence(k)=polv $
        else txsequence(k)=0   ; not used
                                            ; 22;[s,s,s,s],'h','v'
    rxsequence =bytarr(16) & readse,lun1,big_end=bgnd, rxsequence

    ; follow the original convention for txsequence
    for k=0,15 do $
      if rxsequence(k) eq '104' then rxsequence(k)=polh $
      else if rxsequence(k) eq '118' then rxsequence(k)=polv $
      else rxsequence(k)=0   ; not used
    rxsequence=transpose(reform(rxsequence,8,2))
                                            ; 26; n
    pulselength     =long(1)   & readse,lun1,big_end=bgnd, pulselength 
    pulselength     =pulselength*25         ;     n*50 [ns], 50
                                            ; 27; 
    spare           =lonarr(1) & readse,lun1,big_end=bgnd, spare
                                            ; 28; 
    spare           =lonarr(1) & readse,lun1,big_end=bgnd, spare
                                            ; 29; n
    noisesamples    =long(1) & readse,lun1,big_end=bgnd, noisesamples
                                            ; 30;
    rangegates      =long(1) & readse,lun1,big_end=bgnd, rangegates
                                            ; 31; n   
    rangegatedelay  =long(1) & readse,lun1,big_end=bgnd, rangeGateDelay 
    rangegatedelay  =float(rangeGateDelay)*3.75 ;     3.75*n [m]
                                            ; 32; n  
    rangegatespacing=long(1) & readse,lun1,big_end=bgnd, rangegatespacing 
    rangegatespacing=float(rangegatespacing)*3.75 ;     3.75*n [m]
                                            ; 33;
    spare           =fltarr(1) & readse,lun1,big_end=bgnd, spare
                                            ; 34; 
    spare           =fltarr(1) & readse,lun1,big_end=bgnd, spare
                                            ; 35; rx filt. code
    iffilter        =long(1) & readse,lun1,big_end=bgnd, iffilter
                                            ; 36;
    averages        =long(1) & readse,lun1,big_end=bgnd, averages
                                            ; alg.codes: SPPMAG: 9, FFT: 11, 
                                            ; DPPMAG: 12, DPFFT: 13, PPMAG4: 14, 
                                            ; PPMAG6: 15
                                            ; 37; 9,14(7),11,13,15
    alg             =long(1) & readse,lun1,big_end=bgnd, alg
                                            ; 38; number of bins for FFT files
    fftbins  =long(1) & readse,lun1,big_end=bgnd,fftbins
                                            ; 39; fft window: 0 (rectangular),
                                                ; 1 (hanning)
    fftwindow=long(1) & readse,lun1,big_end=bgnd,fftwindow
 
    ; need to check the valid prfs and change npulse if needed; 
    if alg eq pp6alg then begin                                 ; PPMAG6 
      txsequence=txsequence[0:5] & rxsequence=transpose(rxsequence[*,0:4])
      prf=prf(0:5) & npulse=6 
    endif else if alg eq pp4alg or alg eq sppalg then begin     ; SPPMAG/PPMAG   
      txsequence=txsequence[0:3] & 
      if alg eq pp4alg then begin
        rxsequence=transpose(rxsequence[*,0:2])
        prf=prf[0:3]
      endif else begin
        rxsequence=txsequence[0]
        prf=prf[0]
      endelse
    endif else begin                                            ; FFT, DFFT
      txsequence=txsequence[0] & rxsequence=txsequence[0]
      prf=prf[0] & npulse=1
    endelse

    ; define the if filter
      case iffilter of
         1: iffilter=5.01 ; MHz (old 5 MHz Chebychev filter,5.01 for identif.)
         2: iffilter=2.   ; MHz (2 MHz Bessel filter)
         3: iffilter=5.   ; MHz (5 MHz Bessel filter)
         5: iffilter=10.  ; MHz (10 MHz Bessel filter)
         else: iffilter=badf   ; unknown or not allowed  
      endcase
  endelse
endelse
; ---------------------------  End header data -------------------------------

; Some WCR file derivative parameters

  avespectra = fftmaxsamp/(averages*(rangegates+2))     ; for fft
  numsamp    = averages*(rangegates+2)                  ; always le MaxSamp
; sppacqtime = averages*1000./prf[0]                    ; in [ms]
  sppacqtime = averages*1000.*mean(1./prf)              ; in [ms]
  ppmacqtime = averages*1000.*total(1./prf)             ; theoretical acq in ms
  fftacqtime = avespectra*fftbins*1000./prf[0]          ; in [ms]
; print,sppacqtime,ppmacqtime,fftacqtime
  blocksize  = datablocksize+headersize                 ; record size in Bytes
  sppDataRate= long(4./sppacqtime*sppvars* $
               (rangegates+2)/1.024)                    ; in kB/sec
  ppmDataRate= long(4./ppmacqtime*ppmvars* $
               (rangegates+2)/1.024)                    ; in kB/sec
  fftDataRate= long(4./sppacqtime*fftbins* $
               (rangegates+2)/1.024)                    ; in kB/sec

  if alg eq sppalg then acqtime=sppacqtime $
  else if alg eq pp4alg or alg eq pp6alg then acqtime=ppmacqtime $
  else if alg eq fftalg or alg eq dftalg then acqtime=fftacqtime

  tnumrec     = (fs.size-filehdrsz)/blocksize ; total # of recs 
  numrec      = 0                         ; the actual number of records read
  if fs.size ne tnumrec*blocksize+filehdrsz then wcrflag[0]=1

; Create string for the transmitted pulse sequence                      

  txstrpol = strarr(npulse)
  ind    = where(txsequence eq polv)
  if (ind[0] ne -1) then txstrpol[ind] = 'V'
  ind    = where(txsequence eq polH)
  if (ind(0) ne -1) then txstrpol[ind] = 'H'

; Create string array for the received pulse pair sequence                      

  if alg eq sppalg then begin 
    if rxsequence eq polV then ppstrpol='VV' else ppstrpol='HH' 
    txstrpol=txstrpol[0]
  endif else begin
    if npulse gt 1 and wcrdate gt 19990518 then begin
      ppstrpol = strarr(npulse-1,2)
      ind      = where(rxsequence eq polV)
      if (ind[0] ne -1) then ppstrpol[ind] = 'V'
      ind      = where(rxsequence eq polH)
      if (ind(0) ne -1) then ppstrpol[ind] = 'H'
      ppstrpol=reform(ppstrpol[*,0]+ppstrpol[*,1])
    endif else ppstrpol=replicate(' ',npulse-1>1)
  endelse

;-----------------------------------------------------------------------

; Prepare data for reading

; DATA structure for SPPMAG :
;     header     -  5 LONG  ; 40 before Apr 02,23 before Oct 96,25 before May 99
;     pulse pair -  2 FLOAT ; 1 pair (real, imag), H only; V added, Oct,22,96
;     power      -  2 FLOAT ; 2 magnitudes ( V, H ) in [mW]
; Every measurement is averaged of 4 pulses in the burst and 'avrages'/4
; times

; DATA structure for PPMAG4 :
;     header     -  5 LONG  ; 40 before Apr 02,23 before Oct 96,25 before May 99
;     pulse pair -  6 FLOAT ; 3 pairs (real, imag), H only, V added
;     power      -  8 FLOAT ; 8 magnitudes ( V, H ) in [mW]
; Every mesurement is averaged 'averages' times

; DATA structure for PPMAG6 :
;     header     -  5 LONG
;     pulse pair - 10 FLOAT ; 5 pairs (real, imag), H only, V added
;     power      - 12 FLOAT ; 12 magnitudes ( V, H ) in [mW]
; Every mesurement is averaged 'averages' times

; Note: After Oct 96, pulse pairs are recorded for H and V.  Therefore the
; notation, pp, pp12,pp23,pp34 represent HH, HV, VH and VV depending on
; the txsequence. For example HHVV would be:  pp12->HH, pp23->HV, pp34->VV
; After May 99 the receive sequence for pp can be selected and thus Rx_sequence
; determines what pule pairs are recorded.

; DATA structure for FFT :
;     header     -  5 LONG  ; 40 before Apr 02,23 before Oct 96,25 before May 99
;     |fft|^2    -  1 FLOAT ; HH or VV in [W/Hz] or [W/m/s]

; DATA structure for DPFFT :
;     header     -  5 LONG  ; 40 before Apr 02,23 before Oct 96,25 before May 99
;     |fft|^2    -  1 FLOAT ; HH or VV in [W/Hz] or [W/m/s]
;     |fft|^2    -  1 FLOAT ; VH or HV in [W/Hz] or [W/m/s]

; Note: First two records (0, 1 range gates) represent noise and Tx leakage,
; respectively

  if (alg eq sppalg) then begin

;    ppn    - (real, imag) pulse pair noise; H only
;    ppcal  - (real, imag) pulse pair calibration; H only
;    pp     - (real, imag)*rangegates pulse pair measurements; H only
;    magn   - V and H noise measurement
;    magcal - V and H calibration measurement
;    mag    - (V and H)*rangegates data measurements

     data=assoc(lun1,{header :lonarr(headervars)             , $
                      ppn    :fltarr(2,noisesamples)         , $  
                      ppcal  :fltarr(2,txsamples)            , $
                      pp     :fltarr(2,rangegates)           , $
                      magn   :fltarr(2,noisesamples)         , $
                      magcal :fltarr(2,txsamples)            , $
                      mag    :fltarr(2,rangegates)             $
                      },filehdrsz) 
   endif else if (alg eq pp4alg) then begin

     data=assoc(lun1,{header :lonarr(headervars)             , $
                      pp12n  :fltarr(2,noisesamples)         , $
                      pp12cal:fltarr(2,txsamples)            , $
                      pp12   :fltarr(2,rangegates)           , $
                      pp23n  :fltarr(2,noisesamples)         , $
                      pp23cal:fltarr(2,txsamples)            , $
                      pp23   :fltarr(2,rangegates)           , $
                      pp34n  :fltarr(2,noisesamples)         , $  
                      pp34cal:fltarr(2,txsamples)            , $
                      pp34   :fltarr(2,rangegates)           , $
                      mag1n  :fltarr(2,noisesamples)         , $
                      mag1cal:fltarr(2,txsamples)            , $
                      mag1   :fltarr(2,rangegates)           , $
                      mag2n  :fltarr(2,noisesamples)         , $
                      mag2cal:fltarr(2,txsamples)            , $
                      mag2   :fltarr(2,rangegates)           , $
                      mag3n  :fltarr(2,noisesamples)         , $
                      mag3cal:fltarr(2,txsamples)            , $
                      mag3   :fltarr(2,rangegates)           , $
                      mag4n  :fltarr(2,noisesamples)         , $
                      mag4cal:fltarr(2,txsamples)            , $
                      mag4   :fltarr(2,rangegates)             $
                      },filehdrsz)
   endif else if (alg eq pp6alg) then begin

     data=assoc(lun1,{header :lonarr(headervars)             , $
                      pp12n  :fltarr(2,noisesamples)         , $
                      pp12cal:fltarr(2,txsamples)            , $
                      pp12   :fltarr(2,rangegates)           , $
                      pp23n  :fltarr(2,noisesamples)         , $
                      pp23cal:fltarr(2,txsamples)            , $
                      pp23   :fltarr(2,rangegates)           , $
                      pp34n  :fltarr(2,noisesamples)         , $  
                      pp34cal:fltarr(2,txsamples)            , $
                      pp34   :fltarr(2,rangegates)           , $
                      pp45n  :fltarr(2,noisesamples)         , $
                      pp45cal:fltarr(2,txsamples)            , $
                      pp45   :fltarr(2,rangegates)           , $
                      pp56n  :fltarr(2,noisesamples)         , $
                      pp56cal:fltarr(2,txsamples)            , $
                      pp56   :fltarr(2,rangegates)           , $
                      mag1n  :fltarr(2,noisesamples)         , $
                      mag1cal:fltarr(2,txsamples)            , $
                      mag1   :fltarr(2,rangegates)           , $
                      mag2n  :fltarr(2,noisesamples)         , $
                      mag2cal:fltarr(2,txsamples)            , $
                      mag2   :fltarr(2,rangegates)           , $
                      mag3n  :fltarr(2,noisesamples)         , $
                      mag3cal:fltarr(2,txsamples)            , $
                      mag3   :fltarr(2,rangegates)           , $
                      mag4n  :fltarr(2,noisesamples)         , $
                      mag4cal:fltarr(2,txsamples)            , $
                      mag4   :fltarr(2,rangegates)           , $
                      mag5n  :fltarr(2,noisesamples)         , $
                      mag5cal:fltarr(2,txsamples)            , $
                      mag5   :fltarr(2,rangegates)           , $
                      mag6n  :fltarr(2,noisesamples)         , $
                      mag6cal:fltarr(2,txsamples)            , $
                      mag6   :fltarr(2,rangegates)             $
                      },filehdrsz)

   endif else if (alg eq fftalg) then begin

;    fftnoise - noise power spectral density 
;    fftcal   - cal. pulse psd
;    fftps    - data psd
;    averages - number of spectral lines (bins)

     data=assoc(lun1,{header  :lonarr(headervars)            , $
                      fftnoise:fltarr(fftbins,noisesamples)  , $
                      fftcal  :fltarr(fftbins,txsamples)     , $
                      fftps   :fltarr(fftbins,rangegates)      $ 
                      },filehdrsz)

   endif else if (alg eq dftalg) then begin

;    fftnoise - noise power spectral density 
;    fftcal   - cal. pulse psd
;    fftps    - data psd; fftps[*,*,0] - V-ch, fftps[*,*,1] - H-ch
;    fftbins  - number of spectral lines (bins)

     data=assoc(lun1,{header  :lonarr(headervars)            , $
                      fftnoise:fltarr(fftbins,noisesamples,2)  , $
                      fftcal  :fltarr(fftbins,txsamples,2)     , $
                      fftps   :fltarr(fftbins,rangegates,2)      $ 
                      },filehdrsz)
   endif else begin
     print, 'Incorrect algorithm type: ',alg
     print, 'Is the file correctly chosen ?'
     return
   endelse

;-----------------------------------------------------------------------

; Calculate start time and time interval (between profiles)
;
; starttime    - string, start recording date/time
; timeinterval - float, sampling interval in [sec]
; When time correction is applied it is assumed tcorrection+t.header[tind[0]]>0

  t=data(0)
  if swapbytes eq 1 then t=swap_endian(t)
  if tcorrection ne badf then starttime=double(tcorrection) else starttime=0.d0
  starttime=(starttime+double(t.header(tind[0]))+double(t.header(tind[1])/1e6)) mod 86400
  starttime=sec2timestr(starttime)+'.'+ $
            strmid(num2str(starttime-long(starttime),fr=2),2)

; Prepare the accurate start time (to be used for time interval calc.)
  timeinterval=double(t.header(tind[0]))+double(t.header(tind[1]))/1.e6

; Get last profile number

k=tnumrec-1
t=data(k)
if swapbytes eq 1 then t=swap_endian(t)

lstblocknum=t.header(bind)

; Check if it is a good one and if not find the last good profile for
; headers before March 2002 

; if wcrdate lt 20020331 then begin  ; THIS IS NOT FIXED YET
  if lstblocknum le 0 or lstblocknum gt 65536 then begin
     while (lstblocknum le 0 or lstblocknum gt 65536) do begin
       k=k-1
       if k eq -1 then lstblocknum=1 else begin
         t=data(k)
         if swapbytes eq 1 then t=swap_endian(t)
         lstblocknum=t.header(bind)
       endelse
    endwhile
  endif else begin       ; check if the last profile is bad or jumped 
    t=data(k-1)
    if swapbytes eq 1 then t=swap_endian(t)
    if t.header(bind)+1 ne lstblocknum then k=k-1
  endelse
  if k lt tnumrec-1 then begin
    print,'ATTN: ',num2str(tnumrec-1-k),' bad profs found at the end of the file'
    print,'Original prof #: ',num2str(tnumrec),', Corrected: ',num2str(k+1)
    if k lt 3 then message,'Too many bad profiles !'
    wcrflag[1]=tnumrec-k-1
    tnumrec=k+1
  endif         
;endif

; Calculate stop time, time interval (between profs), time length and datarate
; 
;  stopTime     - string, stop recording date/time
;  timeLenght   - float, time duration in [sec] of all recorded profiles
;  timeInterval - float, sampling interval in [sec]

k=tnumrec-1
t=data(k)
if swapbytes eq 1 then t=swap_endian(t)

if tcorrection ne badf then stoptime=double(tcorrection) else stoptime=0.d0
stoptime=(stoptime+double(t.header(tind[0]))+double(t.header(tind[1])/1e6)) mod 86400
stoptime=sec2timestr(stoptime)+'.'+ $
         strmid(num2str(stoptime-long(stoptime),fr=2),2)

timeinterval=float(abs(timeinterval-(double(t.header(tind[0]))+$
             double(t.header(tind[1]))/1.e6)))/(tnumrec-1.)
if timeinterval gt 1 then begin
   wcrflag[2]=1
   message,'Warning: DATE/TIME stamp problem.',/info
endif   
timelength=timeinterval*(tnumrec-1)
datarate=fs.size/timelength/1024.               ; in KB/sec

; Correct lstBlockNum if profiles went over 65536.
; It is assumed: 1) the time between starting wcr and data recording is less
; than approx. 1 hour and 2) the total clock time duration for one wcr file 
; does not exceed about 100 min.
;
; if fstBlockNum+numRec gt 65536 then lstBlockNum=lstBlockNum+65536

; After June 1999 the interrupts are obsolete and thus there is a simple
; way of getting the last profile.  lstBlockNum is not important param. and
; I am not going to maintain it to be correct for projects before 6/99.

  lstBlockNum=fstBlockNum+tnumRec

;t =timestr2sec(strmid(starttime,11,8))
;ty=timestr2sec(strmid(stoptime,11,8))
;if (ty lt t) then ty=ty+86400    ; correct time if midnight is passed 
;t= long((ty-t)/timeinterval)     ; total number of profiles since wcr started
;if (t+fstblocknum gt 65536) then lstblocknum=lstblocknum+65536

; Estimate of raw data images aspect ratio

aspectratio = rangegatespacing/(timeinterval*kaspeed)

; Form range gate vector (geometric center of the range gate)
  
radrng = rangegatedelay+findgen(rangegates)*rangegatespacing

; Maximum Doppler velocity 

maxvel=prf/4.*lambda 

; Create velo for FFT and do some checking for maxvel

if (alg eq fftalg or alg eq dftalg) then begin

   ; FFT spectra are caclulated with the first bin being DC (0 frequency) then 
   ; +df to +Nyquist Freq.(Fn) and then from -Fn+df to -df (df - freq. step).
   ; In other words the fft hasn't been shifted. 
   ; After shifting (done readwcrdata), DC is fftbin/2 index and -Fn is 0 index.
   ; Positive Doppler velocity (negative freq) is toward the radar (radar
   ; convention).

   ; Form velocity axis for shifted spectra.
   velo=-(findgen(fftbins)-fftbins/2.)*2.*maxvel/fftbins
endif else if alg eq sppalg then maxvel=maxvel[0] $
else if alg eq pp4alg then $
   if prf[0] eq prf[1] and prf[1] eq prf[2] then maxvel=maxvel[0] $ 
else if alg eq pp6alg then $
   if (prf(0) eq prf(1) and prf(1) eq prf(2) and $
      prf[2] eq prf[3] and prf[3] eq prf[4] and prf[4] eq prf[5]) then $
      maxvel=maxvel[0]

end
