;$Id: plotwcrcalm.pro,v 1.8 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRCALM
;
; PURPOSE:
;       Make calibration/test plots for data or corner reflector
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRCALM,dir
; 
; INPUTS:
;          Dir: string, wcr file directory to process all wcr files
;               undefined: use pickfile
;               'select': use pickfile with an option to select file(s)
;                         full path and name; process the given wcr file(s)
;
; KEYWORD PARAMETERS:
;        CRmax: output, fltarr(2,n), max returns in dBm for H- and V-ch
;       CRmean: output, fltarr(2,n), mean returns in dBm for the range
;               gate where the max values (CRmax) occur 
;       FILEID: identification string for wcr file: '20YY' (default '200')
;         HCOR: when set correct h channel w.r.t. v channel using 0.95 slope
;         VCOR: when set correct v channel w.r.t. v channel using default vals
;   BIG_ENDIAN: when set, assumes that the input file data are in big endian
;               order.  The default is LITTLE_ENDIAN.
;      INDJUMP: input string
;               'all'  - uses all profiles (default)
;               'even' - uses even profiles for calplot and stat
;               'odd'  - uses odd profiles for calplot and stat
;     NOSAVEPS: when set does not save the ps files(ignored if plotps not set)
;       PLOTPS: when set creates a ps file,(default is x device)
;               2 creates a ps file and skip cross-channel plots for ppmag
;               -1 uses x device and skip cross-channel plots for ppmag 
;      PRINTER: printer name (e.g.,'lj5local','lj4_gv','qms')
;               '': suppress printing (default)
;        PSDIR: string, directory for postscript file(s)
;               Default save ps files in subdir 'ps/', which is created
;               if non-existant.  When dir='select', ps directory is always
;               sub directory of the home directory, otherwise it is sub
;               of dir
;      TIMECOR: if defined gives UTC time correction of the wcr time
;               After June 1998, IRIG card is used to set the time.
;               Normal use of the card is to get the time from a master
;               IRIG card with GPS time.  The default value is set to
;               the leap seconds in openwcrf. 
;       TLABEL: string, page title at the bottom of the page
;               undefined - Experiment .... / Data
;               'cr' - will add Corner Reflector to the label
;               '' - prompt for a label
;               strarr(2), 1st string 'cr' or '', 2nd string: additonal text
;                          to be added before tlabel[0]
;               strarr(n), titles for each file to be processed
;                          (used usually for calibration titles)     
;         HELP: Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       four plots + three plots (2 pages)
;
; EXAMPLES:
;       Corner Reflector calibration plot
;
;       dir     = '/orel3/data99/cs99/'+d+'/'
;       psdir   = dir+d+'ps/'
;       fileid  = 'pp01-'
;       printer = 'lj5local'
;       tlabel  = "Coastal II '99 Data"
;       hcor    = 0
;       plotwcrcalm,dir,pri=printer,/plotps,fil=fileid,ind='all', $
;                   tlabel=['cr',tlabel],psdir=psdir,hcor=hcor
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Sep 1995
;       S.H., Aug 99 -- modified help info and other minor changes
;                    -- added timecor keyword
;       S.H., 11/99  -- partially restored int.cal. vars in wcrdata common
;                       block
;       S.H., 01/00  -- modified common blocks and help display
;       S.H., 04/00  -- fixed a typo and minor changes
;       S.H., 05/00  -- allowed tlabel to be array
;       S.H., 01/01  -- added little_endian keyword
;       S.H., 02/01  -- changed routine name from plotcalm to plotwcrcalm
;       S.H., 09/01  -- changed the position of the tlabel
;       S.H., 09/01  -- added CRmax and CRmean keywords
;       S.H., 01/02  -- modified to use readwcrdata instead of readwcrall
;       S.H., 04/02  -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrcalm,dirin,psdir=psdir,printer=printer,tlabel=tlbl,hcor=hcor,$
    vcor=vcor,plotps=plotps,nosaveps=saveps,indjump=ijump,fileid=fileid,$
    timecor=timecor,big_endian=bgnd,help=help,crmax=crmax,crmean=crmean

@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrcalm'
  return
endif

sm1 = 0             ; smooth for plotwcrnoise
sm2 = 0             ; smooth for plotwcrcr

if n_elements(printer) eq 0 then printer=''
if n_elements(plotps) eq 0 then plotps=0
if keyword_set(saveps) then saveps=0 else saveps=1
if n_elements(fileid) eq 0 then fileID='200'

if n_elements(tlbl) eq 0 then begin
   firstrg= 200
   lastrg = 3000
   tlabel="Experiment Data"
endif else if tlbl(0) eq 'cr' then begin
   firstrg= 120
   lastrg = 220
   tlabel="Corner Reflector"
endif else if tlbl(0) eq '' then begin
   firstrg= 120
   lastrg = 220
   tlabel=""
   read,"Enter test/data label: ",tlabel
endif else begin
   firstrg= 150
   lastrg = 3510
   tlabel=tlbl(0)
endelse

if n_elements(tlbl) eq 2 then tlabel=tlbl(1)+': '+tlabel

rgate   = 10          ; default range gate to be plotted
fileLen = 20

; Make wcr file and dir strings (string vectors)

if n_elements(dirin) eq 0 then begin
   flst=dialog_pickfile(/read,get_path=dir)
   spawn, 'ls '+dir+'?'+fileID+'*',flst
   flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,fileLen)
endif else if dirin eq 'select' then begin
   flst=dialog_pickfile(/read,/multi)
   dir=strsub(flst,0,strpos(flst(0),fileID,/reverse_search)-2)
   flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,fileLen)
endif else begin
   fpos=strpos(dirin[0],fileID,/reverse_search)
   if fpos ge 0 and strpos(dirin[0],'/',/reverse_search) ne strlen(dirin[0])-1 then $
   begin
     flst=strmid(dirin,fpos-1,fileLen)
     dir=strsub(dirin,0,fpos-2)
   endif else begin
     if strmid(dirin,strlen(dirin)-1,1) ne '/' then dir=dirin+'/' else dir=dirin
     spawn, 'ls '+dir+'?'+fileID+'*',flst
     flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,fileLen)
   endelse
endelse

nfiles=n_elements(flst)-1

; Make the postscript file directory

if (plotps ge 1 and saveps eq 1) then begin
   if n_elements(psdir) eq 0 then $
      if dirin[0] eq 'select' then psdir='~/ps' $ 
      else psdir=dir+'ps/' 
   if strmid(psdir,strlen(psdir)-1,1) ne '/' then psdir=psdir+'/'
   if psdir ne '.' and psdir ne './' then spawn,'mkdir '+psdir+' >& /dev/null'
endif

; Define crmax/crmean with big arrays (dont' wanna do something better now)
  crmax=fltarr(2,100)
  crmean=fltarr(2,100)
  crcount=0

; Process selected wcr file(s)

for k=0,nfiles do begin

    if n_elements(tlbl) eq nfiles+1 then tlabel=tlbl(k) $
    else if n_elements(tlbl) eq nfiles+2 then tlabel=tlbl[0]+' / '+tlbl(k+1) 

    fn=flst(k)
    dirk=dir(k*(n_elements(dir) gt 1))
    if strmid(dirk,strlen(dirk)-1,1) ne '/' then dirk=dirk+'/'

    print,'***** Processing '+fn+' ... *****'

    openwcrf,dirk+fn,data,timecor=timecor,big_endian=bgnd

    ; Check validity of firstrg and lastrg

    firstrg = firstrg>radrng[1]
    lastrg  = (lastrg > radrng[2])<radrng[rangegates-1]

    ; Determine the number of profiles to process based on 5 min segments

    profpersec=fix(1./timeinterval)
    segd=300L*profpersec
    seg=segd

    l=segd/4
    if tnumrec-segd lt l then begin seg=tnumrec & nseg=0 & end $
    else begin seg=segd & nseg=tnumrec/seg-(tnumrec mod seg lt l) & end

    for l=0,nseg do begin
       strt = l*seg
       if l eq nseg then stp=tnumrec-1 else stp = ((l+1)*seg-1)
       readwcrdata,strt,stp,timemode=3,big_endian=bgnd,hcor=hcor,vcor=vcor

       if (plotps ge 1) then $
        setxpswin,99,4-2*(alg eq fftalg),xyoff=[0.9,1.85],xysize=[7.2,9.],/noc $
       else setxpswin,0,4-3*(alg eq fftalg),color=41

       if (alg eq fftalg) then begin
          plotwcrfft,rgate=rgate
          xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
          printwcrprms,2,[0.08,-0.05,0.75]
          printwcrstat, 2,[0.52,-0.05,0.75],indjump=ijump
          datestamp,/user,com='      Page '+num2str(l+1)
       endif else begin
          zm = [0, 0, n_elements(prof)-1]
          !p.multi=[0,1,4] & plotwcrnoise,smooth=sm1,zoom=zm,indjump=ijump
          !p.multi=[2,1,4] & plotwcrcr, firstrg,lastrg,smooth=sm2,zoom=zm, $
                                        CRmax=crmx,CRmean=crmn
          if n_elements(crmx) eq 2 then begin
            crmax[*,crcount]=crmx
            crmean[*,crcount]=crmn
          endif else begin
            crmax[0,crcount]=crmx
            crmean[0,crcount]=crmn
          endelse
          crcount=crcount+1

          if (plotps lt 1 and alg ne sppalg) then pause else if alg eq pp4alg $
          or alg eq pp6alg then begin
             xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
             printwcrprms,2,[0.08,-0.05,0.75]
             printwcrstat, 2,[0.52,-0.05,0.75],indjump=ijump
             datestamp,/user,com='      Page '+num2str(l+1)+'-1'
          endif

          if plotps ge 0 and plotps lt 2 then begin
             !p.multi=[0,1,4]
             plotwcrcocr,zoom=zm,indjump=ijump,plotf=pflag,/ppmag
          endif else pflag=0

          if (plotps ge 1 and (alg eq pp4alg or alg eq pp6alg) and pflag eq 0) $
          then begin
             xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
             printwcrprms,2,[0.08,-0.05,0.75]
             printwcrstat, 2,[0.52,-0.05,0.75],indjump=ijump
             datestamp,/user,com='      Page '+num2str(l+1)+'-2'
          endif else if (plotps ge 1) then begin
             xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
             printwcrprms,2,[0.08,-0.05,0.75]
             printwcrstat, 2,[0.52,-0.05,0.75],indjump=ijump
             datestamp,/user,com='      Page '+num2str(l+1)
          endif
       endelse

       if (plotps ge 1) then begin
          resetps2x
          if printer ne '' then spawn,'lpr -h -P'+printer+' idl.ps'
          if saveps then $
             spawn,'mv idl.ps '+ psdir+wcrfile+'.cal'+ $
             num2str(sec2time(timestr2sec(begtime)),fr=0)+'.ps' $
          else spawn,'rm idl.ps'
       endif else if k lt nfiles then pause
       hcorrection=intarr(8)
       vcorrection=intarr(8)
    endfor     ; end for l
    closewcrf
endfor   ; end for k

crmax=crmax(*,0:(crcount-1)>0)
crmean=crmean(*,0:(crcount-1)>0)

end
