;$Id: plotwcrcalm1.pro,v 1.7 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRCALM1
;
; PURPOSE:
;       Make isolation plots (plotwcrcocr) only
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRCALM1,dir
;
; INPUTS:
;          Dir: string, wcr file directory to process all wcr files
;               undefined: use pickfile
;               'select': use pickfile with an option to select file(s)
;                         full path and name; process the given wcr file(s)
;
; KEYWORD PARAMETERS:
;        PSDIR: string, directory for postscript file(s)
;               Default save ps files in subdir 'ps/', which is created
;               if non-existant.  When dir='select', ps directory is always
;               sub directory of the home directory, otherwise it is sub
;                 of dir
;      PRINTER: printer name (e.g.,'lj5local', 'lj4_gv', 'qms')
;               undefined: lj5local
;               '': suppress printing
;       TLABEL: string, page title at the bottom of the page
;               undefined - Experiment .... / Data
;               'cr' - will print Experiment ... /Corner Reflector
;               '' - prompt for a label
;               strarr(2), 1st string as above, 2nd string is appended
;         HCOR: when set correct h channel w.r.t. v channel using 0.95 slope
;         VCOR: when set correct v channel w.r.t. v channel using default vals
;       PLOTPS: when set creates ps files (default is x device)
;     NOSAVEPS: when set does not save the ps files(ignored if plotps not set)
;      INDJUMP: input string
;               'all'  - uses all profiles (default)
;               'even' - uses even profiles for calplot and stat
;               'odd'  - uses odd profiles for calplot and stat
;
;       FILEID: identification string for wcr file: 'ppYY-' (default 'pp01-')
;       TIMECOR: if defined gives UTC time correction of the wcr time
;                After June 1998, IRIG card is used to set the time.
;                Normal use of the card is to get the time from a master
;                IRIG card with GPS time.  The default value is set to
;                the leap seconds in openwcrf. 
;    BIG_ENDIAN: when set, assumes that the input file data are in big endian
;                order.  The default is LITTLE_ENDIAN.
;         HELP: Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       three plots
;       does not handle FFT wcr files
;
; EXAMPLES:
;       Corner Reflector calibration plot ("isolation", gain imbalance plots)
;
;       dir     = '/orel3/data99/cs99/'+d+'/'
;       psdir   = dir+d+'ps/'
;       fileid  = 'pp01-'
;       printer = 'lj5local'
;       tlabel  = "Coastal II '99 Data"
;       hcor    = 0
;       plotwcrcalm1,dir,pri=printer,/plotps,fil=fileid,ind='all', $
;                tlabel=['cr',tlabel],psdir=psdir,hcor=hcor,vcor=vcor
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Sep 1995
;       S.H., Aug 99 -- modified help info and other minor changes
;                       added timecor keyword
;       S.H., 11/99  -- partially restored int.cal. vars in wcrdata common
;                       block
;       S.H., 01/00  -- modified common blocks and help display
;       S.H., 01/01  -- added little_endian keyword
;       S.H., 02/01  -- changed routine name from plotcalm1 to plotwcrcalm1
;       S.H., 01/02  -- modified to use readwcrdata instead of readwcrall
;       S.H., 04/02  -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrcalm1,dir,psdir=psdir,printer=printer,tlabel=tlabel, $
    plotps=plotps,nosaveps=saveps,indjump=ijump,fileid=fileid,$
    timecor=timecor,hcor=hcor,vcor=vcor,big_endian=bgnd,help=help

@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrcalm1'
  return
endif

if n_elements(printer) eq 0 then printer='lj5local'
if keyword_set(plotps) then plotps=1 else plotps=0
if keyword_set(saveps) then saveps=0 else saveps=1
if n_elements(fileid) eq 0 then fileID='pp01-'

if n_elements(tlabel) eq 0 then begin
   firstrg= 200
   lastrg = 3000
   tlabel="Experiment Data"
endif else if tlabel eq 'cr' then begin
   firstrg= 120
   lastrg = 300
   tlabel="Corner Reflector"
endif else if tlabel eq '' then begin
   firstrg= 120
   lastrg = 300
   read,"Enter test/data label: ",tlabel
endif else begin
   firstrg= 150
   lastrg = 3500
endelse

rgate  = 10          ; default range gate to be plotted
fileLen= 20

; Make wcr file and dir strings (string vectors)

if n_elements(dir) eq 0 then begin
   flst=dialog_pickfile(/read,get_path=dir)
   spawn, 'ls '+dir+'?'+fileID+'*',flst
   flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,fileLen)
endif else if dir eq 'select' then begin
   flst=dialog_pickfile(/read,/multi)
   dir=strsub(flst,0,strpos(flst(0),fileID,/reverse_search)-2)
   flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,fileLen)
endif else begin
   fpos=strpos(dir(0),fileID,/reverse_search)
   if fpos ge 0 and strpos(dirsel,'/',/reverse_search) ne strlen(dirsel)-1 then $
   begin
      flst=strmid(dir,fpos-1,fileLen)
      dir=strsub(dir,0,fpos-2)
   endif else begin
      if strmid(dir,strlen(dir)-1,1) ne '/' then dir=dir+'/'
      spawn, 'ls '+dir+'?'+fileID+'*',flst
      flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,fileLen)
   endelse
endelse

nfiles=n_elements(flst)-1

; Make the postscript file directory

if (plotps eq 1 and saveps eq 1) then begin
   if n_elements(psdir) eq 0 then $
      if n_elements(dir) gt 1 then psdir='~/ps/' $
      else psdir=dir+'ps/'
   if strmid(psdir,strlen(psdir)-1,1) ne '/' then psdir=psdir+'/'
   if psdir ne '.' and psdir ne './' then spawn,'mkdir '+psdir+' >& /dev/null'
endif

; Process selected wcr file(s)

for k=0,nfiles do begin
    fn=flst(k)
    dirk=dir(k*(n_elements(dir) gt 1))
    if strmid(dirk,strlen(dirk)-1,1) ne '/' then dirk=dirk+'/'

    print,'***** Processing '+fn+' ... *****'
    openwcrf,dirk+fn,data,timecor=timecor,big_endian=bgnd

    ; Determine the number of profiles to process based on 5 min at a time;

    profpersec=fix(1./timeinterval)
    segd=300L*profpersec
    seg=segd

    l=segd/4
    if tnumrec-segd lt l then begin seg=tnumrec & nseg=0 & end $
    else begin seg=segd & nseg=tnumrec/seg-(tnumrec mod seg lt l) & end

    for l=0,nseg do begin
       strt = l*seg
       if l eq nseg then stp=tnumrec-1 else stp = ((l+1)*seg-1)
       if (alg eq fftalg) then goto, NEXT $
       else begin
         readwcrdata,strt,stp,timemode=3,big_endian=bgnd,hcor=hcor,vcor=vcor
       endelse

       if (plotps eq 1) then $
        setxpswin,99,4-2*(alg eq fftalg),xyoff=[0.9,1.85],xysize=[7.2,9.],/noc $
       else setxpswin,0,4-3*(alg eq fftalg),color=41

       zm = [0, 0, n_elements(prof)-1]
       !p.multi=[0,1,4] & plotwcrcocr,zoom=zm,indjump=ijump,plotf=pflag
       xyouts,0.08,-0.03,tlabel,charsize=0.9,/normal,/noclip
       printwcrprms,2,[0.08,-0.05,0.75]
       printwcrstat, 2,[0.52,-0.05,0.75],indjump=ijump
       datestamp,/user,com='      Page '+num2str(l+1)

       if plotps eq 1 then begin
          resetps2x
          if pflag eq 0 then begin
            if printer ne '' then spawn,'lpr -h -P'+printer+' idl.ps'
            if saveps eq 1 then $
               spawn,'mv idl.ps '+ psdir+wcrfile+'.calcr'+num2str(l)+'.ps' ; $
            ;else spawn,'rm idl.ps'
          endif
       endif else if k lt nfiles then pause
       hcorrection=intarr(8)
       vcorrection=intarr(8)
    endfor     ; end for l
    closewcrf
NEXT:
endfor   ; end for k

end
