;$Id: plotwcrcr.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRCR
;
; PURPOSE:
;       This program uses corner reflector power measurements already in the 
;       common block to plot hh, vv, hv, vh at the range gate of maximum
;       reflection. This program is normally used by running plotwcrcalm,  
;       plotwcrcalm1 or plotwcrcrm.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRCR,firstrg,lastrg
;
; INPUTS:
;       Firstrg:  first range gate in meters for the expected
;                 location of the corner reflector (default, 100 m)
;       Lastrg:   last range gate in meters for the expected
;                 location of the corner reflector (default, 300 m)
;
; KEYWORD PARAMETERS:
;       CRmax:   output, [CRmaxH,CRmaxV], max returns in dBm for H- and V-ch
;       CRmean:  output, [CRmeanH,CRmeanV], mean returns in dBm for the range
;                gate where the max values (CRmax) occur 
;       SMOOTH:  number of smoothing points (0, default)
;       ZOOM:    input array [ readmode, str, stp ]
;                readmode: 0  - str/stp, start/stop record numbers
;                          1  - str/stp, start/stop profile numbers
;                          2  - str/stp, start/stop UTC in hhmmss
;       HELP:    show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       two plots
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, May 1999
;       S.H. Nov 98 -- disabled internal calibration
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Dec 99 -- complied with the new pp arrays 
;       S.H., Jan 00 -- added help keyword
;       S.H., Mar 00 -- added check for valid range gates
;       S.H., Feb 01 -- changed routine name from plotcrcal to plotwcrcr
;       S.H., Sep 01 -- added CRmax and CRmean keywords
;       S.H., Oct 01 -- replaced atan(complex) with atan(imaginary(complex),
;                                                       float(complex))
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrcr, firstrg,lastrg,SMOOTH=smo,ZOOM=zm,HELP=help, $
               CRmax=crmax,CRmean=crmean

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrcr'
  return
endif

if (n_params() le 1) then begin firstrg = 120 > radrng[1]
                                lastrg  = (300 > radrng[5])<radrng[rangegates-1]
                          endif
if (n_params() eq 1) then lastrg = 3000

if (KEYWORD_SET(zm) eq 0) then zm=[0, 0, n_elements(prof)-1]
if (KEYWORD_SET(smo) eq 0) then sm=0 else sm=smo
if (sm lt 3) then sm=0

rgind = wcrzoomrange([1,firstrg,lastrg]) ; range gates where max cal. value is
if rgind[0] eq -1 then message,'Input ranges are not valid'
procrg = n_elements(rgind)

wcrzoomprof,zm,ind,btime,etime
nprof = n_elements(ind)-1

nprof=n_elements(prof(ind))-1
if (nprof le 2) then sm=0
if (nprof lt 2*sm+2) then begin
   print, wcrfile,': ',strtrim(string(nprof+1),2), $
         ' profiles to process !  Skipping the file. '
   return
endif

if (sm gt 0) then begin
   nprof=nprof-sm
   ind=ind(sm:nprof)
   nprof=nprof-sm
endif

xtoplabel= 'Profile'
sp40 = '                                        '
xbotlabel= bTime+sp40+'Time (sec)'+sp40+eTime
pcharsize=1.7

  if (alg eq pp4alg or alg eq pp6alg) then begin

      vvcrmax = 10*alog10(max(vv(rgind,*),vvind))      ; V pol CR max value
      vvrgmax = rgind(0) + (vvind mod procrg)          ; V pol CR max range gate
      vvprofmax = vvind / procrg                       ; V pol CR max profile
;     vTcrmax = 10*alog10(vvtx(vvprofmax))             ; V pol TrnsmtPow @ max
      hvcrmax = 10*alog10(hv(vvrgmax,vvprofmax))       ; V crosspol @ CR max
      vvcrmn  = 10*alog10(mean(vv(vvrgmax,*)))       ; V pol CR mean value

      hhcrmax = 10*alog10(max(hh(rgind,*),hhind))      ; H pol CR max value
      hhrgmax = rgind(0) + (hhind mod procrg)          ; H pol CR max range gate
      hhprofmax = hhind / procrg                       ; H pol CR max profile
;     hTcrmax = 10*alog10(hhtx(hhprofmax))             ; H pol TrnsmtPow @ max
      vhcrmax = 10*alog10(vh(hhrgmax,hhprofmax))       ; H crosspol @ CR max
      hhcrmn  = 10*alog10(mean(hh(hhrgmax,*)))         ; H pol CR mean value

      if (sm eq 0) then begin
         vv1 = vv(vvrgmax,ind)
         hv1 = vv(vvrgmax,ind)
         hh1 = hh(hhrgmax,ind)
         vh1 = vh(hhrgmax,ind)
      endif else begin
         vv1=smooth(vv(vvrgmax,*),sm) & vv1=vv1(ind)
         hv1=smooth(hv(vvrgmax,*),sm) & hv1=hv1(ind)
         hh1=smooth(hh(hhrgmax,*),sm) & hh1=hh1(ind)
         vh1=smooth(vh(hhrgmax,*),sm) & vh1=vh1(ind)
      endelse

      hhnm=mean(hhnoise)
      vvnm=mean(vvnoise)

      crmax =[hhcrmax,vvcrmax]
      crmean=[hhcrmn,vvcrmn]

;     Plot pulse pair (pp) in m/s

      if sm eq 0 then begin         ; do some smoothing anyway
         sm1=9
         ind1=ind(sm1:nprof-sm1)
      endif else begin ind1=ind & sm1=sm & end

      if txsequence(0) eq polh and txsequence(1) eq polh then begin
         ylabel='pp12 Velocity + (m/s)'
         maxv=prf(0)/4*lambda
         if (sm1 eq 0) then begin
            pp1=reform(maxv*atan(imaginary(pp(hhrgmax,0,ind1)), $
                                 float(pp(hhrgmax,0,ind1)))/!pi)
            sw=alog(((hh1-hhnm)>1.e-15)/abs(pp(hhrgmax,0,ind1)))
            ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
         endif else begin 
            pp1=reform(maxv*atan(imaginary(pp(hhrgmax,0,*)), $
                                 float(pp(hhrgmax,0,*)))/!pi)
            pp1=smooth(pp1,sm1) & pp1=pp1(ind1)
            sw=alog(((hh(hhrgmax,*)-hhnm)>1.e-15)/abs(pp(hhrgmax,0,*)))
            ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
            ppsw=smooth(ppsw,sm1) & ppsw=ppsw(ind1)
         endelse
      endif else $
      if txsequence[1] eq polh and txsequence[2] eq polh then begin
         ylabel='pp23 Velocity + (m/s)'
         maxv=prf(1)/4*lambda
         if (sm1 eq 0) then begin 
            pp1=reform(maxv*atan(imaginary(pp(hhrgmax,1,ind1)), $
                                 float(pp(hhrgmax,1,ind1)))/!pi) 
            sw=alog(((hh1-hhnm)>1.e-15)/abs(pp(hhrgmax,1,ind1)))
            ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
         endif else begin 
            pp1=reform(maxv*atan(imaginary(pp(hhrgmax,1,*)), $
                                 float(pp(hhrgmax,1,*)))/!pi) 
            pp1=smooth(pp1,sm1) & pp1=pp1(ind1)
            sw=alog(((hh(hhrgmax,*)-hhnm)>1.e-15)/abs(pp(hhrgmax,1,*)))
            ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
            ppsw=smooth(ppsw,sm1) & ppsw=ppsw(ind1)
         endelse
      endif else $
      if txsequence[2] eq polh and txsequence[3] eq polh then begin
         ylabel='pp34 Velocity + (m/s)'
         maxv=prf(2)/4*lambda
         if (sm1 eq 0) then begin
            pp1=reform(maxv*atan(imaginary(pp(hhrgmax,2,ind1)), $
                                 float(pp(hhrgmax,2,ind1)))/!pi)
            sw=alog(((hh1-hhnm)>1.e-15)/abs(pp(hhrgmax,2,ind1)))
            ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
         endif else begin 
            pp1=reform(maxv*atan(imaginary(pp(hhrgmax,2,*)), $
                                 float(pp(hhrgmax,2,*)))/!pi) 
            pp1=smooth(pp1,sm1) & pp1=pp1(ind1)
            sw=alog(((hh(hhrgmax,*)-hhnm)>1.e-15)/abs(pp(hhrgmax,2,*)))
            ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
            ppsw=smooth(ppsw,sm1) & ppsw=ppsw(ind1)
         endelse
      endif else begin
         ylabel='pp12 Velocity + (m/s)'
         maxv=prf(0)/4*lambda
         if (sm1 eq 0) then begin
            pp1=reform(maxv*atan(imaginary(pp(hhrgmax,0,ind1)), $
                                 float(pp(hhrgmax,0,ind1)))/!pi)
            sw=alog(((hh1-hhnm)>1.e-15)/abs(pp(hhrgmax,0,ind1)))
            ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
         endif else begin 
            pp1=reform(maxv*atan(imaginary(pp(hhrgmax,0,*)), $
                                 float(pp(hhrgmax,0,*)))/!pi)
            pp1=smooth(pp1,sm1) & pp1=pp1(ind1)
            sw=alog(((hh(hhrgmax,*)-hhnm)>1.e-15)/abs(pp(hhrgmax,0,*)))
            ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
            ppsw=smooth(ppsw,sm1) & ppsw=ppsw(ind1)
         endelse
      endelse

      !p.charsize=pcharsize
      subplot=!P.MULTI
      minpp = min(pp1)   & maxpp = max(pp1)
      minppw= min(ppsw) & maxppw= max(ppsw)
      yr = [minpp-0.05*(maxpp-minpp),maxpp+0.15*(maxpp-minpp)]
      yrw= [minppw-0.05*(maxppw-minppw),maxppw+(maxppw-minppw)]
      plot, time(ind1),pp1,yrange=yr,ystyle=5,xstyle=1,ticklen=1,symsize=0.3, $
            xgridstyle=1,ygridstyle=1,xmargin=[8,8],ymargin=[4,4],psym=1, $
            xtitle=xbotlabel,title='smooth='+num2str(sm1,fr=0)+spc(70)
      axis, yaxis=0,ytitle=ylabel,ystyle=1
      !P.MULTI = subplot
      plot,time(ind1),ppsw,psym=3,ystyle=5,xstyle=5,/noerase,yrange=yrw, $
           xmargin=[8,8],ymargin=[4,4]
      axis, yaxis=1,ytitle='Velocity width ... [m/s]',ystyle=1
       oplot,[time(ind1(0)+20),time(ind1(n_elements(ind1)-1))],[0,0]


      if !P.MULTI(2) gt 2 then $
         if !P.MULTI(0) eq 0 then !P.MULTI(0)=!P.MULTI(2)-1 $
         else  !P.MULTI(0)=!P.MULTI(0)-1

;     Correlation coefficients for the trnsmt. power for V and H

      Cvvhv=strmid(strtrim(string(round(100*correlate(vv1,hv1))/100.),1),0,4)
      Chhvh=strmid(strtrim(string(round(100*correlate(hh1,vh1))/100.),1),0,4)
      Cvvhh=strmid(strtrim(string(round(100*correlate(vv1,hh1))/100.),1),0,4)
      Cvhhv=strmid(strtrim(string(round(100*correlate(vh1,hv1))/100.),1),0,4)

;     Correlation coefficients for the trnsmt. and received power 

;      Cvvvt=strmid(strtrim(round(100*correlate(vv1,vvtx(ind)))/100.,1),0,4)
;      Chhht=strmid(strtrim(round(100*correlate(hh1,hhtx(ind)))/100.,1),0,4)
;      Cvvht=strmid(strtrim(round(100*correlate(vv1,hhtx(ind)))/100.,1),0,4)

      xbotlabel=xbotlabel+'!C'                                              + $
          'VVmax='+num2str(vvcrmax,fr=1)                                    + $
          'dB, HV@max='+num2str(hvcrmax,fr=1)                               + $
          'dB, VVmn='+ num2str(vvcrmn,fr=1)                                 + $
;         'dB, vT='  + num2str(vTcrmax,fr=1)                                + $
          'dB, Vrg=' + num2str(rangegatedelay+rangegatespacing*vvrgmax,fr=0)+ $
          'm, Vprof='+ num2str(prof(vvprofmax),fr=0) + '!C'                 + $
          'HHmax='   + num2str(hhcrmax,fr=1)                                + $
          'dB, VH@max='+num2str(vhcrmax,fr=1)                               + $
          'dB, HHmn='+ num2str(hhcrmn,fr=1)                                 + $
;         'dB, hT='  + num2str(hTcrmax,fr=1)                                + $
          'dB, Hrg=' + num2str(rangegatedelay+rangegatespacing*hhrgmax,fr=0)+ $
          'm, Hprof='+ num2str(prof(hhprofmax),fr=0)
      ylabelL='vv: -, hv: -. (dBm)'
      ylabelR='hh: --, vh: ...'


      text= 'Cvvhv='+Cvvhv+',  Chhvh='+Chhvh+',  Cvvhh='+Cvvhh+ $
            ',  Cvhhv='+Cvhhv+ $
;           ',  Cvvht='+Cvvht+',  Chhht='+Chhht+ $
            '; smooth: '+num2str(sm,fr=0)

      mx = max([max(10*alog10(vv1)), max(10*alog10(hh1)), $
                max(10*alog10(vh1)), max(10*alog10(hv1))])

      mn = min([min(10*alog10(vv1)), min(10*alog10(hh1)), $ 
                min(10*alog10(vh1)), min(10*alog10(hv1))])
      yr = [mn-0.05*(mx-mn), mx+0.15*(mx-mn)]

      subplot=!P.MULTI
      plot, prof(ind),10*alog10(vv1), $ 
            yrange=yr,ystyle=5,xstyle=5,ymargin=[6,4],xmargin=[8,8]
      axis, xaxis=1,xtitle=xtoplabel,xstyle=1
      axis, yaxis=0,ytitle=ylabelL,ystyle=1
      oplot,prof(ind),10*alog10(hh1),line=2
      oplot,prof(ind),10*alog10(vh1),line=1
      !P.MULTI = subplot
      plot, time(ind),10*alog10(hv1), line=3, $ 
            yrange=yr,ystyle=5,xstyle=5,ymargin=[6,4],xmargin=[8,8],/noerase
      axis, xaxis=0,xtitle=xbotlabel,xstyle=1
      axis, yaxis=1,ytitle=ylabelR,ystyle=1

   endif else $

   if (alg eq sppalg) then begin

      if (txsequence[0] eq polh) then begin
         copol=hh
         crpol=vh
;        coT=hhtx
         coN=mean(hhnoise)
      endif else begin
         copol=vv
         crpol=hv
;        coT=vvtx
         coN=mean(vvnoise)
      endelse
                                                     ; V/H pol CR max value
      cocrmax = 10*alog10(max(copol(rgind,ind(0):ind(nprof)),coind)) 
      corgmax = rgind(0) + (coind mod procrg)        ; V/H pol CR max range gate
      coprofmax = ind(0) + coind / procrg            ; V/H pol CR max profile
;     coTcrmax = 10*alog10(coT(coprofmax))           ; V/H pol TrnsmtPow @ max
      crcrmax = 10*alog10(crpol(corgmax,coprofmax))  ; V/H crosspol @ max
                                                     ; V/H pol CR mean value
      cocrmn  = 10*alog10(mean(copol(corgmax,ind(0):ind(nprof)))) 

      crmax =cocrmax
      crmean=cocrmn

;     Plot pulse pair (pp) in m/s

      if sm eq 0 then begin         ; do some smoothing to the pp anyway
         sm1=9
         ind1=ind(sm1:nprof-sm1)
      endif else begin ind1=ind & sm1=sm & end

      ylabel='pp Velocity + (m/s) '
      maxv=prf(0)/4*lambda
      if (sm1 eq 0) then begin
         pp1=maxv*atan(imaginary(pp(corgmax,ind1)),float(pp(corgmax,ind1)))/!pi
         sw=alog(((copol(corgmax,ind1)-coN)>1.e-15)/abs(pp(corgmax,ind1))) 
         ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
      endif else begin
         pp1=maxv*atan(imaginary(pp(corgmax,*)),float(pp(corgmax,*)))/!pi
         sw=alog(((copol(corgmax,*)-coN)>1.e-15)/abs(pp(corgmax,*)))  
         ppsw=maxv*sqrt(2)/!pi*sqrt(abs(sw))*sign(sw)
         pp1=smooth(pp1,sm1) & pp1=pp1(ind1)
         ppsw=smooth(ppsw,sm1) & ppsw=ppsw(ind1)
      endelse

      if (sm eq 0) then begin
         copol = copol(corgmax,ind)
         crpol = crpol(corgmax,ind)
      endif else begin
         copol=smooth(copol(corgmax,*),sm1) & copol=copol(ind)
         crpol=smooth(crpol(corgmax,*),sm1) & crpol=crpol(ind)
      endelse

      !P.CHARSIZE=pcharsize
      subplot=!P.MULTI
      minpp = min([min(pp1),min(ppsw)])
      maxpp = max([max(pp1),max(ppsw)])
      minpp = min(pp1)   & maxpp = max(pp1)
      minppw= min(ppsw) & maxppw= max(ppsw)
      yr = [minpp-0.05*(maxpp-minpp),maxpp+0.15*(maxpp-minpp)]
      yrw= [minppw-0.05*(maxppw-minppw),maxppw+(maxppw-minppw)]
      plot, time(ind1),pp1,yrange=yr,ystyle=5,xstyle=1,ticklen=1,symsize=0.3, $
            xgridstyle=1,ygridstyle=1,xmargin=[8,8],ymargin=[4,4], $
            xtitle=xbotlabel,title='smooth='+num2str(sm1,fr=0)+spc(70),psym=1
      axis, yaxis=0,ytitle=ylabel,ystyle=1
      !P.MULTI = subplot
      plot,time(ind1),ppsw,psym=3,ystyle=5,xstyle=5,/noerase,yrange=yrw, $
           xmargin=[8,8],ymargin=[4,4]
      axis, yaxis=1,ytitle='Velocity width ... [m/s]',ystyle=1
      oplot,[time(ind1(0)+20),time(ind1(n_elements(ind1)-1))],[0,0]

      Ccocr=strmid(strtrim(string(round(100*correlate(copol, $ 
                   crpol))/100.),1),0,4)

;     Correlation coefficients for the trnsmt. and received power 

;      Ccocot=strmid(strtrim(round(100*correlate(copol,cot(ind)))/100.,1),0,4)
        
      if (txsequence(0) eq polh) then begin
         text= 'Chhvh='+Ccocr   + $
;              ', Chhht='+Ccocot+ $
               '; smooth: '+num2str(sm,fr=0)
         xbotlabel=xbotlabel+'!C'                                            + $
           'HHmax='+ num2str(cocrmax,fr=1)                                   + $
           'dB, VH@max='+num2str(crcrmax,fr=1)                               + $
           'dB, HHmn='+ num2str(cocrmn,fr=1)                                 + $
;          'dB, hT='  + num2str(coTcrmax,fr=1)                               + $
           'dB, Hrg=' + num2str(rangegatedelay+rangegatespacing*corgmax,fr=0)+ $
           'm, Hprof='+ num2str(prof(coprofmax),fr=0)
         ylabel='hh: -, vh: ... (dBm)'

      endif else begin
         text='Cvvhv='+Ccocr   + $
;             ', Cvvvt='+Ccocot+ $
              '; smooth: '+num2str(sm,fr=0)
         xbotlabel=xbotlabel+'!C'                                            + $
           'VVmax='+ num2str(cocrmax,fr=1)                                   + $
           'dB, HV@max='+num2str(crcrmax,fr=1)                               + $
           'dB, VVmn='+ num2str(cocrmn,fr=1)                                 + $
;          'dB, vT=' + num2str(coTcrmax,fr=1)                                + $
           'dB, Vrg=' + num2str(rangegatedelay+rangegatespacing*corgmax,fr=0)+ $
           'm, Vprof='+ num2str(prof(coprofmax),fr=0)
         ylabel='vv: -, hv: ... [dBm]' 

      endelse

      mx = max([max(10*alog10(copol)), max(10*alog10(crpol))])
      mn = min([min(10*alog10(copol)), min(10*alog10(crpol))])
    
      yr = [mn-0.05*(mx-mn),mx+0.15*(mx-mn)]

      if !P.MULTI(2) gt 2 then $
         if !P.MULTI(0) eq 0 then !P.MULTI(0)=!P.MULTI(2)-1 $
         else  !P.MULTI(0)=!P.MULTI(0)-1

      subplot=!P.MULTI
      plot, prof(ind),10*alog10(copol),ytitle=ylabel, $
            yrange=yr,ystyle=1,xstyle=5,ymargin=[6,4],xmargin=[8,8]
      axis, xaxis=1,xtitle=xtoplabel,xstyle=1

      !P.MULTI = subplot
      plot, time(ind),10*alog10(crpol), ytitle=ylabel, line=1, $ 
            yrange=yr,ystyle=1,xstyle=5,ymargin=[6,4],xmargin=[8,8],/noerase
      axis, xaxis=0,xtitle=xbotlabel,xstyle=1

  endif

  xyouts, time(ind(0))+0.1*(time(nprof)-time(ind(0))), $
          mx+0.05*(mx-mn), text,charsize=0.7
  print,text
  pos=strpos(xbotlabel,'!C',120)
  if pos ne -1 then begin
     print,strmid(xbotlabel,108,pos-108)
     print,strmid(xbotlabel,pos+2,strlen(xbotlabel)-1)
  endif else print,strmid(xbotlabel,108,strlen(xbotlabel)-1)
  print,'**********************************'

end
