;$Id: plotwcrcrm.pro,v 1.6 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRCRM
;
; PURPOSE:
;       Make calibration/test plots for data or corner reflector 
;       using a single wcr file or data in the common blocks
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRCRM
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;       HCOR:    when set correct h channel w.r.t. v channel using 0.95 slope  
;       VCOR:    when set correct v channel w.r.t. v channel using default vals 
;       INDJUMP: input string
;                'all'  - uses even and odd profiles (default)
;                'even' - uses even profiles only
;                'odd'  - uses odd profiles only
;       NOSAVEPS:when set does not save the ps files(ignored if plotps not set) 
;       PLOTPS:  when set creates a ps file,(default is x device)
;                2 creates a ps file and skip cross-channel plots for ppmag
;                -1 uses x device and skip cross-channel plots for ppmag 
;       PRINTER: string for the printer, (default, 'lj5local')
;                printer='' will supress printing
;       RANGE:   [firstrg, lastrg], search between firstrg  and lastrg range
;                gates  in meters for the return from the corner reflector 
;                (default [120,350])
;       READWCR: when set reads data from common block otherwise from a file
;                (default, 0)
;       SETXWIN: set window manager (default, 0)
;       SMOOTH:  number of smoothing points (0, default)
;                when input [sm1,sm2] performes corresponding different 
;                smoothings for calplot (noise and cal signal) and calcrplot
;                (received signal)
;       TIMECOR: if defined gives UTC time correction of the wcr time
;                After June 1998, IRIG card is used to set the time.
;                Normal use of the card is to get the time from a master
;                IRIG card with GPS time.  The default value is set to
;                the leap seconds in openwcrf. 
;       TLABEL:  string for graphics title (default, 'CORNER REFLECTOR')
;       ZOOMPROF:input array [ rmode, str, stp ]
;                rmode:    0  - str/stp, start/stop record numbers
;                          1  - str/stp, start/stop profile numbers
;                          2  - str/stp, start/stop UTC in hhmmss
;     BIG_ENDIAN:when set, assumes that the input file data are in big endian
;                order.  The default is LITTLE_ENDIAN.
;       HELP:    Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       four plots + three plots (2 pages)
;
; EXAMPLES:
;       Corner Reflector calibration plot
;
;       .run openwcrm
;       readwcrdata,data
;       tlabel  = "Coastal II '99 Data/Corner Reflector"
;       plotwcrcrm,range=[120,250],pri=printer,plotps=0,ind='all', $
;                  tlabel=tlabel,/readwcr,setx=1
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Dec 1995
;       S.H., Aug 99 -- modified help info and other minor changes
;                       added timecor keyword 
;       S.H., 11/99  -- partially restored int.cal. vars in wcrdata common 
;                       block
;       S.H., Jan 00 -- added help keyword
;       S.H., Jan 01 -- added little_endian keyword
;       S.H., Feb 01 -- changed routine name from plotcrcalm to plotwcrcrm
;       S.H., Jan 02 -- modified to use readwcrdata instead of readwcrall,
;                       obsolete rmode keyword
;       S.H., Apr 02 -- changed LITTLE_ENDIAN keyword to BIG_ENDIAN
;
; OBSOLETE KEYWORDS:
;       RMODE:   0 : reads profs given strt/stp record/position number(default)
;                1 : reads profiles w/ numbers between strt/stp
;                2 : reads profiles within strt/stp UTC
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrcrm, smooth=sm,range=rg,plotps=ps,setxwin=xwin,noerase=e,hcor=hcor, $
                readwcr=readwcr,printer=pr,tlabel=tlabel,rmode=rm,zoomprof=zm, $
                nosaveps=saveps,indjump=ijump,timecor=timecor,vcor=vcor, $
                big_endian=bgnd,help=help

@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrcrm'
  return
endif

if (n_elements(sm) eq 0) then sm1 = (sm2=0) $
else if (n_elements(sm) eq 1) then sm1 = (sm2=sm(0)) else begin
   sm1=sm(0) & sm2=sm(1)
endelse
if (KEYWORD_SET(rg) eq 0) then begin
   firstrg= 120
   lastrg = 350
endif else begin
   firstrg=rg(0)
   lastrg=rg(1)
endelse

if n_elements(ps) eq 0 then ps=0 
if (KEYWORD_SET(xwin) eq 0) then xwin=0
if keyword_set(saveps) then saveps=0 else saveps=1

; readwcr : 0 - read wcr file, 1 - use common block data    
if (KEYWORD_SET(readwcr) eq 0) then readwcr=0 

if (n_elements(pr) eq 0) then pr='lj5local'
if (KEYWORD_SET(tlabel) eq 0) then tlabel='Experiment / Corner Reflector'

if (readwcr eq 0) then fn=pickfile(/read)

if ps gt 0 then begin
   setxpswin,99,4,xyoff=[0.9,1.85],xysize=[7.2,9.],/noc
endif else $
   setxpswin,xwin,4,color=41

if (readwcr eq 0) then begin
   openwcrf,fn,timecor=timecor,big_endian=bgnd
   if (KEYWORD_SET(rm) eq 0) then begin
      readm=0 & str=0 & stp=tnumrec-1
   endif else begin
      readm=rm(0) & str=rm(1) & stp=rm(2)
   endelse
   readwcrdata,str,stp,timemode=3,big_endian=bgnd,hcor=hcor,vcor=vcor
endif else time=time-time[0]

; Check validity of firstrg and lastrg

firstrg = firstrg>radrng[1]
lastrg  = (lastrg > radrng[5])<radrng[rangegates-1]

if (KEYWORD_SET(zm) eq 0) then zm = [0, 0, n_elements(prof)-1]

!p.multi=[0,1,4] & plotwcrcal,smooth=sm1,zoom=zm,indjump=ijump
!p.multi=[2,1,4] & plotwcrcr, firstrg,lastrg,smooth=sm2,zoom=zm

xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
printwcrprms,2,[0.08,-0.04,0.75]
printwcrstat, 2,[0.52,-0.04,0.75]

if alg eq pp4alg or alg eq pp6alg then begin
  datestamp,/user,com='      Page 1'+'-1'
  if ps eq 0 then pause
  if ps eq 0 or ps eq 1 then begin
    !p.multi=[0,1,4] & plotwcrcocr,zoom=zm,indjump=ijump,plotf=pflag
  endif  

  xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
  printwcrprms,2,[0.08,-0.04,0.75]
  printwcrstat, 2,[0.52,-0.04,0.75]
  datestamp,/user,com='      Page 1'+'-2'
endif else datestamp ,/user,com='      Page 1'

if ps gt 0 then begin
   resetps2x
   if (pr ne '') then spawn,'lpr -h -P'+pr+' idl.ps'
   if saveps then $
      spawn,'mv idl.ps '+ wcrfile+'.cal'+ $
      num2str(sec2time(timestr2sec(begtime)),fr=0)+'.ps' $
   else spawn,'rm idl.ps'
endif

end
