;$Id: plotwcrfft.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRFFT
;
; PURPOSE:
;       This program uses wcr data already in the common block to plot, fftps
;       at given range gate, fftTx and fftnoise. This program is normally used 
;       by running plotwcrfftm or plotwcrtestm. (fftTx plot has been disabled)
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRFFT
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;       PLOTPS:  99 - creates a ps file,(0 to 31 is x device)
;       RGATE:   input integer, when given plots corresponding range gate
;                default is 10 (0 is the first range gate) 
;       SMOOTH:  number of smoothing points (0, default)
;                [sm1,sm2], sm1 - vel. smoothing points, sm2 - profile smoothing
;       TLABEL:  string, page title at the bottom of the page
;                ( default 'Experiment Data')
;       ZOOM:    input array [ startvel, stopvel ] for velocity plots only
;       PROFILE: when set determines record number to be plotted for vel. plots
;                (default is the average of all)
;       HELP:    Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       two pages w/ 3 plots each
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Dec 1995
;       S.H., Nov 98 -- Disabled inetrnal cal. measurements
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Jan 00 -- added help keyword
;       S.H., Nov 00 -- added +- 1 sigma for the vel. noise plot
;                       added noise,leak and signal power plots
;                       modified smoothing, added plotps
;       S.H., Jan 01 -- minor bugs fixed
;       S.H., Feb 01 -- changed routine name from plotfftcal to plotwcrfft
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrfft,SMOOTH=sm,ZOOM=zm,Profile=profb,rgate=rgate,tlabel=tlabel, $
               plotps=plotps,help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrfft'
  return
endif

; Check input

  if n_elements(tlabel) eq 0 then tlabel='Experiment data'
  if n_elements(plotps) eq 0 then plotps=0

  nprof=n_elements(fftnoise(0,*))

  if (n_elements(sm) eq 0) then begin 
     sm1=0
     sm2=0
  endif else begin 
     if n_elements(sm) eq 1 then begin
         if (sm lt 3) then sm=0
         sm1=sm & sm2=sm
     endif else begin
         if (sm[0] lt 3) then sm1=0 else sm1=sm[0]
         if (sm[1] lt 3) then sm2=0 else sm2=sm[1]
     endelse
  endelse

   
  if (KEYWORD_SET(profb) eq 0) then profb=-1 $
  else if (profb lt -1) then profb=-1 $
  else if (profb gt nprof-1) then profb=nprof-1

  v=velo

  if (KEYWORD_SET(zm) eq 0) then zm=[min(v),max(v)]

  ind=where(v ge zm(0) and v le zm(1))
  nvel=n_elements(v[ind])-1
  
if (alg eq fftalg) then begin

;  In the following segment fftN and fftTx are averaged with fftbins^2
;  instead of fftbins to account for dt (time interval) = 1 (no time scaling)

   if (profb eq -1) then begin
      fftT=total(ffttx,2)/nprof
      fftT=10.*alog10(fftT(ind)/fftbins^2)
      fftN=total(fftnoise,2)/nprof
      fftN2=std(fftnoise,2)
      fftN1=10*alog10((fftN[ind]-fftN2[ind])/fftbins^2 > 1e-15)
      fftN2=10*alog10((fftN[ind]+fftN2[ind])/fftbins^2)
      fftN=10.*alog10(fftN[ind]/fftbins^2)
   endif else $
   if (sm1 eq 0) then begin
      fftT=10*alog10(ffttx(ind,profb))
      fftN=10*alog10(fftnoise[ind,profb])
   endif else begin
      fftT=smooth(ffttx(*,profb),sm1)
      fftT=10*alog10(fftT(ind)/fftbins^2)
      fftN=smooth(fftnoise(*,profb),sm1)
      fftN=10*alog10(fftN(ind)/fftbins^2)
   endelse

   xtoplabel=''
   xbotlabel='Velocity [m/s]'
   ylabelL='fftTx: -, [dBm/(m/s)]'
   ylabelR='fftNoise: ... , [dBm/(m/s)] ' 

   if profb eq -1 then begin
      mx = max(fftN2)
      mn = min(fftN1) > (-90.)
   endif else begin
      mx = max(fftN)
      mn = min(fftN)
   endelse
   yr = [mn-0.05*(mx-mn),mx+0.15*(mx-mn)]

   if plotps lt 32 then setxpswin,plotps,2,color=41 $
   else begin
      setxpswin,99,3,xyoff=[0.9,1.85],xysize=[7.2,8.5],/noc
      !p.charsize=1.7
   endelse

   plot, v(ind),fftN, yrange=yr,ystyle=5,xstyle=5,xmargin=[8,8],ymargin=[4,4]
   grid,yrange=yr,xmargin=[8,8],ymargin=[4,4]
      axis, xaxis=1,xtitle=xtoplabel,xstyle=1
      axis, yaxis=1,ytitle=ylabelR,ystyle=1
      axis, yaxis=0,ytitle=ylabelR,ystyle=1
      axis, xaxis=0,xtitle=xtoplabel,xstyle=1
   if profb eq -1 then begin
      oplot,v(ind),fftN1,line=2
      oplot,v(ind),fftN2,line=2
   endif

   if (profb eq -1) then $ 
      text='smoothed spectra: '+ num2str(nprof) $
   else $
      text='smooth: '+ num2str(sm1)+', record: '+num2str(profb)

   xyouts,-abs(v[ind[0]])+0.1*abs(v[ind[nvel]]-v[ind[0]]),mx+0.05*(mx-mn), $
          text,charsize=0.8

   mx = max(fftT)
   mn = min(fftT)
   yr = [mn-0.05*(mx-mn),mx+0.15*(mx-mn)]

   plot, v(ind),fftT,yrang=yr,ystyl=5,xstyl=5,xmargi=[8,8],ymargi=[4,4],psym=-2
   grid,yrange=yr,xmargin=[8,8],ymargin=[4,4]
      axis, xaxis=1,xtitle=xbotlabel,xstyle=1
      axis, yaxis=1,ytitle=ylabelL,ystyle=1
      axis, yaxis=0,ytitle=ylabelL,ystyle=1
      axis, xaxis=0,xtitle=xbotlabel,xstyle=1

   if (profb eq -1) then $ 
      text='bias: '+ num2str((v[where(fftT eq max(fftT))])[0])+' m/s' $
   else $
      text='bias: '+num2str(v[where(fftT eq max(fftT))])+' m/s'

   xyouts,-abs(v[ind[0]])+0.1*abs(v[ind[nvel]]-v[ind[0]]),mx+0.05*(mx-mn), $
          text,charsize=0.8

;   contour,10.*alog10(ffttx(ind,*)),v(ind),findgen(nprof), $
;           ystyle=5,xstyle=5,xmargin=[8,8],ymargin=[4,4]
;      axis, xaxis=1,xtitle=xtoplabel,xstyle=1
;      axis, xaxis=0,xtitle=xbotlabel,xstyle=1
;      axis, yaxis=1,ytitle='',ystyle=1
;      axis, yaxis=0,ytitle='Prof. Record',ystyle=1

    if ((size(time))(0) gt 0) then begin
       xtime=time
       xtitle=['[sec]','[dBm]'] 
    endif else begin
       xtime=findgen(nprof)
       xtitle=['record #','[dBm]']
    endelse
    ttl=wcrfile+': Doppler Spectra @ '+num2str(radrng(10))+' m'

; Plot signal spectra at rgate range gate if no input
 
    if n_elements(rgate) eq 0 then rgate=10
    ttl=wcrfile+': Doppler Spectra @ '+num2str(radrng(rgate),fr=0)+' m'

    if not (!d.flags and 1) then $
      aximage,10.*alog10(reform(fftps(ind,rgate,*))/fftbins^2),xtime,/order,$
      title=ttl,reverse(v(ind)),bar=1,setw=-1,/tr,xtitle=xtitle, $
      ytitle='[m/s]',/timeax,asp=round(aspectratio) $
    else $
      aximage,10.*alog10(reform(fftps(ind,rgate,*))/fftbins^2),xtime,/order,$
      title=ttl,reverse(v(ind)),bar=1,/tr,xtitle=xtitle,ytitle='[m/s]', $
      /timeax,charsize=1.5 ;,/revcol

    if (!d.flags and 1) then begin
       xyouts,0.08,-0.03,tlabel,charsize=0.9,/normal,/noclip
       printwcrprms,2,[0.08,-0.05,0.75]
       printwcrstat, 2,[0.52,-0.05,0.75],indjump=ijump
       datestamp,/user,com=wcrfile+' '+begtime+'-'+endtime+'      Page 1'
    endif else setxpswin,(plotps+1)<32,3

; Plot fft noise, leakage, and signal power (corresponding range gate)

    fftN=total(fftnoise,1)/fftbins^2
    fftT=total(ffttx,1)/fftbins^2
    fftN1=total(reform(fftps[*,rgate,*]),1)/fftbins^2 ; signal power at rgate

    if sm2 gt 0 then begin
      fftN=10.*alog10(smooth(fftN,sm2))
      fftT=10.*alog10(smooth(fftT,sm2))
      fftN1=10.*alog10(smooth(fftN1,sm2))
    endif else begin
      fftN=10.*alog10(fftN)
      fftT=10.*alog10(fftT)
      fftN1=10.*alog10(fftN1)
    endelse

    mx = max(fftN)
    mn = min(fftN)
    yr = [mn-0.05*(mx-mn),mx+0.15*(mx-mn)]
    text='smooth: '+ num2str(sm2)

    plot,time,fftN,ytit='fftnoise [dBm]',ystyle=1,xstyle=5, $
         yrange=yr,psym=3,charsize=1.7
    xyouts,time[0]+0.1*(time[nprof-1]-time[0]),mx+0.05*(mx-mn), $
         text,charsize=0.8
    timeaxis,tit='UTC',charsize=1.7

    plot,time,fftT,ytit='fftTx [dBm]',ystyle=1,xstyle=5,psym=3,charsize=1.7
    timeaxis,title='UTC',charsize=1.7

    plot,time,fftN1,ytit='fftPower [dBm]',ystyle=1,xstyle=5, $
         tit='Received Power: '+num2str(rgate,fr=0)+' range gate ('+   $
         num2str(radrng(rgate),fr=0)+'m)',charsize=1.7
    timeaxis,title='UTC',charsize=1.7,/major
    grid,charsize=1.7,/ygrid

    if (!d.flags and 1) then begin
       xyouts,0.08,-0.03,tlabel,charsize=0.9,/normal,/noclip
       printwcrprms,2,[0.08,-0.05,0.75]
       printwcrstat, 2,[0.52,-0.05,0.75],indjump='all'
       datestamp,/user,com=wcrfile+' '+begtime+'-'+endtime+'      Page 2'
       resetps2x
    endif
endif else $
    message, 'Not an FFT file !'

end
