;$Id: plotwcrtest.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRTEST
;
; PURPOSE:
;       This program makes plots of noise, leakage, 0 and last range gate
;       data for data quality checks. The data must be already loaded into
;       the wcr common block and the time vector has to be in seconds or
;       hhmmss. This program works for non-FFT files. For FFT files use 
;       plotwcrfftm or plotwcrfft.pro.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRTEST
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;       INDJUMP: input string
;                'all'  - uses even and odd profiles (default)
;                'even' - uses even profiles only
;                'odd'  - uses odd profiles only
;       LPAGE:   input, integer, leading page number (e.g., 1 in page 1-3)
;                (default, no leading number, e.g., page 3)
;       NOISO:   when set does not plot co/cross ratios
;       NOLEAK:  when set does not plot Tx pulse leakage
;       NONOISE: when set does not plot noise channels and time
;       NORG:    when set does not plot data for range gates
;       RANGEG:  input, intarr of up to 3 elements with range gates
;                (default [0,1,rangegates-1])
;       SETWIN:  input integer, X window display (0-31) (default is 0)
;                99 - postscript device
;       SMOOTH:  number of smoothing points (0, default)
;       TLABEL:  string, page title at the bottom of the page
;                ( default 'Experiment Data')
;       ZOOM:    input array [ readmode, str, stp ]
;                readmode: 0  - str/stp, start/stop record numbers
;                          1  - str/stp, start/stop profile numbers
;                          2  - str/stp, start/stop UTC in hhmmss
;       HELP:    show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       Creates plots using the current graphics device (usually x or ps)
;       Time vector is allowed in two formats:  seconds or hhmmss.dd
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Jan 2000
;       S.H., Jan 00 - added help keyword
;       S.H., Apr 00 - fixed typo in hh indexing
;       S.H., Feb 01 - changed routine name from plotvxitest to plotwcrtest
;       S.H., Oct 01 - replaced atan(complex) with atan(imaginary(complex),
;                                                       float(complex))
;       S.H., Apr 02 - modify to work with multiple noise samples in one
;                      profile by averaging them
;       S.H., May 02 - added time jumps check plot (commented out ppn plot)
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrtest, smooth=smo,zoom=zm,indjump=ijump,noiso=noiso,noleak=noleak,$
                 nonoise=nonoise,rangeg=rngg,norg=norg,setwin=setwin, $
                 tlabel=tlabel,lpage=lpage,help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrtest'
  return
endif

; Check the input

  if alg ne sppalg and alg ne pp4alg and alg ne pp6alg then begin
     message,'Not a sppmag or ppmag file. Exit.',/info
     return
  endif

  if n_elements(ijump) eq 0 then ijump='all'
  if n_elements(tlabel) eq 0 then tlabel='Experiment data'
  if n_elements(lpage) eq 0 then lpage='' else lpage=num2str(lpage)+'-'

  if (KEYWORD_SET(zm) eq 0) then zm=[0, 0, numrec-1]
  if (KEYWORD_SET(smo) eq 0) then sm=0 else sm=smo
  if sm lt 3 then sm=0 else if sm mod 2 eq 0 then sm=sm+1

  if KEYWORD_SET(noleak) then noleak=1 else noleak=0
  if KEYWORD_SET(noiso) then noiso=1 else noiso=0
  if KEYWORD_SET(nonoise) then nonoise=1 else nonoise=0
  if KEYWORD_SET(norg) then norg=1 else norg=0
  if KEYWORD_SET(setwin) eq 0 then setwin=0

; Determine zoom and jump indices

  wcrzoomprof, zm,ind,btime,etime

  if strlowcase(ijump) eq 'even' then $
     ind=ind(lindgen(n_elements(ind)/2)*2) $
  else if strlowcase(ijump) eq 'odd' then $
     ind=ind(lindgen((n_elements(ind)-1)/2)*2+1) $
  else ijump='all'

; Find if there are enough profiles to process

  nprof=n_elements(ind)
  if (nprof le 2) then sm=0
  if (nprof lt 2*sm+2) then begin
     print, wcrfile,': ',strtrim(string(nprof+1),2), $
           ' profiles to process !  Exit. '
     return
  endif

; Check if smoothing is requested

  if (sm gt 0) then begin
     nprof=nprof-sm
     ind=ind(sm:nprof)
     nprof=nprof-sm+1
  endif

; Determine range gates to be plotted

  if n_elements(rngg) eq 0 then rangeg=[0,1,rangegates-1] $
  else if n_elements(rngg) eq 1 then rangeg=[rngg,1,rangegates-1] $
  else if n_elements(rngg) eq 2 then rangeg=[rngg,rangegates-1] $
  else rangeg=rngg

  ; Find mid rg between 5th and rangegates-5 with maximum average reflectivity
  ; using h-ch; if h-ch not available use v-ch

  ;  l=5  ; begin/end range gates limit
  ;  if alg eq sppalg and txsequence[0] eq polh or alg eq pp4alg $
  ;  or alg eq pp6alg then begin
  ;     if size(hh,/n_dimensions) eq 2 then $
  ;        rgsel=max(mean(hh[5:rangegates-5,*],2),k) $
  ;     else $
  ;        rgsel=max(mean(reform(hh[5:rangegates-5,0,*]),2),k)
  ;  endif else rgsel=max(mean(vv[5:rangegates-5,*],2),k)
  ;  rgsel=[0,k+l,rangegates-1]

; Initilize graphics device and some parameters

  page=1
  color=128
  chsize=1.7
  maxnplots=4        ; plot no more than 4/3 pulses/pp
  xlrm=[2,8]         ; x-axis left and right margins(for most of the plots)
  ybtm=[3,2]         ; y-axis bottom and top margins(for most of the plots)

  ; Determine number of plots for the noise channels (2*nplots)
  ; nplots is limited to maximum of maxnplots

  if size(hhnoise,/n_dimensions) eq 1 then nplots=1 $
  else if size(hhnoise,/n_dimensions) eq 2 then nplots=npulse<maxnplots $
  else begin
    message,"Multiple noise samples. Averaged.",/info
    hhnoise=mean(hhnoise,1)
    vvnoise=mean(vvnoise,1)
    nplots=npulse<maxnplots
  endelse

  ; Determine the number of plots for pp noise channel (max maxnplots-1 plots)

  ; pplots=1+((npulse<maxnplots)-2)*(alg eq pp4alg or alg eq pp6alg)
  pplots=1  ; serves the time plot

  ; Determine number of plots for the first page (noise channels)

  np=2*nplots+pplots

  ; and set the device for the first page, next pages have !p.multi redefined

  setxpswin,setwin,np
  !p.charsize=chsize

; Prepare bottom and top x-axes (time in secs and record number)

  xbotlabel= 'UTC'
  xtoplabel= 'Profile'+'(smooth='+num2str(sm)+')'

  if timemode le 1 then $
     message,'There is no valid time vector.' $
  else if timemode eq 2 then xb=time2sec(time[ind]) $
  else if timemode eq 3 then xb=timestr2sec(btime)+time[ind] $
  else xb=time[ind]

  xt=prof[ind]-prof[0]

; Plots for: hhnoise, vvnoise, ppnoise

if nonoise eq 0 then begin
  ; Prepare hh and vv noise data

  if (sm eq 0) then begin
     if nplots eq 1 then begin
        hhn=hhnoise[ind]
        vvn=vvnoise[ind]
     endif else begin
        hhn=hhnoise[*,ind]
        vvn=vvnoise[*,ind]
     endelse
  endif else begin
     if nplots eq 1 then begin
        hhn=(smooth(hhnoise,sm))[ind]
        vvn=(smooth(vvnoise,sm))[ind]
     endif else begin
        hhn=fltarr(nplots,nprof)
        vvn=fltarr(nplots,nprof)
        for k=0,nplots-1 do begin
           hhn[k,*]=(smooth(hhnoise[k,*],sm))[ind]
           vvn[k,*]=(smooth(vvnoise[k,*],sm))[ind]
        endfor
     endelse
  endelse

  ; Extract phase noise in m/s and smooth it if requested
  ; ppn is not used currently and is replaced by time plot 

;  if n_elements(maxvel) eq 1 then begin
;    if pplots eq 1 then begin
;        ppn=reform(maxvel*atan(imaginary(ppnoise),float(ppnoise)))/!pi
;        if sm gt 0 then ppn=(smooth(ppn,sm))[ind] else ppn=ppn[ind]
;    endif else begin
;       ppn=reform(maxvel*atan(imaginary(ppnoise[0:pplots-1,*]), $
;                              float(ppnoise[0:pplots-1,*])))/!pi
;        if sm gt 0 then $
;           for k=0,pplots-1 do ppn[k,0:nprof-1]=(smooth(ppn[k,*],sm))[ind] $
;        else ppn=ppn[*,ind]
;    endelse
;  endif else $
;    if sm eq 0 then ppn=reform(replicarr(maxvel[0:pplots-1],nprof)* $
;       atan(imaginary(ppnoise[0:pplots-1,ind]), $
;            float(ppnoise[0:pplots-1,ind])))/!pi  $
;    else for k=0,pplots-1 do ppn[k,0:nprof-1]= $
;       (smooth(reform(replicarr(maxvel[0:pplots-1],numrec)* $
;        atan(imaginary(ppnoise[0:pplots-1,*]), $
;             float(ppnoise[0:pplots-1,*])))/!pi,sm))[ind]

  ; Prepare time plot.  Calculate time intervals

  ; Check if we crossed midnight
  k = (abs(max(time)-min(time)) gt 70000.)

  dt=(shift(time,-1)-time)[0:numrec-2]
  if k then begin
     junk=where(abs(dt) gt 70000.)
     if junk[0] ne -1 and n_elements(junk) eq 1 then dt[junk]=0.
  endif

  ; Do the plotting

  yhlabel='hhn [dBm]'
  yvlabel='vvn [dBm]'
;  yplabel='ppn [m/s]'
  yplabel='TimeInt [s]'

  hhn = 10*alog10(hhn)
  vvn = 10*alog10(vvn)

  ; hh and vv noise plots

  if nplots eq 1 then begin

    mxn = max(hhn)
    mnn = min(hhn)
    yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]

    plot, xb,hhn,psym=3,color=color, $
          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=xlrm,yrange=yrn
    timeaxis,/xbottom,/nolabels
    axis, yaxis=1,ytitle=yhlabel,ystyle=1
    lastplot
    plot, xt,ind,/noerase,/nodata, $
          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=xlrm,yrange=yrn
    axis, xaxis=1,xtitle=xtoplabel,xstyle=1,xtickformat='(I6)'
    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
    nextplot

    mxn = max(vvn)
    mnn = min(vvn)
    yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]

    plot, xb,vvn,psym=3,color=color, $
          ystyle=5,xstyle=5,ymargin=[2,0],xmargin=xlrm,yrange=yrn
    timeaxis,/xbottom,charsize=chsize
    axis, yaxis=1,ytitle=yvlabel,ystyle=1
    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)

  endif else begin    ; for nplots gt 1 plot the first 4 pulses only

    ; hh noise plots
    mxn = max(hhn)
    mnn = min(hhn)
    yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]

    ; First plot
    plot, xb,hhn[0,*],psym=3,color=color, $
          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=xlrm,yrange=yrn
    timeaxis,/xbottom,/nolabels
    axis, yaxis=1,ystyle=1,ytitle=yhlabel ;,yticks=3
    lastplot
    plot, xt,ind,/noerase,/nodata, $
          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=xlrm,yrange=yrn
    axis, xaxis=1,xtitle=xtoplabel,xstyle=1,xtickformat='(I6)'
    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
    nextplot
    ; Middle plots
    for k=1,nplots-2 do begin
      plot, xb,hhn[k,*],psym=3,color=color, $
         ystyle=5,xstyle=5,ymargin=[0,0],xmargin=xlrm,yrange=yrn
      timeaxis,/xbottom,/nolabels
      axis, yaxis=1,ystyle=1
      axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
    endfor
    ; Last plot
    plot, xb,hhn[nplots-1,*],psym=3,color=color, $
         ystyle=5,xstyle=5,ymargin=[0,0],xmargin=xlrm,yrange=yrn
    timeaxis,/xbottom,/nolabels
    axis, yaxis=1,ystyle=1,ytitle=yhlabel
    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)

    ; vv noise plots
    mxn = max(vvn)
    mnn = min(vvn)
    yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]

    ; First plot
    plot, xb,vvn[0,*],psym=3,color=color, $
          ystyle=5,xstyle=5,ymargin=[0,0],xmargin=xlrm,yrange=yrn
    timeaxis,/xbottom,/nolabels
    axis, yaxis=1,ystyle=1,ytitle=yvlabel
    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
    ; Middle plots
    for k=1,nplots-1 do begin  ; if using Last plot change to nplots-2
      plot, xb,vvn[k,*],psym=3,color=color, $
         ystyle=5,xstyle=5,ymargin=[0,0],xmargin=xlrm,yrange=yrn
      timeaxis,/xbottom,/nolabels
      axis, yaxis=1,ystyle=1
      axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
    endfor
;    ; Last plot  (disabled and added to the middle plots)
;    plot, xb,vvn[nplots-1,*],psym=3,color=color, $
;         ystyle=5,xstyle=5,ymargin=[2,0],xmargin=xlrm,yrange=yrn
;    timeaxis,/xbottom,charsize=chsize
;    axis, yaxis=1,ystyle=1 ;,ytitle=yvlabel ;,yticks=3
;    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)

  endelse

  ; pp noise plots; Disabled and replaced with dt plot (attached to noise plots)

  mxn = max(dt)
  mnn = min(dt)
  yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]

  if nplots eq 1 then begin
    plot, xt,dt,psym=1,symsize=0.5,ystyle=5,xstyle=5,ymargin=ybtm,xmargin=xlrm,$
          yrange=yrn
    axis, xaxis=1,xstyle=1,xtickformat='(I6)'
    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
    lastplot
    plot, xb,dt,/noerase,/nodata, $
          ystyle=5,xstyle=5,ymargin=ybtm,xmargin=xlrm,yrange=yrn
    timeaxis,title=xbotlabel,/xbottom,charsize=chsize
    axis, yaxis=1,ytitle=yplabel,ystyle=1
  endif else begin
    plot, xb,dt,psym=1,symsize=0.5, $
         ystyle=5,xstyle=5,ymargin=[2,0],xmargin=xlrm,yrange=yrn
    timeaxis,title=xbotlabel,/xbottom,charsize=chsize
    axis, yaxis=1,ystyle=1,ytitle=yplabel
    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
  endelse

;  mxn = max(ppn)
;  mnn = min(ppn)
;  yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]

;  if pplots eq 1 then begin
;    plot, xb,ppn,psym=3,color=color, $
;          ystyle=5,xstyle=5,ymargin=ybtm,xmargin=xlrm,yrange=yrn
;    oplot,[xb[0],xb[nprof-1]],[0,0],line=2
;    timeaxis,title=xbotlabel,/xbottom,charsize=chsize
;    axis, yaxis=1,ytitle=yplabel,ystyle=1
;    lastplot
;    plot, xt,ind,/noerase,/nodata, $
;          ystyle=5,xstyle=5,ymargin=ybtm,xmargin=xlrm,yrange=yrn
;    axis, xaxis=1,xstyle=1,xtickformat='(I6)'
;    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)

;  endif else begin
;    ; First plot
;    plot, xb,ppn[0,*],psym=3,color=color, $
;          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=xlrm,yrange=yrn
;    oplot,[xb[0],xb[nprof-1]],[0,0],line=2
;    timeaxis,/xbottom,/nolabels
;    axis, yaxis=1,ystyle=1,ytitle=yplabel,yticks=3
;    lastplot
;    plot, xt,ind,/noerase,/nodata, $
;          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=xlrm,yrange=yrn
;    axis, xaxis=1,xstyle=1,xtickformat='(I6)'
;    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
;    nextplot
;    ; Middle plots
;    for k=1,pplots-2 do begin
;      plot, xb,ppn[k,*],psym=3,color=color, $
;         ystyle=5,xstyle=5,ymargin=[0,0],xmargin=xlrm,yrange=yrn
;      oplot,[xb[0],xb[nprof-1]],[0,0],line=2
;      timeaxis,/xbottom,/nolabels
;      axis, yaxis=1,ystyle=1
;      axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
;    endfor
;    ; Last plot
;    plot, xb,ppn[pplots-1,*],psym=3,color=color, $
;         ystyle=5,xstyle=5,ymargin=[2,0],xmargin=xlrm,yrange=yrn
;    oplot,[xb[0],xb[nprof-1]],[0,0],line=2
;    timeaxis,title=xbotlabel,/xbottom,charsize=chsize
;    axis, yaxis=1,ystyle=1,ytitle=yplabel,yticks=3
;    axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
;  endelse

  if  setwin[0] eq 99 then begin
     printwcrstamp,tlabel,page=lpage+num2str(page),indjump=ijump,statt=stattext
     page=page+1
  endif else if noiso eq 0 or noleak eq 0 or norg eq 0 then pause

endif

; Prepare page 2 (leakage and isolation)

  np=2*(noleak eq 0)+$
       (noiso eq 0)*((alg eq sppalg)+3*(alg eq pp4alg or alg eq pp6alg))
  !p.multi=[0,0,np,0,0]

; Plot the leakage

if noleak eq 0 then begin

  ; Prepare h and v leakage data (use only 0 pulse if not averaged)

  if (sm eq 0) then begin
     if alg eq sppalg and txsequence[0] eq polh then begin
        vl=vvlk[ind]
        hl=hhtx[ind]
     endif else if alg eq sppalg and txsequence[0] eq polv then begin
        vl=vvtx[ind]
        hl=hhlk[ind]
     endif else if alg eq pp4alg or alg eq pp6alg then begin
        if size(hhtx,/n_dimensions) eq 1 then hl=hhtx[ind] else $
           hl=reform(hhtx[0,ind])
        if size(vvtx,/n_dimensions) eq 1 then vl=vvtx[ind] else $
           vl=reform(vvtx[0,ind])
     endif
  endif else begin
     if alg eq sppalg and txsequence[0] eq polh then begin
        vl=(smooth(vvlk,sm))[ind]
        hl=(smooth(hhtx,sm))[ind]
     endif else if alg eq sppalg and txsequence[0] eq polv then begin
        vl=(smooth(vvtx,sm))[ind]
        hl=(smooth(hhlk,sm))[ind]
     endif else if alg eq pp4alg or alg eq pp6alg then begin
        if size(hhtx,/n_dimensions) eq 1 then hl=(smooth(hhtx,sm))[ind] $
        else hl=(reform(smooth(hhtx[0,*],sm)))[ind]
        if size(vvtx,/n_dimensions) eq 1 then vl=(smooth(vvtx,sm))[ind] $
        else vl=(reform(smooth(vvtx[0,*],sm)))[ind]
     endif
  endelse

  yhlabel='hhl [dBm]'
  yvlabel='vvl [dBm]'

  hl=10*alog10(hl)
  vl=10*alog10(vl)

  ; h leakage plot

  mxn = max(hl)
  mnn = min(hl)
  yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]

  plot, xb,hl,psym=3,color=color,chars=chsize-0.5*(!p.multi[2] eq 2), $
        ystyle=5,xstyle=5,ymargin=ybtm,xmargin=xlrm,yrange=yrn
  timeaxis,/xbottom,charsize=chsize-0.5*(!p.multi[2] eq 2)
  axis, yaxis=1,ytitle=yhlabel,ystyle=1,chars=chsize-0.5*(!p.multi[2] eq 2)
  lastplot
  plot, xt,ind,/noerase,/nodata,chars=chsize-0.5*(!p.multi[2] eq 2), $
        ystyle=5,xstyle=5,ymargin=ybtm,xmargin=xlrm,yrange=yrn
  axis, xaxis=1,xtitle=xtoplabel,xstyle=1,xtickformat='(I6)',$
        chars=chsize-0.5*(!p.multi[2] eq 2)
  axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28), $
        chars=chsize-0.5*(!p.multi[2] eq 2)
  nextplot

  ; v leakage plot

  mxn = max(vl)
  mnn = min(vl)
  yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]

  plot, xb,vl,psym=3,color=color,chars=chsize-0.5*(!p.multi[2] eq 2), $
        ystyle=5,xstyle=5,ymargin=ybtm,xmargin=xlrm,yrange=yrn
  timeaxis,title=xbotlabel,/xbottom,charsize=chsize-0.5*(!p.multi[2] eq 2)
  axis, yaxis=1,ytitle=yvlabel,ystyle=1,chars=chsize-0.5*(!p.multi[2] eq 2)
  lastplot
  plot, xt,ind,/noerase,/nodata,chars=chsize-0.5*(!p.multi[2] eq 2), $
        ystyle=5,xstyle=5,ymargin=ybtm,xmargin=xlrm,yrange=yrn
  axis, xaxis=1,xstyle=1,xtickformat='(I6)', $
        chars=chsize-0.5*(!p.multi[2] eq 2)
  axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28), $
        chars=chsize-0.5*(!p.multi[2] eq 2)
  nextplot

endif

; Plot co- cros- channels ratios

if noiso eq 0 then begin

  plotwcrcocr,zoom=zm,indjump=ijump

endif

; Change page and prepare page 3 (reflectivity/pp plots)

if noiso eq 0 or noleak eq 0 then $
   if  setwin[0] eq 99 then begin
     printwcrstamp,tlabel,page=lpage+num2str(page),indjump=ijump,statt=stattext
     page=page+1
   endif else if norg eq 0 then pause

if norg eq 0 then begin

   np=9 & !p.multi=[0,0,np,0,0]

; Prepare data from the first, second and last range gates; use first pp only

  for k=0,2 do begin
    rg=rangeg[k]
    if (sm eq 0) then begin
      if alg eq sppalg then begin
         yplabel='pp [m/s]'
         ppn=maxvel[0]*reform(atan(imaginary(pp[rg,ind]),float(pp[rg,ind])))/!pi
         if txsequence[0] eq polh then begin
            hhn=hh[rg,ind]
            vvn=vh[rg,ind]
            yhlabel='hh [dBm]'
            yvlabel='vh [dBm]'
         endif else if txsequence[0] eq polv then begin
            hhn=hv[rg,ind]
            vvn=vv[rg,ind]
            yhlabel='hv [dBm]'
            yvlabel='vv [dBm]'
         endif
      endif else if alg eq pp4alg or alg eq pp6alg then begin
         yplabel='pp12 [m/s]'
         ppn=maxvel[0]*reform(atan(imaginary(pp[rg,0,ind]),float(pp[rg,0,ind])))/!pi
         if size(hh,/n_dimensions) eq 2 then begin
            hhn=reform(hh[rg,ind])
            hl =reform(hv[rg,ind])
         endif else begin
            hhn=reform(hh[rg,0,ind])
            hl =reform(hv[rg,0,ind])
         endelse
         if size(vv,/n_dimensions) eq 2 then begin
            vvn=reform(vv[rg,ind])
             vl =reform(vh[rg,ind])
         endif else begin
            vvn=reform(vv[rg,0,ind])
            vl =reform(vh[rg,0,ind])
         endelse
         yhlabel=['hh-[dBm]','hv-[dBm]']
         yvlabel=['vv...[dBm]','vh...[dBm]']
      endif
    endif else begin                  ; apply smoothing
       if alg eq sppalg then begin
          yplabel='pp [m/s]'
          ppn=maxvel[0]*(smooth(reform(atan(imaginary(pp[rg,*]), $
                         float(pp[rg,*]))),sm))[ind]/!pi
          if txsequence[0] eq polh then begin
             hhn=(smooth(hh[rg,*],sm))[ind]
             vvn=(smooth(vh[rg,*],sm))[ind]
             yhlabel='hh [dBm]'
             yvlabel='vh [dBm]'
          endif else if txsequence[0] eq polv then begin
             hhn=(smooth(hv[rg,*],sm))[ind]
             vvn=(smooth(vv[rg,*],sm))[ind]
             yhlabel='hv [dBm]'
             yvlabel='vv [dBm]'
          endif
       endif else if alg eq pp4alg or alg eq pp6alg then begin
          yplabel='pp12 [m/s]'
          ppn=maxvel[0]*(smooth(reform(atan(imaginary(pp[rg,0,*]), $
                         float(pp[rg,0,*]))),sm))[ind]/!pi
          if size(hh,/n_dimensions) eq 2 then begin
             hhn=(smooth(hh[rg,*],sm))[ind]
             hl =(smooth(hv[rg,*],sm))[ind]
          endif else begin
             hhn=(smooth(reform(hh[rg,0,*]),sm))[ind]
             hl =(smooth(reform(hv[rg,0,*]),sm))[ind]
          endelse
          if size(vv,/n_dimensions) eq 2 then begin
             vvn=(smooth(vv[rg,*],sm))[ind]
             vl =(smooth(vh[rg,*],sm))[ind]
          endif else begin
             vvn=(smooth(reform(vv[rg,0,*]),sm))[ind]
             vl =(smooth(reform(vh[rg,0,*]),sm))[ind]
          endelse
          yhlabel=['hh-[dBm]','hv-[dBm]']
          yvlabel=['vv...[dBm]','vh...[dBm]']
       endif
    endelse

    if alg eq sppalg then begin  ; SPPMAG

      vvn=10*alog10(vvn)
      hhn=10*alog10(hhn)

      ; First plot
      mxn = max(vvn)
      mnn = min(vvn)
      yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]
      plot, xb,vvn,psym=3,color=color, $
          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=xlrm,yrange=yrn
      timeaxis,/xbottom,/nolabels
      axis, yaxis=1,ystyle=1,ytitle=yvlabel ;,yticks=3
      lastplot
      plot, xt,ind,/noerase,/nodata, $
          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=xlrm,yrange=yrn
      axis, xaxis=1,xtitle=xtoplabel+' for rg '+num2str(rg), $
          xstyle=1,xtickformat='(I6)'
      axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
      nextplot
      ; Middle plots
      mxn = max(hhn)
      mnn = min(hhn)
      yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]
      plot, xb,hhn,psym=3,color=color, $
         ystyle=5,xstyle=5,ymargin=[0,0],xmargin=xlrm,yrange=yrn
      timeaxis,/xbottom,/nolabels
      axis, yaxis=1,ystyle=1,ytitle=yhlabel
      axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)
      ; Last plot
      mxn = max(ppn)
      mnn = min(ppn)
      yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]
      plot, xb,ppn,psym=3,color=color, $
         ystyle=5,xstyle=5,ymargin=[2,0],xmargin=xlrm,yrange=yrn
      oplot,[xb[0],xb[nprof-1]],[0,0],line=2
      if k lt 2 then timeaxis,/xbottom,charsize=chsize $
      else timeaxis,title=xbotlabel,/xbottom,charsize=chsize
      axis, yaxis=1,ystyle=1,ytitle=yplabel ;,yticks=3
      axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)

    endif else begin             ; PPMAG

      vvn=10*alog10(vvn)
      hhn=10*alog10(hhn)
      vl =10*alog10(vl)
      hl =10*alog10(hl)

      ; First plot (cross-pol reflectivities)
      mxn = max([vl,hl])
      mnn = min([vl,hl])
      yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]
      plot, xb,hl,color=color,$
          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=[3,8],yrange=yrn
      timeaxis,/xbottom,/nolabels
      axis, yaxis=1,ystyle=1,ytitle=yhlabel[1] ;,yticks=3
      lastplot
      plot, xt,vl,/noerase,psym=3, $
          ystyle=5,xstyle=5,ymargin=[0,2],xmargin=[3,8],yrange=yrn
      axis, xaxis=1,xtitle=xtoplabel+'s for rg '+num2str(rg), $
          xstyle=1,xtickformat='(I6)'
      axis, yaxis=0,ystyle=1,ytitle=yvlabel[1],ytickname=replicate(' ',28)
      nextplot
      ; Middle plots  (co-pol reflectivities)
      mxn = max([vvn,hhn])
      mnn = min([vvn,hhn])
      yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]
      plot, xb,hhn,color=color, $
         ystyle=5,xstyle=5,ymargin=[0,0],xmargin=[3,8],yrange=yrn
      timeaxis,/xbottom,/nolabels
      axis, yaxis=1,ystyle=1,ytitle=yhlabel[0]
      axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28),ytitle=yvlabel[0]
      oplot, xb,vvn,psym=3
      ; Last plot    (first pulse pair)
      mxn = max(ppn)
      mnn = min(ppn)
      yrn = [mnn-0.05*(mxn-mnn), mxn+0.05*(mxn-mnn)]
      plot, xb,ppn, psym=3, $
         ystyle=5,xstyle=5,ymargin=[2,0],xmargin=[3,8],yrange=yrn
      oplot,[xb[0],xb[nprof-1]],[0,0],line=2
      if k lt 2 then timeaxis,/xbottom,charsize=chsize $
      else timeaxis,title=xbotlabel,/xbottom,charsize=chsize
      axis, yaxis=1,ystyle=1,ytitle=yplabel ;,yticks=3
      axis, yaxis=0,ystyle=1,ytickname=replicate(' ',28)

    endelse
  endfor

  if setwin[0] eq 99 then $
    printwcrstamp,tlabel,page=lpage+num2str(page),indjump=ijump,statt=stattext

endif

resetps2x
end
