;$Id: plotwcrtestm.pro,v 1.7 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRTESTM
;
; PURPOSE:
;       Make test plots for multiple file data
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRTESTM,dirsel
;
; INPUTS:
;       Dirsel: string, wcr file directory to process all wcr files
;               undefined: use pickfile
;               'select': use pickfile with an option to select file(s)
;                         full path and name; process the given wcr file(s)
;
; KEYWORD PARAMETERS:
;      FILEID:  identification string for wcr file: '20YY-' (default '200')
;      PPMNOAV:  when set, same pol pulses in the burst are not averaged for
;                PPMAG alg.  The consequence of this is that hh, vv, hv and vh
;                become 3-D arrays, and hhnoise, vvnoise, hhtx, vvtx, hhlk
;                and vvlk 2-D arrays
;      INDJUMP: input string
;                'all'  - uses even and odd profiles (default)
;                'even' - uses even profiles only
;                'odd'  - uses odd profiles only
;      HCOR:    when set correct h channel w.r.t. v channel using 0.95 slope
;      VCOR:    when set correct v channel w.r.t. v channel using default vals
;      NOISO:   when set does not plot co/cross ratios
;      NOLEAK:  when set does not plot Tx pulse leakage
;      NONOISE: when set does not plot noise channels
;      NORG:    when set does not plot data for range gates
;      RANGEG:  input, intarr of up to 3 elements with range gates
;                (default [0,1,rangegates-1])
;      SMOOTH:  number of smoothing points (0, default)
;      TIMECOR: if defined gives UTC time correction of the wcr time
;               After June 1998, IRIG card is used to set the time.
;               Normal use of the card is to get the time from a master
;               IRIG card with GPS time.  The default value is set to
;               the leap seconds in openwcrf. 
;      TLABEL:  string, page title at the bottom of the page
;               default (undefined): 'Experiment Data'
;               '' - prompt for an input title
;      ZOOM:    input array [ readmode, str, stp ]
;               readmode: 0  - str/stp, start/stop record numbers
;                         1  - str/stp, start/stop profile numbers
;                         2  - str/stp, start/stop UTC in hhmmss
;
;     NOSAVEPS: when set does not save the ps files(ignored if plotps not set)
;       PLOTPS: when set creates ps files (default is x device)
;      PRINTER: printer name (e.g.,'lj5local','lj4_gv','qms', default lj5local)
;               '': suppress printing
;        PSDIR: string, directory for postscript file(s)
;               Default save ps files in subdir 'ps/', which is created
;               if non-existant.  When dir='select', ps directory is always
;               sub directory of the home directory, otherwise it is sub
;               of dir
;    BIG_ENDIAN:when set, assumes that the input file data are in big endian
;               order.  The default is LITTLE_ENDIAN.
;        HELP:  Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       plot 3 pages
;
; EXAMPLES:
;       Corner Reflector calibration plot
;
;       dir     = '/orel3/data99/cs99/'+d+'/'
;       psdir   = dir+d+'ps/'
;       fileid  = 'pp01-'
;       printer = 'lj5local'
;       tlabel  = "Coastal II '99 Data"
;       hcor    = 0
;       plotwcrtestm,dir,pri=printer,/plotps,fil=fileid,ind='all', $
;                    tlabel=tlabel,psdir=psdir
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Jan 2000
;       S.H., Jan 00 - added help keyword
;       S.H., Jan 01 - added little_endian keyword
;       S.H., Feb 01 - changed routine name from plotvxitestm to plotwcrtestm
;       S.H., Jul 01 - fixed minor bug
;       S.H., Apr 02 - replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;       S.H., Jun 02 - added printwcrprms and printwcrstat for diagnostic
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrtestm,dirsel,psdir=psdir,printer=printer,tlabel=tlbl,plotps=plotps, $
    nosaveps=nosaveps,indjump=ijump,fileid=fileid,timecor=timecor, $
    noiso=noiso,noleak=noleak,nonoise=nonoise,rangeg=rangeg,norg=norg, $
    hcor=hcor,vcor=vcor,smooth=sm,ppmnoav=ppmnoav,big_endian=bgnd,help=help

@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrtestm'
  return
endif

if n_elements(printer) eq 0 then printer='lj5local'
if keyword_set(plotps) then plotps=1 else plotps=0
if keyword_set(nosaveps) then saveps=0 else saveps=1
if n_elements(fileid) eq 0 then fileID='200'

if n_elements(tlbl) eq 0 then begin
   tlbl="Experiment Data"
endif else if tlbl(0) eq '' then begin
   tlbl=""
   read,"Enter test/data label: ",tlbl
endif

fileLen = 20

; Make wcr file and dir strings (string vectors)

if n_elements(dirsel) eq 0 then begin
   flst=dialog_pickfile(/read,get_path=dir)
   spawn, 'ls '+dir+'?'+fileID+'*',flst
   flst=strmid(flst,strpos(flst[0],fileID,/reverse_search)-1,fileLen)
endif else if dirsel eq 'select' then begin
   flst=dialog_pickfile(/read,/multi)
   dir=strsub(flst,0,strpos(flst[0],fileID,/reverse_search)-2)
   flst=strmid(flst,strpos(flst[0],fileID,/reverse_search)-1,fileLen)
endif else begin
   fpos=strpos(dirsel[0],fileID,/reverse_search)
   if fpos ge 0 and strpos(dirsel,'/',/reverse_search) ne strlen(dirsel)-1 then $
   begin
      flst=strmid(dirsel,fpos-1,fileLen)
      dir=strsub(dirsel,0,fpos-2)
   endif else begin
      if strmid(dirsel,strlen(dirsel)-1,1) ne '/' then dir=dirsel+'/' $
      else dir=dirsel
      spawn, 'ls '+dir+'?'+fileID+'*',flst
      flst=strmid(flst,strpos(flst[0],fileID,/reverse_search)-1,fileLen)
   endelse
endelse

nfiles=n_elements(flst)-1

; Make the postscript file directory

if (plotps eq 1 and saveps eq 1) then begin
   if n_elements(psdir) eq 0 then $
      if dirsel[0] eq 'select' then psdir='~/ps' $
      else psdir=dir+'ps/'
   if strmid(psdir,strlen(psdir)-1,1) ne '/' then psdir=psdir+'/'
   if psdir ne '.' and psdir ne './' then spawn,'mkdir '+psdir+' >& /dev/null'
endif

; Process selected wcr file(s)

for k=0,nfiles do begin
    fn=flst(k)
    dirk=dir(k*(n_elements(dir) gt 1))
    if strmid(dirk,strlen(dirk)-1,1) ne '/' then dirk=dirk+'/'

    print,'***** Processing '+fn+' ... *****'
    openwcrf,dirk+fn,timecor=timecor,big_endian=bgnd

    ; Determine the number of profiles to process based on 10 min segments

    profpersec=fix(1./timeinterval)
    if profpersec lt 1 then begin
	  segd=15000L
       message,'Timeinterval problem; using fixed '+num2str(segd)+' profs.',/info
    endif else segd=600L*profpersec
    seg=segd

    l=segd/4
    if tnumrec-segd lt l then begin seg=tnumrec & nseg=0 & end $
    else begin seg=segd & nseg=tnumrec/seg-(tnumrec mod seg lt l) & end

    for l=0,nseg do begin
       strt = l*seg
       if l eq nseg then stp=tnumrec-1 else stp = ((l+1)*seg-1)

       readwcrdata, strt, stp, ppmnoav=ppmnoav,big_endian=bgnd
       print,'================================================================='
       print,'==========',wcrfile,'; Segment: ',l,' of ',nseg   
       printwcrprms
       printwcrstat
       print,'================================================================='

       if (alg eq fftalg) then begin
          plotwcrfft,rgate=10,tlabel=tlbl,plotps=plotps*99
       endif else begin
          if keyword_set(hcor) then wcrhcor,/silent
          if keyword_set(vcor) then wcrvcor,/silent
          zm = [0, 0, numrec-1]
          plotwcrtest,smooth=sm,zoom=zm,indjump=ijump,noiso=noiso,  $
            noleak=noleak,nonoise=nonoise,rangeg=rangeg,norg=norg, $
            setwin=plotps*99+(1-plotps)*0,tlabel=tlbl,lpage=100*(k+1)+l+1
       endelse

       if (plotps eq 1) then begin
          if printer ne '' then spawn,'lpr -h -P'+printer+' idl.ps'
          if saveps then $
             spawn,'mv idl.ps '+ psdir+wcrfile+'.tst'+ $
             num2str(sec2time(timestr2sec(begtime)),fr=0)+'.ps' $
          else spawn,'rm idl.ps'
       endif else if k lt nfiles then pause
       hcorrection=intarr(8)
       vcorrection=intarr(8)
    endfor     ; end for l
    closewcrf
endfor   ; end for k

end
