;$Id: printwcrprms.pro,v 1.9 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PRINTWCRPRMS
;
; PURPOSE:
;       Prints radar parameters for a given data file
;        - file name and algorithm
;        - start/stop time, duration
;        - approx. time series length and sampling interval in seconds
;        - time correction if any and DSPs clock difference statistics
;        - profs per second statistics, ignoring first and last second
;        - profiles/time-segment loaded
;        - data rate in KB/s
;        - data flags [1,999,1]: file size doesn't match,
;                              # of bad profiles at the file end ,
;                              WCR computer date and timestamps date don't match
;                              or IRIG connection/data was lost/corrupted                          
;        - Tx pulse width, IF filter, pulse pair sequence
;        - Tx pulses polarization, prf
;        - range gate delay, range resolution, # range gates
;        - # averaged pulses
;        - range-to-profile (R/P) aspect ratio for 90 m/s A/C speed
;        - interrupt (data off) profile numbers or # of interrupts 
;          (for mode=2 only); interrupts were elliminated after 1998
;        - max. velocity and vel. resolution (FFT only), fftwindow used
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PRINTWCRPRMS,Mode,Inprm
;
; INPUTS:
;       Mode:   0 : displays the information 
;               1 : saves the info in ascii file
;               2 : prints WCR params. at a given position (normal units)
;                   using the current device 
;       Inprm:  for mode = 0  - ignore
;                          1  - string, file name
;                          2  - fltarr(3), paper position in normal units and
;                               charsize; [0.15,0.15,0.85] default
; KEYWORD PARAMETERS:
;       PROFSTAT: output, lonarr(n), returns an array with the number of 
;                 profiles per each whole second of time.  If time array is not
;                 available (data were not read, readwcrdata) returns -999
;       HELP:     Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Nov 1995
;       S.H., Sep 96 -- added duration and interrupts to the output
;       S.H., Nov 96 -- corrected for the new header
;       S.H., Aug 99 -- modified help info and other minor changes in the print
;       S.H., Nov 99 -- minor changes
;       S.H., Jan 00 -- added help keyword
;       S.H., Apr 00 -- fixed a problem with converting prf to string
;       S.H., Jan 01 -- added dftalg mode print, added aspect ratio print out
;                       for fft files in mode=2
;       S.H., Feb 01 -- small change to DSPDiff print (for mode=2)
;                       changed routine name from printvxiprms to printwcrprms
;       S.H., Jun 01 -- changed time correction print out; add ??? after
;                       duration to acknowledge problem with time stamps;
;                       added wcrflag[2] print
;       S.H., Dec 01 -- added statistics for sampling irregularities: min, max,
;                       and stdev of the number of profiles per second; added
;                       profstat keyword
;       S.H., Apr 02 -- added acqtime and irigflag
;       S.H., May 02 -- modified the output when irigflag=-2; changed dspsync
;                       print
;       S.H., Nov 02 -- changed dspsync print again
;       S.H., Jan 03 -- changed acqtime print
;       S.H., Mar 03 -- changed irigflag
;       S.H., Jul 03 -- added irigflag[2] to mode 2 display
;       S.H., Nov 03 -- minor change in aspect print for mode=2, added noiseflag
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro printwcrprms, mode, inprm, profstat=profstat, help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'printwcrprms'
  return
endif

badf=-999.
profstat=-999

if (n_params() eq 0) then mode = 0

if (alg eq sppalg) then algstr='SPPMAG' $
else if (alg eq pp4alg) then algstr='PPMAG4' $
else if (alg eq pp6alg) then algstr='PPMAG6' $
else if (alg eq fftalg or alg eq dftalg) then begin
   if (alg eq fftalg) then algstr='FFT'$
   else algstr='DPFFT'
   if wcrdate lt 19960101 then  $
      avespectra=53376/(fftbins*(rangeGates+2)) $
   else avespectra=averages
   if fftwindow eq 0 then fftwin='Rect' else $
   if fftwindow eq 1 then fftwin='Hann' else $
   if fftwindow eq 2 then fftwin='FlatTop' else $
   if fftwindow eq 3 then fftwin='GaussTop' else $
   if fftwindow eq 4 then fftwin='Blackman' else $
   if fftwindow eq 5 then fftwin='Gaussian'
endif

; Check start/stop time for crossing midnight and calculate duration

  starthour=fix(strmid(startTime,0,2))
  stophour=fix(strmid(stopTime,0,2))
  dur=timestr2sec(stopTime)-timestr2sec(startTime)
  if starthour gt stophour then dur=dur+86400L

; Calculate profiles_per_second statistics

  if numrec gt 0 then $  
    if n_elements(time) eq numrec then begin
      profstat=histogram(time-long(time[0]),min=0.)
      nps=n_elements(profstat)-2
    endif else nps=0 $
  else nps=0

; Check irigflag and assign correct time label

  if irigflag[0] eq -2 then timel='Count' $
  else if irigflag[0] lt 1 then timel='Time' $
  else timel='UTC'


; Print the info

if (mode eq 0) then begin
   print, '############### ',wcrfile,' - ',algstr,' ###############'

   temp=num2str(irigflag[0],fr=0)
   if n_elements(irigflag) gt 1 then $
    temp=temp+'/'+num2str(irigflag[1])+'/'+num2str(irigflag[2]) 
   print, 'IRIG: '+temp+','+timel+': ',starttime+' to '+stoptime
   if tcorrection eq badf then $
      print, 'UTC Correction applied: Not applied' $
   else $ 
     print, 'UTC Correction applied:'+num2str(tcorrection,fr=2)+' s'
   print, 'Duration: ',num2str(dur,fract=0),' s', $
          ', Time Length:',num2str(timelength,fract=2), ' s'
   if abs(dur-timelength) gt 1 then print,'Time gaps or Date problem !!!
   print, 'Time Interval: ',strmid(strtrim(timeinterval,2),0,5),' s',$
          ' ('+num2str(1./timeinterval,fr=1)+'pps)'   
   if dspsync[0] ne badf then begin
     print, 'TimeInt(mean,min,max [ms]): ',num2str(dspsync[3:5],fr=1)
     print, 'DSPs clock diff(mean,min,max [ms]): ',num2str(dspsync[0:2],fr=1)
   endif
   if profstat[0] ne -999 and nps gt 2 then $
   print, 'PPS(min,max,med): ',num2str(min(profstat[1:nps])),',',$
          num2str(max(profstat[1:nps])),',',num2str(median(profstat[1:nps]))
   print, 'Prof: ', strtrim(fstblocknum,2),' to ',strtrim(lstblocknum,2), $
          ', Number of profiles (read/total): ', $
           strtrim(numrec,2),'/',strtrim(tnumrec,2)
   print,'wcrFlag(size,#badprof,WCR/IRIG date mismatch):',wcrflag
   if numrec gt 0 and numrec ne tnumrec then print,'ProcTime/Recs: ',$
      begtime,'-',endtime,'/',num2str(prof(0)-fstblocknum),'-', $
      num2str(prof(numrec-1)-fstblocknum)
   print,'A/D ACQtime: ',num2str(acqtime),', Data Rate: ',num2str(datarate,fr=1),' KB/sec'  
   print,''
   temp=strtrim(string(prf/1000),2)+' '
   print, ['PRF: ',temp,'kHz; ','Pulsewidth:',num2str(pulselength), $
           'ns, IFfilt:',num2str(iffilter),'MHz']
   if npulse gt 1 then print, ['Tx Pol:',txstrpol,', PP Pol:',ppstrpol] $
   else print, 'Tx Pol:',txstrpol
   print, 'Range Delay: ',strmid(strtrim(string(rangegatedelay),2),0,5),' m, ', $
          'Gate Spacing: ', strmid(strtrim(string(rangegatespacing),2),0,5),' m, ', $  
          'Range Gates: ',strtrim(string(rangegates),2)
   print, 'RangeGate/Profile Spacing Aspect Ratio: ', $
          strmid(strtrim(string(aspectratio),2),0,5), $
          ' (@ ',num2str(kaspeed),' m/s)'
   print,''
   if (alg eq fftalg or alg eq dftalg) then begin
      print,'FFT bins: ',strtrim(string(fftbins),2), $
            ', Window: ',fftwin,', Averaged Spectra: ',strtrim(avespectra,2)
      print,'Nyquist Freq:',strtrim(string(prf(0)/2.),2),' Hz, ', $
            'Freq. Res.: ',strtrim(string(float(prf(0)/fftbins)),2),' Hz'
      print,'Max. Velocity: ',strtrim(string(maxvel),2)+' m/s'
      print,'Vel. Resolution: ',strtrim(string(2.*maxvel/fftbins),2)+' m/s'
   endif else begin
      print,'Averaged Pulses: ',strtrim(averages,2)
      print,'Noise pulses used for h, v:',num2str(noiseflag,fr=0)
   endelse
   if (n_elements(interrupts) gt 1) then begin
      print,'Interrupts (start/stop profiles): '
      print, interrupts
   endif
   print,''
endif else $

if (mode eq 1) then begin
   if (n_elements(inprm) eq 0) then inprm=strmid(wcrfile,0,11)+'.prms'
   get_lun, lun1
   openu,lun1,inprm, /append
   printf,lun1, '############### ',wcrfile,' - ',algstr,' ###############'
   temp=num2str(irigflag[0],fr=0)
   if n_elements(irigflag) gt 1 then $
    temp=temp+'/'+num2str(irigflag[1])+'/'+num2str(irigflag[2]) 
   printf,lun1, 'IRIG: ',temp,','+timel+': ',starttime+' to '+stoptime
   if tcorrection eq badf then $
      printf, lun1, 'UTC Correction applied: Not applied' $
   else $
      printf, lun1,'UTC Correction applied:'+num2str(tcorrection,fr=2)+' s'
   printf,lun1, 'Duration: ',num2str(dur,fract=0),' s', $
                ', Time Length:',num2str(timelength,fract=2), ' s'
   if abs(dur-timelength) gt 1 then printf,lun1,'Time gaps or Date problem !!!
   printf,lun1, 'Time Interval: ',strmid(strtrim(timeinterval,2),0,5),' s',$
          ' ('+num2str(1./timeinterval,fr=1)+'pps)'
   if dspsync[0] ne badf then begin
     printf, lun1,'TimeInt(mean,min,max [ms]): ',num2str(dspsync[3:5],fr=1)
     printf, lun1,'DSPs clock diff(mean,min,max [ms]): ',num2str(dspsync,fr=1)
   endif
   if profstat[0] ne -999 and nps gt 2 then $
   printf, lun1,'PPS(min,max,med): ',num2str(min(profstat[1:nps])),',',$
          num2str(max(profstat[1:nps])),',',num2str(median(profstat[1:nps]))
   printf,lun1, 'Prof: ', strtrim(fstblocknum,2),' to ',strtrim(lstblocknum,2),$
                ', Number of profiles (read/total): ', $
          strtrim(numrec,2),'/',strtrim(tnumrec,2)
   printf,lun1,'wcrFlag(size,#badprof,WCR/IRIG date mismatch):',wcrflag
   if numrec gt 0 and numrec ne tnumrec then printf,lun1,'ProcTime/Recs: ',$
      begtime,'-',endtime,'/',num2str(prof(0)-fstblocknum),'-', $
      num2str(prof(numrec-1)-fstblocknum)  
   printf,lun1,'A/D ACQtime: ',num2str(acqtime),', Data Rate: ',num2str(datarate,fr=1),' KB/sec'  
   printf,lun1,''
   temp=strtrim(string(prf/1000),2)+' '
   printf,lun1,['PRF: ',temp,'kHz; ','Pulsewidth:',num2str(pulselength), $
           'ns, IFfilt:',num2str(iffilter),'MHz']
   if npulse gt 1 then printf,lun1,['Tx Pol:',txstrpol,', PP Pol:',ppstrpol] $
   else printf,lun1, 'Tx Pol:',txstrpol
   printf,lun1, 'Range Delay: ',strmid(strtrim(string(rangegatedelay),2),0,5),' m, ', $
          'Gate Spacing: ', strmid(strtrim(string(rangegatespacing),2),0,5),' m, ', $  
          'Range Gates: ',strtrim(string(rangegates),2)
   printf,lun1, 'RangeGate/Profile Spacing Aspect Ratio: ', $
          strmid(strtrim(string(aspectratio),2),0,5), $
          ' (@ ',num2str(kaspeed),' m/s)'
   printf,lun1,''
   if (alg eq fftalg or alg eq dftalg) then begin
      printf,lun1,'FFT bins: ',strtrim(string(fftbins),2), $
            ', Window: ',fftwin,', Averaged Spectra: ',strtrim(avespectra,2)
      printf,lun1,'Nyquist Freq:',strtrim(string(prf(0)/2.),2),' Hz, ', $
            'Freq. Res.: ',strtrim(string(float(prf(0)/fftbins)),2),' Hz'
      printf,lun1,'Max. Velocity: ',strtrim(string(maxvel),2)+' m/s'
      printf,lun1,'Vel. Resolution: ',strtrim(string(2.*maxvel/fftbins),2)+' m/s'

   endif else begin
      printf,lun1,'Averaged Pulses: ',strtrim(averages,2)
      printf,lun1,'Noise pulses used for h v:',num2str(noiseflag,fr=0)
   endelse
   if (n_elements(interrupts) gt 1) then begin
      printf,lun1,'Interrupts(start/stop profiles): '
      printf,lun1, interrupts
   endif
   printf,lun1,''
   close, lun1
   free_lun, lun1
endif else $

if (mode eq 2) then begin
   if (n_elements(inprm) eq 0) then inprm=[0.15, 0.15, 0.85]

   temp=num2str(irigflag[0],fr=0)
   if n_elements(irigflag) gt 1 then temp=temp+'/'+num2str(irigflag[1],fr=0)+$
                                               '/'+num2str(irigflag[2],fr=0)
   text=wcrfile+' - '+algstr+', IRIG:'+temp+'!C'+ $
       timel+': '+starttime+' to '+stoptime + $
       ', Dur: '+num2str(dur,fract=0)+'s'
   if abs(dur-timelength) gt 1 then text=text+' ???'

   if (n_elements(interrupts) gt 1) then $
     text=text+',!CNum of Interrupts: '+num2str(n_elements(interrupts)/2-1,fr=0)

   text=text+'!C'+'TimeLngth: '+num2str(timelength,fract=2)+ 's,'+ $
       ' TimeInt: '+num2str(1000.*timeinterval,fr=0)+'ms'+ $
       ' ('+num2str(1./timeinterval,fr=1)+'pps)'+'!C'
 
   if tcorrection eq badf then temp='No' else temp=num2str(tcorrection,fr=1)+'s'
   text=text+'TimeCor='+temp

   if profstat[0] ne -999 and nps gt 2 then $
     text=text+', PPS(min,max,med): '+num2str(min(profstat[1:nps]))+','+$
     num2str(max(profstat[1:nps]))+','+num2str(median(profstat[1:nps]),frac=0)+'!C'   

   if dspsync[0] ne badf then begin
     text=text+'Timeint/DSPdiff(mn,min,mx):'+num2str(dspsync[3],fr=0)+$
          ','+num2str(dspsync[4],fr=0)+','+num2str(dspsync[5],fr=0)+ $
          '/'+num2str(dspsync[0],fr=0)+','+num2str(dspsync[1],fr=0)+ $
          ','+num2str(dspsync[2],fr=0)+'!C'        
   endif else text=text+'!C'

   text=text+'Prof: '+strtrim(fstBlockNum,2)+' to '+strtrim(lstBlockNum,2) + $
       ', NumProf(r/t): '+strtrim(numrec,2)+'/'+strtrim(tnumrec,2)

   if numrec gt 0 and numrec ne tnumrec then text=text+'!C'+'ProcTime/Recs: '+$
      begtime+'-'+endtime+'/'+num2str(prof(0)-fstblocknum)+'-'+ $
      num2str(prof(numrec-1)-fstblocknum)  

   text=text+'!C'+'ADacqtime: '+num2str(acqtime,fr=0)+'ms; DRate: '+ $
              num2str(datarate,fr=1)+'KB/s'
   text=text+'; Flags: '+num2str(wcrflag[0])+', '+num2str(wcrflag[1])+ $
              ', '+num2str(wcrflag[2])

   if (alg eq fftalg or alg eq dftalg) then $
      text=text+'; R/P Aspect:: '+num2str(aspectratio,fr=1)

   text=text+'!C'+'Pulse: '+ num2str(pulselength)+'ns, IFfilt: '+ $
        num2str(iffilter,frac=1)+'MHz'

   if npulse gt 1 then begin
     temp=byte(ppstrpol+replicate(' ',n_elements(ppstrpol)))
     text=text+'; ppPol: '+string(reform(temp,n_elements(temp)))
     temp=byte(txstrpol)
     text=text+'!C'+'TxPol: '+string(reform(temp,n_elements(temp)))
   endif else begin
     temp=byte(txstrpol)
     text=text+'; TxPol: '+string(reform(temp,n_elements(temp)))
   endelse 
   temp=byte(strtrim(string(round(prf)/1000),2)+' ')
   ind=where(temp eq 0B)
   if ind[0] ne -1 then temp[ind]=32B
   temp=string(reform(temp,n_elements(temp)))
   text=text+', PRF: '+temp+'kHz, Noiseflg:'+strjoin(num2str(noiseflag),',')

   text=text+'!CRange: Delay '+num2str(rangegatedelay,fr=0)+ $
      'm, Spacing '+num2str(rangegatespacing,fr=0)+'m, '+ $
      'Gates '+strtrim(string(rangegates),2)

   if (alg eq fftalg or alg eq dftalg) then begin
      text=text+'!C'+'FFT bins: '+strtrim(string(fftbins),2)+ $
        ', Ave. Spectra: '+strtrim(string(avespectra),2)+ $ 
        ', Window: '+fftwin+'!C' + $ 
        'Max. Vel.: '+strmid(strtrim(string(maxvel(0)),2),0,4)+' m/s, '+ $
        'Vel. Res.: '+strmid(strtrim(string(2.*maxvel(0)/fftbins),2),0,4)+' m/s'
   endif else $
     text=text+'!C'+'Ave. Pulses: '+strtrim(averages,2)+ $
     ', Rng/Prof Aspect@'+num2str(kaspeed,fr=0)+'m/s: '+num2str(aspectratio,fr=1) 

   xyouts, inprm(0),inprm(1),text,charsize=inprm(2),/normal,/noclip
endif

end
