;$Id: printwcrstat.pro,v 1.6 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PRINTWCRSTAT
;
; PURPOSE:
;       Prints min, max, mean, std (in dBm) and varc=std/mean (%) for noise,
;       internal calibration measurements, hh/vv, cross_ch/co_ch and DSP jumps
;       in the noise channels of a given data file.  
;       Note: Internal calibration stat is disabled.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PRINTWCRSTAT, Mode, Inprm
;
; INPUTS:
;       Mode:    0 : displays the information 
;                1 : saves the info in ascii file
;                2 : prints WCR params. at a given position (normal units)
;                    using the current device 
;       Inprm:   for mode = 0  - ignore
;                           1  - string, output file name, default is the
;                                first 11 chars from wcrfile plus '.stat'
;                                this mode is intended for multiple file proc.
;                           2  - fltarr(3), paper position in normal units and
;                                charsize; [0.45,0.15,0.85] default
;
; KEYWORD PARAMETERS:
;       INDJUMP: 'all'  - uses even and odd profiles (default)               
;                'even' - uses even profiles only
;                'odd'  - uses odd profiles only
;
;       RANGE:    [begin_rg, end_rg], (default [3,rangegates-1])
;                 range gate zoom for co/cross channel statistics
;
;       STATTEXT: string array with stat information.  Works only with mode 2.
;       HELP:     show this text
;
; OUTPUTS:        
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       print radar data statistics to the current device
;       if stattext is empty ('') then it is an output string array
;       if stattext is not empty printwcrstat prints it to the graphics device  
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Nov 1995
;       S.H., Nov 96 -- corrected for the new header
;       S.H., May 98 -- added stat for hh/vv, vh/hh and hv/vv
;                       added stat for DSP jumps in the noise channels
;                       added indjump
;                       stat for internal cal. disabled
;                       added range zoom
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Dec 99 -- restored stat calc for the leakage (hhtx,vvtx,...)
;                       added stattext, modified the printout 
;       S.H., Jan 00 -- added help keyword
;       S.H., Jan 01 -- added print for dpfft mode
;       S.H., Feb 01 -- changed routine name from printvxistat to printwcrstat
;       S.H., Oct 01 -- replaced atan(complex) with atan(imaginary(complex),
;                                                        float(complex))
;       S.H., Jan 02 -- do not display leak and cross channels stat when not
;                       loaded
;       S.H., Apr 02 -- minor change in the print out format
;       S.H., Aug 02 -- removed DSP jumps stat (to many calc to make it 
;                       meaningless); if done it would be useful to add stat
;                       for the leak (higher power) where jumps are most often
;                       observed
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro printwcrstat,mode,inprm,stattext=stattext,indjump=ijump,range=rgzm,help=help

@commonwcrinclude

on_error,2

if n_elements(stattext) eq 0 then stattext=''
if stattext ne '' then begin
   mode=2
   goto, display
endif

if keyword_set(help) then begin
  doc_library,'printwcrstat'
  return
endif

if (n_params() eq 0) then mode = 0
if n_elements(ijump) eq 0 then ijump='   '  ; 'all' replaced with spaces 

nprof=numrec

if strlowcase(ijump) eq 'even' then $
   ind=lindgen(nprof/2)*2 $
else if strlowcase(ijump) eq 'odd' then $
   ind=lindgen((nprof-1)/2)*2+1 $
else begin ind=lindgen(nprof) & ijump='   ' & end

if n_elements(rgzm) eq 0 then begin
  ind2   =3                 ; don't use first gates because of leakage
  ind3   =rangegates-1      ; last range gate to use
endif else if n_elements(rgzm) eq 1 then begin
  ind2   =rgzm
  ind3   =rangegates-1
endif else begin
  ind2   =rgzm(0)
  ind3   =rgzm(1)
endelse

velfac0=maxvel[0]
nvel   =n_elements(maxvel)
ps     =txsequence
minsig =1.58489e-6        ; -58 dBm minimum signal for co/cross power
gi     =10.^(0.5)         ; approx. gain inbalance, vv/hh=3 dB
iso    =10.^(1.6)         ; approx. isolation, 16 dB
bad    =-99               ; for unreliable statistics

;**** SPPMAG and PPMAG *********************************************************

if (alg eq sppalg or alg eq pp4alg or alg eq pp6alg) then begin
   
; Initialize titles for SPPMAG and PPMAG

   text1=ijump

   if total(hcorrection) ge 2 and total(vcorrection) ge 2 then $
      text1=text1+', '+'h/vcor Min    Max   Mean   Std   Varc%' $
   else if total(hcorrection) ge 2 then $
      text1=text1+', '+'hcor   Min    Max   Mean   Std   Varc%' $
   else if total(vcorrection) ge 2 then $
      text1=text1+', '+'vcor   Min    Max   Mean   Std   Varc%' $
   else $
      text1=text1+'        Min    Max    Mean   Std   Varc%'

   text2='hhN(dB)   '
   text3='vvN(dB)   '
   text4='hhL(dB)   '
   text5='vhL(dB)   '
   text6='vvL(dB)   '
   text7='hvL(dB)   '
   text8='ppN(m/s)  '  ; not used for PPMAG
   text9='ppT(m/s)  '  ; not used for PPMAG
   text16='hh/vv(dB) '  ; modified for SPPMAG 
   text17='vh/hh(dB) '  ; not used for SPPMAG
   text18='hv/vv(dB) '  ; not used for SPPMAG
   text19='hnjmp(dB) '  ; h-ch jumps between "ping" and "pong"
   text20='vnjmp(dB) '  ; v-ch jumps between "ping" and "pong"

;  hhnoise and vvnoise statistics (averaged for all noise samples and pulses)

   if size(hhnoise, /n_dim) gt 1 then hhn=mean(hhnoise,1) else hhn=hhnoise
   if size(hhnoise, /n_dim) eq 3 then hhn=mean(hhn,1)

   if size(vvnoise, /n_dim) gt 1 then vvn=mean(vvnoise,1) else vvn=vvnoise
   if size(vvnoise, /n_dim) eq 3 then vvn=mean(vvn,1)

   mnhn   = mean(hhn(ind))       & mnvn   = mean(vvn(ind))
   stdhn  = std(hhn(ind))        & stdvn  = std(vvn(ind))

   ; do the rest for the noise channels after co/cross statistics is calculated

;  Tx leakage statistics
   
   if (txsequence(0) eq polH or alg eq pp4alg or alg eq pp6alg) and $
   (n_elements(hhtx) gt 1 and n_elements(vvlk) gt 1) then begin
      ; Average all leak pulses 
      if size(hhtx,/n_dim) gt 1 then hht=mean(hhtx,1) else hht=hhtx
      if size(vvlk,/n_dim) gt 1 then vvl=mean(vvlk,1) else vvl=vvlk
         
      mnhT   = mean(hht(ind))       &  mnvL   = mean(vvl(ind))     
      stdhT  = std(hht(ind))        &  stdvL  = std(vvl(ind))
      varchT = 100.*stdhT/mnhT      &  varcvL = 100.*stdvL/mnvL
      mnhT   = 10.*alog10(mnhT)     &  mnvL   = 10.*alog10(mnvL)
      stdhT  = 10.*alog10(stdhT)    &  stdvL  = 10.*alog10(stdvL)
      minhT  = 10.*alog10(min(hht(ind),max=maxhT))
      minvL  = 10.*alog10(min(vvl(ind),max=maxvL))
      maxhT  = 10.*alog10(maxhT)    &  maxvL  = 10.*alog10(maxvL)
      text4=text4+num2str(minht,fr=2,len=6)+' '+ $ 
                  num2str(maxht,fr=2,len=6)+' '+ $
                  num2str(mnht,fr=2,len=6) + ' '+$
                  num2str(stdht,fr=2,len=6) +' '+ $
                  num2str(varcht,fr=1,len=4)
      text5=text5+num2str(minvl,fr=2,len=6)+' '+$ 
                  num2str(maxvl,fr=2,len=6)+' '+ $
                  num2str(mnvl,fr=2,len=6) + ' '+$
                  num2str(stdvl,fr=2,len=6) + ' '+$
                  num2str(varcvl,fr=1,len=4)
   endif 
 
   if (txsequence(0) eq polv or alg eq pp4alg or alg eq pp6alg) and $
   (n_elements(hhlk) gt 1 and n_elements(vvtx) gt 1) then begin
      ; Average all leak pulses 
      if size(hhlk,/n_dim) gt 1 then hhl=mean(hhlk,1) else hhl=hhlk
      if size(vvtx,/n_dim) gt 1 then vvt=mean(vvtx,1) else vvt=vvtx

      mnhL   = mean(hhlk(ind))      &  mnvT   = mean(vvtx(ind))     
      stdhL  = std(hhlk(ind))       &  stdvT  = std(vvtx(ind))
      varchL = 100.*stdhL/mnhL     &  varcvT = 100.*stdvT/mnvT
      mnhL   = 10.*alog10(mnhL)    &  mnvT   = 10.*alog10(mnvT)
      stdhL  = 10.*alog10(stdhL)   &  stdvT  = 10.*alog10(stdvT)
      minhL  = 10.*alog10(min(hhlk(ind),max=maxhL)) 
      minvT  = 10.*alog10(min(vvtx(ind),max=maxvT))
      maxhL  = 10.*alog10(maxhL)   &  maxvT  = 10.*alog10(maxvT)
      text6=text6+num2str(minvt,fr=2,len=6)+' '+ $ 
                  num2str(maxvt,fr=2,len=6)+' '+ $
                  num2str(mnvt,fr=2,len=6) +' '+ $
                  num2str(stdvt,fr=2,len=6) +' '+ $
                  num2str(varcvt,fr=1,len=4)
      text7=text7+num2str(minhl,fr=2,len=6)+' '+ $ 
                  num2str(maxhl,fr=2,len=6)+' '+ $
                  num2str(mnhl,fr=2,len=6) + ' '+$
                  num2str(stdhl,fr=2,len=6) +' '+ $
                  num2str(varchl,fr=1,len=4)
   endif 

;  ppnoise statistics (averaged for all noise samples and pulses)

   ppn=atan(imaginary(ppnoise),float(ppnoise))/!pi
   if nvel eq 1 then ppn=velfac0*ppn $
   else ppn=replicarr(maxvel,numrec)*ppn

   if size(ppnoise, /n_dim) gt 1 then ppn=mean(ppn,1)
   if size(ppnoise, /n_dim) eq 3 then ppn=mean(ppn,1)
   ppn=ppn[ind]

   mnpn   = mean(ppn)
   stdpn  = std(ppn)
   varcpn = 100.*stdpn/abs(mnpn)
   minpn  = min(ppn,max=maxpn)

   text8=text8 +num2str(minpn,fr=2,len=6)+' '+ $ 
                num2str(maxpn,fr=2,len=6)+' '+ $
                num2str(mnpn,fr=2,len=6) +' '+ $
                num2str(stdpn,fr=2,len=6)

   if n_elements(pptx) gt 1 then begin
     ppt=atan(imaginary(pptx),float(pptx))/!pi
     if nvel eq 1 then ppt=velfac0*ppt $
     else ppt=replicarr(maxvel[nvel-1],numrec)*ppn
     ppt=ppt[ind]

     mnpt   = mean(ppt)
     stdpt  = std(ppt)
     varcpt = 100.*stdpt/abs(mnpt)
     minpt  = min(ppt,max=maxpt)

     text9=text9 +num2str(minpt,fr=2,len=6)+' '+ $ 
                  num2str(maxpt,fr=2,len=6)+' '+ $
                  num2str(mnpt,fr=2,len=6) +' '+ $
                  num2str(stdpt,fr=2,len=6)
   endif

   if (alg eq sppalg) then begin

; SPPMAG: hv/vv or vh/hh statistics

      if txsequence(0) eq polh and n_elements(vh) gt 1 then begin
        text16='vh/hh(dB) '
        ind1=where(hh(ind2:ind3,ind) gt minsig*iso)
        if n_elements(ind1) lt 3 then begin
           mnvhl=(stdvhl=(minvhl=(maxvhl=(varcvhl=-99))))
        endif else begin
           temp    = ((vh[ind2:ind3,ind])[ind1]-mnvn)/ $
                     ((hh[ind2:ind3,ind])[ind1]-mnhn)
           mnvhl   = mean(temp)
           stdvhl  = std(temp)
           varcvhl = 100.*stdvhl/mnvhl
           minvhl  = min(temp,max=maxvhl) 
           mnvhl   = 10.*alog10(mnvhl)   &  minvhl  = 10.*alog10(minvhl)
           stdvhl  = 10.*alog10(stdvhl)  &  maxvhl  = 10.*alog10(maxvhl)
         endelse
         text16=text16+num2str(minvhl,fr=2,len=6)+' '+ $ 
                       num2str(maxvhl,fr=2,len=6)+' '+ $
                       num2str(mnvhl,fr=2,len=6) +' '+ $
                       num2str(stdvhl,fr=2,len=6) + ' '+$
                       num2str(varcvhl,fr=1,len=4)
      endif else if n_elements(hv) gt 1 then begin
         text16='hv/vv(dB) '
         ind1=where(vv(ind2:ind3,ind) gt minsig*gi*iso)
         if n_elements(ind1) lt 3 then begin
           mnhvl=(stdhvl=(minhvl=(maxhvl=(varchvl=-99))))
         endif else begin
           temp    = ((hv[ind2:ind3,ind])[ind1]-mnhn)/ $
                     ((vv[ind2:ind3,ind])[ind1]-mnvn)
           mnhvl   = mean(temp)
           stdhvl  = std(temp)
           varchvl = 100.*stdhvl/mnhvl
           minhvl  = min(temp,max=maxhvl) 
           mnhvl   = 10.*alog10(mnhvl)   &  minhvl  = 10.*alog10(minhvl)
           stdhvl  = 10.*alog10(stdhvl)  &  maxhvl  = 10.*alog10(maxhvl)
         endelse
         text16=text16+num2str(minhvl,fr=2,len=6)+' '+ $ 
                       num2str(maxhvl,fr=2,len=6)+' '+ $
                       num2str(mnhvl,fr=2,len=6) +' '+ $
                       num2str(stdhvl,fr=2,len=6) + ' '+$
                       num2str(varchvl,fr=1,len=4)
      endif

   endif else begin
     
; PPMAG:  hh/vv, hv/vv, vh/hh statistics (averaged for all pulses)
   if n_elements(vh) gt 1 and n_elements(hv) gt 1 then begin
      ; Average all same pol pulses 
      if size(hh,/n_dim) eq 3 then hht=mean(hh,2) else hht=hh
      if size(vv,/n_dim) eq 3 then vvt=mean(vv,2) else vvt=vv
      if size(hv,/n_dim) eq 3 then hhl=mean(hv,2) else hhl=hv
      if size(vh,/n_dim) eq 3 then vvl=mean(vh,2) else vvl=vh

    ind1=where(hht(ind2:ind3,ind) gt minsig)
    if n_elements(ind1) lt 3 then begin
       mnzdr=(stdzdr=(minzdr=(maxzdr=(varczdr=-99))))
    endif else begin
       temp    = ((hht[ind2:ind3,ind])[ind1]-mnhn)/ $
                 ((vvt[ind2:ind3,ind])[ind1]-mnvn)
       mnzdr   = mean(temp)
       stdzdr  = std(temp)
       varczdr = 100.*stdzdr/mnzdr
       minzdr  = min(temp,max=maxzdr)

       mnzdr   = 10.*alog10(mnzdr)   &  minzdr  = 10.*alog10(minzdr)
       stdzdr  = 10.*alog10(stdzdr)  &  maxzdr  = 10.*alog10(maxzdr)

    endelse
    text16=text16+num2str(minzdr,fr=2,len=6)+' '+ $ 
                  num2str(maxzdr,fr=2,len=6)+' '+ $
                  num2str(mnzdr,fr=2,len=6) +' '+ $
                  num2str(stdzdr,fr=2,len=6) + ' '+$
                  num2str(varczdr,fr=1,len=4)

    ind1=where(hht(ind2:ind3,ind) gt minsig*iso)
    if n_elements(ind1) lt 3 then begin
       mnvhl=(stdvhl=(minvhl=(maxvhl=(varcvhl=-99))))
    endif else begin
       temp    = ((vvl[ind2:ind3,ind])[ind1]-mnvn)/ $
                 ((hht[ind2:ind3,ind])[ind1]-mnhn)
       mnvhl   = mean(temp)
       stdvhl  = std(temp)
       varcvhl = 100.*stdvhl/mnvhl
       minvhl  = min(temp,max=maxvhl)

       mnvhl   = 10.*alog10(mnvhl)   &  minvhl  = 10.*alog10(minvhl)
       stdvhl  = 10.*alog10(stdvhl)  &  maxvhl  = 10.*alog10(maxvhl)
 
    endelse
    text17=text17+num2str(minvhl,fr=2,len=6)+' '+ $ 
                  num2str(maxvhl,fr=2,len=6)+' '+ $
                  num2str(mnvhl,fr=2,len=6) +' '+ $
                  num2str(stdvhl,fr=2,len=6) + ' '+$
                  num2str(varcvhl,fr=1,len=4)
     
     ind1=where(vvt(ind2:ind3,ind) gt minsig*gi*iso)
     if n_elements(ind1) lt 3 then begin
        mnhvl=(stdhvl=(minhvl=(maxhvl=(varchvl=-99))))
     endif else begin
        temp    = ((hhl[ind2:ind3,ind])[ind1]-mnhn)/ $
                  ((vvt[ind2:ind3,ind])[ind1]-mnvn)
        mnhvl   = mean(temp)
        stdhvl  = std(temp)
        varchvl = 100.*stdhvl/mnhvl
        minhvl  = min(temp,max=maxhvl) 

        mnhvl   = 10.*alog10(mnhvl)   &  minhvl  = 10.*alog10(minhvl)
        stdhvl  = 10.*alog10(stdhvl)  &  maxhvl  = 10.*alog10(maxhvl)

     endelse
     text18=text18+num2str(minhvl,fr=2,len=6)+' '+ $ 
                   num2str(maxhvl,fr=2,len=6)+' '+ $
                   num2str(mnhvl,fr=2,len=6) +' '+ $
                   num2str(stdhvl,fr=2,len=6) + ' '+$
                   num2str(varchvl,fr=1,len=4)
   endif
   endelse

;  Finish noise statistics

   varchn = 100.*stdhn/mnhn      & varcvn = 100.*stdvn/mnvn
   mnhn   = 10.*alog10(mnhn)     & mnvn  = 10.*alog10(mnvn)
   stdhn  = 10.*alog10(stdhn)    & stdvn = 10.*alog10(stdvn)
   minhn  = 10.*alog10(min(hhn(ind),max=maxhn))
   minvn  = 10.*alog10(min(vvn(ind),max=maxvn))
   maxhn  = 10.*alog10(maxhn)    & maxvn = 10.*alog10(maxvn)

   text2=text2+num2str(minhn,fr=2,len=6)+' '+ $ 
               num2str(maxhn,fr=2,len=6)+' '+ $
               num2str(mnhn,fr=2,len=6) +' '+ $
               num2str(stdhn,fr=2,len=6) + ' '+$
               num2str(varchn,fr=1,len=4)

   text3=text3+num2str(minvn,fr=2,len=6)+' '+ $ 
               num2str(maxvn,fr=2,len=6)+' '+ $
               num2str(mnvn,fr=2,len=6) +' '+ $
               num2str(stdvn,fr=2,len=6) +' '+ $
               num2str(varcvn,fr=1,len=4)


; Mean jumps between "ping" and "pong" DSP caclculated from the
; V and H noise channel 

    ind1=lindgen(nprof/2)*2
    ind2=ind1+1

    mnhndsp   = 10.*alog10(mean(hhn[ind2])/mean(hhn[ind1]))   
    mnvndsp   = 10.*alog10(mean(vvn[ind2])/mean(vvn[ind1]))   
 
    text19=text19+'              '+num2str(mnhndsp,fr=2,len=6) 
    text20=text20+'              '+num2str(mnvndsp,fr=2,len=6)



;**** FFT  *********************************************************************

endif else $
if (alg eq fftalg) then begin

   text1='             M(0)   M(1) sqrt(M(2))'
   text2='             [dB]  [m/s]  [m/s]'
   text3='fftNoise:  '
   text4='fftTxLeak: '
   text5='' & text6='' & text7='' & text8='' & text9='' 
   text16=''& text17=''& text18=''& text19=''& text20=''

   v=velo
   dv=2.*velfac0/fftbins

   nprof=n_elements(fftnoise(0,*))
   mom=fltarr(3,2)

   fftNa=total(fftnoise,2)   
   fftNa=fftNa/nprof       /(fftbins)

   mom(0,0)=total(fftNa)
   mom(1,0)=total(fftNa*v)/mom(0,0)
   mom(2,0)=sqrt(total((v-mom(1,0))^2*fftNa)/mom(0,0))

   ; In order to match the stat for noise channel (hhnoise/vvnoise)
   ; dt (not used for the stat for noise) is assumed 1 and
   ; therefore df (dv) = 1/fftbins.

     mom(0,0)=10.*alog10(mom(0,0)/fftbins)

   ; Next line can be used if stat for noise is calculated using actual dt

   ; mom(0,0)=10.*alog10(mom(0,0)*dv)

   text3=text3+num2str(mom(0,0),fr=2,len=6)+ '  '+ $
               num2str(mom(1,0),fr=2,len=6)+ '  '+ $
               num2str(mom(2,0),fr=2,len=5)

   if n_elements(ffttx) gt 1 then begin
     fftTa=total(ffttx,2)                 
     fftTa=fftTa/nprof       /(fftbins)   ; from Galloway (raw fft power)

     mom(0,1)=total(fftTa)
     mom(1,1)=total(fftTa*v)/mom(0,1)
     mom(2,1)=sqrt(total((v-mom(1,1))^2*fftTa)/mom(0,1))

     ; In order to match the stat for leak channel (TrnsmtPow)
     ; dt (not used for the stat for TrnsmtPow) is assumed 1 and
     ; therefore df (dv) = 1/fftbins.

     mom(0,1)=10.*alog10(mom(0,1)/fftbins)

     ; Next line can be used if stat for TrnsmtPow is calculated 
     ; using the actual dt

     ; mom(0,1)=10.*alog10(mom(0,1)*dv)

     text4=text4+num2str(mom(0,1),fr=2,len=6)+ '  '+ $
                 num2str(mom(1,1),fr=1,len=5)+ '  '+ $
                 num2str(mom(2,1),fr=1,len=5)
   endif

endif else if (alg eq dftalg) then begin

   text1='              M(0)   M(1) sqrt(M(2))'
   text2='              [dB]  [m/s]  [m/s]'
   text3='fftHnoise:  '
   text4='fftVnoise:  '
   text5='fftHTxLeak: '
   text6='fftVTxLeak: '
   text7='' & text8='' & text9='' 
   text16=''& text17=''& text18=''& text19=''& text20=''

   v=velo
   dv=2.*velfac0/fftbins

   nprof=n_elements(fftnoise(0,0,*))
   mom=fltarr(3,4)

   fftNa=total(fftnoise,3)   
   fftNa=fftNa/nprof       /(fftbins)

   ; Noise stat

   mom[0,0]=total(fftNa[*,1])
   mom[0,1]=total(fftNa[*,0])

   mom[1,0]=total(fftNa[*,1]*v)/mom[0,0]
   mom[1,1]=total(fftNa[*,0]*v)/mom[0,1]

   mom[2,0]=sqrt(total((v-mom[1,0])^2*fftNa[*,1])/mom[0,0])
   mom[2,1]=sqrt(total((v-mom[1,1])^2*fftNa[*,1])/mom[0,0])

   ; In order to match the stat for noise channel (hhnoise/vvnoise)
   ; dt (not used for the stat for noise) is assumed 1 and
   ; therefore df (dv) = 1/fftbins.

   mom[0,0]=10.*alog10(mom[0,0]/fftbins)
   mom[0,1]=10.*alog10(mom[0,1]/fftbins)

   text3=text3+num2str(mom(0,0),fr=2,len=6)+ '  '+ $
               num2str(mom(1,0),fr=2,len=5)+ '  '+ $
               num2str(mom(2,0),fr=2,len=5)
   text4=text4+num2str(mom(0,1),fr=2,len=6)+ '  '+ $
               num2str(mom(1,1),fr=2,len=5)+ '  '+ $
               num2str(mom(2,1),fr=2,len=5)

   ; Leak stat
   
   if n_elements(ffttx) gt 1 then begin
     fftTa=total(ffttx,3)                 
     fftTa=fftTa/nprof       /(fftbins)   ; from Galloway (raw fft power)

     mom[0,2]=total(fftTa[*,1])
     mom[0,3]=total(fftTa[*,0])

     mom[1,2]=total(fftTa[*,1]*v)/mom[0,2]
     mom[1,3]=total(fftTa[*,0]*v)/mom[0,3]

     mom[2,2]=sqrt(total((v-mom[1,2])^2*fftTa[*,1])/mom[0,2])
     mom[2,3]=sqrt(total((v-mom[1,3])^2*fftTa[*,0])/mom[0,3])

     ; In order to match the stat for leak channel (TrnsmtPow)
     ; dt (not used for the stat for TrnsmtPow) is assumed 1 and
     ; therefore df (dv) = 1/fftbins.

     mom[0,2]=10.*alog10(mom[0,2]/fftbins)
     mom[0,3]=10.*alog10(mom[0,3]/fftbins)

     text5=text5+num2str(mom(0,2),fr=2,len=6)+ '  '+ $
                 num2str(mom(1,2),fr=2,len=5)+ '  '+ $
                 num2str(mom(2,2),fr=2,len=5)
     text6=text6+num2str(mom(0,3),fr=2,len=6)+ '  '+ $
                 num2str(mom(1,3),fr=2,len=5)+ '  '+ $
                 num2str(mom(2,3),fr=2,len=5)
   endif
endif


;**** PRINT STATISTICS ********************************************************

; DISPLAY output

DISPLAY:

if (mode eq 0) then begin
   print, '############## ',wcrfile,' ###############'
   print, ''
   print, text1
   print, text2
   print, text3

   if (strlen(text4) gt 15) then print, text4
   if (strlen(text5) gt 15) then print, text5
   if (strlen(text6) gt 15) then begin & print, text6 & print, text7 & end
   if (strlen(text8) gt 15) then begin & print, text8 & end
   if (strlen(text9) gt 15) then print,text9
   if (strlen(text16) gt 15) then print, text16
   if (strlen(text17) gt 15) then begin & print, text17 & print, text18 & end
   if (strlen(text19) gt 15) then begin & print, text19 & print, text20 & end
   print,''

endif else $

; FILE output

if (mode eq 1) then begin
   if (n_elements(inprm) eq 0) then inprm=strmid(wcrfile,0,11)+'.stat'
   get_lun, lun1
   openu,lun1,inprm, /append
   printf,lun1, '############## ',wcrfile,' ###############'
   printf,lun1, ''
   printf,lun1, text1
   printf,lun1, text2
   printf,lun1, text3
   if (strlen(text4) gt 15) then printf,lun1,text4
   if (strlen(text5) gt 15) then printf,lun1,text5
   if (strlen(text6) gt 15) then begin 
       printf,lun1,text6 & printf,lun1,text7
   endif
   if (strlen(text8) gt 15) then printf,lun1,text8
   if (strlen(text9) gt 15) then printf,lun1,text9
   if (strlen(text16) gt 15) then printf,lun1, text16
   if (strlen(text17) gt 15) then begin
       printf,lun1, text17 & printf,lun1, text18 
   endif
   if (strlen(text19) gt 15) then begin 
       printf,lun1, text19 & printf,lun1, text20
   endif
   printf,lun1,''
   free_lun, lun1

endif else $

; GRAPHICS output

if (mode eq 2) then begin 

   if (n_elements(inprm) eq 0) then inprm=[0.15, 0.45, 0.75]

   if stattext eq '' then begin

     text=text1+'!C'+text2+'!C'+text3

     if txsequence[0] eq polH and alg eq sppalg and $
        n_elements(hhtx) gt 1 and n_elements(vvlk) gt 1 then $
        text=text+'!C'+text4+'!C'+text5 $
     else if txsequence[0] eq polV and alg eq sppalg and $ 
        n_elements(hhlk) gt 1 and n_elements(vvtx) gt 1 then $
        text=text+'!C'+text6+'!C'+text7 $
     else if alg eq pp4alg or alg eq pp6alg and $
        n_elements(hhlk) gt 1 and n_elements(vvlk) gt 1 then $
        text=text+'!C'+text4+'!C'+text6 $
     else if alg eq fftalg and n_elements(ffttx) gt 1 then $
        text=text+'!C'+text4 $
     else if (alg eq dftalg) then begin
        text=text+'!C'+text4
        if n_elements(ffttx) gt 1 then text=text+'!C'+text5+'!C'+text6
     endif

     if (strlen(text8) gt 15) then text=text+'!C'+text8
     if (strlen(text9) gt 15) then text=text+'!C'+text9
     if (strlen(text16) gt 15) then text=text+'!C'+text16
     if (strlen(text17) gt 15) then text=text+'!C'+text17+'!C'+text18
     if (strlen(text19) gt 15) then text=text+'!C'+text19+'!C'+text20

     stattext=text

     xyouts, inprm(0),inprm(1),"!11"+text,charsize=inprm(2), $
             /normal,/noclip,font=0
     xyouts,0,0,"!3",font=-1

   endif else begin

      xyouts, inprm(0),inprm(1),"!11"+stattext,charsize=inprm(2), $
              /normal,/noclip,font=0
      xyouts,0,0,"!3",font=-1

   endelse
endif

end
