;$Id: readwcrdata.pro,v 1.10 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       READWCRDATA
; 
; PURPOSE:
;       Load into the wcrdata common block radar data from a WCR file (see 
;       showwcrprms). The routine also determines numrec, begtime, endtime,
;       basetime, noiseflag, and dspsync.
;       
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       READWCRDATA, Startrec, Stoprec
;
; INPUTS:
;       Startrec:  long, start record (0 to tnumrec-2, default, 0) or
;                  long, start time in hhmmss (when timein keyword is set)
;       Stoprec:   long, stop record (1 to tnumrec-1, default, tnumrec-1)
;                  long, stop time in hhmmss (when timein keyword is set)
;
; KEYWORD PARAMETERS:
;       CLOSEWCR: if set closes the wcr file
;       NOCROSS:  when set cross-channel data are not loaded
;       FFTP:     when set, calculate returned power and noise power in mW
;                 and load them in hh (and/or vv) and hhnoise (and/or vvnoise)
;                 (FFT and DPFFT modes only)  
;       FILENAME: string, input wcr file name;
;                 use 'load' to get a file selction;
;                 if not given assumes the file is already open 
;       HCOR:     when set correct all H-channel data. 
;                 Use hcor=-1 to suppress H correction info print
;       NOLEAK:   When set transmitter leakage data are not loaded
;       PPMNOAV:  0, all noise measurements for same ch. pulses  in a burst are 
;                    averaged; if less than the burst size use needed number of
;                    noise meas. with the minimum mean (default)
;                 1, same ch. pulses in the burst for all data are not averaged.
;                    The consequence of this is that hh, vv, hv and vh become 
;                    3-D arrays, and hhnoise,vvnoise, hhtx,vvtx,hhlk,vvlk - 2-D
;                 2, use one (match ch.)pulse noise meas.with the minimum mean
;                 3, as 2 for h-ch noise and 0 for v-ch noise
;                 4, as 2 for v-ch noise and 0 for h-ch noise                 
;       PPONLY:   when set load pulse pairs data only (does not load co-power,
;                 cross-power, noise and leak data channels). Valid for SPPMAG
;                 and PPMAG algorithms only. PPMNOAV, NOLEAK, NOCROSS, FFTP, 
;                 HCOR and VCOR keywords are ignored
;       PPP:      output, fltarr(rangegates,npulse-1,profs); received power 
;                 calculated from pulse pairs (see Notes) 
;       RAWTIME:  input, integer,load UNIX time
;                  -1 - return unix time in the named variable and this is the 
;                       only action by the readwcrdata
;                 output, dblarr(numrec-1), raw UNIX time (includes fract. secs)
;                       rawtime is corrected if tcorrection > 0
;       TIMEIN:   when set, inputs are times in hhmmss (see Notes)
;       TIMECOR:  float, if given defines UTC time correction (tcorrection).
;                 This time correction, if defined, overrides the time  
;                 correction used (the default is leap seconds) in openwcrf 
;                 After June 1998, IRIG card is used to set the time.
;                 Normal use of the card is to get the time from a master
;                 IRIG card with GPS time.  Thus the default value is the leap 
;                 seconds(to see default leap seconds, check tcorrection after 
;                 opening a file).
;
;       TIMEMODE: 2 : hhmmss.dd          ; dblarr(stoprec-startrec+1)
;                 3 : sssss.dd           ; seconds starting from 0.dd   
;                 4 : sssss.dd  (default); seconds after midnight
;                 If tcorrection exists time will be corrected
;
;       VCOR:     when set correct all V-channel data. 
;                 Use vcor=-1 to suppress V correction info print
;     BIG_ENDIAN: when set, assumes that the file data are in big endian order.
;                 The default is LITTLE_ENDIAN. Ignored for data before 03/2002  
;       WCRLUN:   lun, default is 31
;       HELP:     show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       This routine will not work properly with WCR files before 1998
;
; NOTES:
;       When using TIMEIN keyword keep in mind the following possible problem.
;       I use the time median interval between wcr profiles to find which 
;       portion of the data to read (this is to make it more efficient).
;       However there is a KA data system problem during IRIG slewing time  
;       which causes time jumps in the wcr IRIG time (not fixed yet, Dec 2000).
;       When this jumps are present extracted data may not match the desired 
;       time range.
;
;       PPP power represents the product of the returned amplitudes from the 
;       pulses forming the pulse pair. 
;       For example if Tx pulse sequence is HHVV then:
;         ppp[*,0,*]=1/averages*sum(abs(H1)exp(i*angle(H1))*abs(H2)exp(-i*angle(H2)) 
;       while returned powers for H1 and H2 are:
;         hh[*,0,*]=1/averages*sum(abs(H1)^2)   ; PPMNOAV keyword is set to 1
;         hh[*,1.*]=1/averages*sum(abs(H2)^2)
;       As you can see PPP is coherently averaged for all the averaged pulse
;       pairs while the regular returned power is non-coherently averaged. 
;       
;-      
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Dec 1999
;       S.H., 01/00  - added vcor keyword and help keyword
;       S.H., 04/00  - modified ppmnoav keyword to extract single noise pulses
;                      added testtime keyword (not documented)
;       S.H., 07/00  - added rawtime keyword
;       S.H., 11/00  - for ppmag with single pol pulse sequence assign low 
;                      values to the missing channel and issue warning;
;                      rearranged fftps for data after 1999
;       S.H., 12/00  - added timein keyword
;       S.H., 01/01  - fixed a minor bugs, added fftP and little_endian keywords
;       S.H., 02/01  - changed routine name from readvxidata to readwcrdata
;       S.H., 06/01  - minor change in the help text for TIMECOR
;       S.H., 10/01  - changed how noise channels are loaded (for ppmag only)
;       S.H., 01/02  - added wcr channel designation variables for data reading
;       S.H., 01/02  - added NOCROSS and NOLEAK keywords
;       S.H., 02/02  - added basetime (wcrdata common block), modified TIMEMODE,
;                      fixed yet another annoying bug in unwrapping prof, added
;                      noiseflag, added PPONLY keyword
;       S.H., 02/02  - limit startrec>0 and stoprec<(tnumrec-1)
;       S.H., 04/02  - modified to work with new data format; replaced
;                      LITTLE_ENDIAN keyword with BIG_ENDIAN
;       S.H., 05/02  - modified dspsync, modified rawtime keyword
;       S.H., 08/02  - for multiple noise samples use only 0 noise sample 
;       S.H., 12/02  - changed definition of timeinterval from mean to median
;                      value; timelength is also recalculated; dspsync was
;                      expanded to include statistics for timeinterval; fixed
;                      an error in timelength update
;       S.H., 02/03  - added PPP keyword
;       S.H., 05/03  - fixed a bug when crossing midnight and using timein; 
;                      when using timein timeinterval is initialized to the
;                      value reported by openwcrf to make the reading more
;                      consistent (in cases time intervals are very irregular
;                      or there are significant time jumps)
;       S.H., 11/03  - modified PPMNOAV, for multiple noise samples select the
;                      one with minimum mean
;
; UNDOCUMENTED FEATURES:
;       TESTTIME keyword:  when setmeasures and print the elapsed time reading
;                          the raw data using readwcrraw
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro readwcrdata,startrec,stoprec,filename=fn,ppmnoav=ppmnoav,rawtime=rawtime,  $
                wcrlun=wcrlun,timemode=tmode,timecor=timecor,closewcr=closewcr,$
                timein=timein,hcor=hcor,vcor=vcor,fftp=fftp,nocross=nocross, $
                noleak=noleak,big_endian=bgnd,help=help,testtime=testtime, $
                pponly=pponly,ppp=ppp

@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'readwcrdata'
  return
endif

if n_elements(ppmnoav) eq 0 then ppmnoav=0 else $
   if ppmnoav lt 0 or ppmnoav gt 4 then begin
      message,'Ivalid ppmnoav value.',/info
      return
   endif

if n_elements(tmode) eq 0 then tmode=4 $
else $
  if tmode lt 2 or tmode gt 4 then begin
     message,'Incorrect TIMEMODE',/info
     return
  endif

badf=-999.

; Transmitter/Receiver channel designations after March 2002
  hch=1 & vch=0
; No need to swap channels in the software.  The cables between IF and VXI were 
; swapped. H-channel of IF became V-channel and vice versa.  
;  if wcrdate lt 20020301 then begin 
;     hch=1 & vch=0
;  endif else begin
;     hch=0 & vch=1
;  endelse

; Open the input file if needed

  if n_elements(fn) ne 0 then begin
    if fn eq 'load' then fn=dialog_pickfile(/read, filter='?????-??-*')
    if fn eq '' then return 
    openwcrf,fn,wcrlun=wcrlun,timecor=timecor,big_endian=bgnd
  endif

; If timecor is defined make sure tcorrection is the same, if not make the
; necessary changes

  if n_elements(timecor) gt 0 then $
     if tcorrection ne timecor then begin
        k=timecor-tcorrection
        tcorrection=timecor
        str=double(timestr2sec(starttime))+double('0.'+strmid(starttime,9,2))
        str=str+k
        if str lt 0. then str=86400.d0+str
        starttime=sec2timestr(str)+'.'+strmid(num2str(str-long(str),fr=2),2)
        str=double(timestr2sec(stoptime))+double('0.'+strmid(stoptime,9,2))
        str=str+k
        if str lt 0. then str=86400.d0+str
        stoptime=sec2timestr(str)+'.'+strmid(num2str(str-long(str),fr=2),2)
     endif

; Read raw data structure

  ; Check which records to read. If timein is set the record range is 
  ; determined approximately and refined after the time vector is loaded.

  if (n_params() eq 0) then begin
    str = 0L 
    spr = tnumrec-1 
  endif else if (n_params() eq 1) then begin
    str = long(startrec)>0
    spr = tnumrec-1
  endif else begin
    str = long(startrec)>0
    spr = long(stoprec)
  endelse

  if keyword_set(timein) and n_params() gt 0 then begin
    stt=long(timestr2sec(starttime))
    startt=time2sec(str)
    if n_params() eq 1 then stopt=long(timestr2sec(stoptime))+1 $
    else stopt=time2sec(spr)

    if startt-stt lt -70000. then str=startt+86400. else str=startt
    if stopt-str lt -70000. then stp=stopt+86400. else stp=stopt

    if str ge stp then message,'Incorrect start/stop time inputs.'

    ; There are small deviations in the time interval between profiles.
    ; So I'll read a few extra seconds to account for this in the initial
    ; reading.

    ; In addition there might be occasional bigger time gap (e.g., DSP
    ; request not processed by OS on time or KA IRIG problems that cause 
    ; time jumps. In this cases extracted data may not match the desired
    ; time range. 


    ; Restore timeinterval to the initial value reported by openwcrf

    timeinterval=float(time2sec(double(strmid(stoptime,0,2)+strmid(stoptime,3,2)+$
                  strmid(stoptime,6)))-time2sec(double(strmid(starttime,0,2)+$
                  strmid(starttime,3,2)+strmid(starttime,6)))+$
                  86400.*(timestr2sec(stoptime) lt stt))/(tnumrec-1)

    k = 500L 
    n=long((stp-str)/timeinterval)+k 

    if n ge tnumrec then begin
      str = 0L
      spr = tnumrec-1
    endif else begin
      str = (long((str-stt)/timeinterval))-k/2L > 0
      spr = str+n < tnumrec-1
    endelse
  endif else spr=spr < (tnumrec-1)

  if keyword_set(testtime) then begin
     print,'Reading raw data ...' & ltime=systime(1)
  endif

  readwcrraw,recraw,str,spr,big_endian=bgnd, closewcr=closewcr
  if keyword_set(testtime) then print,'Done reading:',systime(1)-ltime,'s' 

; Get raw time from the header. Correct time if tcorrection is available.
; Calculate time after midnight (timemode=4)

  ind=3-2*(wcrdate gt 20020331)
  ind1=ind+1
  if n_elements(rawtime) eq 1 then junk=rawtime[0] else junk=0
  rawtime=double(recraw.header[ind])+double(recraw.header[ind1])/1d6
  if tcorrection ne badf and tcorrection ne 0. then $
     rawtime=rawtime+double(tcorrection)
  if junk eq -1 then return
  basetime=long(rawtime[0])

  timemode=4
  time=rawtime mod 86400
 
; If timein is set refine extracted data to match exactly requested times

  if keyword_set(timein) then begin
    ind=where(time ge startt)
    if ind[0] eq -1 then message,"Wrong startt. Can't happen here." $
	else begin & startr=str & str=ind[0] & startr=startr+str & end

    ind=where(time le stopt)
	if ind[0] eq -1 then message,"Wrong stopt. Can't happen here." $
	else spr=ind[n_elements(ind)-1]
	
    recraw=recraw[str:spr]
    time=time[str:spr]
    rawtime=rawtime[str:spr]
    basetime=long(rawtime[0])	
  endif else startr=str
  
; Get profile numbers from the header and determine numrec  

  prof  =recraw.header[1-(wcrdate gt 20020331)]   
  numrec=n_elements(prof)

;  if wcrdate lt 20020331 then begin    ; THIS IS NOT DONE RIGHT YET in CPAS
  ; Correct profile numbers greater than 65536.  This is because a WCR program
  ; uses 2 byte register (unsigned integer) for the profile numbers.
  ; Profile counter starts from 1 and folds on 65536. For data before March 2002

    ind=where(prof eq 1)
    if ind[0] ne -1 then begin
      if ind[0] eq 1 then if n_elements(ind) gt 1 then ind=ind[1:*]
      k=n_elements(ind)
      if ind[0] ne 1 then begin
        if k eq 2 then begin
           prof[ind[0]:ind[1]-1]=prof[ind[0]:ind[1]-1]+65536
           prof[ind[1]:*]=prof[ind[1]:*]+2L*65536
        endif else begin
          for l=0L,k-2 do $
            prof[ind[l]:ind[l+1]-1]=prof[ind[l]:ind[l+1]-1]+(l+1L)*65536
          prof[ind[k-1]:*]=prof[ind[k-1]:*]+k*65536
        endelse
      endif
    endif

  ; Correct profiles for the startr if necessary

    if startr gt 0 then prof=prof+long(fstblocknum+startr)/65536*65536
;  endif 

; Calculate mean,std, max-min for the DSP clock differences
 
  ind=2*lindgen(numrec/2)
  junk=(rawtime)(ind+1)-(rawtime)(ind)
  dspsync=[mean(junk),min(junk),max(junk)]

; Recalculate timeinterval based on the median and add the statistics to dspsync

  junk=float((rawtime-shift(rawtime,1))[1:*])
  timeinterval=median(junk)
  timelength=timeinterval*(tnumrec-1)
  dspsync=float(1000.*[dspsync,mean(junk),min(junk),max(junk)])
  
; Find begtime, endtime and again convert time according to timemode if needed

  begtime=sec2timestr(time[0])
  endtime=sec2timestr(time[numrec-1])
 
  if tmode eq 2 then begin
     time=sec2time(time)
     timemode=2
  endif else if tmode eq 3 then begin
     time=rawtime-basetime
     timemode=3
  endif

; Process data structure according to the algorithm

  ; Get the dimensions

  Nn=noisesamples
  if alg eq sppalg then      Nmode=2  $
  else if alg eq pp4alg then Nmode=7  $
  else if alg eq pp6alg then Nmode=11 $ 
  else if alg eq fftalg then Nmode=1  $
  else if alg eq dftalg then Nmode=2  $
  else begin
    message,'Unknown algorithm.',/info
    return
  endelse 
  Nm2=Nmode/2

  ; Extract pp/ppp and if pponly is set quit

  if alg eq sppalg then $
    pp=reform(complex(recraw.data[0,Nn+1:*,0],recraw.data[1,Nn+1:*,0])) $    
  else if alg eq pp4alg or alg eq pp6alg then $
    pp=reform(complex(recraw.data[0,Nn+1:*,0:Nm2-1],recraw.data[1,Nn+1:*,0:Nm2-1]))
  ppp=abs(pp)

  if keyword_set(pponly) then begin
    if keyword_set(testtime) then $
      print,'Done readwcrdata:',systime(1)-ltime,'s' 
    return
  endif

  ; Load noise and Tx leak channels

  if alg eq fftalg then begin                      ; FFT
    fftnoise=reform(recraw.data[*,0:Nn-1,0])*1000. ; 1000 converts from W to mW
    if keyword_set(noleak) eq 0 then $
      ffttx   =reform(recraw.data[*,Nn,0])*1000.   ; bug in fft mode

  ; Rearrange the spectra if data after 1999. 
  ; FFT data were already shifted in the DSP before 1999 (according to John
  ; Galloway) - has not been checked carefully.

  ; FFT spectra are caclulated with the first bin being DC (0 frequency) then 
  ; +df to +Nyquist Freq.(Fn) and then from -Fn+df to -df (df - freq. step).
  ; In other words the fft hasn't been shifted. 
  ; After shifting, DC is fftbin/2 index and -Fn is 0 index.
  ; Positive Doppler velocity (negative freq) is toward the radar (radar
  ; convention).

    if (wcrdate gt 19990518) then begin 
       fftnoise=shift(fftnoise,fftbins/2,0)
       if keyword_set(noleak) eq 0 then $
         ffttx=shift(ffttx,fftbins/2,0)

    ; Following lines can be used if the velocity convention is changed from
    ; positive being toward the radar to being away from the radar. 

    ;   fftnoise=reverse(shift(fftnoise,fftbins/2-1,0),1)
    ;   ffttx=reverse(shift(ffttx,fftbins/2-1,0),1)
    endif

  endif else if alg eq dftalg then begin           ; DPFFT
    fftnoise=reform(recraw.data[*,0:Nn-1,*])*1000. ; 1000 converts from W to mW
    if keyword_set(noleak) eq 0 then $
      ffttx   =reform(recraw.data[*,Nn,*])*1000.     ; bug in fft mode

  ; Rearrange the spectra if data after 1999.

    if (wcrdate gt 19990518) then begin 
       fftnoise=shift(fftnoise,fftbins/2,0,0)
       if keyword_set(noleak) eq 0 then $
         ffttx=shift(ffttx,fftbins/2,0,0)

    ; Following lines can be used if the velocity convention is changed from
    ; positive being toward the radar to being away from the radar. 

    ;   fftnoise=reverse(shift(fftnoise,fftbins/2-1,0,0),1)
    ;   ffttx=reverse(shift(ffttx,fftbins/2-1,0,0),1)
    endif
  
  endif else begin                                 ; SPPMAG, PPMAG
    vvnoise=reform(recraw.data[vch,0:Nn-1,Nm2:Nmode-1])
    hhnoise=reform(recraw.data[hch,0:Nn-1,Nm2:Nmode-1])
    ppnoise=complex(recraw.data[0,0:Nn-1,0:Nm2-1],recraw.data[1,0:Nn-1,0:Nm2-1])
    ppnoise=reform(ppnoise)
    if keyword_set(noleak) eq 0 then begin
      pptx   =complex(recraw.data[0,Nn,0:Nm2-1],recraw.data[1,Nn,0:Nm2-1])
      pptx   =reform(pptx)
      if alg eq sppalg then begin
        if txsequence[0] eq polv then begin
           vvtx=reform(recraw.data[vch,Nn,Nm2])
           hhlk=reform(recraw.data[hch,Nn,Nm2])
        endif else begin
           vvlk=reform(recraw.data[vch,Nn,Nm2])
           hhtx=reform(recraw.data[hch,Nn,Nm2])
        endelse
      endif else begin
        ind1=where(txsequence eq polv)+Nm2
        if ind1[0] lt Nm2 then begin
           message,'WARNING: PPMAG mode w/ H-pol pulses only.',/info
           print,'               Assign low values to V co- and cross- channels.'
           vvtx=fltarr(1,numrec)+1e-6
           hhlk=fltarr(1,numrec)+1e-6
        endif else begin
           vvtx=reform(recraw.data[vch,Nn,ind1])
           hhlk=reform(recraw.data[hch,Nn,ind1])
        endelse

        ind =where(txsequence eq polh)+Nm2
        if ind[0] lt Nm2 then begin
           message,'WARNING: PPMAG mode w/ V-pol pulses only.',/info
           print,'               Assign low values to H co- and cross- channels.'
           vvlk=fltarr(1,numrec)+1e-6
           hhtx=fltarr(1,numrec)+1e-6
        endif else begin
          vvlk=reform(recraw.data[vch,Nn,ind])
          hhtx=reform(recraw.data[hch,Nn,ind])
        endelse
      endelse
    endif 
  endelse

  ; Load magnitude data

  if alg eq sppalg then begin                                  ; SPPMAG
     noiseflag=1234
     if txsequence[0] eq polv then begin
        vv=reform(recraw.data[vch,Nn+1:*,1])
        if keyword_set(nocross) eq 0 then $
           hv=reform(recraw.data[hch,Nn+1:*,1])
     endif else begin
        if keyword_set(nocross) eq 0 then $
          vh=reform(recraw.data[vch,Nn+1:*,1])
        hh=reform(recraw.data[hch,Nn+1:*,1])
     endelse

     if n_elements(hcor) gt 0 then begin
       if hcor eq -1 then silent=1 else silent=0
       wcrhcor,silent=silent
     endif
     if n_elements(vcor) gt 0 then begin
       if vcor eq -1 then silent=1 else silent=0
       wcrvcor,silent=silent
     endif
  endif else if alg eq pp4alg or alg eq pp6alg then begin     ; PPMAG
     ind1=where(txsequence eq polv)+Nm2
     if ind1[0] lt Nm2 then begin
       vv=fltarr(rangegates,1,numrec)+1e-6
       if keyword_set(nocross) eq 0 then $
         hv=fltarr(rangegates,1,numrec)+1e-6
     endif else begin
       vv=reform(recraw.data[vch,Nn+1:*,ind1])
       if keyword_set(nocross) eq 0 then $
         hv=reform(recraw.data[hch,Nn+1:*,ind1])
     endelse

     ind =where(txsequence eq polh)+Nm2
     if ind[0] lt Nm2 then begin
       if keyword_set(nocross) eq 0 then $
         vh=fltarr(rangegates,1,numrec)+1e-6
       hh=fltarr(rangegates,1,numrec)+1e-6
     endif else begin
       if keyword_set(nocross) eq 0 then $
         vh=reform(recraw.data[vch,Nn+1:*,ind])
       hh=reform(recraw.data[hch,Nn+1:*,ind])
     endelse

     noiseflag=lonarr(2)
     if ppmnoav ne 1 then begin

        if Nn gt 1 then begin      ; use the noise sample with min mean
           l=min(mean(reform(hhnoise,Nn,numrec*npulse),2),ind)
           l=min(mean(reform(vvnoise,Nn,numrec*npulse),2),ind1)
           hhnoise=reform(hhnoise[ind,*,*])
           vvnoise=reform(vvnoise[ind1,*,*])
        endif

        if ppmnoav eq 2 then begin

        ;  select noise measurement pulse with the minimum mean noise
        ;  for both, h- and v-ch
	 
           l=min(mean(hhnoise,2),ind)
           l=min(mean(vvnoise,2),ind1)
           hhnoise=reform(hhnoise[ind,*])
           vvnoise=reform(vvnoise[ind1,*])
           noiseflag=[ind+1L,ind1+1L]

        endif else begin

        ; average corresponding (to the pulse sequence) number of noise 
        ; measurements for v-ch(ppmnoav=3)/h-ch(ppmnoav=4) and noise 
        ; measurements from noise pulse with min mean for the other h/v ch.

          if ppmnoav eq 3 then begin
            l=min(mean(hhnoise,2),ind)
            hhnoise=reform(hhnoise[ind,*])
            noiseflag[0]=ind+1L
          endif else begin
            ind =where(txsequence eq polh)
            if ind[0] eq -1 then begin
              hhnoise=mean(hhnoise,1)
              noiseflag[0]=1234L*(alg eq pp4alg)+123456L*(alg eq pp6alg)
            endif else begin
              k=n_elements(ind)-1
              ind=(sort(mean(hhnoise,2)))[0:k]
              hhnoise=mean(hhnoise[ind,*],1)
              for l=0,k do noiseflag[0]=noiseflag[0]+(ind[l]+1)*(10^(k-l))            
            endelse
          endelse                          

          if ppmnoav eq 4 then begin
            l=min(mean(vvnoise,2),ind1)
            vvnoise=reform(vvnoise[ind1,*])
            noiseflag[1]=ind1+1L	  
          endif else begin
            ind1=where(txsequence eq polv)
            if ind1[0] eq -1 then begin
              vvnoise=mean(vvnoise,1) 
              noiseflag[1]=1234L*(alg eq pp4alg)+123456L*(alg eq pp6alg)
            endif else begin
              k=n_elements(ind1)-1
              ind1=(sort(mean(vvnoise,2)))[0:k]
              vvnoise=mean(vvnoise[ind1,*],1)
              for l=0,k do noiseflag[1]=noiseflag[1]+(ind1[l]+1)*(10^(k-l))            
            endelse
          endelse          

        endelse

        if keyword_set(noleak) eq 0 then begin
          vvtx=mean(vvtx,1)
          hhlk=mean(hhlk,1)
          vvlk=mean(vvlk,1)
          hhtx=mean(hhtx,1)
        endif

        if keyword_set(nocross) eq 0 then begin
          hv=mean(hv,2)
          vh=mean(vh,2)
        endif

        vv=mean(vv,2)
        hh=mean(hh,2)
     endif

     if n_elements(hcor) gt 0 then begin
       if hcor eq -1 then silent=1 else silent=0
       wcrhcor,silent=silent
     endif
     if n_elements(vcor) gt 0 then begin
       if vcor eq -1 then silent=1 else silent=0
       wcrvcor,silent=silent
     endif
  endif else if alg eq fftalg then begin                     ; FFT

  ; Load Doppler spectra

    fftps=reform(recraw.data[*,Nn+1:*,0])

  ; Rearrange the spectra if data after 1999 (see notes above)

  ; FFT spectra are caclulated with the first bin being DC (0 frequency) then 
  ; +df to +Nyquist Freq.(Fn) and then from -Fn+df to -df (df - freq. step).
  ; In other words the fft hasn't been shifted. 
  ; After shifting, DC is fftbin/2 index and -Fn is 0 index.
  ; Positive Doppler velocity (negative freq) is toward the radar (radar
  ; convention).

    if (wcrdate gt 19990518) then $ 
       fftps=shift(fftps,fftbins/2,0,0)

    ; The following line can be used if the velocity convention is changed
    ; from positive being toward the radar to being away from the radar. 

    ;   fftps=reverse(shift(fftps,fftbins/2-1,0,0),1)

  ; Convert power from Watts to mW
  
    fftps=1000.*fftps

  ; Calculate returned power and system noise if requested

    if keyword_set(fftp) then begin
      if txsequence[0] eq polv then begin
         vv=total(fftps,1)/fftbins^2
         vvnoise=total(fftnoise,1)/fftbins^2
      endif else begin
         hh=total(fftps,1)/fftbins^2
         hhnoise=total(fftnoise,1)/fftbins^2
      endelse
    endif
  endif else if alg eq dftalg then begin                     ; DPFFT

  ; Load Doppler spectra

    fftps=reform(recraw.data[*,Nn+1:*,*])

  ; Rearrange the spectra if data after 1999 (see notes above)

    if (wcrdate gt 19990518) then $ 
       fftps=shift(fftps,fftbins/2,0,0,0)

    ; The following lines can be used if the velocity convention is changed 
    ; from positive being toward the radar to being away from the radar. 

    ;   fftps=shift(fftps,fftbins/2-1,0,0,0)
    ;   fftps[*,*,0,*]=reverse(reform(fftps[*,*,0,*]),1) ; reverse works for up 
    ;   fftps[*,*,1,*]=reverse(reform(fftps[*,*,1,*]),1) ; to 3-D arrays

  ; Convert power from Watts to mW
  
    fftps=1000.*fftps

  ; Calculate returned signal and noise power if requested

    if keyword_set(fftp) then begin
      if txsequence[0] eq 15 then begin
         vv=total(reform(fftps[*,*,0,*]),1)/fftbins^2
         if keyword_set(nocross) eq 0 then $
           hv=total(reform(fftps[*,*,1,*]),1)/fftbins^2
         vvnoise=total(reform(fftnoise[*,0,*]),1)/fftbins^2 
         hhnoise=total(reform(fftnoise[*,1,*]),1)/fftbins^2 
      endif else begin
         hh=total(reform(fftps[*,*,1,*]),1)/fftbins^2
         if keyword_set(nocross) eq 0 then $
           vh=total(reform(fftps[*,*,0,*]),1)/fftbins^2
         vvnoise=total(reform(fftnoise[*,0,*]),1)/fftbins^2 
         hhnoise=total(reform(fftnoise[*,1,*]),1)/fftbins^2 
      endelse
    endif
  endif 

  if keyword_set(testtime) then print,'Done readwcrdata:',systime(1)-ltime,'s' 

end
