;$Id: readwcrfftspec.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+ 
; NAME:
;       READWCRFFTSPEC
;
; PURPOSE:
;       Read wcr file fft data between 'strt' and 'stp' profiles
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       READWCRFFTSPEC, Data, Strt, Stp
;
; INPUTS:
;       Data:  wcr file structure
;       Strt:  start reading point
;              For rmode = 0  -  strt/stp record in the wcr file
;                          1  -  input:  strt/stp profile numbers
;                                output: strt/stp corresponding wcr file records
;                          2  -  input:  strt/stp UTC in hhmmss, long
;                                output: strt/stp corresponding wcr file records
;       Stp:   stop reading point (see above)
;
; KEYWORD PARAMETERS:
;       BIG_ENDIAN:    when set, assumes that the file data are in big
;                      endian order.  The default is LITTLE_ENDIAN.  
;       HELP:          Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       Load into wcrdata block, fftps: fltarr(fftbins,rangegates,n)
;
; NOTES:
;       1) 'strt' and 'stp' are not necessarily associated with the actual
;          profile numbers; just determine the start/stop of the data segment.
;          In order to load data for desired start/stop profile set or UTC frame
;          use READWCRTB routine first
;       2) n=strt-stp+1
;       3) fftbins = number of spectral line
;       4) common blocks wcrparams and wcrdata must be set first
;       5) Raw data are calibrated in Watts instead of mW.  Coorection
;          factor of 1000 (30dB) is applied to convert to mW
; 
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Nov 1995
;       S.H., Nov 96 -- corrected for the new header
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Jan 00 -- added help keyword
;       S.H., Nov 00 -- rearranged fftps for data after 1999
;       S.H., Jan 01 -- change sign convention (positive vel. toward the radar)
;                       added little_endian keyword and made the routine
;                       independant on the data byteorder in the input file
;       S.H., Feb 01 -- changed routine name from readvxifftspec to 
;                       readwcrfftspec 
;       S.H., Apr 02 -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro readwcrfftspec, data, strt, stp, help=help, big_endian=bgnd

common wcrparams
common wcrdata
                  
; Check for start/stop profile input

  on_error,2

  if keyword_set(help) then begin
    doc_library,'readwcrfftspec'
    return
  endif

  nrec=tnumrec-1
  if (n_params() eq 1) then begin strt = 0 
                                  stp = nrec 
                            endif
  if (n_params() eq 2) then stp = nrec

  n = stp-strt+1 & block= lonarr(n)
  if (n le 0) or (strt lt 0) or (stp gt nrec) then $
     message,'Wrong start/stop record number(s) !'
  numRec=n 

; Check byte order for file data and this computer architecture

  swapbytes = 1L   ; 1 is default for data swaping

  if keyword_set(bgnd) then byteorder,swapbytes,/swap_if_little_endian $
  else byteorder,swapbytes,/swap_if_big_endian

; Read data

  if (alg eq fftalg) then begin

     fftps = fltarr(fftbins,rangegates,n)
     for k = long(strt), stp do begin
         temp = k-strt
         t = data(k)
         fftps(*,*,temp) = t.fftps
     endfor
     if swapbytes eq 1 then fftps=swap_endian(fftps)

  endif else $
     message,'Proc. stopped: '+wcrfile+' is not FFT.'

  ; Rearrange the spectra if data after 1999

    if (wcrdate gt 19990518) then $ 
       fftps=shift(fftps,fftbins/2,0,0)
    ;  fftps=reverse(shift(fftps,fftbins/2-1,0,0),1)

  ; Convert power from Watts to mW
  
  fftps=1000.*fftps

end
