;$Id: readwcrnoise.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+ 
; NAME:
;       READWCRNOISE
;
; PURPOSE:
;       Read wcr file noise and leakage data between 'strt' and 'stp' profs
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       READWCRNOISE, Data, Strt, Stp
;
; INPUTS:
;       Data:  wcr file structure
;       Strt:  start reading point
;              For rmode = 0  -  strt/stp record in the wcr file
;                          1  -  input:  strt/stp profile numbers
;                                output: strt/stp corresponding wcr file records
;                          2  -  input:  strt/stp UTC in hhmmss, long
;                                output: strt/stp corresponding wcr file records
;       Stp:   stop reading point (see above)
;
; KEYWORD PARAMETERS:
;       PPMNOAV:     when set, same pol pulses in the burst are not averaged for
;                    PPMAG alg.  The consequence of this is that hhnoise,  
;                    vvnoise,hhtx, vvtx, hhlk, vvlk become 2-D arrays 
;       BIG_ENDIAN:  when set, assumes that the file data are in big
;                    endian order.  The default is LITTLE_ENDIAN.  
;       HELP:        Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       Load into wcrdata block:
;       vvnoise    - fltarr(n), V noise measurements
;       hhnoise    - fltarr(n), H noise measurements
;       vvtx       - fltarr(n), co-leak from V trnsmt pulse (after 1996)
;       hhtx       - fltarr(n), co-leak from H trnsmt pulse (after 1996)
;       vvlk       - fltarr(n), cross-leak from H trnsmt pulse (after 1996)
;       hhlk       - fltarr(n), cross-leak from V trnsmt pulse (after 1996)
;       ppnoise    - complexarr(n), pulse pair noise (SPPMAG)
;                  - complexarr(3,n), pulse pair noise  (PPMAG4)
;                  - complexarr(5,n), pulse pair noise  (PPMAG6)
;       pptx       - complexarr(n), pulse pair tx leak (SPPMAG)(DISABLED)
;                  - complexarr(3,n), pulse pair tx leak (PPMAG4)(DISABLED)
;                  - complexarr(5,n), pulse pair tx leak (PPMAG6)(DISABLED)
;       fftnoise   - fltarr(averages,n), noise spectrum
;       ffttx      - fltarr(averages,n), tx pulse leak spectrum
;
; NOTES:
;       1) 'strt' and 'stp' are not necessarily associated with the actual
;          profile numbers; just determine the start/stop of the data segment.
;          In order to load data for desired start/stop profile set or UTC frame
;          use READWCRTB routine first
;       2) n=strt-stp+1
;       3) common blocks wcrparams and wcrdata must be set first
;       4) FFT raw data are calibrated in Watts instead of mW.  Coorection
;          factor of 1000 (30dB) is applied to convert to mW
; 
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Nov 1995
;       S.H., Nov 96 -- corrected for the new header
;       S.H., Nov 98 -- disabled all cal. measurements
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Nov 99 -- partially restored tx leak variables and combined
;                       pulse pair arrays (changed wcrdata) for sppmag and ppmag
;       S.H., Jan 00 -- added help keyword
;       S.H., Jan 01 -- changed sign convention for FFT (pos.vel. toward radar)
;                       added little_endian keyword and made the routine
;                       independant on the data byteorder in the input file
;       S.H., Feb 01 -- changed routine name from readvxinoisecal to
;                       readwcrnoise
;       S.H., Apr 02 -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro readwcrnoise, data,strt,stp,ppmnoav=ppmnoav,big_endian=bgnd,help=help

common wcrparams                
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'readwcrnoise'
  return
endif

; Disable internal calibration (commented out) and assign bad values

  badval=-999.
  vvtx=badval
  hhtx=badval
  vvlk=badval
  hhlk=badval
  ffttx=badval  
  pptx=badval

;  magn        - local, fltarr(2,n), V and H noise measurements
;  magHcal     - local, fltarr(2,n), V and H cal. meas. for H transmitted
;  magVcal     - local, fltarr(2,n), V and H cal. meas. for V transmitted
                
; Check for start/stop profile input

  nrec=tnumrec-1
  if (n_params() eq 1) then begin strt = 0 
                                  stp = nrec 
                            endif
  if (n_params() eq 2) then stp = nrec

  n = stp-strt+1 & block= lonarr(n)
  if (n le 0) or (strt lt 0) or (stp gt nrec) then $ 
     message,'Wrong start/stop record number(s) !'
  numrec=n  

  if keyword_set(ppmnoav) eq 1 and (alg eq pp4alg or alg eq pp6alg) then begin
     magn     = fltarr(2,npulse,n)
     magcal   = fltarr(2,npulse,n)  
  endif else if alg ne fftalg then begin
     magn     = fltarr(2,n)
     magcal   = fltarr(2,n)  
     magHcal  = fltarr(2,n)
     magVcal  = fltarr(2,n)
  endif

  npH = (size(where(txsequence eq polh)))[1]
  npV = (size(where(txsequence eq polv)))[1]

; Check byte order for file data and this computer architecture

  swapbytes = 1L   ; 1 is default for data swaping

  if keyword_set(bgnd) then byteorder,swapbytes,/swap_if_little_endian $
  else byteorder,swapbytes,/swap_if_big_endian

; Read data

  if (alg eq sppalg) then begin

     ppnoise = complexarr(n)
     pptx= complexarr(n)

     for k = long(strt), stp do begin 
         temp = k-strt
         t = data(k)
         if swapbytes eq 1 then t=swap_endian(t)
         magcal(0:1,temp) = t.magcal
         magn(0:1,temp) = t.magn
         ppnoise(temp) = complex(t.ppn(0),t.ppn(1))
         pptx(temp) = complex(t.ppcal(0),t.ppcal(1))
     endfor
     if (total(txsequence) eq npulse*polh ) then begin
         magHcal = magcal
     endif else $
     if (total(txsequence) eq npulse*polv ) then begin
         magVcal = magcal
     endif else begin
         message,'SPPmag: Invalid Pulse sequence !'
     endelse

  endif else $
  if (alg eq pp4alg or alg eq pp6alg) then begin

     ppnoise = complexarr(npulse-1,n)
     pptx= complexarr(npulse-1,n)

     if (npH+npV ne npulse) then $
         message,'PPmag: Invalid Pulse sequence !'
     for k  = long(strt), stp do begin
         temp = k-strt
         t = data(k)
         if swapbytes eq 1 then t=swap_endian(t)

       if keyword_set(ppmnoav) then begin
          magn(*,0,temp) = t.mag1n
          magn(*,1,temp) = t.mag2n
          magn(*,2,temp) = t.mag3n
          magn(*,3,temp) = t.mag4n
          magcal(*,0,temp) = t.mag1cal 
          magcal(*,1,temp) = t.mag2cal 
          magcal(*,2,temp) = t.mag3cal 
          magcal(*,3,temp) = t.mag4cal
          if (alg eq pp6alg) then begin
             magn(*,4,temp) = t.mag5n
             magcal(*,4,temp) = t.mag5cal
             magn(*,5,temp) = t.mag6n
             magcal(*,5,temp) = t.mag6cal
          endif
 
       endif else begin        
         magn(*,temp) = t.mag1n
         magn(*,temp) = magn(*,temp) + t.mag2n
         magn(*,temp) = magn(*,temp) + t.mag3n
         magn(*,temp) = magn(*,temp) + t.mag4n
         
         if (txsequence(0) eq polh) then $
              magHcal(*,temp) = magHcal(*,temp) + t.mag1cal $ 
         else magVcal(*,temp) = magVcal(*,temp) + t.mag1cal
 
          if (txsequence(1) eq polh) then $
               magHcal(*,temp) = magHcal(*,temp) + t.mag2cal $
          else magVcal(*,temp) = magVcal(*,temp) + t.mag2cal

         if (txsequence(2) eq polh) then $ 
               magHcal(*,temp) = magHcal(*,temp) + t.mag3cal $
          else magVcal(*,temp) = magVcal(*,temp) + t.mag3cal

          if (txsequence(3) eq polh) then $
               magHcal(*,temp) = magHcal(*,temp) + t.mag4cal $
          else magVcal(*,temp) = magVcal(*,temp) + t.mag4cal

         if (alg eq pp6alg) then begin
            magn(*,temp) = magn(*,temp) + t.mag5n
            magn(*,temp) = magn(*,temp) + t.mag6n
            if (txsequence(4) eq polh) then $
                 magHcal(*,temp) = magHcal(*,temp) + t.mag5cal $
            else magVcal(*,temp) = magVcal(*,temp) + t.mag5cal
            if (txsequence(5) eq polh) then $
                 magHcal(*,temp) = magHcal(*,temp) + t.mag6cal $
            else magVcal(*,temp) = magVcal(*,temp) + t.mag6cal
         endif

       endelse

         ppnoise(0,temp) = complex(t.pp12n(0),t.pp12n(1))
         ppnoise(1,temp) = complex(t.pp23n(0),t.pp23n(1))
         ppnoise(2,temp) = complex(t.pp34n(0),t.pp34n(1))

         pptx(0,temp) = complex(t.pp12cal(0),t.pp12cal(1))
         pptx(1,temp) = complex(t.pp23cal(0),t.pp23cal(1))
         pptx(2,temp) = complex(t.pp34cal(0),t.pp34cal(1))

         if (alg eq pp6alg) then begin
            ppnoise(3,temp) = complex(t.pp45n(0),t.pp45n(1))
            ppnoise(4,temp) = complex(t.pp56n(0),t.pp56n(1))
            pptx(3,temp) = complex(t.pp45cal(0),t.pp45cal(1))
            pptx(4,temp) = complex(t.pp56cal(0),t.pp56cal(1))
         endif

     endfor

  endif else $
  if (alg eq fftalg) then begin

     fftnoise = fltarr(fftbins,n)
     ffttx = fltarr(fftbins,n)
     for k = long(strt), stp do begin
         temp = k-strt
         t = data(k)
         if swapbytes eq 1 then t=swap_endian(t)
         fftnoise(*,temp) = t.fftnoise
         ffttx(*,temp) = t.fftcal
     endfor

     ; Correct power from Watts to mW
  
     fftnoise=1000.*fftnoise
     ffttx=1000.*ffttx

     ; Rearrange the spectra if data after 1999 (see notes inside openwcrf.pro)

     if (wcrdate gt 19990518) then begin 
     ; fftnoise=reverse(shift(fftnoise,fftbins/2-1,0),1)
     ; ffttx=reverse(shift(ffttx,fftbins/2-1,0),1)
       fftnoise=shift(fftnoise,fftbins/2,0)
       ffttx=shift(ffttx,fftbins/2,0)
     endif

  endif else $
     message,'Proc. stopped: '+wcrfile+' is not SPPmag, PPmag or FFT.'

  if keyword_set(ppmnoav) eq 1 and (alg eq pp4alg or alg eq pp6alg) then begin
     hind=where(txsequence eq polh)
     vind=where(txsequence eq polv)
     vvnoise = reform(magn(0,*,*)) 
     hhnoise = reform(magn(1,*,*))
     vvtx = reform(magcal(0,vind,*)) 
     hhtx = reform(magcal(1,hind,*))
     vvlk = reform(magcal(0,hind,*))
     hhlk = reform(magcal(1,vind,*))

  endif else if (alg eq sppalg or alg eq pp4alg or alg eq pp6alg) then begin

     if alg eq pp4alg or alg eq pp6alg then begin
        magn     = magn / npulse
        magHcal  = magHcal / npH
        magVcal  = magVcal / npV
     endif

     vvnoise = reform(magn(0,*))
     hhnoise = reform(magn(1,*))

      if (alg eq pp4alg or alg eq pp6alg or txsequence(0) eq polv) then begin
         vvtx = reform(magVcal(0,*))
         hhlk = reform(magVcal(1,*))
      endif

      if (alg eq pp4alg or alg eq pp6alg or txsequence(0) eq polh) then begin
         vvlk = reform(magHcal(0,*))
         hhtx = reform(magHcal(1,*))
      endif
  endif

end
