;$Id: readwcrpowerpp.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+ 
; NAME:
;       READWCRPOWERPP
;
; PURPOSE:
;       Read wcr file returned power and pulse pair data from 'strt' to 'stp'
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       READWCRPOWERPP, Data, Strt, Stp
;
; INPUTS:
;       Data:  wcr file structure
;       Strt:  start reading point
;              For rmode = 0  -  strt/stp record in the wcr file
;                          1  -  input:  strt/stp profile numbers
;                                output: strt/stp corresponding wcr file records
;                          2  -  input:  strt/stp UTC in hhmmss, long
;                                output: strt/stp corresponding wcr file records
;       Stp:   stop reading point (see above)
;
; KEYWORD PARAMETERS:
;       PPMNOAV:       when set, same pol pulses in the burst are not averaged
;                      for PPMAG alg.  The consequence of this is that hh,vv,hv 
;                      and vh become 3-D arrays 
;       BIG_ENDIAN:    when set, assumes that the file data are in big
;                      endian order.  The default is LITTLE_ENDIAN.  
;       HELP:          show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       Load into wcrdata block:
;       vv          - fltarr(rangeGates,n), copol V power measurements
;       hh          - fltarr(rangeGates,n), copol H power measurements
;       vh          - fltarr(rangeGates,n), crosspol V power measurements
;       hv          - fltarr(rangeGates,n), crosspol H power measurements
;       vh          - fltarr(rangeGates,n), crosspol V power measurements
;       pp          - complexarr(rangeGates,n), pulse pair meas. (SPPMAG)
;                   - complexarr(rangeGates,3,n), pulse pair meas. (PPMAG4)
;                   - complexarr(rangeGates,5,n), pulse pair meas. (PPMAG6)
;
; NOTES:
;       1) 'strt' and 'stp' are not necessarily associated with the actual
;          profile numbers; just determine the start/stop of the data segment.
;          In order to load data for desired start/stop profile set or UTC frame
;          use READWCRTB routine first
;       2) n=strt-stp+1
;       3) common blocks wcrparams and wcrdata must be set first
;       4) For SPPMAG the output is either hh, vh or vv, hv. 
; 
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Oct 1995
;       S.H., Nov 96 -- corrected for the new header
;       S.H., Aug 99 -- modified help info and other minor changes
;                       added ppmag6 mode
;       S.H., Dec 99 -- combined pulse pairs in one array for sppmag and ppmag
;       S.H., Jan 00 -- added help keyword
;       S.H., Jan 01 -- added little_endian keyword and made the routine
;                       independant on the data byteorder in the input file
;       S.H., Feb 01 -- changed routine name from readvxipowerpp to
;                       readwcrpowerpp
;       S.H., Apr 02 -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro readwcrpowerpp, data,strt,stp,ppmnoav=ppmnoav,big_endian=bgnd,help=help

common wcrparams
common wcrdata
                 
; Check for start/stop profile input

  on_error,2

  if keyword_set(help) then begin
    doc_library,'readwcrpowerpp'
    return
  endif

  nrec=tnumrec-1
  if (n_params() eq 1) then begin strt = 0 
                                  stp = nrec 
                            endif
  if (n_params() eq 2) then stp = nrec

  n = stp-strt+1 & block= lonarr(n)
  if (n le 0) or (strt lt 0) or (stp gt nrec) then $
     message,'Wrong start/stop record number(s) !'
  numRec=n

; Check byte order for file data and this computer architecture

  swapbytes = 1L   ; 1 is default for data swaping

  if keyword_set(bgnd) then byteorder,swapbytes,/swap_if_little_endian $
  else byteorder,swapbytes,/swap_if_big_endian

; Read data

  if (alg eq sppalg) then begin
     tps = total(txsequence)
     if (tps eq npulse*polh or tps eq npulse*polv) then begin
        vh_temp = fltarr(rangegates,n)
        hh_temp = fltarr(rangegates,n)
        pp = complexarr(rangegates,n)
        for k = long(strt), stp do begin
            temp = k-strt
            t = data(k)
            if swapbytes eq 1 then t=swap_endian(t)
            vh_temp(*,temp)  = t.mag(0,*)
            hh_temp(*,temp)  = t.mag(1,*)
            pp(*,temp) = complex(t.pp(0,*),t.pp(1,*))
        endfor
        if (tps eq npulse*polv ) then begin
            vv = vh_temp
            hv = hh_temp
        endif else begin
            vh = vh_temp
            hh = hh_temp
        endelse
     endif else message,'SPPmag: Invalid Pulse sequence !'

  endif else $
  if (alg eq pp4alg or alg eq pp6alg) then begin

     npH = (size(where(txsequence eq polh)))(1)
     npV = (size(where(txsequence eq polv)))(1)

     if not(npH+npV eq 4 or npH+npV eq 6) then $
         message,'PPmag4/PPmag6: Invalid Pulse sequence !'

     if keyword_set(ppmnoav) then begin
        vh = fltarr(rangegates,npH,n)
        hh = fltarr(rangegates,npH,n)
        vv = fltarr(rangegates,npV,n)
        hv = fltarr(rangegates,npV,n)
     endif else begin
        vh = fltarr(rangegates,n)
        hh = fltarr(rangegates,n)
        vv = fltarr(rangegates,n)
        hv = fltarr(rangegates,n)
     endelse

     pp = complexarr(rangegates,npulse-1,n)

     for k  = long(strt), stp do begin
         temp = k-strt
         t = data(k)
         if swapbytes eq 1 then t=swap_endian(t)

         pp(*,0,temp) = complex(t.pp12(0,*),t.pp12(1,*))
         pp(*,1,temp) = complex(t.pp23(0,*),t.pp23(1,*))
         pp(*,2,temp) = complex(t.pp34(0,*),t.pp34(1,*))
         if (alg eq pp6alg) then begin     
            pp(*,3,temp) = complex(t.pp45(0,*),t.pp45(1,*))
            pp(*,4,temp) = complex(t.pp56(0,*),t.pp56(1,*))
         endif

         if keyword_set(ppmnoav) then begin
            hk=0 & vk=0
            if (txsequence(0) eq polh) then begin
               vh(*,hk,temp) = t.mag1(0,*)
               hh(*,hk,temp) = t.mag1(1,*)
               hk=hk+1
            endif else begin
               vv(*,vk,temp) = t.mag1(0,*)
               hv(*,vk,temp) = t.mag1(1,*)
               vk=vk+1
            endelse

            if (txsequence(1) eq polh) then begin
               vh(*,hk,temp) = t.mag2(0,*)
               hh(*,hk,temp) = t.mag2(1,*)
               hk=hk+1
            endif else begin
               vv(*,vk,temp) = t.mag2(0,*)
               hv(*,vk,temp) = t.mag2(1,*)
               vk=vk+1
            endelse

            if (txsequence(2) eq polh) then begin
               vh(*,hk,temp) = t.mag3(0,*)
               hh(*,hk,temp) = t.mag3(1,*)
               hk=hk+1
            endif else begin
               vv(*,vk,temp) = t.mag3(0,*)
               hv(*,vk,temp) = t.mag3(1,*)
               vk=vk+1
            endelse
        
            if (txsequence(3) eq polh) then begin
               vh(*,hk,temp) = t.mag4(0,*)
               hh(*,hk,temp) = t.mag4(1,*)
            endif else begin
               vv(*,vk,temp) = t.mag4(0,*)
               hv(*,vk,temp) = t.mag4(1,*)
            endelse

            if (alg eq pp6alg) then begin     
               if (txsequence(4) eq polh) then begin
                  vh(*,hk,temp) = t.mag5(0,*)
                  hh(*,hk,temp) = t.mag5(1,*)
               endif else begin
                  vv(*,vk,temp) = t.mag5(0,*)
                  hv(*,vk,temp) = t.mag5(1,*)
               endelse
               if (txsequence(5) eq polh) then begin
                  vh(*,hk,temp) = t.mag6(0,*)
                  hh(*,hk,temp) = t.mag6(1,*)
               endif else begin
                  vv(*,vk,temp) = t.mag6(0,*)
                  hv(*,vk,temp) = t.mag6(1,*)
               endelse
            endif 
  
         endif else begin
            if (txsequence(0) eq polh) then begin
               vh(*,temp) = vh(*,temp) + t.mag1(0,*)
               hh(*,temp) = hh(*,temp) + t.mag1(1,*)
            endif else begin
               vv(*,temp) = vv(*,temp) + t.mag1(0,*)
               hv(*,temp) = hv(*,temp) + t.mag1(1,*)
            endelse

            if (txsequence(1) eq polh) then begin
               vh(*,temp) = vh(*,temp) + t.mag2(0,*)
               hh(*,temp) = hh(*,temp) + t.mag2(1,*)
            endif else begin
               vv(*,temp) = vv(*,temp) + t.mag2(0,*)
               hv(*,temp) = hv(*,temp) + t.mag2(1,*)
            endelse

            if (txsequence(2) eq polH) then begin
               vh(*,temp) = vh(*,temp) + t.mag3(0,*)
               hh(*,temp) = hh(*,temp) + t.mag3(1,*)
            endif else begin
               vv(*,temp) = vv(*,temp) + t.mag3(0,*)
               hv(*,temp) = hv(*,temp) + t.mag3(1,*)
            endelse
        
            if (txsequence(3) eq polH) then begin
               vh(*,temp) = vh(*,temp) + t.mag4(0,*)
               hh(*,temp) = hh(*,temp) + t.mag4(1,*)
            endif else begin
               vv(*,temp) = vv(*,temp) + t.mag4(0,*)
               hv(*,temp) = hv(*,temp) + t.mag4(1,*)
            endelse

            if (alg eq pp6alg) then begin     
               if (txsequence(4) eq polH) then begin
                  vh(*,temp) = vh(*,temp) + t.mag5(0,*)
                  hh(*,temp) = hh(*,temp) + t.mag5(1,*)
               endif else begin
                  vv(*,temp) = vv(*,temp) + t.mag5(0,*)
                  hv(*,temp) = hv(*,temp) + t.mag5(1,*)
               endelse
               if (txsequence(5) eq polH) then begin
                  vh(*,temp) = vh(*,temp) + t.mag6(0,*)
                  hh(*,temp) = hh(*,temp) + t.mag6(1,*)
               endif else begin
                  vv(*,temp) = vv(*,temp) + t.mag6(0,*)
                  hv(*,temp) = hv(*,temp) + t.mag6(1,*)
               endelse
            endif
         endelse
     endfor
     if not keyword_set(ppmnoav) then begin
         hh = hh / npH
         vh = vh / npH
         vv = vv / npV
         hv = hv / npV
     endif
  endif else $
     message,'Proc. stopped: '+wcrfile+' is not SPPmag or PPmag.'

end
