;$Id: readwcrraw.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       READWCRRAW
;
; PURPOSE:
;       Open and Load into the memory raw data from a WCR file 
;       
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       READWCRRAW, Recraw, Startrec, Stoprec
;
; INPUTS:
;       Startrec:  long, start record (0 to tnumrec-2, default, 0)
;       Stoprec:   long, stop record (1 to tnumrec-1, default, tnumrec-1)
;
; KEYWORD PARAMETERS:
;       FILEHEADER:    output, structure containing the file header
;                      This keyword is ignored for data before April 2002
;       FILENAME:      string, input wcr file name; 
;                      if not given assumes the file is already open 
;                      use 'load' to get a file selction;
;       BIG_ENDIAN:    when set, assumes that the file data are in big endian
;                      order.  The default is LITTLE_ENDIAN. For data files
;                      before Mar 2002 this keyword is ignored.  
;       WCRLUN:        lun, default is 30 if FILENAME is given and 31 if not
;       CLOSEWCR:      if set closes the wcr file  
;       HELP:          show this text
;
; OUTPUTS:
;       Recraw:  Nr-element array of structures
;                {header: lonarr(headervars),data:fltarr(Nch,Nn+Ntx+Nrg,Nmode)}
;                headervars= 25 after Oct 1996 and 40 after May 1999
;                Nch   = 2       ; real & imag for pulse pairs and V & H 
;                                  for magnitudes ; SPPMAG and PPMAG modes
;                        fftbins ; for FFT mode
;                Nn    = noise samples (always 1 before May 1999)
;                Ntx   = 1, Tx pulse leakage; internal cal. pulse (before 1997)
;                Nrg   = # of range gates
;                Nmode =  2 - for SPPMAG (1 pps, 1 mags)
;                         7 - for PPMAG4 (3 pps, 4 mags)
;                        11 - for PPMAG6 (5 pps, 6 mags)
;                         1 - for FFT
;                         2 - for DPFFT (co- and cross- spectra)
;                Nr    = # of records read (max Nr=tnumrec)
;
; COMMON BLOCKS:
;       wcrparams
;
; SIDE EFFECTS:
;       This routine will not work with WCR files before 1997 
;       Use readwcrall for data before Oct 1996
;
;       When using the routine to read equal size blocks of data
;       it is most time efficient (as long as the structure is not deleted 
;       from the memory it is initilized only the first time and then
;       just checks if the new data block has the same size)
;-      
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Nov 1999
;       S.H., Jan 00 -- added help keyword
;       S.H., Apr 00 -- fixed a bug with blocksize
;                       tried to optimize memory/speed
;       S.H., Nov 00 -- added DPFFT mode
;       S.H., Jan 01 -- changed readu routine with my readse that takes care
;                       of byte swaping if needed; added little_endian keyword
;       S.H., Feb 01 -- changed routine name from readvxiraw to readwcrraw
;                       modified the initialization of recraw structure
;       S.H., Apr 02 -- modified to work with a new data format and changed
;                       LITLE_ENDIAN with BIG_ENDIAN, added FILEHEADER
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro readwcrraw,recraw,startrec,stoprec,filename=fn,wcrlun=wcrlun, $
               fileheader=fileheader,big_endian=bgnd,closewcr=closewcr,help=help 

on_error,2

@commonwcrinclude

if keyword_set(help) then begin
  doc_library,'readwcrraw'
  return
endif

if n_params() lt 1 then begin
   message,'Usage: readwcrraw,recraw,startrec,stoprec',/info
   return
endif 

Ntx        = 1    ; leakage samples
Nch        = 2    ; for SPPMAg and PPMAG modes; fftbins for FFT mode
Nn         = 1    ; always for data files before 1999
filehsz    = 0    ; always for data before 2002
headervars = 25   ; for data files from Oct 1996 to May 1999
sppa       = 9    ; SPPMAG mode
pp4a       = 14   ; PPMAG4 mode, after May 1999 the code is 14  
pp6a       = 15   ; PPMAG6 mode
ffta       = 11   ; FFT mode
fftd       = 13   ; DPFFT mode


if n_elements(fn) ne 0 then begin

   if fn eq 'load' then  fn=dialog_pickfile(/read, filter='?pp??-*')
   if fn eq '' then return 

   if (keyword_set(wcrlun) eq 0) then wcrlun = 30

   close, wcrlun     ; close the file if it was not close previously
   openr,wcrlun,fn

   filenl   = 20   ; this is fixed and used to update fileIDstr

   ; Get WCR file name and WCR date

   fileid=strmid(fn,strlen(fn)-filenl,3)        ; allows renaming 

   wcrf=strmid(fn,strpos(fn,fileid,/reverse_search),filenl)
   if strmid(wcrf,3,2) lt '90' then wcrd='20'+strmid(wcrf,3,2) $
   else wcrd='19'+strmid(wcrf,3,2)
   wcrd=long(wcrd+strmid(wcrf,6,2)+strmid(wcrf,9,2))
   if wcrd lt 19900101 or wcrd gt 20200101 then begin
      message,'WCR file naming problem warning: '+num2str(wcrf),/info
   endif

   ; Read header data  

   if wcrd lt 20020331 then begin
     bgnd         =1
     version      =long(1) & readse,wcrlun,big_end=bgnd, version 
     fstblockn    =long(1) & readse,wcrlun,big_end=bgnd, fstblockn
     datablocksz  =long(1) & readse,wcrlun,big_end=bgnd, datablocksz ; in bytes
     startt       =lonarr(2) & readse,wcrlun,big_end=bgnd, startt ; UTC [s, us]
   endif else begin
     point_lun, wcrlun, 202L
     datablocksz  =long(1) & readse,wcrlun,big_end=bgnd, datablocksz ; in bytes
     fstblockn    =long(1) & readse,wcrlun,big_end=bgnd, fstblockn
     startt       =lonarr(2) & readse,wcrlun,big_end=bgnd, startt ; UTC [s, us]
   endelse  

   ; Check if time stamps match wcrdate

   caldat, floor(startt[0]/86400d0)+2440588L, mm, dd, yyyy
   k=yyyy*10000L+mm*100+dd

   if wcrd ne k then $
     if wcrd lt k then begin
        message,'Using time stamps to define data date: '+num2str(k),/info
        wcrd=k
     endif else $
        message,'Using file name to define data date: '+num2str(wcrd),/info

   ; Check which software revision (at this point by date) and read needed data
 
   if (wcrd lt 19961001) then $
      message,'Does not work with WCR data prior to Oct 1996' $
   else if (wcrd lt 19990501) then begin
      pp4a      =7
      dummy     =lonarr(10)  & readse,wcrlun,big_end=bgnd,dummy
      Nrg       =long(1)     & readse,wcrlun,big_end=bgnd,Nrg
      dummy     =lonarr(6)   & readse,wcrlun,big_end=bgnd,dummy
      al        =long(1)     & readse,wcrlun,big_end=bgnd,al
      fftb      =long(1)     & readse,wcrlun,big_end=bgnd,fftb
   endif else if (wcrd lt 20020331) then begin
      headervars=40
      dummy       =lonarr(24)  & readse,wcrlun,big_end=bgnd,dummy
      Nn          =long(1)     & readse,wcrlun,big_end=bgnd,Nn
      Nrg         =long(1)     & readse,wcrlun,big_end=bgnd,Nrg
      dummy       =lonarr(6)   & readse,wcrlun,big_end=bgnd,dummy
      al          =long(1)     & readse,wcrlun,big_end=bgnd,al
      fftb        =long(1)     & readse,wcrlun,big_end=bgnd,fftb
   endif else begin
      filehsz=206L
      headervars=5L
      point_lun, wcrlun,78L
      Nn          =long(1)     & readse,wcrlun,big_end=bgnd,Nn
      Nrg         =long(1)     & readse,wcrlun,big_end=bgnd,Nrg
      dummy       =lonarr(6)   & readse,wcrlun,big_end=bgnd,dummy
      al          =long(1)     & readse,wcrlun,big_end=bgnd,al
      fftb        =long(1)     & readse,wcrlun,big_end=bgnd,fftb
   endelse

   ; Find the total number of records

   headersz  = headervars*4L                          ; header size in Bytes
   blocksize = datablocksz+headersz                   ; record size in Bytes
   dummy     = fstat(wcrlun)
   Nr        = (dummy.size-filehsz)/blocksize         ; total # of recs

endif else begin  ; the file must be open, so get the necessary parameters
   if (keyword_set(wcrlun) eq 0) then wcrlun = 31
   Nr  = tnumrec
   Nrg = rangegates
   wcrd= wcrdate

   if (wcrdate lt 19961001) then $
      message,'Does not work with WCR data prior to Oct 1996' $
   else if wcrdate gt 19990501 then begin 
     Nn = noisesamples  
     if wcrdate lt 20020331 then begin
       bgnd=1
       headervars=40L 
     endif else headervars=5L
   endif
   al        = alg
   fftb      = fftbins
   blocksize = datablocksize+headersize
   filehsz   = filehdrsz
endelse

; Find dimensions
                                    
  if al eq sppa then               Nmode=2 $
  else if al eq pp4a then          Nmode=7 $
  else if al eq pp6a then          Nmode=11 $ 
  else if al eq ffta then begin 
                                   Nmode=1 
                                   Nch  =fftb
  endif else if al eq fftd then begin 
                                   Nmode=2 
                                   Nch  =fftb
  endif else message,'Unknown algorithm.'

; Check which records to read

  if (n_params() eq 1) then begin
   startrec = 0L 
   stoprec = Nr-1 
  endif else if (n_params() eq 2) then $ 
   stoprec = stoprec < (Nr-1)
  n=long(stoprec)-long(startrec)+1L

; Read file header if any

  if wcrd gt 20020331 then begin
    fileheader = {              $
                   version:          1L,          $
                   prf:              lonarr(6),   $ 
                   ppf:              lonarr(6),   $        
                   txsequence:       bytarr(6),   $
                   rxsequence:       bytarr(6,2), $
                   pulselength:      0L,          $
                   txsamples:        0L,          $   
                   noisesamples:     0L,          $
                   rangegates:       0L,          $
                   rangegatedelay:   0L,          $
                   rangegatespacing: 0L,          $
                   rxprotdelay:      0L,          $
                   rxatten:          0L,          $
                   iffilter:         0L,          $
                   averages:         0L,          $
                   alg:              0L,          $
                   fftbins:          0L,          $
                   fftwindow:        0L,          $
                   pulse_mode:       1L,          $ 
                   irigstatus:       1L,          $ 
                   pl:               1L,          $ 
                   min_ips:          1L,          $ 
                   ips:              lonarr(6),   $ 
                   min_prf:          1L,          $ 
                   rgd:              0.0D,        $ 
                   rgs:              0.0D,        $ 
                   memsize:          1L,          $ 
                   hostavg:          1L,          $ 
                   totalavg:         1L,          $
                   one_profile_time: 0.0D,        $ 
                   size:             0UL          $ 
                 }
    point_lun, wcrlun,0
    readse,wcrlun,big_end=bgnd,fileheader
  endif   

; Read requested records 

  ;point_lun, wcrlun, filehsz+startrec*4L*(headervars+Nch*(Nn+Ntx+Nrg)*Nmode)
  point_lun, wcrlun, filehsz+long(startrec)*blocksize

  if n_elements(recraw) eq 0 then begin
     recraw=replicate({header:lonarr(headervars), $
                       data:fltarr(Nch,Nn+Ntx+Nrg,Nmode)},n)
  endif else begin
     dummy = n_elements(recraw)*n_elements(recraw[0].data)
     if dummy ne n*long(Nn+Ntx+Nrg)*long(Nmode*Nch) then $
           recraw=replicate({header:lonarr(headervars), $
                             data:fltarr(Nch,Nn+Ntx+Nrg,Nmode)},n)
  endelse

  readse,wcrlun,big_end=bgnd,recraw

  if keyword_set(closewcr) then close, wcrlun

end
