;$Id: readwcrtb.pro,v 1.6 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       READWCRTB
;
; PURPOSE:
;       Read wcr file time and prof recordings between 'strt' and 'stp' 
;       and correct the time according to tcorrection variable.
;       Determine the interruptions (data on/data off, obsolete after 
;       May 1999) if any.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       READWCRTB, Data, Strt, Stp
;
; INPUTS:
;       Data:  wcr file structure
;       Strt:  start reading point
;              For rmode = 0  -  strt/stp record in the wcr file
;                          1  -  input:  strt/stp profile numbers
;                                output: strt/stp corresponding wcr file records
;                          2  -  input:  strt/stp UTC in hhmmss, long
;                                output: strt/stp corresponding wcr file records
;       Stp:   stop reading point (see above)
;
; KEYWORD PARAMETERS:
;       TIMEMODE: 1 : [sec,microsec]; lonarr(stp-strt+1)
;                 2 : hhmmss.dd     ; dblarr(stp-strt+1)   
;                 3 : sssss.dd      ; dblarr(stp-strt+1) (default)
;       READMODE: 0 : reads profs given strt/stp record number(default)
;                 1 : reads profiles between strt/stp profile numbers
;                 2 : reads profiles within strt/stp UTC    
;        BADPROF: output, array with profile numbers of potentially bad profiles
;        RAWTIME: input, integer,load UNIX time
;                  1 - return unix time in the named variable
;                 -1 - return unix time in the named variable and this is the 
;                      only action by the readwcrtb
;                 output, dblarr(numrec-1), raw UNIX time based on timemode = 1
;                         and readmode = 0; it is corrected if tcorrection > 0
;     BIG_ENDIAN: when set, assumes that the file data are in big endian
;                 order.  The default is BIG_ENDIAN.  
;       HELP:     show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams 
;
; SIDE EFFECTS:
;       Load into wcrdata block:
;       time      - (see tmode), UTC time for every profile in 'mode' format
;       begtime   - (see rmode=2), UTC desired data segment beginning time
;       endtime   - (see rmode=2), UTC desired data segment end time 
;       prof      - lonarr(numRec-1), profile numbers
;       interrupts- lonarr(numIntr,2), (first,last) profiles for interrupts
;       dspsync   - fltarr(3) mean,std and max-min of DSPs' time diff.
;
;       tcorrection is applied to time data if tcorrection ne -999.
;       and timemode is 1 or 2. For timemode=3 time is corrected for
;       fractional seconds in tcorrection (if any) only.  The whole seconds
;       correction is applied to starttime/begtime and stoptime/endtime. 
;       In this case time[0] could be negative number but gt -1.
;
; NOTES:
;       1) READMODE 2 and 3 are slower than 1. It reads all the data
;          to find the desired actual profile or time range 
;       2) For data before May 1999:  TIMEMODE 2 and 3 calculate time using the 
;          clock reading from the odd DSP profiles only and interpolate for the
;          even. 
;          ATTN: The 2 DSPs were not syncronized and therefore the 
;                time is not consistent (small offset, 0.5 to 1.0 sec).
;                This has been fixed after May 1999.
;       3) TIMEMODE 1 gives the raw time readings from the 2 DSPs.  
;          TIMEMODE 1 is not allowed for READMODE 1 and 2.
;       4) See openwcrf.pro for wcrdata and wcrparams structure descriptions.
;       5) interrupts is a 2-d aray with first and last profile numbers for 
;          data on/data off WCR DAQ mode.  If there is no interrupts its value
;          is 0.  For rmode=0, interrupts are between strt and stp; for all
;          other modes interrupts are for the whole wcr file. Interrupts are
;          obsolete after May 1999.
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Sep 1995
;       S.H.,Apr 98 -- added DSPSYNC
;       S.H.,Apr 99 -- BADPROF and modified interrupts determination 
;       S.H.,Aug 99 -- added tcorrection
;                      modified help info and other minor changes
;       S.H.,Nov 99 -- corrected begtime and endtime for tcorrection
;       S.H.,Jan 00 -- added help keyword
;       S.H.,Jul 00 -- added rawtime keyword
;       S.H.,Jan 01 -- added little_endian keyword and made the routine 
;                      independant on the data byteorder in the input file
;       S.H.,Feb 01 -- changed routine name from readvxitb to readwcrtb
;       S.H.,Apr 02 -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro readwcrtb,data,strt,stp,timemode=tmode,readmode=rmode,badprof=badprof,$
    rawtime=rawtime,big_endian=bgnd,help=help 

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'readwcrtb'
  return
endif

nr        = tnumrec-1
dt        = timeinterval
mininterrp= 50             ; min # of prof jump to register an interrupt
badf      = -999.          ; float bad value

tind1=3-2*(wcrdate gt 20020331)
tind2=tind1+1
bind =1-(wcrdate gt 20020331)

; Check the input

if (keyword_set(rawtime) eq 0) then rawt=0 else rawt=rawtime[0]
if (keyword_set(tmode) eq 0) then tmode=3
if (keyword_set(rmode) eq 0) then rmode=0

if not (rmode eq 1 or rmode eq 2 or rmode eq 0) then $
   message,'Inncorrect readmode !'

if ((n_params() lt 3 or tmode eq 1) and rmode gt 0) then $
   message,'Incorrect readmode, timemode or number of parameters !'

if (rmode eq 1) then $ 
   if (strt lt fstblocknum or stp gt lstblocknum) then $
      message,'Incorrect start/stop profile number(s) !'

if (rmode eq 2) then begin
   tstr=long(timestr2sec(starttime)) 
   k=stoptime
   k=long(timestr2sec(k)) 
   if (strt lt tstr*(tstr lt k) or stp gt k ) then $
      message,'Incorrect start/stop UTC !'
endif

if (n_params() eq 1) then begin
   rmode = 0 
   strt = 0
   stp = nr
endif else if (n_params() eq 2) then stp = nr

if (rmode eq 0) then begin
  n = stp-strt+1
  if (n le 1) or (strt lt 0) or (stp gt nr) then $
     message,'Wrong start/stop record number(s) or stop-start<=1 !'
endif else begin
  n = tnumrec
  ptstrt=strt
  ptstp=stp
  strt=0
  stp=nr
endelse

strt = long(strt)

; Check byte order for file data and this computer architecture

  swapbytes = 1L   ; 1 is default for data swaping

  if keyword_set(bgnd) then byteorder,swapbytes,/swap_if_little_endian $
  else byteorder,swapbytes,/swap_if_big_endian

; Get raw time and exit if requested

if rawt eq -1 then begin
   rawtime=dblarr(n)
   for k = long(strt), stp do begin
      temp = k-strt
      t = data(k)
      if swapbytes eq 1 then t=swap_endian(t)
      rawtime[temp] = double(t.header(tind1))+double(t.header(tind2)/1e6)         
     endfor
     if tcorrection ne badf and tcorrection ne 0. then $
       rawtime=rawtime+tcorrection
   return                           ; EXIT (input rawtime = -1) 
endif

if tcorrection ne badf then begtime=(endtime=long(tcorrection)) $ 
else begtime=(endtime=0.d0)

if strt eq 0 then begtime = starttime
if stp eq nr then endtime = stoptime

; Process according to TIMEMODE. Use clock from odd DSP profile (even
; index of 'time') and interpolate for the even; for mode 2 and 3 only
; and for data before May 1999

  prof = lonarr(n)
  timemode=tmode   ;  save the timemode

  if (tmode eq 1) then begin
     time = lonarr(2,n)
     for k = long(strt), stp do begin
         temp = k-strt
         t = data(k)
         if swapbytes eq 1 then t=swap_endian(t)
         prof(temp) = t.header(bind)
         time(0:1,temp) = t.header(tind1:4)         
     endfor
   
    ; Correct time if time correction is available. Assume t[0,0] gt tcorrection

    if tcorrection ne badf and tcorrection ne 0. then begin
       t3=long(tcorrection)
       t4=long((tcorrection-t3)*1e6)
       time[0,*]=time[0,*]+t3
       time[1,*]=time[1,*]+t4
       ind=where(time[1,*] ge 1000000L)
       if ind[0] ne -1 then begin
          time[0,ind]=time[0,ind]+1L
          time[1,ind]=time[1,ind]-1000000L
       endif
    endif     

    ; Assamble rawtime if requested
 
    if rawt eq 1 then rawtime=double(time[0,*])+double(time[1,*]/1e6)

    ; Calculate mean,std,min and max for the DSP clock differences
 
    ind=2*lindgen(n/2)
    dspsync[0:2]=float([mean((double(time(0,*))+double(time(1,*)/1e6))(ind+1)-$
    (double(time(0,*))+double(time(1,*)/1e6))(ind)),$
    std((double(time(0,*))+double(time(1,*)/1e6))(ind+1)-$
    (double(time(0,*))+double(time(1,*)/1e6))(ind)),$
    max((double(time(0,*))+double(time(1,*)/1e6))(ind+1)-$
    (double(time(0,*))+double(time(1,*)/1e6))(ind))-$
    min((double(time(0,*))+double(time(1,*)/1e6))(ind+1)-$
    (double(time(0,*))+double(time(1,*)/1e6))(ind))])

    dspsync=dspsync*1000.

  endif else if (tmode eq 2) or (tmode eq 3) then begin

     time = dblarr(n)
     t = data(strt)
     if swapbytes eq 1 then t=swap_endian(t)
     t31= t.header(tind1)
     t = data(strt+1)
     if swapbytes eq 1 then t=swap_endian(t)
     if (t.header(tind1) lt t31) then t31=t.header(tind1)
     for k = long(strt), stp do begin
        temp = k-strt
        t = data(k)
        if swapbytes eq 1 then t=swap_endian(t)
        prof(temp) = t.header(bind)
        time(temp) = double(t.header(tind1))+double(t.header(tind2)/1e6)
     endfor

     ; Assamble rawtime if requested
 
     if rawt eq -1 then rawtime=time+tcorrection

     ; Correct time for fractional second correction if any.  

       temp=tcorrection-long(tcorrection)
       if tcorrection ne badf and abs(temp) gt 0. then time=time+temp 

     ; Calculate mean,std, max-min for the DSP clock differences
 
     ind=2*lindgen(n/2)
     dspsync[0:2]=float([mean((time)(ind+1)-(time)(ind)),$
             std((time)(ind+1)-(time)(ind)),$
             max((time)(ind+1)-(time)(ind))-min((time)(ind+1)-(time)(ind))])

     if (n gt 2 and wcrdate lt 19990501) then begin

     ;  Correct the time for odd profiles       
        t  = strt  & if (strt mod 2) then t = strt+1
        t3 = stp-1 & if (stp  mod 2) then t3 = stp
        even = lindgen((t3-t+1)/2 - 1)*2+(strt mod 2)

     ;  Use the difference dt     
        time(even+1) = time(even)+dt
        k = t3-strt
        time(k) = time(k-1)+dt
        if (t ne strt) then time(0) = time(1)-dt

     endif   

     if (tmode eq 2) then begin     
 
     ; Correct time for whole seconds correction if available.  
     ; It is assumed that time[0]+tcorrection > 0

       if tcorrection ne badf and abs(tcorrection) gt 1. then begin
         time=time+long(tcorrection)
       endif 

     ; Calculate time in hhmmss.dd
       t3 = floor(time)
       time = time-t3
       t3 = t3 mod 86400
       t4 = t3/3600*10000
       t3 = t3 mod 3600
       time = double(t4+t3/60*100+(t3 mod 60))+time
     endif else time = time - t31
  endif else $
     message,'Inncorrect timemode !'

  if (rmode eq 0 and n_params() eq 2) then begin
     t=data(strt) & if swapbytes eq 1 then t=swap_endian(t)
     begtime=sec2timestr((begtime+t.header(tind1)) mod 86400)

  endif else if (rmode eq 0 and n_params() gt 2) then begin
     t=data(strt) & if swapbytes eq 1 then t=swap_endian(t)
     begtime=sec2timestr((begtime+t.header(tind1)) mod 86400)
     t=data(stp) &  if swapbytes eq 1 then t=swap_endian(t)
     endtime=sec2timestr((endtime+t.header(tind1)) mod 86400)

  endif else if (rmode eq 1) then begin
     strt=where(prof ge ptstrt) & strt=strt(0)
     stp=where(prof le ptstp) & stp=stp(n_elements(stp)-1)
     t=data(strt) & if swapbytes eq 1 then t=swap_endian(t)
     begtime=sec2timestr((begtime+t.header(tind1)) mod 86400)
     t=data(stp) &  if swapbytes eq 1 then t=swap_endian(t)
     endtime=sec2timestr((endtime+t.header(tind1)) mod 86400)
     prof=prof(strt:stp)
     if (tmode eq 2) then time=time(strt:stp) $
     else if (tmode eq 3) then time=time(strt:stp)-floor(time(strt))
     if rawt eq -1 then rawtime=rawtime[strt:stp]

  endif else if (rmode eq 2) then begin
     if (tmode eq 2) then begin
        strt=(where(long(time) ge ptstrt))(0) 
        stp=(where(long(time) ge ptstp))(n_elements(stp)-1)

        t=data(strt) & if swapbytes eq 1 then t=swap_endian(t)
        begtime=sec2timestr((begtime+t.header(tind1)) mod 86400)
        t=data(stp) &  if swapbytes eq 1 then t=swap_endian(t)
        endtime=sec2timestr((endtime+t.header(tind1)) mod 86400)

        time=time(strt:stp)         
     endif else $
     if (tmode eq 3) then begin
        ptstrt=time2sec(ptstrt)
        tstr=time2sec(tstr)-86400.*(tstr gt ptstp)
        strt=where(long(time) ge ptstrt-tstr) & strt=strt(0)
        ptstp=time2sec(ptstp)
        stp=where(long(time) le ptstp-tstr) & stp=stp(n_elements(stp)-1)

        t=data(strt) & if swapbytes eq 1 then t=swap_endian(t)
        begtime=sec2timestr((begtime+t.header(tind1)) mod 86400)
        t=data(stp) &  if swapbytes eq 1 then t=swap_endian(t)
        endtime=sec2timestr((endtime+t.header(tind1)) mod 86400)

        time=time(strt:stp)-floor(time(strt))
     endif
     prof=prof(strt:stp)
     if rawt eq -1 then rawtime=rawtime[strt:stp]
  endif

; Check for bad profiles

  ; n=stp-strt

  ind=where(prof le 0 or prof gt 65536)
  if ind(0) ne -1 then begin
     k=n_elements(ind)
     print,'ATTN: ',num2str(k),' bad profs found.'
     print,'        Profiles are not removed from the data or header.'
     if k lt 3 then message,'Too many bad profiles !'  
  ;   n=n-k
  endif 

; Correct profile numbers greater than 65536.  This is because a WCR program
; uses 2 byte register (unsigned integer) for the profile numbers.
; Profile counter starts from 1 and folds on 65536.

  ind=where(prof gt 0 and prof le 65536)     ; also needed for the interrupts 
  k=n_elements(ind)-1                        
  sprof=shift(prof(ind),1) 
  ind1=where(prof(ind(1:k))-sprof(1:k) lt 0)
  if ind1(0) ne -1 then begin
     ind1=ind1+1
     if n_elements(ind1) eq 1 then $
        prof(ind(ind1(0):k))=prof(ind(ind1(0):k))+65536 $
     else $
        for l=1,n_elements(ind1)-1 do $
            prof(ind(ind1(l-1):ind1(l)))=prof(ind(ind1(l-1):ind1(l)))+l*65536
  endif 

  ; Correct profiles for the strt if necessary

  if strt gt 0 then prof=prof+long(fstblocknum+strt)/65536*65536

; Calculate interrupts if any (for PPMAG and SPPMAG only)
  
  if alg ne fftalg then begin 
     sprof=shift(prof(ind),1)
     ind1=where(abs(prof(ind(1:k))-sprof(1:k)) gt mininterrp)
     if (ind1(0) eq -1) then $
        interrupts=0 $
     else begin
        interrupts=lonarr(n_elements(ind1)+1,2)
        interrupts(*,0)=prof(ind([0,ind1+1]))
        interrupts(*,1)=prof(ind([ind1,n]))
     endelse
  endif else interrupts=0
 
; Get the number of read profiles

  numrec=n_elements(prof) 

end
