;$Id: wcracvel.pro,v 1.10 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;      WCRACVEL
;
; PURPOSE:
;      Returns aircraft (AC) velocity contribution (fltarr(m) in m/s) into the 
;      WCR beam w.r.t. ground. Positive contribution means away from the radar.
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;      res=wcracvel(acvel,roll,pitch,heading,beaminit,gpscor=gpscor,rmcor=rmcor,$
;                   radarm=radarm,wcrb=wcrb,wcrt=wcrt,acv=acv,deg=deg,help=help)
;
;      res=wcracvel(acvel,trnsm,beaminit)
;
; INPUTS:
;      Acvel:   fltarr(m,3),   AC velocity [m/s] in Earth coordinates 
;                              [[east-west vel],[north-south vel],[vert. vel]]
;      Roll:    fltarr(m),     AC roll in radians                  or
;               fltarr(3,3,m), AC to Earth transformation matrix   or
;               fltarr(3,m),   WCR beam in Earth coordinate system 
; 
;      Pitch:   fltarr(m), AC pitch in radians 
;      Heading: fltarr(m), AC heading in radians
;
;      Instead of roll,pitch,heading the AC to Earth coordinate systems 
;      transformation matrix, trnsm, fltarr(3,3,n) can be given
;
;      Beaminit: Initialize antenna beam; 
;                Default beams:
;                 0         - side-looking KingAir, KA, (main) 12"antenna
;                 1         - up-looking KA (main+mirror) 12" antenna
;                 2         - KA side-fore antenna(~35 deg)
;                 3         - KA nadir 18" antenna
;                 4         - KA down-fore 15" antenna(~30 deg)
;                 5         - NCAR C130 nadir 12" antenna (7/16/01)
;                 6         - NCAR C130 down-aft 12" antenna(7/16/01)
;                 7         - NRC CV580 side looking 12" antenna
;                 8         - NRC CV580 nadir looking 12" antenna
;                 fltarr(m) - side/up antenna for KA determined by the KA mirror 
;                             (0-side, 1-up), m > 3
;                Antenna beam pointing vector:
;                 fltarr(3)  - beam unit vector in AC coordinate system
;                              [AC along axis, right wing, down]
;                 fltarr(3,m)- beam unit vectors for all data points
;
; KEYWORD PARAMETERS:
;      GPSCOR:  input, [[uerr],[verr]], E-W and N-S velocity corrections
;               (default - keyword not defined; correction is not applied)
;      RMCOR:   input, [[rollr],[pitchr],[yawr]], AC attitude rates in rad/sec
;                      (default - not defined; correction is not applied)
;               output, if input is defined as above the output is radar moment
;                       correction
;      RADARM:  wcr antenna position in meters (AC coords) w.r.t INS;
;               default is [-1.88,0.,-0.79] for UWKA side/up antenna; antenna 
;               installations on other aircraft are described in wcracvelm.pro  
;      WCRT:    output, fltarr(3,3,n), AC-to-Earth transformation matrix
;      WCRB:    output, fltarr(3,n), WCR beam in Earth coordinate system  
;      ACV:     output, fltarr(n,3), Acvel(GPS/RMCOR corrected if requested)
;                       in AC corrdinates.
;      DEG:     when set the attitude params are given in degrees
;      HELP:    show this text
;
; SIDE EFFECTS:
;       The function returns an empty string if the usage is incorrect
;
; NOTES:
;       Vertical velocity component of acvel could be chosen from several 
;       calculated in KA processing:  inertial baro (hwp3), ins component 
;       (hivs), UW adaptive filter (wnb). 
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov, December 2000
;      S.H., Jan 01 -- minor changes
;      S.H., Oct 01 -- changes in the help
;      S.H., Aug 02 -- changed keyword gpsc to gpscor
;      S.H., Feb 03 -- changed kaaggat to ac2gt
;      S.H., Jul 03 -- fixed a bug with passing deg keyword
;      S.H., Oct 03 -- added ACV keyword; removed possible radar beam input
;      S.H., Nov 03 -- changes in the help
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcracvel, acvel,roll,pitch,heading,beaminit,gpscor=gpscor,rmcor=rmcor,$ 
                   radarm=radarm,wcrb=wcrb,wcrt=wcrt,acv=acv,deg=deg,help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'wcracvel'
  return,''
endif

if not(n_params() eq 3 or n_params() eq 5) then begin
   message,'Usage:res=wcracvel(acvel,roll,pitch,heading,beaminit,gpscor=gpscor,$",/info
   print,'                       rmcor=rmcor,radarm=radarm,wcrb=wcrb,wcrt=wcrt,$'
   print,'                       deg=deg,help=help)'
   print,'                  res=wcracvel(acvel,trnsm,beaminit)'
   return,''
endif

; Applied GPS correction to ins ac velocity if requested

  if n_elements(gpscor) gt 0 then $
     acvg=[[acvel[*,0]-gpscor[*,0]], [acvel[*,1]-gpscor[*,1]], [acvel[*,2]]] $
  else acvg=acvel

; Calculate transformation matrix if not given otherwise assign the proper input

  if n_params() eq 5 then wcrT=ac2gt(roll,pitch,heading,deg=deg) $
  else if n_params() eq 3 then begin wcrT=roll & beaminit=pitch & end 

; Find the number of measurements and initialize the output array

  n = n_elements(acvg[*,0])
  wcracv=fltarr(n)

; Calculate the wcr beam in Earth coordinates if not given

  if n_params() gt 2 then begin

    wcrB=wcrbeam(wcrT,beaminit,deg=deg)

    ; Calculate radar moment contribution if requested

    if n_elements(rmcor) gt 0 then begin

      ; Default KA radar antenna arm in meters and AC coords
      ; (8/00 - determined by Dave Leon)

      if n_elements(radarm) eq 0 then radarm=[-1.88,0.,-0.79]   

      ; calculate radar moment contribution (rmcor x radarm), 
      ; where rmcor=[[rollr],[pitchr],[yawr]]

      if keyword_set(deg) then begin
         rollr =!dtor*rmcor[*,0]
         pitchr=!dtor*rmcor[*,1]
         yawr  =!dtor*rmcor[*,2]
      endif else begin
         rollr =rmcor[*,0]
         pitchr=rmcor[*,1]
         yawr  =rmcor[*,2]
      endelse

      rmcor[*,0]=(pitchr*radarm[2] -   yawr*radarm[1])
      rmcor[*,1]=(  yawr*radarm[0] -  rollr*radarm[2])
      rmcor[*,2]=( rollr*radarm[1] - pitchr*radarm[0])

      ; transfrom to ground coordinates
 
      for k=0L,n-1 do rmcor[k,*]=rmcor[k,*]#wcrT[*,*,k]

      ; add radar moment to the ac velocity to get the ac vel at the radar
       
      acvg=acvg+rmcor

    endif
  endif

; Calculate the AC vel component into the wcr beam and acv in AC coord.

  acv=fltarr(n,3)
  wcrT=transpose(wcrT,[1,0,2])  ; change to g2ac transformation matrix
  for k=0L,n-1 do begin
     wcracv[k]=wcrB[*,k]##acvg[k,*]
     acv[k,*]=wcrT[*,*,k]##acvg[k,*]
  endfor

; Return the AC vel component into the wcr beam

  return, wcracv 

end
