;$Id: wcracvelm.pro,v 1.11 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRACVELM
;
; PURPOSE:
;      Returns aircraft (AC) velocity contribution (fltarr(m) in m/s) into the
;      WCR beam w.r.t. ground. Then wcracvel is called to calculate the velocity
;      contribution. Positive contribution means away from the radar.
;      
;      This routine calls acloadins to load the appropriate data if acdata
;      is not given. AC data files from UWKA (N2UW) or NCAR/C130 (N130AR)
;      only are allowed.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;      res=wcracvelm(acdata,cdfid,beaminit=beaminit,startt=startt,stopt=stopt, $
;                  gpsc=gpsc,rmcor=rmcor,radbeam=radbeam,acv=acv,wcracw=wcracw,$
;                  filename=filename,ncnames=ncnames,close=close,help=help)                   
;
; INPUTS:
;       Acdata: structure, AC IRS/INS data matching AC time
;               ATTN: If defined ignores cdfid and filename inputs otherwise
;                     acdata is output
;               acdata.time   -> double(m), AC unix time in sec (see Notes)
;             ; Following angles are in radians or degrees if deg keyword set
;               acdata.roll   -> fltarr(m), AC roll angle
;               acdata.pitch  -> fltarr(m), AC pitch angle
;               acdata.heading-> fltarr(m), AC true heading from North
;             ; AC ground velocities in [m/s]
;               acdata.ug     -> fltarr(m), East-West AC ground velocity
;               acdata.vg     -> fltarr(m), North-South AC ground velocity
;               acdata.wg     -> fltarr(m), AC vertical velocity
;             ; UWKA GPS corrections of ins AC velocities (ignore for C130)
;               acdata.uerr   -> fltarr(m), E-W vel correction in m/s
;               acdata.verr   -> fltarr(m), N-S vel correction in m/s
;             ; AC measured winds in [m/s]
;               acdata.uw     -> fltarr(m), East wind velocity component
;               acdata.vw     -> fltarr(m), North wind velocity component
;               acdata.ww     -> fltarr(m), vertical wind velocity component
;             ; AC attitude rates in rad/sec or deg/sec (if deg keyword is set)
;               acdata.rollr  -> fltarr(m,3), roll rate
;               acdata.pitchr -> fltarr(m,3), pitch rate
;               acdata.yawr   -> fltarr(m,3), yaw rate
;             ;
;               acdata.tas    -> fltarr(m), true air speed in m/s
;             ;
;             ; UWKA radar wing mirror position 
;               acdata.wcrmirror -> intarr(m), KA radar wing mirror position
;                                   (0-side, 1-up, for side/up antenna)
;
;             ; AC data info: aircraft id (N2UW, N130AR or NRC-CV580)
;               acdata.info   -> string, aircraft ID
;
;             Note: If any of the non-critical data (e.g., attitude rate) is
;                   not available, -1 is recorded
;
;       Cdfid:  netCDF file ID, returned from a previous call to
;               ncdf_open, ncdf_create, ncdf_varload or ncdf_fcopy
;               (ignored if acdata exist)
;
; KEYWORD PARAMETERS:
;       ACV:      output, fltarr(m,3) matching ACTIME, AC velocity (GPS/RMCOR 
;                         corrected if requested) in AC corrdinates.
;       BEAMINIT: input, Initialize antenna beam; 
;                 Default beams:
;                  -1(default) - use UWKA radar mirror variable for side/up
;                                pointing direction of UWKA WCR main antenna
;                   0         - side-looking KingAir, KA, (main) 12"antenna
;                   1         - up-looking KA (main+mirror) 12" antenna
;                   2         - KA side-fore antenna(~35 deg)
;                   3         - KA nadir 18" antenna
;                   4         - KA down-fore 15" antenna(~30 deg)
;                   5         - NCAR C130 nadir 12" antenna (7/16/01)
;                   6         - NCAR C130 down-aft 12" antenna(7/16/01)
;                   7         - NRC CV580 side looking 12" antenna
;                   8         - NRC CV580 nadir looking 12" antenna
;                   fltarr(m) - side/up antenna for KA determined by the  
;                               KA mirror (0-side, 1-up), m > 3
;                 Antenna beam pointing vector:
;                   fltarr(3)  - beam unit vector in AC coordinate system
;                                [AC along axis, right wing, down]
;                   fltarr(3,m)- beam unit vectors for all data points
;
;       CLOSE:    when set AC netcdf file will be closed(ignored if acdata exist).
;       GPSC:     when set does NOT apply GPS correction to AC E-W and N-S
;                 velocity for UWKA only (ignored for NCAR C130)
;                 NCAR C130 AC E-W and N-S ground speed are GPS corrected.
;       FILENAME: input, string, AC netCDF file name; if given will open netCDF
;                 file first. In this case Cdfid is an output parameter
;                 (ignored if acdata exist)
;       NCNAMES:  input, string array, AC netcdf varnames to be used; if NCNAMES
;                  not defined N2UW or N130AR default var names are used
;                  (ignored if acdata exist) 
;                 
;                  ncnames[0] - AC true air speed in m/s
;                  ncnames[1] - AC roll [deg]
;                  ncnames[2] - AC pitch [deg]
;                  ncnames[3] - AC true heading [deg]
;                  ncnames[4] - AC east-west(EW) ground velocity [m/s]
;                  ncnames[5] - AC north-south(NS) ground velocity [m/s]
;                  ncnames[6] - AC vertical velocity [m/s]
;                  ncnames[7] - AC East component of the wind [m/s]
;                  ncnames[8] - AC North component of the wind [m/s]
;                  ncnames[9] - AC vertical wind [m/s]
;                  ncnames[10]- AC roll rate [rad/s] (use '' if N/A)
;                  ncnames[11]- AC pitch rate [rad/s](use '' if N/A)
;                  ncnames[12]- AC heading rate [rad/s] (use '' if N/A)
;                  ncnames[13]- AC EW ground vel. GPS correction(use '' if N/A)
;                  ncnames[14]- AC NS ground vel. GPS correction(use '' if N/A)
;
;                  Note: define first 10 names only if the last 5 vars are
;                        not needed; see ACLOADINS.PRO help for default variable 
;                        names
;       RADBEAM:  output, fltarr(3,m), WCR beam in Earth coordinate system
;       RMCOR:    input, integer/fltarr(3), applies antenna moment correction
;                    0 - antenna moment correction NOT applied  
;                    1 - antenna moment correction applied
;                    [x,y,x] - ant. position in aircraft coord w.r.t IRS in meters
;                 Default: for UWKA or NRC Convair not applied (0) 
;                          for NCAR C130 applied(1)
;                 Default antenna positions w.r.t. INS/IRS in meters:
;                          N2UW side/up antenna: [-1.88,0.,  -0.79]
;                          NCAR C130 antennas:   [-18., 0.,   0.00] 
;                          NRC CV580 antennas:   [-7.04,0.75, 0.00]
;                 All N2UW antenna positions in AC coordinates(meters from INS):
;                          side/up antenna   [-1.88,0.,-0.79]
;                          side_fore antenna [-1.22,0.,-0.79]
;                          down antenna      [-1.88,0., 0.  ]
;                          done-fore antenna [-2.00,0., 0.  ]
;
;       STARTT:   input, long, start time in hhmmss (ignored if acdata exist)
;       STOPT:    input, long, stop time in hhmmss including last second record
;                 (ignored if acdata exist)
;       WCRACW:   output, fltarr(m) matching ACTIME, AC measured wind contribution
;                 into the WCR beam
;       HELP:     shows this text
;
; OUTPUTS:
;       Acdata: structure, return AC IRS/INS data matching AC time
;               (for discription see INPUTS)
;       Cdfid:  netCDF file ID, returned if Filename is given
;
; SIDE EFFECTS:
;       The function returns an empty string for help and if the usage is
;       incorrect
;
; NOTES:
;       KingAir netCDF files follow the NCAR-RAF/nimbus convention.  This
;       convention requires that the time dimension, 'time' (unlimited or not),
;       is the first (last for idl, which uses fortran order) and the next
;       dimension, if exists, is always samples per second, 'sps'.
;
;       Actime time vector is assembled from two variables:
;       'base_time' - seconds since 00:00:00 Jan 1 1970
;       'time_offset' - seconds since base_time, a sequence w/ 1 sec step
;       The result is a vector representing seconds since 00:00:00 Jan 1 1970.
;       actime is resampled according to the input file INS data sps dimension.
;
;       If startt and stopt are given netcdf variables are extracted within the
;       specified time. If stopt is not defined the variables are read untill
;       the end of the file.  And if both, startt and stopt are not defined or
;       are invalid the complete data set for the variables are read.
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, December 2000
;       S.H., Jan 01 -- minor changes
;       S.H., Oct 01 -- changes in the help, added ncnames and beaminit keywords
;       S.H., Feb 02 -- modified rmcor keyword added close keyword
;       S.H., Mar 02 -- added close, tas and wcrmirror keywords, changed RMCOR
;                       default for C130
;       S.H., Aug 02 -- changed gpsc to gpscor amd some changes in handling it
;       S.H., Oct 02 -- added wcracw keyword, modified ncnames and beaminit
;       S.H., Feb 03 -- added acdata keyword, modified ncnames and help text
;                       replaced ka with ac in var names where appropriate and
;                       kaaggat to ac2gt
;       S.H., May 03 -- major change in the formal parameters - now can use
;                       acdata structure as input and skips reading the data
;       S.H., Sep 03 -- removed AC 'noread' option for ncnames; now if acdata
;                       exist no data is being read from the AC data file.
;       S.H., Oct 03 -- added NRC Convair580 aircraft (currently acdata vars
;                       have default names identical to N2UW); added ACV keyword
;                       and eliminated wcrbeam calculation before calling
;                       wcracvel when rmcor not applied
;       S.H., Nov 03 -- changes in the help
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcracvelm,acdata,cdfid,beaminit=beaminit,startt=startt,stopt=stopt, $
                   gpsc=gpsc,rmcor=rmcor,radbeam=radbeam,acv=acv,wcracw=wcracw,$
                   filename=filename,ncnames=ncnames,close=close,help=help

  on_error,2

; Check some of the inputs

  if keyword_set(help) then begin
    doc_library,'wcracvelm'
    return,''
  endif

  if n_params() eq 0 then begin
    print,"Usage: res=wcracvelm(acdata,cdfid,beaminit=beaminit,startt=startt, $"
    print,"                     stopt=stopt,gpscor=gpscor,rmcor=rmcor,acv=acv,$"
    print,"                     radbeam=radbeam,wcracw=wcracw,ncnames=ncnames,$"
    print,"                     filename=filename,close=close,help=help)"
    return,''
  endif

  if n_elements(beaminit) eq 0 then beam=-1 else beam=beaminit

  ;NCNAMES:  input, string 'noread' - will skip reading AC data from the 
  ;           AC netcdf file. This requires ACDATA structure to be input.
  ;           FILENAME keyword and Cdfid input are ignored. 
  ;readflg=1
  ;if n_elements(ncnames) eq 1 then $
  ;if ncnames eq 'noread' then begin
  ;  readflg=0
  ;  if n_elements(acdata) eq 0 then $
  ;    message,"ACDATA is not defined while 'noread' is requested"
  ;endif
  ; If readflg=1 and filename is given open the file first.  
  ; Close the old one if exists.
  ;  if readflg eq 1 then begin

  if n_elements(acdata) eq 0 then begin
    if n_elements(filename) gt 0 then begin
      if n_elements(Cdfid) eq 1 then ncdf_close,Cdfid
      Cdfid=ncdf_open(filename)
    endif
    ncdf_attget,Cdfid,'Aircraft',acname, /global 
    acname=strupcase(string(acname))
  endif else acname=acdata.info

  if n_elements(rmcor) eq 0 then begin
    if acname eq 'N130AR' then begin
       rmc=1 & radarm=[-18.,0.,0.]
    endif else if acname eq 'NRC-CV580' then begin
       rmc=0 & radarm=[-7.04,-0.75,0.]  ; not applied due to undetermined
                                        ; problem in attitude rates
     ; rmc=0 & radarm=[ 0.,   0.  ,0.]  ; new INS  after 19 Nov 03
    endif else rmc=0
  endif else begin
    if n_elements(rmcor) eq 1 then begin
       if rmcor eq 0 then rmc=0 else begin
         rmc=1
         if rmcor ne 0 and acname eq 'N2UW' then $
           radarm=[-1.88,0.,-0.79]  $            ; side antenna default
         else if rmcor ne 0 and acname eq 'N130AR' then $
           radarm=[-18.,0.,0.] $                 ; valid for both down antennas
         else if rmcor ne 0 and acname eq 'NRC-CV580' then $
           radarm=[-7.04,-0.75,0.] $             ; side& down antenna default
         ; radarm=[ 0.,   0.  ,0.] $             ; new INS after 19 Nov 03
         else message,'Invalid rmcor value.'
       endelse
    endif else if n_elements(rmcor) eq 3 then begin
       radarm=rmcor
       rmc=1
    endif else message,'Invalid rmcor value.'
  endelse

; Load data

  deg  =0        ; convert all angles into radians

 ;if readflg then $
  if n_elements(acdata) eq 0 then acdata=acloadins(Cdfid,startt=startt,$
         stopt=stopt,deg=deg,ncnames=ncnames,sps=sps,close=close,/wcrdvc)

; Update beaminit if necessary

  if beam[0] eq -1 and acname eq 'N2UW' then beam=acdata.wcrmirror

; Check if attitude rate vars are available

  if acdata.rollr[0] eq -1 then begin
     message,'Attitude rate not available. Ignore radar moment correction.',/inf
     rmc=0
  endif

; Check if acvelocity errors exists and gps correction for KingAir is requested

  if acname eq 'N130AR' or acname eq 'NRC-CV580' then gpsc=1 $
  else if n_elements(gpscor) eq 0 then gpsc=0 $
  else gpsc=abs(gpscor)<1
  if acname eq 'N2UW' then if gpsc eq 0 and acdata.uerr[0] eq -1 then begin
     message,'AC vel errors not available. Ignore gps correction.',/inf
     gpsc=1
  endif 

; Process

  ; calculate transformation matrix
  wcrT   =ac2gt(acdata.roll,acdata.pitch,acdata.heading)

  case 2-gpsc+2*rmc of
     1: begin ; radar moment NOT APPLIED, GPS correction NOT APPLIED
         wcracv= wcracvel([[acdata.ug],[acdata.vg],[acdata.wg]],wcrT,beam,$
                 acv=acv,wcrb=radbeam)
        end
     2: begin ; radar moment NOT applied, GPS correction APPLIED
         wcracv= wcracvel([[acdata.ug],[acdata.vg],[acdata.wg]],wcrT,beam,$
                 gpscor=[[acdata.uerr],[acdata.verr]],acv=acv,wcrb=radbeam)
        end
     3: begin ; radar moment APPLIED, GPS correction NOT APPLIED
         wcracv= wcracvel([[acdata.ug],[acdata.vg],[acdata.wg]],wcrT,beam,$
                 rmcor=[[acdata.rollr],[acdata.pitchr],[acdata.yawr]],$
                 radarm=radarm,wcrb=radbeam,acv=acv)
        end
     4: begin ; radar moment APPLIED, GPS correction APPLIED
         wcracv= wcracvel([[acdata.ug],[acdata.vg],[acdata.wg]],wcrT,beam,$
                 gpscor=[[acdata.uerr],[acdata.verr]],$
                 rmcor=[[acdata.rollr],[acdata.pitchr],[acdata.yawr]],$
                 radarm=radarm,wcrb=radbeam,acv=acv)
        end
  endcase

  ; calculate wind component into WCR beam and return wcracv

  wcracw =wcracwind([[acdata.uw],[acdata.vw],[acdata.ww]],radbeam) 
  return, wcracv

end

