;$Id: wcracwind.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;      WCRACWIND
;
; PURPOSE:
;      Returns aircraft (AC) measured wind contribution (fltarr(n) in m/s) into 
;      the WCR beam. Positive contribution means away from the radar.
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;      res=wcracwind(acwind,roll,pitch,heading,beaminit, $
;                    wcrb=wcrb,wcrt=wcrt,deg=deg,help=help)
;
;      res=wcracwind(acwind,trnsm,beaminit)
;
;      res=wcracwind(acwind,wcrbeam)
;
; INPUTS:
;      Acwind:  fltarr(n,3),   AC measured wind in [m/s] 
;                              [[wind East comp],[wind North comp],[vert.wind]]
;      Roll:    fltarr(n),     AC roll in radians                  or
;      Trnsm:   fltarr(3,3,n), AC to Earth transformation matrix   or
;      Wcrbeam: fltarr(3,n),   WCR beam in Earth coordinate system 
; 
;      Pitch:   fltarr(n), AC pitch in radians 
;      Heading: fltarr(n), AC heading in radians
;
;      Instead of roll,pitch,heading the AC to Earth coordinate systems 
;      transformation matrix, trnsm, fltarr(3,3,n), or already calculated
;      WCR beam, wcrbeam, fltarr(3,n), in ground corrdinates can be given
;
;      Beaminit: input, Initialize antenna beam; 
;                Default beams:
;                  0         - side-looking KingAir, KA, (main) 12"antenna
;                  1         - up-looking KA (main+mirror) 12" antenna
;                  2         - KA side-fore antenna(~35 deg)
;                  3         - KA nadir 18" antenna
;                  4         - KA down-fore 15" antenna(~30 deg)
;                  5         - NCAR C130 nadir 12" antenna (7/16/01)
;                  6         - NCAR C130 down-aft 12" antenna(7/16/01)
;                  7         - NRC CV580 side looking 12" antenna
;                  8         - NRC CV580 nadir looking 12" antenna
;                  fltarr(n) - side/up antenna for KA determined by the  
;                              KA mirror (0-side, 1-up), m > 3
;                Antenna beam pointing vector:
;                  fltarr(3)  - beam unit vector in AC coordinate system
;                               [AC along axis, right wing, down]
;                  fltarr(3,n)- beam unit vectors for all data points
;
; KEYWORD PARAMETERS:
;      WCRT:     output, fltarr(3,3,n), AC-to-Earth transformation matrix
;      WCRB:     output, fltarr(3,n), WCR beam in Earth coordinate system  
;      DEG:      when set the attitude params are given in degrees
;      HELP:     show this text
;
; SIDE EFFECTS:
;      The function returns an empty string if the usage is incorrect
;
; NOTES:
;       
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov, August 2002
;      S.H., Nov 2003 - changes in the help text
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcracwind, acwind,roll,pitch,heading,beaminit, $ 
                    wcrb=wcrb,wcrt=wcrt,deg=deg,help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'wcracwind'
  return,''
endif

if not(n_params() eq 2 or n_params() eq 3 or n_params() eq 5) then begin
   message,'Usage: res=wcracwind(acwind,roll,pitch,heading,beaminit,wcrb=wcrb,wcrt=wcrt,deg=deg,help=help)',/info
   print,'                  res=wcracwind(acwind,trnsm,beaminit)'
   print,'                  res=wcracwind(acwind,wcrbeam)'
   return,''
endif

; Calculate transformation matrix if not given otherwise assign the proper input

  if n_params() eq 5 then wcrT=ac2gt(roll,pitch,heading,deg=deg) $
  else if n_params() eq 3 then begin wcrT=roll & beaminit=pitch & end $
  else wcrB=roll 

; Find the number of measurements and initialize the output array

  n = n_elements(acwind[*,0])
  wcracw=fltarr(n)

; Calculate the wcr beam in Earth coordinates if not given

  if n_params() gt 2 then wcrB=wcrbeam(wcrT,beaminit)

; Calculate the AC wind component into the wcr beam

  for k=0L,n-1 do wcracw[k]=wcrB[*,k]##acwind[k,*]

; Return the AC wind component into the wcr beam

  return, wcracw 

end
