;$Id: wcrbeam.pro,v 1.10 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;      WCRBEAM
;
; PURPOSE:
;      Returns WCR antenna beam coordinates in Earth or aircraft (AC) 
;      coordinate system; fltarr(3,m) unit vectors in AC time
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;      res=wcrbeam(roll,pitch,heading,beaminit,deg=deg,acbeam=acbeam,help=help)
;      res=wcrbeam(trnsm,beaminit,acbeam=acbeam,help=help)
;
; INPUTS:
;      Roll:    fltarr(m), AC roll in radians 
;      Pitch:   fltarr(m), AC pitch in radians 
;      Heading: fltarr(m), AC heading in radians
;
;      Instead of roll,pitch,heading the AC to Earth coordinate systems 
;      transformation matrix can be given.
;
;      Trnsm:   fltarr(3,3,m)
;
;      Beaminit: Initialize antenna beam position
;                Default beam pointing angles:
;                 0          - side-looking KingAir, KA, (main) 12"antenna
;                              default vector: [-0.01221,0.999898,0.00317781]
;                 1          - up-looking KA (main) 12" antenna
;                              default vector: [ 0.052336, 0.,-0.99862951]
;                 2          - KA side-fore looking antenna (~35 deg)
;                              default vector: [0.591778,0.805532,0.024110]
;                 3          - KA nadir 18" antenna
;                              default vector: [-0.05547,0.0019240,0.998436]
;                 4          - KA down-fore looking 15" antenna (~30 deg)
;                              default vector: [0.4408,0.00613254,0.897537]
;                 5          - NCAR C130 nadir 12" antenna (7/16/01)
;                              default vector:[-0.0426, 0., 0.999092]
;                 6          - NCAR C130 down-aft looking 12" antenna(7/16/01)
;                              default vector:[-0.5745, 0., 0.818153]
;                 7          - NRC CV580 side looking 12" antenna
;                              default vector: [-0.004593,0.999717,0.023354]
;                 8          - NRC CV580 nadir 12" antenna
;                              default vector:[-0.0448,0.,0.999]
;                 fltarr(m)  - side/up antenna for KA determined by the  
;                              KA mirror (0-side, 1-up), m > 3
;
;                Antenna beam pointing angles:
;                 fltarr(3)  - beam unit vector in AC coordinate system
;                              [AC along axis, right wing, down]
;                 fltarr(3,m)- beam unit vectors for all data points 
;
; KEYWORD PARAMETERS:
;      DEG:      when set the attitude angles are given in degrees
;      ACBEAM:   when set wcrbeam returns beam coordinates in AC coord. system
;                (identical to beaminit input fltarr(3,m))
;      HELP:     show this text
;
; SIDE EFFECTS:
;       The function returns an empty string if the usage is incorrect
;
; NOTES:
;       
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov, August 2000
;      S.H., Oct 00 -- allowed trnsm as an input
;      S.H., Oct 01 -- added new antenna orientations
;      S.H., Jul 02 -- added KA down-antenna coordinates
;      S.H., Feb 03 -- changed kaaggat to ac2gt
;      S.H., Jul 03 -- fixed bugs with passing deg keyword
;      S.H., Oct 03 -- added WCR beam angles for NRC Convair
;      S.H., Nov 03 -- changes in the help
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrbeam, roll,pitch,heading,beaminit, deg=deg,acbeam=acbeam,help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrbeam'
  return,''
endif

if not(n_params() eq 2 or n_params() eq 4) then begin
   message,'Usage: res=wcrbeam(roll,pitch,heading,beaminit, deg=deg,acbeam=acbeam,help=help)',/info
   print,'                  res=wcrbeam(trnsm,beaminit)'
   return,''
endif

; Default UWKA or NCAR C130 radar beamvectors

  upbeam   = [ 0.052336, 0.,        -0.9986295 ] ; Dave's measure from 1996
; sidebeam = [-0.0169289,0.99985671, 0.        ] ; Dave's old measure
; sidebeam = [-0.01128,  0.999933,   0.002743  ] ; Dave's new measure.,9/18/2001
  sidebeam = [-0.0122113,0.999898,   0.00317781] ; Sam's measure.,Jul-Aug 2003
; side2beam= [ 0.581156, 0.812945,   0.0371314 ] ; Dave's measurement, 9/18/2001
  side2beam= [ 0.591778, 0.805532,   0.024110  ] ; Sam's measure.,Jul-Aug 2003
; downbeam = [-0.05624,  0.0026,     0.9984    ] ; Dave's measurement, 5/26/2002
  downbeam = [-0.0554711,0.0019240,  0.998436  ] ; Sam's measure.,Jul-Aug 2003
; down2beam= [ 0.439,    0.005,      0.898     ] ; Dave;s measurement, 5/26/2002
  down2beam= [ 0.440805, 0.00613254, 0.897537  ] ; Sam's measure.,Jul-Aug 2003
  c130down = [-0.0426,   0.,         0.999092  ] ; Dave's measurement, Jan 2002   
  c130down2= [-0.5745,   0.,         0.818153  ] ; Dave's measurement, Jan 2002  
  cv580side= [-0.004593, 0.99971682, 0.023354  ] ;   
  cv580down= [-0.0448,   0.,         0.999     ] ;   

; Calculate transformation matrix if not given

  if n_params() eq 4 then wcrT=ac2gt(roll,pitch,heading,deg=deg) $
  else begin wcrT=roll & beaminit=pitch & end

; Extract the number of profiles

  n=(size(roll))[size(roll,/n_dim)]

; Prepare the AC radar beam vector according to beaminit 

  if size(beaminit,/n_dim) eq 2 then begin
     if keyword_set(acbeam) then return,beaminit $
     else acbeam=beaminit
  endif else if n_elements(beaminit) eq 1 then begin
     if beaminit lt 2 then $
       acbeam=replicarr(sidebeam*(1-beaminit)+upbeam*beaminit,n) $
     else if beaminit eq 2 then acbeam=replicarr(side2beam,n)    $
     else if beaminit eq 3 then acbeam=replicarr(downbeam,n)     $
     else if beaminit eq 4 then acbeam=replicarr(down2beam,n)    $ 
     else if beaminit eq 5 then acbeam=replicarr(c130down,n)     $ 
     else if beaminit eq 6 then acbeam=replicarr(c130down2,n)    $ 
     else if beaminit eq 7 then acbeam=replicarr(cv580side,n)    $ 
     else if beaminit eq 8 then acbeam=replicarr(cv580down,n) 
  endif else if n_elements(beaminit) eq 3 then begin
     acbeam=replicarr(beaminit,n) 
  endif else if n_elements(beaminit) gt 3 then begin
     acbeam=sidebeam#(beaminit eq 0)+upbeam#beaminit
  endif

; Transform acbeam to ground-based

  for k=0L,n-1 do acbeam[*,k]=acbeam[*,k]#wcrT[*,*,k]

; Return radar beam in ground coordinates

  return,acbeam

end
