;$Id: wcrbeamfind.pro,v 1.2 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRBEAMFIND
;
; PURPOSE:
;       Finds unit vector (pointing angles in aircraft, AC,coordinate system) 
;       of a wcr antenna beam from wcr surface returns
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrbeamfind,beaminit,drng,dprof,status=status,timeoffset=timeoffset, $
;                   ground=ground,acncf=acncf,wcrfn=wcrfn,ppselect=ppselect, $
;                   nowcrread=nowcrread,epstop=epstop,limser=limser,$
;                   report=report,dispfun=dispfun,help=help
;
; INPUTS:
;       Beaminit:  initial guess for the beam unit vector:
;                  fltarr(3), [ac_along_axis,right_wing,down]; requires ppselect
;                         keyword to be defined or
;                  0    - side-looking KingAir, N2UW, (main) 12"antenna
;                         default guess: [0.0, 1.0, 0.0]
;                  2    - KA side-fore looking antenna (~35 deg)
;                         default guess: [0.6, 0.8, 0.0]
;                  3    - KA nadir 18" antenna
;                         default guess: [0.0, 0.0, 1.0]
;                  4    - KA down-fore looking 15" antenna (~30 deg)
;                         default guess: [0.4, 0.0, 0.9]
;                  5    - NCAR C130 nadir 12" antenna
;                         default guess: [0.0, 0.0, 1.0]
;                  6    - NCAR C130 down-aft looking 12" antenna (~36 deg)
;                         default guess: [-0.6,0.0, 0.8]
;                  7    - NRC CV580 side looking 12" antenna
;                         default guess: [0.0, 1.0, 0.0]
;                  8    - NRC CV580 nadir 12" antenna
;                         default guess :[0.0, 0.0, 1.0]
;                
;       Drng:      [first,last] range of wcr data range gates to be used
;                  default is all range gates
;       Dprof:     [first,last] wcr data profile range to be used; default is 
;                  all  profiles; to input Dprof, Drng must exists (use 0 for default)
;           
; KEYWORD PARAMETERS:
;       ACNCF:     input,string; AC 25 HZ netcdf data file name or netcdf ID;
;                  If not defined the program uses AC data loaded in a previous
;                  call to this routine
;       DISPFUN:   when set displays obj. function and parameters evaluation 
;                  during the minimization search                  
;       EPSTOP:    input,float; specify the constained minimization procedure
;                  (see constrained_min) convergence criterium (fractional
;                  change of obj. function less than EPSTOP for 2 consecutive
;                  iterations); default 1.e-3
;       GROUND:    input, float; surface return threshold in dBm;
;                  Default (not defined) is no thresholding; surface is
;                  determined as the max returnf for every profile 
;       LIMSER:    input, integer, maximum number of 1-d minimization searches
;                  (see constrained_min); default is 20
;       NOWCRREAD: when set does not read wcr raw data and use already loaded 
;                  wcr  data. If drng and dprof are defined it uses the subset 
;                  of the already loaded data.
;       PPSELECT:  input, string; type of pulse pair to be used: 'HH','VV'
;                  for the given beaminit vector (ignored if beaminit 0 to 6)
;       REPORT:    input, string,path/name for the CONSTRAINED_MIN report file.
;                  If the specified file does not exist, it will be created; if
;                  existing it will be overwritten.  Default(not defined) does
;                  not create a report file.
;       STATUS:    output, integer, solution status (see constrained_min, inform) 
;                  0 - optimal solution was found
;       TIMEOFFSET:input/output, float; timeoffset between WCR and AC clocks
;                  (wcrtime+timeoffset=actime);
;                  input: 0 or not defined - clocks are synched (default)
;                        -1.0 to 1.0       - the value is used for time synch
;                        +-9.xx            - the routine uses +-0.xx as initial
;                                            guess and makes an attempt to
;                                            determine the optimal time offset
;                  output: optimal timeoffset in sec (only for +-9.xx input)
;       WCRFN:     input string; WCR raw data file name.  If not given the
;                  needed file is assumed open. The [drng,dprof] data is 
;                  loaded unless NOWCRREAD keyword is set.
;       HELP:      shows this text
;
; OUTPUTS:
;       Beaminit:  fltarr(3), [ac_along_axis,right_wing,down], beam unit vector
;
; SIDE EFFECTS:
;
; PROCEDURE:
;
; NOTES:
;
; EXAMPLES:
;       1) Side beam, IHOP02, May 16
;       .run commonwcrset
;       openwcrf,'/sova4/ihop02/may16/W2002-05-16-11-57-50'
;       readwcrdata,120307,120348,/timein       
;       ncf='/net/owl/data/kingair_data/ihop02/processed/20020516.c25.nc'
;       beami=0 & beam=beami 
;       wcrbeamfind,beam,timeoff=0.1,status=status,ground=-20.,/nowcr, $
;          report='IHOP020516.sidebeam.txt',acncf=ncf,/dispfun
;
;       2) Side beam, IHOP, June 19
;       .run commonwcrset
;       openwcrf,'/sova4/ihop02/jun19/W2002-06-19-20-36-21'
;       readwcrdata,203600,203730,/timein
;       ncf='/net/owl/data/kingair_data/ihop02/processed/20020619a.c25.nc'
;       beami=0 & beam=beami 
;       wcrbeamfind,beam,timeoff=9.00,status=status,ground=-25.,/nowcr, $
;          report='IHOP020619.sidebeam.txt',acncf=ncf
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, June 2003
;       S.H., Oct 2003 - modified ground keyword
;       S.H., Nov 2003 - changes in the help text
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function beamoptfun,x
; Calculates the optimization objective function and one constrained
; function. The objective function is (wcrdopvel-wcracvel)^2, where
; wcrdopvel is wcr measured Doppler from surface returns and wcracvel
; is the calculated aircraft motion into the wcr beam. The constrained
; function is beam[0]^2+beam[1]^2+beam[2]^2 = 1. CONSTRAINED_MIN is used
; to search for a minimum in the objective function given some constrained
; for x.  x=[beam[0],beam[1],beam[2]] or x=[beam[0],beam[1],beam[2],timeoffset]
; The constrained for x are the following: -1<x[0:2]<1, abs(x[3])<maxtimeoffset

  common wcrbeamfind_temp,acdata,wcrdv,wcrt,timeoff,maxtimeoff,threshind, $
                          maxv,dispf

  on_error,2

  dt    = 0.04       ; time step in seconds based on 25 Hz data 
 
  if n_elements(x) eq 4 then timehack=x[3] else timehack=timeoff

; Prepare objective function

  ; calculate aircraft motion into the wcr beam    
  wcracv=wcracvelm(acdata,beaminit=x[0:2])

  ; try to secure wcr time to be a subset of ac time
  n=n_elements(wcrt)
  ind1=(where(acdata.time gt wcrt[0]))[0]
  ind2=(where(acdata.time ge wcrt[n-1]))[0]
  junk=2L*long(maxtimeoff/dt)+1L
  ind1=(ind1-junk)>0
  ind2=(ind2+junk)<n_elements(acdata.time)-1

  ; calculate corrected Doppler (wcrdopvel-wcracvel)
  wcrdvc=wcrdopcor(wcrdv,wcrt,wcracv[ind1:ind2],acdata.time[ind1:ind2],$
                   maxv=maxv,timehack=timehack)

  ; evaluate the objective and the constrain functions
  g1=double(total(wcrdvc[threshind]^2)/(n_elements(threshind)))
  g2=double(x[0]^2+x[1]^2+x[2]^2) 

  if dispf then print,[g1,g2,!radeg*[atan(x[1]/(x[0]+1.e-7)), $
                       atan(x[2]/sqrt(x[0]^2+x[1]^2+1.e-7))],x]
;  if dispf then print,[g1,g2,x]

  return,[g1,g2]
end

pro wcrbeamfind,beaminit,drng,dprof,status=status,timeoffset=timeoffset, $
                ground=ground,acncf=acncf,wcrfn=wcrfn,ppselect=ppselect, $
                nowcrread=nowcrread,epstop=epstop,limser=limser,report=report,$
                dispfun=dispfun,help=help

  @commonwcrinclude
  common wcrbeamfind_temp,acdata,wcrdv,wcrt,timeoff,maxtimeoff,threshind, $
                          maxv,dispf

  on_error,2

  ; Fetch the help if wanted

  if keyword_set(help) then begin
    doc_library,'wcrbeamfind'
    return
  endif

  ; Check in/out params and some inhouse work

  if n_params() lt 1 then begin
    message,"Usage: wcrfindbeam,beaminit,drng,dprof,status=status, $",/info
    print,"               timeoffset=timeoffset,ground=ground,acncf=acncf,$"
    print,"               wcrfn=wcrfn,ppselect=ppselect,nowcrread=nowcrread,$"
    print,"               epstop=epstop,limser=limser,dispfun=dispfun,$"
    print,"               report=report,help=help"
    return
  endif

  if n_elements(beaminit) eq 3 then begin
    if n_elements(ppselect) eq 0 then $
      message,'ppselect must be defined when beaminit is a vector'
    beam=beaminit
  endif else begin
    case beaminit of
      0: begin & beam=[0.0, 1.0, 0.0] & ppselect='HH' & end
      2: begin & beam=[0.6, 0.8, 0.0] & ppselect='VV' & end
      3: begin & beam=[0.0, 0.0, 1.0] & ppselect='VV' & end
      4: begin & beam=[0.4, 0.0, 0.9] & ppselect='HH' & end
      5: begin & beam=[0.0, 0.0, 1.0] & ppselect='HH' & end
      6: begin & beam=[-0.6,0.0, 0.8] & ppselect='VV' & end
      7: begin & beam=[0.0, 1.0, 0.0] & ppselect='HH' & end
      8: begin & beam=[0.0, 0.0, 1.0] & ppselect='VV' & end
      else: message,'Invalid input for beaminit'
    endcase
  endelse

  maxtimeoff=1.0                     ; timeoffset constrain
  nstop=2                            ; max number of itterations that satisfy
                                     ; EPSTOP convergence criterium
  gcomp='beamoptfun'                 ; objective function routine name
  nobj=0                             ; objective function index
  gbnd=[[0,1],[0,1]]                 ; constrain function bounds
  x   =beam                          ; initial guess for the constrained min
  xbnd=[[-1.,-1.,-1.,-maxtimeoff], $
        [ 1., 1., 1., maxtimeoff]]   ; optimization variable bounds

  if n_elements(timeoffset) eq 0 then begin
    timeoff=0.
    xbnd=xbnd[0:2,*]
  endif else begin
    if abs(timeoffset) ge 9 then begin
      timeoff=timeoffset mod 9
      if timeoff gt maxtimeoff then $
        message,'timeoffset exceeds '+num2str(maxtimeoff)+'s'
      x=[beam,timeoff]
    endif else begin
      timeoff=timeoffset
      xbnd=xbnd[0:2,*]
    endelse
  endelse

  if n_elements(ground) eq 1 then grnd=10.^(-0.1*abs(ground)) else grnd=0

  if keyword_set(dispfun) then dispf=1 else dispf=0
  if n_elements(epstop) eq 0 then epstop=1.e-3
  if n_elements(limser) eq 0 then limser=20

  ; Read/handle wcr raw data according to drng and dprof

  wcrreadf=1B
  if keyword_set(nowcrread) then begin  ; <--- wcr data is already available
    if timemode eq 3 then wcrtime=double(basetime)+time $
    else if timemode eq 4 then wcrtime=double(basetime-long(time[0]))+time $
    else message,'timemode must be 3 or 4'

    if n_elements(dprof) eq 1 then begin
      wcrt=wcrtime[dprof:*]
      if ppselect eq 'HH' then refl=hh[*,dprof:*] else refl=vv[*,dprof:*]
    endif else if n_elements(dprof) eq 2 then begin
      wcrt=wcrtime[dprof[0]:dprof[1]]
      if ppselect eq 'HH' then refl=hh[*,dprof[0]:dprof[1]] $
      else refl=vv[*,dprof[0]:dprof[1]]
    endif else begin
      wcrt=wcrtime
      if ppselect eq 'HH' then refl=hh else refl=vv
    endelse
  endif else begin                      ; <--- Read the raw wcr data
    junk=1
    if n_elements(wcrfn) eq 0 then junk=0 $
    else if n_elements(wcrfile) eq 1 then $
         if wcrfile eq strmid(wcrfn,19,/reverse_offset) then junk=0
    if junk then begin
      print,'Openning wcr raw file ...'
      openwcrf,wcrfn
    endif
    wcrreadf=0B
    print,'Reading wcr raw data ...'
    if n_elements(dprof) eq 0 then $
      readwcrdata,rawtime=wcrtime,/nocross,/noleak $
    else if n_elements(dprof) eq 1 then $
      readwcrdata,dprof,rawtime=wcrtime,/nocross,/noleak $
    else if n_elements(dprof) eq 2 then $
      readwcrdata,dprof[0],dprof[1],rawtime=wcrtime,/nocross,/noleak
    wcrt=wcrtime
    if ppselect eq 'HH' then refl=hh else refl=vv
  endelse

  ; Calculate uncorrected Doppler velocity for the appropriate antenna beam;
  ; If more than one pulse pair is available use the one with the bigger prf.

  wcrdv=wcrdopvel(ppselect=ppselect,maxv=maxv)

  if size(wcrdv,/n_dim) eq 3 then begin
    if n_elements(wcrdv[0,0,*]) eq 2 then $
      junk=max([max(wcrdv[*,*,0]),max(wcrdv[*,*,1])],ind1) $
    else if n_elements(wcrdv[0,0,*]) eq 3 then $
      junk=max([max(wcrdv[*,*,0]),max(wcrdv[*,*,1]),max(wcrdv[*,*,2])],ind1)
    wcrdv=wcrdv[*,*,ind1]
  endif

  ; Get the rng/prof segment of the Doppler and find surface return indices
  ; according to grnd
 
  if wcrreadf then begin
    if n_elements(dprof) eq 1 then wcrdv=wcrdv[*,dprof:*] $
    else if n_elements(dprof) eq 2 then wcrdv=wcrdv[*,dprof[0]:dprof[1]]
  endif

  if n_elements(drng) eq 1 then begin
    wcrdv=wcrdv[drng:*,*]
    refl=refl[drng:*,*]
  endif else if n_elements(drng) eq 2 then begin
    wcrdv=wcrdv[drng[0]:drng[1],*]
    refl=refl[drng[0]:drng[1],*]
  endif else begin
    wcrdv=wcrdv[3:*,*]
    refl=refl[3:*,*]
  endelse

  if grnd eq 0 then junk=max(refl,threshind,dim=1) $
  else threshind=where(refl gt grnd)
  
  nind=n_elements(threshind)
  print, 'Found '+num2str(nind)+' data points from the surface.'
  if nind lt 10 then begin
    message,'Too few surface points.',/info
    return
  endif

  ; Load AC data if needed
 
  if n_elements(acncf) ne 0 then begin
    if size(acncf,/type) eq 7 then ncid=ncdf_open(acncf) else ncid=acncf
    print,'Reading King Air data ...'
    acdata=acloadins(ncid,/close)
  endif

  ; Do the constrained optimization to find optimal beam angles and timeoffset

  if dispf then print,'        ',stradj(['ObjFun','ConstrFun', $
;                      'AC_x','AC_y','-AC_z','Timeoff'],len=15)
    'Az[deg]','El[deg]','AC_x','AC_y','-AC_z','Timeoff'],len=15)

  constrained_min,x,xbnd,gbnd,nobj,gcomp,status,report=report, $
    epstop=epstop,nstop=nstop,limser=limser

  beaminit=x[0:2]
  if n_elements(x) eq 4 then timeoffset=x[3]

end
