;$Id: wcrcalcoeff.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+ 
; NAME:
;      WCRCALCOEFF
;
; PURPOSE:
;      Calculate HH calibration coeff. from corner reflector measurements.
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;      WCRCALCOEFF,rCR,PcrdB,ccoeff,hhnoise_sigmadB,ZedBmin=ZedBmin
;      WCRCALCOEFF [,ccoeff]  - prints out min detectable signal
;
; INPUTS:
;      rCR:     scalar/array of distances to the corner reflector in meters
;      PcrdB:   max received power from CR for every distance in dBm
;      ccoeff:  scalar, cal. constant in dBZ (default, 31)
;
; KEYWORD PARAMETERS:
;      hhnoise_sigmadB: averaged sigma for the noise in dB
;      ZedBmin:         min detectable signal in dBZ for 0.06 to 15 km and 
;                       2*sigma detection threshold
;      HELP:            display this text
;
; OUTPUTS:
;      ccoeff:  scalar/array of cal. constant(s) in dBZ
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov, May 1996
;      S.H., Aug 99 - updated help text and other minor changes
;      S.H., Jan 00 - added help keyword
;      S.H., Feb 01 - changed routine name from calcoeff wcrcalcoeff  
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrcalcoeff,rCR,PcrdB,ccoeff,hhnoise_sigmadB=hhnoise_sigmadB,ZedBmin=ZedBmin

if keyword_set(help) then begin
  doc_library,'wcrcalcoeff'
  return
endif

if n_params() le 1 then goto, MINSIG

; ************************* Derivations *************************************
;
; PtCdB   - cal. constant from corner reflector @ 1 km in dBm, output
; avecal - averaged calibration power from the cr calib. measurements, input
; calpwr - scalar or array of cal. power during the flight, input 
;
; Weather radar equation
;
; Prw=(Pt*g^2*lambda^2*l(r)^2*lr*pi*theta^2)/ $ 
;     ((4*pi)^3*r^2*8*log(2))*(c*tau)/2*eta(r))
;
;
; Point target radar equation
;
; Prp=(Pt*gp^2*lambda^2*sigma)/((4*pi)^3*r^4) = (Pt*C)*sigma/((4*pi)^3*r^4)
;
;
; Calculate radar constant, C, from a point target, corner reflector, CR
;
; l^2*lr    - losses,attenuation 
; g         - antenna gain
; theta     - antenna beamwidth in rad
; lambda    - RF wave length in m;  ; 0.00315836494 m in free space
; c         - speed of light in vacuum [m/s]
; m         - refractive index for air (eq. 2.19, Doviak & Zrnic)
; rf        - microwave frequency in Hz
; tau       - pulse width in s

  tau    = 200e-09
  rf     = 94.92e09
  m      = 1.0003
  c      = 2.99792e08/m
  lambda = c/rf           
  theta  = 0.7*!DTOR

; rCR       - range to the corner reflector in meters
; PcrdB     - received power from the CR at range rCR in dB
; Pcr1dB    - PcrdB recalculated for 1 km range
; sCR       - sigma for trihedral corner reflector 
;             with base a=0.051m and side l=0.036 m
; PtC       - total radar constant calculated for 1 km range

  sCR=4*!pi*(0.036^4)/(3*lambda^2)                       ; 0.705723
  Pcr1dB= 40.*alog10(rCR)   + PcrdB
  PtCdB = 30.*alog10(4*!pi) + Pcr1dB - 10.*alog10(sCR)


; From Weather radar equation

  ; Pr         - received power from the target
  ; r          - range to the target

  ; eta = Pr*(512.*!pi^2*alog(2)*2.)/(theta^2*c*tau)*r^2/PtC

  ; kw = |(m^2-1)/(m^2+2)|^2, m=n-jnk - complex refractive index of water
    kw = 0.711

  ; Ze   = (1.e03*lambda)^4/(kw*!pi^5)*eta
  ; ZedB = 10.*alog10(Ze)

  ccoeff= 10.*alog10((512.*!pi^2*alog(2)*2.)/(theta^2*c*tau)) + $
          10.*alog10((1.e03*lambda)^4/(kw*!pi^5))
  ccoeff=ccoeff+60.                               ; 60 dB for eta in [mm^2/m^3] 
  ccoeff=ccoeff+60.                               ; 60 dB for r given in km
  ccoeff=ccoeff-PtCdB

  print,"**********************************************************************"
  print,"Cal coeff: ",ccoeff,"

; *****************************************************************************

MINSIG:  

  if n_params() eq 1 then aveccoeff=rcr else aveccoeff=48.  ; ave H cal. in dB

  if n_elements(hhnoise_sigmadB) eq 0 then hhnoise_sigmadB=-78.
  PrdB=hhnoise_sigmadB

; ZedB=ccoeff+10.*alog10(Pr)+20.*alog10(r)  ; [mm^6/m^3] and r in [km]

; Min detectable Z for 1*sigma noise threshold

;  if n_elements(avelacpwrdB) gt 0 and n_params() eq 4 then begin 
;  if n_params() eq 4 then begin 
;  if n_elements(ccoeff) gt 1 then aveccoeff=mean(ccoeff) else aveccoeff=ccoeff
;  Zedbmin=aveccoeff+avecalpwrdB+PrdB-calpwrdb
  
; [km]  0.1    0.5  1.0    1.5     2.0     2.5    3.0     3.5     4.0     4.5
; rngdB -20.  -6.0  0.0    3.5     6.0     7.96   9.5    10.9    12.0    13.1

  rng=  [75., 100.,200.,500.,1000.,2000.,3000.,4500.,7500.,15000.]  ; [m]
; rngdB=[35.6,40.0,46.0,54.0,60.0, 66.0, 69.5, 73.1, 77.5, 83.5 ]

  rngdB=20.*alog10(rng) 
  
  rngdB=rngdB-60   ; reference is 1 km 

  Zedbmin=aveccoeff+PrdB+rngdB

; endif

  print,""
print,"*********************************************************************************************"
  print,"Coeff (H ch.):",aveccoeff,", Threshold:",PrdB
  print,'Range [m]  : ',num2str(rng,fr=1,len=7)
  print,'Zemin [dBZ]: ',num2str(ZedBmin,fr=1,len=7)
  print,"*********************************************************************************************"
  
return

; Plot minimum signal curve

  rng=20.+findgen(400)*5.
  rngdB=20.*alog10(rng)-60
  Zedbmin=aveccoeff+PrdB+rngdB

  ; setxpswin,99,1
  plot,rng,ZedBmin,ytitle='dBZ',xtitle='Range [m]',$
       title='Minimum Detectable Signal in dBZ.  Cal.Const.= '+num2str(aveccoeff)+' Threshold= '+num2str(PrdB)+' (H ch)'

  ;xyouts,0.2,-0.1,"Wyoming King Air Cloud Radar -- ARM IOP, 16 Sep 1997, Ponca City",/norm
  grid
  datestamp

return

rng=50.+findgen(60)*5.
rngdB=20.*alog10(rng)-60
Zmin=53.-75+rngdB
setxpswin,99,1,xysize=[4.,3.]
plot,rng,zmin,title='Close Range Minimum Signal',ytitle='Sensitivity [dBZ]',xtitle='Range [m]'
plots,[60,60],[-48,-37],lin=2  
plots,[120,120],[-48,-37],lin=2
xyouts,60,-49,'B'
xyouts,120,-49,'A'
grid
resetps2x


end
