;$Id: wcrcalcoref.pro,v 1.9 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;	WCRCALCOREF
;
; PURPOSE:
;
;	Calibrate co-reflectivity (hh,vv,abs(pp)). 
;	Allows additional pre-calibration profile/range gate averaging.
;
;    ATTN: This routine does not work for multi-pulse reflectivity data which
;          occur for PPMAG modes if PPMNOAV=1 is used (same pol/beam pulses 
;          are not averaged). The work around is to use the input params
;          instead of the data in the wcrdata common block 
;
; CATEGORY:
;
;	wcrtools
;
; CALLING SEQUENCE:
;
;	coref=wcrcalcoref(thresh,calcoef,wcrstr=wcrstr)
;
;    or
;
;	coref=wcrcalcoref(thresh,calcoef,refl,conoise,rrng)
;
; INPUT:
;
;    Thresh:  float, detection threshold value above mean noise (in dB) or,
;             fltarr(2), thresh[0]+10*alog(thresh[1]) is used as a threshold
;               where thresh[0] is the threshold in dB and thresh[1]
;               is a threshold factor (e.g.,[-70,2] -> -67 dB thresh)
;               [-999,factor] will cause using 10.*alog10(factor*noise_sigma),
;               where noise_sigma is evaluated from the noise for nswin
;               profiles.(default threshold is one sigma of the noise,
;               i.e., thresh=-999 or thresh=[-999,1.])
;
;    Calcoef: float, calibration constant in dB
;
;    Refl:    fltarr(nrg,nprof),received power in mW
;                
;    Conoise: fltarr(nprof), co-channel noise vector in mW
;
;    Rrng:    fltarr(nrg), radar range vector in meters  
;
; KEYWORD PARAMETERS:
;
;    ATT      - float, two-way attenuation coefficient (dB/km)
;    MNOISE   - output,fltarr(n,2); mean noise subtracted from the received
;                 co-pol power and threshold level(in dBZ @ 1km); If 
;                 thresh=[-999,1] mnoise[*,1] is the noise standard deviation 
;                 in dBZ @ 1km (see notes)
;    NAN      - when set (1), force NaN to be assigned to all values 
;                 below the threshold. Any value different than 1 will 
;                 be used as missing value in dBZ.
;                 Maximum valid value for nan is -150.
;               Default(keyword not set) is -32767.
;    NAV      - short integer, number of profiles to be avaraged
;                 intarr(2), number of profiles and range gates to average 
;                 default is no averaging               
;    NOTHRESH - when set, only mean noise is subtracted (no thresholding) 
;    NSWIN    - input,long, smoothing noise-window size(default 99); should be 
;                 an odd number(1 is added for evens); it is recommended not to 
;                 exceed 10% of the processed number of profiles;  values less
;                 than 3 and greater than 50% of all profiles cause the use of
;                 constant mean and stdev of the noise data
;    SILENT   - when set suppress info print
;    WCRSTR   - when set uses wcr common block for data (refl,noise,rng)
;                 input, string, 'hh' (default) or 'vv'
;    HELP     - Show this text
;
; COMMON BLOCKS:
;     wcrdata
;
; NOTES:
;     For PPMAG modes calibration when received power threshold is based on the
;     standard deviation of the measured noise, noiseflag and txsequence are 
;     used to determine how the noise data were generated by readwcrdata 
;     (ppmnoav keyword).  For PPMAG modes the noise can be assambled in
;     different ways not necessarily matching the noise in the loaded H/V power
;     which is always averaged (in readwcrdata) of all H/V pulses in a burst.
;     In case of ppmnoav=2,3,4; the loaded hhnoise and/or vnoise are/is using 
;     a single pulse noise measurements and therefore its standard deviation
;     may be higher than the noise contribution into the averaged received power
;     (e.g., if txstrpol=['H','H','V','V']) and ppmnoav=3 then std(hhnoise) would
;      be 1.4 dB higher than the std of the noise contribution in hh)
;     In these cases the threshold is corrected by the needed factor.
;
;     The above corection is performed only when wcrstr is defined
;
; EXAMPLES:
;     hhc=wcrcalcoref([-999,2.],hhcal,hh,hhnoise,vvnoise,rrng)
;     vvc=wcrcalcoref([-999,2.],vvcal,vv,vvnoise,hhnoise,rrng,nav=5)
;     hhc=wcrcalcoref([-999,2.],hhcal,wcrstr='hh')
;     hhc=wcrcalcoref([-999,2.],hhcal,wcrstr='hh',nswin=199) 
;     hhc=wcrcalcoref(-75.,hhcal,wcrstr='hh'); threshold -75dB above smoothed noise
;
;-
; MODIFICATION HISTORY:
;     Written by:  Samuel Haimov, March 1998
;     S.H., Aug 1999 -- modified noise subtraction options
;     S.H., Jan 2000 -- added help keyword  
;     S.H., Apr 2000 -- modified nswin keyword (added const. noise level)
;     S.H., Feb 2001 -- changed routine name from calcoref to wcrcalcoref
;     S.H., Sep 2001 -- minor fixes in the help text
;     S.H., Oct 2001 -- checks for the size of the noise window to be less
;                       than the number of (output) profiles
;     S.H., Feb 2002 -- added mnoise keyword, changed slightly the beginning
;                       and end points of the smoothed noise, modified nswin,
;                       modified NAN keyword
;     S.H., Mar 2002 -- added NOTHRESH keyword
;     S.H., Aug 2002 -- fixed noisesample > 1; use first noise sample
;     S.H., Nov 2002 -- fixed bug in making nsw an odd number
;     S.H., Feb 2003 -- few changes in the help text
;     S.H., Nov 2003 -- change the calculation of the threshold (take into
;                       account ppmnoav=2,3,4); removed string input for
;                       calcoeff
;     S.H., Dec 2003 -- major revision of nswin functionality
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrcalcoref,thresh,calcoef,refl,conoise,rrng,nav=nav,$
                     mnoise=mnoise,nswin=nswin,nan=nan,att=att,wcrstr=wcrstr,$
                     nothresh=nothresh,silent=silent,help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrcalcoref'
  return,''
endif

; Initilize and do some checking

  if n_elements(nan) eq 0 then novalue=-32767. $
  else begin
    if nan eq 1 then novalue=!values.f_nan $
    else  if finite(nan) then novalue=(-abs(nan))<(-150.) $
    else novalue=!values.f_nan
  endelse

  if n_elements(nswin) eq 0 then nswin=99

  if (n_elements(thresh) eq 1) then thresh=[thresh, -999.]

  if n_elements(nav) eq 0 then begin
    navp=1 & navr=1
  endif else if n_elements(nav) eq 1 then begin
    navr=1 & navp=nav[0]
  endif else begin 
    navp=nav[0] & navr=nav[1]
  endelse

  threshfac=1.                      ; scale factor for the noise thresholding
  nosignal=1.e-15                   ; level after thresholding 
  nosignaldB = novalue              ; missing value

  if n_elements(wcrstr) ne 0 then begin
     if (n_params() lt 2) then begin
        message,'Usage: coref=wcrcalcoref(thresh,calcoef,wcrstr=wcrstr)',/info
        return,-1
     endif
     if string(wcrstr) eq 'vv' then begin
       refl=vv
       conoise=vvnoise
       coch=polv
     endif else begin
       refl=hh
       conoise=hhnoise
       coch=polh
     endelse
     rrng=radrng
     noiseflg=noiseflag
  endif else begin
     if (n_params() lt 5) then begin
       message,'Usage: coref=wcrcalcoref(thresh,calcoef,refl,conoise,rrng)',/info
       return,-1
     endif
     noiseflg=0
  endelse 

  nprof=n_elements(refl[0,*])       ; number of profiles
  nrg=n_elements(rrng)              ; number of range gates

; Find output number of profiles/range gates and do some housework

  np=nprof/long(navp)                        ; output profiles
  nr=nrg/long(navr)                          ; ouput range gates
  anoise=conoise(0:np*navp-1)                ; size to multiple of navp

  arefl=refl(0:nr*navr-1,0:np*navp-1)        ; size to multiple of navp
  arng=rrng(0:nr*navr-1)                     ; size to multiple of navr

  if navr gt 1 then arng=rebin(arng,nr)      ; adjust range
  if navp gt 1 then anoise=rebin(anoise,np)  ; adjust noise

  rcorr=20.*alog10(arng)-60.                 ; range correction in dB km

; Check if nsw exceed 50% of the total number of profiles.

  nsw=abs(nswin)+(nswin mod 2 eq 0)
  if nsw gt 0.5*np then begin
    nsw=0
    message,'Noise window exceeds 50% of the total output profiles.',/info
    message,'Will use constant stdev of the noise,/info
  endif 

; Average requested number of profiles and range gates

  if navp gt 1 or navr gt 1 then arefl=rebin(arefl,nr,np)

; Treshold: set to the given value or thrashfac*sigma above noise level; 
;           sigma is dynamicallycalculated over nsw window

  if (thresh[1] gt -999.) then threshfac=thresh[1]
  if (thresh[0] gt -999.) then begin
     nthresh=fltarr(np)
     nthresh=threshfac*replicate(10.^(thresh[0]/10.),np)
  endif else begin
  ;  before setting the threshold using std(noise) check if
  ;  threshfac should be adjusted (readwcrdata,ppmnoav=2,3,4)
     if n_elements(noiseflg) gt 1 then begin
       if coch eq polh and noiseflg[0] lt 10 and noiseflg[0] gt 0 then      $
	    threshfac=threshfac/sqrt(n_elements(where(txsequence eq polh)))    $
       else if coch eq polv and noiseflg[1] lt 10 and noiseflg[0] gt 0 then $
         threshfac=threshfac/sqrt(n_elements(where(txsequence eq polv)))
     endif
   ; set the threshold
     if nsw lt 3 then begin
       nthresh=replicate(std(anoise)*threshfac,np) 
     endif else begin
       nthresh=fltarr(np)
       i=np-nsw-1
       for k=0L,i do nthresh(k)=std(anoise(k:k+nsw))
       nthresh=[replicate(nthresh(0),(nsw-1)/2),nthresh(0:i),$
                replicate(nthresh(i),(nsw-1)/2+1)]*threshfac
     endelse
  endelse

  if keyword_set(silent) eq 0 then begin
     print,'------------------------------------'
     print,wcrfile+'('+num2str(prof[0]-fstblocknum)+','+ $
           num2str(prof[numrec-1]-fstblocknum)+') Calibration:'
  endif

; Smooth the noise for smooth noise correction

  if nsw lt 3 then begin
     if keyword_set(silent) eq 0 then print,'Constant mean noise correction.'
     anoise[*]=mean(anoise)
  endif else begin
     if keyword_set(silent) eq 0 then print,'Smooth noise correction.'
     nw=nsw/2
     anoise=smooth(anoise,nsw)
     anoise=[reverse(anoise(nw:nw*2-1)),anoise(nw:np-nw-1), $
             reverse(anoise(np-nw*2:np-nw-1))]
  endelse  

; Calibrate

  if keyword_set(silent) eq 0 then begin
     print,''
     print,'Calibration constant: ',num2str(calcoef[0])
     print,'Min thresh: ',num2str(10.*alog10(min(nthresh))), $
           ', Max thresh: ',num2str(10.*alog10(max(nthresh)))
     if abs(nswin) gt 500 and abs(nswin) le 1000 and thresh[0] ne -999 then begin 
        print,'Const. Noise level: ',num2str(-abs(nswin-500))+' dBm'
     endif else begin
        print,'Mean thresholod in dB: ', num2str(10.*alog10(mean(nthresh)))
        print,'Threshold st. dev.(dB): ', num2str(std(10.*alog10(nthresh)))
        print,'Threshold factor: ',num2str(threshfac,fr=2)
        print,'Noise window: ',num2str(nsw)+'/'+num2str(nswin)
        if keyword_set(nothresh) then junk1='No' else junk1='Yes'
        print,'Threshold applied: '+junk1
     endelse
     print,'------------------------------------'
     print,''
  endif

  calv=-1
  calv=fltarr(nr,np)

  calv=arefl-replicarr(anoise,nr,dim=1)

  if keyword_set(nothresh) then indthresh=where(calv lt nosignal) $
  else indthresh=where(calv lt replicarr(nthresh,nr,dim=1))

  if (indthresh[0] ne -1) then calv[indthresh]=nosignal
  calv=10.*alog10(calv)+replicarr(rcorr,np)+calcoef[0]
  if indthresh[0] ne -1 then calv[indthresh]=nosignaldB

; Correct for attenuation (this is relative correction ;-arng[0])

  if n_elements(att) ne 0 then calv=calv+replicarr((arng-arng(0))/1000.*att,np)

; Save mean noise and threshold

  mnoise=10.*[[alog10(anoise)],[alog10(nthresh)]]+calcoef[0]

return,calv
end


