;$Id: wcrcalcrref.pro,v 1.8 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;	WCRCALCRREF
;
; PURPOSE:
;
;	Calibrate cross-reflectivity  vh, hv, LDRvh, LDRhv, ZDR. 
;	Allows additional pre-calibration profile/range gate averaging.
;
;    ATTN: This routine does not work for multi-pulse reflectivity data
;          which occurs for PPMAG modes if PPMNOAV=1 is used (same pol pulses 
;          are not averaged). The work around is to use the input params
;          instead of the data in the wcrdata common block 
;
; CATEGORY:
;
;	wcrtools
;
; CALLING SEQUENCE:
;
;	crref=wcrcalcrref(thresh,calcoef,wcrstr=wcrstr)
;
;    or
;
;    crref=wcrcalcrref(thresh,calcoef,refl,noise,rrng)
;
; INPUT:
;
;    Thresh:  float, detection threshold value above mean noise (in dB) or,
;             fltarr(2), thresh[0]+10*alog(thresh[1]) is used as a threshold
;               where thresh[0] is the threshold in dB and thresh[1]
;               is a threshold factor (e.g.,[-70,2] -> -67 dB thresh)
;               [-999,factor] will cause using 10.*alog10(factor*noise_sigma),
;               where noise_sigma is evaluated from the noise for nswin
;               profiles.(default threshold is one sigma of the noise,
;               i.e., thresh=-999 or thresh=[-999,1.])
;
; 	Calcoef: fltarr(4),[crrefcal,corefcal,isodB]
;               e.g., for vh:  [vvcal,   hhcal,   vhisodB]
;                     for hv:  [hhcal,   vvcal,   hvisodB]
;                     for zdr: [hhcal,   vvcal]
;
; 	Refl:    fltarr(3*nrg,nprof) or fltarr(2*nrg,nprof), received power[mW]
;             [crref(nrg,nprof),coref1(nrg,nprof),coref2(nrg,nprof)]
;               e.g., for PPMAG  vh:  [vh,hh,vv], or hv: [hv,vv,hh] 
;                     for SPPMAG vh:  [vh,hh],    or hv: [hv,vv] 
;                     for        zdr: [hh,vv]
;                    
; 	Noise:   fltarr(2*nprof), [crnoise,conoise]
;               e.g., for vh: [vvnoise,hhnoise], for hv: [hhnoise,vvnoise]
;
; 	Rrng:    fltarr(nrg), radar range vector in meters  
;
; KEYWORD PARAMETERS:
;               
;    ATT      - float, two-way attenuation coefficient (dB/km)
;    MNOISE   - output,fltarr(n,2); mean noise subtracted from the received
;                  cross-pol power and threshold level (in dBZ @ 1km); If 
;                  thresh=[-999,1] mnoise[*,1] is the noise standard deviation
;                  in dBZ @ 1km (see notes)
;               mnoise is not returned for wcrstr='zdr'
;    NAN      - when set (1), force NaN to be assigned to all values 
;                  below the threshold. Any value different than 1 will 
;                  be used as missing value in dBZ.
;                  Maximum valid value for nan is -150.
;               Default (keyword not set)is -32767.
;    NAV      - short integer, number of profiles to be avaraged
;                  intarr(2), number of profiles and range gates to average 
;                  default is no averaging   
;    NOTHRESH - when set, only mean noise is subtracted (no thresholding) 
;    NSWIN    - input,long, smoothing noise-window size(default 99); should be 
;                 an odd number(1 is added for evens); it is recommended not to 
;                 exceed 10% of the processed number of profiles;  values less
;                 than 3 and greater than 50% of all profiles cause the use of
;                 constant mean and stdev of the noise data
;    SILENT   - when set suppress info print
;    WCRSTR   - when set uses wcr common block for data (refl,noise,rng)
;                  input, string, 'vh', 'vh','ldrhv','ldrvh','zdr'
;    HELP     - Show this text
;
; COMMON BLOCKS:
;    wcrdata
;
; NOTES:
;    For PPMAG modes calibration and reflectivity threshold based on the
;    standard deviation of the measured noise, noiseflag and txsequence are 
;    used to determine how the noise data were generated by readwcrdata 
;    (ppmnoav keyword).  For PPMAG modes the noise can be assambled in
;    different ways not necessarily matching the noise in the loaded H/V power
;    which is always averaged (in readwcrdata) of all H/V pulses in a burst.
;    In case of ppmnoav=2,3,4; the loaded hhnoise and/or vnoise are/is using 
;    a single pulse noise measurements and therefore its standard deviation
;    may be higher than the noise contribution into the averaged received power
;    (e.g., if txstrpol=['H','H','V','V']) and ppmnoav=3 then std(hhnoise) would
;     be 1.4 dB higher than the std of the noise contribution in hh)
;    In these cases the threshold is corrected by the needed factor.
;
;    The above corection is performed only when wcrstr is defined and not for
;    'ZDR'
;
; EXAMPLES:
;    vhc=wcrcalcrref([-999,2.],[vvcal,hhcal,vhiso],[vh,hh], $
;                    [vvnoise,hhnoise],rng)                      ; SPPMAG
;    hvc=wcrcalcrref([-999,2.],[hhcal,vvcal,hviso],[hv,vv,hh], $
;                    [hhnoise,vvnoise],rng)                      ; PPMAG
;    vhc=wcrcalcrref([-999,2.],[vvcal,hhcal,vhiso],nav=5,wcrstr='vh')
;    hvc=wcrcalcrref([-999,2.],[hhcal,vvcal,hviso],nav=5,wcrstr='hv')
;
;    ldrhv=wcrcalcrref([-999,2.],[hhcal,vvcal,hviso],nav=5,wcrstr='ldrhv')
;
;    zdr=wcrcalcrref(-80,[hhcal,vvcal],[hh,vv],[hhnoise,vvnoise], $
;                    rng,wcrstr='zdr')
;    zdr=wcrcalcrref([-999,2.],[hhcal,vvcal],wcrstr='zdr')
;
;-
; MODIFICATION HISTORY:
;
;     Written by:  Samuel Haimov, March 1998
;     S.H., Jun 98 -- added ldr and zdr calibrations
;     S.H., Nov 98 -- modified cross-channel calibration
;     S.H., Aug 99 -- modified noise subtraction options
;     S.H., Jan 00 -- added help keyword  
;     S.H., Apr 00 -- modified nswin keyword (added const. noise level)
;                     fixed a bug; missing parameter in wcrcalcoref calls
;     S.H., Feb 01 -- changed routine name from calcrref to wcrcalcrref 
;     S.H., Feb 02 -- added mnoise keyword, changed slightly the beginning
;                     and end points of the smoothed noise; fixed problems
;                     when data in SPPMAG mode, modified nswin, modified nan
;     S.H., Mar 02 -- added nothresh keyword
;     S.H., Aug 02 -- fixed noisesample > 1; use first noise sample
;     S.H., Nov 02 -- added nsw reduction if it exceeds 0.5*np
;     S.H., Feb 03 -- few changes in the help text 
;     S.H., Nov 03 -- change the calculation of the threshold (take into
;                     account ppmnoav=2,3,4)
;     S.H., Dec 03 -- major revision of nswin functionality
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrcalcrref,thresh,calcoef,refl,noise,rrng,nav=nav,nswin=nswin,$
                     mnoise=mnoise,nan=nan,att=att,wcrstr=wcrstr,$
                     nothresh=nothresh,silent=silent,help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrcalcrref'  
  return,''
endif

; Initilize and do some checking

  if n_elements(nan) eq 0 then novalue=-32767. $
  else begin
    if nan eq 1 then novalue=!values.f_nan $
    else  if finite(nan) then novalue=(-abs(nan))<(-150.) $
    else novalue=!values.f_nan
  endelse

  if n_elements(nswin) eq 0 then nswin=99

  if (n_elements(thresh) eq 1) then thresh=[thresh, -999.]

  if n_elements(nav) eq 0 then begin
    navp=1 & navr=1
  endif else if n_elements(nav) eq 1 then begin
    navr=1 & navp=nav[0]
  endif else begin 
    navp=nav[0] & navr=nav[1]
  endelse

  threshfac=1.                ; scale factor for the noise thresholding
  indthrash=-1                  
  nosignal=1.e-15             ; level after thresholding
  nosignaldB = novalue        ; missing value
  minsignaldB=-100.
  alphaiso=0.5                ; splitting factor for isomw (between Tx and Rx)

  if n_elements(wcrstr) ne 0 then begin
     wcrstr=string(wcrstr) 
     if (n_params() eq 0 or n_params() lt 2) then begin
        message,'Usage: crref=wcrcalcrref(thresh,[crcal,cocal,iso],[crref,coref1,coref2],[crnoise,conoise],rng)',/info
        print,'                   crref=wcrcalcrref(thresh,[crcal,cocal,iso],wcrstr=wcrstr)'
        print,"                   zdr=wcrcalcrref(thresh,[hhcal,vvcal],wcrstr='zdr')"
        return,-1
     endif 
     if wcrstr eq 'vh' or (wcrstr eq 'ldrvh' and n_params() lt 5) then begin
        if keyword_set(silent) eq 0 then $
           message,'Using wcr common block data',/info
        if alg eq pp4alg or alg eq pp6alg then refl=[vh,hh,vv] $
        else begin
          refl=[vh,hh]
          alphaiso=1
        endelse
         noise=[vvnoise,hhnoise]
         noisech=polv
     endif else if wcrstr eq 'hv' or (wcrstr eq 'ldrhv' and n_params() lt 5) then begin
        if keyword_set(silent) eq 0 then $
           message,'Using wcr common block data',/info
        if alg eq pp4alg or alg eq pp6alg then refl=[hv,vv,hh] $
        else begin
          refl=[hv,vv]
          alphaiso=1
        endelse
        noise=[hhnoise,vvnoise]
        noisech=polh
     endif else if wcrstr eq 'zdr' then begin

     ;  For ZDR finish the job here. Var. naming is a bit confusing.
        if n_params() lt 5 then begin 
           if keyword_set(silent) eq 0 then $
              message,'Using wcr common block data',/info
           coref1=wcrcalcoref(thresh,calcoef[0],wcrstr='hh',nav=nav,$
                  nswin=nswin,nan=nosignaldB,nothresh=nothresh,att=att,/silent)
           coref2=wcrcalcoref(thresh,calcoef[1],wcrstr='vv',nav=nav,$
                  nswin=nswin,nan=nosignaldB,nothresh=nothresh,att=att,/silent)
        endif else begin
           if keyword_set(silent) eq 0 then message,'Using external data',/info 
           nrg=n_elements(rrng)                    
           coref1=refl(0:nrg-1,*)                        ;  hh
           coref2=refl(nrg:2*nrg-1,*)                    ;  vv
           nprof=n_elements(noise)/2
           crnoise=noise(0:nprof-1)                      ;  hhnoise
           conoise=noise(nprof:2*nprof-1)                ;  vvnoise
           coref1=wcrcalcoref(thresh,calcoef[0],coref1,crnoise,rrng,$
            nav=nav,nswin=nswin,nan=nosignaldB,nothresh=nothresh,att=att,/silent)
           coref2=wcrcalcoref(thresh,calcoef[1],coref2,conoise,rrng,$
            nav=nav,nswin=nswin,nan=nosignaldB,nothresh=nothresh,att=att,/silent)
        endelse

        ; Calculate zdr and assign novalue to all points below thresh.

        indthresh=where(coref1 lt minsignaldb or coref2 lt minsignaldb)
        calv=coref1-coref2
        if indthresh(0) ne -1 then calv(indthresh)=novalue
        
        return,calv
     ;  End of ZDR calibration   

     endif else if keyword_set(silent) eq 0 and n_params() gt 4 then $
        message,'Using external data',/info
     rrng=radrng
     noiseflg=noiseflag
  endif else begin
     if (n_params() lt 5) then begin
       message,'Usage: crref=wcrcalcrref(thresh,[crcal,cocal,iso],[crref,coref1,coref2],[crnoise,conoise],rng)',/info
       return,-1
     endif
     noiseflg=0
     if keyword_set(silent) eq 0 then message,'Using external data',/info
  endelse 

  if n_elements(calcoef) lt 3 then message,'Incorrect CALCOEF !'

  crcalcoef=calcoef(0) 
  isodB=-abs(calcoef(2))                     ; always negative
  isomw=10.^(0.1*(isodB))                    ; isolation in mW
  ldrcal=calcoef(1)-calcoef(0)               ; gain imbalance
  ldrcalmw=10.^(0.1*ldrcal)                  ; ldr cal const in mW

  nprof=n_elements(refl[0,*])                ; number of profiles
  nrg  =n_elements(rrng)                     ; number of range gates

; Find if both corefl channels are available

  if alphaiso ne 1. and 2*nrg eq n_elements(refl[*,0]) then alphaiso=1

; Extract co-pol and cross-pol noise

  conoise=noise(0:nprof-1)
  crnoise=noise(nprof:2*nprof-1)

; Find output number of profiles/range gates and do some housework

  np=nprof/long(navp)                        ; output profiles
  nr=nrg/long(navr)                          ; ouput range gates

  crref =refl(0:nrg-1,*)                     ; cross ch to be calibrated
  coref1=refl(nrg:2*nrg-1,*)                 ; co ch.(hh or vv depend. on crref)
  acrnoise=crnoise(0:np*navp-1)              ; size to multiple of navp
  aconoise=conoise(0:np*navp-1)              ; size to multiple of navp
  acrref=crref(0:nr*navr-1,0:np*navp-1)      ; size to multiple of navp
  acoref1=coref1(0:nr*navr-1,0:np*navp-1)    ; size to multiple of navp

  if alphaiso ne 1 then begin
    coref2=refl(2*nrg:3*nrg-1,*)             ; co ch.(vv or hh depend. on crref)
    acoref2=coref2(0:nr*navr-1,0:np*navp-1)  ; size to multiple of navp
  endif

  arng=rrng(0:nr*navr-1)                     ; size to multiple of navr

  if navr gt 1 then arng=rebin(arng,nr)      ; adjust range
  rcorr=20.*alog10(arng)-60.                 ; range correction in dB km
  if navp gt 1 then begin                    ; adjust noise
     acrnoise=rebin(acrnoise,np)
     aconoise=rebin(aconoise,np)
  endif 

; Check if nsw exceed 50% of the total number of profiles.

  nsw=abs(nswin)+(nswin mod 2 eq 0)
  if nsw gt 0.5*np then begin
    nsw=0
    message,'Noise window exceeds 50% of the total output profiles.',/info
    message,'Will use constant stdev of the noise,/info
  endif 

; Average requested number of profiles and range gates

  if navp gt 1 or navr gt 1 then begin
     acrref=rebin(acrref,nr,np)
     acoref1=rebin(acoref1,nr,np)
     if alphaiso ne 1 then acoref2=rebin(acoref2,nr,np)
  endif 

; Treshold: set to thrashfac*sigma above noise level; sigma is dynamically
; calculated over nsw window

  if (thresh[1] gt -999.) then threshfac=thresh(1)
  if (thresh[0] gt -999.) then begin
     nthresh=fltarr(np)
     nthresh=threshfac*replicate(10.^(0.1*thresh[0]),np)
  endif else begin
  ;  before setting the threshold using std(noise) check if
  ;  threshfac should be adjusted (readwcrdata,ppmnoav=2,3,4)
     if n_elements(noiseflg) gt 1 then begin
       if noisech eq polh and noiseflg[0] lt 10 and noiseflg[0] gt 0 then      $
	    threshfac=threshfac/sqrt(n_elements(where(txsequence eq polh)))       $
       else if noisech eq polv and noiseflg[1] lt 10 and noiseflg[0] gt 0 then $
         threshfac=threshfac/sqrt(n_elements(where(txsequence eq polv)))
     endif
   ; set the threshold
     if nsw lt 3 then begin
        nthresh=replicate(std(acrnoise)*threshfac,np) 
     endif else begin
        nthresh=fltarr(np)
        i=np-nsw-1
        for k=0L,i do nthresh(k)=std(acrnoise(k:k+nsw))
        nthresh=[replicate(nthresh(0),(nsw-1)/2),nthresh(0:i),$
                 replicate(nthresh(i),(nsw-1)/2+1)]*threshfac
     endelse
  endelse

  if keyword_set(silent) eq 0 then begin
     print,'------------------------------------'
     print,wcrfile+'('+num2str(prof[0]-fstblocknum)+','+ $
          num2str(prof[numrec-1]-fstblocknum)+') Calibration (cross products):' 
  endif

; Smooth the noise for smooth noise correction

  if nsw lt 3 then begin
     if keyword_set(silent) eq 0 then print,'Constant mean noise correction.'
     acrnoise[*]=mean(acrnoise)
  endif else begin
     if keyword_set(silent) eq 0 then print,'Smooth noise correction.'
     nw=nsw/2
     acrnoise=smooth(acrnoise,nsw)
     acrnoise=[reverse(acrnoise(nw:nw*2-1)),acrnoise(nw:np-nw-1),$
               reverse(acrnoise(np-nw*2:np-nw-1))]
     aconoise=smooth(aconoise,nsw)
     aconoise=[reverse(aconoise(nw:nw*2-1)),aconoise(nw:np-nw-1),$
               reverse(aconoise(np-nw*2:np-nw-1))]
  endelse

; Calibrate

  if keyword_set(silent) eq 0 then begin
     print,''
     print,'Calibration constants: ',num2str(calcoef)
     print,'Min thresh: ',num2str(10.*alog10(min(nthresh))), $
           ', Max thresh: ',num2str(10.*alog10(max(nthresh)))
     if abs(nswin) gt 500 and abs(nswin) le 1000 and thresh[0] ne -999 then begin 
        print,'Const. Noise level: ',num2str(-abs(nswin-500))+' dBm'
     endif else begin
        print,'Mean thresholod in dB: ', num2str(10.*alog10(mean(nthresh)))
        print,'Threshold st. dev.(dB): ', num2str(std(10.*alog10(nthresh)))
        print,'Threshold factor: ',num2str(threshfac,fr=2)
        print,'Noise window: ',num2str(nsw)+'/'+num2str(nswin)
        if keyword_set(nothresh) then junk1='No' else junk1='Yes'
        print,'Threshold applied: '+junk1
        print,'------------------------------------'
        print,''
     endelse
  endif

  calv=-1
  calv=fltarr(nr,np)

  ; Calibrate cross refl, using two corefl to account for finite isolation
  ; during transmission and reception. alphaiso is the factor that defines
  ; out- and in- contribution from the 2-way measured isolation. If SPPMAG
  ; data ignore alphaiso and use only one corefl

  if alphaiso ne 1 then $  
    calv=acrref-replicarr(acrnoise,nr,dim=1)- $
         alphaiso*isomw*ldrcalmw*(acoref1-replicarr(aconoise,nr,dim=1))- $
         (1-alphaiso)*isomw*(acoref2-replicarr(acrnoise,nr,dim=1)) $
  else $
    calv=acrref-replicarr(acrnoise,nr,dim=1)- $
         isomw*ldrcalmw*(acoref1-replicarr(aconoise,nr,dim=1))

  ; Threshold noisy points and substitute with nosignaldB

  if keyword_set(nothresh) then $
    indthresh=where(calv lt nosignal) $
  else $
    indthresh=where(calv lt replicarr(nthresh,nr,dim=1))
  if (indthresh(0) ne -1) then calv(indthresh)=nosignal
  calv=10.*alog10(calv)+replicarr(rcorr,np)+crcalcoef
  if indthresh(0) ne -1 then calv(indthresh)=nosignaldB

; Correct for attenuation

  if n_elements(att) ne 0 then calv=calv+replicarr((arng-arng(0))/1000.*att,np)
 
; If this is LDR calibrate the co channel and calculate LDR

  if n_elements(wcrstr) ne 0 then $
  if strmid(strlowcase(wcrstr),0,3) eq 'ldr' then begin
    ; find if thresh needs to be adjusted
    thresh1=thresh  
    if thresh[0] eq -999. and thresh[1] ne -999. and n_elements(noiseflg) gt 1 then begin
      if noisech eq polh and noiseflg[1] lt 10 and noiseflg[1] gt 0 then      $
	   thresh1[1]=thresh1[1]/sqrt(n_elements(where(txsequence eq polv)))     $
      else if noisech eq polv and noiseflg[0] lt 10 and noiseflg[1] gt 0 then $
        thresh1[1]=thresh[1]/sqrt(n_elements(where(txsequence eq polh)))
    endif
    ; get the co-pol reflectivity
    coref1=wcrcalcoref(thresh1,calcoef[1],coref1,conoise,rrng,$
          nav=nav,nswin=nswin,nan=nosignaldB,nothresh=nothresh,att=att);,/silent)
    ; calculate ldr
    indthresh=where(calv lt minsignaldb or coref1 lt minsignaldb)
    calv=calv-coref1
  endif 

; Assign missing values where no signal (below threshold)

  if indthresh(0) ne -1 then calv(indthresh)=novalue

; Save mean noise and threshold

  mnoise=10.*[[alog10(acrnoise)],[alog10(nthresh)]]+crcalcoef 


return,calv
end
