;$Id: wcrcalinfo.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+ 
; NAME:
;      WCRCALINFO
;
; PURPOSE:
;      Calibration constants for WCR experiments.
;
;      This routine has been modified to work with multi-beam configs
;      after 2002. 
; 
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;      WCRCALINFO,expstr,calconst
;
; INPUT:
;      Expstr: input, string, experiment acronym
;              If EXPSTR is not given or valid, calibration info for all 
;              experiments is printed.
;              If 'NAMES' returns all allowed experiment acronims
;
; KEYWORD PARAMETERS:
;      HELP:   Show this text
;
; OUTPUTS:
;      Calconst - output, fltarr(5), [hhcal1,vvcal1,vvcal2,vvcal3,hhcal4] 
;                 for 250 ns pulse (subtract 3.5 dB for 500 ns pulse)
;                 -999 is used when no cal. constant is available
;
;                 hhcal1 - cal const for N2UW side/up antenna H-pol, 
;                           or C130 nadir antenna (July 2001)
;                 vvcal1 - cal const for N2UW side/up antenna V-pol,
;                           or C130 down-aft antenna (July 2001)
;                 vvcal2 - cal const for N2UW nadir antenna
;                 vvcal3 - cal const for N2UW side-fore antenna
;                 hhcal4 - cal const for N2UW down-fore antenna
;
;                 ATTEN: only hhcal1 and vvcal1 are available untill 2002
;
;-
; MODIFICATION HISTORY:
;     Written by:  Samuel Haimov, October 1997
;     S.H., Aug 99 - added CS99 info and H/V cal specifics
;     S.H., Jan 00 - added help keyword
;     S.H., Feb 01 - changed routine name from calinfo to wcrcalinfo
;     S.H., Sep 03 - modified calconst to include multi-antenna configgs.
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrcalinfo,expstr,calconst,help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrcalinfo'
  return
endif

acronyms=['SCMS95','CS95','WYICE97','CTD97','ARM97','TRAC98','CLARE98', $
          'CS99','WYICE00_1','WYICE00_2','DYCOMS01_1','DYCOMS01_2', $
          'HBDD01_1','HBDD01_2','CLEX01','IHOP02','HICU03']

if n_params() eq 0 then goto,Info $
else begin
   case strupcase(expstr(0)) of
     'SCMS95'    : calconst=[57.,52.]
     'CS95'      : calconst=[57.,51.7]
     'WYICE97'   : calconst=[52.7,47.]
     'CTD97'     : calconst=[52.,47.]
     'ARM97'     : calconst=[54.,49.5]
     'TRAC98'    : calconst=[45.,44.5]
     'CLARE98'   : calconst=[50.,49.5]
     'CS99'      : calconst=[33.0,-999.]
     'WYICE00_1' : calconst=[47.,46.]
     'WYICE00_2' : calconst=[47.,46.]
     'DYCOMS01_1': calconst=[49.5,52.]
     'DYCOMS01_2': calconst=[49.5,51.5]
     'HBDD01_1'  : calconst=[49.,52.3]
     'HBDD01_2'  : calconst=[49.,54.2]
     'CLEX01'    : calconst=[49.3,47.8]
     'IHOP02'    : calconst=[49.0,-999.,50.0,55.0,53.5]
     'HICU03'    : calconst=[48.5,-999.,50.5,55.5,54.5]
     'NAMES' : calconst=acronyms
   else      : goto, Info  
   endcase
   return
endelse
Info:

print,"----------------------- WCR MODIFICATION INFO --------------------------"
print,""
print,"March 2002. RF unit circulator network H and V ports are reorganized."
print,"First they are switched, meaning H-port becomes V and V-port becomes H."
print,"Thus the shortest waveguide path (minimum loss) to the 0.3m conical lens"
print,"antenna carry H-pol signals. Second.  Two mechanical waveguide switches"
print,"(S1 and S2) are installed on the new designated H and V ports of the"
print,"circulator network.  S1 is a E-plane two-port switch and is used to"
print,"connect to the 0.3m conical lens antenna (side/up pointing beam) and the"
print,"0.38m inch casegrain antenna (down forward pointing beam).  S2 is H-plane"
print,"(V-pol), 3-port switch and may connects to the 0.3m conical lens antenna"
print,"(via OMT), 0.3m cassegrain antenna (side forward pointing beam) and 0.46m"
print,"cassegrain antenna (nadir pointing beam).  From the calibration point of" 
print,"view this configuration not only has the receivers swapped for H and V"
print,"but has new waveguides and new additional antennas. Every antenna"
print,"configuration has its own set of calibration coefficients."
print,""
print,"The original (LIFD) log-detectors were modified by MITEQ, Inc. in the"
print,"spring of 2000 and in July 2000 were installed in WCR.  The slope "
print,"problem was fixed and no significant error in the slope under pulsing"
print,"mode is present. New non-linear lookup tables were measured and "
print,"installed for all IF filters (2,5,10 MHz)."  
print,""
print,"The H-detector was replaced in July 1999 and the V-detector was replaced"
print,"in February 2000.  The new detectors (MLIF) exhibit some increase in the"
print,"linearity errors (non-linearities) in pulsing mode."
print,"With the new detectors and lookup tables there is no significant change"
print,"of the slope for either of the channels and the gain imbalance is about"
print,'1.2 dB (not finilized as of May 2000)'  
print,""
print,"In August 1999 it was found that H and V detectors change their transfer"
print,"function in pulsing mode in comparison with the specified by the vendor."
print,"Thus the lookup tables used untill Aug 1999 need correction."
print,"The cal. coefficients and the lookup tables for the experiments before"
print,"Aug 99 are not corrected at this point." 
print,"" 
print,"H-lookUp table correction for H-channel and H/V gain imbalance before"
print,'1999. This correction is not related to the problem discovered in 1999' 
print,"which applies to both H and V channels."
print,""
print,"    <Gain Imbalance>: -4.5 dBm (-3.5 to -5.5, included in cal. consts.)"
print,"    Slope (w.r.t. V channel, in dB): 0.95 (corrected before calibration)"
print,""
print,"    Cross channels are not calibrated and the following assumption"
print,"    is made:  hvcal=hhcal, vhcal=vvcal and vhiso=hviso."
print,""
print,"Isolation measured from drizzle in May '97: < 17.5 dB"
print,""
print,"Further analysis(May'98) with experimental data showed that the relative"
print,"difference between VH and HV isolation is between 0.5 and 0.8 dB"
print,"and that VHiso > HViso"
print,""
print,"SCMS and CS experiments, Florida, Jul-Aug & Oregon,Aug-Sep 1995 (in dB)
print,"   hhcal = 57., vvcal = 52., hhcalpow = 4.,vvcalpow = 9."
print,"   Mengistu corrections to CS:  vvcal=51.7, vhiso=-17.7, hviso=-17.4"
print,"   Innternal calibration (hh/vvcalpow) not used"
print,""
print,"WYICE experiment, Laramie, Feb-Apr 1997 (in dB)
print,"   hhcal = 52., vvcal = 47., hhcalpow = -29.5"
print,"   Additional analysis with data showed about -5.7 dB gain imbalance"
print,"   The new cal constants (May 1998) are:"
print,"   hhcal = 52.7, vvcal = 47., vhiso = -16.2, hviso = -15.5"
print,"   Innternal calibration (hh/vvcalpow) not used"
print,""
print,"CTD experiment, Monteray, CA, Jun 1997 (in dB)"
print,"   hhcal = 52., vvcal = 47., hhcalpow = -30.,vvcalpow = -26."
print,"   vhiso = hviso = -16 dB (not checked)"
print,"   Innternal calibration (hh/vvcalpow) not used"
print,""
print,"ARM experiment, Ponca City, OK, Sep 1997 (in dB)"
print,"   hhcal = 54., vvcal = 49.5"
print,"   vhiso = hviso = -16 dB (not checked)"
print,""
print,"TRAC experiment, Paris, France, Jun 1998 (in dB) reported by Andy/Dave"
print,"   'v' and 'h' notations refer to 2 H-polarized antennas"
print,"   hhcal = 47.0,                  vvcal = 46.5"
print,"          (43.1 old Dave's value)        (42.4 -old Dave's value)"
print,"   vhiso = hviso = -50 dB (2 antennas, no OMT, not checked)"
print,""
print,"CLARE experiment, Chilbolton, UK, Oct 1998 (in dB)"
print,"   'v' and 'h' notations refer to 2 H-polarized antennas"
print,"   hhcal = 50.0, vvcal = 49.5 ; for 250 ns pulse"
print,"   hhcal = 46.5, vvcal = 46.0 ; for 500 ns pulse"
print,"   vhiso = hviso = -50 dB (2 antennas, no OMT, not checked)"
print,""
print,"CS experiment, Corvallis, OR, Aug 1999 (in dB)"
print,"   hhcal = 33.0, vvcal=-999, not calibrated ; for 250 ns pulse"
print,"   hhcal = 29.5, vvcal=-999, not calibrated ; for 500 ns pulse"
print,"   vhiso = hviso = -16 dB (not checked)"
print,"   A new H-log detector was installed; V-log detector exibits satuarion"
print,"   in pulsing mode and requires lookup table correction. It is of no"
print,"   importance for this experiment where H-pol was used."
print,""
print,"WYICE experiment, Laramie, WY, April 11-May 15, 2000 (in dB)"
print,"   hhcal = 47.0, vvcal = 46.0               ; for 250 ns pulse"
print,"   vhiso = hviso = -17 dB (not checked)"
print,"   The new H-log and V-log detectors were used."
print,"WYICE experiment, Laramie, WY, October 16- November 30, 2000 (in dB)"
print,"   hhcal = 48.7, vvcal = 48.1               ; for 250 ns pulse"
print,"   vhiso = hviso = -17 dB (not checked)"
print,"   The modified H-log and V-log detectors with non_linear lookup "
print,"   tables were used."
print,""
print,"DYCOMS experiment,San Diego, CA, 5-30 July 2001 (in dB)"
print,"   'v' and 'h' notations refer to 2 linearly polarized antennas"
print,"   DYCOMS01_1:  June 26 to July 15"
print,"    h-ch is down pointing antenna, v-ch is down-aft (44 deg)"
print,"    hhcal = 49.5, vvcal = 52.0 ; for 250 ns pulse "
print,"    hhcal = 46.0, vvcal = 48.5 ; for 500 ns pulse"
print,"   DYCOMS01_2:  July 16 to August 03"
print,"    h-ch is down pointing antenna, v-ch is down-aft (36 deg)"
print,"    hhcal = 49.5, vvcal = 51.5 ; for 250 ns pulse" 
print,"    hhcal = 46.0, vvcal = 48.0 ; for 500 ns pulse"
print,""
print,"   vhiso = hviso, better than -50 dB"
print,""
print,"CLEX-9 experiment, Laramie, Wyoming, 8 Oct- 4 Nov, 2001 (in dB)"
print,"   hhcal = 49.3, vvcal = 47.8 ; for 250 ns pulse"
print,"   vhiso = hviso, better than -16 dB"
print,""
print,"IHOP02 experiment, Oklahoma City, Oklahoma, 13 May- 25 Jun, 2002 (in dB)"
print,"   h-ch is Side/Up pointing antenna or Down-slant antenna (30 deg fore)"
print,"   v-ch is Down (nadir) pointing antenna " 
print,"   SideUp antenna: hhcal = 49.0 ;  for 250 ns pulse"
print,"   Nadir antenna:  vvcal = 50.0 ;  for 250 ns pulse"
print,"   Down-slant antenna:  not calibrated"
print,""
print,"HICU03 experiment, Laramie, Wyoming, 7 July - 31 Aug, 2003 (in dB)"
print,"   h-ch is Side/Up pointing antenna or Down-slant antenna (30 deg fore)"
print,"   v-ch is Down (nadir) pointing antenna or Side-slant antenna (34 deg fore) " 
print,"   SideUp antenna:    hhcal1 = 48.5 ;  for 250 ns pulse"
print,"   Side-fore antenna: vvcal3 = 55.0 ;  for 250 ns pulse"
print,"   Nadir antenna:     vvcal2 = 50.5 ;  for 250 ns pulse"
print,"   Down-fore antenna: hhcal4 = 54.5 ;  for 250 ns pulse"
print,""
print,"Experiment acronyms valid for CALCONST:"
print,"" 
print,acronyms
print,""
print,"-----------------------------------------------------------------------

end
