;$Id: wcrcdl2nc.pro,v 1.6 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRCDL2NC
;
; PURPOSE:
;       Generate WCR netcdf file from a prototype cdl file (ncgen utility)
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrcdl2nc,cdlfile,profrange,ncfile=ncfile,edit=edit,wcropen=wcropen, $
;                 average=average,beams=beams,indir=indir,outdir=outdir, $
;                 savecdl=savecdl,silent=silent,big_endian=bgnd,help=help
;
; INPUTS:
;       Cdlfile:   string, name of project prototype cdl file
;                  Recommended name is <project>.cdl or <project>.<ver_date>.cdl
;                  ATTN: Do not add directory path; use INDIR for dir location
;       Profrange: lonarr(2), [start,stop]; profile range to be recorded in
;                  Ncfile. Default is [0,tnumrec-1]; tnumrec: all profiles in
;                  the open raw WCR file (or the file to be opened when
;                  WCROPEN is a string or string array)
;
; KEYWORD PARAMETERS:
;       AVERAGE: input, integer/intarr(2), # of profiles/[profiles,rangegates]
;                to be averaged (default is [2,0]; 0 is same as 1)
;                Note: 2 profs are arveraged by default to take care of a
;                small DAQ calibration problem
;       BEAMS:   input,integer, active beams config (default 1).
;                Possible configurations:
;                  1,2,3,4 - single beam, side/up, down, side-fore, down-fore
;                  12,13,24 - two beams (e.g., 13 - side/up and side-fore)
;                Note: this input parameter will be modified and wcrbeams
;                variable used when the MW switches positions are recorded
;       EDIT:    input, string/strarr, what to edit in the cdlfile:
;                 'all' - perform all necessary editing (default)
;                 'dim' - edit/update netcdf dimension related lines
;                 'glb' - edit/update netcdf global attributes lines
;                 'alg' - edit/update algorithm (SPPMAG,PPMAG,FFT) related lines
;                 'gen' - no editing, cdl file is directly converted to netcdf
;       INDIR:   input,string, directory path to cdlfile (default current dir)
;
;       BIG_ENDIAN: when set, assumes that the raw WCR file data are in
;                      big endian order.  The default is LITTLE_ENDIAN.
;                      This keyword is ignored for date prior to March 2002.
;       NCFILE:  input, string, name of generated output netcdf file starting
;                  with control character $.  If the first char. of the string
;                  is not $ the name is ignored and a default name is created.
;                  If ncfile is not defined or empty string "" ncfile name is
;                  generated by the program and is an output parameter.
;                output, string; WCR.<alg>.<cdlfile>.<yyyymmdd>.<hhmmss_hhmmss>.nc,
;                  where cdlfile is truncated to its first dot; <alg> is WCR
;                  mode: SPP, PP4, PP6 or FFT; yymmdd and hhmmss_hhmmss are
;                  begin and end time of the data in the netcdf file.
;                  If EDIT='gen' then Ncfile=<cdlfile>.nc
;       OUTDIR:  input,string, directory path to ncfile (default current dir)
;       SAVECDL: input, integer, default is 0
;                  0 - generates netcdf file and removes the modified cdl file
;                  1 - saves the modified cdl file, does not generate netcdf
;                  2 - saves the modified cdl file and generate netcdf file
;                  If cdl file is saved it uses ncfile name with extension cdl
;       SILENT:  when set supress routine informational messages
;       WCROPEN: input, integer,lun for an open raw WCR file (default is 31) or
;                input, strarr(3), input parameters to open a raw WCR file:
;                  wcropen[0] - path and file name for a raw WCR file
;                  wcropen[1] - UTC time correction(as string, default -13.)
;                  wcropen[2] - average aircraft speed in m/s (default 90.)
;       HELP:    shows this text
;
; OUTPUTS:
;       Ncfile:  string, name of generated output netcdf file (if no input)
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       Write a binary netcdf file to the output directory. This routine
;       also writes a temporary (unless savecdl keyword is set) cdl file in
;       the input directory. It is the updated version of the input cdl file.
;
;       Shell executed netcdf ncgen utility is used in the conversion.
;
;       The variables of the new netcdf file are filled with the netcdf default
;       fill values
;
;       The following dimensions, variables, attributes and comments are
;       modified depneding on the value of EDIT:
;       * EDIT = 'alg' or 'all'
;         - Remove comment lines in the cdl file if any
;         - Add a comment line to acknowledge that this is a program generated
;           file and the date of generation
;         - Remove //SPP, //PPM or //FFT lines depending on the WCR algorithm
;         - Remove cross-channel noise variables (for FFT only)
;         - Remove cross-channels for multiple active beams
;         - Remove wcrbeams if only beam 1 is used
;         - Remove wcrmirror if beam 1 (side/up) is not used (beams:2,3,4,24)
;         - Make all aircraft id string consistent with Aircraft attribute
;       * EDIT = 'glb' or 'all'
;         - Update: WCRdaqalg,WCRtxseq,WCRppseq (remove for FFT),WCRpulsepacket,
;                   WCRcdlfile,WCRdaqfile,WCRpulsewidth,WCRpulseav(remove for
;                   FFT),WCRprfHz, WCRprfrange, WCRprfvelocity, WCRrangegates,
;                   WCRrangesampling, WCRrangeresolution, UTCcorrection,
;                   WCRactivebeamsconfig
;         - Remove or Update: WCRfftbins, WCRfftave, WCRfftwin, WCRfftres
;       * EDIT = 'dim' or 'all'
;         - Check and Update: profile, range, vector3, beam, bin, copp
;         - Update: WCRprofrange, WCRprofav, WCRrangeav, WCRtimeint
;
; NOTES:
;       * Before calling wcrcdl2nc the raw WCR file must be open (openwcrf)
;         or WCROPEN keyword defined
;       * Default output netcdf file name uses yyyymmdd.hhmmss_hhmmss determined
;         from loaded WCR raw data segment and global attribute UTCcorrection
;         (tcorrection). If later on in the processing UTCcorrection changes and
;         /or global attribute Timeoffsetsync is not zero the netcdf file name
;         may not represent accurately the time interval of the recoreded data.
;       * The program uses simple editing and requires certain control strings
;         in the input cdlfile ('//PPM', '//SPP','//FFT', '//*', '*//'). It also
;         searches for certain dimension, variable and attribute names to be
;         updated or removed depending on the WCR data acquisition algorithm.
;
; EXAMPLES:
;
;       1) Example using CLEX-9 experiment data
;
;       ; Set raw WCR data common blocks (not needed, loaded for convinience)
;
;       .run commonwcrset
;
;       ; Generate empty netcdf file from the prepared CLEX01.20020208.cdl
;       ; The program will generate (temporarily) according to savecdl a
;       ; WCR.<wcr_alg>.CLEX01.<yyyymmdd>.<hhmmss_hhmmss>.cdl and the output
;       ; netcdf file WCR.<wcr_alg>.CLEX01.<yyyymmdd>.<hhmmss_hhmmss>.nc
;       ; containing the updated relevant dimensions and global attributes.
;       ; The output file is ready to be filled with the processed WCR
;       ; variables using wcrwrite1nc for the opened WCR raw file and the
;       ; requested profile range.
;
;       ncfile=''
;       wcropen=['/sova4/clex01/nov01/Wpp01-11-01-14-50-06','-13.','90.']
;       wcrcdl2nc,'CLEX01.20020218.cdl',[200,500],ave=2,beams=1, $
;                  edit='all',ncfile=ncfile,indir='.',outdir='.',savecdl=2, $
;                  wcropen=wcropen,silent=silent
;
;       2) As above but first opening the raw data file
;
;       openwcrf,'/sova4/clex01/nov01/Wpp01-11-01-14-50-06',
;       wcrcdl2nc,wcropen,'CLEX01.20020218.cdl',[200,500],ave=2,ncfile=ncfile, $
;                  savecdl=2,/silent
;
;       3) As 2) but loading all raw data.  This could be useful if wcrwrite1nc
;       is called after wcrcdl2nc is done (see examples for wcrwrite1nc).
;
;       openwcrf,'/sova4/clex01/nov01/Wpp01-11-01-14-50-06' ; default lun is 31
;       readwcrdata,200,500,timemode=3,/noleak
;       wcrcdl2nc,'CLEX01.20020218.cdl',[200,500],ave=2,ncfile=ncfile, $
;                 savecdl=2,/silent
;
;       4) Generate netcdf for WCR data from MARSF
;
;       wcrcdl2nc,'marsf.cdl',indir='wcrtools/',ave=2,ncfile=ncfile, $
;                 wcropen=['W2003-03-27-17-43-25','0.']
;
;-
; MODIFICATION HISTORY:
;       Written by: Samuel Haimov, February 2002
;       SH, 04/02 - replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;       SH, 02/03 - minor modification to account for the sppmag prf change
;       SH, 04/03 - modified to work for MARSF
;       SH, 11/03 - allowed multiple occurences of beam, copp and bin words
;                   in the dimension secton bu only one line in that section 
;                   may have the dim name and '='
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrcdl2nc,cdlfile,profrange,ncfile=ncfile,edit=edit,wcropen=wcropen, $
              average=average,beams=beams,indir=indir,outdir=outdir, $
              savecdl=savecdl,silent=silent,big_endian=bgnd,help=help

@commonwcrinclude

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrcdl2nc'
    return
  endif

; Check inputs

  if n_params() lt 1 then begin
    message,"Usage: wcrcdl2nc, cdlfile,profrange, $",/info
    print,"                    ncfile=ncfile,edit=edit,average=average, $"
    print,"                    beams=beams,indir=indir,outdir=outdir, $"
    print,"                    savecdl=savecdl,big_endian=bgnd, $"
    print,"                    wcropen=wcropen,silent=silent,help=help"
    return
  endif

  ; Check wcropen

  if n_elements(wcropen) eq 0 then wcropen=31

  if size(wcropen,/type) eq 7 then begin
    if n_elements(wcropen) eq 1 then begin
      junk1=-13.d0
      junk2=90.
    endif else if n_elements(wcropen) eq 2 then begin
      junk1=double(wcropen[1])
      junk2=90.
    endif else if n_elements(wcropen) eq 2 then begin
      junk1=float(wcropen[1])
      junk2=float(wcropen[2])
    endif
    openwcrf,wcropen[0],data,timecor=junk1,acspeed=junk2
  endif else if size(wcropen,/type) gt 3 then begin
    message, 'Invalid input for wcropen. Must be a string or an integer.',/info
    return
  endif else begin
    data=assoc(wcropen,{header:lonarr(headersize/4), $
                        data:fltarr(datablocksize/4)},filehdrsz)
  endelse

  ; Check profrange

  if n_elements(profrange) eq 0 then profrange=[0,tnumrec-1] $
  else if n_elements(profrange) eq 1 then begin
    if profrange ge tnumrec-1 then $
       message,'Invalid profrange for '+wcrfile
    profrange=[profrange,tnumrec-1]
  endif else if profrange[0] ge profrange[1] or profrange[0] lt 0 or $
  profrange[1] ge tnumrec then message,'Invalid profrange for '+wcrfile

  ; Check active beams configuration input

  if n_elements(beams) eq 0 then beams=1 else $
    if beams lt 1 and beams gt 4 and beams ne 12 and beams ne 13 and $
    beams ne 24 then begin
      message,'Invalid value for beams.',/info
      return
    endif

  if keyword_set(silent) eq 0 then $
      message,'Active beams configuration: '+num2str(beams),/info

  ; Append '/' to in and out directory paths if needed

  if n_elements(indir) eq 0 then indir='./' else $
    if strpos(indir,'/',/reverse_search)+1 ne strlen(indir) then indir=indir+'/'
  if n_elements(outdir) eq 0 then outdir='./' else $
    if strpos(outdir,'/',/reverse_search)+1 ne strlen(outdir) then $
      outdir=outdir+'/'
  if n_elements(savecdl) eq 0 then savecdl=0

  ; Check EDIT

  editall=(editgen=(editalg=(editdim=(editglb=0))))
  if n_elements(edit) eq 0 then editall=1 $
  else if n_elements(edit) gt 3 then begin
    message,'EDIT cannot have more than 3 elements',/info
    return
  endif else begin
    editall=((where(edit eq 'all') ne -1))[0]
    editgen=((where(edit eq 'gen') ne -1))[0]
    editdim=((where(edit eq 'dim') ne -1))[0]
    editglb=((where(edit eq 'glb') ne -1))[0]
    editalg=((where(edit eq 'alg') ne -1))[0]
    if editall+editgen+editdim+editglb+editalg eq 0 then begin
      message,'Invalid EDIT value(s)',/info
      return
    endif
  endelse

  ; Check profile and rangegate averages and sizes

  if editgen eq 0 then begin
    if n_elements(average) eq 0 then begin
      profav=2 & rangeav=1
    endif else if n_elements(average) eq 1 then begin
      profav=average>1 & rangeav=1
    endif else begin
      profav=average[0]>1 & rangeav=average[1]>1
    endelse

    profiles=(profrange[1]-profrange[0]+1)/long(profav)
    rngts=rangegates/long(rangeav)

    if profiles lt 1 or rngts lt 1 then begin
      message,'The resultant dimension size for profile/range is less than 1', $
              /info
      return
    endif
  endif

  ; Compose ncfile name if needed.  No check for ncfile if defined and not empty

  if n_elements(ncfile) eq 0 then ncfile=''
  if strmid(ncfile,0,1) ne '$' then ncfile=''
  if ncfile eq '' then begin
    if editgen then begin
      junk1=strpos(cdlfile,'.cdl')
      if junk1 eq -1 then junk1=strlen(cdlfile)
      ncfile=strmid(cdlfile,0,junk1)
    endif else begin
      junk1=strpos(cdlfile,'.')
      if junk1 eq -1 then junk1=strlen(cdlfile)
      junk1=strmid(cdlfile,0,junk1)+'.'
      if alg eq sppalg      then junk1='SP'+txstrpol[0]+'.'+junk1 $
      else if alg eq pp4alg then junk1='PP4.'+junk1 $
      else if alg eq pp6alg then junk1='PP6.'+junk1 $
      else if alg eq fftalg then junk1='FFT.'+junk1

      if wcrdate lt 20020331 then bgnd=1
      swapbytes = 256L   ; 256 is default for no byte swapping
      if keyword_set(bgnd) then byteorder,swapbytes,/swap_if_little_endian $
       else byteorder,swapbytes,/swap_if_big_endian

      junk2=(data(profrange[0])).header[3-2*(wcrdate gt 20020331):4-2*(wcrdate gt 20020331)]
      if swapbytes eq 1 then junk2=swap_endian(junk2)
      ind1=sec2timestr(double(junk2[0] mod 86400)+ $
                       double(junk2[1])/1d6+double(tcorrection))
      ind1 =strmid(ind1,0,2)+strmid(ind1,3,2)+strmid(ind1,6,2)
      junk2=(data(profrange[1])).header[3-2*(wcrdate gt 20020331):4-2*(wcrdate gt 20020331)]
      if swapbytes eq 1 then junk2=swap_endian(junk2)
      ind2=sec2timestr(double(junk2[0] mod 86400)+ $
                       double(junk2[1])/1d6+double(tcorrection))
      ind2 =strmid(ind2,0,2)+strmid(ind2,3,2)+strmid(ind2,6,2)


      ncfile='WCR.'+junk1+num2str(wcrdate)+'.'+ind1+'_'+ind2
    endelse
  endif

; ----------------------------------------------------------------------------
; (EDIT='gen') Compile and generate netcdf file directly from cdlfile and exit
; ----------------------------------------------------------------------------

  if editgen then begin
    if keyword_set(silent) eq 0 then $
      message,'Checking CDL syntax and generating '+ncfile+' ....',/info
    spawn,'ncgen '+indir+cdlfile,errmsg
    if errmsg ne '' then begin
      message,'Compilation errors reported by ncgen utility:',info
      print,''
      print,errmsg
      return
    endif else begin
      junk1=strpos(ncfile,'.nc')
      if junk1 eq -1 then junk1='.nc ' else junk1=' '
      spawn,'ncgen -o '+outdir+ncfile+junk1+indir+cdlfile
    endelse
    return
  endif

; ----------------------------------------------------------------------------
; Read the input cdl file and check if WCR_BeamConfig is consistent with beams
; ----------------------------------------------------------------------------

  ; Read in cdl file

  incdl=getfile(indir+cdlfile)

  ;Extract WCR_BeamConfig

  ind1=where(strpos(incdl,':WCR_BeamConfig') ne -1)
  ind1=strpos(incdl[ind1],strtrim(string(beams),2))
  if ind1 eq -1 then begin
    message,"Input beams="+num2str(beams)+" not found in WCR_BeamConfig",/info
    return
  endif

; ----------------------------------------------------------------------------
; Read Aircraft global attribute. Default AC id is N2UW
; ----------------------------------------------------------------------------

  platform='N2UW'
  junk1=where(strpos(incdl,':Aircraft ') ne -1)
  if junk1[0] eq -1 then junk1=where(strpos(incdl,':Aircraft=') ne -1)
  if junk1[0] eq -1 then message,"Global attribute Aircraft is missing"
  ind1=strpos(incdl[junk1[0]],'"')
  ind2=strpos(incdl[junk1[0]],'"',/reverse_search)
  platform=strtrim(strmid(incdl[junk1],ind1+1,ind2-ind1-1),2)

  ; Verify that if plaform is MARSF beams is always 1

  if platform eq 'MARSF' and beams ne 1 then begin
    message,'Platform is MARSF. beams is forced to 1',/info
    beams=1
  endif

; ------------------------------------------------------------------------
; (EDIT='alg' or 'all') Edit relevant lines in the file
; ------------------------------------------------------------------------

  if editall or editalg then begin

    ; Remove comment lines from the cdl file if any
      ind1=where(strpos(incdl,'//*') eq -1)
      ind2=where(strpos(incdl,'*//') eq -1)
      if ind1[0] ne -1 and total(ind1-ind2) eq 0 then incdl=incdl[ind1]

    ; Add a comment line to acknowledge that this is a program generated file
      if strtrim(incdl[1],2) eq '' then $
        incdl[1]='//* This file is created by wcrcdl2nc, '+systime()+' *//

    if alg eq pp4alg or alg eq pp6alg then begin

    ; Remove //SPP lines from incdl
      ind1=where(strpos(incdl,'//SPP') eq -1)
      incdl=incdl[ind1]

    ; Remove //FFT lines from incdl
      ind1=where(strpos(incdl,'//FFT') eq -1)
      incdl=incdl[ind1]

    endif else if alg eq sppalg or alg eq fftalg then begin

    ; Remove //PPM lines from incdl
      ind1=where(strpos(incdl,'//PPM') eq -1)
      incdl=incdl[ind1]

    ; Remove appropriate co-, cross-variables and noise vars for FFT
      if txsequence[0] eq polh then begin
        ind1=where(strpos(incdl,'vv(profile,range)') eq -1)
        incdl=incdl[ind1]
        ind1=where(strpos(incdl,'vv:') eq -1)
        incdl=incdl[ind1]
        ind1=where(strpos(incdl,'hv(profile,range)') eq -1)
        incdl=incdl[ind1]
        ind1=where(strpos(incdl,'hv:') eq -1)
        incdl=incdl[ind1]
        if alg eq fftalg then begin
          ind1=where(strpos(incdl,'vmnoise(profile)') eq -1)
          incdl=incdl[ind1]
          ind1=where(strpos(incdl,'vmnoise:') eq -1)
          incdl=incdl[ind1]
          ind1=where(strpos(incdl,'vsnoise:') eq -1)
          incdl=incdl[ind1]
        endif
      endif else begin
        ind1=where(strpos(incdl,'hh(profile,range)') eq -1)
        incdl=incdl[ind1]
        ind1=where(strpos(incdl,'hh:') eq -1)
        incdl=incdl[ind1]
        ind1=where(strpos(incdl,'vh(profile,range)') eq -1)
        incdl=incdl[ind1]
        ind1=where(strpos(incdl,'vh:') eq -1)
        incdl=incdl[ind1]
        if alg eq fftalg then begin
          ind1=where(strpos(incdl,'hmnoise(profile)') eq -1)
          incdl=incdl[ind1]
          ind1=where(strpos(incdl,'hmnoise:') eq -1)
          incdl=incdl[ind1]
          ind1=where(strpos(incdl,'hsnoise:') eq -1)
          incdl=incdl[ind1]
        endif
      endelse

    ; For SPPMAG remove //FFT lines
      if alg eq sppalg then begin
        ind1=where(strpos(incdl,'//FFT') eq -1)
        incdl=incdl[ind1]
      endif

    endif

    ; Remove cross-channels for multiple active beams

    if beams gt 10 then begin
      ind1=where(strpos(incdl,'hv(profile,range)') eq -1)
      incdl=incdl[ind1]
      ind1=where(strpos(incdl,'hv:') eq -1)
      incdl=incdl[ind1]
      ind1=where(strpos(incdl,'vh(profile,range)') eq -1)
      incdl=incdl[ind1]
      ind1=where(strpos(incdl,'vh:') eq -1)
      incdl=incdl[ind1]
    endif

    ; Remove wcrbeams if only beam 1 is used

    if beams eq 1 then begin
      ind1=where(strpos(incdl,'short wcrbeams') eq -1)
      incdl=incdl[ind1]
      ind1=where(strpos(incdl,'wcrbeams:') eq -1)
      incdl=incdl[ind1]

    ; Remove wcrmirror variable if beam 1 is not used

    endif else if beams ne 1 and fix(beams/10) ne 1 and round(beams/10. mod 1) $
    ne 1 then begin
      ind1=where(strpos(incdl,'wcrmirror(profile)') eq -1)
      incdl=incdl[ind1]
      ind1=where(strpos(incdl,'wcrmirror:') eq -1)
      incdl=incdl[ind1]
    endif

    ; Make all references to AC id string consistent.  Default AC id is N2UW

    if platform ne 'N2UW' then begin
      ind1=where(strpos(incdl,'N2UW') ne -1)
      if ind1[0] ne -1 then begin
        for k=0,n_elements(ind1)-1 do begin
          ind2=strpos(incdl[ind1[k]],'N2UW')
          incdl[ind1[k]]=strmid(incdl[ind1[k]],0,ind2)+platform+ $
                         strmid(incdl[ind1[k]],ind2+4)
        endfor
      endif
    endif

  endif     ; EDITALG

; ------------------------------------------------------------------------
; (EDIT='glb' or 'all') Update relevant global attributes
; ------------------------------------------------------------------------

  if editall or editglb then begin

    ;******************************* WCRdaqalg ********************************

    ind1=where(strpos(incdl,':WCRdaqalg') ne -1)
    if alg eq sppalg then junk1='"SPPMAG" ;' $
    else if alg eq pp4alg then junk1='"PPMAG4" ;' $
    else if alg eq pp6alg then junk1='"PPMAG6" ;'
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+' '+junk1

    ;******************************* WCRtxseq *********************************

    ind1=where(strpos(incdl,':WCRtxseq') ne -1)
    if n_elements(txstrpol) eq 4 then junk1='"'+txstrpol[0]+txstrpol[1]+ $
                                  txstrpol[2]+txstrpol[3]+'" ;' $
    else if n_elements(txstrpol) eq 6 then junk1='"'+txstrpol[0]+txstrpol[1]+ $
                        txstrpol[2]+txstrpol[3]+txstrpol[4]+txstrpol[5]+'" ;' $
    else if n_elements(txstrpol) eq 1 then junk1='"'+txstrpol+'" ;'
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+' '+junk1

    ;******************************* WCRppseq *********************************

    if alg eq fftalg then begin
    ; For FFT this is not a meaningfull attribute; so remove it
      ind1=where(strpos(incdl,':WCRppseq') eq -1)
      incdl=incdl[ind1]
    endif else begin
      ind1=where(strpos(incdl,':WCRppseq') ne -1)
      if n_elements(ppstrpol) eq 3 then junk1='"'+ppstrpol[0]+','+ppstrpol[1]+ $
                                        ','+ppstrpol[2]+'" ;' $
      else if n_elements(ppstrpol) eq 5 then junk1='"'+ppstrpol[0]+','+ $
           ppstrpol[1]+','+ppstrpol[2]+','+ppstrpol[3]+','+ppstrpol[4]+'" ;' $
      else if n_elements(ppstrpol) eq 1 then junk1='"'+ppstrpol+'" ;'
      incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+' '+junk1
    endelse

    ;******************************* WCRpulsepacket ***************************

    ind1=where(strpos(incdl,':WCRpulsepacket') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                ' '+num2str(npulse)+' ;'

    ;******************************* WCRcdlfile *******************************

    ind1=where(strpos(incdl,':WCRcdlfile') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                '"'+cdlfile+'" ;'

    ;******************************* WCRdaqfile *******************************

    ind1=where(strpos(incdl,':WCRdaqfile') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                '"'+wcrfile+'" ;'

    ;******************************* WCRpulsewidth ****************************

    ind1=where(strpos(incdl,':WCRpulsewidth') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                ' '+num2str(pulselength)+' ;'

    ;******************************* WCRpulseav *******************************

    if alg eq fftalg then begin
    ; For FFT this is not a meaningfull attribute; so remove it
      ind1=where(strpos(incdl,':WCRpulseav') eq -1)
      incdl=incdl[ind1]
    endif else begin
      ind1=where(strpos(incdl,':WCRpulseav') ne -1)
      incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                  ' '+num2str(averages)+' ;'
    endelse

    ;******************************* WCRprfHz *********************************

    ind1=where(strpos(incdl,':WCRprfHz') ne -1)
    junk1=num2str(prf[0],fr=1)+'f'
    if n_elements(prf) ge 4 then junk1=junk1+','+num2str(prf[1],fr=1)+'f,'+$
                          num2str(prf[2],fr=1)+'f,'+num2str(prf[3],fr=1)+'f' $
    else if n_elements(prf) eq 6 then junk1=','+num2str(prf[4],fr=1)+'f,'+ $
                                      num2str(prf[5],fr=1)+'f'
    junk1=junk1+' ;'
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+' '+junk1

    ;******************************* WCRprfrange ******************************

    ind1=where(strpos(incdl,':WCRprfrange') ne -1)
    junk2=3e8/(2.*prf)
    junk1=num2str(junk2[0],fr=2)+'f'
    if n_elements(junk2) ge 4 and alg ne sppalg then $
      junk1=junk1+','+num2str(junk2[1],fr=2)+'f,'+$
            num2str(junk2[2],fr=2)+'f,'+num2str(junk2[3],fr=2)+'f' $
    else if n_elements(prf) eq 6 then junk1=','+num2str(junk2[4],fr=2)+'f,'+ $
                                      num2str(junk2[5],fr=2)+'f'
    junk1=junk1+' ;'
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+' '+junk1

    ;******************************* WCRprfvelocity ***************************

    ind1=where(strpos(incdl,':WCRprfvelocity') ne -1)
    junk1=num2str(maxvel[0],fr=2)+'f'
    if n_elements(maxvel) ge 4 then junk1=junk1+','+num2str(maxvel[1],fr=2)+$
                   'f,'+num2str(maxvel[2],fr=2)+'f,'+num2str(maxvel[3],fr=2)+'f' $
    else if n_elements(maxvel) eq 6 then junk1=','+num2str(maxvel[4],fr=2)+'f,'+ $
                                         num2str(maxvel[5],fr=2)+'f'
    junk1=junk1+' ;'
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+' '+junk1

    ;******************************* WCRrangegates ****************************

    ind1=where(strpos(incdl,':WCRrangegate') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                ' '+num2str(rangegates)+' ;'

    ;******************************* WCRrangesampling *************************

    ind1=where(strpos(incdl,':WCRrangesampling') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                ' '+num2str(rangegatespacing,fr=1)+'f ;'

    ;******************************* WCRrangeresolution ***********************

    ind1=where(strpos(incdl,':WCRrangeresolution') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                ' '+num2str(150./iffilter,fr=1)+'f ;'

    ;******************************* UTCcorrection ****************************

    ind1=where(strpos(incdl,':UTCcorrection') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                ' '+num2str(tcorrection,fr=2)+'f ;'

    ;*************************** WCRactivebeamsconfig *************************

    ind1=where(strpos(incdl,':WCRactivebeamsconfig') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                ' '+num2str(beams)+' ;'


    ;**************************************************************************
    ; Remove or Update WCRfftbins, WCRfftave, WCRfftwin, WCRfftres
    ;**************************************************************************

     if alg eq fftalg then begin
       ind1=where(strpos(incdl,':WCRfftbins') ne -1)
       incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                   ' '+num2str(fftbins,fr=0)+' ;'

       ind1=where(strpos(incdl,':WCRfftave') ne -1)
       incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                   ' '+num2str(averages,fr=0)+' ;'

       if fftwindow eq 0 then junk1='Rect' $
       else if fftwindow eq 1 then junk1='Hann' $
       else if fftwindow eq 2 then junk1='Taper' $
       else if fftwindow eq 3 then junk1='Blackman' $
       else if fftwindow eq 4 then junk1='Gaussian' $
       else if fftwindow eq 5 then junk1='Gaussian Top'

       ind1=where(strpos(incdl,':WCRfftwin') ne -1)
       incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                   '"'+junk1+'" ;'

       ind1=where(strpos(incdl,':WCRfftres') ne -1)
       incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                   ' '+num2str(2.*maxvel/fftbins,fr=2)+' ;'

     endif else begin
       ind1=where(strpos(incdl,':WCRfftbins') eq -1)
       incdl=incdl[ind1]
       ind1=where(strpos(incdl,':WCRfftave') eq -1)
       incdl=incdl[ind1]
       ind1=where(strpos(incdl,':WCRfftwin') eq -1)
       incdl=incdl[ind1]
       ind1=where(strpos(incdl,':WCRfftres') eq -1)
       incdl=incdl[ind1]
     endelse

  endif     ; EDITGLB

; ------------------------------------------------------------------------
; (EDIT='dim' or 'all') Check dimension vector3; Update dimensions profile,
; range, copp, beam and bin
; ------------------------------------------------------------------------

  if editall or editdim then begin

    ; Find dimension section

    ind1=(where(strpos(incdl,'dimensions:') ne -1))[0]
    if ind1 eq -1 then begin
      message,"Can't find dimension section in "+cdlfile, /info
      return
    endif
    ind2=(where(strpos(incdl,'variables:') ne -1))[0]
    if ind2 eq -1 then begin
      message,"Can't find variable section in "+cdlfile, /info
      return
    endif

    ndim=ind2-ind1-1

    ; Check for profile, range and vector3 dims. If OK update profile and range

    if ndim lt 3-(platform eq 'MARSF')+(alg eq pp4alg or alg eq pp6alg) then begin
      message,"Required dimension(s) is/are missing in "+cdlfile, /info
      print,        " (allowed dims: profile,range,copp,beam,bin,vector3) "
      return
    endif else begin
     ; Check if profile, range and vector3 dimensions exist and modify incdl

      junk1=where(strpos(incdl[ind1:ind2],'profile') ne -1)
      if junk1 eq -1 then begin
        message,"'profile' dimension not found in "+cdlfile, /info
        return
      endif else begin
        junk2=strpos(incdl[ind1+junk1],'=')
        if junk2 eq -1 then begin
          message,'CDL syntax error (missing =) in '+cdlfile+', line '+ $
                  num2str(ind1+junk1),/info
          return
       endif else $
        incdl[ind1+junk1]=strmid(incdl[ind1+junk1],0,junk2+1)+' '+ $
             num2str(profiles)+' ;'
     endelse

      junk1=where(strpos(incdl[ind1:ind2],'range') ne -1)
      if junk1 eq -1 then begin
        message,"'range' dimension not found in "+cdlfile, /info
        return
      endif else begin
        junk2=strpos(incdl[ind1+junk1],'=')
        if junk2 eq -1 then begin
          message,'CDL syntax error (missing =) in '+cdlfile+', line '+ $
                  num2str(ind1+junk1),/info
          return
        endif else $
          incdl[ind1+junk1]=strmid(incdl[ind1+junk1],0,junk2+1)+' '+ $
                            num2str(rngts)+' ;'
      endelse

      if platform ne 'MARSF' then begin
        junk1=where(strpos(incdl[ind1:ind2],'vector3') ne -1)
        if junk1 eq -1 then begin
          message,"'vector3' dimension not found in "+cdlfile, /info
          return
        endif
      endif

      if alg eq pp4alg or alg eq pp6alg then begin

      ; For PPMAG process copp and beam dimensions

        ; Determine copp dimension and update it

        copp=n_elements(where(ppstrpol eq 'HH' or ppstrpol eq 'VV'))

        junk1=where(strpos(incdl[ind1:ind2],'copp') ne -1)
        if junk1[0] eq -1 then begin
          if copp eq 1 then begin
            if keyword_set(silent) eq 0 then $
              message,"'copp' dim. not found in "+cdlfile+'. Ignore.', /info
          endif else begin
            message,"'copp' dimension not found in "+cdlfile, /info
            return
          endelse
        endif else begin
          junk2=strpos(incdl[ind1+junk1],'=')
          k=(where(junk2 ne -1))[0]
          if k eq -1 then begin
            message,'CDL syntax error (copp dim missing =) in '+cdlfile,/info
            return
          endif else $
            incdl[ind1+junk1[k]]=strmid(incdl[ind1+junk1[k]],0,junk2[k]+1)+ $
                                 ' '+num2str(copp)+' ;'
        endelse

        ; Determine beam dimension and update it for non MARSF platforms

        if platform ne 'MARSF' then begin
          junk1=string(byte(total(byte(txstrpol))/n_elements(txstrpol)))
          if beams gt 10 and (where(junk1 eq txstrpol))[0] ne -1 then beam=1 $
          else if beams gt 10 then beam=2 else beam=1

          junk1=where(strpos(incdl[ind1:ind2],'beam') ne -1)
          if junk1[0] eq -1 then begin
            if beam eq 1 then begin
              if keyword_set(silent) eq 0 then $
                message,"'beam' dim. not found in "+cdlfile+'. Ignore.', /info
            endif else begin
              message,"'beam' dimension not found in "+cdlfile, /info
              return
            endelse
          endif else begin
            junk2=strpos(incdl[ind1+junk1],'=')
            k=(where(junk2 ne -1))[0]
            if k eq -1 then begin
              message,'CDL syntax error (beam dim missing =) in '+cdlfile,/info
              return
            endif else $
              incdl[ind1+junk1[k]]=strmid(incdl[ind1+junk1[k]],0,junk2[k]+1)+ $
                                   ' '+num2str(beam)+' ;'
          endelse
        endif 

      endif else if alg eq fftalg then begin

      ; For FFT process bin dimension

        junk1=where(strpos(incdl[ind1:ind2],'bin') ne -1)
        if junk1[0] eq -1 then begin
          message,"'bin' dimension not found in "+cdlfile, /info
          return
        endif else begin
          junk2=strpos(incdl[ind1+junk1],'=')
          k=(where(junk2 ne -1))[0]
          if k eq -1 then begin
            message,'CDL syntax error (bin dim missing =) in '+cdlfile,/info
            return
          endif else $
            incdl[ind1+junk1[k]]=strmid(incdl[ind1+junk1[k]],0,junk2[k]+1)+ $
                                 ' '+num2str(fftbins)+' ;'
        endelse

      endif

    endelse

    ; Edit relevant global attributes

    ;******************************* WCRprofrange *****************************

    ind1=where(strpos(incdl,':WCRprofrange') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                num2str(profrange[0])+','+num2str(profrange[1])+' ;'

    ;******************************* WCRprofav ********************************

    ind1=where(strpos(incdl,':WCRprofav') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                num2str(profav)+' ;'

    ;******************************* WCRrangeav ********************************

    ind1=where(strpos(incdl,':WCRrangeav') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                num2str(rangeav)+' ;'

    ;******************************* WCRtimeint *******************************

    ind1=where(strpos(incdl,':WCRtimeint') ne -1)
    incdl[ind1]=strmid(incdl[ind1],0,strpos(incdl[ind1],'=')+1)+ $
                num2str(timeinterval*profav,fr=6)+'f ;'

  endif     ; EDITDIM

; ------------------------------------------------------------------------
; Record the updated cdl file in a temporary <ncfile>.cdl
; ------------------------------------------------------------------------

  if keyword_set(silent) eq 0 then $
    message,'Writing '+ncfile+'.cdl ....',/info
  putfile,indir+ncfile+'.cdl',incdl

; ------------------------------------------------------------------------
; Check for syntax errors and generate netcdf file using ncgen utility
; ------------------------------------------------------------------------

  spawn,'ncgen '+indir+ncfile+'.cdl 2>&1',/sh,errmsg

  if errmsg[0] ne '' then begin
    print,''
    message,'Compilation errors reported by ncgen utility:',/info
    print,errmsg
    return
  endif else begin
    junk1=strpos(ncfile,'.nc')
    if junk1 eq -1 then junk1=ncfile+'.nc' else junk1=ncfile
    if savecdl ne 1 then begin
      if keyword_set(silent) eq 0 then $
        message,'Generating '+ncfile+' ....',/info
      spawn,'ncgen -o '+outdir+junk1+' '+indir+ncfile+'.cdl'
    endif
    if savecdl eq 0 then begin
      if keyword_set(silent) eq 0 then $
        message,'Removing '+ncfile+'.cdl ....',/info
      file_delete, indir+ncfile+'.cdl'
    endif
    ncfile=junk1
  endelse

  if keyword_set(silent) eq 0 then message,'wcrcdl2nc is finished.',/info

end
