;$Id: wcrcdv2nc.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRCDV2NC
;
; PURPOSE:
;       Write corrected for platform motion, Doppler velocity and related
;       variables to the corresponding WCR netcdf file
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrcdv2nc,ncf,cdv,beamvector,acvcbeam,acwcbeam,tas,mirror, $
;          timesync=timesync,misval=misval,ncdir=ncdir,cdvstatus=cdvstatus,$
;          revision=revision,close=close,silent=silent,help=help
;
; INPUTS:
;      Ncf:        long, file id of a netcdf file open in WRITE mode or
;                  string, WCR data netcdf file name
;                  Note: When Ncf is a file string, the raw WCR data matching
;                        the Ncf netcdf file data MUST be loaded.  If Ncf is
;                        a netcdf file id NO raw WCR data are required.
;      Cdv:        fltarr(range,profile) or fltarr(copp,range,profile);
;                  corrected for the platform motion radar Doppler velocity
;                  field(s) in m/s
;      Beamvector: fltarr(3,profile) or fltarr(beam,3,profile);
;                  (East,North,vertical) radar beam(s) unit vectors
;      Acvcbeam:   fltarr(profile) or fltarr(beam,profile);
;                  Aircraft velocity component into WCR beam(s) in m/s
;      Tas:        fltarr(profile); aircraft true air speed in m/s
;                  If not given the program does not update wcraspect variable
;      Mirror:     fltarr(profile); side/up antenna beam pointing direction
;                  (0 - side, 1 - up, -1 - in transition).  If not given the
;                  program does not update wcrmirror variable (used for N2UW
;                  only).
;
; KEYWORD PARAMETERS:
;       CLOSE:     when set netcdf file will be closed. Default is to keep the
;                  file open in WRITE mode. CAUTION: If a writable netcdf file
;                  is not closed before exiting IDL, the disk copy of the file
;                  may not reflect recent data changes or new definitions.
;       CDVSTATUS: input,string, processing status for cdv variable
;                  (default: 'platform motion corr.=Y, radar moment corr.=N,
;                             GPS corr.=Y; noise thresh.=N, folded=Y')
;       MISVAL     missing value for corrected Doppler field
;                  (default: same as for uncorrected Doppler velocity)
;       NCDIR:     input,string, directory path to ncf (default current dir)
;       REVISION:  input,string or strarr(2) netcdf file revision
;                  revision[0] - update Revision_X attribute (default 'Corrected
;                                Doppler velocity'); If the string starts with $
;                                the last Revision_X attribute is overwritten
;                                otherwise a new Revision_X is created.  If only
;                                '$' the last Revision_X will be overwritten with
;                                the default string 'Corrected Doppler velocity'
;                  revision[1] - update RevisionStatus attribute(default is 'OK')
;                  If input a string, update Revision_X attribute only;
;                  If empty string(s) '' - do not update the attribute(s)
;       SILENT:    when set supress routine informational messages
;       HELP:      shows this text
;
; OUTPUTS:
;       Ncf:      when ClOSE is not set and Ncf is a file name returns the
;                 opened WCR netcdf file id
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;
;       The following variables and attributes of the netcdf files are updated:
;       - time_offset (if timesync ne 0)
;       - dv and attributes status and missing value (if misval given)
;       - wcraspect and attributes status and missing_value
;       - wcrbeamvector and beamchannel attribute
;       - acvcbeam and beamchannel attribute
;       - acwcbeam and beamchannel attribute
;       - wcrbeams and beamchannel attribute (at this time no input from WCR
;         or aircraft data systems is available and global attribute
;         WCRactivebeamsconfig value is used)
;       - wcrmirror (if exist)
;
;       The RevisionStatus and Revision_X global attributes are updated and
;       RevisionNumber attribute is increased by 1
;
; NOTES:
;
; EXAMPLES:
;
;       1) Example, assuming the netcdf file is already open and needed
;          variables calculated
;
;       wcrcdv2nc,ncid,cdv,beamvector,acvcbeam,acwcbeam,tas,mirror, $
;                 timesync=-0.2,misval=!values.f_nan,rev=['$','Test']
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, February 2002
;       SH, 03/2002 - added cdvstatus keyword
;       SH, 10/2002 - added acwcbeam input variable
;       SH, 04/2004 - changed min/max AC tas from 30/150 to 50/200
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrcdv2nc,ncf,cdv,beamvector,acvcbeam,acwcbeam,tas,mirror, $
              timesync=timesync,misval=misval,ncdir=ncdir,revision=revision,  $
              close=close,cdvstatus=cdvstatus,silent=silent,help=help

  @commonwcrinclude

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrcdvnc'
    return
  endif

; Check inputs

  if n_params() lt 5 then begin
    message,"Usage: wcrcdvnc,ncf,cdv,beamvector,acvcbeam,acwcbeam,tas, $",/info
    print,"                   mirror,timesync=timesync,misval=misval, $"
    print,"                   ncdir=ncdir,cdvstatus=cdvstatus,$"
    print,"                   revision=revision,silent=silent,help=help"
    return
  endif else if n_params() eq 5 then begin
    tas=-999
    mirror=-999
  endif else if n_params() eq 6 then begin
    if (where(tas gt 1))[0] eq -1 then begin
      mirror=tas
      tas=-999
    endif else mirror=-999
  endif
  if n_elements(tas) eq 0 then tas=-999
  if n_elements(mirror) eq 0 then mirror=-999

  if n_elements(revision) eq 0 then begin
    revstat='OK'
    revatt='Corrected Doppler velocity'
    revadd=1
  endif else begin
    if n_elements(revision) ge 1 then begin
      if revision[0] eq '' then begin 
        revstat=''
        revatt=''
      endif else begin
        revstat=''
        revadd=(strpos(revision[0],'$') ne 0)
        if revadd eq 1 then revatt=revision[0] $
        else begin
          if strlen(revision[0]) eq 1 then $
            revatt='Corrected Doppler velocity' $
          else revatt=strmid(revision[0],1)
        endelse
      endelse
    endif 
    if n_elements(revision) gt 1 then revstat=revision[1]
  endelse

  if n_elements(timesync) eq 0 then timesync=0.

  if size(ncf,/type) eq 7 then begin

    ; Check ncfile directory path

    if n_elements(ncdir) eq 0 then ncdir='./' $
    else if strpos(ncdir,'/',/reverse_search)+1 ne strlen(ncdir) then $
      ncdir=ncdir+'/'

    ; Open ncfile in WRITE mode and load profile,range,beam,copp dimensions,
    ; all var names and some needed global attributes

    opennc=1
    junk1=strpos(ncf,'.nc')
    if junk1 eq -1 then junk1='.nc' else junk1=''
    ncid=ncdf_open(ncdir+ncf+junk1,/write)
  endif else begin
    opennc=0
    ncid=ncf
  endelse

  ncnames=ncdf_varnames(ncid)
  dimnames=ncdf_dimnames(ncid,dimsize=junk1)

  rangedim=junk1[where(dimnames eq 'range')]
  profdim=junk1[where(dimnames eq 'profile')]
  if (where(dimnames eq 'beam'))[0] ne -1 then $
    beamdim=(junk1[where(dimnames eq 'beam')])[0] $
  else beamdim=0
  if (where(dimnames eq 'copp'))[0] ne -1 then $
    coppdim=(junk1[where(dimnames eq 'copp')])[0] $
  else coppdim=0

  ncdf_attget,ncid,'WCRprofav',   profav,   /global
  ncdf_attget,ncid,'WCRrangeav',  rangeav,  /global
  ncdf_attget,ncid,'WCRprofrange',profrange,/global
  ncdf_attget,ncid,'WCRdaqfile',  daqfile,  /global & daqfile=string(daqfile)
  ncdf_attget,ncid,'WCRtimeint',  timeint,  /global
  ncdf_attget,ncid,'WCRrangesampling',rangesampling,/global

  rdim=rangedim*rangeav-1
  pdim=profdim*profav-1

; Prepare beams and beamchannel (match WCR_BeamChannel attribute)

  ncdf_attget,ncid,'WCRactivebeamsconfig',beams,/global & beams=fix(beams)

  if beams eq 1 then beamchannel='h&v' $
  else if beams lt 4 then beamchannel='v' $
  else if beams eq 4 then beamchannel='h' $
  else if beams lt 20 then beamchannel='h,v' $
  else beamchannel='v,h'

;------------------------------------------------------------------------------
; Check if the loaded WCR raw data matches the file data (only when opennc=1)
;------------------------------------------------------------------------------

  if opennc then begin

  ; Compare raw wcr data file

    if daqfile ne wcrfile then begin
      message,'Raw WCR data files do not match.',/info
      ncdf_close,ncid
      return
    endif

  ; Compare the loaded profile segment

    if profrange[0] ne prof[0]-fstblocknum or $
       profrange[1] ne prof[numrec-1]-fstblocknum  then begin
      message,'Profile segments do not match.',/info
      ncdf_close,ncid
      return
    endif

  ; Compare profile and range dimension

    if profdim ne numrec/profav then begin
      message,"Profile dim("+num2str(profdim)+") doesn't match loaded profiles"$
              +'('+num2str(numrec/profav)+')',/info
      ncdf_close,ncid
      return
    endif

    if rangedim ne rangegates/rangeav then begin
      message,'Range dim('+num2str(rangedim)+") doesn't match loaded rangegates"$
              +'('+num2str(rangegates/rangeav)+')',/info
      ncdf_close,ncid
      return
    endif

  endif

;------------------------------------------------------------------------------
; Check input arrays dimensions and update netcdf variables
;------------------------------------------------------------------------------

  ; Corrected Doppler velocity cdv <------------------------------

  junk1=size(cdv,/n_dim)
  if junk1 lt 2 or junk1 gt 3 then begin
    message,'cdv has less than 2 or more than 3 dimensions',/info
    ncdf_close,ncid
    return
  endif else begin
    if junk1 eq 2 then begin
      if coppdim gt 1 then begin
        message,'cdv copp dim does not match netcdf file dimension size',/info
        ncdf_close,ncid
        return
      endif else if coppdim eq 1 then begin
        if profav gt 1 or rangeav gt 1 then $
          junk2=reform(rebin(cdv[0:rdim,0:pdim],rangedim,profdim), $
                       1,rangedim,profdim) $
        else junk2=reform(cdv,1,rangedim,profdim)
      endif else begin
        if profav gt 1 or rangeav gt 1 then $
          junk2=rebin(cdv[0:rdim,0:pdim],rangedim,profdim) $
        else junk2=cdv
      endelse
    endif else begin
      if coppdim eq 0 then begin
        message,'cdv copp dim does not match netcdf file dimension size',/info
        ncdf_close,ncid
        return
      endif else if coppdim eq 1 then begin
        if profav gt 1 or rangeav gt 1 then $
          junk2=reform(rebin(cdv[0:rdim,0:pdim],rangedim,profdim), $
                       1,rangedim,profdim) $
        else junk2=reform(cdv,1,rangedim,profdim)
      endif else begin
        if profav gt 1 or rangeav gt 1 then $
          junk2=rebin(cdv[*,0:rdim,0:pdim],coppdim,rangedim,profdim) $
        else junk2=cdv
      endelse
    endelse
  endelse

  ncdf_update,ncid,'dv',junk2,'var',silent=silent
  if n_elements(misval) eq 1 then $
    ncdf_update,ncid,'missing_value',misval,'dv',silent=silent
  if n_elements(cdvstatus) eq 0 then $
    junk1='platform motion corr.=Y, radar moment corr.=N, '+ $
          'GPS corr.=Y; noise thresh.=N, folded=Y' $
  else junk1=cdvstatus
  ncdf_update,ncid,'status',junk1,'dv',silent=silent

  ; Radar beam unit vector (beamvector) <------------------------------

  junk1=size(beamvector,/n_dim)
  if junk1 lt 2 or junk1 gt 3 then begin
    message,'beamvector has less than 2 or more than 3 dimensions',/info
    ncdf_close,ncid
    return
  endif else begin
    if junk1 eq 2 then begin
      if beamdim gt 1 then begin
        message,'beam dim does not match netcdf file dimension size',/info
        ncdf_close,ncid
        return
      endif else if beamdim eq 1 then begin
        if profav gt 1 then $
          junk2=reform(rebin(beamvector[*,0:pdim],3,profdim),1,3,profdim) $
        else junk2=reform(beamvector,1,3,profdim)
      endif else begin
        if profav gt 1 then $
          junk2=rebin(beamvector[*,0:pdim],3,profdim) $
        else junk2=beamvector
      endelse
    endif else begin
      if beamdim eq 0 then begin
        message,'beam dim does not match netcdf file dimension size',/info
        ncdf_close,ncid
        return
      endif else if beamdim eq 1 then begin
        if profav gt 1 then $
          junk2=reform(rebin(beamvector[*,0:pdim],3,profdim),1,3,profdim) $
        else junk2=reform(beamvector,1,3,profdim)
      endif else begin
        if profav gt 1 then $
          junk2=rebin(beamvector[*,*,0:pdim],beamdim,3,profdim) $
        else junk2=beamvector
      endelse
    endelse
  endelse

  ncdf_update,ncid,'wcrbeamvector',junk2,'var',silent=silent
  ncdf_update,ncid,'beamchannel',beamchannel,'wcrbeamvector',silent=silent

  ; Aircraft velocity component (acvcbeam) <------------------------------

  junk1=size(acvcbeam,/n_dim)
  if junk1 lt 1 or junk1 gt 2 then begin
    message,'acvcbeam has less than 1 or more than 2 dimensions',/info
    ncdf_close,ncid
    return
  endif else begin
    if junk1 eq 1 then begin
      if beamdim eq 1 then begin
        if profav gt 1 then $
          junk2=reform(rebin(acvcbeam[0:pdim],profdim),1,profdim) $
        else junk2=reform(acvcbeam,1,profdim)
      endif else begin
        if profav gt 1 then $
          junk2=rebin(acvcbeam[0:pdim],profdim) $
        else junk2=acvcbeam
      endelse
    endif else begin
      if beamdim eq 1 then begin
        if profav gt 1 then $
          junk2=reform(rebin(acvcbeam[0:pdim],profdim),1,profdim) $
        else junk2=reform(acvcbeam,1,profdim)
      endif else begin
        if profav gt 1 then $
          junk2=rebin(acvcbeam[*,0:pdim],beamdim,profdim) $
        else junk2=acvcbeam
      endelse
    endelse
  endelse

  ncdf_update,ncid,'acvcbeam',junk2,'var',silent=silent
  ncdf_update,ncid,'beamchannel',beamchannel,'acvcbeam',silent=silent

  ; Aircraft measured wind component (acwcbeam) <------------------------------

  junk1=size(acwcbeam,/n_dim)
  if junk1 lt 1 or junk1 gt 2 then begin
    message,'acwcbeam has less than 1 or more than 2 dimensions',/info
    ncdf_close,ncid
    return
  endif else begin
    if junk1 eq 1 then begin
      if beamdim eq 1 then begin
        if profav gt 1 then $
          junk2=reform(rebin(acwcbeam[0:pdim],profdim),1,profdim) $
        else junk2=reform(acwcbeam,1,profdim)
      endif else begin
        if profav gt 1 then $
          junk2=rebin(acwcbeam[0:pdim],profdim) $
        else junk2=acwcbeam
      endelse
    endif else begin
      if beamdim eq 1 then begin
        if profav gt 1 then $
          junk2=reform(rebin(acwcbeam[0:pdim],profdim),1,profdim) $
        else junk2=reform(acwcbeam,1,profdim)
      endif else begin
        if profav gt 1 then $
          junk2=rebin(acwcbeam[*,0:pdim],beamdim,profdim) $
        else junk2=acwcbeam
      endelse
    endelse
  endelse

  ncdf_update,ncid,'acwcbeam',junk2,'var',silent=silent
  ncdf_update,ncid,'beamchannel',beamchannel,'acvcbeam',silent=silent

  ; Side/Up antenna beam position (wcrmirror) <------------------------------

  if where(ncnames eq 'wcrmirror') ne -1 then begin
    if mirror[0] ne -999 then begin
      if profav gt 1 then junk2=rebin(fix(mirror[0:pdim]),profdim,/sample) $
      else junk2=fix(mirror)
      ncdf_update,ncid,'wcrmirror',junk2,'var',silent=silent
    endif else $
      if keyword_set(silent) eq 0 then $
        message,'Mirror is not defined. wcrmirror update ignored.',/info
  endif else begin
    if mirror[0] ne -999 then $
      if keyword_set(silent) eq 0 then $
        message,'WCRmirror is not defined. mirror input ignored.',/info
  endelse

  ; Range/Profile aspect ratio (wcraspect) <------------------------------

  if where(ncnames eq 'wcraspect') ne -1 then begin
    if tas[0] ne -999 then begin

      ; Mark any values of tas < 50 or > 200 m/s as missing
      if n_elements(misval) eq 1 then junk3=misval $
      else ncdf_attget,ncid,'wcraspect','missing_value',junk3
      ind1=where(tas lt 50. or tas gt 200.)
      if ind1[0] ne -1 then tas[ind1]=-999.

      ; Calculate mean aspect ratio
      junk2=rangesampling/(timeint*tas)
      if profav gt 1 then junk2=rebin(junk2[0:pdim],profdim)
      ind1=where(junk2 lt 0.)
      if ind1[0] ne -1 then junk2[ind1]=junk3
      ncdf_update,ncid,'wcraspect',junk2,'var',silent=silent

      ; Write the data
      junk1='using aircraft air speed'
      ncdf_update,ncid,'status',junk1,'wcraspect',silent=silent
      if n_elements(misval) eq 1 then $
        ncdf_update,ncid,'missing_value',misval,'wcraspect',silent=silent
    endif else $
      if keyword_set(silent) eq 0 then $
        message,'Tas is not defined. wcraspect update ignored.',/info
  endif else begin
    if tas[0] ne -999 then $
      if keyword_set(silent) eq 0 then $
        message,'wcraspect is not defined. tas input ignored.',/info
  endelse

  ; Active beams (wcrbeams) currently a scalar duplicating WCRactive beams <----

  if where(ncnames eq 'wcrbeams') ne -1 then begin
    ncdf_update,ncid,'wcrbeams',beams,'var',silent=silent
    ncdf_update,ncid,'beamchannel',beamchannel,'wcrbeams',silent=silent
  endif else if keyword_set(silent) eq 0 then $
    message,'wcrbeams is not defined. Ignored.',/info

;------------------------------------------------------------------------------
; Update Timeoffsetsync attribute and time_offset var if needed
;------------------------------------------------------------------------------

  if timesync ne 0. then begin
    ncdf_varget,ncid,'time_offset',junk2
    junk2=junk2+timesync
    ncdf_update,ncid,'time_offset',junk2,'var',silent=silent
  endif
  ncdf_update,ncid,'Timeoffsetsync',timesync,'global',silent=silent

;------------------------------------------------------------------------------
; Update global attributes: ProcessAuthor,DateProcessed,RevisionDate,
;                           RevisionStatus,RevisionNumber,Revision_1
;------------------------------------------------------------------------------

  ncdf_update,ncid,'ProcessAuthor',getenv('USER')+'@'+getenv('HOST'),'global', $
                    silent=silent
  ncdf_update,ncid,'DateProcessed',systime(),'global',silent=silent
  ncdf_update,ncid,'RevisionDate',systime(),'global',silent=silent
  if revstat ne '' then $
    ncdf_update,ncid,'RevisionStatus',revstat,'global',silent=silent

  ; Check if the file had previous updates. If so add new Revision attributes.

  if revatt ne '' then begin
    ncdf_attget,ncid,'RevisionNumber',junk1,/global
      if revadd eq 0 then begin
        if junk1 gt 1 then begin
          ncdf_attget,ncid,'Revision_'+num2str(junk1-1),junk2,/global
          if strtrim(strlowcase(string(junk2)),2) eq strtrim(strlowcase(revatt),$
          2) then begin
            ncdf_control,ncid,/redef
            ncdf_attdel,ncid,'Revision_'+num2str(junk1),/global
            ncdf_control,ncid,/endef
            ncdf_control,ncid,/sync
            ncdf_update,ncid,'RevisionNumber',junk1-1,'global',silent=silent
            if keyword_set(silent) eq 0 then $
              message,'Duplicate Revision_X. Last Revision_X deleted.',/info
          endif else $
            ncdf_update,ncid,'Revision_'+num2str(junk1),revatt,'global', $
                              silent=silent
        endif else $
          ncdf_update,ncid,'Revision_1',revatt,'global',silent=silent
      endif else begin
      ncdf_attget,ncid,'Revision_'+num2str(junk1),junk2,/global
      if strtrim(junk2) ne '' then junk1=junk1+1
      if strtrim(strlowcase(string(junk2)),2) eq strtrim(strlowcase(revatt),2)$
      then begin
        if keyword_set(silent) eq 0 then $
          message,'Duplicate Revision_X. New Revision_X ignored.',/info
      endif else begin
        if junk1 gt 1 then begin
          if keyword_set(silent) eq 0 then $
            message,'New RevisionNumber:'+num2str(junk1),/info
          ncdf_update,ncid,'RevisionNumber',junk1,'global',silent=silent
          ncdf_addnew,ncid,'Revision_'+num2str(junk1),revatt,'global', $
                           silent=silent
        endif else $
          ncdf_update,ncid,'Revision_1',revatt,'global',silent=silent
      endelse
    endelse
  endif

;------------------------------------------------------------------------------
; Sync the file and close it if CLOSE is set
;------------------------------------------------------------------------------

  ncdf_control,ncid,/sync

  if keyword_set(close) then begin
    if keyword_set(silent) eq 0 then $
      message,'NetCDF file(ID='+num2str(ncid)+') closed.',/info
    ncdf_close, ncid
  endif else ncf=ncid

 if keyword_set(silent) eq 0 then message,'wcrcdvnc is finished.',/info

end
