;$Id: wcrcreatenc.pro,v 1.6 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRCREATENC
;
; PURPOSE:
;       Create netcdf file(s) from a given cdl file, WCR raw data file(s), and
;       optional WCR platform motion(aircraft) navigational data file
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrcreatenc,wcrfile,cdlfile,beams,profrange,acfile=acfile, $
;                   acnames=acnames,acspeed=acspeed,average=average,$
;                   utccorr=utccorr,nswin=nswin,timesync=timesync,unfold=unfold, $
;                   gpsc=gpsc,rmcor=rmcor,misval=misval,savecdl=savecdl,$
;                   ncdir=ncdir,revision=revision,ncfile=ncfile,big_endian=bgnd,$
;                   silent=silent,help=help
;
; INPUTS:
;       Wcrfile:   string, path/name of a raw WCR data file or
;                  strarr(m), m WCR raw files for the same Cdlfile and Acfile
;       Cdlfile:   string, path/name of project prototype cdl file; recommended
;                    name is <project>.cdl or <project>.<ver_date>.cdl
;       Beams:     integer, active beams config.  Possible configs:
;                    1,2,3,4 - single beam: side/up, down, side-fore, down-fore
;                    12,13,24 - two beams (e.g., 12 - side/up and down)
;                  intarr(m), as above but for all m WCR files
;       Profrange: long integer, number of profiles to be recorded in a single
;                    netcdf file(minimum 50). The number of netcdf files created
;                    depends on the total number of profiles in the raw wcrfile.
;                  lonarr(2), [start,stop]; profile range to be recorded in
;                    Ncfile. Default is [0,tnumrec-1]; tnumrec: all profiles in
;                    wcrfile
;                  lonarr(2,n), an array of [start,stop] for wcrfile. This means
;                    n netcdf files will be created matching each pair of
;                    profrange
;                  lonarr(1,m), as first input option but for all m WCR files
;                  lonarr(2,n,m), as lonarr(2,n) but for all m WCR files
;
;                  If profrange is not defined one netcdf file is created for
;                    every wcrfile
;
; KEYWORD PARAMETERS:
;       ACFILE:   input,string, path/name of the relevant aircraft data netcdf 
;                 file. If acfile is not defined or empty string, no correction 
;                 of the WCR Doppler velocity is performed.  The content of
;                 ACNAMES, if any, is ignored, as well as the keywords GPSC,
;                 RMCOR and TIMESYNC. For MARSF acfile should not be defined.
;       ACNAMES:  input, string array, A/C netcdf varnames to be used
;                  (if not defined N2UW or N130AR default var names are used)
;                  acnames[0] - AC true air speed in m/s
;                  acnames[1] - AC roll [deg]
;                  acnames[2] - AC pitch [deg]
;                  acnames[3] - AC true heading [deg]
;                  acnames[4] - AC east-west(EW) ground velocity [m/s]
;                  acnames[5] - AC north-south(NS) ground velocity [m/s]
;                  acnames[6] - AC vertical velocity [m/s]
;                  acnames[7] - AC East component of the wind [m/s]
;                  acnames[8] - AC North component of the wind [m/s]
;                  acnames[9] - AC vertical wind [m/s]
;                  acnames[10]- AC roll rate [rad/s] (use '' if N/A)
;                  acnames[11]- AC pitch rate [rad/s](use '' if N/A)
;                  acnames[12]- AC heading rate [rad/s] (use '' if N/A)
;                  acnames[13]- AC EW ground vel. GPS correction(use '' if N/A)
;                  acnames[14]- AC NS ground vel. GPS correction(use '' if N/A)
;
;                  Note: define first 10 names only if the last 5 vars are
;                        not needed; see ACLOADINS.PRO help for default variable 
;                        names
;       ACSPEED:  input, float; average aircraft speed in m/s (default 90.)
;                   The default value is close to the N2UW research speed. 
;                   This value/keyword is ignored if acfile is defined.
;       AVERAGE:  input, integer/intarr(2), # of profiles/[profiles,rangegates]
;                   to be averaged (default is [2,0]; 0 is same as 1)
;     BIG_ENDIAN: when set, assumes that the WCR file data are in big endian order.
;                   The default is LITTLE_ENDIAN. Ignored for data before 03/2002 
;       GPSC:     when set does NOT apply GPS correction to A/C E-W and N-S
;                   velocity for UWKA only (ignored for NCAR C130)
;                   NCAR C130 A/C E-W and N-S ground speed are GPS corrected.
;                   NRC CV580 GPSC is not needed.
;       MISVAL:   input, when set (1) use NaN as missing values; Values
;                   different than 1 will be used as missing values in dBZ.
;                   Default value is -32767. Maximum valid value is -150.
;       NCDIR:    input,string, directory path to to the directory where
;                   created netcdf file(s) will be saved (default current dir)
;       NCFILE:   output, string or strarr(n*m); Name convention:
;                   WCR.<alg>.<cdlfile>.<yyyymmdd>.<hhmmss_hhmmss>.nc,
;                   where cdlfile is truncated to its first dot; <alg> is WCR
;                   mode: SPP, PP4, PP6 or FFT; yymmdd and hhmmss_hhmmss are
;                   begin and end time of the data in the netcdf file.
;       NSWIN:    input, long, lonarr(3), lonarr(3,n), or lonarr(3,n,m)
;                   nswin[0:1] - smoothing noise-window size(default 99);should 
;                                be an odd number(1 is added for evens); it is 
;                                recommended not to exceed 10% of the processed 
;                                number of profiles;  values less than 3 and 
;                                greater than 50% of all profiles cause the use
;                                of constant mean and stdev of the noise data 
;                   nswin[2]   - defines which noise measurement pulses to use:
;                     0 - (default) use matching number of noise measurement 
;                         pulses with minimum mean
;                     1 - uses noise measurement from one noise pulse with
;                         min mean (for both h- and v-ch)
;                     2 - as 1 for h-ch and 0 for v-ch noise 
;                     3 - as 1 for v-ch and 0 for h-ch noise 
;
;                   If NSWIN is a scalar, same value is applied to both channels
;                   as smoothing window size and NSWIN[2] is the default.
;                   If NSWIN has 2 elements, nswin[0] is applied to H and V
;                   noise channels and nswin[1] is used as nswin[2].
;
;                 lonarr(3,n) -as above but different for every profrange
;                   pair(ncfile). All nswin values must be given explicitly.
;                   If m > 1 and nswin is lonarr(3,n), same value(s) of nswin 
;                   is/are used for every raw WCR file.
;
;                 lonarr(3,n,m) - as above but different for every
;                   wcr file. All nswin values must be given explicitly
;                   except when lonarr(3,1,m) but n>1; then same values for each
;                   raw WCR file (m) will be used for every segment(n)                  
;
;       RMCOR:    integer/fltarr(3), applies antenna moment correction
;                    0 - antenna moment correction NOT applied  
;                    1 - antenna moment correction applied
;                    [x,y,x] - ant. position in aircraft coord w.r.t IRS in meters
;                 Default: for UWKA or NRC Convair not applied (0) 
;                          for NCAR C130 applied(1)
;                 Default antenna positions can be seen in wcracvelm.pro
;       REVISION: input,string or strarr(2) netcdf file revision
;                 revision[0] - update Revision_X attribute (default, depends
;                               on what is processed); If the string starts with
;                               $ the last Revision_X attribute is overwritten
;                               otherwise a new Revision_X is created. If only
;                               '$' the last Revision_X will be overwritten
;                               with the default string.
;                 revision[1] - update RevisionStatus attribute(default is 'OK')
;                 If input a string, update Revision_X attribute only;
;                 If empty string(s) '' - do not update the attribute(s)
;       SAVECDL:  input, integer, default is 0 (see wcrcdl2nc for more info)
;                   0 - removes the modified cdl file for every ncfile
;                   2 - saves the modified cdl file for every ncfile
;                   If cdl file is saved it uses ncfile name with extension cdl
;       SILENT:   when set supress routine informational messages
;       TIMESYNC: input,float, WCR time correction to sync with A/C IRS(INS)
;                   data, in seconds  (default 0.0)
;                 input fltarr(m), as above but different for every WCR file
;       UNFOLD:   input, Doppler velocity unfolding procedure (see notes)
;                 0          - no unfolding is performed (default)
;                 1          - use AC measured winds
;                 fltarr(3)  - [u,v,w], use the given wind vector
;                              u,v,w are allowed the following values:
;                              float: actual wind value to be used in m/s
;                                888: use AC measured values for all 
;                                     profiles of this component
;                                777: use mean of AC measured values from 
;                                     all profiles of this component
;                 ATTN: Note yet modified to work for multiple files/prof_ranges
;
;       UTCCORR:  if defined gives UTC time correction of the wcr time
;                   After June 1998, IRIG card is used to set the time.
;                   Normal use of the card is to get the time from a master
;                   IRIG card with GPS time. Thus the default value is the leap
;                   seconds(to see default leap seconds, check tcorrection
;                   after opening a file).
;       HELP:     shows this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;
;      See the side effects described in wcrcdl2nc, wcrwrite1nc and wcrwrite2nc
;
; NOTES:
;
; EXAMPLES:
;
;       1) Example using CLEX-9 experiment data (single antenna config, N2UW)
;
;       wcrdir='/sova4/clex01/oct31b/'
;       wcrfile=['Wpp01-10-31-16-16-07','Wpp01-10-31-16-36-17', $
;                'Wpp01-10-31-16-51-25','Wpp01-10-31-17-06-50']
;       cdlfile='/wcrproc/CLEX01.20020218.cdl'
;       acfile ='/net/bobcat/disk2/kingair_data/20011031_2.c25.nc'
;       profrange=10000L
;       ave=2
;       beams=1
;       timesync=[-0.3,-0.3,-0.15,-0.15]
;       ; the following line create nswin[3,1,4], so nswin[*,0,m] will be used
;       ; for any n greater than 1; happens for 3rd WCR file only
;       nswin=[[[99,99,0]],[[99,99,1]],[[199,199,0]],[[99,99,0]]]
;
;       wcrcreatenc,wcrdir+wcrfile,cdlfile,beams,profrange, $
;                   acfile=acfile,ncfile=ncfile,average=ave,timesync=timesync
;
;       2) Example using DYCOMS II experiment data (2 down antennas, N130AR)
;          Implemented in a 2 step procedure, where 'cdv' is calculated and
;          written in the netcdf files using wcrupdatenc routine. Note: since
;          acfile was not used for wcrcreatenc, a couple of global attributes
;
;       wcrdir='/net/fox/fox1/WCR/DYCOMS01/jul27/'
;       wcrfile='Wpp01-07-27-21-31-38'
;       cdlfile='/home/haimov/wcr/wcrproc/DYCOMS01.07.20020305.cdl'
;       acfile ='/net/fox/fox1/C130/hi_rate/RF09.20010727.191201_034427.PNI.nc'
;       ncdir  ='/home/haimov/wcr/wcrproc/'
;       profrange=30000L
;       beams=24
;       timesync=-0.125
;
;       wcrcreatenc,wcrdir+wcrfile,cdlfile,beams,profrange,ncdir=ncdir,$
;                   ncfile=ncfile,average=2,timesync=timesync,utccorr=0.
;
;       wcrupdatenc,ncdir+ncfile,wcrdir,'cdv',acpath=acfile,timesync=timesync
;
;       3) Example using HICU02 data
;
;       wcrfile='W2002-07-19-17-05-58'
;       cdlfile='/home/haimov/wcr/wcrproc/HICU02.20021009.cdl'
;       acfile ='/net/owl/data/kingair_data/20020719.c25.nc'
;       ncdir  ='/home/haimov/wcr/wcrproc'
;       beams=13
;       timesync=0.6
;
;       wcrcreatenc,wcrfile,cdlfile,beams,acfile=acfile,ncdir=ncdir,$
;                   ncfile=ncfile,average=2,timesync=timesync
;
;       4) Example using ground data from MARSF.  The trailer was oriented
;       North-South, with the antenna directed to 0 deg azimuth and 0 deg 
;       elevation.  This corresponds to H-pol for v-channel and V-pol for h-ch
;
;       wcrfile='temp/mar27/W2003-03-27-17-43-25'
;       cdlfile='temp/mar27/MARSF.cdl'
;       wcrcreatenc,wcrfile,cdlfile
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, March 2002
;       SH, 03/02 - modified to work with multiple raw WCR files
;       SH, 04/02 - added big_endian keyword
;       SH, 10/02 - minor changes in the help text
;       SH, 04/03 - few more minor changes in the help text assoc. with MARSF
;       SH, 05/03 - modified acnames, added UNFOLD keyword
;       SH, 09/03 - modified values for UNFOLD, acfile was changed from input
;                   parameter to keyword
;       SH, 11/03 - changes in the help text
;       SH, 12/03 - modified nswin inputs
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrcreatenc,wcrf,cdlfile,beams,profrange,acfile=acfile,acnames=acnames, $
                average=average,utccorr=utccorr,timesync=timesync,nswin=nswin, $
                unfold=unfold,gpsc=gpsc,rmcor=rmcor,misval=misval, $
                savecdl=savecdl,ncdir=ncdir,revision=revision,big_endian=bgnd,$
                acspeed=acspeed,ncfile=ncfile,silent=silent,help=help

  @commonwcrinclude

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrcreatenc'
    return
  endif

; Initilize

  nwcr=n_elements(wcrf)
  count   =0           ; counter for the number of netcdf files created
  maxcount=100         ; max number of ncfile before a slow assignment kicks in
  ncfile  =strarr(maxcount)
  minprofs=50          ; minimum profiles to process
  nswinsz =[99,99]     ; default noise smoothing window size for h- and v-ch

; Check inputs

  if n_params() lt 2 then begin
    message,"Usage: wcrcreatenc,wcrfile,cdlfile,beams,profrange,$",/info
    print,"                   acfile=acfile,average=average,utccorr=utccorr,$"
    print,"                   misval=misval,timesync=timesync,nswin=nswin,unfold=unfold,$"
    print,"                   gpsc=gpsc,rmcor=rmcor,savecdl=savecdl,ncdir=ncdir,$"
    print,"                   acspeed=acspeed,ncfile=ncfile,revision=revision,$"
    print,"                   big_endian=bgnd,silent=silent,help=help"
    return
  endif else if n_params() eq 2 then begin
    bms=1
    profrng=0
  endif else if n_params() eq 3 then begin
    bms=beams
    profrng=0
  endif else begin
    bms=beams
    profrng=profrange
  endelse

  if n_elements(acfile) eq 0 then acf='' else acf=acfile

  junk1=size(profrng,/n_dim)
  if (junk1 eq 1 and n_elements(profrng) gt 2) or $
     (junk1 eq 2 and n_elements(profrng[*,0]) eq 1 and n_elements(profrng[0,*])$
      ne nwcr) or (junk1 eq 3 and n_elements(profrng[*,0,0]) ne 2 and $
      n_elements(profrng[0,0,*]) ne nwcr) then begin
    message,'Invalid input for profrange',/info
    return
  endif

  ; Check if the files exist

  for k=0,nwcr-1 do begin
    spawn,'dir '+wcrf[k]+' 2>&1',/sh,junk1
    ind1=strpos(junk1,'such file or directory')
    if ind1 ne -1 then begin
      print,''
      message,'WCRFILE is not found:',/info
      print,junk1
      return
    endif
  endfor

  spawn,'dir '+cdlfile+' 2>&1',/sh,junk2
    ind1=strpos(junk2,'such file or directory')
    if ind1 ne -1 then begin
    print,''
    message,'CDLFILE is not found:',/info
    print,junk2
    return
  endif else begin
    junk1=strpos(cdlfile,'/',/reverse_search)
    if junk1 eq -1 then cdlf=cdlfile $
    else begin
      indir=strmid(cdlfile,0,junk1+1)
      cdlf=strmid(cdlfile,junk1+1)
    endelse
  endelse

  if acf ne '' then begin
    spawn,'dir '+acf+' 2>&1',/sh,junk2
    ind1=strpos(junk2,'such file or directory')
    if ind1 ne -1 then begin
      print,''
      message,'ACFILE is not found:',/info
      print,junk2
      return
    endif
  endif

  ; Check beams value and some of the keywords

  if n_elements(bms) eq 1 then bms=replicate(bms,nwcr)
  ind1=where((bms lt 1 or bms gt 4) and (bms ne 12 and  bms ne 13 and bms ne 24))
  if ind1[0] ne -1 or n_elements(bms) ne nwcr then begin
    message,'Invalid value(s)/size for beams.',/info
    return
  endif
  nocross=intarr(nwcr)
  ind1=where(bms gt 10)
  if ind1[0] ne -1 then nocross[ind1]=1

  if n_elements(timesync) eq 0 then timesync=fltarr(nwcr) $
  else if n_elements(timesync) eq 1 then timesync=replicate(timesync,nwcr) $
  else if n_elements(timesync) ne nwcr then begin
    message,'Invalid size for timesync.',/info
    return
  endif

  if not(n_elements(acnames) eq 0 or n_elements(acnames) eq 10 or $
     n_elements(acnames) eq 13 or n_elements(acnames) eq 15) then $
     message,'Incorrect input for acnames'

  if n_elements(ncdir) eq 0 then ncdir='./' $
  else if strpos(ncdir,'/',/reverse_search)+1 ne strlen(ncdir) then begin
    ncdir=ncdir+'/'
    spawn,'dir '+ncdir+' 2>&1',/sh,junk2
    if (strpos(junk2,'such file or directory'))[0] ne -1 then begin
      print,''
      message,'NCDIR is not found:',/info
      print,junk2
      return
    endif
  endif

  if keyword_set(savecdl) then savecdl=2 else savecdl=0

  if n_elements(revision) eq 0 then $
    revision=['$Reflectivity and Corrected Doppler velocity','OK'] $
  else if n_elements(revision) eq 1 then $
    revision=[revision,'OK']

;##############################################################################
; Cycle for all raw WCR files
;##############################################################################

for m=0,nwcr-1 do begin

  ;----------------------------------------------------------------------------
  ; Open wcrfile and find out how many netcdf files will be created
  ;----------------------------------------------------------------------------

  openwcrf,wcrf[m],timecor=utccorr,acspeed=acspeed,big_endian=bgnd

  ; Arrange profrange

  junk1=size(profrng,/n_dim)
  if junk1 eq 0 then begin
    if n_elements(profrng) eq 1 and profrng[0] ge minprofs then begin
      nseg=long(tnumrec/profrng)
      if nseg eq 0 then begin
        nseg=1
        if keyword_set(silent) eq 0 then begin
          message,"Profrange is greater than the existing profiles.",/info
          print,'               Ignore and use total number of profiles.'
        endif
      endif
    endif else nseg=1
    if nseg eq 1 then profr=[0,tnumrec-1] $
    else begin
      profr=lonarr(2,nseg)
      for k=0L,nseg-1 do profr[*,k]=[k*profrng,(k+1)*profrng-1L]
      if profr[1,nseg-1] ne tnumrec-1 then profr[1,nseg-1]=tnumrec-1
    endelse
  endif else if junk1 eq 1 then begin
    nseg=1
    profr=profrng
    ind1=where(profrng gt tnumrec-1)
    if ind1[0] ne -1 then begin
      profr[ind1]=tnumrec-1
      if keyword_set(silent) eq 0 then begin
        message,"Profrange includes values greater than existing profiles.",/info
        print,'               Ignore and use total number of profiles.'
      endif
    endif
    ind1=where(profrng lt 0)
    if ind1[0] ne -1 then begin
      profr[ind1]=0L
      if keyword_set(silent) eq 0 then begin
        message,"Profrange includes values less than 0. Use 0 instead.",/info
      endif
    endif
  endif else if junk1 eq 2 then begin
    if n_elements(profrng[*,0]) eq 1 then begin
      if profrng[0,m] ge minprofs then begin
        nseg=long(tnumrec/profrng[0,m])
        if nseg eq 0 then begin
          nseg=1
          if keyword_set(silent) eq 0 then begin
            message,"Profrange includes values greater than existing profiles.",/info
            print,'               Ignore and use total number of profiles.'
          endif
        endif
      endif else nseg=1
      if nseg eq 1 then profr=[0,tnumrec-1] $
      else begin
        profr=lonarr(2,nseg)
        for k=0L,nseg-1 do profr[*,k]=[k*profrng[0,m],(k+1)*profrng[0,m]-1L]
        if profr[1,nseg-1] ne tnumrec-1 then profr[1,nseg-1]=tnumrec-1
      endelse
    endif else begin
      nseg=n_elements(profrng[0,*])
      profr=profrng
    endelse
  endif else begin
    nseg=n_elements(profrng[0,*,m])
    profr=reform(profrng[*,*,m])
  endelse

  ; Arrange nswin

  junk1=n_elements(nswin)
  junk2=size(nswin,/n_dim)

  if nseg eq 1 then begin
    if junk2 eq 2 then begin
      message,'Invalid input for nswin',/info
      return
    endif else if junk2 eq 3 then junk1=n_elements(nswin[*,0,0])
    if junk1 eq 0 then begin
      nsw=nswinsz
      ppmnoav=0
    endif else begin
      if junk1 eq 1 then begin
        if junk2 eq 1 then nsw=[nswin,nswin] $
        else nsw=[nswin[0,0,m],nswin[0,0,m]]
        ppmnoav=0
      endif else if junk1 eq 2 then begin
        if junk2 eq 1 then begin
          nsw=[nswin[0],nswin[0]]
          ppmnoav=nswin[1]
        endif else begin
          nsw=[nswin[0,0,m],nswin[0,0,m]]
          ppmnoav=nswin[1,0,m]
        endelse
      endif else begin
        if junk2 eq 1 then begin
          nsw=[nswin[0],nswin[1]]
          ppmnoav=nswin[2]
        endif else begin
          nsw=[nswin[0,0,m],nswin[1,0,m]]
          ppmnoav=nswin[2,0,m]
        endelse
      endelse
    endelse
  endif else begin
    if junk1 eq 0 then begin
      nsw=replicarr(nswinsz,nseg)
      ppmnoav=fltarr(nseg)
    endif else begin
      if junk2 eq 2 then begin
        if n_elements(nswin[*,0]) ne 3 and n_elements(nswin[0,*]) ne nseg then $
        begin
          message,'Invalid input for nswin',/info
          return
        endif
        nsw=nswin[0:1,*]
        ppmnoav=nswin[2,*]
      endif else if junk2 eq 3 then begin
        if n_elements(nswin[0,*,0]) eq 1 then begin
          nsw=replicarr(nswin[0:1,0,m],nseg)
          ppmnoav=replicate(nswin[2,0,m],nseg)
        endif else begin
          if n_elements(nswin[*,0,0]) ne 3 and n_elements(nswin[0,*,0]) ne nseg $
          then begin
            message,'Invalid input for nswin',/info
            return
          endif
          nsw=nswin[0:1,*,m]
          ppmnoav=nswin[2,*,m]
        endelse
      endif else begin
        if junk1 eq 1 then begin
          nsw=[nswin,nswin]
          ppmnoav=0
        endif else if junk1 eq 2 then begin
          nsw=[nswin[0],nswin[0]]
          ppmnoav=nswin[1]
        endif else begin
          nsw=[nswin[0],nswin[1]]
          ppmnoav=nswin[2]
        endelse
        nsw=replicarr(nsw,nseg)
        ppmnoav=replicate(ppmnoav,nseg)
      endelse
    endelse
  endelse

  ;----------------------------------------------------------------------------
  ; Cycle creating the requested netcdf files
  ;----------------------------------------------------------------------------

  if keyword_set(silent) eq 0 then begin
    print,'=============================================='
    print,'Processing WCR file: '+wcrfile
    print,'=============================================='
    print,''
  endif

  for k=0L,nseg-1 do begin
    if keyword_set(silent) eq 0 then begin
      print,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      print,'Creating netcdf file: '+num2str(k+1)+' of '+num2str(nseg)
      print,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      print,''
      message,'********* Reading WCR raw data ...',/info
      print,'  beamsconfig=',num2str(bms[m]),', ppmnoav=',num2str(ppmnoav[k]),$
            ', profrange=',num2str(profr[0,k])+','+num2str(profr[1,k])
      print,'timesync=',num2str(timesync[m],fr=2),', nsw=',num2str(nsw[*,k],$
            fr=0,len=7)
            
    endif
    readwcrdata,profr[0,k],profr[1,k],timemode=3,nocross=nocross[m],/noleak, $
                ppmnoav=ppmnoav[k]+(ppmnoav[k] gt 0),big_endian=bgnd

    if keyword_set(silent) eq 0 then $
      message,'********* Generating a netcdf file ...',/info
    wcrcdl2nc,cdlf,reform(profr[*,k])<(tnumrec-1),ncfile=ncid,     $
              average=average,savecdl=savecdl,silent=silent,beams=bms[m], $
              indir=indir,outdir=ncdir
    if count ge maxcount then ncfile=[ncfile,ncid] $
    else ncfile[count]=ncid
    count=count+1

    if keyword_set(silent) eq 0 then $
      message,'********* Calculate/Write reflectivity and velocity ...',/info
    wcrwrite1nc,ncid,ncdir=ncdir,nswin=reform(nsw[*,k]),misval=misval, $
                revision=revision,silent=silent

    if acf ne '' then begin
      if keyword_set(silent) eq 0 then $
        message,'********* Calculate/Write corrected Doppler velocity ...',/info
      wcrwrite2nc,acf,ncid,acnames=acnames,gpsc=gpsc,rmcor=rmcor,unfold=unfold,$
                  timesync=timesync[m],revision=revision,silent=silent
    endif

    ncdf_close,ncid
    if keyword_set(silent) eq 0 then begin
      print,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      print,ncfile[count-1]+' completed.'
      print,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
    endif

  ;----------------------------------------------------------------------------
  ; End netcdf files cycle for the given WCR raw file
  ;----------------------------------------------------------------------------

  endfor

  ;----------------------------------------------------------------------------
  ; Close raw WCR data
  ;----------------------------------------------------------------------------

  closewcrf

;##############################################################################
; End cycle for WCR files
;##############################################################################

endfor

ncfile=reform(ncfile[0:(count-1)>0])

;------------------------------------------------------------------------------
; Done.
;------------------------------------------------------------------------------

 if keyword_set(silent) eq 0 then message,'wcrcreatenc is finished.',/info

end
