;$Id: wcrdd.pro,v 1.1 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRDD
;
; PURPOSE:
;       Performs dual-Doppler analysis using airborne WCR horizontal or
;       vertical dual-beam data and aircraft(AC) supplemental data
;
;       In addition to this program, the user should get familiar at least
;       with wcrdddata.pro and wcrddinit.pro routines, and wcrddprms.dat 
;       prototype input parameters file. All routines associated with wcrdd 
;       are named starting with 'wcrdd'
;
;       Currently there are no routines developed to aid in the visualization
;       of the Dual-Doppler analysis results in IDL.  A filter wcrdd2tplot.pro
;       is provided to allow the use of Tecplot visualization package for this
;       purpose.
;
; CATEGORY:
;       WCR wcrtools
;
; CALLING SEQUENCE:
;       wcrdd,starttime,stoptime,ddprmsf,beams,wcrprm,ddd,grddd,dddaux, $
;             acnames=acnames,acddd=acddd,wcrddd=wcrddd,windinfo=windinfo,$
;             help=help           
;
; INPUTS:
;       Starttime:long, start time in hhmmss. Enter -1 to start from the
;                 beginning of the file
;       Stoptime: long, stop time in hhmmss NOT including stop second radar
;                 profiles. Enter -1 to read to the end of the file.
;       Ddprmsf:  input,string, path/name to a data file containing dual- 
;                 Doppler grid and other processing parameters. If the
;                 file does not exist, it will be created interactively. 
;       Beams:    dual-antenna configuration:
;                  a) fltarr(3,2), [[hbeam],[vbeam]], unit vectors in AC coord.
;                     [AC along axis, right wing, down] for H and V beams or,
;                  b) use default beam unit vectors in AC coord. for each valid
;                     configuration (beam vectors can be seen in wcrbeam.pro):
;                     0 - N2UW KA dual-down antennas
;                         hbeam: down_fore 15" antenna (~30 deg down-fore)
;                         vbeam: down 18" antenna (near nadir)
;                     1 - N2UW KA dual-side antennas
;                         hbeam: side 12" antenna
;                         vbeam: side_fore 12" antenna (~35 deg side-fore)
;                     2 - NCAR C130 dual-down antennas (7/16/2001-7/30/2001)
;                         hbeam: down 12" antenna
;                         vbeam: down-aft 12" antenna
;
;       Wcrprm:   fltarr(2-7) or fltarr(12),parameters related to the WCR
;                 raw data, [hhcal,vvcal,thresh,timeoffset,ppselect,leakrg, 
;                            radmom]:
;
;                   hhcal: WCR h-channel/beam calibration constant
;                   vvcal: WCR v-channel/beam calibration constant
;                  thresh: Reflectivity threshold given in number of noise
;                          channel standard deviations (e.g., 2.5 means all
;                          data points below 2.5*std(noise) are marked as
;                          no_signal or missing using NaN. If 0 is entered
;                          no actual thresholding is done and only the points
;                          below the mean noise are marked with NaNs
;              timeoffset: WCR time correction to sync with AC INS(IRS)
;                          data in seconds  (default/not_defined 0.0); negative
;                          timeoffset means WCR time is ahead of AC time.
;                          For KA, GPS leap seconds correction is applied and
;                          must not be included in timeoffset.
;                ppselect: select desired pulse pairs (pp) for uncorrected
;                          Doppler velocity (default/not_defined 0):
;                      0 - PPMAG4 (HHVV/VVHH pulse sequence only, no choice)
;                          PPMAG6, uses h-/v-beam matching pp with max prf
;                     XY - PPMAG6, X, Y are the desired pp counted from left
;                          in the pulse sequence (e.g., for HHHVVV 12 is not
;                          valid but 14 or 25 are OK)
;                  leakrg: number of first range gates removed due to Tx 
;                          leakage (default/not_defined 0)
;                  radmom: input for applying antenna moment correction
;                     a)   fltarr(6),[hradmom,vradmom], antenna positions in AC
;                          coordinates (meters from INS) for H and V beams
;                          (if not defined and Beams is fltarr(3,2) antenna
;                           moment correction is NOT applied; IGNORED if
;                           beams=0 to 2) or
;                     b)   integer, code for applying antenna moment correction
;                      0 - antenna moment correction NOT applied
;                      1 - antenna moment correction applied
;                          Antenna positions in AC coord. (meters from INS)
;                          can be seen in wcracvelm.pro
;                          If radmom not defined: for UWKA not applied (0)
;                                                 for NCAR C130 applied(1)
;
; KEYWORD PARAMETERS:
;       ACNAMES:  input, string array, AC netcdf data varnames to be used:
;                 (if not defined N2UW or N130AR default var names are used)
;                 acnames[0] - AC true air speed in m/s
;                 acnames[1] - AC (phi)  roll [deg]
;                 acnames[2] - AC (theta)pitch [deg]
;                 acnames[3] - AC (psi)  true heading [deg]
;                 acnames[4] - AC (ug) east-west(EW) ground velocity [m/s]
;                 acnames[5] - AC (vg) north-south(NS) ground velocity [m/s]
;                 acnames[6] - AC (wg) vertical velocity [m/s]
;                 acnames[7] - AC (uw) East component of the wind [m/s]
;                 acnames[8] - AC (vw) North component of the wind [m/s]
;                 acnames[9] - AC (ww) vertical wind [m/s]
;                 acnames[10]- AC roll rate [rad/s] (use '' if N/A)
;                 acnames[11]- AC pitch rate [rad/s](use '' if N/A)
;                 acnames[12]- AC heading rate [rad/s] (use '' if N/A)
;                 acnames[13]- AC EW ground vel. GPS correction(use '' if N/A)
;                 acnames[14]- AC NS ground vel. GPS correction(use '' if N/A)
;                 acnames[15]- AC latitude [deg] (use '' if N/A)
;                 acnames[16]- AC longitude [deg](use '' if N/A)
;                 acnames[17]- AC pressure altitude [m](use '' if N/A)
;                 acnames[18]- AC geometric altitude [m](use '' if N/A)
;                 acnames[19]- AC sideslip angle[deg]
;
;       ACDDD:    output, structure containing AC data used for the dual-Doppler
;                 analysis (n,rg - radar profiles/rangegates to be processed):
;                 Acddd.beta   -> fltarr(n), AC sideslip angle [rad]
;                 Acddd.lat    -> fltarr(n), latitude position of the AC [deg]
;                 Acddd.lon    -> fltarr(n), longitude position of the AC [deg]
;                 Acddd.palt   -> fltarr(n), AC pressure altitude [m]
;                 Acddd.phi    -> fltarr(n), AC roll angle [rad]
;                 Acddd.psi    -> fltarr(n), AC true heading [rad from North]
;                 Acddd.ralt   -> fltarr(n), AC geometric altitude [m]
;                 Acddd.rarm   -> fltarr(3,2),[h_antenna,v_antenna], antenna
;                                             positions w.r.t. AC INS in AC
;                                             coodinate system
;                 Acddd.theta  -> fltarr(n), AC pitch angle [rad]
;                 Acddd.ug     -> fltarr(n), East-West AC ground velocity [m/s]
;                 Acddd.uw     -> fltarr(n), East wind velocity component [m/s]
;                 Acddd.vg     -> fltarr(n), North-South AC grnd velocity [m/s]
;                 Acddd.vw     -> fltarr(n), North wind velocity component [m/s]
;                 Acddd.wg     -> fltarr(n), AC vertical velocity [m/s]
;                 Acddd.ww     -> fltarr(n), vertical wind vel. component [m/s]
;
;       WCRDDD:   output,structure containing WCR data used for the dual-doppler
;                 analysis:
;                 Wcrddd.n     -> number of profiles from start to stop time
;                 Wcrddd.rg    -> number of radar range gates
;                 Wcrddd.time  -> dblarr(n),    WCR time stamps (UNIX seconds)
;                 Wcrddd.maxvel-> fltarr(2),    h,v-beam Nyquist velocities
;                 Wcrddd.beamh -> fltarr(n,3),  h-beam vector in Earth coordinates
;                 Wcrddd.beamv -> fltarr(n,3),  v-beam vector in Earth coordinates
;                 Wcrddd.dvh   -> fltarr(n,rg), AC motion corrected Doppler
;                                               velocity for h-beam in [m/s]
;                 Wcrddd.dvv   -> fltarr(n,rg), AC motion corrected Doppler
;                                               velocity for v-beam in [m/s]
;                 Wcrddd.rng   -> fltarr(rg),   radar beam range gates in meters
;                 Wcrddd.Zh    -> fltarr(n,rg), h-ch/beam reflectivity in dBz
;                 Wcrddd.Zhstd -> fltarr(n),    h-ch/beam noise St.Dev. in dBZ@1km
;                 Wcrddd.Zv    -> fltarr(n,rg), v-ch/beam reflectivity in dBz
;                 Wcrddd.Zvstd -> fltarr(n),    h-ch/beam noise St.Dev. in dBZ@1km
;
;       WINDINFO: output,fltarr(NXI,NET,3),(U,V,W) components in the GLOBAL UFRS 
;                 of the external wind.(from wnddd.wext, see wcrddinit.pro)
;       HELP:     shows this text
;
; OUTPUTS:
;       Ddd:    structure containing the dual-Doppler analysis data:
;               ddd.vel   -> fltarr(NXI,NET,3), dual-Doppler velocities;
;                            x,y,z vel.components for every cell in WBRRS
;                            (only the relevant two components are significant)
;               ddd.winfo -> fltarr(NXI,NET,3), external wind info for every
;                            grid cell in LOCAL WBRRS (WINDINFO,see above, is 
;                            the same but in GLOBAL UFRS)
;               ddd.z     -> fltarr(NXI,NET,3), averaged reflectivities for
;                            each grid cell, from h-channel(zh),v-channel(zv) 
;                            and their average(zz)
;
;       Dddaux: structure containing some auxiliary data resulting from the 
;               dual-Doppler processing(NXI/NET: along/across flight grid cells):
;               dddaux.ang_err->fltarr(NXI,NET),lesser angle between best-fit 
;                               plane & expected scanning plane in radians
;               dddaux.dtsf   ->fltarr(2,n,rg,3), h/v beam points coordinates in
;                               WBRRS;First dimension refers to [h-ch,v-ch]                                  
;               dddaux.h_ndx  ->ptrarr(NXI,NET),indices(in 1-D) of h-beam points 
;                               falling in the grid cells; h_ndx[i,j] is a 
;                               pointer to a lonarr(h_npts[i,j]); 
;               dddaux.h_npts ->intarr(NXI,NET),h-beam # of pts falling in every
;                               grid cell
;               dddaux.h_odx  ->ptrarr(NXI,NET),indices(in 1-D) of h-beam points
;                               falling out of their expected grid cells; h_odx[i,j]
;                               is a pointer to lonarr(h_npts[i,j]);(leg=1 only)  
;               dddaux.h_opts ->intarr(NXI,NET),h-beam # of pts falling out of
;                               their expected grid cells; (leg=1 only)
;               dddaux.v_ndx  ->ptrarr(NXI,NET),indices(in 1-D) of v-beam points 
;                               falling in the grid cells; v_ndx[i,j] is a 
;                               pointer to lonarr(v_npts[i,j]);
;               dddaux.v_npts ->intarr(NXI,NET),v-beam # of pts falling in every
;                               grid cell
;               dddaux.v_odx  ->ptrarr(NXI,NET),indices(in 1-D) of v-beam points
;                               falling out of their expected grid cells; v_odx[i,j]
;                               is a pointer to lonarr(v_npts[i,j]);(leg=1 only)
;               dddaux.v_opts ->intarr(NXI,NET),v-beam # of pts falling out of
;                               their expected grid cells; (leg=1 only)
;               dddaux.swthpts->intarr(NXI,NET),number of pts falling out of
;                               every grid cell due to limited swath
;
;       Grddd:  structure containing grid parameters and other processing
;               option parameters:
;               grddd.alt0 -> float, altitude in meters of the first grid cell
;                             corner point; can be used to locate the grid in 
;                             the 3-D space
;               grddd.crd1 -> fltarr(NXI),along track s-distance of cell
;                             center points [m]
;               grddd.crd2 -> fltarr(NET), normal to track n-distance of cell
;                             center points [m]
;               grddd.csize-> fltarr(2),[DHOR,DVER]: grid cell size along
;                             track(DHOR) and across track(DVER) directions.
;                             If HSR(along track spacing ratio,grddd.srat[0])
;                             and/or VSR(across track spacing ratio,grddd.srat[1])
;                             ne 1 then they represent the initial cell size
;               grddd.dims -> fltarr(2),[NXI, NET]: number of grid cells along
;                             the track and across the track
;               grddd.equid-> integer,criterium for construction of the grid
;                             for curtain-leg case(leg=1);See wcrddgrid1.pro
;                              0=s-curvilinear (along track),constant length arcs.
;                              1=constant length chords.
;                             Note: this parameter controls how the grid is
;                             constructed along-track direction. If 0 then
;                             curvilinear track distance (s-coord.) is divided
;                             in equal intervals, i.e. in equal-length arcs, or
;                             if a spacing ratio is assigned (HSR) in a stretching
;                             fashion arcs. If equid=1, then g0p (first row of
;                             grid points) are assigned along the track direction
;                             with constant length segments from point to point,
;                             or if HSR<>0 in a stretching fashion.
;               grddd.gp   -> fltarr(NXI+1,NET+1,3), coordinates  of the
;                             grid cell corner points[m], in GLOBAL
;                             WBRRS (Global WindBasedRotatedReferenceSystem:
;                             z-down, y-rotated accordingly to left-hand rule,
;                             x along track for leg type=0 along North otherwise,
;                             moving with a given advection velocity).
;               grddd.gtime-> dblarr(NXI),approx time associated with the
;                             grid center points [UNIX seconds]
;               grddd.leg  -> integer, type of flight leg
;                              0-straight leg; use anytime the AC track is
;                                considered straight
;                              1-curtain leg; non-straight flight legs; analysis
;                                is slower in this case
;               grddd.let  -> float,(input) length of grid normal to track [m]
;               grddd.pfct -> lonarr(2),first and last profile number to be
;                             scanned (used for dual-doppler); usually these are
;                             1st and last prof. read from wcr; if grddd.rst=1
;                             either 1st or last, depending on beam config may
;                             be different
;               grddd.ptrk -> integer, criterium for the points along the
;                             direction normal(perpendicular) to the track
;                              0-follow strictly vertical/horizontal direction
;                              1-follow the straight beam direction
;                             This parameter controls how the grid is built
;                             along ETA. If ptrk=0 then the points along ETA
;                             direction are constructed following a straight
;                             vertical path(VPDD) or straight horizontal (HBDD)
;                             If ptrk=1 then those points are constructed following
;                             the actual straight beam direction (down or side).
;               grddd.RM2  -> fltarr(3,3,NXI) for grddd.leg=1 and fltarr(3,3)
;                             for grddd.leg=0;  rotation matrix to go from UFRS
;                             (Unrotated FixedReference System: E,N,Up) to WBRRS
;               grddd.rst  -> integer(0/1),reset flag. If set to 1 it forces the
;                             grid origin to be colocated with the first point of
;                             the nadir/side beam, past which data from both,
;                             nadir/side and the slanted beams data are present.
;                             It attempts at getting rid of the first triangle.
;                             For C-130 cases it attempts at getting rid of the
;                             last triangle.
;               grddd.srat -> fltarr(2),[HSR,VSR],grid spacing ratios along the
;                             track and normal to track. The cell size along the
;                             2 directions is determined as:
;                              -along XI,  etasize[i+1]=etasize[i]*HSR
;                              -along ETA, etasize[j+1]=etasize[j]*VSR
;               grddd.wcrt -> [start time, stop time] of the WCR data segment 
;                             in UNIX seconds
;               grddd.swth -> grid out-of-the-plane swath [m]
;               grddd.wadv -> fltarr(3),[wadve,wadvn,wadvu], (East,North,UP)
;                             advection velocity used in making of the grid, 
;                             which become air-relative(WBRRS):
;                               [0.,0.,0.]    - WBRRS is not moving but coincides
;                                               with FRRS(FixedRotatedRefSys).
;                               [Ve,Vn,Vu]    - constant advection velocity
;                                               along E,N,UP Ref.Sys.(UFRS).
;                               [888,888,888] - WBRRS is moving with the AC
;                                               measured wind velocities w.r.t.
;                                               UFRS, point by point along track
;                               [777,777,777] - WBRRS is moving with measured
;                                               mean AC wind w.r.t. UFRS
;                               [666,666,666] - WBRRS is moving with a mean
;                                               flow calculated from the Doppler
;                                               field, w.r.t. UFRS
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       Plots graphs of AC Eulerian Angles and of AC ground track if requested.
;
;       Output dddaux contains several [nxi,net] pointer arrays to integer 
;       arrays of variable sizes.  If dddaux is to be deleted after running 
;       wcrdd the user should also clean all dereferenced pointer arrays by
;       running heap_gc routine after dddaux is deleted.
;
; PROCEDURE:
;
; NOTES AND ABBREVIATIONS:
;
;      UFRS: it is the UnrotatedFixedReferenceSystem. x,y,z along East,North,Up.
;            its origin is fixed with the first position of the aircraft
;            according to the portion of the file under investigation. It can be
;            considered anchored to the ground.
;      FRRS: it is the FixedRotatedReferenceSystem. x,y,z along East,North,Down
;            for the case leg=1, otherwise x is along the track, z points
;            vertically down and y points according to the lefthand rule for the
;            reference system. Its origin may be thought as anchored to the
;            first position of the radar straight beam antenna in the portion of
;            the file under investigation.
;      WBRS: it is the WindBasedReferenceSystem. It is a GLOBAL reference system
;            (as opposed to LOCAL for each single cell of the grid), with x,y,z
;            aligned with UFRS. Its origin moves with the advection velocity (3
;            components) of the wind as dictated by grddd.wadv.
;      WBRRS(GLOBAL): it is the GLOBAL WindBasedRotatedReferenceSystem. In case
;            of leg=0, the x is aligned with the AC track, z points downward and
;            y to the right of the track. For leg=1, x is along East, z points
;            downward and y points according to the left hand rule. Its origin
;            moves with the advection velocity (3 components) of the wind as
;            dictated by grddd.wadv.
;      WBRRS(LOCAL): it is the LOCAL WindBasedRotatedReferenceSystem. It is used
;            only in case of leg=1; there is one LOCAL WBRRS per grid column.
;            The x is aligned along two consecutive(along track) grid corner
;            points; if ptrk=0 then z is on a vertical plane pointing down and y on
;            a horizontal plane to the right of the track; if ptrk=1 and VDD
;            enabled then z is determined by an averge direction obtained
;            connecting 2 points (O & Q) belonging to the 1st and 2nd rows of
;            the grid as follows:O=(g0p[i,*]+g0p[i+1,*])/2,
;                                Q=(gp[i,1,*]+gp[i+1,1,*])/2;
;            y is then fixed by the lefthand rule. If ptrk=1 and HBDD enabled
;            then y is determined in an analogous way as for z with VDD, and z
;            determined by the lefthand rule.
;            Its origin is associated with O, but in the process of assessing
;            whether a point belongs to a cell it is associated with the
;            upper-left corner point of the cell for simplicity. For the way the
;            g0p (and gp) points are constructed, the origins of these local
;            ref. sys. move with the advection velocity (3 components) of the
;            wind as dictated by grddd.wadv.
;    XI,ETA: leading directions of the grid (along and normal-to-track)
;            row,column: by row we mean a series of consecutive points along XI.
;                        by column - along ETA.
;   NXI,NET: number of grid center points along XI and ETA
;   HSR,VSR: spacing ratios along the XI and ETA directions.
;       IDW: InverseDistanceWeighting, used in weighting info coming from various
;            points in each cell based on their distance from the center of the cell.
;       BSW: BeamSkewnessWeighting,  used in weighting info coming from various
;            points in each cell based on their associated beam-unit-vector
;            component normal to the intended plane of scan.
;
; EXAMPLE:
;
;-
; MODIFICATION HISTORY:
;       Written by: Rick Damiani and Samuel Haimov, September 2003
;       S.H., Jan 04 - modified the calls to wcrddwext and few minor changes
;       S.H., Mar 04 - added wcrt field to grddd (in the help)
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.
;

pro wcrdd, startt,stopt,ddprmsf,beams,wcrprm,ddd,grddd,dddaux,acnames=acnames,$
           acddd=acddd,wcrddd=wcrddd,windinfo=winds_ext,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrdd'
    return
  endif

  if (n_params() lt 6) then begin
    message,"Usage: starttime,stoptime,ddprmsf,beams,wcrprm,ddd,grddd,$",/info
    print,"                   dddaux,acnames=acnames,windinfo=windinfo,$"
    print,"                   acddd=acddd,wcrddd=wcrddd,help=help"
    return
  endif

;..........................................................................
; Try to clean up some garbage (doesn't work well but will keep it for now)

  if n_elements(dddaux) gt 0 then begin
    undefine,dddaux
    heap_gc ;this cleans all the dumped unreferenced pointer material
  endif

;..........................................................................
; Determine dual-beam configuration (0,2: dual-down or 1: dual-side)

  if n_elements(beams) eq 1 then beamopt=beams $
  else begin 
  ; Determine beamopt from sum of the vects in AC Ref.Sys.
    junk1=max(abs(total(beams,2)),beamopt)
    if beamopt ne 1 then beamopt=0+2*(beams[0,1] lt -0.2)
  endelse 

;..........................................................................
; Read/create dual-Doppler grid&processing parameters file ddprmsf

  wcrddinit,ddprmsf,fstruct,grddd,pcddd,wnddd

  if pcddd.unfold eq 2 then begin
    unfold=grddd.wadv 
    junk1=where(unfold eq 666.)
    if junk1[0] ne -1 then unfold[junk1]=777.
  endif else unfold=pcddd.unfold ;unfolding flag

;..........................................................................
; Read WCR and AC data; AC data is synched with wcr time

  wcrdddata,fstruct.wcrinf,fstruct.acinf,startt,stopt,beams,wcrprm,$
            wcrddd,acddd,acnames=acnames,unfold=unfold,/closeac,/closewcr
  message,'WCR and AC data loaded;',/info

;..........................................................................
; Determine mean [U,V,W] wind measured from the AC in UFRS (East, North,UP)
; This info may be used for the advection wind, for external info on the winds
; (used in SVDC procedure), and for the final velocity display in air-relative 
; reference coord. system

  ind=where(grddd.wadv eq 777.)
  if ind[0] ne -1 then $
  ; use avg AC measured winds for advection if requested
    grddd.wadv[ind]=([mean(acddd.uw),mean(acddd.vw),mean(acddd.ww)])[ind] $
  else begin 
  ; use WCR measured avg Doppler for advection if requested
    ind=where(grddd.wadv eq 666.)
    if ind[0] ne -1 then begin

    ; average measured Doppler along the beam vectors; minus in dvhm,dvvm 
    ; accounts for the radar sign convention
      junk2= make_array(wcrddd.rg,value=1.)

      junk1=-[mean(wcrddd.dvh*(reform(wcrddd.beamh[*,0])#junk2),/nan),$
              mean(wcrddd.dvh*(reform(wcrddd.beamh[*,1])#junk2),/nan),$
              mean(wcrddd.dvh*(reform(wcrddd.beamh[*,2])#junk2),/nan) ]

      junk2=-[mean(wcrddd.dvv*(reform(wcrddd.beamv[*,0])#junk2),/nan),$
              mean(wcrddd.dvv*(reform(wcrddd.beamv[*,1])#junk2),/nan),$
              mean(wcrddd.dvv*(reform(wcrddd.beamv[*,2])#junk2),/nan) ]

    ; retrieve the mean flow in UFRS
    ; use AC measured mean wind for the missing velocity component

      LSV =[norm(junk1),norm(junk2)]       ; avg vels along h/v beams   
      LSM =[[junk1/LSV[0]],[junk2/LSV[1]]] ; [3,2] vel.components of h/v beams
      grddd.wadv[ind]=(wcrdddv(LSM,LSV, $
                      v_ext=[mean(acddd.uw),mean(acddd.vw),mean(acddd.ww)]))[ind]
    endif
  endelse

;..........................................................................
; Construct the grid and calculate the dual-Doppler

  ; *****************  Process Straight Leg (grddd.leg=0) *****************
  if grddd.leg eq 0 then begin

    wcrddgrid0,beamopt,acddd,wcrddd,grddd,dtsf,acstats,strk, $
               acdisp=pcddd.plts,ddbreak=pcddd.inter

    message,'Straight leg GRID completed;',/info

    if wnddd.wflag ne 0 then begin ; get external winds info
      wcrddwext,beamopt,acddd,wcrddd,grddd,wnddd,strk,grddd.crd1,$
                sndfile=fstruct.sndf
      winds_ext=wnddd.wext ;(NXI,NET,3) winds(N-E-UP,Global WBRS=UFRS)
      message,'External winds determined;',/info
    endif else begin
      winds_ext=0.
      message,'External winds info not used;',/info
    endelse

    wcrddproc0,beamopt,dtsf,wcrddd,grddd,pcddd,ddd,dddaux,pcstats,$ 
               winds_ext=winds_ext
    message,'Straight leg Dual-Doppler processed.',/info

  ; ***************** Process Curtain Leg (grddd.leg=1) *******************
  endif else begin

    wcrddgrid2,Dtsf,Acddd,Grddd,Wcrddd,beamopt,pcddd.plts,$
               strk,acstats,strk_g,interactive=pcddd.inter
    message,'Curtain leg GRID completed;',/info

    if wnddd.wflag ne 0 then begin ; get external winds info
      wcrddwext,beamopt,acddd,wcrddd,grddd,wnddd,strk,strk_g,sndfile=fstruct.sndf
      winds_ext=wnddd.wext ;(NXI,NET,3) winds(N-E-UP,Global WBRS=UFRS)
      message,'External winds determined;',/info
    endif else begin
      winds_ext=0.
      message,'External winds info not used;',/info
    endelse

    wcrddproc2,dtsf,wcrddd,grddd,pcddd,beamopt,strk,acddd.theta,acddd.phi, $
               acddd.beta,winds_ext=winds_ext,ddd,pcstats,dddaux
    message,'Curtain leg Dual-Doppler processed;',/info

  endelse

;..........................................................................
; Generate the echo file

  wcrddecho,fstruct,grddd,pcddd,wnddd,acstats,pcstats
  message,fstruct.echof+' saved;',/info
  print,""
  message,'Done.',/info

end
