;$Id
;+
; NAME:
;      WCRDD2TPLOT
;     
; PURPOSE:
;      This routine is part of the WCR airborne Dual-Doppler analysis package
;      (wcrdd.pro). Writes an ascii file with selected Dual-Doppler outputs
;      suitable for 2-D/3-D visualization by TecPlot (preplot utility).
;
; CATEGORY:
;      WCR wcrtools
;
; CALLING SEQUENCE:
;      wcrdd2tplot,tecplotf,ddd,grddd,dddaux,$
;                   alt=alt,twobeam=twobeam,planar=planar,help=help
;
; INPUTS:
;      Tecplotf: string, path/filename for the ASCII output file
;      Ddd:      structure containing the dual-Doppler analysis data
;                (see wcrdd.pro for more help)
;      Grddd:    structure containing grid parameters and other processing
;                option parameters (see wcrdd.pro for more help)
;      Dddaux:   structure containing some auxiliary data resulting from the 
;                (see wcrdd.pro for more help)
;
; KEYWORD PARAMETERS:
;      ALT:     when set, the grid vertical coordinate is computed by
;               adding the grddd.alt0 value as 1st corner point altitude.
;      TWOBEAM: when set, assigns NaNs to the velocities associated with 
;               cells containing data from one beam only (if any).
;      PLANAR:  input,integer, sets the visualization layout
;               0 - visualization in 3-D (default)
;               1 - 2-D horizontal plane
;               2 - 2-D vertical plane
;      HELP:    shows this text
;
; OUTPUTS:
;  
; SIDE EFFECTS:
;      Write an ascii file in a directory and name given by tecplotf
;
; NOTE:
;      A note about the 3-D/2-D axes for VPDD and HBDD.
;      For 3-D layout the axes x-y-z are East-North-Up
;      For 2-D VPDD: x-z are Along_track-Up (if alt is set); VW=W is positive up
;                    or Along_track-Across_track(alt not set); VW=W is positive up
;      For 2-D HBDD: x-y are Along_track-Across_track; VW=V is positive along y
;                    and y-axis(to the aircraft right) must be reversed in tecplot
;
; EXAMPLE:
;
;      tecplotf='data/20030826.182022.ddat'
;      wcrdd2tplot,tecplotf,ddd,grddd,dddaux,planar=2,/alt
;
;      Run TecPlot utility from OS prompt to prepare final input file 
;      <prompt> preplot -d 20030826.182022.ddat 
;      preplot will output a file 20030826.182022.plt
;      then run tecplot and use the menus to load 20030826.182022.plt
;-
; MODIFICATION HISTORY:
;       Written by: Rick Damiani and Samuel Haimov, March 2004
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.
;

pro wcrdd2tplot,tecplotf,ddd,grddd,dddaux,$
                twobeam=twobeam,planar=planar,alt=alt,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrdd2tplot'
    return
  endif

  if (n_params() lt 4) then begin
    message,"Usage: wcrdd2tplot,tecplotf,ddd,grddd,dddaux, $",/info
    print,"                     twobeam=twobeam,planar=planar,alt=alt,help=help"
    return
  endif

  if n_elements(planar) eq 0 then planar=0

; Mark grid cells with one beam vel only as empty if requested

  if keyword_set(twobeam) then begin
    ind=where([[[dddaux.h_npts]],[[dddaux.h_npts]],[[dddaux.h_npts]]] eq 0)
    ddd.vel[ind]=!values.f_nan
    ind=where([[[dddaux.v_npts]],[[dddaux.v_npts]],[[dddaux.v_npts]]] eq 0)
    ddd.vel[ind]=!values.f_nan
  endif

; Missing value for TecPlot is -32768. This will replace the NaNs in ddd

  misval=-32768.

; Prepare time stamps in hhmmss

  time=replicarr(long(sec2time(grddd.gtime mod 86400.)),grddd.dims[1])

; ------------------------------------------------------------------------------
; Prepare data for 2-D visualization

if planar gt 0 then begin

  ; ...........................................................................
  ; Get the center points (we already have them for a plane) and make axes

    X= replicarr(grddd.crd1,grddd.dims[1])
    YZ=replicarr(grddd.crd2,grddd.dims[0],/dim)
    
  ; ...........................................................................
  ; Add gridd altitude if asked and VPDD

    if keyword_set(alt) and planar eq 2 then YZ=grddd.alt0-YZ

  ; ...........................................................................
  ; Extract the dual-doppler velocity components (in WBRRS) from ddd.vel
  ; and calculate their wind-relative values
  
    if grddd.leg eq 0 then begin  
    ; extract the vels and make the vertical component to be up for VPDD 
      junk1=(3-2*planar) ; define the sign if V or W
      U    =ddd.vel[*,*,0]
      Uwnd =ddd.winfo[*,*,0]
      Urel =U-Uwnd
      VW   =junk1*ddd.vel[*,*,planar]
      VWwnd=junk1*ddd.winfo[*,*,planar]
      VWrel=VW-VWwnd
    endif else begin
    ; transfer results into a local vertical plane (beamopt ne 1) containing 
    ; the local WBRRS-x_axis or into a local horizontal plane (beamopt eq 0). 
    ; Note that z in this new reference system is up, so we do not need to
    ; change sign of VW.

      U    =fltarr(grddd.dims[0],grddd.dims[1])
      Urel =fltarr(grddd.dims[0],grddd.dims[1])
      VW   =fltarr(grddd.dims[0],grddd.dims[1])
      VWrel=fltarr(grddd.dims[0],grddd.dims[1])
      Uwnd =fltarr(grddd.dims[0],grddd.dims[1])
      VWwnd=fltarr(grddd.dims[0],grddd.dims[1])

    ; calculate the transformation matrix RM3 to convert from WBRRS to a 
    ; local side or up plane reference system (to be used for visualization) 

      junk1=transpose(reform(sqrt(1.-grddd.rm2[2,0,*]^2.)))
      junk2=reform(grddd.rm2[2,*,*])
      junk3=reform(grddd.rm2[2,1:2,*])/replicarr(junk1,2,/dim)
      junk4=fltarr(1,grddd.dims[0])

      RM3=[[[ junk1,                  junk4,      junk2[0,*]]],$
           [[-junk2[0,*]*junk3[0,*], -junk3[1,*], junk2[1,*]]],$
           [[-junk2[0,*]*junk3[1,*],  junk3[0,*], junk2[2,*]]] ] 
      RM3=transpose(RM3,[2,0,1])
    
      junk1=ddd.vel-ddd.winfo        ;relative velocity in the local WBRRS
      for l=0,grddd.dims[1]-1 do $
        for k=0,grddd.dims[0]-1 do begin
          junk2=RM3[*,*,k] ## reform(ddd.vel[k,l,*])  
          junk3=RM3[*,*,k] ## reform(junk1[k,l,*])
          junk4=RM3[*,*,k] ## reform(ddd.winfo[k,l,*])

          U[k,l]    =junk2[0]     
          Urel[k,l] =junk3[0] 

          VW[k,l]   =junk2[planar]
          VWrel[k,l]=junk3[planar]

          Uwnd[k,l]= junk4[0]
          VWwnd[k,l]=junk4[planar]
        endfor
    endelse

  ; ...........................................................................
  ; Variables to be recorded in tecplotf

    junk1=' "Zh" "Zv" "Zavg" "U" "Urel"'
    junk2=' "V" "Vrel" "Uwnd" "Vwnd"' 
    junk3=' "W" "Wrel" "Uwnd" "Wwnd"' 
    junk4=' "ANGERR" "HNPTS" "VNPTS" '
    case planar*10+grddd.leg of
      10: begin
          vstr='VARIABLES= "X_along_track" "Y_horizontal_range"'+junk1+junk2+  $
               junk4+' "TIME"'
          var=[[[X]],[[YZ]],[[ddd.z[*,*,0]]],[[ddd.z[*,*,1]]],[[ddd.z[*,*,2]]],$
               [[U]],[[Urel]],[[VW]],[[VWrel]],[[Uwnd]],[[VWwnd]],             $
               [[dddaux.ang_err*!radeg]],[[dddaux.h_npts]],[[dddaux.v_npts]],  $
               [[time]]]
          end
      11: begin
          vstr='VARIABLES= "X_along_track" "Y_horizontal_range"'+junk1+junk2+  $
               junk4+' "HOPTS" "VOPTS" "TIME"'
          var=[[[X]],[[YZ]],[[ddd.z[*,*,0]]],[[ddd.z[*,*,1]]],[[ddd.z[*,*,2]]],$
               [[U]],[[Urel]],[[VW]],[[VWrel]],[[Uwnd]],[[VWwnd]],             $
               [[dddaux.ang_err*!radeg]],[[dddaux.h_npts]],[[dddaux.v_npts]],  $
               [[dddaux.h_opts]],[[dddaux.v_opts]],[[time]]]
          end
      20: begin
          if keyword_set(alt) then $
            vstr='VARIABLES="X_along_track" "Z_up"'+junk1+junk3+junk4+' "TIME"'$
          else                                                                 $
            vstr='VARIABLES="X_along_track" "Z_vertical_range"'+junk1+junk3+   $
                 junk4+' "TIME"'

          var=[[[X]],[[YZ]],[[ddd.z[*,*,0]]],[[ddd.z[*,*,1]]],[[ddd.z[*,*,2]]],$
               [[U]],[[Urel]],[[VW]],[[VWrel]],[[Uwnd]],[[VWwnd]],             $
               [[dddaux.ang_err*!radeg]],[[dddaux.h_npts]],[[dddaux.v_npts]],  $
               [[time]]]
          end
      21: begin
          if keyword_set(alt) then $
            vstr='VARIABLES= "X_along_track" "Z_up"'+junk1+junk3+junk4+        $
                 ' "HOPTS" "VOPTS" "TIME"'                                     $
          else                                                                 $
            vstr='VARIABLES= "X_along_track" "Z_vertical_range"'+junk1+junk3+  $
                 junk4+' "HOPTS" "VOPTS" "TIME"'

          var=[[[X]],[[YZ]],[[ddd.z[*,*,0]]],[[ddd.z[*,*,1]]],[[ddd.z[*,*,2]]],$
               [[U]],[[Urel]],[[VW]],[[VWrel]],[[Uwnd]],[[VWwnd]],             $
               [[dddaux.ang_err*!radeg]],[[dddaux.h_npts]],[[dddaux.v_npts]],  $
               [[dddaux.h_opts]],[[dddaux.v_opts]],[[time]]]
          end
    else: message,'Wrong planar or grddd.leg value.'
    endcase

endif else begin
; ------------------------------------------------------------------------------
; Prepare data for 3-D visualization

  ; ...........................................................................
  ; Find the 3-D grid cell center coordinates in WBRRS and convert to WBRS

  ; This is what is passed to tecplot: cell centroids should be the C.G. of 
  ; the quadrilaterals since most of the times they are squares (or rectangles) 
  ; they coincide with the C.G. of the quadrilateral corner points. Furthermore,
  ; for the case of curtain leg and ptrk=1(following the straight beam in the 
  ; perp.to track dir) these quadrilaterals may be twisted in space. Finding the
  ; C.G. from the C.G. of 4 triangles inside the quadrilateral is complicated. 
  ; Using the C.G. of the points is a valid alternative which gives good 
  ; approximation within the dimension of the cell, especially if assume that
  ; the grid is not too distorted.

  ; Interpolate to find mid(centroid) points for every component

    cp=fltarr(grddd.dims[0],grddd.dims[1],3) ;[m] centroid points coordinates

    expand,grddd.gp[*,*,0],2*grddd.dims[0]+1,2*grddd.dims[1]+1,junk
    cp[*,*,0]=junk[1:2*grddd.dims[0]-1:2,1:2*grddd.dims[1]-1:2]

    expand,grddd.gp[*,*,1],2*grddd.dims[0]+1,2*grddd.dims[1]+1,junk
    cp[*,*,1]=junk[1:2*grddd.dims[0]-1:2,1:2*grddd.dims[1]-1:2]

    expand,grddd.gp[*,*,2],2*grddd.dims[0]+1,2*grddd.dims[1]+1,junk
    cp[*,*,2]=junk[1:2*grddd.dims[0]-1:2,1:2*grddd.dims[1]-1:2]

    ; transform to Global WBRS (Z is up)
    if grddd.leg then begin
      cp[*,*,1]=-cp[*,*,1]
      cp[*,*,2]=-cp[*,*,2]
    endif else begin
      for l=0,grddd.dims[1]-1 do $
        for k=0,grddd.dims[0]-1 do $
          cp[k,l,*]=reform(cp[k,l,*])##grddd.rm2
    endelse

    ; add grid altitude if asked (VPDD or HBDD since it is in 3-D)
    if keyword_set(alt) then cp[*,*,2]=cp[*,*,2]+grddd.alt0

  ; ...........................................................................
  ; Extract the dual-doppler velocity components (in WBRRS) from ddd.vel
  ; and calculate their wind-relative values

    UVW   =ddd.vel
    UVWwnd=ddd.winfo
    UVWrel=UVW-UVWwnd

  ; ...........................................................................
  ; Find grid cells with valid data only 

    ind=where(finite(UVW) eq 1)
    if ind[0] ne -1 then ind=array_indices(UVW,ind)
    
  ; ...........................................................................
  ; Convert velocity to WBRS

    if grddd.leg eq 0 then begin
    ; transform velocity from WBRRS to WBRS: 
    ; transpose(grddd.RM2) ## reform(ddd.vel[k,l,*]) 
     for k=0,n_elements(ind[0,*])-1 do begin
      UVW[ind[0,k],ind[1,k],*]   =reform(UVW[ind[0,k],ind[1,k],*])   ##grddd.RM2
      UVWrel[ind[0,k],ind[1,k],*]=reform(UVWrel[ind[0,k],ind[1,k],*])##grddd.RM2
      UVWwnd[ind[0,k],ind[1,k],*]=reform(UVWwnd[ind[0,k],ind[1,k],*])##grddd.RM2
     endfor
    endif else begin     ;leg=1
     for k=0,n_elements(ind[0,*])-1 do begin
      UVW[ind[0,k],ind[1,k],*]   =reform(UVW[ind[0,k],ind[1,k],*])   ##grddd.RM2[*,*,ind[0,k]]
      UVWrel[ind[0,k],ind[1,k],*]=reform(UVWrel[ind[0,k],ind[1,k],*])##grddd.RM2[*,*,ind[0,k]]
      UVWwnd[ind[0,k],ind[1,k],*]=reform(UVWwnd[ind[0,k],ind[1,k],*])##grddd.RM2[*,*,ind[0,k]]
     endfor
    endelse

  ; ...........................................................................
  ; Variables to be recorded in tecplotf

    vstr='VARIABLES= "East" "North" "Up" "Zh" "Zv" "Zavg" "U" "Urel" "V" "Vrel"'+$
         ' "W" "Wrel" "Uwnd" "Vwnd" "Wwnd" "ANGERR" "HNPTS" "VNPTS"'
    if grddd.leg then begin
      vstr=vstr+' "HOPTS" "VOPTS" "TIME"'
      var=[[[cp[*,*,0]]],[[cp[*,*,1]]],[[cp[*,*,2]]],[[ddd.z[*,*,0]]],        $
           [[ddd.z[*,*,1]]],[[ddd.z[*,*,2]]],[[UVW[*,*,0]]],[[UVWrel[*,*,0]]],$
           [[UVW[*,*,1]]],[[UVWrel[*,*,1]]],[[UVW[*,*,2]]],[[UVWrel[*,*,2]]], $
           [[UVWwnd[*,*,0]]],[[UVWwnd[*,*,1]]],[[UVWwnd[*,*,2]]],             $
           [[dddaux.ang_err*!radeg]],[[dddaux.h_npts]],[[dddaux.v_npts]],     $
           [[dddaux.h_opts]],[[dddaux.v_opts]],[[time]]]
    endif else begin
      vstr=vstr+' "TIME"'
      var=[[[cp[*,*,0]]],[[cp[*,*,1]]],[[cp[*,*,2]]],[[ddd.z[*,*,0]]],        $
           [[ddd.z[*,*,1]]],[[ddd.z[*,*,2]]],[[UVW[*,*,0]]],[[UVWrel[*,*,0]]],$
           [[UVW[*,*,1]]],[[UVWrel[*,*,1]]],[[UVW[*,*,2]]],[[UVWrel[*,*,2]]], $
           [[UVWwnd[*,*,0]]],[[UVWwnd[*,*,1]]],[[UVWwnd[*,*,2]]],             $
           [[dddaux.ang_err*!radeg]],[[dddaux.h_npts]],[[dddaux.v_npts]],[[time]]]
    endelse

  endelse  ; end 3_D

; ------------------------------------------------------------------------------
; Transpose var to make it columnwise in the file and Replace NaN with misval

  var=transpose(var,[2,0,1])
  nvar=string(n_elements(var[*,0,0])-1)

  ind=where(finite(var) eq 0)
  if ind[0] ne -1 then var[ind]=misval

; ------------------------------------------------------------------------------
; Write the tecplot ascii file

  junk1=strmid(js2unixtime(grddd.wcrt[0]),11,8)
  junk2=js2unixtime(grddd.wcrt[1])
  junk2=strmid(junk2,11,9)+strmid(junk2,8,3)+strmid(junk2,4,4)+strmid(junk2,20)

  tstr='TITLE="'+junk1+'-'+junk2+'"'

  zstr='ZONE T="GRID CENTER POINTS", DATAPACKING=POINT I='+num2str(grddd.dims[0])+$
       ', J='+num2str(grddd.dims[1])

  get_lun, lun1
  openw,lun1,tecplotf

  printf,lun1,tstr
  printf,lun1,vstr
  printf,lun1,zstr
  printf,lun1,format='('+nvar+'f10.2,i8)',var 

  close, lun1
  free_lun, lun1
end
