;$Id: wcrdddata.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRDDDATA
;
; PURPOSE:
;       Prepare input for WCR airborne dual-doppler analysis (wcrdd.pro) 
;       WCR raw data file and an aircraft (AC) data file(netcdf, N2UW KingAir 
;       or NCAR C130) are used.
;
; CATEGORY:
;       WCR wcrtools
;
; CALLING SEQUENCE:
;       wcrdddata,wcrfile,acfile,starttime,stoptime,beams,wcrprm,wcrddd, $
;                 acddd,acid=acid,acnames=acnames,closeac=closeac, $
;                 closewcr=closewcr,unfold=unfold,help=help
;
; INPUTS:
;       Wcrfile:   string, path/name of a raw WCR data file
;       Acfile:    string, path/name of the aircraft(AC) data netcdf file
;                  Note: should have at least 10Hz INS data (25 Hz optimal)
;       Starttime: long, start time in hhmmss. Enter -1 to start from the
;                  beginning of the file
;       Stoptime:  long, stop time in hhmmss NOT including stop second radar
;                  profiles. Enter -1 to read to the end of the file.
;       Beams:     dual-antenna configuration:
;                   a) fltarr(3,2), [[hbeam],[vbeam]], unit vectors in AC coord.
;                      [AC along axis, right wing, down] for H and V beams or,
;                   b) use default beam unit vectors in AC coord. for each valid
;                      configuration (beam vectors can be seen in wcrbeam.pro):
;                      0 - N2UW KA dual-down antennas
;                          hbeam: down_fore 15" antenna (~30 deg down-fore)
;                          vbeam: down 18" antenna (near nadir)
;                      1 - N2UW KA dual-side antennas
;                          hbeam: side 12" antenna
;                          vbeam: side_fore 12" antenna (~35 deg side-fore)
;                      2 - NCAR C130 dual-down antennas (7/16/2001-7/30/2001)
;                          hbeam: down 12" antenna
;                          vbeam: down-aft 12" antenna
;
;       Wcrprm:    fltarr(2-6),fltarr(11),parameters related to WCR
;                  raw data, [hhcal,vvcal,thresh,timeoffset,ppselect,leakrg,
;                             radmom]:
;
;                  hhcal:  WCR h-channel/beam calibration constant
;                  vvcal:  WCR v-channel/beam calibration constant
;                  thresh: Reflectivity threshold given in number of noise
;                          channel standard deviations (e.g., 2.5 means all
;                          data points below 2.5*std(noise) are marked as
;                          no_signal or missing using NaN. If 0 (default)
;                          no actual thresholding is done and only the points
;                          below the mean noise are marked with NaNs
;                  timeoffset: WCR time correction to sync with AC INS(IRS)
;                          data in seconds  (default/not_defined 0.0); negative
;                          timeoffset means WCR time is ahead of AC time.
;                          For KA GPS leap seconds correction is applied and
;                          must not be included in timeoffset.
;                  ppselect: select desired pulse pairs (pp) for uncorrected
;                            Doppler velocity (default/not_defined 0):
;                     0  - PPMAG4 (HHVV/VVHH only valid pulse sequence, no choice)
;                          PPMAG6, uses h-/v-beam matching pp with max prf
;                     XY - PPMAG6, X, Y are the desired pp counted from left
;                          in the pulse sequence (e.g., for HHHVVV 12 is not
;                          valid but 14 or 25 are OK)
;                   leakrg: number of range gates removed due to Tx leakage
;                           (default/not_defined 0)
;                   radmom: input for applying antenna moment correction
;                     a) fltarr(6),[hradmom,vradmom], antenna positions in AC
;                            coordinates (meters from INS) for H and V beams
;                            (if not defined and beams is fltarr(3,2) antenna
;                            moment correction is NOT applied;
;                            IGNORED if beams=0 to 2) or
;                     b) integer, code for applying antenna moment corrections
;                        0 - antenna moment correction NOT applied
;                        1 - antenna moment correction applied
;                            Antenna positions in AC coord. (meters from INS)
;                            can be seen in wcracvelm.pro
;                        If not defined: for UWKA not applied (0)
;                                        for NCAR C130 applied(1)
;
; KEYWORD PARAMETERS:
;       ACID:     output, Acfile id if left open (CLOSEAC not set)
;       ACNAMES:  input, string array, AC netcdf varnames to be used
;                  (if not defined N2UW or N130AR default var names are used)
;                  acnames[0] - AC true air speed in m/s
;                  acnames[1] - AC roll [deg]
;                  acnames[2] - AC pitch [deg]
;                  acnames[3] - AC true heading [deg]
;                  acnames[4] - AC east-west(EW) ground velocity [m/s]
;                  acnames[5] - AC north-south(NS) ground velocity [m/s]
;                  acnames[6] - AC vertical velocity [m/s]
;                  acnames[7] - AC East component of the wind [m/s]
;                  acnames[8] - AC North component of the wind [m/s]
;                  acnames[9] - AC vertical wind [m/s]
;                  acnames[10]- AC roll rate [rad/s] (use '' if N/A)
;                  acnames[11]- AC pitch rate [rad/s](use '' if N/A)
;                  acnames[12]- AC heading rate [rad/s] (use '' if N/A)
;                  acnames[13]- AC EW ground vel. GPS correction(use '' if N/A)
;                  acnames[14]- AC NS ground vel. GPS correction(use '' if N/A)
;                  ncnames[15]- AC latitude [deg] (use '' if N/A)
;                  ncnames[16]- AC longitude [deg](use '' if N/A)
;                  ncnames[17]- AC pressure altitude [m](use '' if N/A)
;                  ncnames[18]- AC geometric altitude [m](use '' if N/A)
;                  ncnames[19]- AC sideslip angle[deg]
;
;       CLOSEAC:  input, when set AC netcdf file will be closed.
;       CLOSEWCR: input, when set WCR raw data file will be closed.
;       UNFOLD:   input, Doppler velocity unfolding procedure (see notes)
;                 0          - no unfolding is performed (default)
;                 1          - use AC measured winds profile by profile
;                 fltarr(3)  - [u,v,w], use the given wind vector
;                              u,v,w are allowed the following values:
;                              float: actual wind value to be used in m/s
;                                888: use AC measured values for all 
;                                     profiles of this component
;                                777: use mean of AC measured values from 
;                                     all profiles of this component
;                 fltarr(n,3)- use different wind vector for each profile
;       HELP:     shows this text
;
; OUTPUTS:
;       Wcrddd:   structure containing WCR data needed for dual-doppler analysis
;                 Wcrddd.n     -> number of profiles from start to stop time
;                 Wcrddd.rg    -> number of radar range gates
;                 Wcrddd.time  -> dblarr(n),    WCR time stamps (UNIX seconds)
;                 Wcrddd.maxvel-> fltarr(2),    h,v-beam Nyquist velocities
;                 Wcrddd.beamh -> fltarr(n,3),  h-beam vector in Earth coordinates
;                 Wcrddd.beamv -> fltarr(n,3),  v-beam vector in Earth coordinates
;                 Wcrddd.dvh   -> fltarr(n,rg), AC motion corrected Doppler
;                                               velocity for h-beam in [m/s]
;                 Wcrddd.dvv   -> fltarr(n,rg), AC motion corrected Doppler
;                                               velocity for v-beam in [m/s]
;                 Wcrddd.rng   -> fltarr(rg),   radar beam range in meters
;                 Wcrddd.Zh    -> fltarr(n,rg), h-ch/beam reflectivity in dBz
;                 Wcrddd.Zhstd -> fltarr(n),    h-ch/beam noise St.Dev. in dBZ@1km
;                 Wcrddd.Zv    -> fltarr(n,rg), v-ch/beam reflectivity in dBz
;                 Wcrddd.Zvstd -> fltarr(n),    h-ch/beam noise St.Dev. in dBZ@1km
;
;       Acddd:    structure containing AC data needed for dual-doppler analysis
;                 Acddd.beta   -> fltarr(n), AC sideslip angle [rad]
;                 Acddd.lat    -> fltarr(n), latitude position of the AC [deg]
;                 Acddd.lon    -> fltarr(n), longitude position of the AC [deg]
;                 Acddd.palt   -> fltarr(n), AC pressure altitude [m]
;                 Acddd.phi    -> fltarr(n), AC roll angle [rad]
;                 Acddd.psi    -> fltarr(n), AC true heading [rad from North]
;                 Acddd.ralt   -> fltarr(n), AC geometric altitude [m]
;                 Acddd.rarm   -> fltarr(3,2),[h_antenna,v_antenna], antenna
;                                              positions w.r.t. AC INS in AC
;                                              coodinate system
;                 Acddd.theta  -> fltarr(n), AC pitch angle [rad]
;                 Acddd.ug     -> fltarr(n), East-West AC ground velocity [m/s]
;                 Acddd.uw     -> fltarr(n), East wind velocity component [m/s]
;                 Acddd.vg     -> fltarr(n), North-South AC ground velocity [m/s]
;                 Acddd.vw     -> fltarr(n), North wind velocity component [m/s]
;                 Acddd.wg     -> fltarr(n), AC vertical velocity [m/s]
;                 Acddd.ww     -> fltarr(n), vertical wind velocity component [m/s]
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;
; NOTES:
;       When UNFOLD keyword is defined and non-zero, an attempt to unfold
;       non-varying (single, double, etc.) folding in the Doppler velocity 
;       field is performed.  It is based on the assumption that the AC  
;       measured winds or the given mean winds are causing the folding. 
;       Thus centering the corrected for aircraft motion measured folded
;       Doppler velocities around the wind contribution into the WCR beam
;       is expected to render non-folded velocities.
;
;       KA/C130 netCDF files follow the NCAR-RAF/nimbus convention.  This
;       convention requires that the time dimension, 'time' (unlimited or not),
;       is the first (last for idl, which uses column_based order) and the next
;       dimension, if exists, is always samples per second, 'sps'.
;
;       Aircraft time stamps for every data point are assembled using sps
;       dimension and the following two netcdf variables:
;       'base_time' - seconds since 00:00:00 Jan 1 1970
;       'time_offset' - seconds since base_time, a sequence with 1 sec step
;
;       This routine uses jhuapl idl library routine fixang.pro
;
; EXAMPLES:
;       1) From IHOP 2002 Experiment
;
;       wcrdddata,'W2002-05-24-19-58-02', $
;         '/net/owl/data/kingair_data/ihop02/processed/20020524.c25.nc',$
;         200830,200930,0,[53.5,50.0,0,0.84],wcrddd,acddd,/closeac,/closew
;
;       1) From DYCOMSII 2001 Experiment
;
;       beams=2
;       hhcal=49.5 & vvcal=51.5
;       thresh=0                   ; no thresholding
;       timeoffset=-0.125
;       ppselect=14
;       wcrdddata,'/net/fox/fox1/WCR/DYCOMS01/jul24/Wpp01-07-24-11-03-24',  $
;         '/net/fox/fox1/C130/hi_rate/RF07.20010724.055334_154831.PNI.nc', $
;         110700,110900,beams,[hhcal,vvcal,thresh,timeoffset,ppselect], $
;         wcrdd,acdd,/closeac,/closew
;
;-
; MODIFICATION HISTORY:
;       Written by:  Rick Damiani & Samuel Haimov, February 2003
;       S.H., Feb 2003 -- replaced ka prefix with ac where appropriate
;       S.H., May 2003 -- moved wcrtime as part of wcrddd structure;
;                         transposed the 2-d arrays in wcrddd and added
;                         two new fields; modified to work for HBDD01
;                         experiment; added UNFOLD keyword; modified acnames
;       S.H., Jun 2003 -- modified wcrprm input, added beams input
;       S.H., Aug 2003 -- added geometric altitude to acddd
;       S.H., Sep 2003 -- changed allowed input values for UNFOLD; added
;                         sideslip angle to acddd; replaced ztrue field
;                         in acdata with palt field
;       S.H., Oct 2003 -- added leakrg parameter to wcrprm inputs
;       S.H., Nov 2003 -- changes in the help text
;
; UNDOCUMENTED FEATURES:
;      This program works with dual-antenna data from HBDD test
;      experiment in Sep 2001 (this was before swapping the H and V
;      channel designation).  To accomodate the old designation wcrdddata
;      swaps h- and v-ch data. All data in wcrddd structure designated by
;      h and v are based on the new naming and thus hhcal/wcrprm[0]
;      and vvcal/wcrprm[1] should be using V-ch and H-ch reported
;      calibration coefficients, respectively.
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.
;

pro wcrdddata,wcrf,acfile,startt,stopt,beams,wcrprm,wcrddd,acddd,acid=acid,$
              acnames=acnames,closeac=closeac,closewcr=closewcr,$
              unfold=unfold,help=help

  @commonwcrinclude

  on_error,2

; ...........................................................................
; Check some of the inputs

  if keyword_set(help) then begin
    doc_library,'wcrdddata'
    return
  endif

  if n_params() lt 8 then begin
    print,"Usage: wcrfile,acfile,starttime,stoptime,beams,wcrprm,wcrddd,acddd,$"
    print,"                 acid=acid,acnames=acnames,closeac=closeac,$"
    print,"                 closewcr=closewcr,unfold=unfold,help=help"
    return
  endif

  ; Check if the data files exist

  spawn,wcrf+' 2>&1',/sh,junk1
  ind1=strpos(junk1,'such file or directory')
  if ind1 ne -1 then begin
    print,''
    message,'WCRFILE is not found:';,/info
    print,junk1
    return
  endif

  spawn,acfile+' 2>&1',/sh,junk2
  ind1=strpos(junk2,'such file or directory')
  if ind1 ne -1 then begin
    print,''
    message,'ACFILE is not found:';,/info
    print,junk2
    return
  endif

  ; Assign wcrprm values

  if n_elements(wcrprm) ge 2 and n_elements(wcrprm) le 12 then begin
    thresh= 0
    ppsel = 0
    leakrg= 0
    hhcal = wcrprm[0]
    vvcal = wcrprm[1]
    if n_elements(wcrprm) ge 3 then begin
      thresh=wcrprm[2]
      if n_elements(wcrprm) ge 4 then begin
        timeoffset=wcrprm[3]
        if n_elements(wcrprm) ge 5 then begin
          ppsel=fix(wcrprm[4])
          if n_elements(wcrprm) eq 6 then begin
            leakrg=fix(wcrprm[5])
            if n_elements(wcrprm) eq 7 then radmom=wcrprm[6] $
            else if n_elements(wcrprm) eq 12 then radmom=2
          endif
        endif
      endif
    endif
  endif else begin
     message,'Wcrprm input must have 2 to 7 or 12 elements' ;,/info
     return
  endelse

; ...........................................................................
; Open AC file and get the data

  acdata=acloadins(acncid,startt=startt,stopt=stopt,close=closeac,$
                   ncnames=acnames,filename=acfile)
  if keyword_set(closeac) ne 0 then acid=acncid
  acname=acdata.info

  ; Initialize beaminit and antenna positions w.r.t. INS

  if n_elements(beams) eq 1 then begin
    if beams lt 0 or beams gt 2 then begin
      message,'Invalid value for beams.';,/info
      return
    endif else begin
      case beams of
        0: begin
             beaminith=4 & beaminitv=3
             radarmh=[-2.00,0., 0.] & radarmv=[-1.88,0., 0.]
           end
        1: begin
             beaminith=0              & beaminitv=2
             radarmh=[-1.88,0.,-0.79] & radarmv=[-1.22,0.,-0.79]
           end
        2: begin
             beaminith=5 & beaminitv=6
             radarmh=[-18., 0., 0.] & radarmv=[-18., 0., 0.]
           end
      endcase
    endelse
  endif else begin
    if n_elements(beams) ne 6 then begin
      message,'Beams must have 1 or (3,2) elements.';,/info
      return
    endif else begin
      beaminith=beams[*,0]
      beaminitv=beams[*,1]

      ; Determine dual-beam configuration (0,2: dual-down or 1: dual-side)
      ; and assign radar arm vectors
      junk1=max(abs(total(beams,2)),junk2)
      if junk2 ne 1 then junk2=0+2*(beams[0,1] lt -0.2)
      case junk2 of
        0: begin
             radarmh=[-2.00,0., 0.] & radarmv=[-1.88,0., 0.]
           end
        1: begin
             radarmh=[-1.88,0.,-0.79] & radarmv=[-1.22,0.,-0.79]
           end
        2: begin
             radarmh=[-18., 0., 0.] & radarmv=[-18., 0., 0.]
           end
      endcase

      if n_elements(radmom) eq 1 then $
        if radmom eq 2 then begin
          radarmh=wcrprm[5:7]
          radarmv=wcrprm[8:10]
        endif
    endelse
  endelse

; ...........................................................................
; Open WCR file and check pp selection

  if acname eq 'N130AR' then openwcrf,wcrf,timecor=0. $
  else openwcrf,wcrf

  if ppsel eq 0 then begin
    ind1=where(ppstrpol eq 'HH' or ppstrpol eq 'VV')
    if n_elements(ind1) lt 2 then begin
      message,'h- and/or v-beam Doppler velocity is not available';,/info
      closewcrf
      return
    endif else begin
      pphi=where(ppstrpol eq 'HH')
      ppvi=where(ppstrpol eq 'VV')
      if pphi[0] eq -1 or ppvi[0] eq -1 then begin
        message,'h- and/or v-beam Doppler velocity is not available';,/info
        closewcrf
        return
      endif
      if n_elements(ind1) gt 2 and n_elements(maxvel) gt 1 then begin
        maxvelh=max(maxvel[pphi],ind2)
        pphi=pphi[ind2]
        maxvelv=max(maxvel[ppvi],ind2)
        ppvi=ppvi[ind2]
      endif else begin
        pphi=pphi[0]
        ppvi=ppvi[0]
        if n_elements(maxvel) gt 1 then begin
          maxvelh=maxvel[pphi[0]]
          maxvelv=maxvel[ppvi[0]]
        endif else begin
          maxvelh=maxvel[0]
          maxvelv=maxvel[0]
        endelse
      endelse
    endelse
    pphi=pphi+1
    ppvi=ppvi+1
  endif else begin
    ind1=ppsel/10-1
    ind2=(ppsel mod 10)-1
    if ppstrpol[ind1] eq 'HH' and ppstrpol[ind2] eq 'VV' then begin
      pphi=ind1+1 & ppvi=ind2+1
    endif else if ppstrpol[ind1] eq 'VV' and ppstrpol[ind2] eq 'HH' then begin
      pphi=ind2+1 & ppvi=ind1+1
    endif else begin
      message,'ppselect in wcrprm input is incorrect' ;,/info
      closewcrf
      return
    endelse
    if n_elements(maxvel) gt 1 then begin
      maxvelh=maxvel[ind1]
      maxvelv=maxvel[ind2]
    endif else begin
      maxvelh=maxvel[0]
      maxvelv=maxvel[0]
    endelse
  endelse

; ...........................................................................
; Read the requested radar data segment

  if startt eq -1 then startt=sec2time(timestr2sec(starttime))
  if stopt eq -1 then stopt=sec2time(timestr2sec(stoptime))
  readwcrdata,startt,stopt,rawtime=wcrtime,/timein,/nocross,/noleak

; ...........................................................................
; Correct time stamps for negative jumps if data from year 2002

  if wcrdate gt 20020000 and wcrdate le 20021231 then wcrtime=wcrtfp1scor(wcrtime)

; ...........................................................................
; Check if begtime/endtime match startt and stopt

  junk1=sec2time(long(wcrtime[0] mod 86400.))
  if startt ne junk1 then begin
    message,'Start time does not match WCR time. Use new start time '+ $
            time2timestr(junk1),/info
    startt=junk1
  endif

  junk2=sec2time(round(wcrtime[n_elements(wcrtime)-1] mod 86400.))
  if stopt ne junk2 then begin
    message,'Stop time does not match WCR time. Use new stop time '+ $
            time2timestr(junk2),/info
    stopt=junk2
  endif

; ...........................................................................
; Swap h and v channels if date is before 2002.

  if wcrdate gt 20010900 and wcrdate le 20020000 then begin
    junk3='vv'  & junk4='hh'
    junk1=ppvi  & ppvi=pphi  & pphi=junk1
  endif else begin
    junk3='hh'  & junk4='vv'
  endelse

; ...........................................................................
; Initilize output structures
  
  ind1=rangegates-leakrg
  wcrddd={n:    numrec,                   rg:    ind1,                 $
          time: wcrtime,                  maxvel:[maxvelh,maxvelv],    $
          beamh:fltarr(numrec,3),         beamv: fltarr(numrec,3),     $
          dvh:  fltarr(numrec,ind1),      dvv:   fltarr(numrec,ind1),  $
          rng:  radrng[leakrg:*],                                                $
          zh:   fltarr(numrec,ind1),      zv:    fltarr(numrec,ind1),  $
          zhstd:fltarr(numrec),           zvstd: fltarr(numrec)}

  acddd ={lat: fltarr(numrec),lon:  fltarr(numrec),ralt:fltarr(numrec),$
          phi: fltarr(numrec),theta:fltarr(numrec),psi: fltarr(numrec),$
          ug:  fltarr(numrec),vg:   fltarr(numrec),wg:  fltarr(numrec),$
          uw:  fltarr(numrec),vw:   fltarr(numrec),ww:  fltarr(numrec),$
          beta:fltarr(numrec),palt: fltarr(numrec),rarm:[[radarmh],[radarmv]]}

; ...........................................................................
; Calculate Zh, Zhstd, Zv, and Zvstd
; !!!!!!!!!! Note: Thresholding is done using reflectivity which may    !!!!!!!
; !!!!!!!!!!       not be good enough                                   !!!!!!!

  if thresh gt 0 then junk1=[-999.,thresh] else junk1=-999
  wcrddd.zh=(transpose(wcrcalcoref(junk1,hhcal,wcrstr=junk3,mnoise=junk2,$
                         nothresh=(thresh eq 0),/nan,/sil)))[*,leakrg:*]
  wcrddd.zhstd=junk2[*,1]
  wcrddd.zv=(transpose(wcrcalcoref(junk1,vvcal,wcrstr=junk4,mnoise=junk2,junk2,$
                         nothresh=(thresh eq 0),/nan,/sil)))[*,leakrg:*]
  wcrddd.zvstd=junk2[*,1]

; ...........................................................................
; Calculate corrected Doppler velocity for h- and v-beams

  if n_elements(radmom) eq 0 then begin
    wcrddd.dvh=(transpose(wcrdopcorm(wcrtime,acdata=acdata,timehack=timeoffset,$
                ppselect=pphi,ncnames=acnames,beaminit=beaminith,radbeam=junk2,$
                unfold=unfold)))[*,leakrg:*]
    wcrddd.beamh=transpose(ac2wcr(junk2,acdata.time,wcrtime,$
                                  timehack=timeoffset,/errorm))
    wcrddd.dvv=(transpose(wcrdopcorm(wcrtime,acdata=acdata,timehack=timeoffset,$
                ppselect=ppvi,beaminit=beaminitv,radbeam=junk2, $
                unfold=unfold)))[*,leakrg:*]                         
    wcrddd.beamv=transpose(ac2wcr(junk2,acdata.time,wcrtime,timehack=timeoffset))
  endif else begin
    if radmom eq 2 then begin & junk3=radarmh & junk4=radarmv & end $
    else junk3=(junk4=radmom)
    wcrddd.dvh=(transpose(wcrdopcorm(wcrtime,acdata=acdata,timehack=timeoffset,$
                ppselect=pphi,rmcor=junk3,ncnames=acnames,beaminit=beaminith,  $
                radbeam=junk2,unfold=unfold)))[*,leakrg:*]
    wcrddd.beamh=transpose(ac2wcr(junk2,acdata.time,wcrtime,$
                                  timehack=timeoffset,/errorm))
    wcrddd.dvv=(transpose(wcrdopcorm(wcrtime,acdata=acdata,timehack=timeoffset,$
                ppselect=ppvi,rmcor=junk4,beaminit=beaminitv, $
                radbeam=junk2,unfold=unfold)))[*,leakrg:*]
    wcrddd.beamv=transpose(ac2wcr(junk2,acdata.time,wcrtime,timehack=timeoffset))
  endelse

; ...........................................................................
; Resample acdata to match wcrtime (use linear interpolation)

  acddd.lat=ac2wcr(acdata.lat,acdata.time,wcrtime,timehack=timeoffset)
  acddd.lon=ac2wcr(acdata.lon,acdata.time,wcrtime,timehack=timeoffset)

  acddd.phi=ac2wcr(acdata.roll,acdata.time,wcrtime,timehack=timeoffset)
  acddd.theta=ac2wcr(acdata.pitch,acdata.time,wcrtime,timehack=timeoffset)

  junk2=fixang(acdata.heading,/rad) ; unwrap to fix angles crossing 360/0
  acddd.psi=ac2wcr(junk2,acdata.time,wcrtime,timehack=timeoffset)
  acddd.psi=acddd.psi mod (2*!pi)   ; wrap after the interpolation is done

  acddd.ug=ac2wcr(acdata.ug,acdata.time,wcrtime,timehack=timeoffset)
  acddd.vg=ac2wcr(acdata.vg,acdata.time,wcrtime,timehack=timeoffset)
  acddd.wg=ac2wcr(acdata.wg,acdata.time,wcrtime,timehack=timeoffset)

  acddd.uw=ac2wcr(acdata.uw,acdata.time,wcrtime,timehack=timeoffset)
  acddd.vw=ac2wcr(acdata.vw,acdata.time,wcrtime,timehack=timeoffset)
  acddd.ww=ac2wcr(acdata.ww,acdata.time,wcrtime,timehack=timeoffset)

  acddd.palt=ac2wcr(acdata.palt,acdata.time,wcrtime,timehack=timeoffset)
  acddd.ralt=ac2wcr(acdata.ralt,acdata.time,wcrtime,timehack=timeoffset)
  acddd.beta=ac2wcr(acdata.beta,acdata.time,wcrtime,timehack=timeoffset)

; ...........................................................................
; Close wcr input data file if requested

  if keyword_set(closewcr) then closewcrf

end
