;$Id
;+
; NAME:
;      WCRDDDV
;
; PURPOSE:
;      This routine is part of the WCR airborne Dual-Doppler analysis package
;      (wcrdd.pro). It calculates the 3-D velocity vector along the grid 
;      coordinates from the available doppler velocity measurements of two WCR 
;      beams. The procedure uses external data to pre-condition the system (due
;      to lack of 3rd dimension measurements). External estimate of the winds is
;      used to calculate the component of the velocity/wind perpendicular to the 
;      best-fit plane of the two beams.
;
; CATEGORY:
;      WCRTOOLS
;
; CALLING SEQUENCE:
;      res=wcrdddv(LSM,LSV,v_ext=v_ext,condnum=condnum,vnorm=vnorm)
;
; INPUTS:
;      LSM: fltarr(3,k), beam unit vector components, where k is data points
;                        available for each grid cell
;      LSV: fltarr(k),   Doppler velocity measurements from h and v beams
;
; KEYWORD PARAMETERS:
;      CONDNUM: input, float,max ratio between the largest and smallest singular
;               values(sv). If max(w[i])/w[i] gt condnum, where w[i]=1/(sv), then
;               w is set to 0. (default, 100)
;      V_EXT:   input, float(3), external wind vector in local WBRRS 
;               (default,[0,0,0])
;      VNORM:   output, float(3), normal vector to the solution space (least-
;               square best-fit plane of the beams). It is returned only if the 
;               rank of LSM is 2. 
;      HELP:    shows this text
;
; OUTPUTS:
;      Res:     dblarr(3), velocity vector in xi,eta,zi of the FRRS
;
; SIDE EFFECTS:
;
; PROCEDURE:
;      Use IDL SVDC routine (in double precision) to find sv of the input matrix
;      LSM. After conditioning the sv the solution is calculated as the least-
;      squires minimum norm.  When V_EXT is used, the solution (altough not the
;      minimum norm any more) takes advantage of the provided a priori
;      information for the unresolved (null space) velocity component. 
;
; EXAMPLE:
;
;-
; MODIFICATION HISTORY:
;      Written by: Rick Damiani and Samuel Haimov, January 2004
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.

function wcrdddv,LSM,LSV,v_ext=v_ext,condnum=condnum,vnorm=vnorm,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrdddv'
    return,''
   endif

  if (n_params() lt 2) then begin
    message,"Usage: vel=wcrdddv(LSM,LSV,v_ext=v_ext,condnum=condnum,$",/info
    print,"                    vnorm=vnorm,help=help)"
    return,''
  endif

  if n_elements(condnum) eq 0 then condnum=100.

; ..........................................................................
; Perform the singular value(sv) decomposition of LSM
; LSM [3,k], W [3], U [3,k], V [3,3]

  svdc,LSM,W,U,V,/double  ; W: sv vector; LSM=U##diag_matrix(W)##transpose(V)
  
; ..........................................................................
; Condition the singular values and make diag(1/W)

  n=n_elements(W)
  junk=dblarr(n)
  ind1=where(max(W)/(W>(machar(/double)).(9)) lt condnum,compl=ind2)
  if ind1[0] ne -1 then junk[ind1]=1.d0/W[ind1]

  W=diag_matrix(junk) ; [3,3]
  
; ..........................................................................
; Calculate the solution of the linear system. It is expected that the system 
; will be in most of the cases a rank deficient (i.e. rank<=2). In fact all the 
; other beam unit vectors are very close to be linearly dependent from the 
; slanted and straight one; unless the AC was really jittering a lot, the beams 
; should almost reside on a common plane, hence rank should be 2 in most cases.  
; If v- or h-beam data are missing entairly for a given grid cell then
; the rank could be 1. 3 is the max rank in any case, but again it should be 
; very rare that sv conditioning would not discard the cross-wind component.

  dvel=V ## W ## TRANSPOSE(U) ## LSV  ; [3] , LSV [k]

; NOTE: This is a minimum norm solution because rank 2 should be most likely
; and it implies no wind perp. to the best-fit plane of the beams. This has got 
; errors, since if that component is <>0, it will show up in the in-plane 
; components, because not all the beams reside exactly on the plane 
; (i.e.:it is a best-fit plane).

; Same result can be obtained with LA_LEAST_SQUARES instead of using svdc
; dvel=LA_LEAST_SQUARES(LSM,LSV,/double,method=2,rank=rankLSM,rcond=1.e-3, $
;                       status=status) ;,
; but have no access to the null space contribution
 
; ..........................................................................
; Try to improve the solution by using the external wind information

; NOTE: to correct the initial solution, just sum the component of the estimated
; wind perpendicular to the plane of the beams. In fact the null space is
; represented exactly by the vector normal to the plane of the beams. This is if
; the rank of LSM is 2. If it is 1 then the null-space is the plane 
; perpendicular to the best fit beam vector. Should account for the wind 
; component on that plane. The beauty of the method is: the columns of V whose 
; same-numbered W's are zeroes give an orthonormal basis for the null-space.

  if n_elements(v_ext) eq 3 and ind2[0] ne -1 then begin

  ; Find the projection of the v_ext along the null space (line)
    junk=(V[ind2[0],*]#v_ext)[0]*V[ind2[0],*] ; dot_product*vector [3]
    
    if n_elements(ind2) eq 2 then $
      junk=junk+(V[ind2[1],*]#v_ext)[0]*V[ind2[1],*] ; [3]  as above (plane)
    
    dvel=dvel+junk
  endif

; ..........................................................................
; Export the normal vector of the best-fit-plane (from the null space V)

  if ind2[0] eq 2 then $                    ; i.e., valid for rank=2 only
       vnorm=float(reform(V[ind2[0],*])) $  ; in local WBRRS
  else vnorm=0.                             ; for rank=1

return,float(dvel)

end
