;$Id: wcrddecho.pro,v 1.1 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRDDECHO
;
; PURPOSE:
;       This routine is part of the WCR airborne Dual-Doppler analysis package
;       (wcrdd.pro). It writes out in an ascii file the values of the parameters
;       used for Dual-Doppler analysis(data taken from grddd, pcddd and wnddd 
;       structures). Statistics of selected aircraft(AC) attitude parameters
;       (acstats) and processing results(pcstats) are also recorded.
;
; CATEGORY:
;       WCR wcrtools
;
; CALLING SEQUENCE:
;       wcrddecho,fstruct,grddd,pcddd,wnddd,acstats,pcstats,help=help
;
; INPUTS:
;       Fstruct: structure containing the names of the input/output files
;                (output from wcrddinit.pro)
;       Grddd:   structure containing grid parameters and other processing
;                (output from wcrddinit.pro, wcrddgrid0/1.pro)
;       Pcddd:   structure containing processing parameters
;                (output from wcrddinit.pro,wcrddgrid0/1.pro)
;       Wnddd:   structure containing external wind info processing parameters
;                (output from wcrddinit.pro,wcrddwext.pro)
;       Acstats: structure containing statistics of the aircraft attitude angles 
;                (output from wcrddgrid0/1.pro)
;       Pcstats: structure containing some statistics of the result from the
;                processing (output from wcrddproc0/1.pro)
;
; KEYWORD PARAMETERS:
;       HELP:    shows this text
;
; OUTPUTS:
;
; SIDE EFFECTS:
;      This program creates an ascii file using fstruct.echof to define the
;      location path and the file name
;
;      The routine expects to find grid structure fields called ALT0,DIMS,CSIZE,
;      STRTT and STOPT. ALT0 value is printed in the message area (below the top
;      title) of the output file. DIMS is used to verify if a change in the grid
;      dimension size took place in the course of the dual-Doppler processing. 
;      CSIZE is printed out next to DIMS. STRTT and STOPT are printed in 
;      hh:mm:ss.
;
;      Any field(tag) name in pcstats which starts with 'ang' is assumed
;      in radians and field names containing 'crank' and 'empty' are in %.
;
;      This routine uses getfile and putfile from JHUAPL idl library
;
; PROCEDURE:
;
; NOTES:
;
; EXAMPLE:
;
;-
; MODIFICATION HISTORY:
;      Written by: Rick Damiani and Samuel Haimov, September 2003
;      S.H., Oct 03 - added grddd.csize to the output file
;      S.H., Dec 03 - added pcstats.empty
;      S.H., Mar 04 - added grddd.strtt and grddd.stopt to the output file;
;                     added flight date to the message line
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.
;

pro wcrddecho, fstruct,grddd,pcddd,wnddd,acstats,pcstats,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrddecho'
    return
  endif

  if (n_params() eq 0) then begin
    message,"Usage: wcrddecho,fstruct,grddd,pcddd,wnddd,acstats,pcstats,help=help",/info
    return
  endif

; Initilize

  cs0 =';#'                        ; beginning comment string id
  cs1 ='#;'                        ; ending comment string id

  cs2 =strjoin(replicate('#',76))  ; major comment separator
  cs3 =strjoin(replicate('.',76))  ; minor comment separator

; ............................................................................
; Read fstruct.prmsf

  junk1=file_test(fstruct.prmsf,/read)
  if junk1 then fstr=getfile(fstruct.prmsf,/quiet) $
  else message,fstruct.ddprms+' not found.'

; ............................................................................
; Write new title line, write the date the file is generated in 4th line of the 
; file and clear the 5th line

  junk1='GRID AND PROCESSING PARAMETERS/STATISTICS FOR WCR DUAL-DOPPLER PROCESSOR'
  fstr[1]=cs0+'  '+junk1+'  '+cs1
  junk1=strmid(fstruct.echof,strpos(fstruct.echof,'/',/reverse_search)+1)
  junk2=cs0+' '+junk1+' generated: '
  junk3=junk2+systime()+string(replicate(32B,54-strlen(junk2)))+cs1
  fstr[3]=junk3
  fstr[4]=cs0+string(bytarr(76)+32B)+cs1

; ............................................................................
; Find all no comment lines and get data and name lines

  ind     = where(strmid(fstr,0,2) ne cs0 and fstr ne '')
  nind    = n_elements(ind)-1
  datstr  = strtrim(fstr[ind],2)
  namestr = strmid(strtrim(fstr[ind-1],2),2)
  name2str= strmid(strtrim(fstr[ind-2],2),2)

; ............................................................................
; Remove all comment lines after the last data line (not needed for the echo)
  fstr=fstr[0:ind[nind]+1]
  nfstr   = n_elements(fstr)

; ............................................................................
; Populate data lines with the apprpriate data from the structures

  ; Find all tag names

  grdtags=tag_names(grddd)
  pctags =tag_names(pcddd)
  wntags =tag_names(wnddd)
  actags =tag_names(acstats)
  pcstags=tag_names(pcstats)

  ; Search for the tag names and assign the appropriate values 
  ; from grddd, pcddd and wnddd to data lines and then merge with the file str

  for k=0,nind do begin
    junk1=strsplit(datstr[k],' ',/extract)
    junk2=strsplit(namestr[k],' ',/extract)
    junk3=1
    kk=0
    while kk le n_elements(junk1)-1 do begin
    ; search for grddd tags ...........................     ; !!!!!!!!!!!!!!!!!!
      ind1=(where(grdtags eq strupcase(junk2[kk])))[0]      ; without making ind1
      if ind1[0] ne -1 then begin                           ; scalar where in the
        junk3=fix(n_elements(where(junk2 eq grdtags[ind1]))); line won't work right
      ; For DIMS find if any change took place and acknowledge it in the echo  
        if strupcase(junk2[kk]) eq 'DIMS' then begin
          if (where(grddd.(ind1) ne junk1[kk:kk+junk3-1]))[0] ne -1 then begin
            fstr[4]=cs0+' Grid dimensions have changed !!!'+string(bytarr(43)+32B)+cs1
          endif
        ; Remember data line number that contain DIMS (later add CSIZE,STRTT and
        ; STOPT to it)
          csizeline=k
        endif                    
        junk1[kk:kk+junk3-1]=num2str(grddd.(ind1),len=10,fr=2)
        ind2=k   ; find last data line not counting file name lines
      endif
    ; search for pcddd tags............................
      ind1=(where(pctags eq strupcase(junk2[kk])))[0]
      if ind1[0] ne -1 then begin
        junk3=n_elements(where(junk2 eq pctags[ind1]))
        junk1[kk:kk+junk3-1]=num2str(pcddd.(ind1),len=10,fr=2)
        ind2=k
      endif
    ; search for wnddd tags............................
      ind1=(where(wntags eq strupcase(junk2[kk])))[0]
      if ind1[0] ne -1 then begin
        junk3=n_elements(where(junk2 eq wntags[ind1]))
        junk1[kk:kk+junk3-1]=num2str(wnddd.(ind1),len=10,fr=2)
        ind2=k
      endif
      kk=kk+junk3
    endwhile
    datstr[k]=strjoin(junk1)
    fstr[ind[k]]=datstr[k]  
  endfor

; ............................................................................
; Add grddd.csize values to the string array after grddd.dims 
 
  junk1=strtrim(strmid(fstr[ind[csizeline]-2],0,strlen(fstr[ind[csizeline]-2])-2))
  junk1=junk1+'      DHOR'+'      DVER'+'     START'+'      STOP'
  fstr[ind[csizeline]-2]=junk1+string(replicate(32B,78-strlen(junk1)))+cs1

  junk1=strtrim(strmid(fstr[ind[csizeline]-1],0,strlen(fstr[ind[csizeline]-1])-2))
  junk1=junk1+'     CSIZE'+'     CSIZE'+'      WCRT'+'      WCRT'
  fstr[ind[csizeline]-1]=junk1+string(replicate(32B,78-strlen(junk1)))+cs1
   
  fstr[ind[csizeline]]=fstr[ind[csizeline]]+ $
                       strjoin(num2str(round(grddd.csize),len=10))+ $
                       '  '+sec2timestr(grddd.wcrt[0] mod 86400)+ $
                       '  '+sec2timestr(grddd.wcrt[1] mod 86400)    

; ............................................................................
; Create a string array for acstats and pcstats, structures

  fstr1   =strarr(2*5+n_elements(actags)+n_elements(pcstags))

  ; Prepare table for acstats

  fstr1[0]=cs0+cs2+cs1
  fstr1[1]='AIRCRAFT_ATTITUDE_STATISTICS'
  junk1   =(76-strlen(fstr1[1]))/2
  junk2   =junk1-(strlen(fstr1[1]) mod 2)
  fstr1[1]=cs0+string(bytarr(junk1)+32B)+fstr1[1]+string(bytarr(junk1)+32B)+cs1
  fstr1[2]=cs0+cs3+cs1
  junk3   ='                     MEAN[deg]  STD[deg]  MIN[deg]  MAX[deg]'
  fstr1[3]=cs0+junk3+string(bytarr(16)+32B)+cs1
  fstr1[4]=cs0+cs3+cs1

  for k=0,n_elements(actags)-1 do begin
    junk1=string(bytarr((4-n_elements(acstats.(k)))*10+18)+32B)
    junk2=strjoin(num2str(!RADEG*acstats.(k),len=10,fr=2))
    junk3=stradj(actags[k],len=18,adj=1)
    fstr1[k+5]=cs0+junk3+junk2+junk1+cs1
  endfor

  ; Prepare table for pcstats

  fstr1[5+n_elements(actags)]=cs0+cs2+cs1
  fstr1[6+n_elements(actags)]='DUAL_DOPPLER_PROCESSING_STATISTICS'
  junk1                      =(76-strlen(fstr1[6+n_elements(actags)]))/2
  junk2                      =junk1-(strlen(fstr1[6+n_elements(actags)]) mod 2)
  fstr1[6+n_elements(actags)]=cs0+string(bytarr(junk1)+32B)+ $
                              fstr1[6+n_elements(actags)]+string(bytarr(junk1)+32B)+cs1
  fstr1[7+n_elements(actags)]=cs0+cs3+cs1
  junk3   ='                        MEAN       STD       MIN       MAX'
  fstr1[8+n_elements(actags)]=cs0+junk3+string(bytarr(18)+32B)+cs1
  fstr1[9+n_elements(actags)]=cs0+cs3+cs1
  
  for k=0,n_elements(pcstags)-1 do begin
    junk1=string(bytarr((4-n_elements(pcstats.(k)))*10+18)+32B)
    if strlowcase(strmid(pcstags[k],0,3)) eq 'ang' then begin
      junk3='[deg]'
      junk2=strjoin(num2str(!RADEG*pcstats.(k),len=10,fr=2))
    endif else if strpos(strlowcase(pcstags[k]),'crank') ne -1 or $
                  strpos(strlowcase(pcstags[k]),'empty') ne -1 then begin
      junk3='[%]'
      junk2=strjoin(num2str(pcstats.(k),len=10,fr=2)) 
    endif else begin
      junk3='' 
      junk2=strjoin(num2str(pcstats.(k),len=10,fr=2))
    endelse
    junk3=stradj(pcstags[k]+junk3,len=18,adj=1)
    fstr1[10+n_elements(actags)+k]=cs0+junk3+junk2+junk1+cs1
  endfor

; ............................................................................
; Add a message line for grddd.alt0

  junk1=' Altitude(msl) of the first grid cell: '+num2str(grddd.alt0,fr=0)+'m; '
  junk2=js2unixtime(grddd.wcrt[0])
  junk1=junk1+'Flight date: '+strmid(junk2,8,3)+strmid(junk2,4,4)+strmid(junk2,20)
  junk1=cs0+junk1+string(replicate(32B,76-strlen(junk1)))+cs1
; ............................................................................
; Merge fstr1 information to fstr

  fstr=[fstr[0:4],junk1,fstr[5:ind[ind2]],fstr1,fstr[ind[ind2]+1:*]]
; ............................................................................
; Write the output file

  putfile,fstruct.echof,fstr

end
