;$Id:
;+
; NAME:
;      WCRDDGRID0
;
; PURPOSE:
;      This routine is part of the WCR airborne Dual-Doppler analysis package
;      (wcrdd.pro). It generates a 3-D grid to be used for the dual-Doppler 
;      calculations (wcrddproc0.pro) assuming a straight-leg pattern
;
; CATEGORY:
;      WCR wcrtools
;
; CALLING SEQUENCE:
;      wcrddgrid0, beamopt,acddd,wcrddd,grddd,dtsf,acstats,strk, $
;                  acdisp=acdisp,ddbreak=ddbreak,help=help
;
; INPUTS:
;      Beamopt: integer,dual-antenna configuration 
;                0    - N2UW KA dual-down antennas
;                1    - N2UW KA dual-side antennas
;                2    - NCAR C130 dual-down antennas 
;      Acddd:   structure containing AC data (for description run wcrdd, /help)
;      Wcrddd:  structure containing WCR data(for description run wcrdd, /help)
;      Grddd:   structure containing grid parameters and other processing
;               option parameters(for description run wcrdd, /help)
;
; KEYWORD PARAMETERS:
;      ACDISP:  input, when set plots AC attitude angles and ground track
;      DDBREAK: input, when set allows the user to stop the dual-Doppler 
;                      processing. AC attitude statistical information is 
;                      displayed.
;      HELP:    shows this text
;
; OUTPUTS:
;      Grddd:   structure containing grid parameters and other processing
;               option parameters(for description run wcrdd, /help)
;      Acstats: structure containing statistics of the aircraft attitude angles 
;               Acstats.beta -> fltarr(4),[rad] mean,sdev,min,max sideslip angle 
;               Acstats.hdg  -> float, average heading w.r.t. GND
;               Acstats.hdgv -> float, average heading w.r.t. WBRS (virtual heading)
;               Acstats.pitch-> fltarr(4),[rad] mean,sdev,min,max 
;               Acstats.roll -> fltarr(4),[rad] mean,sdev,min,max
;               Acstats.yaw2 -> fltarr(4),[rad] mean,sdev,min,max (w.r.t.heading)
;               Acstats.yaw2v-> fltarr(4),[rad] mean,sdev,min,max(w.r.t.vheading)
;      Dtsf:    fltarr(2,n,rg,3), h-beam(dtsf[0,*,*,*]) and v-beam(dtsf[1,*,*,*])
;               data points(n: profiles, rg: range gates) coordinates in WBRRS 
;      Strk:    fltarr(n), s-coordinate for the length of track. It is used only
;               by wcrddwext routine so far.
;
; SIDE EFFECTS:
;
; PROCEDURE:
;
; NOTES:
;      If there is a large bank and/or yaw, the grid will result longer than 
;      you would expect due to the fact that the last farthest beam point 
;      determines its extent. This may then cause problems, as the time 
;      associated with grid center points is interpolated starting from the 
;      0th rangegate straight-beam points, which may result to have much 
;      shorter xi values then the max extent of the grid. The extrapolation 
;      to grid center points whose xi values are higher than the 0th rg 
;      points may result in wrong numbers, i.e. the time may even go back. 
;      This could be a problem if you want to use the time later on. So,
;      reccomendation is to avoid these situations by changing to leg=1.
;
;      Beamopt for NCAR C130 uses h-beam as the straight down beam; this
;      will change for future installations when v-beam will be the nadir.
;      Appropriate changes to the use of beamopt must be made (perhaps will
;      add a new configuration to preserve the ability to work with DYCOMS01)
;
; EXAMPLE:
;
;-
; MODIFICATION HISTORY:
;      Written by: Rick Damiani and Samuel Haimov, October 2003
;      S.H., Mar 04 - fill in the time stamps grddd.strtt and grddd.stopt
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.
;

pro wcrddgrid0,beamopt,acddd,wcrddd,grddd,dtsf,acstats,strk, $
               acdisp=acdisp,ddbreak=ddbreak,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrddgrid0'
    return
  endif

  if (n_params() lt 6)  then begin
    message,"Usage: wcrddgrid0,beamopt,acddd,wcrddd,grddd,dtsf,acstats,strk, $",/info
    print,"                     acdisp=acdisp,ddbreak=ddbreak,help=help"
    return
  endif

; =============================================================================
; Determine WCR range gate spacing and prepare acstats structure. Will use
; Wind Based Reference System (WBRS: N-E-UP moving with the wind w.r.t. ground)

  dflt=99999.0                      ; code for the use of a default value
  rgs =wcrddd.rng[1]-wcrddd.rng[0]  ; range gate spacing in meters  

  acstats= {beta: fltarr(4), hdg:   0.,       hdgv: 0., $
            roll: fltarr(4), pitch: fltarr(4),yaw2: fltarr(4), yaw2v: fltarr(4)}

  ; ...........................................................................
  ; Retrieve the avg. heading. We may consider later the issue of first
  ; averaging Ug and Vg and then calculating the heading. 

  junk=atan(acddd.Ug,acddd.Vg)                    ;(-pi,pi]
  ind =where(junk lt 0.)
  if ind[0] ne -1 then junk[ind]=junk[ind]+2.*!pi ;( 0,2pi]
  acstats.hdg=mean(junk)

  ; ...........................................................................
  ; Compile AC velocity vector and wind vector in WBRS

  acvel=[[acddd.ug],[acddd.vg],[acddd.wg]] ;AC meas. AC vel. in UFRS (N,E,UP)
  junk =[[acddd.uw],[acddd.vw],[acddd.ww]] ;AC meas. wind  in UFRS (N,E,UP)

  ; ...........................................................................
  ;find constant components of grddd.wadv (ind1)
  ;find follow point by point AC measured wind (ind2)

  ind1=where(grddd.wadv ne 888.,compl=ind2)

  ; ...........................................................................
  ; AC relative velocity w.r.t. WBRS

  if ind1[0] ne -1 then $
    acvel[*,ind1]=acvel[*,ind1]-make_array(wcrddd.n,value=1.)#grddd.wadv[ind1]
  if ind2[0] ne -1 then $
    acvel[*,ind2]=acvel[*,ind2]-junk[*,ind2] 
  
  ; ...........................................................................
  ; Retrieve average heading with respect to the WBRS 
  ;(will call it 'virtual heading')

  junk=atan(acvel[*,0],acvel[*,1])                     ;(-pi,pi]
  ind =where(junk lt 0.)                                         
  if ind[0] ne -1 then junk[ind]=junk[ind] +2.*!pi     ;( 0,2pi]
  acstats.hdgv=mean(junk)

  ; ...........................................................................
  ; Calculate acstats values (in rad)

  acstats.pitch=[mean(Acddd.theta,/nan),stddev(Acddd.theta,/nan),$
                  min(Acddd.theta,/nan),   max(Acddd.theta,/nan) ]

  acstats.roll=[mean(Acddd.phi,/nan),stddev(Acddd.phi,/nan),$
                 min(Acddd.phi,/nan),   max(Acddd.phi,/nan) ]

  ; yaw with respect to the average heading, it should be equal to the 
  ; difference between virtual_hdg and hdg_avg

  acstats.yaw2=[  mean(fixang(Acddd.psi,/rad)-acstats.hdg,/nan),$
                stddev(fixang(Acddd.psi,/rad)-acstats.hdg,/nan),$
                   min(fixang(Acddd.psi,/rad)-acstats.hdg,/nan),$
                   max(fixang(Acddd.psi,/rad)-acstats.hdg,/nan) ]

  ; yaw with respect to the average virtual heading, it represents a sort of 
  ; sideslip angle w.r.t. avg dir (in this case it is >0 if relative wind is 
  ; to the left of the AC nose)

  acstats.yaw2v=[  mean(fixang(Acddd.psi,/rad)-acstats.hdgv,/nan),$
                 stddev(fixang(Acddd.psi,/rad)-acstats.hdgv,/nan),$
                    min(fixang(Acddd.psi,/rad)-acstats.hdgv,/nan),$
                    max(fixang(Acddd.psi,/rad)-acstats.hdgv,/nan) ]

  ; sideslip. This is the real thing period.
  ; It was acstats.beta2=fixang(Acddd.psi,/rad)-mean(virtual_hdg). beta2 is
  ; very similar to beta except for the effect of the roll angle which is not 
  ; included. beta is measured on a plane perpendicular to the x_body,z_body; 
  ; psi-virt._hdg on a horizontal plane, also the sign is opposite and beta2 
  ; here is calculated w.r.t. an avg direction. Currently beta2 and beta is
  ; plotted when ACDISP is set. beta2 was an array calculated pt-by-pt and then
  ; used in wcrddproc2 to estimate TOLI/J. Now we are going to use the real beta
  ; from acddd for that. For the plots we will use beta2

  acstats.beta=[mean(Acddd.beta,/nan),stddev(Acddd.beta,/nan),$
                 min(Acddd.beta,/nan),   max(Acddd.beta,/nan) ]

; =============================================================================
; Plot AC attitude and track, and print the statistics if requested

  if keyword_set(acdisp) then wcrddacdisp,wcrddd,acddd,acstats,ddbreak=ddbreak

; =============================================================================
; Check whether or not to proceed

  if keyword_set(ddbreak) then begin
    junk=''
    read,prompt='Proceed ?[y/n]=',junk
    if (strlowcase(junk) eq 'n') then begin
      message,'User quit.',/info
      retall
    endif
  endif

; =============================================================================
; Prepare transformation matrix RM2 between UFRS and WBRRS. It allows handling
; XI direction along the "air-track" with z down.

  grddd.RM2=[ [sin(acstats.hdgv),  cos(acstats.hdgv),    0.],$
              [cos(acstats.hdgv), -sin(acstats.hdgv),    0.],$
              [0.,                 0.,                  -1.] ]

; =============================================================================
; Beam points 3-D coordinates calculation

; Find the coordinates of the data points (for each beam and profile)
; starting from the first available position of the AC, which is the origin
; of a temporary unrotated fixed coord.system(N-E-UP).  Beams are called h and
; v.  For beam dimension, h is first index(0) and v is second index(1).

  ; ...........................................................................
  ; Transform radar arm vector from AC to ground ref. system (UFRS or WBRS which
  ; is UFRS translated in space)

  cr=cos(acddd.phi)    &  sr=sin(acddd.phi)     ; roll
  cp=cos(acddd.theta)  &  sp=sin(acddd.theta)   ; pitch 
  cy=cos(acddd.psi)    &  sy=sin(acddd.psi)     ; yaw

  i2r=fltarr(2,wcrddd.n,3) ; hbeam and vbeam ins-to-radar vectors for each prof.

  ; transform the wcr dist. vector to ground ref. system for all profs.(n,3)
  ; i2r=transpose(RM)##junk ; where RM is GRND-to-AC transform. matrix,
  ; RM=transpose(Tz##(Th##(Tp##Tr))); Tr,Tp,Th - roll, pitch, heading rotation
  ; matrices and Tz is a rotation matrix from down to up (see ac2gt.pro)
  ; Note that Th uses heading, which is angle from North, while psi is the yaw
  ; w.r.t. initial x axis.  Thus RM can also be expressed in terms of Eulerian 
  ; angles as RM=Tphi##(Ttheta##(Tpsi##Tz1)), where Tz1 accounts for z down and
  ; making psi measured from North (heading), Tphi=Tr, Ttheta=Tp.

  for k=0,1 do begin
    junk=reform(Acddd.rarm[*,k]) ; dist. from ins to wcr in AC cooord. system
    i2r[k,*,*]=[ $
       [cp*sy*junk[0] +(cy*cr+sp*sr*sy)*junk[1]  +(sp*sy*cr-sr*cy)*junk[2]],$
       [cp*cy*junk[0] +(sp*sr*cy-cr*sy)*junk[1]  +(sp*cy*cr+sr*sy)*junk[2]],$
       [sp*junk[0]    - cp*sr*junk[1]            - cp*cr*junk[2]          ] ]
  endfor

  ; ...........................................................................
  ; Initialize distance variables

    ; s-coordinate: length of track; used for interpolating external wind only
    strk=fltarr(wcrddd.n)

    ; INS/CG (INS is located at Center of Gravity) dist. from WBRS (it was UFRS
    ; before acvel became w.r.t. WBRS)
    dx=fltarr(3)

    ; Beam points Distance matrix w.r.t. WBRS ([profile,rangegate,3,beam])
    d=fltarr(wcrddd.n,wcrddd.rg,3,2)

    ; Beam points Distance matrix w.r.t. WBRRS (WBRS with z down)
    dtsf=fltarr(2,wcrddd.n,wcrddd.rg,3)

  ; ...........................................................................
  ; Depending on the beam config find which grid plane (VDD or HDD) to process
  ; and which channel is the straight beam 

  if beamopt ne 1 then iplane =2 else iplane =1  ; 1=side,eta or 2=vertical,zi
  if beamopt ne 0 then ibeam=0 else ibeam=1      ; 0=h,1=v

  ; ...........................................................................
  ; Store the first position of the straight beam antenna positions
  ; It will be used for the origin of the grid

  drx=reform(i2r[ibeam,0,*]) ;(3) [m]

  ; ...........................................................................
  ; For every profile calculate d[prof,rg,3,beams] and strk [prof]

  junk      =replicarr(replicarr(wcrddd.rng,3),2)
  d[0,*,*,*]=replicarr(transpose(reform(i2r[*,0,*])),wcrddd.rg,/dim) + $
             replicarr(reform([[wcrddd.beamh[0,*]],[wcrddd.beamv[0,*]]]),$
                       wcrddd.rg,/dim)*junk

  for k=1,wcrddd.n-1 do begin

  ; Update INS position distances (use better integration ?)
 
    dx=dx+(acvel[k,*]+acvel[k-1,*])/2.*(wcrddd.time[k]-wcrddd.time[k-1])

  ; Calculate beam point distances in WBRS  with origin at INS position for the 
  ; initial profile(the beginning of the leg)

    d[k,*,*,*]=replicarr(replicarr(dx,2),wcrddd.rg,/dim) + $
               replicarr(transpose(reform(i2r[*,k,*])),wcrddd.rg,/dim) + $
               replicarr(reform([[wcrddd.beamh[k,*]],[wcrddd.beamv[k,*]]]),$
                         wcrddd.rg,/dim)*junk

  ; Calculate track length along the track s-coordinate) w.r.t. WBRS
                
    strk[k]=strk[k-1]+(wcrddd.time[k]-wcrddd.time[k-1]) $        ; use better  
                     *norm(reform(acvel[k,*]+acvel[k-1,*])/2.)   ; quadrature ?
  endfor

  ; ...........................................................................
  ; Reset d so that WBRS origin(0,0,0) is at the first position of the straight
  ; beam antenna

  for k=0,2 do d[*,*,k,*]=d[*,*,k,*]-drx[k]

  ; ...........................................................................
  ; Caclulate dtsf: d in WBRRS (distance vectors in WBRRS along the virtual hdg)
  ; dtsf[beam,prof,rg,*]=grddd.RM2 ## reform(d[prof,rg,*,beam])

  junk=grddd.rm2
  dtsf[*,*,*,0]=transpose(junk[0,0]*d[*,*,0,*]+junk[1,0]*d[*,*,1,*]+ $
                          junk[2,0]*d[*,*,2,*],[3,0,1,2])
  dtsf[*,*,*,1]=transpose(junk[0,1]*d[*,*,0,*]+junk[1,1]*d[*,*,1,*]+ $
                          junk[2,1]*d[*,*,2,*],[3,0,1,2])
  dtsf[*,*,*,2]=transpose(junk[0,2]*d[*,*,0,*]+junk[1,2]*d[*,*,1,*]+ $
                          junk[2,2]*d[*,*,2,*],[3,0,1,2])

  ; Reset dtsf for 0(origin) of the z-axis(iplane=2) or y-axis(iplane=1)
  ; so that WBRRS is fixed at the highest z (or leftmost y) reached by the AC

  ; !!!!!!! Figure out why junk=0 is a special case and no reset is done
  ;         For now we will ignore this (the commented out line for junk below)

  ; Minimum distance from WBRRS initial origin along eta of the straight beam
  ; antenna (in meters)

  junk=min(dtsf[ibeam,*,*,iplane])
  ; junk=junk-wcrddd.rng[0]*(junk ne 0.) 
  junk=junk-wcrddd.rng[0] 
  dtsf[*,*,*,iplane]=dtsf[*,*,*,iplane]-junk

  ; Initilize grddd.alt0 with the additional displacement between the initial 
  ; WBRRS origin and the maximum altitude(or leftmost y) reached by the AC

  grddd.alt0=junk 

; =============================================================================
; GRID generation

  ; ...........................................................................
  ; Save first and last profile numbers and times for the processed segment

  grddd.pfct[0]=0          ; will be changed if grddd.rst=1 and beamopt ne 2
  grddd.pfct[1]=wcrddd.n-1 ; will be changed if grddd.rst=1 and beamopt eq 2

  grddd.wcrt[0]=wcrddd.time[0]
  grddd.wcrt[1]=wcrddd.time[grddd.pfct[1]]

  ; ...........................................................................
  ; Establish grid vertical/lateral length 

  if grddd.let eq dflt then grddd.let=(max(dtsf[*,*,*,iplane])+rgs/2.)

  ; ...........................................................................
  ; Reset dtsf (WBRRS) to get rid of 1st 'triangle' containing one beam data
  ; only and update grddd.pfct and grddd.csize if requested 

  ; Note: the resetting is performed on the basis of WBRS. Think about being 
  ;       fixed on WBRS and calculate the initial distance of the farthest point
  ;       of the slanted beam. That distance will be covered by the AC in a 
  ;       shorter or longer time w.r.t. the same distance on the ground. However
  ;       the distance stays the same in the same WBRS. It is the AC reaching it
  ;       in shorter/faster time.  

  if grddd.rst eq 1 then begin

  ; ...........................................................................
  ; Find dtsf correction (junk) depending on beam config and update grddd.pfct

    ; .........................................................................
    if beamopt eq 2 then begin  ; Process for NCAR C130 dual-down ant. config. 

    ; xi directional cosine(<0) of the slanted beam at last profile, n
      junk1=(grddd.RM2 ## wcrddd.beamv[wcrddd.n-1,*])[0] 

    ; find xi grid start position (junk in [m]) where both beams are present
      junk=dtsf[0,0,0,0] ; xi coord. of nadir beam 1st prof (since slanted beam
                         ; is pointed aft

    ; find new grddd.pfct[1] at max xi coordinate allowed
      ; max xi: max(dtsf[0,wcrddd.n-1,*,0])+grddd.let*junk1/sqrt(1.-junk1^2)
     
      junk1=min(abs(dtsf[ibeam,*,*,0]-max(dtsf[0,wcrddd.n-1,*,0])- $
                    grddd.let*junk1/sqrt(1.-junk1^2)),ind1)
      grddd.pfct[1]=ind1 mod wcrddd.n

    ; .........................................................................
    endif else begin ; Process for KingAir dual-ant. configurations

    ; assign the slanted beam
      if beamopt eq 0 then junk1=wcrddd.beamh[0,*] else junk1=wcrddd.beamv[0,*]

    ; xi directional cosine of the slanted beam at 1st profile, 0
      junk1=(grddd.RM2 ## junk1)[0] 

    ; find xi grid start position (junk in [m]) where both beams are present
      junk=grddd.let*junk1/sqrt(1.-junk1^2) 

    ; find new grddd.pfct[0] from where both beams are present
      junk1=min(abs(dtsf[ibeam,*,*,0]-junk),ind1)
      grddd.pfct[0]=ind1 mod wcrddd.n

    endelse

  ; Update dtfs(WBRRS) and strk(s-coordinate)

    dtsf[*,*,*,0]=dtsf[*,*,*,0]-junk
    strk=strk-strk[grddd.pfct[0]]

  endif

  ; ...........................................................................
  ; Establish along track and across track grid cell size in [m]

  ; Along track cell size(or 1st/initial if HSR<>1); use straight beam 0th rg
  ; Note:  Consider limiting HSR,VSR based on minimum cell size(rg)
  ; and maximum cell size (~swath)
  junk1=grddd.srat[0]^indgen(grddd.dims[0])
  grddd.csize[0]=dtsf[ibeam,grddd.pfct[1],0,0]/total(junk1) ;dtsf/NXI for HSR=1 

  ; Across track cell size(or 1st/initial if VSR<>1)
  junk2=grddd.srat[1]^indgen(grddd.dims[1])
  grddd.csize[1]=grddd.let/total(junk2)                     ; LET/NET for VSR=1

  ; ...........................................................................
  if keyword_set(ddbreak) then begin
    print,FORMAT='("Initial Cell size DHOR*DVER*SWTH [m*m*m]=",3(F7.2,:,"*"))',$
     grddd.csize[0],grddd.csize[1],grddd.swth
    print,FORMAT='("Other grid parameters(HSR,VSR,LET[m])=",2(F6.2,","),F8.2)',$
     grddd.srat[0],grddd.srat[1],grddd.LET
    junk=''
    read,prompt='Proceed ?[y/n]=',junk
    if (strlowcase(junk) eq 'n') then begin
      message,'User quit.',/info
      retall
    endif
  endif

  ; ...........................................................................
  ; Determine along track(s-coord.) and normal to track distances to the grid
  ; cell centers 

  ; NXI along-track coords of cell center points[m]
  grddd.crd1=grddd.csize[0]*(total(junk1,/cumulative)-junk1/2.)

  ; NET normal-to-track coords of cell center points [m]
  grddd.crd2=grddd.csize[1]*(total(junk2,/cumulative)-junk2/2.)

  ; ...........................................................................
  ; Determine grid cell coordinates (cell corner points) in WBRRS
  ; grddd.gp[NXI+1,NET+1,3]

  grddd.gp[*,*,0     ]=[0.,grddd.crd1+grddd.csize[0]/2.*junk1] # $
                       make_array(grddd.dims[1]+1,value=1.)
  grddd.gp[*,*,iplane]=make_array(grddd.dims[0]+1,value=1.) # $
                       [0.,grddd.crd2+grddd.csize[1]/2.*junk2]
                       
; =============================================================================
; Prepare time stamps vector[NXI] for the grid. Time stamps (in UNIX seconds)
; represent the center points of the grid cells along the track (dblarr(nxi))

  junk1=min(reform(abs(dtsf[ibeam,*,0,0]-grddd.crd1[0])),ind1)
  junk1=min(reform(abs(dtsf[ibeam,*,0,0]-grddd.crd1[grddd.dims[0]-1])),ind2)
  junk1=spl_init(reform(dtsf[ibeam,ind1:ind2,0,0]),wcrddd.time[ind1:ind2],/double)
  grddd.gtime=spl_interp(reform(dtsf[ibeam,ind1:ind2,0,0]),$
                         wcrddd.time[ind1:ind2],junk1,grddd.crd1,/double)

; =============================================================================
; Determine pressure altitude of the 1st corner point using PALT

  grddd.alt0=acddd.palt[grddd.pfct[0]]-grddd.alt0                       

end