;$Id: wcrddinit.pro,v 1.1 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRDDINIT
;
; PURPOSE:
;       This routine is part of the WCR airborne Dual-Doppler analysis package
;       (wcrdd.pro). It reads/creates dual-Doppler input parameters file and 
;       defines data structures necessary to perform dual-Doppler analysis
;
; CATEGORY:
;       WCR wcrtools
;
; CALLING SEQUENCE:
;       wcrddinit,ddprmsf,fstruct,grddd,pcddd,wnddd,help=help
;
; INPUTS:
;       Ddprmsf: string, path/name to a data file containing Dual-Doppler
;                grid and other processing parameters. If the file does
;                not exist, it will be created interactively using wcrddprms.dat
;                default data file (located in wcrtools directory).
;
; KEYWORD PARAMETERS:
;       HELP:    shows this text
;
; OUTPUTS:
;       Fstruct: structure containing the names of the input/output files
;                Fstruct.wcrinf -> string, path/name wcr input raw file
;                Fstruct.acinf  -> string, path/name ac input netcdf file
;                Fstruct.prmsf  -> string, ddprmsf with user defined
;                                          dual-doppler input parameters
;                Fstruct.echof  -> string, path/name echo file of
;                                          dual-doppler parameters used
;                Fstruct.sndf   -> string, path/name to a sounding data file
;                                  containing: 1)wind data in 4 columns: 
;                                  altitude(msl), wind vel. components [m/s] 
;                                  along East,North,Up, or
;                                              2)AC altitude(AGL) in 2 col.:
;                                  unix_time[sec],agl[m]
;
;       Grddd:   structure containing grid parameters and other processing
;                option parameters(for description run wcrdd, /help)
;
;       Pcddd:   structure containing processing parameters
;                pcddd.cndnum -> float, condition number: max ratio between the
;                                largest and smallest singular values(s.v.).
;                                When calculating the reciprocals of the s.v.
;                                if this number is exceeded max(w(i)/w(i)>cndnum)
;                                then 1/w(i) is set to 0.
;                pcddd.inter  -> integer(0/1),interactive flag.
;                                 1 - user is allowed to stop the program after
;                                     some stats are presented.
;                pcddd.plts   -> integer(0/1), plots flag. {default=0}
;                                 1 - plot Eulerian AC angles and AC track
;                pcddd.tol    -> lonarr(2),[TOLI,TOLJ] i-idx,j-idx tolerances in
;                                the search for point-containing cells; a point
;                                appearing not to fall into the expected [i,j] cell
;                                might then be checked to belong to a cell in
;                                the interval [i-TOLI,i+TOLI;j-TOLJ,j+TOLJ]. It
;                                is suggested to use low values to speed up the
;                                process; it may be necessary to increase it if
;                                the AC motion is very "jittery".
;                pcddd.unfold -> integer[0/1/2], Unfolding flag:
;                                 0: no unfolding
;                                 1: Doppler vel. unfolded using the winds
;                                    component into the beams as measured
;                                    by the AC, profile by profile;
;                                 2: grddd.wadv advection velocity is used
;                                    as wind values to unfold the Doppler vel.
;                                    except for wadv=666, in which case it will
;                                    be interpreted as wadv=777 for unfolding
;                pcddd.whtprm -> fltarr(3-6), data weighting parameters:
;                                [Code,  Exp1, Exp2,  ROI,  Thresh1,  Thresh2]
;
;                                Code=0: NO weighting is applied.
;                                Code=1: IDW, inverse distance weighting
;                                        [Code,Exp1,-1,ROI,Thresh1,-1]
;                                Code=2: BSW, beam skewness weighting
;                                        [Code,-1,Exp2,-1,-1,Thresh2]
;                                Code=3: IDW+BSW
;                                        [Code,Exp1,Exp2,ROI,Thresh1,Thresh2]
;                                Code=4: CRW, Cressman weighting
;                                        [Code,-1,-1,ROI,Thresh1,-1]
;                                Code=5: CRW+BSW
;                                        [Code,-1,Exp2,ROI,Thresh1,Thresh2]
;                                Code=6: EXW, exponential weighting
;                                        [Code,Exp1,-1,ROI,Thresh1,-1]
;                                Code=7: EXW+BSW
;                                        [Code,Exp1,Exp2,ROI,Thresh1,Thresh2]
;
;                                Exp1/2:    exponent value(s) in the power 
;                                           weighting for the weigthing method(s) 
;                                           used; suggested for the IDW is 1.0 
;                                           or 2.0 and for the EXW, 0.1
;
;                                ROI:       radius of influence; 99999 will use 
;                                           the average size of the grid cell 
;                                           (DHOR+DVER)/2)
;
;                                Thresh1/2: cutoff threshold to discard points
;                                           (default -1, no thresholding) that:
;                                  IDW: that fall beyond 
;                                       (1+Thresh1)*(min(dist)>(0.1*ROI)), where
;                                       dist is the distance of the points from
;                                       the grid cell center
;                                  BSW: are associated with normal-to-reference-
;                                       plane components of the beam unit vectors 
;                                       (normals) beyond (1+Thresh2)*min(normals). 
;                                       If min(normals) is less than sin(1 deg)
;                                       then threshold is (1+Thresh2)*sin(1deg). 
;                                       E.g., whtprm[2]=0. will cause all data 
;                                       points that deviate more than 
;                                       min(1 deg,asin(min(normals))) from the
;                                       desired grid plane to be discarded.
;                                  CRW: that fall beyond 
;                                       (1+Thresh1)*(min(dist)>(0.1*ROI)), where
;                                       dist is the distance of the points from
;                                       the grid cell center
;                                  EXW: that fall beyond 
;                                       (1+Thresh1)*(min(dist)>(0.1*ROI)), where
;                                       dist is the distance of the points from
;                                       the grid cell center
; 
;                pcddd.xytol  -> fltarr(2),[xi-tolerance,eta-tolerance],
;                                (size relative).Tolerances in xi,eta directions
;                                in the search for point-containing cells.
;                                Example:[0.2,0.3] means a point is considered
;                                within cell[i,j] if it falls in a cube of size
;                                (xi_size+/-0.2*xi_size)*(eta_size+/-0.3*eta_size)*swth.
;
;       Wnddd:   structure containing external wind info processing parameters
;                wnddd.d0     -> float, Displacement height.(Usually~0.3 h0).
;                                Used in making a log vertical profile.
;                wnddd.hf     -> float, Factor describing the ratio between
;                                flight level altitude and boundary layer height
;                                zi*hf=palt. Used in making Ekman's Layer
;                                vertical profile, where above and at the top
;                                of the boundary layer conditions are supposed
;                                to be constant and geostrophic.
;                wnddd.kappa  -> float,Von-Karmann's constant. Used
;                                in making a log vertical profile.(default=0.41)
;                wnddd.wa     -> float,"a" coeff. of the parabola describing
;                                the vertical profile of the vertical velocity,
;                                w. Used in making a log vertical profile or
;                                Ekman's Layer vertical profile:
;                                       w=a(z-z0)^2.+b(z-z0), where b is calc. 
;                                based on flight level boundary condition
;                wnddd.wext   -> fltarr(NXI,NET,3),(U,V,W) components in the
;                                GLOBAL WBRS(=UFRS) of the external wind.
;                wnddd.wflag  -> integer, External info about the winds for SVDC:
;                                 0: No external wind info is used.
;                                 1: Use whatever has been adopted for wadv;
;                                    if it is not a constant wind vector then
;                                    replicate 'profile by profile wind' along
;                                    the other direction.
;                                 2: Use a vertical sounding,stored in a file.
;                                 3: Use whatever criteria adopted in wadv
;                                    and a vertical log profile along the normal
;                                    to track direction (valid for vertical scan):
;                                       u=u_* / k *ln((z-d0)/z0);
;                                    Vertical velocity is calculated by:
;                                       w=a(z-z0)^2.+b(z-z0) (a=wnddd.wa) 
;                                 4: Use whatever criteria adopted in wadv and
;                                    a vertical Ekman's profile along the normal
;                                    to track direction (valid for vertical scan):
;                                       u= ug*(1-exp(-c*z)cos(c*z)),
;                                       v= ug*exp(-c*z)sin(c*z),
;                                       c=(f/2*nu)^0.5, calc. using wnddd.hf
;                                    Vertical velocity is calculated by:
;                                       w=a(z-z0)^2.+b(z-z0) (a=wnddd.wa) 
;                                 5: Use point-by-point AC-measured winds,
;                                    independently from wadv.
;                                 6: Use averaged AC-measured wind
;                                    independently from wadv.
;                                 7: Use average coming from the corrected
;                                    velocities, before the dual-doppler
;                                    processor.(It is a better estimate of
;                                    the mean flow from the doppler field).
;                wnddd.z0     -> float, Roughness height.(Usually~0.1 h0).
;                                Used in making a log vertical profile.
;                wnddd.zagl   -> float, flag determining how surface elevation
;                                is calculated:
;                                 0:     use mean AC radar altimeter altitude
;                                 1:     use radar alt. redistributed along xi
;                                 2:     use altitude(AGL) data stored in a file
;                                        (not implemented yet)
;                                 float: use given value as const. AGL in meters;
;                                        must be greater than or equal to 75.0 m
;                                 float: threshold in dBZ used to find surface 
;                                        from WCR straight beam reflectivity;
;                                        must be less than 75.0 dBZ. Surface
;                                        elevation is assigned interpolating
;                                        WCR surface range gates data along xi.
;
; SIDE EFFECTS:
;      The input file ddprmsf is an ascii file and is expected to comply with
;      the following rules:
;
;   0) Data values equal to 99999 mean that actual values will be calculated by
;      an appropriate wcrdd*.pro routine. These values can be found in the
;      echo file generated by wcrddecho.pro.
;   1) All non-data lines must have 1st char ";" followed by "#"            
;      Ending non-data lines using "#" followed by ";" as last char is     
;      optional for generating better looking report if needed.             
;   2) First data line must contain the type of flight leg (leg=0|1) to
;      be processed and grid dimensions NXI and NET.
;   3) Data lines can have spaces anywhere except in the input values.      
;      Data lines cannot be empty lines. If a data file path/name is not    
;      known use '*'.  This will allow you to select a path/file.           
;   4) Empty lines and lines with spaces are allowed anywhere. They are     
;      ignored.                                                             
;   5) The line preceeding a data line (ignoring white space lines) must be 
;      a comment line containing the names of the parameters given in the   
;      data line.  These names will be used to generate the proper input    
;      data in the appropriate data structures.                             
;   6) Parameter names must match structure tag names (case  insensitive)   
;      in grddd, pcddd and wnddd. Their meaning can be found in the help of 
;      wcrddinit.pro. Tag names must be unique for all three structures     
;      Same name may appear in sequence which means that the structure field
;      is an array.                                                         
;   7) If the line containing the names have same name more than once (array
;      field the line before it is an additional name line given names for  
;      each element of the array.  This is needed only if wcrddinit is used 
;      to create this file and compliments prompt info during input process.
;   8) The comment lines preceeding the data file names are identifier lines
;      and must contain the following identifiers: WCR_FILE, AC_FILE,       
;      GRID_PROC_FILE, ECHO_FILE, SOUNDING_FILE separated by spaces from other  
;      informational text                                                   
;   9) Except for the data lines and the corresponding preceeding comment   
;      lines with the parameters names all other comment lines are ignored. 
;  10) The file must start with at least 6 comment lines. They are reserved 
;      for program messages. 
;
; NOTES:
;      Some components of the output structures are defined in this procedure
;      but their actual values are calculated elsewhere
;
; EXAMPLE:
;
;-
; MODIFICATION HISTORY:
;    Written by: Rick Damiani and Samuel Haimov, August 2003
;    S.H., Jan 04 - changed meaning of sounding file; modified wnddd.zalt
;    S.H., Mar 04 - added wcrt field in grddd giving the start and stop time
;                   for the WCR data used. They are populated in wcrddgrid0/1
;                   and used by wcrddecho and wcrddtplot routines
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.
;

pro wcrddinit, ddprmsf,fstruct,grddd,pcddd,wnddd,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrddinit'
    return
  endif

  if (n_params() eq 0) then begin
    message,"Usage: wcrddinit,ddprmsf,fstruct,grddd,pcddd,wnddd,help=help",/info
    return
  endif

; Initilize

  cs0     =';#'               ; beginning comment string id
  cs1     ='#;'               ; ending comment string id
  wcrlib  ='wcrtools'         ; WCR idl library
  wcrdddat='wcrddprms.dat'    ; WCR dual-doppler parameters default file
  dflt    = 99999             ; code value for default

; ............................................................................
; Check if ddprmsf exists. If so read the content otherwise read wcrddprms.dat
; with the default parameter values located in wcrlib directory

  junk=file_test(ddprmsf,/read)
  if junk then begin
    flg=0B
    fstr=getfile(ddprmsf)
  endif else begin
    flg=1B
    junk1=strpos(!path,wcrlib)
    if junk1 eq -1 then begin
      message,wcrlib+' library is not in the idl path'
    endif else begin
      junk2=strpos(!path,':',junk1,/reverse_search)
      junk=strmid(!path,junk2+1,junk1-junk2-1)
    endelse
    fstr=getfile(junk+wcrlib+'/'+wcrdddat)
  endelse

; ............................................................................
; Find all no comments lines and get data and name lines

  ind     = where(strmid(fstr,0,2) ne cs0 and fstr ne '')
  nind    = n_elements(ind)-1
  datstr  = strtrim(fstr[ind],2)
  namestr = strmid(strtrim(fstr[ind-1],2),2)
  name2str= strmid(strtrim(fstr[ind-2],2),2)
  nfstr   = n_elements(fstr)

; ............................................................................
; If requested ask for the desired input parameters and write ddprmsf

  if flg then begin

  ; Ask for the input parameters; avoid GRID_PROC_FILE line

    junk=strupcase(strmid((strtrim(namestr,1)),0,14))
    ind1=where(strmid(junk,0,14) eq 'GRID_PROC_FILE')

    junk=''
    for k=0,nind do begin
      if k ne ind1[0] then begin
        junk1=strsplit(datstr[k],' ',/extract)
        junk2=strsplit(namestr[k],' ',/extract)
        junk3=strsplit(name2str[k],' ',/extract)
        cnt=0
        for kk=0,n_elements(junk1)-1 do begin
          if junk1[kk] eq '*' then begin
            junk1[kk]=dialog_pickfile(/read,tit='Select '+junk2[kk]+' or Cancel')
          endif else begin
            junk5=n_elements(where(junk2[kk] eq junk2))
            if junk5 eq 1 then junk4=junk2[kk] else begin
              junk4=junk2[kk]+'|'+junk3[cnt]
              cnt=cnt+1
            endelse
            read,prompt='Enter '+junk4+'(default '+junk1[kk]+'): ',junk
            if junk ne '' then junk1[kk]=strcompress(junk,/remove_all)
          endelse
        endfor
        if (strlen(junk1))[0] gt 10 and n_elements(junk1) eq 1 then $
          datstr[k]='   '+junk1[0] $
        else datstr[k]=strjoin(stradj(junk1,len=10,adj=1))
        fstr[ind[k]]=datstr[k]
      endif else begin
        datstr[k]='   '+ddprmsf
        fstr[ind[k]]=datstr[k]
      endelse
    endfor

  ; Write the date the file is generated in 4th line of the file

    junk1=strmid(ddprmsf,strpos(ddprmsf,'/',/reverse_search)+1)
    junk2=cs0+' '+junk1+' generated: '
    junk3=junk2+systime()+string(replicate(32B,54-strlen(junk2)))+cs1
    fstr[3]=junk3

  endif

  ; Write the date of the last read in the 5th line of the file

  junk1=cs0+' file last read: '
  junk2=junk1+systime()+string(replicate(32B,54-strlen(junk1)))+cs1
  fstr[4]=junk2

  ; Update ddprmsf

  putfile,ddprmsf,fstr

; ............................................................................
; Define output structures

  ; get the dimensions NXI and NET; must be the first data line in datstr

  datstr=strtrim(datstr)
  junk1=strsplit(datstr[0],' ',/extract)
  LEG=long(junk1[0])
  NXI=long(junk1[1])
  NET=long(junk1[2])

  ; structure containing input and echo data file names

  fstruct= {wcrinf:'',acinf:'',prmsf:'',echof:'',sndf: ''}

  ; structure for dual-doppler grid parameters

  grddd=  {alt0:  0.,                   crd1:  fltarr(NXI), crd2:  fltarr(NET),$
           csize: fltarr(2),            dims:  [NXI,NET],   equid: 0,          $           
           gp:    fltarr(NXI+1,NET+1,3),gtime: dblarr(NXI), leg:   LEG,        $
           let:   0.,                   pfct:  lonarr(2),   ptrk:  0,          $
           RM2:   fltarr(3,3,NXI*LEG>1),rst:   0,           srat:  fltarr(2),  $
           wcrt:  lonarr(2),                   swth:  0.,   wadv:  fltarr(3)}

  ; structure for dual-doppler processing parameters

  pcddd=  {cndnum: 0., inter:  0,         tol:    lonarr(2),    plts:   0 , $
           unfold: 0,  whtprm: fltarr(6), xytol:  fltarr(2)}

  ; structure for the external wind data

  wnddd={d0: 0., hf:   0., kappa: 0.,wa: 0.,wext: fltarr(NXI,NET,3),wflag: 0, $
         z0: 0., zagl: 0.}

; ............................................................................
; Fill in the values for output structures according to to ddprmsf

  ; Find all tag names

  grdtags=tag_names(grddd)
  pctags=tag_names(pcddd)
  wntags=tag_names(wnddd)

  ; Search for the tag names and assign the appropriate values

  for k=1,nind do begin
    junk1=strsplit(datstr[k],' ',/extract)
    junk2=strsplit(namestr[k],' ',/extract)
    junk3=1
    kk=0
    while kk le n_elements(junk1)-1 do begin
    ; search grddd tags ...............................     ; !!!!!!!!!!!!!!!!!!
      ind1=(where(grdtags eq strupcase(junk2[kk])))[0]      ; without making ind1
      if ind1[0] ne -1 then begin                           ; scalar where in the
        junk3=fix(n_elements(where(junk2 eq grdtags[ind1]))); line won't work right
        grddd.(ind1)=junk1[kk:kk+junk3-1]
      endif
    ; search pcddd tags................................
      ind1=(where(pctags eq strupcase(junk2[kk])))[0]
      if ind1[0] ne -1 then begin
        junk3=n_elements(where(junk2 eq pctags[ind1]))
        pcddd.(ind1)=junk1[kk:kk+junk3-1]
      endif
    ; search wnddd tags................................
      ind1=(where(wntags eq strupcase(junk2[kk])))[0]
      if ind1[0] ne -1 then begin
        junk3=n_elements(where(junk2 eq wntags[ind1]))
        wnddd.(ind1)=junk1[kk:kk+junk3-1]
      endif
      kk=kk+junk3
    endwhile
  endfor

  ; Substitute 99999 with the known defaults

  if grddd.equid     eq dflt then grddd.equid    = 0
  if grddd.leg       eq dflt then grddd.leg      = 0
  if grddd.ptrk      eq dflt then grddd.ptrk     = 0
  if grddd.rst       eq dflt then grddd.rst      = 0
  if grddd.srat[0]   eq dflt then grddd.srat[0]  = 0.
  if grddd.srat[1]   eq dflt then grddd.srat[1]  = 0.
  if grddd.swth      eq dflt then grddd.swth     = 5000.
  if grddd.wadv[0]   eq dflt then grddd.wadv[0]  = 0.
  if grddd.wadv[1]   eq dflt then grddd.wadv[1]  = 0.
  if grddd.wadv[2]   eq dflt then grddd.wadv[2]  = 0.

  if pcddd.cndnum    eq dflt then pcddd.cndnum   = 100.
  if pcddd.inter     eq dflt then pcddd.inter    = 0
  if pcddd.plts      eq dflt then pcddd.plts     = 0
  if pcddd.unfold    eq dflt then pcddd.unfold   = 0
  if pcddd.xytol[0]  eq dflt then pcddd.xytol[0] = 0.
  if pcddd.xytol[1]  eq dflt then pcddd.xytol[1] = 0.

  if pcddd.whtprm[0] eq dflt then pcddd.whtprm[0]= 0. $
  else if pcddd.whtprm[0] gt 0 then begin
    if pcddd.whtprm[1] eq dflt then $
      pcddd.whtprm[1]= 2.*(pcddd.whtprm[0] lt 6)+0.1*(pcddd.whtprm[0] gt 5) 
    if pcddd.whtprm[2] eq dflt then pcddd.whtprm[2]=  2.
    if pcddd.whtprm[4] eq dflt then pcddd.whtprm[4]= -1.
    if pcddd.whtprm[5] eq dflt then pcddd.whtprm[5]= -1.    
  endif

  if wnddd.d0        eq dflt then wnddd.d0       = 0.
  if wnddd.hf        eq dflt then wnddd.hf       = 0.
  if wnddd.kappa     eq dflt then wnddd.kappa    = 0.41
  if wnddd.wa        eq dflt then wnddd.wa       = 0.
  if wnddd.wflag     eq dflt then wnddd.wflag    = 1
  if wnddd.z0        eq dflt then wnddd.z0       = 0.
  if wnddd.zagl      eq dflt then wnddd.zalg     = 0.

  ; Find the data file names; use id strings

  junk=strupcase(strmid(strtrim(namestr,1),0,14))
  ind1=lonarr(5)
  ind1[0]=where(strmid(junk,0,7)  eq 'AC_FILE')
  ind1[1]=where(strmid(junk,0,8)  eq 'WCR_FILE')
  ind1[2]=where(strmid(junk,0,14) eq 'GRID_PROC_FILE')
  ind1[3]=where(strmid(junk,0,9)  eq 'ECHO_FILE')
  ind1[4]=where(strmid(junk,0,9)  eq 'SOUNDING_FILE')

  if ind1[4] eq -1 and wnddd.wflag eq 2 then begin
    message,'SOUNDING_FILE id and/or path/file strings are missing'
  endif else begin
    junk =where(ind1[0:3] eq -1)
    junk1=where(datstr[ind1[0:3]] eq '')
    if junk[0] ne -1 or junk1[0] ne -1 then $
      message,'There are missing file IDs or path/file strings'
  endelse

  ; Fill in the file structure

  fstruct.acinf =datstr[ind1[0]]
  fstruct.echof =datstr[ind1[3]]
  fstruct.prmsf =ddprmsf
  fstruct.wcrinf=datstr[ind1[1]]
  if wnddd.wflag eq 2 then fstruct.sndf=datstr[ind1[4]]

end

