;$Id
;+
; NAME:
;      WCRDDPROC0
;
; PURPOSE:
;      This routine is part of the WCR airborne Dual-Doppler analysis package
;      (wcrdd.pro).It calculates dual-Doppler velocities using straight leg 
;      grid.
;
; CATEGORY:
;      WCR wcrtools
;
; CALLING SEQUENCE:
;      wcrddproc0,beamopt,dtsf,wcrddd,grddd,pcddd,ddd,dddaux,pcstats,
;                 winds_ext=winds_ext,help=help
;
; INPUTS:
;      Beamopt: integer,dual-antenna configuration 
;                0    - N2UW KA dual-down antennas (h-down_fore, v-nadir)
;                1    - N2UW KA dual-side antennas (h-side, v-side_fore)
;                2    - NCAR C130 dual-down antennas (h-nadir,v-down-aft)
;      Dtsf:    fltarr(2,n,rg,3), h-beam(dtsf[0,*,*,*]) and v-beam(dtsf[1,*,*,*])
;               data points(n: profiles, rg: range gates) coordinates in WBRRS 
;      Wcrddd:  structure containing WCR data(for description run wcrdd, /help)
;      Grddd:   structure containing grid parameters and other processing
;               option parameters(for description run wcrdd, /help)
;      Pcddd:   structure containing processing parameters(for description run
;               wcrddinit, /help)
;
; OUTPUTS:
;      Ddd:     structure containing the dual-Doppler analysis output data
;               (for description run wcrdd, /help)
;      Dddaux:  structure containing some auxiliary data resulting from the 
;               dual-Doppler processing(for description run wcrdd, /help)
;      Pcstats: structure containing some statistics of the result from the
;               processing. The data arrays used are contained in Dddaux struct.
;                Pcstats.inpts  -> fltarr(4),mean,sdev,min,max of the
;                                  number of points per cell from both channels
;                Pcstats.hinpts -> fltarr(4),mean,sdev,min,max of the number of
;                                  points per cell from h-channel
;                Pcstats.vinpts -> fltarr(4),mean,sdev,min,max of the number of 
;                                  points per cell from v-channel
;                Pcstats.swpts  -> long, number of points falling out of grid
;                                  due to limited swath
;                Pcstats.angerr -> fltarr(4),[rad] mean,sdev,min,max of the
;                                  dddaux.ang_err 
;                Pcstats.crank1 -> % of grid cells with SVDC matrix rank < 2
;                Pcstats.empty  -> % of grid cells with no data points accepted
;                                  (either below threshold or geometrically rejected)
;                
;                Note: The statistics for inpts,hinpts,vinpts,swpts, and crank1
;                      is based on the grid cells that contain at least one
;                      data point that was used in the velocity retrieval.
;
; KEYWORD PARAMETERS:
;      WINDS_EXT: input,fltarr(NXI,NET,3), external info on the winds in UFRS
;                 (output from wcrddwext.pro; see also wnddd.wext in wcrddinit).
;                 Will be transformed into WBBRS and then passed to wcrdddv.pro.
;                 It is needed to correct for the wind perpendicular to the 
;                 plane of the beams. If not defined or 0 external wind not used.
;
;      HELP:      shows this text
;
; SIDE EFFECTS:
;
; PROCEDURE:
;
; EXAMPLE:
;
;-
; MODIFICATION HISTORY:
;    Written by: Rick Damiani and Samuel Haimov, November 2003
;    S.H., Dec 03 - added pcstats.nempty and modified some of the stat calc.
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.
;

pro wcrddproc0,beamopt,dtsf,wcrddd,grddd,pcddd,ddd,dddaux,pcstats, $
               winds_ext=winds_ext,help=help
  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrddproc0'
    return
   endif

  if (n_params() lt 6)  then begin
    message,"Usage: wcrddproc0,beamopt,dtsf,wcrddd,grddd,pcddd,ddd, $",/info
    print,"                     dddaux,pcstats,winds_ext=winds_ext,help=help)"
    return
  endif

  ; ...........................................................................
  ; Depending on the beam config find which grid plane (VDD or HDD) to process

  if beamopt ne 1 then iplane =2 else iplane =1  ; 1=side,eta or 2=vertical,zeta

  ; ...........................................................................
  ; Transform wcr beam pointing vectors from UFRS to WBRRS
  ; beamh(n,3) -> bhrm2(n,3); beamv(n,3) -> bvrm2(n,3)

  bhrm2=fltarr(wcrddd.n,3)
  bvrm2=fltarr(wcrddd.n,3)

  for k=0,wcrddd.n-1 do begin
    bhrm2[k,*]=grddd.RM2 ## wcrddd.beamh[k,*]
    bvrm2[k,*]=grddd.RM2 ## wcrddd.beamv[k,*]
  endfor

; =============================================================================
; Assign (beam) data points to the grid cells they belong to

  ; ...........................................................................
  ; Prepare the output structures; use NaNs as missing values

  ddd=   {                                                            $ 
          vel:   fltarr(grddd.dims[0],grddd.dims[1],3)*!values.f_nan, $
          winfo: fltarr(grddd.dims[0],grddd.dims[1],3),               $
          z:     fltarr(grddd.dims[0],grddd.dims[1],3)*!values.f_nan  $
; after wcrddproc2 is modified change the name from z to dbz
;         dbz:   fltarr(grddd.dims[0],grddd.dims[1],3)*!values.f_nan  $
         }
  dddaux={                                                            $ 
          ang_err:fltarr(grddd.dims[0],grddd.dims[1])*!values.f_nan,  $
          dtsf:   dtsf,                                               $   
          h_ndx:  ptrarr(grddd.dims[0],grddd.dims[1],/alloc),         $
          h_npts: intarr(grddd.dims[0],grddd.dims[1]),                $
          v_ndx:  ptrarr(grddd.dims[0],grddd.dims[1],/alloc),         $
          v_npts: intarr(grddd.dims[0],grddd.dims[1]),                $
          swthpts:intarr(grddd.dims[0],grddd.dims[1])                 $
          }

  pcstats={inpts: fltarr(4), hinpts: fltarr(4), vinpts: fltarr(4),          $ 
           swpts: fltarr(4), angerr: fltarr(4), crank1: 0.       , empty: 0.}

  ; ...........................................................................
  ; Calculate max coordinates (xi,eta) of the grid

  mxi =grddd.crd1[grddd.dims[0]-1]+0.5*grddd.csize[0]*grddd.srat[0]^(grddd.dims[0]-1)
  meta=grddd.crd2[grddd.dims[1]-1]+0.5*grddd.csize[1]*grddd.srat[1]^(grddd.dims[1]-1)

  ; ...........................................................................
  ; Distribute data points for both beams, all range gates and appropriate profs
  ; Along this process save data for statistical analysis

  for k=0,1 do begin                              ; h and v beams
    for l=grddd.pfct[0],grddd.pfct[1] do begin    ; profiles belong to the grid
      for m=0L,wcrddd.rg-1 do begin                ; all range gates

      ; save data point coordinates (for more convinient use)
        xp=dtsf[k,l,m,0]         ; dimension along (straight) AC track
        yp=dtsf[k,l,m,iplane]    ; dimension across (straight) AC track
        zp=dtsf[k,l,m,3-iplane]  ; dimension along swath

      ; distribute a data point, which is not a missing value and is in the grid
      ; boundaries
        if ((k eq 0)*finite(wcrddd.zh[l,m]) or k*finite(wcrddd.zv[l,m])) and $
        (xp ge 0) and (xp le mxi) and (yp ge 0) and (yp le meta) then begin

	     if grddd.srat[0] eq 1. then $ ;"<NXI-1" include pts for which xp=mxi                      
               i=floor(xp/grddd.csize[0])<(grddd.dims[0]-1) $           
          else i=ceil(alog10(1.-xp/grddd.csize[0]*(1.-grddd.srat[0]))/ $
                      alog10(grddd.srat[0])-1)<(grddd.dims[0]-1) 

          if grddd.srat[1] eq 1. then $ ;"<NET-1" include pts for which yp=meta 
               j=floor(yp/grddd.csize[1])<(grddd.dims[1]-1) $
          else j=ceil(alog10(1.-yp/grddd.csize[1]*(1.-grddd.srat[1]))/ $
                      alog10(grddd.srat[1])-1)<(grddd.dims[1]-1) 

          if abs(zp) gt grddd.swth/2. then $
          ; exclude a data point extending beyond the allowed swath
            dddaux.swthpts[i,j]=dddaux.swthpts[i,j]+1 $
          else begin ; data point is in the allowed swath 
          ; count all accepted data points and save their 2-D indices as 1-D ind.
            if k then begin
              if n_elements(*dddaux.v_ndx[i,j]) eq 0 then $
                *dddaux.v_ndx[i,j]=l+m*wcrddd.n $
              else $
                *dddaux.v_ndx[i,j]=[*dddaux.v_ndx[i,j],l+m*wcrddd.n]
              dddaux.v_npts[i,j]=dddaux.v_npts[i,j]+1
            endif else  begin
              if n_elements(*dddaux.h_ndx[i,j]) eq 0 then $
                *dddaux.h_ndx[i,j]=l+m*wcrddd.n $
              else $
                *dddaux.h_ndx[i,j]=[*dddaux.h_ndx[i,j],l+m*wcrddd.n]
              dddaux.h_npts[i,j]=dddaux.h_npts[i,j]+1
            endelse
          endelse

        endif

      endfor                                      ; m, end range gate loop
    endfor                                        ; l, end profile loop
  endfor                                          ; k, end beam loop  

  ; ...........................................................................
  ; Determine the total number of accepted data points for each cell and the
  ; indices of the cell that contain them and the number of empty cells

  npts=dddaux.h_npts+dddaux.v_npts
  ndx=where(npts gt 0,ncompl=nempty)

; =============================================================================
; Calculate 3-D velocities from WCR dual-beam radial velocity and ext. wind info

  ; ...........................................................................
  ; Initialize needed vectors

    ; vnorm: normal to the solution plane (best fit beam plane)
    vnorm=0. 

    ; nrp: normal to the local reference plane for straight leg (leg=0);
    ; it is just one plane in WBRRS normal to the desired scanning plane
    ; NOTE: will be used by the weighing procedures and by the procedure to 
    ; find the angle between the solution plane and the desired scanning plane.

    if iplane eq 2 then nrp=[0.,1.,0.] else nrp=[0.,0.,1.] 

  ; ...........................................................................
  ; Process radial velocities for every gridd cell

    for i=0,grddd.dims[0]-1 do begin
      for j=0,grddd.dims[1]-1 do begin

      ; Incorporate external wind (if any) in WBRRS
      if n_elements(winds_ext) gt 0 then $
        ddd.winfo[i,j,*]= grddd.RM2 ## reform(winds_ext[i,j,*])

      ; For the non-empty cells prepare input to SVD solver 
        if npts[i,j] ne 0 then begin

        ; ---------------------------------------------------------------------  
        ; Process h-beam accepted data points

          if dddaux.h_npts[i,j] ne 0 then begin

            ind = (*dddaux.h_ndx[i,j])[0:dddaux.h_npts[i,j]-1] ; 1-D indices
            ind1= ind mod wcrddd.n                             ; profile indices
            LSM = bhrm2[ind1,*]      ; SVDC not weighted system matrix; [ind1,3] 

            ; Weight the data points if requested (otherwise use weights=1)

            if pcddd.whtprm[0] gt 0 then begin

              ; determine ROI for whtprm if not given
              if pcddd.whtprm[3] gt 99999 then $
                pcddd.whtrprm[ind2[0]]=total(grddd.size)/2.

              ind2=ind/wcrddd.n                               ; rg indices
              d0=[grddd.crd1[i],grddd.crd2[j],0.]             ; cell[i,j] center 

              ; find all h-beam valid points in dtsf; dtsf[ind]=dtsf[0,pf,rg,3];

              ind3=[2*ind1 + 2*ind2*wcrddd.n,                         $
                    2*ind1 + 2*ind2*wcrddd.n + 2*wcrddd.n*wcrddd.rg,  $
                    2*ind1 + 2*ind2*wcrddd.n + 2*wcrddd.n*wcrddd.rg*2 ]

              d=reform(dtsf[ind3],dddaux.h_npts[i,j],3)

              ; calculate the weights and then weight them again according to
              ; the fraction h_npts[i,j]/npts[i,j] to secure total weight = 1

              whts=wcrddwhts(d,d0,LSM,nrp,pcddd.whtprm)*dddaux.h_npts[i,j]
              if total(whts) eq 0 then $
                message,'For cell['+num2str(i)+','+num2str(j)+ $
                        ']: h_beam weights are all zero',/info

            endif else whts=make_array(dddaux.h_npts[i,j],value=1.)

            ; weigh h-refl., svdc system matrix, and accepted radial vel.(LSV)

            ddd.z[i,j,0]=10.*alog10(total(10.^(wcrddd.zh[ind]/10.)*whts)/ $
                                    dddaux.h_npts[i,j])
            LSM=LSM*(whts/npts[i,j]#(1.+fltarr(3)))
            LSV=-wcrddd.dvh[ind]*whts/npts[i,j]

          endif    ; end h-beam accepted data points

        ; ---------------------------------------------------------------------  
        ; Process v-beam accepted data points

          if dddaux.v_npts[i,j] ne 0 then begin

            ind = (*dddaux.v_ndx[i,j])[0:dddaux.v_npts[i,j]-1] ; 1-D indices
            ind1= ind mod wcrddd.n                             ; profile indices
            junk= bvrm2[ind1,*]      ; SVDC not weighted system matrix; [ind1,3] 

            ; Weight the data points if requested (otherwise use weights=1)

            if pcddd.whtprm[0] gt 0 then begin

              ; determine ROI for whtprm if not given
              if pcddd.whtprm[3] gt 99999 then $
                pcddd.whtrprm[ind2[0]]=total(grddd.size)/2.

              ind2=ind/wcrddd.n                               ; rg indices
              d0=[grddd.crd1[i],grddd.crd2[j],0.]             ; cell[i,j] center 

              ; find all v-beam valid points in dtsf; dtsf[ind]=dtsf[1,pf,rg,3];
              ind3=[1+2*ind1 + 2*ind2*wcrddd.n,                         $
                    1+2*ind1 + 2*ind2*wcrddd.n + 2*wcrddd.n*wcrddd.rg,  $
                    1+2*ind1 + 2*ind2*wcrddd.n + 2*wcrddd.n*wcrddd.rg*2 ]

              d=reform(dtsf[ind3],dddaux.v_npts[i,j],3)

              ; calculate the weights and then weight them again according to
              ; the fraction v_npts[i,j]/npts[i,j] to secure total weight = 1

              whts=wcrddwhts(d,d0,junk,nrp,pcddd.whtprm)*dddaux.v_npts[i,j]
              if total(whts) eq 0 then $
                message,'For cell['+num2str(i)+','+num2str(j)+ $
                        ']: v_beam weights are all zero',/info

            endif else whts=make_array(dddaux.v_npts[i,j],value=1.)

            ; weigh v-refl., svdc system matrix, and accepted radial vel.(LSV);
            ; append to LSM and LSV from h-beam and calc. mean refl. (if any)

            ddd.z[i,j,1]=10.*alog10(total(10.^(wcrddd.zv[ind]/10.)*whts)/ $
                                    dddaux.v_npts[i,j])
            junk=junk*(whts/npts[i,j]#(1.+fltarr(3)))
            if dddaux.h_npts[i,j] ne 0 then begin
              LSM=[LSM,junk]
              LSV=[LSV,-wcrddd.dvv[ind]*whts/npts[i,j]]
              ddd.z[i,j,2]=10.*alog10(10.^(ddd.z[i,j,1]/10.)+$
                                      10.^(ddd.z[i,j,0]/10.))-3.0103
            endif else begin
              LSM=junk
              LSV=-wcrddd.dvv[ind]*whts/npts[i,j]
            endelse

          endif    ; end v-beam accepted data points
        ; ---------------------------------------------------------------------  
        ; Run SVD solver
        
          LSM=transpose(reform(LSM)) ; [3,npts[i,j]]
          LSV=reform(LSV)

          ddd.vel[i,j,*]=wcrdddv(LSM,LSV,v_ext=reform(ddd.winfo[i,j,*]), $
                                 vnorm=vnorm,condnum=pcddd.cndnum)
 
         ; Calculate angle between solution plane and desired scanning plane
         ; (vnorm is the normal to the best-fit plane in the WBRRS)

         if n_elements(vnorm) ne 1 then $
           dddaux.ang_err[i,j]=acos(abs(transpose(vnorm)#nrp)) $ ;[rad]
         else pcstats.crank1=pcstats.crank1+1L

        endif      ; end non-empty grid cells

      endfor       ; end i, xi loop
    endfor         ; end j, eta loop

; =============================================================================
; Calculate some statistics for the processed data

  ; ...........................................................................
  ; statistics for all data points falling out due to swth.
  pcstats.swpts=[mean(dddaux.swthpts[ndx]),stddev(dddaux.swthpts[ndx]),$
                 min(dddaux.swthpts[ndx]), max(dddaux.swthpts[ndx])]
  if pcstats.swpts[3] gt 0 then $
    message,'Total number of points rejected due to limited swath: '+$
    num2str(total(dddaux.swthpts[ndx])),/info  ; using all is identical to [ndx]

  ; ...........................................................................
  ; statistics for accepted h-beam data points 
  pcstats.hinpts=[mean(dddaux.h_npts[ndx]),std(dddaux.h_npts[ndx]),$
                  min(dddaux.h_npts[ndx]),max(dddaux.h_npts[ndx])]

  ; ...........................................................................
  ; statistics for accepted v-beam data points 
  pcstats.vinpts=[mean(dddaux.v_npts[ndx]),std(dddaux.v_npts[ndx]),$
                  min(dddaux.v_npts[ndx]),max(dddaux.v_npts[ndx])]

  ; ...........................................................................
  ; statistics for all accepted data points 
  pcstats.inpts= [mean(npts[ndx]),stddev(npts[ndx]),min(npts[ndx]),max(npts[ndx])]

  ; ...........................................................................
  ; statistics for the angle between best-fit plane & expected scanning plane 
  pcstats.angerr=[mean(dddaux.ang_err,/nan),std(dddaux.ang_err,/nan),$
                  min(dddaux.ang_err,/nan),max(dddaux.ang_err,/nan)]  ;[rad]

  ; ...........................................................................
  ; % of grid cells for which SVDC system matrix has rank less than 2
  pcstats.crank1=100.*pcstats.crank1/n_elements(ndx)

  ; ...........................................................................
  ; % of empty grid cells 
  pcstats.empty=100.*float(nempty)/(grddd.dims[0]*grddd.dims[1])

end


