;$Id:
;+
; NAME:
;      WCRDDWEXT
;     
; PURPOSE:
;      This routine is part of the WCR airborne Dual-Doppler analysis package
;      (wcrdd.pro). It determines external info on winds. Winds given as U,V,W 
;      in the UFRS, for every cell in the grid.
;
; CATEGORY:
;      WCR wcrtools
;
; CALLING SEQUENCE:
;      wcrddwext,beamopt,acddd,wcrddd,grddd,wnddd,strk,strk_g,sndfile=sndfile,$
;                help=help
;
; INPUTS:
;      Beamopt: integer,dual-antenna configuration 
;                0    - N2UW KA dual-down antennas
;                1    - N2UW KA dual-side antennas
;                2    - NCAR C130 dual-down antennas 
;      Acddd:   structure containing AC data used for the dual-Doppler
;               analysis (see wcrdd.pro help for more info)
;      Wcrddd:  structure containing WCR data used for the dual-doppler
;               analysis (see wcrdd.pro help for more); used if wnddd.wflag=7
;      Grddd:   structure containing grid parameters and other processing
;               option parameters (see wcrdd.pro help for more info)
;      Wnddd:   structure containing external wind info processing parameters
;               (see wcrddinit.pro help for more info)
;      Strk:    fltarr(n), s-coordinate for the length of track (n: radar profs)
;      Strk_g:  fltarr(NXI), s-coordinate for the length of track at grid 1st 
;               row center points. For leg=0 this is grddd.crd1; for leg=1 it is
;               an output from wcrddgrid1.pro
;    
; KEYWORD PARAMETERS:
;      SNDFILE: input, string, complete path to a data file with:
;               1) vertical profile of wind(sounding) given in columns as 
;                  [ALT(m),U(m/s),V(m/s),W(m/s)] in UFRS (when wnddd.wflag=2) or
;               2) AC altitude, AGL in meters given in two columns as
;                  [unix_time_stamp(sec),AGL(m)] (wnddd.zagl=2, wflag=3,4)
;      HELP:    shows this text
;
; OUTPUTS:
;      Wnddd.wext: fltarr(NXI,NET,3), winds along the UFRS (E,N,UP) [m/sec]
;                  at grid cell centers.
;
; SIDE EFFECTS:
;
; PROCEDURE:
;
; EXAMPLE:
;
;-
; MODIFICATION HISTORY:
;    Written by: Rick Damiani and Samuel Haimov, December 2003
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy 
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.

pro wcrddwext, beamopt,acddd,wcrddd,grddd,wnddd,strk,strk_g,sndfile=sndfile,$
               help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrddwext'
    return
  endif

  if (n_params() eq 0) then begin
    message,"Usage: wcrddwext,beamopt,acddd,wcrddd,grddd,wnddd,strk, $",/info
    print,"                strk_g,sndfile=sndfile,help=help"
    return
  endif

; .............................................................................
; Prepare pressure altitude and ground elevation information if needed 

  if wnddd.wflag gt 1 and wnddd.wflag lt 5 then begin

  ; For wflag 2 construct grid altitude, in meters, for VPDD only: 
  ;     gpalt - 1st grid row altitude;
  ;     galt  - grid center points alt.
  ;******************************************************************
  ; NOTE: gpalt and galt are altitude msl, but we may get rid of msl
  ; data altogether and just use the AGL data in the future. 
  ;******************************************************************
    
    if grddd.leg eq 0 then begin

    ; Assign a fixed altitude for the straight leg since this is the way grid0
    ; is constructed. Use max flight altitude (grid0 based on the max alt.)
    ; for 1st grid row. Discard any small displacement between altimeter 
    ; position and straight beam antenna position which determines the 
    ; z-coordinates of the cell center points.

      junk1=max(acddd.palt)
      gpalt=make_array(grddd.dims[0],value=junk1) ;(NXI)[m]

    ; Calculate the pressure altitudes for all grid center points 

      galt=make_array(grddd.dims[0],val=1.)#(junk1-grddd.crd2) ;(NXI,NET)

    endif else begin
    
    ; Interpolate to find mid(center) points in WBRRS (junk1) and then get  
    ; their z in WBRS (junk2)

      expand,grddd.gp[*,*,2],2*grddd.dims[0]+1,2*grddd.dims[1]+1,junk1 
      junk2=-junk1[1:2*grddd.dims[0]-1:2,1:2*grddd.dims[1]-1:2] ;(NXI,NET)

    ; Resample ac altitude on the grid columns

      gpalt=resample(acddd.palt,strk,strk_g,/silent) ;(NXI) grid 1st row alt

    ; Calculate the pressure altitude for all grid center points
    
      galt=gpalt#make_array(grddd.dims[1],val=1.)+junk2 ;(NXI,NET)

    endelse

  ; For wflag 3 and 4 prepare ground elevation(msl) in meters: 
  ;     gpzelv - for each grid column
  ;     zelv   - for each grid cell center point

    if wnddd.wflag gt 2 then begin

    ; Determine AGL in meters according to wnddd.zagl

      case wnddd.zagl of
        0: begin ; >>>>>>>>>> use mean AC radar altimeter altitude (ralt2); NXI
             zelv=(fltarr(grddd.dims[0])+1.)*mean(acddd.ralt,/nan)
           end
        1: begin ; >>>>>>>>>> use radar altimeter redistributed along xi; NXI
             zelv=resample(acddd.ralt,strk,strk_g,/silent)
           end
        2: begin ; >>>>>>>>>> use AGL data file
             ;  WILL DO THIS LATER
             message,'Using AGL data file option is not available yet'
           end
      else:begin     
             if wnddd.zagl ge 75. then begin ; >>>>>>>>>> use fixed AGL value

               zelv=make_array(grddd.dims[0],val=wnddd.zagl)

             endif else begin                ; >>>>>>>>>> use WCR defined AGL

               ; use WCR thresholded reflectivity to find rg indices of ground

               if beamopt eq 0 then $ ; UWKA v-channel (straight beam)
                 ind=where(wcrddd.zv ge wnddd.zagl) $
               else $                 ; C130 h-channel (straight beam)
                 ind=where(wcrddd.zh ge zzth) 

               if ind[0] eq -1 then $
                 message,'No surface points found in WCR reflectivity data' $
               else ind=ind/wcrddd.n  ;rg indices associated with the surface 

               ; determine AGL for each grid cell center point 

               ind=congrid(ind,wcrddd.n,/interp) 
               zelv=resample(wcrddd.rng[ind],strk,strk_g,/silent) ;NXI

             endelse
           end
      endcase

    ; Calculate ground elevation in meters from MSL: 

      gpzelv=gpalt-zelv  ;(NXI) 
      zelv  =galt -zelv#make_array(grddd.dims[1],val=1.) ;(NXI,NET) 

    ; Prepare winds using wadv criterium for the grid first row cells 
    
      ; Prepare wind East component
      if (grddd.wadv[0] ne 888.) then $
      ; use the constant value provided or the AC mean measured winds (prepared in wcrdd)
        wext0=(fltarr(grddd.dims[0])+1.)*grddd.wadv[0] $
      else $
      ; use AC measured winds interpolated at every grid cell
        wext0= resample(acddd.uw,strk,strk_g,/silent) ;(NXI) u-comp at palt

      ; Prepare wind North component
      if (grddd.wadv[1] ne 888.) then $
      ; use the constant value provided or the AC mean measured winds (prepared in wcrdd)
        wext1=(fltarr(grddd.dims[0])+1.)*grddd.wadv[1] $
      else $
      ; use AC measured winds interpolated at every grid cell
        wext1= resample(acddd.vw,strk,strk_g,/silent) ;(NXI) v-comp at palt

      ; Prepare wind Up component
      if (grddd.wadv[2] ne 888.) then $
      ; use the constant value provided or the AC mean measured winds (prepared in wcrdd)
        wext2=(fltarr(grddd.dims[0])+1.)*grddd.wadv[2] $
      else $
      ; use AC measured winds interpolated at every grid cell
        wext2= resample(acddd.ww,strk,strk_g,/silent) ;(NXI) w-comp at palt     

      ; Prepare initial velocity magnitude and direction (use horiz. wind: wext0 and wext1)
        cosa=cos(atan(wext1,wext0))  ;(NXI) cos ang between x,y vel. comp. at flying alt.)
        sina=sqrt(1.-cosa^2)         ;(NXI) sin ang between x,y vel. comp. at flying alt.)
        wmag=reform(wext0^2+wext1^2) ;(NXI) horizontal wind magnitude at flying altitude

    endif
  endif

; .............................................................................

      ;***************************************************************
      ; Note:  We will use resample routine which conditions the input
      ;        signal (LP filter) when the output is decimated more
      ;        than 10%.  We may have to check for possible filter
      ;        distortions.  Is interpolation good enough ?
      ;***************************************************************

  case wnddd.wflag of

   1: begin ; >>>>>>>>>>>>> use grddd.wadv selection <<<<<<<<<<<<<<<<<<

      ; Prepare wind East component
        if (grddd.wadv[0] eq 888.) then begin ; use AC measured UW
          junk=resample(acddd.uw,strk,strk_g,/silent)
          wnddd.wext[*,*,0]= junk#make_array(grddd.dims[1],val=1.) 
        endif else $ ; use grddd.wadv value
          wnddd.wext[*,*,0]= make_array(grddd.dims,val=grddd.wadv[0])

      ; Prepare wind North component
        if (grddd.wadv[1] eq 888.) then begin ; use AC measured UW
          junk=resample(acddd.vw,strk,strk_g,/silent)
          wnddd.wext[*,*,1]= junk#make_array(grddd.dims[1],val=1.) 
        endif else $ ; use grddd.wadv value
          wnddd.wext[*,*,1]= make_array(grddd.dims,val=grddd.wadv[1])

      ; Prepare wind Up component
        if (grddd.wadv[2] eq 888.) then begin ; use AC measured UW
          junk=resample(acddd.ww,strk,strk_g,/silent)
          wnddd.wext[*,*,2]= junk#make_array(grddd.dims[1],val=1.) 
        endif else $ ; use grddd.wadv value
          wnddd.wext[*,*,2]= make_array(grddd.dims,val=grddd.wadv[2])

      end         
   2: begin ; >>>>>>>>>>>>>>>>> use sounding data <<<<<<<<<<<<<<<<<<<<<<

      ; Read the ascii sounding data file
        junk=getfile(sndfile,/quiet)

      ; Extract data lines from the file string array
        ind=where((strpos(junk,' ;#') eq -1) and (strpos(junk,'#;') eq -1))
        junk=junk[ind]
        junk=float(reform(junk,4,n_elements(junk)/4))

      ; Interpolate wind data onto the grid points
        junk1=make_array(grddd.dims[0],val=1.)
        for k=0,grddd.dims[0]-1 do begin
          wnddd.wext[k,*,0]=junk1#resample(junk[1,*],junk[0,*],galt[k,*],/sil); uw
          wnddd.wext[k,*,1]=junk1#resample(junk[2,*],junk[0,*],galt[k,*],/sil); vw
          wnddd.wext[k,*,2]=junk1#resample(junk[3,*],junk[0,*],galt[k,*],/sil); ww
        endfor

      end
   3: begin ; >>>>>>>>> use wadv criterium and vertical log profile (VPDD only) <<<<<<<<<<
        
      ; Adopt log vert. profile for horizontal wind and parabolic profile for 
      ; vertical wind

        ; find friction velocity u_star for the grid columns (NXI)
        u_star=wmag*wnddd.kappa/alog((gpalt-gpzelv-wnddd.d0)/wnddd.z0);(NXI)[m/s]

        ; replicate it for every cell in the vertical direction
        u_star=u_star#make_array(grddd.dims[1],val=1.)  ;(NXI,NET)

        ; calculate horizontal winds using log vertical profile ustar/k*alog((z-d0)/z0)
        junk=(galt-zelv-wnddd.d0)>0.
        ind=where(junk gt 0.)
        junk[ind]=u_star[ind]/wnddd.kappa*alog(junk[ind]/wnddd.z0)
        wnddd.wext[*,*,0]=cosa*junk
        wnddd.wext[*,*,1]=sina*junk

        ; For the vertical vel. assume parabolic vert. prof: w=wa*(z-z0)^2+b*(z-z0)
        junk=(gpalt-gpzelv-wnddd.z0)
        b=(wext2-wnddd.wa*junk^2)/junk ;(NXI) ; for b use flight level wind data
        b=b#make_array(grddd.dims[1],val=1.)  ; and replicate along eta; (NXI,NET)
        junk=(galt-zelv-wnddd.z0)>0.          ; make vel. zero below ground
        wnddd.wext[*,*,2]=wnddd.wa*junk^2+b*junk 

      end
   4: begin ; >> use wadv criterium and vertical Ekman's profile (VPDD only) <<
   
      ; Adopt log vertical profile for horizontal wind and parabolic profile 
      ; for vertical wind

        ; find c=!pi/z(HF) for the grid columns (NXI)
        junk=gpalt-gpzelv-wnddd.z0 
        c=!pi/(junk/wnddd.hf) ;(NXI) 

        ; find the magnitude of the geostrophic wind from the data at palt 
        ; (flying alt)
         
        ; velocity component perpendicular to the geostrophic direction
        junk1=-wext0*sina+wext1*cosa        ;(NXI)
        ; magnitude of the geostrophic wind
        wg=junk1/(exp(-c*junk)*sin(c*junk)) ; (NXI)

        ; replicate it for every cell in the vertical direction 
        c =c #make_array(grddd.dims[1],val=1.)  ;(NXI,NET)
        wg=wg#make_array(grddd.dims[1],val=1.)  ;(NXI,NET)

        junk=(galt-zelv-wnddd.z0)>0.              ; >0. to put vel=0 below sfc.
        junk1=wg*(1.-exp(-c*junk)*cos(c*junk))    ;(NXI,NET); u
        junk2=wg*exp(-c*junk)*sin(c*junk)         ;(NXI,NET); v
        wnddd.wext[*,*,0]=cosa*junk1-sina*junk2
        wnddd.wext[*,*,1]=sina*junk1+cosa*junk2

        ; For the vert. vel. assume parabolic vert. profile: w=wa*(z-z0)^2+b*(z-z0)
        junk=(gpalt-gpzelv-wnddd.z0)
        b=(wext2-wnddd.wa*junk^2)/junk ;(NXI)    ; determine b using flight level wind data
        b=b#make_array(grddd.dims[1],val=1.)    ; and replicate along eta; (NXI,NET)
        junk=(galt-zelv-wnddd.z0)>0.             ; make vel. zero below ground
        wnddd.wext[*,*,2]=wnddd.wa*junk^2+b*junk 

      end
   5: begin ; >>>>>>> use AC measured winds independent on grddd.wadv <<<<<<<<

      ;******************************************************************
      ; NOTE: for grid.leg=0 the strk,strk_g is just an approx for the 
      ; interpolation (the correct one, from wcrddgrid0, is drx[ibeam,*]) 
      ; but using strk,strk_g should be good enough
      ;******************************************************************

        junk=make_array(grddd.dims[1],val=1.)
        wnddd.wext[*,*,0]= resample(acddd.uw,strk,strk_g,/silent) # junk
        wnddd.wext[*,*,1]= resample(acddd.vw,strk,strk_g,/silent) # junk
        wnddd.wext[*,*,2]= resample(acddd.ww,strk,strk_g,/silent) # junk

      end
   6: begin ; >>>>> use mean AC measured winds independent on grddd.wadv <<<<<<

        wnddd.wext[*,*,0]= make_array(grddd.dims,val=mean(acddd.uw))
        wnddd.wext[*,*,1]= make_array(grddd.dims,val=mean(acddd.vw))
        wnddd.wext[*,*,2]= make_array(grddd.dims,val=mean(acddd.ww))

      end
   7: begin ; >>>> use mean corrected WCR Doppler velocity indep. on wadv <<<<

      ;******************************************************************
      ; NOTE: This is better suited for grid.leg=0 
      ;******************************************************************

        junk=make_array(wcrddd.rg,value=1.)

      ; average measured Doppler along the beam vectors; minus in junk1,junk2 
      ; accounts for the radar sign convention

        junk1=-[mean(wcrddd.dvh*(reform(wcrddd.beamh[*,0])#junk),/nan),$
                mean(wcrddd.dvh*(reform(wcrddd.beamh[*,1])#junk),/nan),$
                mean(wcrddd.dvh*(reform(wcrddd.beamh[*,2])#junk),/nan)]

        junk2=-[mean(wcrddd.dvv*(reform(wcrddd.beamv[*,0])#junk),/nan),$
                mean(wcrddd.dvv*(reform(wcrddd.beamv[*,1])#junk),/nan),$
                mean(wcrddd.dvv*(reform(wcrddd.beamv[*,2])#junk),/nan)]
 
      ; Use velocity decomposition to retrieve the mean flow in UFRS;
      ; use AC measured mean wind for the missing velocity component

        LSV =[norm(junk1),norm(junk2)]       ; avg vels along avg. h/v beams   
        LSM =[[junk1/LSV[0]],[junk2/LSV[1]]] ; [3,2] unit vectors of avg. h/v bms

        junk=wcrdddv(LSM,LSV,v_ext=[mean(acddd.uw),mean(acddd.vw),mean(acddd.ww)])

      ; Distribute calculated mean flow on the grid 

        wnddd.wext[*,*,0]= make_array(grddd.dims,val=junk[0])
        wnddd.wext[*,*,1]= make_array(grddd.dims,val=junk[1])
        wnddd.wext[*,*,2]= make_array(grddd.dims,val=junk[2])

      end
  endcase
end

