;$Id
;+
; NAME:
;      WCRDDWHTS
; 
; PURPOSE:
;      Set of weighting functions used by the dual-Doppler analysis (wcrdd.pro)
;      Returns weights to the Doppler velocity measurements associated with 
;      each grid cell of a Dual-Doppler grid generated by wcrddgrid0/1.pro  
;
; CALLING SEQUENCE:
;      res=wcrddwhts(x,x0,beam,nrp,whtprm, help=help)
;
; INPUTS:
;      X:      fltarr(n,3), coordinates of n points in 3-D space
;      X0:     fltarr(3), center point(for all n points) coordinates 
;              w.r.t which distances are being calculated
;      Beam:   fltarr(n,3), beam pointing unit vectors
;      Nrp:    float(3), normal unit vector of the reference desired scanning 
;              plane. Note: Beam and Nrp have to refer to the same LOCAL
;              reference system.
;      Whtprm: integer or fltarr(3-6), 3 to 6 parameter array defining the 
;              weighting scheme and parameters; 
;              Allowed weighting methods:
;                IDW: inverse distance; 
;                BSW: beam skewness; weights inversly proportional to the length
;                     of the desired plane of sacanning normal vector
;                     [code,Exp2,ROI,Thresh1,Thresh2]
;                CRW: Cressman weighting [code,Exp1,ROI,Thresh1]
;                EXW: exponential weighting [code,Exp1,Exp2,ROI,Thresh1,Thresh2]
;
;              For more information see wcrddinit.pro
;
; OUTPUTS:
;       Res:  fltarr(n), containing weights associated with the data points
;             in a grid cell
;
; KEYWORD PARAMETERS:
;       HELP: prints this help
;
; PROCEDURE:
;      Weights are calculated based on the following methods:
;         IDW: Iiverse distance weighting
;         BSW: Inverse value of beam vector components normal to the expected
;              scanning plane (beam skewness weighting)
;         CRW: Cressman weighting function
;         EXW: Exponential weighting function
;
; EXAMPLE:
;
;-
; MODIFICATION HISTORY:
;    Written by: Rick Damiani and Samuel Haimov, December 2003
;
; Copyright (C) 1996-2003, Samuel Haimov & Rick Damiani, Dept. of Atmos. Sci.,
; University of Wyoming.  This software may be used, copied, or redistributed 
; as long as it is not sold and this copyright notice is reproduced on each copy
; made. This routine is provided as is without any expressed or implied 
; warranties whatsoever. Other limitations apply as described in the Readme file.

;----------------------------------- IDW -------------------------------------- 
function wcrddidw, x,x0,whtprm
;
; whtprm: [Exp1,ROI,Thresh1]
;

  on_error,2

  whts=fltarr(n)      ; points not used in the weighting will be left 0 and
                      ; therefore will be discarded in the SVDC solver

  ; calculate distances from the center point

  junk=fltarr(n)
  for k=0,n-1 do junk[k]=norm(reform(x[k,*])-x0)

  ; find thresholded and non-thresholded data points

  if whtprm[2] ne -1 then $
    ind1=where(junk lt (min(junk)>(0.1*whtprm[1]))*(1.+whtprm[2])) $
  else ind1=lindgen(n)

  ; find the common data points (non-zero distances and below the threshold)

  if ind1[0] ne -1 then message,'No valid points for IDW were found',/info $
  else begin   
    ind2=where(junk[ind1] gt 0.,comp=ind3)

    ; calculate the weights

    if ind2[0] ne -1 then begin
      ind2=ind1[ind2]
      junk=1./(1.+junk[ind2])^whtprm[0]
      whts[ind2]=junk/total(junk)
    end
  endelse

  return,whts

end ; IDW

;---------------------------------- BSW ---------------------------------------
function wcrddbsw, beam,nrp,whtprm
;
; whtprm: [Exp2,Thresh2]
;
  on_error,2

  n=n_elements(beam[*,0])
  whts=fltarr(n)      ; points not used in the weighting will be left 0 and
                      ; therefore will be discarded in the SVDC solver

  junk=abs(beam#nrp)  ;[n] these are abs(components) of the beam vects. normal 
                      ;    to the referenced(desired) scanning plane

  ; find thresholded and non-thresholded data points

  if whtprm[1] ne -1 then $
    ind1=where(junk lt (min(junk)>sin(!pi/180.))*(1.+whtprm[1])) $
  else ind1=lindgen(n)

  ; find the common data points (non-zero distances and below the threshold)

  if ind1[0] eq -1 then message,'No valid points for BSW were found',/info $
  else begin
    ind2=where(junk[ind1] gt 0.)

    ; calculate the weights

    if ind2[0] ne -1 then begin
      ind2=ind1[ind2]
      junk=1./(1.+junk[ind2])^whtprm[0]
      whts[ind2]=junk/total(junk)
    end
  endelse

  return,whts

end ; BSW

;---------------------------------- CRW ---------------------------------------
function wcrddcrw, x,x0,whtprm 
;
; whtprm: [ROI,Thresh1]
;

  on_error,2

  n=n_elements(x[*,0])
  whts=fltarr(n)      ; points not used in the weighting will be left 0 and
                      ; therefore will be discarded in the SVDC solver

  ; calculate distances from the center point

  junk=fltarr(n)
  for k=0,n-1 do junk[k]=norm(reform(x[k,*])-x0)

  ; find thresholded and non-thresholded data points

  if whtprm[1] ne -1 then $
    ind1=where(junk lt (min(junk)>(0.1*whtprm[0]))*(1.+whtprm[1])) $
  else ind1=lindgen(n)

  ; calculate the weights

  if ind1[0] ne -1 then begin
    ind2=where(junk[ind1] le whtprm[0])
    if ind2[0] ne -1 then begin
      ind2=ind1[ind2]
      whts[ind2]=(whtprm[0]^2-junk[ind2]^2)/(whtprm[0]^2+junk[ind2]^2)
      whts[ind2]=whts[ind2]/total(whts[ind2]) ; normalize to 1
    endif else message,'All points for CRW are beyond ROI',/info
  endif else message,'No valid points for CRW were found',/info
 
  return,whts

end  ; CRW

;--------------------------------- EXW ----------------------------------------
function wcrddexw, x,x0,whtprm 
;
; whtprm: [Exp1,ROI,Thresh1]
;

  on_error,2

  n=n_elements(x[*,0])
  whts=fltarr(n)      ; points not used in the weighting will be left 0 and
                      ; therefore will be discarded in the SVDC solver

  junk=fltarr(n)
  for k=0,n-1 do junk[k]=norm(reform(x[k,*])-x0)

  ; find thresholded and non-thresholded data points

  if whtprm[2] ne -1 then $
    ind1=where(junk lt (min(junk)>(0.1*whtprm[1]))*(1.+whtprm[2])) $
  else ind1=lindgen(n) 

  ; calculate the weights

  if ind1[0] ne -1 then begin
    whts[ind1]=exp(alog(whtprm[0])*(junk[ind1]^2/whtprm[1]^2))
    whts[ind1]=whts[ind2]/total(whts[ind1]) ; normalize to 1
  endif else message,'No valid points for EXW were found',/info

  return,whts

end  ; EXW

;-----------------------------------------------------------------------------
function wcrddwhts, x,x0,beam,nrp,whtprm, help=help

  on_error,2

  if KEYWORD_SET(help) then begin
    doc_library,'wcrddwhts'
    return,''
  endif

  if (n_params() lt 5)  then begin
    message,"Usage: res=wcrddwhts(x,x0,beam,nrp,whtprm)",/info
    return,''
  endif

  case whtprm[0] of

    1: begin             ; IDW
         whts=wcrddidw(x,x0,whtprm[[1,3,4]])
       end
    2: begin             ; BSW
         whts=wcrddbsw(beam,nrp,whtprm[[2,5]])
       end
    3: begin             ; IDW+BSW
         whts1=wcrddidw(x,x0,whtprm[[1,3,4]])
         whts2=wcrddbsw(beam,nrp,whtprm[[2,5]])
       end
    4: begin             ; CRW
         whts=wcrddcrw(x,x0,whtprm[[3,4]])
       end
    5: begin             ; CRW+BSW
         whts1=wcrddcrw(x,x0,whtprm[[3,4]])
         whts2=wcrddbsw(beam,nrp,whtprm[[2,5]])
       end
    6: begin             ; EXW
         whts=wcrddexw(x,x0,whtprm[[1,3,4]])
       end
    7: begin             ; EXW+BSW
         whts1=wcrddexw(x,x0,whtprm[[1,3,4]])
         whts2=wcrddbsw(beam,nrp,whtprm[[2,5]])
       end

    else: message,'Incorrect weighting scheme code.'
  endcase

  ; If the weighting scheme employs more than one weighting method
  ; calculate the final weights (renormalize according to the intersection
  ; of the weights)

  if n_elements(whts) eq 0 then begin
    ind1=where(whts1 eq 0.) & ind2=where(whts2 eq 0.)
    if ind2[0] ne -1 then begin 
      whts1[ind2]=0. & whts1=whts1/total(whts1)
    endif
    if ind1[0] ne -1 then begin 
      whts2[ind1]=0. & whts2=whts2/total(whts2)
    endif
    whts =(whts1+whts2)/2.
  endif

  return,whts

end
