;$Id: wcrdopcor.pro,v 1.6 2003/12/23 22:29:47 haimov Exp haimov $
;+
; NAME:
;      WCRDOPCOR
;
; PURPOSE:
;      Returns WCR radial Doppler velocity w.r.t. ground and corrected for 
;      aircraft (AC) motion. Positive velocity is toward the radar. 
;     
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;       res=wcrdopcor(wcrvel,wcrtime,wcracv,actime,timehack=timehack, $
;                     maxvel=maxvel,wcrav=wcrav,wcr2ac=wcr2ac,help=help)
;
; INPUTS:
;       Wcrvel:   2-D or 3-D float array, uncorrected WCR Doppler velocity 
;                 (fltarr(rangegates,numrec,npp-1), see wcrtools/wcrdopvel.pro)
;       Wcrtime:  double array, WCR time vector (UNIX seconds)
;       Wcracv:   float array, AC velocity contribution into the WCR beam
;       Actime:   double array, AC time vector (UNIX seconds)
;
; KEYWORD PARAMETERS:
;       MAXVEL:   float/fltarr(npp-1), Nyquist velocity for each pulse pair;
;                 if npp>2 and maxvel is scalar the value is applied to all
;                 pulse pairs (velocities, wcrvel);
;                 If this keyword is defined the result will always be bound
;                 within +- maxvel
;       TIMEHACK: input, float, WCR time correction to sync with AC ins data,
;                 in seconds  (default 0.0)
;       WCR2AC:   when set match the output to AC data file sampling rate 
;                 (default is WCR irregular sampling, see Notes)
;       WCRAV:    returns Wcracv synchronized w/ wcrtime+timehack; 
;                 if wcr2ac is set returns Wcrvel synchronized w/ actime
;       HELP:     shows this text
;
; OUTPUTS:
;
; SIDE EFFECTS:
;       The function returns an empty string if the usage is incorrect or 
;       help keyword is set
;
; NOTES:
;       Wcracv and actime are equaly spaced in time.  Wcrvel and wcrtime are 
;       non-equaly spaced (varying number of profiles per second and a small 
;       variation in the time interval between the profiles).
;
;       Wcracv/actime and wcrvel/wcrtime must be matched pairs.  In other 
;       words time vectors must correspond to the time dimension/profile of
;       velocity data.
;
;       The return is irregularly gridded and matches the wcrtime (plus any 
;       timehack if defined) unless  wcr2ac keyword is set - then it is
;       regularly gridded and matches actime
;
;       Wcrtime+timehack must be subset of actime. If wcr2ac is set actime must
;       be subset of wcrtime+timehack.  This increases the accuracy of the 
;       interpolated output for the first and last data points.        
;
; PROCEDURE:
;       The functions applies linear/bilinear interpolation scheme 
;       (see, ac2wcr.pro) to match the regularly gridded AC and non-regularly 
;       gridded WCR data.  Then the contribution of the aircraft motion into
;       the WCR beam is subtracted to produce the WCR corrected radial velocity.
;       
; EXAMPLES:
;
; ** 1) Calculate corrected UWKA WCR velocity matching WCR time(profiles) **
;
; Read raw wcr data
;
; .run commonwcrset
; openwcrf,'Wpp00-11-15-19-53-25
;
; readwcrdata,21786,24973,rawtime=wcrtime  ; ground return, 20:13:20 - 20:15:05
;
; Calculate the uncorrected Doppler velocity
;
; wcrdv=wcrdopvel(ppselect='HH')
;
; Get the AC motion component into the beam (ignore, radar moment and gps
; AC velocity correction)
;
; ncf='/net/bobcat/disk1/kingair_data/20001115.c25.nc'
; kastartt=201320  &  kastopt=201506  ; start/stop time for KA data
; wcracv=wcracvelm(acdata,ncid,file=ncf,startt=kastartt,stopt=kastopt)
;
; Assign time hack, if any, and calculate corrected Doppler velocity. Match
; the output to the wcr time sampling (profiles) 
;
; timehack=0.0
; wcrdvel=wcrdopcor(wcrdv,wcrtime,wcracv,acdata.time,timehack=timehack)
;
; Since this example extracts ground hits check the quality of the corrected
; velocity
;
; ind=where(10.*alog10(hh) gt -55.)     ; find high return ground hits 
; wcrduf=fltarr(rangegates,numrec)-100  ; initilize final array
; wcrduf[ind]=wcrdvel[ind]              ; assign only valid data
; ind1=where(wcrdvel gt 30)             ; (max vel is 31.6 m/s for HH pp)
; wcrduf[ind1]=wcrduf[ind1]-31.8        ; unfold; not very nice but it's ok
; 
; aximage,wcrduf,/tr,setw=[-1,42],/bar,min=-1,max=1
; window,0 & plot,wcrduf[ind],psym=3 & grid
; print,[mean(wcrduf[ind]),std(wcrduf[ind])]
; 
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov, December 2000
;      S.H., Jan 01 -- removed few lines of code not needed and 
;                      other minor changes
;      S.H., Feb 01 -- changed routine name from wcrdopvel wcrdopcor
;      S.H., Oct 01 -- minor changes, added wcrav keyword
;      S.H., Jun 02 -- added MAXVEL keyword
;      S.H., Oct 02 -- fixed a bug associated with the use of maxvel
;      S.H., Nov 02 -- fixed another bug related to maxvel
;      S.H., Feb 03 -- renamed all appropriate vars and routines that have
;                      'ka' replaced with 'ac' 
;      S.H., May 03 -- modified example in the help to reflect changes in
;                      wcracvelm routine call                  
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;        

function wcrdopcor, wcrvel,wcrtime,wcracv,actime,timehack=timehack, $
                    maxvel=maxvel,wcrav=wcrav,wcr2ac=wcr2ac,help=help

  on_error,2
 
  if keyword_set(help) then begin
    doc_library,'wcrdopcor'
    return,''
  endif

  if n_elements(maxvel) eq 0 then maxv=0. else maxv=maxvel

  if n_params() lt 4 then begin
    message,"Usage: res=wcrdopcor(wcrvel,wcrtime,wcracv,actime, $",/info
    print,"          timehack=timehack,maxvel=maxvel,wcrav=wcrav,wcr2ac=wcr2ac,help=help)"
    return,''
  endif

; Find out the dimensions and initilize the output

  ndim=size(wcrvel,/n_dim)
  nele=(size(wcrvel))[1:ndim]
  if ndim eq 2 then begin 
    n3=1
  endif else if ndim eq 3 then begin
    n3=nele[2]
    if keyword_set(wcr2ac) then begin
       wcrdvel=fltarr(nele[0],n_elements(wcracv),n3)
       wcrav  =fltarr(nele[0],n_elements(wcracv),n3)
    endif else wcrdvel=fltarr(nele[0],nele[1],n3)
  endif else message,'wcrvel input is incorrect.'


; Match AC and WCR data and return corrected velocity.  The AC contribution
; (wcracv) is subtracted from the uncorrected velocity (wcrvel) to preserve
; the sign convention for the Doppler velocity.  wcracv is positive away 
; from the radar (standard sign convention).  Since the wcracv is seen by 
; the radar in the oposite direction the final corrected velocity based on 
; Doppler sign convention requires that wcracv is subtracted.

  if keyword_set(wcr2ac) eq 0 then begin
     wcrav=ac2wcr(wcracv,actime,wcrtime,timehack=timehack,/errorm)
     if n3 eq 1 then $
        wcrdvel=wcrvel-replicarr(wcrav,nele[0],dim=1) $
     else $
        for k=0,n3-1 do $
           wcrdvel[*,*,k]=reform(wcrvel[*,*,k])-replicarr(wcrav,nele[0],dim=1)
  endif else begin
     if n3 eq 1 then begin
       wcrav=ac2wcr(wcrvel,actime,wcrtime,timehack=timehack,/wcr2ac,/errorm)
       wcrdvel=wcrav-replicarr(wcracv,nele[0],dim=1)
     endif else begin
       for k=0,n3-1 do begin
          wcrav[*,*,k]  =ac2wcr(reform(wcrvel[*,*,k]),actime,wcrtime, $
                                timehack=timehack,/wcr2ac,/errorm)
          wcrdvel[*,*,k]=wcrav[*,*,k]-replicarr(wcracv,nele[0],dim=1)
       endfor
     endelse
  endelse

; If maxvel (max Nyquist Doppler velocity: prf/4.*lambda) is given make
; sure wcrdvel is the standard representative of the 2*maxvel equivalent class

  if maxv[0] ne 0 then begin
    if n3 gt 1 and n_elements(maxv) eq 1 then maxv=replicate(maxv,n3)
    maxv2=2.*maxv
    if n3 eq 1 then begin
      wcrdvel=wcrdvel mod maxv2[0]
      ind=where(wcrdvel gt maxv[0])
      if ind[0] ne -1 then wcrdvel[ind]=wcrdvel[ind]-maxv2[0] 
      ind=where(wcrdvel le -maxv[0])
      if ind[0] ne -1 then wcrdvel[ind]=wcrdvel[ind]+maxv2[0] 
    endif else begin
      for k=0,n3-1 do begin
        wcrav=wcrdvel[*,*,k] mod  maxv2[k]
        ind=where(wcrav gt maxv[k])
        if ind[0] ne -1 then wcrav[ind]=wcrav[ind]-maxv2[k]
        ind=where(wcrav le -maxv[k])
        if ind[0] ne -1 then wcrav[ind]=wcrav[ind]+maxv2[k]
        wcrdvel[*,*,k]=wcrav
      endfor
    endelse
  endif

; Return corrected velocity

  return,wcrdvel

end
