;$Id: wcrdopcorm.pro,v 1.12 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;      WCRDOPCORM
;
; PURPOSE:
;      Returns WCR radial Doppler velocity field corrected for aircraft,
;      AC, motion (synched with wcr time or AC time if wcr2ac keyword is set). 
;
;      This routine uses wcrdopvel to calculate uncorrected Doppler velocity 
;      and wcracvelm to calculate AC velocity contribution to the beam.
;      The corrected Doppler velocity is calculated using wcrdopcor routine.
;      Positive velocity is toward the radar. The wind contribution to the
;      wcr beam is calculated (wcracw) using wcracwind routine.
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;       res=wcrdopcorm(wcrtime,acncf=acncf,acdata=acdata,beaminit=beaminit, $
;                      close=close,gpsc=gpsc,ncnames=ncnames,ppselect=ppselect,$
;                      rmcor=rmcor,timehack=timehack,wcr2ac=wcr2ac, $
;                      radbeam=radbeam,unfold=unfold,wcracv=wcracv,acv=acv, $
;                      wcracw=wcracw,wcrvel=wcrvel,help=help)
;
; INPUTS:
;       Wcrtime: double array, WCR time vector (UNIX seconds)
;
; KEYWORD PARAMETERS:
;       ACDATA:  input/output, structure, AC IRS/INS data matching AC time
;                acdata.time   -> double(m), AC unix time in sec (see Notes)
;                ; Following angles are in radians or degrees if deg keyword set
;                acdata.roll   -> fltarr(m), AC roll angle
;                acdata.pitch  -> fltarr(m), AC pitch angle
;                acdata.heading-> fltarr(m), AC true heading from North
;                ; AC ground velocities in [m/s]
;                acdata.ug     -> fltarr(m), East-West AC ground velocity
;                acdata.vg     -> fltarr(m), North-South AC ground velocity
;                acdata.wg     -> fltarr(m), AC vertical velocity
;                ; UWKA GPS corrections of ins AC velocities (ignore for C130)
;                acdata.uerr   -> fltarr(m), E-W vel correction in m/s
;                acdata.verr   -> fltarr(m), N-S vel correction in m/s
;                ; AC measured winds in [m/s]
;                acdata.uw     -> fltarr(m), East wind velocity component
;                acdata.vw     -> fltarr(m), North wind velocity component
;                acdata.ww     -> fltarr(m), vertical wind velocity component
;                ; AC attitude rates in rad/sec or deg/sec (if deg keyword set)
;                acdata.rollr  -> fltarr(m,3), roll rate
;                acdata.pitchr -> fltarr(m,3), pitch rate
;                acdata.yawr   -> fltarr(m,3), yaw rate
;                ;
;                acdata.tas    -> fltarr(m), true air speed in m/s
;                ;
;                ; UWKA radar wing mirror position 
;                acdata.wcrmirror -> intarr(m), KA radar wing mirror position
;                                    (0-side, 1-up, for side/up antenna)
;                ; AC data info: aircraft id (N2UW or N130AR)
;                acdata.tas    -> string, aircraft ID
;
;                Note: If any of the non-critical data (e.g., attitude rate)
;                      is not available, -1 is recorded
;
;       ACNCF:    input/output string or long, 
;                 string, AC data netcdf file name
;                 long, netcdf file id (if the input acncf was a netcdf 
;                      file name and CLOSE was not set)
;
;       ACV:      output, fltarr(m,3) matching ACTIME, AC velocity (GPS/RMCOR 
;                        corrected if requested) in AC corrdinates.
;
;       BEAMINIT: input, Initialize antenna beam; 
;                 Default beams:
;                  -1(default) - use UWKA radar mirror variable for side/up
;                                pointing direction of UWKA WCR main antenna
;                   0         - side-looking KingAir, KA, (main) 12"antenna
;                   1         - up-looking KA (main+mirror) 12" antenna
;                   2         - KA side-fore antenna(~35 deg)
;                   3         - KA nadir 18" antenna
;                   4         - KA down-fore 15" antenna(~30 deg)
;                   5         - NCAR C130 nadir 12" antenna (7/16/01)
;                   6         - NCAR C130 down-aft 12" antenna(7/16/01)
;                   7         - NRC CV580 side looking 12" antenna, dual-pol
;                   8         - NRC CV580 nadir looking 12" antenna
;                   fltarr(m) - side/up antenna for KA determined by the  
;                               KA mirror (0-side, 1-up), m > 3
;                  Antenna beam pointing vector:
;                   fltarr(3)  - beam unit vector in AC coordinate system
;                                [AC along axis, right wing, down]
;                   fltarr(3,m)- beam unit vectors for all data points
;
;       CLOSE:    when set AC netcdf file will be closed.
;       GPSC:     when set does NOT apply GPS correction to AC E-W and N-S
;                  velocity for UWKA only (for NCAR C130 the default is also GPS
;                  corrected; use ncnames and appropriate C130 var names to use
;                  NO GPS corrected data
;       NCNAMES:  input, string array, AC netcdf varnames to be used; if NCNAMES 
;                  not defined N2UW or N130AR default var names are used)
;                  (ignored if acdata exist) 
;                  
;                  ncnames[0] - AC true air speed in m/s
;                  ncnames[1] - AC roll [deg]
;                  ncnames[2] - AC pitch [deg]
;                  ncnames[3] - AC true heading [deg]
;                  ncnames[4] - AC east-west(EW) ground velocity [m/s]
;                  ncnames[5] - AC north-south(NS) ground velocity [m/s]
;                  ncnames[6] - AC vertical velocity [m/s]
;                  ncnames[7] - AC East component of the wind [m/s]
;                  ncnames[8] - AC North component of the wind [m/s]
;                  ncnames[9] - AC vertical wind [m/s]
;                  ncnames[10]- AC roll rate [rad/s] (use '' if N/A)
;                  ncnames[11]- AC pitch rate [rad/s](use '' if N/A)
;                  ncnames[12]- AC heading rate [rad/s] (use '' if N/A)
;                  ncnames[13]- AC EW ground vel. GPS correction(use '' if N/A)
;                  ncnames[14]- AC NS ground vel. GPS correction(use '' if N/A)
;
;                 Note: define first 10 names only if the last 5 vars are
;                       not needed; see ACLOADINS.PRO help for default variable 
;                       names
;       PPSELECT: input, string; type of pulse pair to be used:
;                   'HH','VV','VH','CO'-all co-pol pulse pairs, and
;                   'ALL'-all pulse pairs(default))
;                 input integer, single pulse pair counted from left to wright
;                  from the pulse sequence. E.g., for HHVV ppselect 1,2, or 3
;                  corresponds to HH, HV and VV pulse pair
;                 Attn: Use for PPMAG only (SPPAMG and FFT ignore this keyword)
;                       and when beaminit is the given as a beam vector(3)
;                       For multi-beam configurations PPSELECT can be 'HH' or
;                       'VV' or single pulse pair (1,2,...) only
;       RMCOR:    integer/fltarr(3), applies antenna moment correction
;                  0 - antenna moment correction NOT applied  
;                  1 - antenna moment correction applied
;                  [x,y,x] - ant. position in aircraft coord w.r.t IRS in meters
;                  Default: for UWKA or NRC Convair not applied (0) 
;                           for NCAR C130  applied(1)
;                 Default antenna positions in meters can be seen in wcracvelm.pro
;
;       TIMEHACK: input,float, WCR time correction to sync with AC INS data,
;                 in seconds  (default 0.0)
;       WCR2AC:   when set match the output to AC data file sampling rate
;                 (default is matching WCR time(irregular sampling), see Notes)
;       RADBEAM:  output, fltarr(3,m), WCR beam in Earth coordinate system
;       UNFOLD:   input, Doppler velocity unfolding procedure (see notes)
;                 0          - no unfolding is performed (default)
;                 1          - use AC measured winds profile by profile
;                 fltarr(3)  - [u,v,w], use the given wind vector
;                              u,v,w are allowed the following values:
;                              float: actual wind value to be used in m/s
;                                888: use AC measured values for all 
;                                     profiles of this component
;                                777: use mean of AC measured values from 
;                                     all profiles of this component
;                 fltarr(m,3)- use user defined wind vector for all data points
;       WCRACV:   output, fltarr(m),AC velocity contribution into the WCR beam; ACTIME synched
;       WCRACW:   output, fltarr(m),AC measured wind contribution into the WCR beam; ACTIME synched
;       WCRVEL:   output, fltarr(rg,n),uncorrected WCR Doppler velocity (see wcrdopvel.pro)
;
;       HELP:     shows this text
;
; SIDE EFFECTS:
;       The function returns an empty string if the usage is incorrect or
;       help keyword is set
;
; NOTES:
;       The raw wcr data including wcrtime must be loaded before a call to
;       wcrdopcorm (see wcrtools library README.txt file for details or
;       the example given bellow).
;
;       wcrtime[0] and wcrtime[last] are used to extract the necessary
;       AC data in the following way:
;         - wcr2ac NOT SET:  AC start time [sec] = floor(wcrtime[0])-1
;                            AC stop time  [sec] = ceil(wcrtime[last])+1
;         - wcr2ac SET:      AC start time [sec] = ceil(wcrtime[0])+1
;                            AC stop time  [sec] = floor(wcrtime[last])-1
;
;       The timehack, if given, serves to sync WCR and AC time/data.
;       The time hack is applied to the WCR time (it should be small if any).
;       The input wcrtime is not changed. You can correct wcrtime with the
;       timehack before calling wcrdopcorm (wcrtime=wcrtime+timehack) - in
;       this case timehack must not be used in the wcrdopcorm call.
;
;       The return is irregularly gridded and match the wcrtime (plus any 
;       timehack if defined). If wcr2ac keyword is set then it is regularly
;       gridded and match actime.  The other output variables are gridded 
;       corresponding to their origin (AC-regular, sps: radbeam;
;       WCR-irregular, profiles/sec: wcrvel).  To convert from one grid to
;       another use ac2wcr routine.
;
;       When UNFOLD keyword is defined and non-zero, an attempt to unfold
;       the Doppler velocity using wind data is performed.  It is based
;       on the assumption that the AC measured winds or the given mean
;       winds are causing the folding, if exists, at any range gate.
;       It is also assumed that the velocity deviation w.r.t. the mean 
;       wind do not exceed +-Vmax. Thus centering the corrected for
;       aircraft motion measured folded Doppler velocities around the wind
;       contribution into the WCR beam is expected to render non-folded 
;       velocities.
;
; PROCEDURE:
;       The functions applies linear/bilinear interpolation scheme
;       (see, ac2wcr.pro) to match the regularly gridded AC and non-regularly
;       gridded WCR data.  Then the contribution of the aircraft motion into
;       the WCR beam is subtracted to produce the WCR corrected radial velocity.
;
; EXAMPLES:
;
; ******* 1) Calculate corrected WCR velocity matching WCR time(profiles) ******
;
; Read raw wcr data
;
; openwcrf,'Wpp00-11-15-19-53-25
;
; Read wcr data. Use start and stop time instead of profile numbers
;
; startt  = 201306   ; due to IRIG time jumps the read data start time
; stopt   = 201505   ; will be 201320; stop time 201505 is not affected
; wcrtime=1          ; get the raw (unix) wcr time and the rest
;                    ; of the wcr data stored in common block
;
; readwcrdata,startt,stopt,rawtime=wcrtime,/timein ; ground, 1000-2500 m range
;
; Define inputs
;
; kancf   = '/net/bobcat/disk1/kingair_data/20001115.c25.nc'
; ppselect= 'HH'
; timehack= 0.0
; beaminit=-1
; gpsc    = 0    ; 0, apply GPS correction (default)
; rmcor   = 0    ; 0, no radar moment correction (default)
; wcr2ac  = 0    ; 0, match the return to the irregular wcr sampling (default)
;
; Calculate the corrected Doppler velocity
;
; wcrdvel=wcrdopcorm(wcrtime,acncf=kancf,timehack=timehack,ppselect=ppselect,$
;                   gpsc=gpsc,rmcor=rmcor,wcr2ac=wcr2ac,acdata=acdata, $
;                   wcrvel=wcrvel,wcracv=wcracv,wcracw=wcracw,radbeam=wcrb)
;
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov, December 2000
;      S.H., Jan 01 -- fixed minor bugs for wcr2ka option
;      S.H., Jan 01 -- fixed AC time to be subset or superset of wcrtime
;      S.H., Feb 01 -- minor fix (assign 0 to undefined timehack)
;                      changed routine name from wcrdopvelm to wcrdopcorm
;      S.H., Oct 01 -- changes in the help, added ncnames and beaminit keywords
;      S.H., Feb 02 -- modified rmcor keyword
;      S.H., Mar 02 -- added close, tas and wcrmirror keywords, changed RMCOR
;                      default for C130
;      S.H., Jun 02 -- modified to use maxvel output from wcrdopvel in wcrdopcor
;      S.H., Oct 02 -- added wcracw keyword
;      S.H., Feb 03 -- added acdata keyword, modified ncnames and help text;
;                      replaced ka prefix with ac where applicable
;      S.H., May 03 -- added unfold keyword, modified formal parameters
;      S.H., Sep 03 -- changed allowed input values for UNFOLD
;      S.H., Oct 03 -- changed beaminit help (added wcr beams for NRC Convair)
;      S.H., Nov 03 -- no need of ppselect to be defined for beaminit=-1,...,8
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrdopcorm, wcrtime,acdata=acdata,acncf=acncf,beaminit=beaminit, $
                     close=close,gpsc=gpsc,ncnames=ncnames,ppselect=ppselect,$
                     rmcor=rmcor,timehack=timehack,wcr2ac=wcr2ac, $
                     radbeam=radbeam,unfold=unfold,wcracv=wcracv,acv=acv, $
                     wcracw=wcracw,wcrvel=wcrvel,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrdopcorm'
    return,''
  endif

  if n_params() ne 1 then begin
    message,"Usage: res=wcrdopcorm(wcrtime,acdata=acdata,acncf=acncf, $",/info
    print,"            beaminit=beaminit,close=close,gpsc=gpsc,ncnames=ncnames,$"
    print,"            ppselect=ppselect,rmcor=rmcor,timehack=timehack,$"
    print,"            wcr2ac=wcr2ac,radbeam=radbeam,unfold=unfold,acv=acv,$"
    print,"            wcracv=wcracv,wcracw=wcracw,wcrvel=wcrvel,help=help)"
    return,''
  endif

; Determine start/stop times for AC data

  stt=wcrtime[0] mod 86400
  spt=wcrtime[n_elements(wcrtime)-1] mod 86400

  if n_elements(timehack) eq 0 then timehack=0.
  if n_elements(unfold) eq 0 then unfold=0

; Make sure the interpolated time is a subset

  if timehack eq 0 then begin
    if keyword_set(wcr2ac) then begin & stt=ceil(stt)+1 & spt=floor(spt)-1
    endif else begin & stt=floor(stt) & spt=ceil(spt)+1. & end
  endif else begin
    if keyword_set(wcr2ac) then begin
       stt=ceil(stt+abs(timehack))+1 & spt=floor(spt-abs(timehack))-1
    endif else begin
       stt=floor(stt-abs(timehack))-1 & spt=ceil(spt+abs(timehack))+1
    endelse
  endelse

; Define ppselect if not given

  if n_elements(ppselect) eq 0 then begin
    ppselect='CO'
    if n_elements(beaminit) eq 1 then begin
      case beaminit of
        0: ppselect='HH'
        1: ppselect='HH'
        2: ppselect='VV'
        3: ppselect='VV'
        4: ppselect='HH'
        5: ppselect='HH'
        6: ppselect='VV'
        7: ppselect='CO'
        8: ppselect='VV'
      endcase
    endif else $
      message,'ppselect not defined; will use all co-pol/beam pulse pairs',/info      
  endif

; Get the uncorrected Doppler velocity

  wcrvel=wcrdopvel(ppselect=ppselect,maxv=maxvel)

; Get the AC motion component into the beam

  if n_elements(acncf) gt 0 then begin
    if size(acncf,/type) eq 7 then filen=acncf $
    else ncid=acncf
  endif

  wcracv=wcracvelm(acdata,ncid,filen=filen,beaminit=beaminit,ncnames=ncnames,$
                   startt=sec2time(stt),stopt=sec2time(spt),close=close, $
                   gpsc=gpsc,rmcor=rmcor,radbeam=radbeam,wcracw=wcracw,acv=acv)

  if keyword_set(close) eq 0 and n_elements(ncid) gt 0 then acncf=ncid

; Calculate corrected Doppler velocity

  wcrdvel=wcrdopcor(wcrvel,wcrtime,wcracv,acdata.time,timehack=timehack, $
                    maxvel=maxvel,wcr2ac=wcr2ac)

; Unfold corrected Doppler velocities if requested

  if (n_elements(unfold) eq 1 and unfold[0] ne 0) or n_elements(unfold) gt 1 $
  then begin
  
    ; Synch radbeam with wcrtime
    radbeam1=ac2wcr(radbeam,acdata.time,wcrtime,timehack=timehack)

    ; Synch and/or calculate the wind component into WCR beam 

    if n_elements(unfold) eq 1 and unfold[0] eq 1 then begin
      ; Use AC measured wind component into wcr beam
      wcrwind=ac2wcr(wcracw,acdata.time,wcrtime,timehack=timehack)       
    endif else begin
      ; Use user given wind
      if n_elements(unfold) eq 3 then begin
        ; Use one wind vector for all profiles
        k=n_elements(wcrvel[0,*,0])
        wind=replicarr(unfold,k,dim=1)

        ; If any component(EW, NS, or up) of the input wind vector is 888
        ; use AC measured values for all profiles of this component
        ind=where(unfold eq 888)

        if ind[0] ne -1 then begin
          wind[*,ind]=([[ac2wcr(acdata.uw,acdata.time,wcrtime,timehack=timehack)],$
            [ac2wcr(acdata.vw,acdata.time,wcrtime,timehack=timehack)],$
            [ac2wcr(acdata.ww,acdata.time,wcrtime,timehack=timehack)]])[*,ind]
        endif

        ; If any component(EW, NS, or up) of the input wind vector is 777
        ; use mean of AC measured values from all profiles of this component
        ind=where(unfold eq 777)
        if ind[0] ne -1 then begin
          wind[*,ind]=replicarr(mean(([[acdata.uw],[acdata.vw],[acdata.ww]])$
                                [*,ind],1),k,dim=1)
        endif        

        ; Calculate the wind component into the WCR beam
        wcrwind=wcracwind(wind,radbeam1)
      endif else wcrwind=wcracwind(unfold,radbeam1)
    endelse   

    ; Unfold for every velocity field

    ndvel=n_elements(maxvel)  ; maxvel output from wcrdopvel

    for k=0,ndvel-1 do begin
      wcrdvel[*,*,k]=wcrwunfold(wcrdvel[*,*,k],maxvel[k],wcrwind)
    endfor  

  endif

; and exit

  return,wcrdvel

end
