;$Id: wcrdopvel.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRDOPVEL
;
; PURPOSE:
;       Calclulate (uncorrected) Mean Doppler velocity from WCR pulse pair or
;       fft data.  Returns fltarr(rangegates, numrec) for SPPAMG and FFT and 
;       fltarr(rangegates,numrec,npulse-1) for PPMAG 
;       
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       res=wcrdopvel()
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;       MAXV:     output, float/fltarr; returns max Doppler velocities,
;                 corresponding to ppselect; this is equivalent or subset
;                 of maxvel in wcrparams common block
;       PPSELECT: input, string; type of pulse pair to be used:
;                   'HH','VV','VH','CO'-all co-pol pulse pairs, and
;                   'ALL'-all pulse pairs(default))
;                 input integer, single pulse pair counted from left to wright
;                   from the pulse sequence. E.g., for HHVV ppselect 1,2, or 3
;                   corresponds to HH, HV and VV pulse pair
;                 Attn: use for PPMAG only (SPPAMG and FFT ignore this keyword)
;       PPTHRESH: input, fltarr(2), [Nstd,Val]:
;                 Nstd - pulse pair number of StDev threshold; if given will
;                        threshold retrieved Doppler velocity using ppnoise
;                        magnitude standard diviation;
;                 Val  - float, to be used for the thresholded data points
;                        (default is !values.f_nan
;       SWIDTH:   input, positive, scalar; force calculation and return 
;                        of spectral width in m/s; no threshold is applied
;
;                        negative, scalar; for SPPAMG and PPMAG only,
;                        threshold for the returned power in dBm; to be used 
;                        for selecting valid data points when calculating
;                        spectral width (see Notes) 
;                         
;                        if the input is not a scalar the calculation is skipped
;
;                 output,SPPMAG: fltarr(rangegates, numRec), pulse pair proc.
;                        PPMAG:  fltarr(rangegates, numRec), pulse pair proc.
;                        FFT:    fltarr(rangegates, numRec), Dopppler spectrum 
;                                second moment; for FFT mode only
;       HELP:     shows this text
;
; OUTPUTS:
;
; SIDE EFFECTS:
;       The function returns an empty string if the usage is incorrect or
;       -1 if ppselect is invalid
;
; NOTES:
;       The sign convention for the Dopler velocity is positive toward the 
;       radar.
;
;       Spectral width (SWIDTH) for SPPAMG and PPMAG data is calculated from
;       a pulse pair.  This is not a good estimator of the spectral width.
;       It also renders negative width for low SNR.  To avoid the negative 
;       values one can define minimum returned power threshold to be used.
;       Currently (Dec 2000) the noise level is close to -65 dBm.
;       I have not noticed negative values for threshold > -55 dBm.
;       The thresholded pixels are set to 0 m/s spectral width.
;
;       ATTEN:  I have not implemented the calculations for PPMAG yet.
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, December 2000
;       S.H., Jan 01 -- accept lowercase ppselect string
;                       added DPFFT mode
;       S.H., Feb 01 -- changed routine name from vxidopvel to wcrdopvel
;       S.H., Oct 01 -- replaced atan(complex) with atan(imaginary(complex),
;                                                        float(complex))
;       S.H., Feb 02 -- modified ppselect keyword
;       S.H., Jun 02 -- added MAXV keyword
;       S.H., Feb 03 -- modified PPSELECT keyword
;       S.H., Mar 04 -- added PPTHRESH
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrdopvel,ppselect=ppselect,maxv=maxv,swidth=swidth, $
                   ppthresh=ppthresh,help=help

  common wcrparams
  common wcrdata

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrdopvel'
    return,''
  endif

; Calculate velocity conversion factor and check if a threshold is given
  velfactor  = maxvel/!pi

  if n_elements(swidth) eq 1 then $
     if swidth lt 0 then thresh=10.^(0.1*swidth) $
     else thresh=1.e-15

; Calculate the mean velocity and the spectral width 

  if alg eq sppalg then begin                    ; ***** SPPMAG *****

     dvel=velfactor*atan(imaginary(pp),float(pp)); calculate mean velocity
                                                 ; positive is toward the radar
     maxv=maxvel

     if n_elements(swidth) eq 1 then begin       ; calculate spectral width
       swidth=fltarr(rangegates,numrec)
       if ppstrpol eq 'HH' then begin

         hhmn=mean(hhnoise)

         if thresh le 1.e-14 then begin
            ind=where((hh-hhmn) ge thresh)
            if ind[0] eq -1 then begin    ; check for valid points, if not QUIT 
              message,'No valid pixels for SWIDTH',/info 
              return,dvel
            endif
         endif else begin
            ind=where(hh ge thresh)
            if ind[0] eq -1 then begin    ; check for valid points, if not QUIT 
              message,'No valid pixels for SWIDTH',/info 
              return,dvel
            endif
         endelse

         swidth[ind]=alog((hh[ind]-hhmn)/abs(pp[ind]))

       endif else if ppstrpol eq 'VV' then begin
         vvmn=mean(vvnoise)

         if thresh le 1.e-14 then begin
            ind=where((vv-vvmn) ge thresh)
            if ind[0] eq -1 then begin    ; check for valid points, if not QUIT 
              message,'No valid pixels for SWIDTH',/info 
              return,dvel
            endif
         endif else begin
            ind=where(vv ge thresh)
            if ind[0] eq -1 then begin    ; check for valid points, if not QUIT 
              message,'No valid pixels for SWIDTH',/info 
              return,dvel
            endif
         endelse

         swidth[ind]=alog((vv[ind]-vvmn)/abs(pp[ind]))
       endif

       swidth[ind]=velfactor*sqrt(2.*abs(swidth[ind]))*sign(swidth[ind])

       ind=0 ; (only one pulse-pair;  use with PPTHRESH)

     endif
  endif else if alg eq pp4alg or alg eq pp6alg then begin ; ***** PPMAG ******

  ;  Calculate mean velocity ; positive is toward the radar

     if n_elements(velfactor) eq 1 then $
        velfactor=replicate(velfactor,npulse-1) $
     else velfactor=velfactor[0:npulse-2]

     if n_elements(ppselect) ne 0 then begin
       if size(ppselect,/type) eq 7 then begin
         if strupcase(ppselect) eq 'ALL' then ind=indgen(npulse-1) $
         else if strupcase(ppselect) eq 'CO' then begin
           ind=where(ppstrpol eq 'HH' or ppstrpol eq 'VV')
           if ind[0] eq -1 then message,'No co-pol pulse pairs available'
         endif else begin
           ind=where(strupcase(ppselect) eq ppstrpol)
           if ind[0] eq -1 then message,'Incorrect ppselect string'
         endelse
       endif else begin
         if ppselect ge 0 and ppselect le npulse-1 then ind=ppselect-1 $
         else begin
           message,'Invalid ppselect input.',/info
           return,-1
         endelse
       endelse
     endif else ind=indgen(npulse-1)
     maxv=velfactor[ind]*!pi

     dvel=fltarr(rangegates,numrec,n_elements(ind))
     for k=0,n_elements(ind)-1 do $
;        dvel[*,*,ind[k]]=velfactor[ind[k]]*atan(imaginary(pp[*,ind[k],*]), $
         dvel[*,*,k]=velfactor[ind[k]]*atan(imaginary(pp[*,ind[k],*]), $
                                            float(pp[*,ind[k],*]))

     dvel=reform(dvel) 

  ;  Calculate spectral width  (not done yet)

  endif else if alg eq fftalg or alg eq dftalg then begin   ; ***** FFT *****

  ;  Calculate mean velocity  (positive is toward the radar) 

     if alg eq fftalg then $
        avel=reform(velo#(fltarr(rangegates*numrec)+1),fftbins,rangegates, $
                    numrec) $
     else $
        avel=reform(velo#(fltarr(2L*rangegates*numrec)+1),fftbins,rangegates, $
                    2,numrec)

     dvel=total(avel*fftps,1)/total(fftps,1)

  ;  Calculate spectral width

     if n_elements(swidth) eq 1 then begin
        swidth=sqrt(total(avel^2*fftps,1)/total(fftps,1)-dvel^2)
     endif

     maxv=maxvel

  endif

  ;  Threshold Doppler velocity using ppnoise and ppthresh

  if n_elements(ppthresh) gt 0 and alg ne fftalg and alg ne dftalg then begin

    ; no signal value
    if n_elements(ppthresh eq 2) then val=ppthresh[1] else val=!values.f_nan
    ; get the pulse-pair power and noise
    ppp=transpose(abs(pp[*,ind,*]),[0,2,1])
    ppn=abs(ppnoise)
    mnppn=min(mean(ppn,2),ind1)
    stdppn=std(ppn[ind1,*])

    for k=0,n_elements(ind)-1 do begin
      ind1=where(ppp[*,*,k]-mnppn le ppthresh*stdppn)
      if ind1[0] ne -1 then begin
       junk=dvel[*,*,k]
       junk[ind1]=val
       dvel[*,*,k]=junk
      endif
    endfor

  endif

  return,dvel

end  
 
