;$Id: wcrdvsnr.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRDVSNR
; 
; PURPOSE:
;       Calculate mean and standard deviation of the error in the measured WCR 
;       Doppler velocity as a function of SNR (signal-to-noise ratio calculated
;       from the received power and measured noise power)
;       
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       WCRDVSNR,dv,pwr,noise,snr,dvsd,dvmn
;
; INPUTS:
;       Dv:    fltarr(rangegates,profs), measured mean Doppler velocity
;       Pwr:   fltarr(rangegates,profs), received raw power in mW
;       Noise: fltarr(profs), measured noise power in mW
;
; KEYWORD PARAMETERS:
;       SNRHIST:  output, lonarr(bins), histogram of the received power;
;                 bins is the size of the output snr.  Bins with less than
;                 10 counts are excluded
;       PLOTHIST: when set display received power(pwr-mean(noise)) histogram 
;                 in dBm
;       PLOTDVE:  when set display Doppler velocity mean and standard deviation 
;                 errors in m/s as function of SNR
;       PWRSNR:   output,fltarr(rangegates,profs),returns input pwr as SNR in dB
;       HELP:     show this text
;
; OUTPUTS:
;       Snr:  fltarr(bins), SNR in dB; bins the number of bins for SNRHIST
;             containing more than 10 data points; bin size is set to 1 dBm;
;             Snr is limitted to -30 dB at the low end
;       Dvsd: fltarr(bins), Doppler velocity error standard deviation in m/s
;       Dvmn: fltarr(bins), Doppler velocity mean error in m/s
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;       When PLOTHIST and/or PLOTDVE are set the corresponding plots are
;       displayed in the current graphics device
;
; NOTES:
;       Dvsd and Dvmn are calculated assuming that two consecutive WCR profiles
;       are fully target correlated while the system noise is uncorrelated.
;
;       At very low SNR Dvsd should be close to stdev of the system phase
;       (velocity) noise (uniformly distributed) and at high SNR Dvsd represents
;       velocity spread due to platform motion and possible decorrelation from
;       profile to profile.
;
;       Results may be corrupted (more likely at high SNR) if measured Doppler 
;       velocity has folded, and/or ground/sea returns are present, and/or 
;       range gates with noticeable transmitter leakage are included in the 
;       data segment used       
;
; EXAMPLE:  Using WCR data from DYCOMS II
;
;     ; Read raw WCR data
;
;       .run commonwcrset
;       openwcrf,'Wpp01-07-24-11-03-24',timecor=0   ; raw time is UTC
;       readwcrdata,110552,113642,/timein ; read data 11:05:52-11:36:42
;       dv=wcrdopvel(ppselect='CO'); calculate Doppler vel. for the down beams
;
;     ; Calculate DV errors for the 20 KHz pulse pair      
;
;       wcrdvsnr,reform(dv[3:60,*,0]),hh[3:60,*],hhnoise,snr20,dvsd20,dvmn20
;
;     ; Calculate DV errors for the 5 KHz pulse pair      
;
;       wcrdvsnr,reform(dv[3:60,*,1]),hh[3:60,*],hhnoise,snr5,dvsd5,dvmn5
;
;     ; Make a postscript file idl.ps with the results
;
;       setxpswin,99,2
;       plot,snr20,dvsd20,psym=1, $
;            title=wcrfile+', 11:05:52-11:36:41; Nadir beam', $
;            xtit='SNR [dB]',ytit='DV err St.Dev. [m/s]'
;       oplot,snr5,dvsd5,psym=2
;       grid
;       plot,snr20,dvmn20,psym=1, $
;            title=wcrfile+', 11:05:52-11:36:41; Nadir beam', $
;            xtit='SNR [dB]',ytit='DV err Mean [m/s]'
;       oplot,snr5,dvmn5,psym=2
;       grid
;       printwcrstamp 
;       resetps2x
;
;-
; MODIFICATION HISTORY:
;       Written by Samuel Haimov, Jan 2003
;       S.H., Mar 2004 - added PWRSNR keyword 
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrdvsnr,dv,pwr,noise,snr,dvsd,dvmn, $
    snrhist=snrhist,pwrsnr=pwrsnr,plothist=plothist,plotdve=plotdve,help=help

on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrdvsnr'
    return
  endif

  if n_params() lt 3 then begin
    message,"Usage: wcrdvsnr,dv,pwr,noise,snr,dvsd,dvmn, $",/info
    print,"                   snrhist=snrhist,plothist=plothist,$"
    print,"                   plotdve=plotdve,pwrsnr=pwrsnr,help=help"
    return
  endif

; Calculate SNR. Use SNR>-30dB and skip 1st profile to match dvs

  mnn   = mean(noise)
  pwrsnr= ((pwr-mnn)/mnn)>1.e-3
  pnr   = pwrsnr[*,1:*]
  ind   = where(pnr gt 1.e-3)
  pnr   = 10.*alog10(pnr[ind])
  pwrsnr= 10.*alog10(pwrsnr)

; Calculate Doppler velocity difference.  Assume two consecutive profiles are
; fully correlated. Taking the difference of two consequtive velocities (phases)
; will cause the result to be dependant on the SNR.  The larger the SNR the
; smaller the contribution of the noise to the phase(vel).

  dvs=dv-shift(dv,0,1)
  dvs=(dvs[*,1:*])[ind]

; Calculate SNR histogram using 1 dB bin size and get reverse indices

  xmin=floor(min(pnr))
  xmax=ceil(max(pnr))
  binsize=1.

  hist=histogram(pnr,min=xmin,max=xmax,binsize=binsize,reverse=hind,loc=xhist)

; Plot Pwr histogram if requested

  if keyword_set(plothist) then begin
    plot,xhist+10.*alog10(mnn),hist,psym=10,/xstyle, $
         xtit='Received power (pwr-mean(noise)) in [dB]',ytit='Frequency'
    grid
  endif

; Calculate stdev(dvs) and mean(dvs) for each SNR histogram bin.
; Bins with less than 10 data points are skipped

  nhist=n_elements(hist)
  snr=fltarr(nhist)
  dvmn=fltarr(nhist)-99.
  dvsd=fltarr(nhist)-99.
  snrhist=fltarr(nhist)
  count=-1
  for k=0,nhist do begin
    if hind[k+1] ge hind[k]+10 then begin    
      count=count+1
      snrhist[count] = hist[k]
      snr[count]      = xhist[k]
      dvmn[count]     = mean(dvs[hind[hind[k]:hind[k+1]-1]])
      dvsd[count]     = std(dvs[hind[hind[k]:hind[k+1]-1]])
    endif
  endfor

  snr     =snr[0:count]
  snrhist =snrhist[0:count]
  dvmn    =dvmn[0:count]
  dvsd    =dvsd[0:count]

; Correct dvsd for using a difference(sum) of two random processes

  dvsd=dvsd/sqrt(2.)

; Plot dvmn and dvsd if requested

  if keyword_set(plotdve) then begin
    plot,snr,dvmn,psym=1,xtit='SNR [dB]',ytit='Mean DV Error [m/s]'
    grid
    plot,snr,dvsd,psym=1,xtit='SNR [dB]',ytit='DV Error St.Dev. [m/s]'
    grid
  endif

end
