;$Id: wcrhcor.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRHCOR
;
; PURPOSE:
;
;       Linear correction of H receiver data (lookup table adjustment).  
;       Data inlcude hhnoise, hhtx, hhlk, hh, hv, ppnoise, pptx, pp.
;
; CATEGORY:
;
;       wcrtools
;
; CALLING SEQUENCE:
;
;       wcrhcor, fitprms,what=what,silent=silent,help=help
; INPUT:
;
;       fitprms:  [slope,intercept] - optional fltarr(2) of the correction fit
;                 default: [0.95,0.]
;
; KEYWORD PARAMETERS:
;
;       what   :  Use this keyword to specify which data to be corrected
;                 Input: string array/scalar; Strings: 'hh','hv','hhtx','hhlk',
;                 'hhnoise','pp','pptx', 'ppnoise'
;                 Example: what=['hh','hhnoise']
;                 Default: ['hhnoise','hh','ppnoise','pp']
;       silent :  Use this keyword to supress some info messages.
;       help   :  Show this text
;
; COMMON BLOCKS:
;
;       common wcrparams
;       common wcrdata
;
; PROCEDURE:
;
;       Uses linear fit to correct lookup table calibration.
;       Intercept can be used to account for receivers' gain difference,
;       and in this case same cal constant should be used for H and V
;       After May 1999 all lookup tables were replaced and generally
;       no major correction is needed.
;
;       hcorrection flags the status of H-ch correction: 0/1 - no/yes
;       hcorrection=[Fhhnoise,Fhh,Fhv,Fhhtx,Fhhlk,Fppnoise,Fpp,Fpptx]
;
; EXAMPLE:
;
;       For PPMAG mode correct hhnoise, hh, ppnoise, pp using the default 
;       correction:
;
;       wcrhcor
;
; SIDE EFFECTS:
;
;       All necessary data must be loaded into the common block first.
;       Co-pol pulse pairs are corrected only (e.g., 'HH' and not 'HV') 
;
;-
; MODIFICATION HISTORY:
;       Written by: Samuel Haimov, February, 1997
;       S.H., 11/99  -- restored Tx leakage vars in wcrdata common 
;                       block, adjusted pp correction
;       S.H., 01/00  -- modified help keyword
;       S.H., 02/00  -- changed routine name from vxihcor to wcrhcor
;       S.H., 10/01  -- replaced atan(complex) with atan(imaginary(complex),
;                                                        float(complex))
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;       

pro wcrhcor, fitprms, what=what, silent=silent, help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrhcor'
  return
endif

if alg eq fftalg then return

if n_params() eq 0 then begin slope=0.95 & intercept=0. & end else begin
   slope=fitprms(0)
   if n_elements(fitprms) eq 1 then intercept=0. else intercept=fitprms(1)
endelse

if slope eq 0. and intercept eq 0 then return

if n_elements(what) eq 0 then $
   if alg eq sppAlg then begin 
      if txsequence(0) eq polH then $
;        what=['hhnoise','hh','hhtx','ppnoise','pptx','pp']
         what=['hhnoise','hh','ppnoise','pp']
      if txsequence(0) eq polv then what=['hhnoise','hv','hhlk']
   endif else if alg eq pp4alg or alg eq pp6alg then begin
;    what=['hhnoise','hh','hv','hhtx','hhlk','ppnoise', 'pp','pptx']
     what=['hhnoise','hh','hv','ppnoise', 'pp']
   endif

pphind=where(ppstrpol eq 'HH')

if not keyword_set(silent) then begin
   print,'H channel data correction in progress ...'
   print,' '
endif

if n_elements(hhnoise) gt 1 and (where(what eq 'hhnoise'))(0) ne -1 $
and hcorrection(0) eq 0 then begin
   hhnoise=10.^(0.1*(10.*alog10(hhnoise)-intercept)/slope)
   hcorrection(0)=1
   if not keyword_set(silent) then print,'HHNOISE corrected'
endif
if (n_elements(hh) gt 1) and (where(what eq 'hh'))(0) ne -1 $
and hcorrection(1) eq 0 then begin
   hh =10.^(0.1*(10.*alog10(hh) -intercept)/slope)
   hcorrection(1)=1
   if not keyword_set(silent) then print,'HH corrected'
endif

if (n_elements(hv) gt 1) and (where(what eq 'hv'))(0) ne -1 $          
and hcorrection(2) eq 0 then begin
   hv =10.^(0.1*(10.*alog10(hv) -intercept)/slope)
   hcorrection(2)=1
   if not keyword_set(silent) then print,'HV corrected'
endif

if (n_elements(hhtx) gt 1) and (where(what eq 'hhtx'))(0) ne -1 $
and hcorrection(3) eq 0 then begin
   hhtx=10.^(0.1*(10.*alog10(hhtx)-intercept)/slope)
   hcorrection(3)=1
   if not keyword_set(silent) then print,'HHTx corrected'
endif

if (n_elements(hhlk) gt 1) and (where(what eq 'hhlk'))(0) ne -1 $
and hcorrection(4) eq 0 then begin
   hhlk=10.^(0.1*(10.*alog10(hhlk)-intercept)/slope)
   hcorrection(4)=1
   if not keyword_set(silent) then print,'HHlk corrected'
endif

if (n_elements(ppnoise) gt 1) and (where(what eq 'ppnoise'))(0) ne -1 $
and (pphind[0] ne -1) and (hcorrection(5) eq 0) then $
   if alg eq sppalg then begin
     phi=atan(imaginary(ppnoise),float(ppnoise))
     mag=abs(ppnoise)
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     ppnoise=mag*exp(complex(0.,phi))
     hcorrection(5)=1
     if not keyword_set(silent) then print,'ppnoise corrected'
   endif else begin
     phi=atan(imaginary(ppnoise[pphind,*]),float(ppnoise[pphind,*]))
     mag=abs(ppnoise[pphind,*])
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     ppnoise[pphind,*]=mag*exp(complex(0.,phi))
     hcorrection(5)=1
     if not keyword_set(silent) then print,'ppnoise corrected'
   endelse

if (n_elements(pp) gt 1) and (where(what eq 'pp'))(0) ne -1 $
and (pphind[0] ne -1) and (hcorrection(6) eq 0) then $
   if alg eq sppalg then begin
      phi=atan(imaginary(pp),float(pp))
      mag=abs(pp)
      mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
      pp=mag*exp(complex(0.,phi))
      hcorrection(6)=1
      if not keyword_set(silent) then print,'co-pol pp corrected'
   endif else begin
     phi=atan(imaginary(pp[*,pphind,*]),float(pp[*,pphind,*]))
     mag=abs(ppnoise[*,pphind,*])
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     ppnoise[*,pphind,*]=mag*exp(complex(0.,phi))
     hcorrection(5)=1
     if not keyword_set(silent) then print,'pp corrected'
   endelse

if (n_elements(pptx) gt 1) and (where(what eq 'pptx'))(0) ne -1 $
and (pphind[0] ne -1) and (hcorrection(7) eq 0) then $
   if alg eq sppalg then begin
     phi=atan(imaginary(pptx),float(pptx))
     mag=abs(pptx)
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     pptx=mag*exp(complex(0.,phi))
     hcorrection(7)=1
     if not keyword_set(silent) then print,'ppTx corrected'
   endif else begin
     phi=atan(imaginary(pptx[pphind,*]),float(pptx[pphind,*]))
     mag=abs(pptx[pphind,*])
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     pptx[pphind,*]=mag*exp(complex(0.,phi))
     hcorrection(5)=1
     if not keyword_set(silent) then print,'ppTx corrected'
   endelse

if not keyword_set(silent) then begin 
   print,'Correction done.'
;  if intercept eq 0. then print,"H/V receivers' gain difference not corrected."
   print,' '
;  bell,2
endif

end  ; wcrhcor


 
