;$Id: wcrloadnc.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRLOADNC
;
; PURPOSE:
;       Load variables from a WCR netcdf file. This version of the WCR netcdf
;       file uses 'profile' dimension instead of 'time' dimension. There is no
;       time dimension for these type of WCR data files and profile dimension
;       represents non-regularly sampled radar data in time. The timestamps
;       matching the profiles can be composed from the netcdf file variables 
;       base_time and time_offset. 
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       result=wcrloadnc(ncid,varname [,filename],startt=startt,stopt=stopt)
;
; INPUTS:
;       ncid:     netCDF variable ID, returned from a previous call to 
;                 ncdf_open, ncdf_create, ncdf_varload or ncdf_fcopy
;       Varname:  string, netCDF variable name
;       Filename: string, optional netCDF file name; if given will open netCDF 
;                 file first. In this case ncid is an output var
;
; KEYWORD PARAMETERS:
;       STARTT:   input, long, start time in hhmmss; (default is base_time,
;                   see Notes); if beyond the time interval of the file, starts
;                   from the beginning
;       STOPT:    input, long, stop time in hhmmss, inclusive the last second 
;                   (default is the end if the file time, see Notes); if beyond
;                   the time interval of the file, read to the end of the file
;       HELP:     show this text ( use print,wcrloadnc(/help) )
;
; OUTPUTS:
;       ncid:     netCDF variable ID, returned if Filename is given
;
; SIDE EFFECTS:
;       The function returns an empty string if the usage is incorrect or 
;       -1 if the variable is not found or
;       -3 if 'base_time' and/or 'time_offset' do not exist.
;       -4 if STARTT and STOPT are not valid for this file
;
; NOTES:
;       The use of STARTT and STOPT requires the existence of netcdf variables
;       'base_time' and 'time_offset', where base_time is the start time of the
;       data in the file given in seconds after 00:00:00 Jan 1, 1970 and 
;       time_offset is offset in seconds from base_time.
;
; EXAMPLES:
;
;       hh=wcrloadnc(ncid,'hh') ; load hh(range,profile)
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, February 2002
;       S.H., Feb 2003 -- renamed routine from wcrload wcrloadnc
;       S.H., Apr 2004 -- fixed bug with startt and stopt handling
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrloadnc,ncid,varname,filename,startt=startt,stopt=stopt,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrloadnc'
    return,''
  endif

  if n_params() lt 2 then begin
    message,"Usage: res=wcrloadnc(ncid,varname [,filename])",/info
    return,''
  endif else if n_params() eq 3 then ncid=ncdf_open(filename)

; Inquire about the varname

  vari=ncdf_varinq1(ncid,varname)
  if vari.id eq -1 then return,-1              ; varname doesn't exist

; Check if startt and stopt are given and is possible to use them

  if n_elements(startt) eq 0 then strt=-1 else strt=startt
  if n_elements(stopt) eq 0 then stpt=-1 else stpt=stopt

  if strt ne -1 or stpt ne -1 then begin
    junk1=ncdf_varinq1(ncid,'base_time')
    if junk1.id eq -1 then return,-3     ; base-time doesn't exist
    junk1=ncdf_varinq1(ncid,'time_offset')
    if junk1.id eq -1 then return,-3     ; time_offset doesn't exist

    ; Load base_time and time_offset
    basetime=ncdf_varload(ncid,'base_time') 
    baset=basetime mod 86400
    timeoffset=ncdf_varload(ncid,'time_offset')
    n=n_elements(timeoffset)

    if strt eq -1 then strt=long(timeoffset[0]) else begin
       strt=long(time2sec(strt))
       if strt lt baset then strt=strt+86400-baset else strt=strt-baset
       strt=(where(strt le long(timeoffset)))[0]
       if strt lt 0 or strt gt n-1 then strt=0L
    endelse

    if stpt eq -1 then stpt=long(timeoffset[n-1]) else begin
       stpt=long(time2sec(stpt))
       if stpt lt baset then stpt=stpt+86400-baset else stpt=stpt-baset
       stpt=(where(stpt le long(timeoffset)+1L))[0]
       if stpt lt 0 or stpt gt n-1 then stpt=n-1
    endelse

    if stpt le strt then return,-4   ; invalid start/stop times

    ; Define offset and count

    offset1=strt
    count1=stpt-offset1+1
    if vari.ndims eq 1 then begin
      offset=offset1
      count=count1
    endif else if vari.ndims eq 2 then begin
      offset=[0,offset1]
      count=[vari.dimsize[0],count1]
    endif else begin
      offset=[0,0,offset1]
      count=[vari.dimsize[0:1],count1]  
    endelse

    ; Load varname

    ncdf_varget,ncid,vari.id,tmp,offset=offset,count=count   

  endif else begin
    ncdf_varget,ncid,vari.id,tmp   
  endelse

  if n_elements(tmp) gt 1 then tmp=reform(tmp) $
  else tmp=tmp[0]

  return,tmp 

end
