;$Id:
;+
; NAME:
;       WCRPLOTNC
;
; PURPOSE:
;       Display reflectivity, Z, and Doppler velocity, DV, for selected WCR 
;       netcdf files. 
;
;       This routine currently can handle Side/Up and/or Down beams data only
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrplotnc, minmax,colortb=colortb,noerase=noerase,startt=startt, $
;                  stopt=stopt,thresh=thresh,help=help
;
; INPUTS:
;       Minmax:  fltarr(2),[dvminmax,zmax]
;                dvminmax - float, Doppler velocity min/max limit in m/s
;                zmax     - float, Reflectivity max limit in dBz
;
; KEYWORD PARAMETERS:
;       COLORTB: intarr(2), [Zcolortb,DVcolortb], color tables for Z and DV
;                (default is [41,43] or [13,17] if 41/43 not available)
;       NOERASE: when set will not erase the widgets or the graphic windows 
;       STARTT:  input, long, start time in hhmmss (see SIDE EFFECTS)
;       STOPT:   input, long, stop time in hhmmss, inclusive the last second 
;       THRESH:  input, float, threshold level factor (2: default, 0: no thresh); 
;                data below thresh*std(noise) is not displayed
;       HELP:    show this text
;
; OUTPUTS:
;
; SIDE EFFECTS:
;       Input files for this routine are selected with the dialog_pickfile
;
;       If STARTT/STOPT is/are defined a single file selection only is allowed.
;       If STARTT and/or STOPT do/does not exist read from/to beginning/end of
;       file.
;
;       Opens one or more graphic windows and/or graphic widgets.
;
; NOTES:
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, April 2004
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrplotnc,minmax,startt=startt,stopt=stopt,thresh=thresh,noerase=noerase,$
              colortb=colortb,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrplotnc'
    return
  endif

; Initilize

  resetps2x
  widgetid=lonarr(2)
  minrefl=-40.
  maxrefl= 25.

  if n_elements(colortb) eq 0 then begin
    loadct,0,get_names=junk
    if n_elements(junk) lt 44 then begin
      zcol =13
      dvcol=17
    endif else begin
      zcol =41
      dvcol=43
    endelse
  endif else if n_elements(colortb) gt 1 then begin
    zcol =colortb[0]
    dvcol=colortb[1]
  endif else zcol=(dvcol=colortb[0])

; Select files to display

  if n_elements(startt)+n_elements(stopt) gt 0 then junk=0 else junk=1
  flst=dialog_pickfile(/read, filter='WCR.*.nc',path=datdir,multi=junk)
  if flst[0] eq '' then return

; Loop over all selected files and display reflectivity and velocity

  for k=0,n_elements(flst)-1 do begin

    message,'Display '+flst[k]+', '+num2str(k+1)+' of '+ $
             num2str(n_elements(flst)),/info

  ; Read the appropriate data from the netcdf files

    ppalg =ncdf_attload(ncid,'WCRtxseq',/global,flst[k])
    beams =long(ncdf_attload(ncid,'WCRactivebeamsconfig',/global))
    wcrf  =ncdf_attload(ncid,'WCRdaqfile',/global)
    misval=ncdf_attload(ncid,'dv','missing_value')

  ; Load corrected Doppler velocity (same var regardless of anttenna config.)

    dvc   =wcrloadnc(ncid,'dv',startt=startt,stopt=stopt)

  ; Make the time, check if it is crossing midnight and fix it

    time  =double(wcrloadnc(ncid,'base_time'))+ $
                  wcrloadnc(ncid,'time_offset',startt=startt,stopt=stopt)
    time  =time mod 86400d0

  ; Get the number of profiles recorded and take care of crossing midnight

    nprof = n_elements(time)

    junk=abs(max(time)-min(time))
    if junk gt 70000. then begin
      ind=where(time ge max(time))+1L
      time[ind:*]=time[ind:*]+86400d0
    endif

  ; Get range-to-profile aspect,radar range and calc the distance flown

    asp   =wcrloadnc(ncid,'wcraspect',startt=startt,stopt=stopt)
    ; find if the AC was still on the ground (tas < 50 m/s)
    ind   = where(asp eq misval)
    if ind[0] ne -1 then begin
      message,'Aircraft still on the ground or wrong TAS',/info
      asp[ind]=!values.f_nan
    endif
    asp   =mean(asp,/nan) 

    rrng  =wcrloadnc(ncid,'range')/1000.
    rgs   =rrng[1]-rrng[0]
    trng  ='Range [km]'

    xtop=[0,rgs/asp*nprof]
    xtt ='Distance [km]'

  ; Figure out if image zoom is needed

    if nprof/asp lt 3.*n_elements(rrng) then imgz=2.+(nprof/asp le 100.) $
    else imgz=0. 

  ; Set flags for reflectivity and velocity limits 

    if n_params() eq 0 then begin
      dvlim=1
      zlim=1 
    endif else begin
      dvlim=0
      zlim=1
      dvminmax=minmax[0]
      if n_elements(minmax) gt 1 then begin
        zmax=minmax[1]
        zlim=0
      endif
    endelse

  ; Get the threshold data

    if n_elements(thresh) eq 0 then thresh=2. else thresh=abs(thresh)
    if thresh gt 0 then begin
     hsnoise=wcrloadnc(ncid,'hsnoise',startt=startt,stopt=stopt)
     vsnoise=wcrloadnc(ncid,'vsnoise',startt=startt,stopt=stopt)
    endif

  ; Display the data depending on the beam configuration

    case beams of
      1   : begin
              hhc=wcrloadnc(ncid,'hh',startt=startt,stopt=stopt)

              if thresh gt 0 then begin
                junk=wcrthresh(hsnoise,rrng*1000.,thresh)
                ind=where(hhc lt junk)
              endif else ind=where(hhc eq misval)

              if ind[0] ne -1 then begin
                hhc[ind]=!values.f_nan
                dvc[ind]=!values.f_nan
              endif

              if zlim eq 0 then junk1=zmax else junk1=max(hhc)
              aximage,hhc,/tr,time,rrng,/timeax,setw=[-1,zcol,0,1],min=minrefl,$
                asp=asp,max=junk1,/bar,tit=wcrf+': Side/Up Reflectivity',      $
                xtit=['UTC','dBZ',xtt],ytit=trng,topx=xtop,widgetid=junk,      $
                imgzoom=imgz
              widgetid[0]=junk

              if dvlim then junk=max(dvc) else junk=abs(dvminmax)
              aximage,-dvc,/tr,time,rrng,/timeax,setw=[-1,dvcol,0,1],ytit=trng,$
                tit=wcrf+': Side/Up Doppler',min=-junk,max=junk,asp=asp,       $
                /bar,xtit=['UTC','m/s, blue:Up/Away',xtt],topx=xtop,           $
                widgetid=junk,imgzoom=imgz
              widgetid[1]=junk
            end
      2   : begin
              vvc=wcrloadnc(ncid,'vv',startt=startt,stopt=stopt)

              if thresh gt 0 then begin
                junk=wcrthresh(vsnoise,rrng*1000.,thresh)
                ind=where(vvc lt junk)
              endif else ind=where(vvc eq misval)

              if ind[0] ne -1 then begin
                vvc[ind]=!values.f_nan
                dvc[ind]=!values.f_nan
              endif

              if zlim eq 0 then junk1=zmax else junk1=min([max(vvc),maxrefl])
              aximage,vvc,/tr,time,reverse(rrng),/timeax,setw=[-1,zcol,0,1],  $
                min=minrefl,max=junk1,asp=asp,tit=wcrf+': Down Reflectivity', $
                /ord,xtit=['UTC','dBZ',xtt],ytit=trng,topx=xtop,widgetid=junk,$
                /bar,imgzoom=imgz
              widgetid[0]=junk

              if dvlim then junk=max(dvc) else junk=abs(dvminmax)
              aximage,dvc,/tr,time,reverse(rrng),/timeax,setw=[-1,dvcol,0,1], $
                tit=wcrf+': Down Doppler',xtit=['UTC','m/s, blue: Up',xtt],   $
                /ord,/bar,asp=asp,ytit=trng,min=-junk,max=junk,topx=xtop,     $
                imgz=imgz,widgetid=junk
              widgetid[1]=junk
            end
      12: begin
              hhc=wcrloadnc(ncid,'hh',startt=startt,stopt=stopt)
              vvc=wcrloadnc(ncid,'vv',startt=startt,stopt=stopt)

              if thresh gt 0 then begin
                junk=wcrthresh(hsnoise,rrng*1000.,thresh)
                ind =where(hhc lt junk)
                junk=wcrthresh(vsnoise,rrng*1000.,thresh)
                ind1=where(vvc lt junk)
              endif else begin
                ind =where(hhc eq misval)
                ind1=where(vvc eq misval)
              endelse

              if ind[0] ne -1 then begin
                junk      =reform(dvc[0,*,*])
                hhc[ind]  =!values.f_nan
                junk[ind] =!values.f_nan
                dvc[0,*,*]=junk
              endif
              if ind1[0] ne -1 then begin
                junk       =reform(dvc[1,*,*])
                vvc[ind1]  =!values.f_nan
                junk[ind1] =!values.f_nan
                dvc[1,*,*]=junk
              endif

              junk=2*rrng[0]/rgs-1
              rng=[-reverse(rrng),(findgen(junk)-fix(junk/2))*rgs,rrng]
              z =[rotate(vvc,5), $
                  fltarr(junk,nprof)-!values.f_nan,hhc]
              dv=[rotate(reform(dvc[1,*,*]),5),$
                  fltarr(junk,nprof)-!values.f_nan, $
                  reform(-dvc[0,*,*])]

              if zlim eq 0 then junk1=zmax else $
                junk1=max([max(hhc),maxrefl])

              aximage,z,time,rng,/tr,/bar,/timeax,setw=[-1,zcol,0,1],$
                tit=wcrf+', Top: Side/Up, Bottom: Down Reflectivity',$
                xtit=['UTC','dBZ',xtt],ytit=trng,asp=asp,topx=xtop,  $
                imgz=imgz,widgetid=junk,min=minrefl,max=junk1
              widgetid[0]=junk

              if dvlim then junk=max(dvc) else junk=abs(dvminmax)

              aximage,dv,time,rng,/timeax,/bar,setw=[-1,dvcol,0,1],max=junk,  $
                /tr,tit=wcrf+', Top:Side/Up, Bottom: Down Doppler',min=-junk, $
                xtit=['UTC','m/s, blue:Up/Away',xtt],ytit=trng,topx=xtop,     $
                asp=asp,widgetid=junk,imgzoom=imgz
              widgetid[1]=junk
              
            end
    else : message,'What is going on here !!!!',info
    endcase

  ; Select how to continue

    junk=''
    read,prompt='Continue(default)/Stop/Quit (c,s,q)',junk
    junk=strmid(strlowcase(strtrim(junk,2)),0,1)
    if junk eq 's' then stop $
    else if junk eq 'q' then begin
      ncdf_close,ncid
      return
    endif

  ; Erase the plots before continuing if not requested otherwise

    if keyword_set(noerase) then noerase=1 else noerase=0
    if noerase eq 0 then begin
      for l=0,n_elements(widgetid)-1 do $
         if widgetid[l] ge 1000 then wdelete,widgetid[l]-1000 $
         else widget_control,widgetid[l],/destroy  
    endif

  endfor

  if junk ne 'q' then ncdf_close,ncid

end 
