;$Id
;+
; NAME:
;    WCRSCOPE
;
; PURPOSE:
;    Creates a postscript file with plots of samples from WCR transmitter
;
; CATEGORY:
;    wcrtools	
;
; CALLING SEQUENCE:
;    wcrscope,fnscope,scopev,scopet,scopes,fnvxi,vvstat,timehack=timehack
;
; INPUTS:
;    fnscope - scope file name
;
; KEYWORDS:
;    timehack- time corr. in sec. If given correct scopet 
;              and scopes using the timehack instead of leapsec  
;
; OUTPUTS:
;    scopev  - scopev[*,0] - mean voltage
;              scopev[*,1] - rms voltage
;    scopet  - time in hhmmss,  corrected for leapsec=-13 (default)
;    scopes  - time in sec,  corrected for leapsec=-13
;
; SIDE EFFECTS:
;    Create PS file named <fnscope>.ps
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Jun 2001
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrscope,fnscope,scopev,scopet,scopes,fnvxi,vvstat,timehack=timehack

;common vxiparams
;common vxidata

; GPS time correction

  if n_elements(timehack) eq 0 then begin 
    leapsec=-13. 
    print,'Using leap seconds correction:',leapsec
  endif else begin 
    leapsec=timehack
    print,'Using timehack correction:',leapsec
  endelse

; Get the template (format) of the scope ascii file and read the file

  restore,'/home/haimov/mantis/HP54602/scope_template.sav'
  scope=read_ascii(fnscope,template=scope_template)

; Find identical elements in scope time field

  findequal,scope.field2,scopet,count,index=ind

; Convert scope time from hh:mm:ss to seconds and correct for leap seconds

  scopef2=timestr2sec(scope.field2)+leapsec
  n=n_elements(scopet)
  scopes=timestr2sec(scopet)+leapsec
  scopet=sec2timestr(scopes)

  print, "Number of seconds: ",num2str(n)

; Average all voltage measurements for every second

  n=n_elements(count)
  scopev=fltarr(n,2)
  ind1=[ind.start,n_elements(scope.field1)]

; Plot Mean and STD Voltage 

  for k=0,n-1 do begin
    scopev[k,0]=total(scope.field1[ind.sort[ind1[k]:ind1[k+1]-1]])/count[k]
    scopev[k,1]=std(scope.field1[ind.sort[ind1[k]:ind1[k+1]-1]])
  endfor

  smin=min(10.*alog10(scope.field1)+30.)-0.5
  smax=max(10.*alog10(scope.field1)+30.)+0.5

  setxpswin,99,2,/lands,psfile=fnscope+'.scope.ps',xyoff=[0.7,10.2], $
            xysize=[10,7]
  !p.thick=3
  
  seg=n/3600
  
  for k=0,seg do begin
  
    if seg eq 0 then ind=lindgen(n) $
    else if k lt seg then ind=k*3600+lindgen(3600) $
    else ind=3600L*seg+lindgen(n-seg*3600)
    
    ind1=where(scopef2 ge scopes[ind[0]] and scopef2 le $
               scopes[ind[n_elements(ind)-1]])
    plot,scopes[ind],10.*alog10(scopev[ind,0])+30.,xsty=5,title=fnscope, $
       ytit='W-Detector Voltage!C(- is ave per sec) [dBm]',ysty=1, $
	  yran=[smin,smax]
    oplot,scopef2[ind1],10.*alog10(scope.field1[ind1])+30.,psym=2,symsize=0.3
    grid,/ygrid
    timeaxis,tit='UTC'

;    plot,scopes[ind],10.*alog10(scopev[ind,1])+30.,xsty=5,title=fnscope, $
;       ytit='W-detector Voltage!CSTD [dBm]',yrange=[-10,10]
;    grid,/ygrid
;    timeaxis,tit='UTC'

    if k eq seg or (k mod 2) eq 1 then datestamp,xypgpos=[1,8],/inch,$
      comm=string(bytarr(25)+32B)+'page '+num2str(k/2+1)+'of'+num2str(seg/2+1)
  endfor
   
  resetps2x


return

; Do as above for the vxifile.  Use vv.

openvxif,fnvxi,data
readvxiall,data
;begtime='17:35:42'
timeh=long(timestr2sec(begtime)+time)

ind1=(where(scopes[0] eq timeh))[0]
ind2=where(scopes[n_elements(scopes)-1] eq timeh)
ind2=ind2[n_elements(ind2)-1]

;vv1=reform(vv[8,ind1:ind2])     ; CR return at 183 m (rgs 30 m)
vv1=reform(vv[11,ind1:ind2])     ; CR return at 225 m  (rgs 15 m)


findequal,timeh[ind1:ind2],subtime,count,index=ind

n=n_elements(count)
vvstat=fltarr(n,2)
ind3=[ind.start,n_elements(vv1)]

for k=0,n-1 do begin
   vvstat[k,0]=total(vv1[ind.sort[ind3[k]:ind3[k+1]-1]])/count[k]
   vvstat[k,1]=std(vv1[ind.sort[ind3[k]:ind3[k+1]-1]])
endfor

return




; Run these lines from the prompt

vxiscope,'WppScope-99-07-05-16-08-55',scopev,scopet,scopes,vvstat

fn1='WppScope99-07-14-17-18-19' & fn2='Wpp99-07-14-17-18-19'
vxiscope,fn1,fn2,scopev,scopet,scopes,vvstat

setxpswin, 99, 3

plot,scopes,10.*alog10(scopev[*,0]),psym=1, xstyle=5,$
     tit=fn1,xtit='Time', $
     ytit='W-band Detector Voltage (ave per sec) [dB]'
timeaxis
errplot,scopes,10.*alog10((scopev[*,0]-scopev[*,1])>1.e-20),$
        10.*alog10(scopev[*,0]+scopev[*,1])
grid,xtitle='Time',xtickname=replicate(' ',30),ytickname=replicate(' ',30)

plot,scopes,10.*alog10(vvstat[*,0]),psym=2, xstyle=5,$
     tit=vxifile,xtit='Time', $
     ytit='VV Reflectivity (ave per sec) [dBm]'
timeaxis
errplot,scopes,10.*alog10((vvstat[*,0]-vvstat[*,1])>1.e-20),$
        10.*alog10(vvstat[*,0]+vvstat[*,1])
grid,xtitle='Time',xtickname=replicate(' ',30),ytickname=replicate(' ',30)

ab=linfit(10.*alog10(scopev[*,0]),10.*alog10(vvstat[*,0]))

plot,10.*alog10(scopev[*,0]),10.*alog10(vvstat[*,0]),psym=5, $
     title='Linear Fit:  ReflPwr[dBm]='+num2str(ab[0])+' + '+ $
            num2str(ab[1])+'*DetVolt[dBV]', $
     xtit='W-band Detector Voltage [dBV]', ytit='V-ch Refl. [dBm]'

oplot,10.*alog10([min(scopev[*,0]),max(scopev[*,0])]), $
      ab[0]+ab[1]*10.*alog10([min(scopev[*,0]),max(scopev[*,0])])

datestamp
resetps2x

end
