;$Id: wcrtfp1scor.pro,v 1.2 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRTFP1SCOR
;
; PURPOSE:
;       This routine corrects a specific problem with the WCR Time (IRIG) card
;       The problem appears as one or more 1 second negative jumps at the top
;       of a second followed by a single recovery positive jump. The problem is 
;       intermittent and appears in some of the WCR data collected during IHOP
;       experiment in 2002.
;       
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       res=wcrtfp1scor(time)
;
; INPUTS:
;       Time:   dbl/fltarr(n), wcr time array for n profiles to be corrected
;
; KEYWORD PARAMETERS:
;       HELP:   when set show this text
;
; OUTPUTS:
;       The function returns an array with corrected time matching the input
;       array type.
;
; COMMON BLOCKS:
;       None
;
; SIDE EFFECTS:
;       There is a small probability that this routine can generate negative
;       time jumps in the corrected time.  This may happen if a fixed 1-s 
;       negative time jump is combined with a simultaneous data gap (excessive
;       time interval between profiles due to data acquisition problem)
;
; NOTES:
;       This routine is intended for correcting the so-called "1-sec negative
;       jumps" in the WCR time stamps caused by a problem with TFP(IRIG) card.
;       It is also expected that a recovery of the one or more negative jumps
;       always takes place right after the negative jumps.  Therefore the 
;       routine checks the matching between the negative jumps and the recovery
;       positive jumps.  If such match does not exists an error message is
;       returned.
;
;       Care should be taken to secure that the time array being corrected
;       does not start or end within a time period when negative jumps/recovery
;       occur.
;
;       The function checks corrected time array for uncorrected or generated 
;       negative time jumps and reports it in an informational message if such
;       are found.  The user must examine the input/output time array and take
;       appropriate action.
;
;       The time problem fixed by this routine occured in the data collected
;       during year 2002 (IHOP02 and HICU02 experiments)
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, January 2003
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrtfp1scor,time,help=help

  on_error,2 

  ; Fetch the help if wanted

  if keyword_set(help) then begin
    doc_library,'wcrtfp1scor'
    return,''
  endif

  ; Check in/out params

  if n_params() ne 1 then begin
    message,"Usage: res=wcrtfp1scor(time)",/info
    return,''
  endif

  ; Initilize

  minjmp=-1.1
  maxjmp=-0.8
  ctime=time

  ; Calculate profile time intervals

  dt=(time-shift(time,1))[1:*]

  ; Find 1-sec negaitive time jumps (current time stamp < previous time stamp)

  bjmpi=where(dt ge minjmp and dt le maxjmp)

  ; Correct negative 1-s time jumps if possible.

  if bjmpi[0] ne -1 then begin

    ; Check if all negative jumps will be recovered  

    nbjump=n_elements(dt[bjmpi])
    fjmpi=where(dt ge min(-dt[bjmpi])); find matching or greater positive jumps
    nfjump=n_elements(dt[fjmpi])
    if nbjump lt total(round(dt[fjmpi])) then $
      message,'Total negative time jumps less than positive recovery jumps' $
    else if nbjump gt total(round(dt[fjmpi])) then $
      message,'Total negative time jumps more than positive recovery jumps'
 
    ; Recover negative jumps

    corr=0.
    for k=bjmpi[0],fjmpi[nfjump-1] do begin
      if time[k+1]-time[k] le maxjmp then corr=corr+1. $
      else if corr ge 1. and time[k+1]-time[k] ge -corr*maxjmp then corr=0.
      ctime[k+1]=time[k+1]+corr
    endfor

    ; Check if any negative jumps are still present

    if (where((ctime-shift(ctime,1))[1:*] lt 0.))[0] ne -1 then $
      message,'Uncorrected negative time jumps still exist.',/info 

  endif

  return,ctime

end
