;$Id: wcrtimeoff.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRTIMEOFF
;
; PURPOSE:
;       Calculates time offset between WCR time stamps and N2UW 
;       KingAir data system time stamps.  It also returns the
;       corresponding correlation coefficient.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrtimeoff,toff,tcov,beam,drng,dprof, acncf=acncf,wcrfn=wcrfn, $
;                  nowcrread=nowcrread,ground=ground,ppselect=ppselect,$
;                  xplot=xplot,filt=filt,more=more,help=help
;
; INPUTS:
;    Beam:   Anntenna beam
;              fltarr(3),  [ac_along_axis,right_wing,down] or
;              0         - side-looking KingAir, KA, (main) 12"antenna
;              1         - up-looking KA (main+mirror) 12" antenna
;              2         - KA side-fore antenna(~35 deg)
;              3         - KA nadir 18" antenna
;              4         - KA down-fore 15" antenna(~30 deg)
;              5         - NCAR C130 nadir 12" antenna (7/16/01)
;              6         - NCAR C130 down-aft 12" antenna(7/16/01)
;              7         - NRC CV580 side looking 12" antenna
;              8         - NRC CV580 nadir looking 12" antenna
;
;    Drng:   [first,last] range of wcr data range gates to be used; default is
;            all range gates
;    Dprof:  [first,last] wcr data profile range to be used; default is  
;            all profiles; to input Dprof, Drng must exists (use 0 for default)
;           
; KEYWORD PARAMETERS:
;    ACNCF:     input,string; AC 25 HZ netcdf data file name or netcdf ID;
;               If not defined the program uses AC data loaded in a previous
;               call to this routine
;    WCRFN:     input string; WCR raw data file name.  If not given the
;               needed file is assumed open. The [drng,dprof] data is 
;               loaded unless NOWCRREAD keyword is set.
;    PPSELECT:  input, string; type of pulse pair to be used: 'HH','VV'
;               for the given beaminit vector (ignored if beam 0 to 6)
;    NOWCRREAD: when set does not read wcr raw data and use already loaded wcr 
;               data. If drng and dprof are defined it uses the subset of the
;               already loaded data.
;    GROUND:    input, float; using surface return data 
;                 1 - finds max returns for every profile and assumes that they
;                     are surface returns. Optimal time offset is calculated
;                     based on cross correlation between AC motion into the
;                     wcr beam and the uncorrected wcr Doppler velocity
;                 x - -abs(x) threshold in dBm; all data above the threshold
;                     are considered surface returns. Optimal time offset is
;                     searched based on minimum mean and stdev of the corrected
;                     Doppler velocity from the surface (the surface is assumed
;                     not moving) 
;                -1 - surface is determined as ground=1 but optimal time offset
;                     is searched like ground=x 
;                ATTN: i)  For 1 and -1 make sure there is a ground return 
;                          throughout the processed data segment.
;                      ii) Also when using correlation analysis(1) any folding
;                          in the uncorrected Doppler needs to be fixed
;    XPLOT:     when set plots the results; if set to 99 creates idl.ps file
;    HELP:      when set show this text
;
; OUTPUTS:
;    Toff:      float, time offset between WCR and AC time in seconds
;               (positive means WCR time is ahead of AC time)
;    Tcov:      GROUND keyword not defined or equal to 1:
;                 float, WCR/AC correaltion coeff. for Toff (-1 to 1).
;               GROUND keyword defined and not equal to 1:
;                 fltarr(2), [mean_grnd_vel, std_grnd_vel] at Toff
;
; SIDE EFFECTS:
;
; PROCEDURE:    GROUND keyword not defined:
;                 Time offset is estimated by cross correlating the aircraft (AC) 
;                 motion into the WCR beam (calculated from the AC INS data and
;                 predefined wcr beam pointing angles) and the WCR measured
;                 Doppler velocity. The coordinate system is ground referenced.
;               GROUND keyword defined:
;                 Search for radar returns above the given threshold and 
;                 assume they are from a surface.  Then calculate the time
;                 offset based on the minimum mean and standard deviation RMS
;                 error of the corrected for aircraft motion Doppler from 0 m/s
;
; NOTES:
;               Measured Doppler velocity could be folded but this procedure 
;               does not make an attempt to correct if folded values are present. 
;               Thus existing folding can corrupt the results.
;
;               The quality of the result (high correlation coefficient)
;               depends on the dynamic of the AC velocity contribution into
;               the radar beam and the steadiness of the wind contribution.  
;               On the other hand extreme maneuvers that cause high variability
;               of the ac velocity contribution may not be the best cases
;               due to increase in measurement errors in the data used.            
;  
; EXAMPLES:
;
;    1) Using cross correlation (GROUND keyword not defined):
;
;    wcrtimeoff,toff,tcov,3,[2,5],[2800,3100],/xplot, $
;               wcrf='/net/fox/fox1/WCR/HICU/jul18/W2002-07-18-16-57-03', $
;               acncf='/net/owl/data/kingair_data/20020718.c25.nc'
;
;    Another test using different range gates could be done the following way:
;
;    wcrtimeoff,toff,tcov,3,[4,7],[2800,3100],/xplot
;
;    Also wcr data can be read before calling wcrtimeoff and then run it with
;    different profile and range gate segments.  Once the aircraft data is
;    read there is no need to read it again as long as the same AC data file
;    is relevant for the examined wcr data. For these cases use /nowcrread
;
;    2) Same data as above but using the ground returns in the data
;       All profiles and range gates are loaded. No new read of King Air
;       data is needed.
;
;    wcrtimeoff,toff,tcov,3,0,0,/xplot,ground=-30., $
;               wcrf='/net/fox/fox1/WCR/HICU/jul18/W2002-07-18-16-57-03'
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, August 2002
;       S.H., Feb 2003 -- replaced prefix ka with ac where appropriate
;       S.H., Apr 2003 -- renamed acload to acloadnc
;       S.H., Jun 2003 -- modified beam and added ppselect keyword
;       S.H., Aug 2003 -- minor change in aximage plot
;       S.H., Oct 2003 -- changed beaminit help (added wcr beams for NRC Convair)
;                         modified ground keyword, added more keyword; added
;                         filt keyword(experimental mode) 
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrtimeoff,toff,tcov,beam,drng,dprof, acncf=acncf,wcrfn=wcrfn, $
               nowcrread=nowcrread,ground=ground,ppselect=ppselect,$
               xplot=xplot,filt=filt,more=more,help=help

  @commonwcrinclude
  common wcrtimeoff_temp,acdata,alpha,beta

  on_error,2

  ; Fetch the help if wanted

  if keyword_set(help) then begin
    doc_library,'wcrtimeoff'
    return
  endif

  ; Check in/out params

  if (n_params() lt 4 and keyword_set(ground) eq 0) or $
     (n_params() lt 3 and keyword_set(ground)) then begin
    message,"Usage: wcrtimeoff,toff,tcov,beam,drng,dprof,acncf=acncf, $",/info
    print,"              wcrfn=wcrfn,nowcrread=nowcrread,ground=ground,filt=filt,$"
    print,"              ppselect=ppselect,xplot=xplot,more=more,help=help"
    return
  endif

  if n_elements(ground) eq 0 then grnd=-999 $
  else if abs(ground) ne 1 then grnd=10.^(-0.1*abs(ground)) else grnd=ground

  dt    = 0.04     ; time step in seconds based on 25 Hz data
  maxlag= 3.0      ; maximum search time offset
  wcrreadf=1B
  if keyword_set(nowcrread) then begin

    ; Create wcrtime time and get the desired range/prof segment.
    ; At this point SNR is not used to control the strength of the signal. 
    ; I may do this in the future if automatic analysis is to be 
    ; done  

    if timemode eq 3 then wcrtime=double(basetime)+time $
    else if timemode eq 4 then wcrtime=double(basetime-long(time[0]))+time $
    else message,'timemode must be 3 or 4'

    if n_elements(dprof) eq 1 then wcrt=wcrtime[dprof:*] $
    else if n_elements(dprof) eq 2 then wcrt=wcrtime[dprof[0]:dprof[1]] $
    else wcrt=wcrtime
  endif else begin

    ; Read the raw wcr data (pulse pair for the desired segment only)

    junk=1
    if n_elements(wcrfn) eq 0 then junk=0 $
    else if n_elements(wcrfile) eq 1 then $
         if wcrfile eq strmid(wcrfn,19,/reverse_offset) then junk=0
    if junk then begin
      print,'Openning wcr raw file ...'
      openwcrf,wcrfn
    endif
    wcrreadf=0B
    print,'Reading wcr raw data ...'
    if n_elements(dprof) eq 0 then $
      readwcrdata,rawtime=wcrtime,/nocross,/noleak $
    else if n_elements(dprof) eq 1 then $
      readwcrdata,dprof,rawtime=wcrtime,/nocross,/noleak $
    else if n_elements(dprof) eq 2 then $
      readwcrdata,dprof[0],dprof[1],rawtime=wcrtime,/nocross,/noleak
    wcrt=wcrtime
  endelse

  ; Calculate uncorrected Doppler velocity for the appropriate antenna beam;
  ; If more than one pulse pair is available use the one with the bigger prf.

  if n_elements(beam) eq 3 then begin
    if n_elements(ppselect) eq 0 then $
      message,'ppselect must be defined when beam is a vector'
    beams=strarr2str(num2str(beam))
  endif else begin
    case beam of
      0: begin & beams=' N2UW Side'       & ppselect='HH' & end
      1: begin & beams=' N2UW Up'         & ppselect='HH' & end
      2: begin & beams=' N2UW Side-fore'  & ppselect='VV' & end
      3: begin & beams=' N2UW Nadir'      & ppselect='VV' & end
      4: begin & beams=' N2UW Down-fore'  & ppselect='HH' & end
      5: begin & beams=' C130 Nadir'      & ppselect='HH' & end
      6: begin & beams=' C130 Down-aft'   & ppselect='VV' & end
      7: begin & beams=' NRC CV580 Side'  & ppselect='HH' & end
      8: begin & beams=' NRC CV580 Nadir' & ppselect='VV' & end
    endcase
  endelse 

  wcrdv=wcrdopvel(ppselect=ppselect,maxv=maxv)

  if size(wcrdv,/n_dim) eq 3 then begin
    if n_elements(wcrdv[0,0,*]) eq 2 then $
      junk=max([max(wcrdv[*,*,0]),max(wcrdv[*,*,1])],ind1) $
    else if n_elements(wcrdv[0,0,*]) eq 3 then $
      junk=max([max(wcrdv[*,*,0]),max(wcrdv[*,*,1]),max(wcrdv[*,*,2])],ind1)
    wcrdv=wcrdv[*,*,ind1]
  endif

  ; Get the rng/prof segment of the Doppler

  r3=0L & r4=numrec-1L
  if wcrreadf then begin
    if n_elements(dprof) eq 1 then r3=dprof $
    else if n_elements(dprof) eq 2 then begin
      r3=dprof[0] & r4=dprof[1]
    endif
  endif 

  r1=3L & r2=rangegates-1L
  if n_elements(drng) eq 1 then r1=drng $
  else if n_elements(drng) eq 2 then begin
    r1=drng[0] & r2=drng[1]
  endif
 
  wcrdv=wcrdv[r1:r2,r3:r4]  
 
  ; Calculate the aircraft motion contribution into the wcr beam
 
  if n_elements(acncf) ne 0 then begin
    if size(acncf,/type) eq 7 then ncid=ncdf_open(acncf) else ncid=acncf
    print,'Reading/processing King Air data ...'
    wcracv=wcracvelm(acdata,ncid,beaminit=beam,acv=acv)
    ;alpha=acloadnc(ncid,'alpha')
    ;beta =acloadnc(ncid,'beta')
  endif else wcracv=wcracvelm(acdata,beaminit=beam,acv=acv)

  ; Calculate time offset based on correlation or surface threshold

  n=n_elements(wcrt)
  ind1=(where(acdata.time gt wcrt[0]))[0]
  ind2=(where(acdata.time ge wcrt[n-1]))[0]

; --------------------------------------------------------------------------
  if grnd eq -999 or grnd eq 1 then begin  ; Correlation calculations

    print,'Correlation analysis ...'

    ; Interpolate wcr Doppler to match aircraft 25 Hz sampling rate

    ind1=ind1+1L    ; make sure actime is subset of wcrtime
    ind2=ind2-1L 
    actime=acdata.time[ind1:ind2] mod 86400   ; in sec after midnight   

    if grnd eq -999 then begin
      wcrdv1=mean(wcrdv,1)
      wcrdv=ac2wcr(wcrdv1,acdata.time[ind1:ind2],wcrt,/wcr2ac,/err)
    endif else begin
      if ppselect eq 'HH' then junk=max(hh[r1:r2,r3:r4],ind,dim=1) $
      else junk=max(vv[r1:r2,r3:r4],ind,dim=1)
      ; ind mod (r2-r1+1) ; range gate index of the max return for every prof.
      wcrdv1=wcrdv[ind]
      wcrdv=ac2wcr(wcrdv1,acdata.time[ind1:ind2],wcrt,/wcr2ac,/err)
    endelse

    ; Detrend (High pass filter) wcracv and wcrdv if requested

    if keyword_set(filt) then begin

      wcracv=wcracv[ind1:ind2]

      ; Unfold 

      ;ii=where(wcrdv gt -7.)      
      ;wcrdv[ii]=wcrdv[ii]-31.6

      ;ii=where(wcrdv[0:1200] gt 4.8)
      ;wcrdv[ii]=wcrdv[ii]-15.8
      ;ii=where(wcrdv[4500:6000] gt 2)
      ;wcrdv[4500+ii]=wcrdv[4500+ii]-15.8

      ;HP filter: get coeff. based on 25Hz sampling and 1.25Hz cutoff
      ;a=digital_filter(0.1,1.,50.,32)
      ;wcracv=convol(wcracv,a)

      ;wcracv=detrend(wcracv[ind1:ind2])
      ;wcrdv=detrend(wcrdv)

      ; LP filter: 
      ; a=digital_filter(0.,0.05,50.,32)
      ; wcracv=convol(wcracv[ind1:ind2],a)
    endif else wcracv=wcracv[ind1:ind2]

    ; Caluclate the cross-correlation function between ac motion component 
    ; and the wcr Doppler for time lags between -maxlag and +maxlag seconds

    junk=findgen(2*long(maxlag/dt)+1)-maxlag/dt 
    c1=c_correlate(wcracv,wcrdv,junk)

    ; Get the max correlation coefficient and the corresponding time lag

    tcov=max(c1,imax)
    toff=junk[imax]*dt

    ; Create a plot if desired

    if n_elements(xplot) eq 1 then begin

      ;Plot also results using pitch, angle of atack and sideslip

      ; c2=c_correlate(acdata.pitch[ind1:ind2],wcrdv,junk)
      ; c3=c_correlate(alpha[ind1:ind2],wcrdv,junk)
      ; c4=c_correlate(beta[ind1:ind2],wcrdv,junk)
      c3=c_correlate(acv[ind1:ind2,0],wcrdv,junk)
      c4=c_correlate(acv[ind1:ind2,1],wcrdv,junk)
      c5=c_correlate(acv[ind1:ind2,2],wcrdv,junk)      

      if xplot eq 99 then setxpswin,99,2 else setxpswin,32,2

      drngs='rg='+num2str(r1)+':'+num2str(r2)
      dprofs=sec2timestr(wcrt[0] mod 86400)+'-'+sec2timestr(wcrt[n-1] mod 86400)
      tt=dprofs+'; '+drngs+ $
         '; Rmax='+num2str(tcov,fr=2)+',Time offset='+num2str(toff,fr=2)

      ymax=max([max(wcracv),max(wcrdv)])
      ymin=min([min(wcracv),min(wcrdv)])

      plot,actime,wcracv,xstyle=5,thick=1,$
        titl=wcrfile+', beam:'+beams,yrange=[ymin,ymax], $
        ;ytit='wcrdv ...;wcracv -;acvx/10-.-;acvy-...-;acvz -- [m/s]'
        ytit='WCRDV ... WCRACV - [m/s]'
        grid,/ygrid
        timeaxis,major=1
      oplot,actime,wcrdv,lin=1

      plot,junk*dt,c1,title=tt,xtit='Time Lag [sec]', yrange=[-1.,1.], $
        ytit='WCRDV Correlation (wcracv -;acvx-.-;acvy-...-;acvz -- )'
      oplot,junk*dt,c3,lin=3
      oplot,junk*dt,c4,lin=4
      oplot,junk*dt,c5,lin=2
      grid

      if xplot ne 99 then setxpswin,32,2 $
      else printwcrstamp

      ymax=max(acv[*,1:2])
      ymin=min(acv[*,1:2])
      plot,actime,acv[ind1:ind2,0],xstyle=5, $
        titl=wcrfile+', beam:'+beams,ytit='AC longitudinal velocity [m/s]'
        grid,/ygrid
        timeaxis,major=1

      plot,actime,acv[ind1:ind2,1],yrange=[ymin,ymax],xstyle=5, $
        titl=wcrfile+', beam:'+beams, $
        ytit='AC lateral (-) and down (--) velocities [m/s]'
        grid,/ygrid
        timeaxis,major=1
      oplot,actime,acv[ind1:ind2,2],lin=2

      if xplot eq 99 then begin
        printwcrstamp
        resetps2x
      endif
    endif
; --------------------------------------------------------------------------
  endif else begin  ; Thresholded surface data or extract surface returns

    if grnd ne -1 then begin
      print,'Tresholded surface return analysis ...'
      if ppselect eq 'HH' then ind=where(hh[r1:r2,r3:r4] gt grnd) $
      else ind=where(vv[r1:r2,r3:r4] gt grnd)
    endif else begin
      print,'Continuous surface return analysis ...'
      if ppselect eq 'HH' then junk=max(hh[r1:r2,r3:r4],ind,dim=1) $
      else junk=max(vv[r1:r2,r3:r4],ind,dim=1)
    endelse

    nind=n_elements(ind)
    print, 'Found '+num2str(nind)+' data points from the surface.'

    if nind lt 10 then begin
      message,'Too few surface points.',/info
      return
    endif

    junk=2L*long(maxlag/dt)+1L
    ind1=ind1-junk    ; make sure wcrtime is subset of actime
    ind2=ind2+junk    

    junk=2*fix(maxlag/dt)
    out=fltarr(4,junk+1)

    for k=0,junk do begin
      timehack=-maxlag+k*dt
      wcrdv1=wcrdopcor(wcrdv,wcrt,wcracv[ind1:ind2],acdata.time[ind1:ind2], $
                       maxv=maxv,timehack=timehack)

      out[*,k]=[-timehack,mean(wcrdv1[ind]),std(wcrdv1[ind]), $
           sqrt(total(wcrdv1[ind]^2)/nind)]
    endfor

    junk=min(out[3,*],k)
    toff=out[0,k]
    tcov=out[1:2,k]

    ; Create a plot if desired

    if n_elements(xplot) eq 1 then begin
      if grnd ne -1 then begin
       if ppselect eq 'HH' then $
        aximage,10.*alog10(hh[r1:r2,r3:r4])>(-abs(ground)),xtit='Profile', $
          /tr,/bar,title=wcrfile+' HH>'+num2str(-abs(ground),fr=1)+'dBm',$
          ytit='Range gate',setw=[-1,41],time[r3:r4],findgen(r2-r1+1)+r1,/timeax $
       else if ppselect eq 'VV' then $
        aximage,10.*alog10(vv[r1:r2,r3:r4])>(-abs(ground)),xtit='Profile', $
          /tr,/bar,title=wcrfile+' VV>'+num2str(-abs(ground),fr=1)+'dBm',$
          ytit='Range gate',setw=[-1,41],time[r3:r4],findgen(r2-r1+1)+r1,/timeax
       if xplot eq 99 then setxpswin,99,3 else setxpswin,32,3
       !p.charsize=1.4
       junk=wcrfile+', beam:'+beams+', Surface Return(>'+num2str(-abs(ground), $
           fr=1)+'dBm)!C'
      endif else begin
       if xplot eq 99 then setxpswin,99,4 else setxpswin,32,4
       !p.charsize=1.4
       junk=''
       ymax=max([max(wcracv[ind1:ind2]),max(wcrdv[ind])])
       ymin=min([min(wcracv[ind1:ind2]),min(wcrdv[ind])])

       plot,wcrt mod 86400,wcrdv[ind],xstyle=5,lin=1,$
        titl=wcrfile+', beam:'+beams+', Surface Return',yrange=[ymin,ymax], $
        ytit='WCRDV ... WCRACV - [m/s]'
        grid,/ygrid
        timeaxis,major=1,chars=1.4
       oplot,actime,wcracv[ind1:ind2]
      endelse

      dprofs=sec2timestr(wcrt[0] mod 86400)+'-'+sec2timestr(wcrt[n-1] mod 86400)
      title=junk+dprofs+'; Time offset='+num2str(toff,fr=2)+', <Dv>='+ $
            num2str(tcov[0],fr=2)+', std(Dv)='+num2str(tcov[1],fr=2)

      plot,out[0,*],out[2,*], /xstyle,ymargin=[0,2],xtickname=replicate(' ',28),$
       ytit='Doppler Velocity STD [m/s]',tit=title
       grid,ymargin=[0,2]
      plot,out[0,*],out[1,*],xstyle=5,ymargin=[0,0], $
       ytit='Mean Doppler Velocity [m/s]'
      grid,ymargin=[0,0]
      plot,out[0,*],out[3,*], /xstyle,ymargin=[4,0], $
       xtit='WCR Time shift [s]',ytit='RMSE Doppler Velocity [m/s]'
      grid,ymargin=[4,0]
      printwcrstamp
      resetps2x
    endif
  endelse

  if keyword_set(more) then stop

end
