;$Id: wcrupdatenc.pro,v 1.6 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRUPDATENC
;
; PURPOSE:
;       Update reflectivity or Doppler velocity variables of WCR netcdf file(s)
;       as well as some other associated variables and attributes 
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrupdatenc,ncfile,wcrdir,writestr,acpath=acpath,acnames=acnames, $
;                   acspeed=acspeed,nswin=nswin,cal=cal,utccorr=utccorr, $
;                   timesync=timesync,unfold=unfold,gpsc=gpsc,rmcor=rmcor, $
;                   revision=revision,big_endian=bgnd,silent=silent,help=help
;
; INPUTS:
;       Ncfile:   string or strarr with path/name(s) of WCR netcdf file(s) to
;                   be updated
;       Wcrdir:   string, path to the raw WCR files associated with Ncfile
;                   (default is current directory)
;       Writestr: input, string, determine what to update; allowed strings:
;                  'all' - reflectivity and corrected doppler velocity vars.
;                  'z'   - reflectivity related variables
;                  'dv'  - uncorrected Doppler velocity related variables
;                  'zdv' - 'z' + 'dv' variables
;                  'cdv' - corrected Doppler velocity
;
; KEYWORD PARAMETERS:
;       ACPATH:   input,string, path to the aircraft navigational netcdf data
;                   file associated with Ncfile (default is current directory)
;                 input, string, path/name to the aircraft navigational netcdf
;                   data file, if 'cdv' is to be written for the first time
;
;       ACNAMES:  input, string array, A/C netcdf varnames to be used
;                   (if not defined N2UW or N130AR default var names are used)
;                   acnames[0] - AC true air speed in m/s
;                   acnames[1] - AC roll [deg]
;                   acnames[2] - AC pitch [deg]
;                   acnames[3] - AC true heading [deg]
;                   acnames[4] - AC east-west(EW) ground velocity [m/s]
;                   acnames[5] - AC north-south(NS) ground velocity [m/s]
;                   acnames[6] - AC vertical velocity [m/s]
;                   acnames[7] - AC East component of the wind [m/s]
;                   acnames[8] - AC North component of the wind [m/s]
;                   acnames[9] - AC vertical wind [m/s]
;                   acnames[10]- AC roll rate [rad/s] (use '' if N/A)
;                   acnames[11]- AC pitch rate [rad/s](use '' if N/A)
;                   acnames[12]- AC heading rate [rad/s] (use '' if N/A)
;                   acnames[13]- AC EW ground vel. GPS correction(use '' if N/A)
;                   acnames[14]- AC NS ground vel. GPS correction(use '' if N/A)
;
;                   Note: define first 10 names only if the last 5 vars are
;                         not needed; see ACLOADINS.PRO help for default 
;                         variable names
;                 This keyword is used with WRITESTR='cdv' and 'all' only
;
;       ACSPEED:  input, float; average aircraft speed in m/s (default 90.)
;                   This keyword is used with WRITESTR='dv' and 'zdv' only
;
;     BIG_ENDIAN: when set, assumes that the WCR file data are in big endian order.
;                   The default is LITTLE_ENDIAN. Ignored for data before 03/2002 
;
;       CAL:      input,fltarr(7),[calHH1,calVV1,calVV2,calVV3,calHH4,HViso,VHiso], 
;                   calibration coefficients in dB; default are the values in
;                   the Ncfile; Note: the order of cal. constants is fixed; thus
;                   either define as many as needed in the given order or use
;                   -999. to skip updating a particular cal. const.             
;                   This keyword is used with WRITESTR='z' and 'all' only
;
;       GPSC:     when set does NOT apply GPS correction to A/C E-W and N-S
;                   velocity for UWKA only (ignored for NCAR C130)
;                   NCAR C130 A/C E-W and N-S ground speed are GPS corrected.
;                   NRC CV580 GPSC is not needed.
;                   (default is the option used for Ncfile, if not available
;                    use the default as defined in wcrcreatenc routine)
;                   This keyword is used with WRITESTR='cdv' and 'all' only
;
;       MISVAL    missing value, to be used for the updated variables
;                   (default: the values saved in the missing_value attributes)
;
;       NSWIN:    input, long, lonarr(3), lonarr(3,n)
;                   nswin[0:1] - smoothing noise-window size(default 99);should 
;                                be an odd number(1 is added for evens); it is 
;                                recommended not to exceed 10% of the processed 
;                                number of profiles;  values less than 3 and 
;                                greater than 50% of all profiles cause the use
;                                of constant mean and stdev of the noise data 
;                   nswin[2]   - defines which noise measurement pulses to use:
;                     0 - (default) use matching number of noise measurement 
;                         pulses with minimum mean
;                     1 - uses noise measurement from one noise pulse with
;                         min mean (for both h- and v-ch)
;                     2 - as 1 for h-ch and 0 for v-ch noise 
;                     3 - as 1 for v-ch and 0 for h-ch noise 
;
;                   If NSWIN is a scalar, same value is applied to both channels
;                   as smoothing window size and NSWIN[2] is the default.
;                   If NSWIN has 2 elements, nswin[0] is applied to H and V
;                   noise channels and nswin[1] is used as nswin[2].
;
;                 lonarr(3,n) -as above but different for every profrange
;                   pair(ncfile). All nswin values must be given explicitly.
;
;                 This keyword is used with WRITESTR='z' and 'all' only
;
;       RMCOR:    integer/fltarr(3), applies antenna moment correction
;                  0 - antenna moment correction NOT applied  
;                  1 - antenna moment correction applied
;                  [x,y,x] - ant. position in aircraft coord w.r.t IRS in meters
;                  Default: for UWKA or NRC Convair not applied (0) 
;                           for NCAR C130 applied(1)
;                  Default antenna position can be seen in wcracvelm.pro
;                 This keyword is used with WRITESTR='cdv' and 'all' only
;
;       REVISION: input,string or strarr(2) netcdf file revision
;                 revision[0] - update Revision_X attribute (default, depends 
;                               on what is processed); If the string starts with
;                               $ the last Revision_X attribute is overwritten
;                               otherwise a new Revision_X is created. If only
;                               '$' the last Revision_X will be overwritten
;                               with the default string.
;                 revision[1] - update RevisionStatus attribute(default is 'OK')
;                 If input a string, update Revision_X attribute only;
;                 If empty string(s) '' - do not update the attribute(s)
;
;       SILENT:   when set supress routine informational messages
;
;       TIMESYNC: input,float, WCR time correction to sync with A/C IRS(INS) 
;                   data, in seconds  (default Timeoffsetsync from Ncfile)
;                 input, fltarr(n), different values for every ncfile 
;                 This keyword is used with WRITESTR='all' and 'cdv' only
;
;       UNFOLD:   input, Doppler velocity unfolding procedure (see notes)
;                 0          - no unfolding is performed (default)
;                 1          - use AC measured winds
;                 fltarr(3)  - [u,v,w], use the given wind vector
;                              u,v,w are allowed the following values:
;                              float: actual wind value to be used in m/s
;                                888: use AC measured values for all 
;                                     profiles of this component
;                                777: use mean of AC measured values from 
;                                     all profiles of this component
;                 ATTN: Note yet modified to work for multiple files/prof_ranges
;
;       UTCCORR:  if defined gives UTC time correction of the wcr time
;                   Default is UTCcorrection from Ncfile.
;                   This keyword is used with WRITESTR='all' and 'cdv' only
;
;       HELP:     shows this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;
;      See the side effects described in wcrwrite1nc and wcrwrite2nc
;
; NOTES:
;
; EXAMPLES:
;
;       1) Example using CLEX-9 experiment data (single antenna config, N2UW)
;
;       ncfile=['WCR.SPP.CLEX01.20011101.145436_160822.nc',$
;               'WCR.SPP.CLEX01.20011101.160822_164239.nc']
;       wcrfile='/sova4/clex01/nov01'
;       writestr='z'
;       revision='Updated reflectivity'
;       wcrupdatenc,ncfile,wcrdir,writestr,cal=[49.3,47.8,16.0],utccorr=-13.,$
;                   revision=revision
;
;       2) Example using DYCOMS II experiment data (2 down antennas, N130AR)
;
;       ncfile=['WCR.PP4.DYCOMS01.20010727.213217_215900.nc', $
;               'WCR.PP4.DYCOMS01.20010727.215900_224630.nc']
;       wcrdir='/sova4/dycoms01/jul27'
;       writestr='cdv'
;       acpath='/net/fox/fox1/C130/hi_rate/RF09.20010727.181636_034427.PNI.nc'
;
;       wcrupdatenc,ncfile,wcrdir,writestr,acpath=acpath,timesync=-0.125, $
;                   rev='$Updated Corrected Doppler'
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, March 2002
;       SH, 04/02 - added big_endian keyword
;       SH, 05/03 - renamed acdata to acpath, modified acnames, added unfold 
;                   keyword, some other changes (e.g., comply with the new
;                   acnames - no AC id)
;       SH, 09/03 - modified values for UNFOLD
;       SH, 11/03 - modified cal update to work with multi-antenna configs.;
;                   add NRC-CV580 aircraft
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrupdatenc,ncfile,wcrdir,writestr,acpath=acpath,acnames=acnames, $
                acspeed=acspeed,nswin=nswin,cal=cal,utccorr=utccorr, $
                timesync=timesync,unfold=unfold,gpsc=gpsc,rmcor=rmcor,$
                revision=revision,big_endian=bgnd,silent=silent,help=help

  @commonwcrinclude

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrupdatenc'
    return
  endif

; Initilize

  nncf=n_elements(ncfile)
  writestri=['all','z','dv','zdv','cdv']
  cv580speed=115.
  cv580name='NRC-CV580'
  c130speed=115.
  c130name='N130AR'
  kaspeed=90.
  kaname='N2UW'
  nswinsz=[99,99]   ; default noise smoothing window size for h- and v-ch

; Check inputs

  if n_params() lt 1 then begin
    message,"Usage: wcrupdatenc,ncfile,wcrdir,writestr,$",/info
    print,"                   acpath=acpath,acnames=acnames,acspeed=acspeed,$"
    print,"                   nswin=nswin,cal=cal,utccorr=utccorr,gpsc=gpsc,$"
    print,"                   timesync=timesync,unfold=unfold,big_endian=bgnd,$"
    print,"                   rmcor=rmcor,revision=revision,silent=silent,help=help"
    return
  endif

  ; Check depending on the number of input parameters and reassign accordingly

  if n_params() eq 1 then begin
    wcrd='./'
    writes='all'
  endif else if n_params() eq 2 then begin
    ind1=where(strlowcase(wcrdir) eq writestri)
    if ind1[0] eq -1 then begin
      wcrd=wcrdir
      writes='all'
    endif else begin
      wcrd='./'
      writes=strlowcase(wcrdir)
    endelse
  endif else begin
    wcrd=wcrdir
    writes=strlowcase(writestr)
  endelse

  ; Check the validity of writestr

  ind1=where(writes eq writestri)
  if ind1[0] eq -1 then begin
    message,'Invalid input for writestr'
  endif

  ; Check if the files/directories exist

  for k=0,nncf-1 do begin
    spawn,'dir '+ncfile[k]+' 2>&1',/sh,junk1  
    if strpos(junk1,'such file or directory') ne -1 then begin 
      print,''
      message,ncfile[k]+' is not found:',/info
      print,junk1
      return
    endif
  endfor

  if strpos(wcrd,'/',/reverse_search)+1 ne strlen(wcrd) then wcrd=wcrd+'/'
  if wcrd ne './' then begin
    spawn,'dir '+wcrd+' 2>&1',/sh,junk1
    if (strpos(junk1,'such file or directory'))[0] ne -1 then begin
      print,''
      message,wcrd+' is not found:',/info
      print,junk1
      return
    endif
  endif

  if n_elements(acpath) eq 0 then acpath='./' $
  else begin
    spawn,'dir '+acpath+' 2>&1',/sh,junk1
    if (strpos(junk1,'such file or directory'))[0] ne -1 then begin
      print,''
      message,acpath+' is not found:',/info
      print,junk1
      return
    endif
  endelse

  ; Check if nswin is consistent 

  if writes eq 'all' or writes eq 'z' or writes eq 'zdv' then begin
    junk1=n_elements(nswin)
    junk2=size(nswin,/n_dim)

    if nncf eq 1 then begin
      if junk2 gt 1 then begin
        message,'Invalid input for nswin',/info
        return
      endif
      if junk1 gt 0 then begin 
        if junk1 eq 1 then begin
          nsw=[nswin,nswin]
          ppmnoav=0
        endif else if junk1 eq 2 then begin
          nsw=[nswin[0],nswin[0]]
          ppmnoav=nswin[1]
        endif else begin
          nsw=[nswin[0],nswin[1]]
          ppmnoav=nswin[2]
        endelse
      endif
    endif else begin    
      if junk1 gt 0 then begin
        if junk2 gt 1 then begin
          if n_elements(nswin[*,0]) ne 3 and n_elements(nswin[0,*]) ne nncf $
          then begin
            message,'Invalid input for nswin',/info
            return
          endif
          nsw=nswin[0:1,*]
          ppmnoav=nswin[2,*]
        endif else begin
          if junk1 eq 1 then begin
            nsw=[nswin,nswin]
            ppmnoav=0
          endif else if junk1 eq 2 then begin
            nsw=[nswin[0],nswin[0]]
            ppmnoav=nswin[1]
          endif else begin
            nsw=[nswin[0],nswin[1]]
            ppmnoav=nswin[2]
          endelse
          nsw=replicarr(nsw,nncf)
          ppmnoav=replicate(ppmnoav,nncf)
        endelse
      endif
    endelse
  endif

; Check acnames consistency

  if not(n_elements(acnames) eq 0 or n_elements(acnames) eq 10 or $
     n_elements(acnames) eq 13 or n_elements(acnames) eq 15) then $
     message,'Incorrect input for acnames'

;------------------------------------------------------------------------------
; Cycle for all netcdf files
;------------------------------------------------------------------------------
 
  for k=0L,nncf-1 do begin

  ; Open ncfile in write mode and read what is needed

    ncid=ncdf_open(ncfile[k],/write)

    ncdf_attget,ncid,'Aircraft',aircraft,/global & aircraft=string(aircraft)
    ncdf_attget,ncid,'WCRprofav',   profav,   /global
    ncdf_attget,ncid,'WCRrangeav',  rangeav,  /global
    ncdf_attget,ncid,'WCRprofrange',profrange,/global
    ncdf_attget,ncid,'WCRdaqfile',  daqfile,  /global & daqfile=string(daqfile)

    write1s=writes
    if writes eq 'cdv' or writes eq 'dv' then pponly=1       

    ; Check/update UTCcorrection and modify write1s accordingly

    if n_elements(utccorr) eq 0 then begin
      if writes eq 'all' then write1s='z' $
      else if writes eq 'cdv' then write1s='' 
      ncdf_attget,ncid,'UTCcorrection',utccor,  /global
    endif else begin
      ncdf_attget,ncid,'UTCcorrection',utccor,  /global
      if utccor eq utccorr then begin
        if writes eq 'all' then write1s='z' $
        else if writes eq 'cdv' then write1s='' 
        if keyword(silent) eq 0 then $
           message,'No need to update time/coord. variables.',/info
      endif else begin
        if writes eq 'all' then begin
          write1s='tz'
          utccor=utccorr
          ncdf_update,ncid,'UTCcorrection',utccor,'global',silent=silent
        endif else if writes eq 'cdv' then begin
          write1s='time'
          utccor=utccorr
          ncdf_update,ncid,'UTCcorrection',utccor,'global',silent=silent
        endif else begin
          if keyword(silent) eq 0 then $
            message,'UTCcorrection is irrelevant for this writestr. Ignored.',/info
        endelse
      endelse
    endelse   

    ; Check acspeed regardless if it is needed or not (not a big deal)

    if n_elements(acspeed) eq 0 then begin
      if strupcase(aircraft) eq c130name then acspeed=c130speed $
      else if strupcase(aircraft) eq cv580name then acspeed=cv580speed $
      else if strupcase(aircraft) eq kaname then acspeed=kaspeed
    endif

    ; Check CAL and NSW if reflectivity is to be updated

    if writes eq 'all' or writes eq 'z' or writes eq 'zdv' then begin

    ; Check MISVAL

      if n_elements(misval) eq 0 then begin
        misv=ncdf_attload(ncid,'hh','missing_value')    
        if misv eq '-1' then misv=ncdf_attload(ncid,'vv','missing_value')
      endif else misv=misval

    ; Update calibration coefficients
      
      junk1=['WCRcalHH1','WCRcalVV1','WCRcalVV2','WCRcalVV4','WCRcalHH4']
      if n_elements(cal) gt 0 then begin
        
        ; update isolation if requested
        if n_elements(cal) gt 5 then if cal[5] ne -999. then begin
          ncdf_update,'WCRisoHV',float(cal[5]),'global'
	  if n_elements(cal) gt 6 then if cal[6] ne -999. then $
	    ncdf_update,'WCRisoVH',float(cal[6]),'global'
          junk2=float(cal[0:4])
        endif else junk2=float(cal)

        ; update the selected cal constants
        ind1=where(junk2 ne -999.)
        if ind1[0] ne -1 then $
          ncdf_update,junk1[ind],junk2[ind1],'global'
      endif

    ; Determine nswin and ppmnoav 

      if n_elements(nswin) eq 0 then begin
        ppmno=0

        ; Process hmnoise if any

        junk1=ncdf_attload(ncid,'hmnoise','allatts')
        if junk1[0] ne '-1' then begin

          ind1=where(junk1[0,*] eq 'smooth')
          nswn=junk1[1,ind1]

          ; find if noise channel or range gate

          ind1=where(junk1[0,*] eq 'pulse')
          if ind1[0] ne -1 then begin
            if string(junk1[0,ind1]) eq 'N/A' then begin
              ncdf_varget1,ncid,'hmnoise',junk2
              nswn=500+abs(junk2)
            endif else begin
              junk2=junk1[1,ind1]
              if junk2 lt 10 then ppmno=junk2
            endelse            
          endif else begin
            ind1=where(junk1[0,*] eq 'rangegate')
            nswn=nswn*1000L+junk1[1,ind1]
          endelse

          ; find co- or cross-channel

          ind1=where(junk1[0,*] eq 'receiver')
          if strlowcase(junk1[1,ind1]) eq 'v' then nswn=-nswn  
        endif 

        ; Process vmnoise if any

        junk1=ncdf_attload(ncid,'vmnoise','allatts')
        if junk1[0] ne '-1' then begin

          ind1=where(junk1[0,*] eq 'smooth')
          nsw1=junk1[1,ind1]

          ; find if noise channel or range gate

          ind1=where(junk1[0,*] eq 'pulse')
          if ind1[0] ne -1 then begin
            if string(junk1[0,ind1]) eq 'N/A' then begin
              ncdf_varget1,ncid,'hmnoise',junk2
              nsw1=500+abs(junk2)
            endif else begin
              junk2=junk1[1,ind1]
              if junk2 lt 10 then ppmnoav=junk2
            endelse            
          endif else begin
            ind1=where(junk1[0,*] eq 'rangegate')
            nsw1=nsw1*1000L+junk1[1,ind1]
          endelse

          ; find co- or cross-channel

          ind1=where(junk1[0,*] eq 'receiver')
          if strlowcase(junk1[1,ind1]) eq 'h' then nsw1=-nsw1  
        endif

        ; Form the nswn

        if n_elements(nswn)+n_elements(nsw1) eq 2 then nswn=[nswn,nsw1] $
        else if n_elements(nsw1) eq 1 then nswn=nsw1

      endif else begin
        nswn=nsw[*,k]
        ppmno=ppmnoav[k]
      endelse

      ppmno=ppmno+(ppmno gt 0)
    endif

    ; Check TIMESYNC, GPSC and RMCOR if corrected Doppler is to be updated

    if writes eq 'all' or writes eq 'cdv' then begin
      write2s='cdv' 

      acf=ncdf_attload(ncid,'AircraftDataSource',/global)
      if strtrim(acf) eq '' then begin
        junk2=strpos(acpath,'/',/reverse_search)
        if junk2 eq -1 then junk2=acpath else junk2=strmid(acpath,junk2+1)
        if strlen(junk2) eq 0 then begin
           ncdf_close,ncid
           message,'A/C data file is not defined.',/info
           return
        endif
        acf=acpath

        ; Update AircraftDataSource and AircraftDateProcessed

        ncdf_update,ncid,'AircraftDataSource',junk2,'global'
        junk1=ncdf_attload(acid,'allatts',acpath,/global)
        ind1=where(junk1[0,*] eq 'DateProcessed')
        if ind1[0] eq -1 then begin
          if keyword_set(silent) eq 0 then begin
            message,"Global attribute DateProcessed missing for "+junk2,/info
            print,'              Updating AircraftDateProcessed ignored.'
          endif
        endif else $
          ncdf_update,ncid,'AircraftDateProcessed',junk1[1,ind1],'global'
        ncdf_close,acid
      endif else begin
        ind1=strpos(acpath,'/',/reverse_search)
        if ind1 ne -1 then begin
          junk1=strpos(acpath,acf)
          if junk1 ne -1 then acf=strmid(acpath,0,junk1)+acf $
          else if ind1+1 eq strlen(acpath) then acf=acpath+acf $
          else acf=acpath+'/'+acf
        endif else $
          if strpos(acpath,acf) eq -1 then acf=acpath+'/'+acf 
      endelse
      spawn,'dir '+acf+' 2>&1',/sh,junk1  
      if strpos(junk1[0],'such file or directory') ne -1 then begin
        print,''
        ncdf_close,ncid
        message,+acf+' is not found:',/info
        print,junk1
        return
      endif

      if n_elements(timesync) eq 0 then $
        ncdf_attget,ncid,'Timeoffsetsync',timesyn,/global $
      else if n_elements(timesync) eq 1 then timesyn=timesync $
      else timesyn=timesync[k]
      timesyn=timesyn[0]

      junk1=ncdf_attload(ncid,'dv','status')

      if n_elements(gpsc) eq 0 then begin
        ind1=strpos(junk1,'GPS')
        if ind1[0] eq -1 then begin
          if keyword_set(silent) eq 0 then $
            message,'GPS corr. N/A in '+ncfile[k]+'. Use default value.',/info
        endif else begin
          junk1=strmid(strmid(junk1,ind1),strpos(strmid(junk1,ind1),'=')+1,1)
          if strupcase(junk1) eq 'N' then gpsc=1
        endelse
      endif

      if n_elements(rmcor) eq 0 then begin
        ind1=strpos(junk1,'moment')
        if ind1[0] eq -1 then begin
          if keyword_set(silent) eq 0 then $
            message,'Radar moment corr. N/A in '+ncfile[k]+ $
                    '. Use default value.',/info
        endif else begin
          junk1=strmid(strmid(junk1,ind1),strpos(strmid(junk1,ind1),'=')+1,1)
          if strupcase(junk1) eq 'Y' then rmcor=1
        endelse
      endif

    endif else write2s=''

    ; Open WCR raw file, read the data and close the file

    if keyword_set(silent) eq 0 then begin
      print,'=================================================================='
      print,'Processing netcdf file: '+ncfile[k]
      print,'=================================================================='
      print,''
      message,'******** Reading WCR raw data ...',/info
    endif
    openwcrf,wcrd+daqfile,timecor=utccor,acspeed=acspeed,big_endian=bgnd
    readwcrdata,profrange[0],profrange[1],timemode=3,nocross=nocross, $
                ppmnoav=ppmno,pponly=pponly,/noleak,big_endian=bgnd
    closewcrf

    ; Update reflectivity, uncorrected Doppler

    if write1s ne '' then begin
      if keyword_set(silent) eq 0 then $
        message,'******** Calculate/Update reflectivity and/or uncorrected velocity ...',/info
      wcrwrite1nc,ncid,writestr=write1s,nswin=nswn,misval=misv, $
                  revision=revision,silent=silent
    endif

    ; Update corrected Doppler

    if write2s ne '' then begin
      if keyword_set(silent) eq 0 then $
        message,'******** Calculate/Update corrected Doppler velocity ...',/info
      wcrwrite2nc,acf,ncid,acnames=acnames,gpsc=gpsc,rmcor=rmcor,unfold=unfold,$ 
                  timesync=timesyn,revision=revision,silent=silent
    endif
    
    ; Ncfile is done. Close the file.

    ncdf_close,ncid

  endfor
;------------------------------------------------------------------------------
; End of cycle. Done
;------------------------------------------------------------------------------

 if keyword_set(silent) eq 0 then message,'wcrupdatenc is finished.',/info

end
