;$Id: wcrvcor.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRVCOR
;
; PURPOSE:
;
;       Linear correction of V receiver data (lookup table adjustment).  
;       Data inlcude vvnoise, vvtx, vvlk, vv, vh, ppnoise, pptx, pp.
;
; CATEGORY:
;
;       wcrtools
;
; CALLING SEQUENCE:
;
;       wcrvcor, fitprms,what=what,silent=silent
; INPUT:
;
;       fitprms:  [slope,intercept] - optional fltarr(2) of the correction fit
;                 default: [0.95,0.]
;
; KEYWORD PARAMETERS:
;
;       what   :  Use this keyword to specify which data to be corrected
;                 Input: string array/scalar; Strings: 'vv','vh','vvtx','vvlk',
;                 'vvnoise','pp','pptx', 'ppnoise'
;                 Example: what=['vv','vvnoise']
;                 Default: ['vvnoise','vv','ppnoise','pp']
;       silent :  Use this keyword to supress some info messages.
;       help:     Show this text
;
; COMMON BLOCKS:
;
;       common wcrparams
;       common wcrdata
;
; PROCEDURE:
;
;       Uses linear fit to correct lookup table calibration.
;       Intercept can be used to account for receivers' gain difference,
;       and in this case same cal constant should be used for H and V
;       After May 1999 all lookup tables were replaced and generally
;       no major correction is needed.
;
;       vcorrection flags the status of V-ch correction: 0/1 - no/yes
;       vcorrection=[Fvvnoise,Fvv,Fvh,Fvvtx,Fvvlk,Fppnoise,Fpp,Fpptx]
;
; EXAMPLE:
;
;       For PPMAG mode correct V-ch using the default correction:
;
;       wcrvcor
;
; SIDE EFFECTS:
;
;       All necessary data must be loaded into the common block first.
;       Co-pol pulse pairs are corrected only (e.g., 'VV' and not 'VH') 
;-
; MODIFICATION HISTORY:
;       Written by: Samuel Haimov, January, 2000
;       S.H., Jan 00 -- added help keyword
;       S.H., Feb 01 -- changed routine name from vxivcor to wcrvcor
;       S.H., Oct 01  -- replaced atan(complex) with atan(imaginary(complex),
;                                                         float(complex))
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrvcor, fitprms, what=what, silent=silent, help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrvcor'
  return
endif

if alg eq fftalg then return

if n_params() eq 0 then begin slope=0. & intercept=0. & end else begin
   slope=fitprms(0)
   if n_elements(fitprms) eq 1 then intercept=0. else intercept=fitprms(1)
endelse

if slope eq 0. and intercept eq 0 then return

if n_elements(what) eq 0 then $
   if alg eq sppAlg then begin 
      if txsequence(0) eq polv then $
;        what=['vvnoise','vv','vvtx','ppnoise','pptx','pp']
         what=['vvnoise','vv','ppnoise','pp']
      if txsequence(0) eq polh then what=['vvnoise','vh','vvlk']
   endif else if alg eq pp4alg or alg eq pp6alg then begin
;    what=['vvnoise','vv','vh','vvtx','vvlk','ppnoise', 'pp','pptx']
     what=['vvnoise','vv','vh','ppnoise', 'pp']
   endif

ppvind=where(ppstrpol eq 'VV')

if not keyword_set(silent) then begin
   print,'V channel data correction in progress ...'
   print,' '
endif

if n_elements(vvnoise) gt 1 and (where(what eq 'vvnoise'))(0) ne -1 $
and vcorrection(0) eq 0 then begin
   vvnoise=10.^(0.1*(10.*alog10(vvnoise)-intercept)/slope)
   vcorrection(0)=1
   if not keyword_set(silent) then print,'VVNOISE corrected'
endif
if (n_elements(hh) gt 1) and (where(what eq 'vv'))(0) ne -1 $
and vcorrection(1) eq 0 then begin
   vv =10.^(0.1*(10.*alog10(vv) -intercept)/slope)
   vcorrection(1)=1
   if not keyword_set(silent) then print,'VV corrected'
endif

if (n_elements(vh) gt 1) and (where(what eq 'vh'))(0) ne -1 $          
and vcorrection(2) eq 0 then begin
   vh =10.^(0.1*(10.*alog10(vh) -intercept)/slope)
   vcorrection(2)=1
   if not keyword_set(silent) then print,'VH corrected'
endif

if (n_elements(vvtx) gt 1) and (where(what eq 'vvtx'))(0) ne -1 $
and vcorrection(3) eq 0 then begin
   vvtx=10.^(0.1*(10.*alog10(vvtx)-intercept)/slope)
   vcorrection(3)=1
   if not keyword_set(silent) then print,'VVTx corrected'
endif

if (n_elements(vvlk) gt 1) and (where(what eq 'vvlk'))(0) ne -1 $
and vcorrection(4) eq 0 then begin
   vvlk=10.^(0.1*(10.*alog10(vvlk)-intercept)/slope)
   vcorrection(4)=1
   if not keyword_set(silent) then print,'VVlk corrected'
endif

if (n_elements(ppnoise) gt 1) and (where(what eq 'ppnoise'))(0) ne -1 $
and (ppvind[0] ne -1) and (vcorrection(5) eq 0) then  $
   if alg eq sppalg then begin
     phi=atan(imaginary(ppnoise),float(ppnoise))
     mag=abs(ppnoise)
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     ppnoise=mag*exp(complex(0.,phi))
     vcorrection(5)=1
     if not keyword_set(silent) then print,'ppnoise corrected'
   endif else begin
     phi=atan(imaginary(ppnoise[pphind,*]),float(ppnoise[pphind,*]))
     mag=abs(ppnoise[pphind,*])
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     ppnoise[pphind,*]=mag*exp(complex(0.,phi))
     vcorrection(5)=1
     if not keyword_set(silent) then print,'ppnoise corrected'
   endelse

if (n_elements(pp) gt 1) and (where(what eq 'pp'))(0) ne -1 $
and (ppvind[0] ne -1) and (vcorrection(6) eq 0) then $
   if alg eq sppalg then begin
      phi=atan(imaginary(pp),float(pp))
      mag=abs(pp)
      mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
      pp=mag*exp(complex(0.,phi))
      vcorrection(6)=1
      if not keyword_set(silent) then print,'co-pol pp corrected'
   endif else begin
     phi=atan(imaginary(pp[*,pphind,*]),float(pp[*,pphind,*]))
     mag=abs(ppnoise[*,pphind,*])
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     ppnoise[*,pphind,*]=mag*exp(complex(0.,phi))
     vcorrection(5)=1
     if not keyword_set(silent) then print,'pp corrected'
   endelse

if (n_elements(pptx) gt 1) and (where(what eq 'pptx'))(0) ne -1 $
and (ppvind[0] ne -1) and (vcorrection(7) eq 0) then $
   if alg eq sppalg then begin
     phi=atan(imaginary(pptx),float(pptx))
     mag=abs(pptx)
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     pptx=mag*exp(complex(0.,phi))
     vcorrection(7)=1
     if not keyword_set(silent) then print,'ppTx corrected'
   endif else begin
     phi=atan(imaginary(pptx[pphind,*]),float(pptx[pphind,*]))
     mag=abs(pptx[pphind,*])
     mag=10.^(0.1*(10.*alog10(mag)-intercept)/slope)
     pptx[pphind,*]=mag*exp(complex(0.,phi))
     vcorrection(5)=1
     if not keyword_set(silent) then print,'ppTx corrected'
   endelse

if not keyword_set(silent) then begin 
   print,'Correction done.'
;  if intercept eq 0. then print,"H/V receivers' gain difference not corrected."
   print,' '
;  bell,2
endif

end  ; wcrvcor


 
