;$Id: wcrvps.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRVPS
;
; PURPOSE:
;       Creates a Corrected Doppler velocity PS file with maximum 3 plots
;       per page
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       WCRVPS, dvel           ; when wcr common blocks are used
;       WCRSPS, dvel,r,tsec    ; for general use
;
; INPUTS:
;    Dvel:  fltarr(rangeGates,profs), velocity in m/s.
;                 If V is not given a widget to pick a WCR file will open.
;    R:     fltarr(rangeGates), range;
;                 mainly for non WCR files (see ASPECT keyword)
;    Tsec:  fltarr(profs), time in sec.
;                 mainly for non WCR files (see ASPECT keyword)
;
; KEYWORD PARAMETERS:
;    ASP:         float, Range/Profile aspect ratio in meters.
;                 When exist won't use wcr common blocks and requires r and tsec
;                 This keyword is intended for non WCR files.  If it is used
;                 for wcr files negative values for asp will print wcr file
;                 info at the bottom of each page. 0 input will disable scaling.
;                 When negative: abs(asp) is used for aspect.
;    COLORTB:     color table number (default is 0, black & white)
;                 Negative color numbers reverse the color tables.
;                 Use -99 for reverse black & white.
;    DIR:         string, when set, the directory for ps file(s) to be stored
;                 default is ps dir under current dir.
;    FNAME:       input, string array, [file,title,xtitle,ytitle,bar_title]
;                 fname[0] - PS output file name; fname is also used for
;                 labeling the plots. Default PS file name uses wcrfile.
;                 Default labels are : 'DV','UTC','Range [m]','m/s'.
;                 If fname[0]='' then it assumes that wcrfile exists and uses
;                 it to form the output ps file name
;    HELP:        show this text
;    IMGORDER:    input, integer, antenna pointing position for the plotted
;                 images ; define the range axis order of the images
;                   0 -  up/side pointing antenna (bottom-to-top image, default)
;                   1 -  downlooking antenna (top-to-bottom image)
;    LANDSCAPE:   create PS file in landscape (default is portrait)
;    MAXLIM:      float, maximum velocity plotted (default  50 m/s)
;    MINGATES:    integer, minimum number of range gates to be plotted
;                 If mingates is defined and greater than the actual number of
;                 range gates the image is plotted using mingates (default, 70)
;    MINLIM:      float, minimum velocity plotted (default -50 m/s)
;    MINMAX:      when set (minmax=1), min and max for every subimage are used
;                 minmax < 0., minmax is used as a fixed min, max is calculated
;                              for every subimage
;                 minmax > 0. (but ne to 1), minmax is used as a fix max,
;                              min is calculated for every subimage
;                 minmax=0,(default, minmax not set), min and max are calculated
;                           for the whole image and used as fixed
;    TLABEL:      string, adds title to the param/stat tables
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       wcr files with small number of range gates (< 50) may cause the pixel
;       size to be too big (because of aspect ratio applied to equally scale
;       the image in range/time axes).  To avoid this problem use mingates
;       keyword set to number of range gates higher than the actual number.
;
; EXAMPLES:
;
;       ; Read raw WCR wcr file and KingAir netcdf 25 Hz data file and
;       ; create a ps file (Wpp00-11-19-21-01-23.ps) with corr. Doppler vel.
;       ; Noise pixels are not thresholded.
;
;       minmax  =  1  ; use local min/max values for every plot
;       minlim  = -5  ; minimum plotted velocity in m/s
;       maxlim  =  5  ; maximum plotted velocity in m/s
;
;       tlabel  = 'Example'
;       wcrdir  = '/net/orel/orel3/wyice00/nov19/'
;       kafile  = '/net/bobcat/disk1/kingair_data/20001119.c25.nc'
;
;       ppmnoav = 2   ; load 1st pulse noise readings only (for PPMAG files)
;       wcrtime = 1   ; force loading WCR raw UNIX time
;
;       openwcrf,wcrdir+'Wpp00-11-19-21-01-23'    ; PPMAG4 file from WYICE00
;       readwcrdata,rawtime=wcrtime,ppmnoav=ppmnoav
;       DopVel = wcrdopvelm(kafile,wcrtime)
;       DopVel = reform(DopVel[*,*,0])            ; use vel from 1st pulse pair
;
;       wcrvps, DopVel,minmax=minmax,minlim=minlim,maxlim=maxlim,tlabel=tlabel
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, February 2001
;       SH, Apr 02 - replaced antpos keyword with imgorder
;       SH, May 02 - fixed a problem with crossing midnight for timeaxis
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrvps, Z,r,tsec,fname=fname,minmax=minmax,minlim=minlim,maxlim=maxlim, $
            asp=asp,dir=dir,tlabel=tlabel,landscape=land,colortb=colortb, $
            imgorder=imgorder,mingates=mingates,help=help

; WCR data common blocks

@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrvps'
  return
endif

if (n_params() eq 0) then begin
   message,'Usage: wcrvps,dvel,r,tsec',/info
   return
endif

; Define a few constants
  pad=3000       ; for short records pad zeros to preserve aspect ratio
  minsignal=-32. ; m/s, minimum valid signal (this can be changed if needed)
  maxsignal= 32. ; default for maxlim
  nosignal =-50. ; expand the velocity image if needed; default for minlim

  if n_elements(mingates) eq 0 then mingates=70

; Create postscript file directory if it does not exist. Use ./ps as default

  if n_elements(dir) eq 0 then dir='./ps/' $
  else if strmid(dir,strlen(dir)-1,1) ne '/' then dir=dir+'/'
  if dir ne '.' and dir ne './' then spawn,'mkdir '+dir+'>& /dev/null'

; Check if this is a request for a wcr file

if (n_elements(aspect) eq 0) then begin

  tsec=time
  aspect=aspectratio
  if (n_elements(fname) eq 0) then fname=wcrfile
  printinfo=1

  gates=rangegates
  profs=numrec
  rr=radrng

endif else begin      ; non wcr file or not using wcr common blocks

  gates=(size(Z))[1]
  profs=(size(Z))[2]
  if asp lt 0 then printinfo=1 else printinfo=0
  aspect=abs(asp)

  if (n_elements(fname) eq 0) then begin
     print, 'Keyword FNAME is undefined !'
     return
  endif

  if n_params() lt 3 then begin
     print,'Range (r) and time (tsec) vectors are undefined!'
     return
  endif else rr=r

endelse

; Check if tsec is crossing midnight

  tsc=tsec
  junk=abs(max(tsec)-min(tsec))
  if junk gt 70000. then begin
    junk=where(tsec ge max(tsec))
    tsc[junk+1:*]=tsec[junk+1:*]+86400d0
  endif 

; Check if by any chance gates=profiles

  if (gates eq profs) then begin
     print,' Range Gates = Num Profiles'
     print,' Assume (rangeGates, Profiles) input image!'
   endif

; If the input image Z is [gates,profs] oriented transpose Zc.

   if (gates eq (size(Z))[1] ) then Zc=transpose(Z) else Zc=Z

; If minimum number of range gates for plotting is given accommodate them

  if mingates gt gates then begin
     rr=[rr,(findgen(mingates-gates)+1)*(rr[1]-rr[0])+rr[gates-1]]
     Zc=[[Zc],[fltarr(profs,mingates-gates)+nosignal]]
     gates=mingates
  endif

; Determine the color preference

  if (n_elements(colortb) eq 0) then begin
     color=0
     revcol=0
  endif else if (colortb eq -99) then begin
     revcol=1
     color=0
  endif else  begin
     if colortb lt 0 then revcol=1 else revcol=0
     color=abs(colortb)
  endelse

; Initialize

  ; Determine the order of the images

  if KEYWORD_SET(imgorder) then begin
     rr=reverse(rr)
     yorder=1
  endif else yorder=0

  ; Set PS device (clean system plot variables first)

  resetps2x
  setxpswin,99,3,psfile=dir+fname(0)+'.ps',land=land,nocolor=(color eq 0)

  ; Assign labels

  if n_elements(fname) gt 0 then begin
     k=n_elements(fname)
     if fname[0] eq '' then fname[0]=wcrfile
     if k lt 5 then btitle='m/s' else btitle=fname(4)
     if k lt 4 then ytitle='Range [m]' else ytitle=fname(3)
     if k lt 3 then xtitle='UTC' else xtitle=fname(2)
     if k lt 2 then title=':  DV' else title=fname(1)
  endif else begin
     title=':  DV'
     xtitle='UTC'
     ytitle='Range [m]'
     btitle='m/s'
  endelse

  title=fname[0]+title
  if KEYWORD_SET(land) then begin
     title=[title,btitle]
     bar=2
  endif else begin
     title=[title,'                                   '+btitle]
     bar=1
  endelse

  ; Assign min/max ranges for the plots and a few other local variables

  if n_elements(minlim) eq 0 then minlim= nosignal
  if n_elements(maxlim) eq 0 then maxlim= maxsignal

  if (n_elements(minmax) eq 0) then begin       ; fixed min/max for all plots
     minZ=min(Zc(where(Zc gt minsignal)))>minlim
     maxZ=max(Zc)<maxlim
  endif else if (n_elements(minmax) eq 2) then begin  ; Assign user's values
     minZ=minmax[0]>minlim
     maxZ=minmax[1]<maxlim
  endif else if (n_elements(minmax) eq 1) then $
     if (minmax[0] gt 0. and minmax[0] ne 1) then maxZ=minmax[0]<maxlim else $
     if (minmax[0] lt 0.) then minZ=minmax[0]>minlim

  gapval=2*max(Zc)*(revcol eq 0) + 2*min(Zc)*(revcol eq 1) ; gap filling value
  xpix=profs
  ypix=gates
  first=0
  last=(lasti=(xpix-1))
  page=1B
  isize=[0,-1]
  true=1B
  graph=0B
  charsize=1.4

  print,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
  print,fname(0),' processing ...'

; Loop over the segments of the image and create plots

   while true do begin

      ; Find min/max for the current segment if requested

      if (n_elements(minmax) eq 1) then begin
         if (minmax[0] lt 0. or minmax[0] eq 1) then $
            maxZ=max(Zc[first:last,*])<maxlim
         if (minmax[0] gt 0.) then begin
            temp=Zc[first:last,*]
            minZ=min(temp(where(temp gt minsignal)))>minlim
         endif
      endif

      print,''
      print,format='("Plotting: Page ",I2,", Graph: ",I2)',page,graph+1
      print,''

      ; Optimize printwcrstat for more than one page

      if page eq 1 then stattext=''

      if isize[0] eq 0 and isize[1] eq -1 then begin
      ;  Determine the right plotting size for the image
      ;  Done once per image, in the beginning

         aximage,Zc[first:last,*],bar=bar,asp=aspect,ch=charsize,isize=isize, $
                /silent
         while (isize[1] lt ypix) do begin
            aximage,temporary([Zc[first:last,*],fltarr(pad,gates)]),$
                    bar=bar, asp=aspect, chars=charsize, isize=isize ,/silent
            pad=pad+3000
         endwhile

      ;  Assign the size of the subimage that preserves the aspect ratio

         last=isize[0]-1
         print,''
         print,'Total number of profiles: ',profs
         print,'Plotted image: profiles= ',num2str(isize[0]), $
               ', range gates= ',num2str(isize[1])
         print,''

      ;  After we determined the size reset the device and start plotting

         resetps2x
         setxpswin,99,3,psfile=dir+fname[0]+'.ps',land=land,color=color, $
                   nocolor=(color eq 0)

         if (last gt lasti) then begin

         ;  Plot the short image or short subimage between two interrupts

            nextplot
            aximage,temporary([Zc[first:lasti,*], $
                    gapval+fltarr(last-lasti,gates)]), $
                    temporary([tsc[first:lasti],tsc[lasti]+ $
                    (tsc[lasti]-tsc[first])/(lasti-first)*  $
                    (findgen(last-lasti)+1)]),rr,asp=aspect,order=yorder, $
                    timeax=1,min=minZ,max=maxZ,title=title,xtitle=xtitle, $
                    ytitle=ytitle,chars=charsize,bar=bar,revc=revcol,/silent
            if printinfo then begin
               if n_elements(tlabel) gt 0 then $
                  xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
               printwcrprms,2,[0.08,-0.04,0.75]
               printwcrstat, 2,[0.52,-0.04,0.75]
            endif
            datestamp,/user,com='      Page '+num2str(fix(page))
            page=page+1B
            erase
            true=0B
         endif else begin

         ;  Plot first full size subimage

            aximage,Zc[first:last,*],tsc[first:last],rr,bar=bar, $
                    asp=aspect,timeax=1,min=minZ,max=maxZ,title=title,$
                    xtitle=xtitle,ytitle=ytitle,chars=charsize,revc=revcol,$
                    order=yorder,/silent
         endelse

      endif else $
         if isize[0] le last-first then $

         ;  Plot the current full size subimage

            aximage,Zc[first:last,*],tsc[first:last],rr,bar=bar, $
                    asp=aspect,timeax=1,min=minZ,max=maxZ,title=title,$
                    xtitle=xtitle,ytitle=ytitle,chars=charsize,revc=revcol,$
                    order=yorder,/silent        $
         else $

         ;  Plot last subimage when it is shorter than full subimage size

            aximage,temporary([Zc[first:last,*], $
                    gapval+fltarr(isize[0]-last+first,gates)]), $
                    temporary([tsc[first:last],tsc[last]+(tsc[last]- $
                    tsc[first])/(last-first)*(findgen(isize[0]-last+first)+1) $
                    ]),rr,bar=bar,asp=aspect,revc=revcol,timeax=1, $
                    min=minZ,max=maxZ,title=title,xtitle=xtitle,ytitle=ytitle, $
                    chars=charsize,order=yorder,/silent

      ;  Make necessary assignments before next graph and/or page

      if true then begin
         if ((graph mod 3) lt 2) then begin
            graph=graph+1B
            first=last-10                     ; overlap 10 profiles
            last=(first+isize[0]) < min([lasti, xpix-1])

         ; If only 50 or less profs left do not bother to plot the rest of
         ; the image.  Print useful info.

            if (first+60 ge xpix-1) or (first+60 gt lasti) then begin
               true=0B
               if printinfo then begin
                  if n_elements(tlabel) gt 0 then $
                     xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
                  printwcrprms,2,[0.08,-0.04,0.75]
                  printwcrstat, 2,[0.52,-0.04,0.75],stattext=stattext
               endif
               datestamp,/user,com='      Page '+num2str(fix(page))
               page=page+1B
               erase
            endif
         endif else begin                             ; end of page (3 graphs)
            if printinfo then begin
               if n_elements(tlabel) gt 0 then $
                  xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
               printwcrprms,2,[0.08,-0.04,0.75]
               printwcrstat, 2,[0.52,-0.04,0.75],stattext=stattext
            endif
            datestamp,/user,com='      Page '+num2str(fix(page))
            page=page+1B
            graph=graph+1
            erase
            first=last-10                     ; overlap 10 profiles
            last=(first+isize[0]) < min([lasti, xpix-1])
            if (first+20 ge xpix-1) or (first+20 gt lasti) then true=0B
         endelse
      endif

   endwhile

   !p.multi(0)=0              ; initialize multi plot @ the top of the page

   resetps2x

end




