;$Id: wcrvpsm.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRVPSM
;
; PURPOSE:
;       Create Corrected Doppler Velocity PS files from one day flight data
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       WCRVPSM, dir, acfile
;
; INPUTS:
;       Dir:      string, wcr data file directory; undefined: use pickfile
;
; KEYWORD PARAMETERS:
;     ACFILE:     string, Aircraft (A/C) data file path & name (ins data);
;                 (undefined: use pickfile)
;     ACONFIG:    input, integer or intarr; antenna configuration
;                 0 - single side/up looking antenna; default
;                 1 - two downlooking antennas (H-ch: down-fore, V-ch: down)
;                 2 - two sidelooking antennas (V-ch: side-fore, H-ch: side/up)
;                 3 - down and side/up antennas (V-ch: down, H-ch: side/up)
;                 4 - two downlooking antennas (V-ch: down, H-ch: down-aft,
;                     this is for C130 DYCOMS01 experiment before H/V channels 
;                     were swapped in march 2002)
;                 5 - down and side antennas for NRC-CV580 WCR installation
;
;                 Note: dual-antenna configs have no polarimetric capability
;
;     COLORTB:    color table number (default is -99, reversed black & white)
;                 Negative color numbers reverse the color tables.
;                 Use -99 for reverse black & white.
;     FILEID:     file identification string. Common substring from the data
;                 file name starting from 2nd char (default 'pp0').
;     HELP:       Show this text
;     BIG_ENDIAN: when set, assumes that the file data are in big endian order
;                 The default is LITTLE_ENDIAN.
;     MINGATES:   integer, minimum number of range gates to be plotted
;                 If mingates is defined and greater than the actual number of
;                 range gates (rangegates) the image is plotted using mingates
;                 (default, rangegates)
;     MINLIM:     float, minimum reflectivity plotted (default -50 m/s)
;     MAXLIM:     float, maximum reflectivity plotted (default  50 m/s)
;     PRINTER:    'lj4', 'lj4_gv', 'qms'
;                 undefined: qms
;                 '': suppress printing
;     PROC:       byte, define what to plot (for dual-pol modes only)
;                       0 - produce plots from H-ch (default)
;                       1 - produce plots from V-ch
;     PSDIR:      string, directory for postscript file(s)
;                 Default save ps files in subdir 'ps/', which is created
;                 if non-existant.  When dir='select', ps directory is always
;                 sub directory of the home directory, otherwise it is sub
;                 of dir
;     THRESH:     negative float, returned power in dBm to be used for velocity
;                 thresholding (default is 3 stdev above min noise)
;                 positive float, n; n*stdev above min noise 
;     TIMECOR:    if defined gives time correction of the wcr time
;                 After June 1998, IRIG card is used to set the time.
;                 Normal use of the card is to get the time from a master
;                 IRIG card with GPS time.  Thus the default value is
;                 the current leap seconds.
;     TIMEHACK:   float, time adjustment to wcr time to precisely sync wcr with
;                 aircraft(A/C) ins variables (default is 0). If defined it is
;                 added to TIMECOR.  DO NOT USE TIMEHACK if TIMECOR includes 
;                 the value of the non-zero time hack. 
;     TLABEL:     string, adds title to the param/stat tables It always add
;                 info about the timehack ('Corrected Doppler Velocity (timehack: ....)')
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;
; EXAMPLES:  See wcrproc_s.pro
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, February 2001
;       S.H., Apr 02 -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN, changed
;                       aconfig; modified TLABEL
;       S.H., Nov 03 -- added NRC-CV580 to aconfig
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

; Function used by wcrvpsm routine to find thresholded data indices

function threshind, proc,noisefac,thresh

on_error, 2

@commonwcrinclude

if n_elements(thresh) eq 0 then begin
   if proc then begin
      ind=where(vv le (min(vvnoise)+noisefac*std(vvnoise)))
   endif else begin
      ind=where(hh le (min(hhnoise)+noisefac*std(hhnoise)))
   endelse
endif else begin
   if thresh lt 0 then begin
      threshl=10^(0.1*thresh)
      if proc then ind=where(vv le threshl) $
      else ind=where(hh le threshl)
   endif else begin
      if proc then $
         ind=where(vv le (min(vvnoise)+thresh*std(vvnoise))) $
      else ind=where(hh le (min(hhnoise)+thresh*std(hhnoise)))
   endelse
endelse

return, ind

end

pro wcrvpsm, dir,acfile=acfile,printer=printer,fileID=fileID,tlabel=tlabel,$
    proc=proc,minlim=minlim,maxlim=maxlim,psdir=psdir,timecor=timecor, $
    timehack=timehack,thresh=thresh,mingates=mingates,aconfig=aconfig, $
    colortb=colortb,big_endian=bgnd,help=help

common wcrparams
common wcrdata

on_error,2

; Check inputs

if keyword_set(help) then begin
  doc_library,'wcrvpsm'
  return
endif

if n_elements(fileID) eq 0 then fileID='pp0'
if n_elements(tlabel) eq 0 then tlab='Corrected Doppler Velocity' $
else tlab=tlabel+' Corrected Doppler Velocity'
tlab=tlab+" (timehack: "+num2str(timehack,fr=2)+"s)"

antflag=0
if n_elements(aconfig) eq 0 then begin ; single antenna, dual position (up/side)
   aconfig=0
   antpos=0
   antstr=['','']
endif else if n_elements(aconfig) eq 1 then begin
   case aconfig of
      0: begin            ; single side/up antenna
           antpos=0
           antstr=['',''] ; [0] - V-ch, [1] - H-ch
         end
      1: begin          ; two downlooking antennas (H-ch: down-fore, V-ch: down)
           antpos=1
           antstr=['down','down_fore']
         end
      2: begin          ; two sidelooking antennas (V-ch: side-fore, H-ch: side/up)
           antpos=0
           antstr=['side-fore','up.side']
         end
      3: begin          ; down and side/up antennas (V-ch: down, H-ch: side/up)
           antflag=1    ; antpos depends on Rx channel; 1 - V, 0 - H
           antstr=['down','up.side']
         end
      4: begin          ; two downlooking antennas (V-ch: down, H-ch: down-aft, C130, V/H swapped)
           antpos=1
           antstr=['down','down-aft']
         end
      5: begin          ; down and side antennas (V-ch: down, H-ch: side)
           antflag=1    ; antpos depends on Rx channel; 1 - V, 0 - H
           antstr=['down','side']
         end
   else: message,'Invalid value for ACONFIG.'
   endcase
endif else begin
   k=where(aconfig lt 0 or aconfig gt 5)
   if k[0] ne -1 then message,'Invalid input for aconfig'
endelse

if n_elements(printer) eq 0 then $
   printer=''   ; supress printing

if n_elements(dir) eq 0 then $
   flst=dialog_pickfile(/read, filter='*',get_path=dir, $
        title='Pick any file to get the directory')

spawn, 'ls '+dir+'?'+fileID+'*',flst
flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,20)
nfiles=(size(flst))[1]-1

if n_elements(acfile) eq 1 then ncf=acfile $
else ncf=dialog_pickfile(/read, filter='*', $
         title='Pick A/C NetCDF data file (25 Hz)')

if n_elements(psdir) eq 0 then $
   if n_elements(dir) gt 1 then psdir='~/ps/' $
   else psdir=dir+'ps/'
if strmid(psdir,strlen(psdir)-1,1) ne '/' then psdir=psdir+'/'
if psdir ne '.' and psdir ne './' then spawn,'mkdir '+psdir+' >& /dev/null'

if (n_elements(colortb) eq 0) then col=-99 else col=colortb

noisefac=3  ; noisefac*std(noise) when thresh is not defined

; Process all files

for k=0,nfiles do begin
    fn=flst(k)
    openwcrf,dir+fn,timecor=timecor+timehack,big_endian=bgnd

    print,'+++++++++++++++++++++++++++++++++++++++'
    print,'Reading WCR file (',fn,') ...'

    if (alg eq pp4alg or alg eq pp6alg) then print,'PPMAG' $
    else if (alg eq sppalg) then print,'SPPMAG' $
    else if alg eq fftalg then print,'FFT' $
    else if alg eq dftalg then print,'DPFFT'

    if n_elements(proc) eq 0 then $
       if txsequence[0] eq polv then procv=1 else procv=0

    print,'+++++++++++++++++++++++++++++++++++++'

    ; Determine the number of profiles to process;
    ; 164mm: plot xsize, 49mm: plot ysize; 3 plots per page

    if n_elements(mingates) eq 0 then gates=(mingates=rangegates) $
    else gates=max([rangegates,mingates])

    profpersec=fix(1./timeinterval)
    segd=long(164./49.*gates*rangegatespacing/kaspeed*profpersec*3.)

    if tnumrec-segd lt 1000 then begin
       seg=tnumrec
       nseg=0
       print,'Total Segments: 1'
       print,''
    endif else begin
       seg=segd
       nseg=tnumrec/seg
       lastseg=tnumrec-nseg*seg
       print,'Total Segs: ',num2str(nseg+1)
       if lastseg le 200 then begin
          nseg=nseg-1
          print,'Last segment (size: ',num2str(lastseg),') ignored.'
       endif
       print,''
    endelse

    ; Prepare aconfig for this file

    if n_elements(aconfig) gt 1 then begin
      antflag=0
      case aconfig[k] of
        0: begin            ; single side/up antenna
             antpos=0
             antstr=['',''] ; [0] - V-ch, [1] - H-ch
           end
        1: begin
             antpos=1
             antstr=['down','down_fore']
           end
        2: begin
             antpos=0
             antstr=['side-fore','up.side']
           end
        3: begin
             antflag=1
             antstr=['down','up.side']
           end
        4: begin
             antpos=1
             antstr=['down','down-aft']
           end
        5: begin
             antflag=1
             antstr=['down','side']
           end
      endcase      
    endif

;   ************************  FFT or DPFFT mode ***************************

    if (alg eq fftalg or alg eq dftalg) then begin

       for ll=0,nseg do begin
          strt = ll*seg < (tnumrec-1)
          stp = ((ll+1)*seg-1) < (tnumrec-1)
          wcrtime=1
          readwcrdata, strt,stp,big_endian=bgnd,rawtime=wcrtime
          Z=wcrdopcorm(wcrtime,acncf=ncf)
          Z=reform(Z[*,*,0])

          print,'----> Current segment: '+num2str(ll)
          print,'----> Begin Time: ',Begtime
          print,'----> Profiles: '+num2str(tnumrec)+', Processing: '+ $
                       num2str(strt)+' - '+num2str(stp)

          fname=strmid(begtime,0,2)+strmid(begtime,3,2)+strmid(begtime,6,2)

          ; Extract noise and signal power and determine thresh indices

          if txsequence[0] eq polv then begin
             if alg eq fftalg then begin
                vv=total(fftps,1)/fftbins^2
                vvnoise=total(fftnoise,1)/fftbins^2
             endif else begin
                vv=total(reform(fftps[*,*,0,*]),1)/fftbins^2
                vvnoise=total(reform(fftnoise[*,0,*]),1)/fftbins^2
             endelse
             fname=[wcrfile+'.'+fname+'.FFT_VVD'+antstr[0],' ']
             if antflag then antpos=1
          endif else begin
             if alg eq fftalg then begin
                hh=total(fftps,1)/fftbins^2
                hhnoise=total(fftnoise,1)/fftbins^2
             endif else begin
                hh=total(reform(fftps[*,*,1,*]),1)/fftbins^2
                hhnoise=total(reform(fftnoise[*,1,*]),1)/fftbins^2
             endelse
             fname=[wcrfile+'.'+fname+'.FFT_HHD'+antstr[1],' ']
             if antflag then antpos=0
          endelse

          ; Find indices of the pixels to be thresholded

          ind=threshind((txsequence[0] eq polv),noisefac,thresh)

          if n_elements(ind) ge n_elements(Z) then begin
             message,'No pixels above threshold.',/info
             goto, S_LOOPEND_FFT
          endif

          ; Threshold velocity field and plot it

          if ind[0] ne -1 then Z[ind]=2.*minlim
          wcrvps,Z,fname=fname,col=col,dir=psdir,tlab=tlab,/minmax,$
                minlim=minlim,maxlim=maxlim,mingates=mingates,imgord=antpos

         ; Print output PS file if requested

          if printer ne '' then $
             spawn,'lp -o nobanner -d '+printer+' '+psdir+fname+'*.ps'

         ; Process to next data segment; if the last go to next file
       S_LOOPEND_FFT:
       endfor   ; end for ll

    endif else begin

;   *******************  SPPMAG, PPMAG4 or PPMAG6 mode ********************

       for ll=0,nseg do begin
          strt = ll*seg < (tnumrec-1)
          stp = ((ll+1)*seg-1) < (tnumrec-1)

          wcrtime=1
          readwcrdata, strt,stp,big_end=bgnd,rawtime=wcrtime,ppmnoav=2
          Z=wcrdopcorm(wcrtime,acncf=ncf)

          print,'----> Current segment: '+num2str(ll)
          print,'----> Begin Time: ',Begtime
          print,'----> Profiles: '+num2str(tnumrec)+', Processing: '+ $
                       num2str(strt)+' - '+num2str(stp)

          fname=strmid(begtime,0,2)+strmid(begtime,3,2)+strmid(begtime,6,2)

          if alg eq sppalg then begin                 ; SPPMAG mode
             Z=reform(Z[*,*,0])
             if txsequence(0) eq polv then begin
                fname=[wcrfile+'.'+fname+'.'+'VVD'+antstr[0],' ']
                if antflag then antpos=1
             endif else begin
                fname=[wcrfile+'.'+fname+'.'+'HHD'+antstr[1],' ']
                if antflag then antpos=0
             endelse

             ; Find indices of the pixels to be thresholded

             ind=threshind((txsequence(0) eq polv),noisefac,thresh)

          endif else begin                          ; PPMAG modes

             if procv then begin
                fname=[wcrfile+'.'+fname+'.'+'VVD'+antstr[0],' ']
                ind=(where(txsequence eq polv))[0]
                if ind eq -1 then message,'No V-pol transmission'
                if antflag then antpos=1
                Z=reform(Z[*,*,ind])
             endif else begin
                fname=[wcrfile+'.'+fname+'.'+'HHD'+antstr[1],' ']
                ind=(where(txsequence eq polv))[0]
                if ind eq -1 then message,'No V-pol transmission'
                if antflag then antpos=1
                Z=reform(Z[*,*,ind])
             endelse

            ; Find indices of the pixels to be thresholded

             ind=threshind(procv,noisefac,thresh)

          endelse

          if n_elements(ind) ge n_elements(Z) then begin
             message,'No pixels above threshold.',/info
             goto, S_LOOPEND_SPP
          endif

          ; Threshold velocity field and plot it

          if ind[0] ne -1 then Z[ind]=2.*minlim
          wcrvps,Z,fname=fname,col=col,dir=psdir,tlab=tlab,/minmax,$
                 minlim=minlim,maxlim=maxlim,mingates=mingates,imgord=antpos

         ; Print output PS file if requested

          if printer ne '' then $
             spawn,'lp -o nobanner -d '+printer+' '+psdir+fname+'*.ps'

         ; Process to next data segment; if the last go to next file
       S_LOOPEND_SPP:
       endfor   ; end for ll

    endelse
    closewcrf

endfor   ; end for k

end
