;$Id: wcrwrite1nc.pro,v 1.7 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRWRITE1NC
;
; PURPOSE:
;       Write WCR time and dimension related variables, calculated reflectivity,
;       uncorrected Doppler velocity and associated attributes to an existing
;       netcdf file matching loaded WCR raw data
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrwrite1nc,ncf,ncdir=ncdir,loadwcr=loadwcr,nswin=nswin,close=close, $
;                  writestr=writestr,revision=revision,silent=silent, $
;                  misval=misval,big_endian=bgnd,help=help
;
; INPUTS:
;       Ncf:   string, name of the netcdf file matching the WCR loaded data or
;              integer, id of netcdf file matching the WCR loaded data
;              (see also OUTPUTS)
;
; KEYWORD PARAMETERS:
;     BIG_ENDIAN: when set, assumes that the WCR file data are in big endian order.
;                 The default is LITTLE_ENDIAN. Ignored for data before 03/2002 
;       CLOSE:    when set netcdf file will be closed. Default is to keep the
;                 file open in WRITE mode. CAUTION: If a writable netcdf file
;                 is not closed before exiting IDL, the disk copy of the file
;                 may not reflect recent data changes or new definitions.
;       LOADWCR:  input,string; WCR raw data directory.  If this keyword is
;                   defined it is assumed that the raw WCR data matching ncf
;                   is not loaded. ATTEN: DO NOT DEFINE this keyword if the raw
;                   WCR data is already loaded in wcrparams and wcrdata common
;                   blocks (see the example below)
;       MISVAL:   input, when set (1) use NaN as missing values; Any value
;                   different than 1 will be used as missing value in dBZ.
;                   Default value is -32767. Maximum valid value is -150.
;       NCDIR:    input,string, directory path to ncf (default current dir)
;       NSWIN:    input, long or lonarr(3)
;                   nswin[0:1] - smoothing noise-window size(default 99);should 
;                                be an odd number(1 is added for evens); it is 
;                                recommended not to exceed 10% of the processed 
;                                number of profiles;  values less than 3 and 
;                                greater than 50% of all profiles cause the use
;                                of constant mean and stdev of the noise data 
;                   nswin[2]   - defines which noise measurement pulses to use:
;                     0 - (default) use matching number of noise measurement 
;                         pulses with minimum mean
;                     1 - uses noise measurement from one noise pulse with
;                         min mean (for both h- and v-ch)
;                     2 - as 1 for h-ch and 0 for v-ch noise 
;                     3 - as 1 for v-ch and 0 for h-ch noise 
;
;                   If NSWIN is a scalar, same value is applied to both channels
;                   as smoothing window size and NSWIN[2] is the default.
;                   If NSWIN has 2 elements, nswin[0] is applied to H and V
;                   noise channels and nswin[1] is used as nswin[2].
;
;       REVISION: input,string or strarr(2) netcdf file revision
;                 revision[0] - update Revision_X attribute (default 'Reflectivity
;                               and uncorrected Doppler velocity'); If the string 
;                               starts with $ the last Revision_X attribute is 
;                               overwritten otherwise a new Revision_X is created.
;                               If only '$' the last Revision_X will be overwritten
;                               with the default string. If WRITESTR eq 'all'
;                               the value, if any, is ignored and the default is
;                               used to update Revision_1.  All other existing 
;                               Revision_X for X>1 are deleted.
;                 revision[1] - update RevisionStatus attribute(default is 'OK')
;                 If input a string, update Revision_X attribute only;
;                 If empty string(s) '' - do not update the attribute(s)
;       SILENT:  when set supress routine informational messages
;       WRITESTR:input, string, determine what to write; allowed strings:
;                  'all'  - all relevant variables (default, see side effects)
;                  'time' - time related variables and other coordinates
;                  'z'    - reflectivity related variables
;                  'dv'   - Uncorrected Doppler velocity related variables
;                  'tz'   - 'time' + 'z' variables
;                  'tdv'  - 'time' + 'dv' variables
;                  'zdv'  - 'z' + 'dv' variables
;       HELP:    shows this text
;
; OUTPUTS:
;       Ncf:     when ClOSE is not set and ncf is the netcdf file name returns
;                WCR netcdf file id
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       Write time and dimension related variables, reflectivity, uncorrected
;       Doppler velocity and relevant attributes into an existing netcdf file.
;
;       The following variables and attributes are updated:
;       - base_time, time_offset, range, range_correction, psd_range(FFT only)
;       - global attributes AcquisitionDate, TimeInterval, WCRstattimeint,
;         WCRDSPclockstat,WCRppsstat
;       - dv with attributes missing_value,pulsepair/receiver,prf,status
;       - fftpsd with attributes receiver,transmitter and prf (FFT only)
;       - fftnoise with attribute receiver,transmitter (FFT only)
;       - wcraspect with attribute status.  At this level of processing
;         all valies are identical and based on assumed average aircraft speed
;         (taken from wcr common block wcrparams variable kaspeed). This speed
;         is defined during WCR raw file opening with openwcrf (acspeed keyword)
;       - reflectivity and noise variables: hh, vv, vh, hv, hmnoise, vmnoise,
;         hsnoise, vsnoise
;       - reflectivity and noise variables related attributes: missing_value,
;         receiver,transmitter,pulse/rangegate,smooth
;       - update global attributes: ProcessAuthor,DateProcessed,RevisionDate,
;                                   RevisionStatus,RevisionNumber,Revision_1
;
;       If the ncf had been modified between running wcrcdl2nc and
;       wcrwrite1nc all the variables and attributes described above are
;       overwritten. The Revision_X global attributes if X>1 are deleted and
;       a new Revision_1 attribute is created when WRITESTR = 'all'
;
; NOTES:
;
; EXAMPLES:
;
;       1) Example using CLEX-9 experiment data (see also wcrcdl2nc example)
;
;       .run openwcrm   ; it outputs data from openwcrf
;       ncf=''
;       wcrcdl2nc,'CLEX01.20020218.cdl',[1200,2199],ncfile=ncf, $
;                 savecdl=2,beams=1
;
;       readwcrdata,1200,2199,timemode=3, ppmnoav=0, /noleak
;       wcrwrite1nc,ncf,nswin=[99,99],rev=['','Test'],/close
;
;       2) As above but using WCRLOAD, assume ncf is given and no close
;
;       wcrwrite1nc,ncf,nswin=[99,0],rev=['','Test'],loadwcr='/sova4/clex01/'
;
;-
; MODIFICATION HISTORY:
;       Written by:   Samuel Haimov, March 2002
;       SH, 03/02 - added WRITESTR and changed REVSTAT to REVISION keyword
;       SH, 04/02 - added big_endian keyword
;       SH, 06/02 - commented out cross-pol cal constants (currently not in use)
;       SH, 08/02 - modified cal constants names to accommodate the multiple
;                   beam cal. constants
;       SH, 12/02 - added WCRtimeintstat attribute to be recorded using
;                   modified dspsync array from readwcrdata
;       SH, 04/03 - renamed WCRtimeintstat to WCRstattimeint, FlightDate to
;                   AcquisitionDate,modified to work with MARSF data 
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro noise_update,ncid,nswin,mnoise,channel,silent=silent

  @commonwcrinclude

  on_error,2

  ; Compile variables

  mnstr=channel+'mnoise'
  snstr=channel+'snoise'
  if channel eq 'h' then begin
    coch='h' & crch='v'
  endif else begin
    coch='v' & crch='h'
  endelse

  ; Update mean noise and noise standard deviation

  ncdf_update,ncid,mnstr,mnoise[*,0],'var',silent=silent
  ncdf_update,ncid,snstr,mnoise[*,1],'var',silent=silent

  ; Update (h/v)mnoise attributes: channel,pulse,smooth

  if nswin gt 0 then ncdf_update,ncid,'receiver',coch,mnstr,silent=silent $
  else ncdf_update,ncid,'receiver',crch,mnstr,silent=silent

  ; Commented out lines below reflect the change in the nswin inputs.
  ; They are left to remind me that I may use cross channel range gate for 
  ; noise in the future and it has to be recorded properly in an attribute

  ;if abs(nswin) gt 500 and abs(nswin) le 1000 then begin
  ;  k=0
  ;  ncdf_update,ncid,'pulse','N/A',mnstr,silent=silent
  ;endif else if abs(nswin) gt 1000 then begin
  ;  k=long(abs(nswin))/1000L & k=k+(k mod 2 eq 0)
  ;  junk2=ncdf_attload(ncid,mnstr,'allatts')
  ;  ind1=where(junk2 eq 'pulse')
  ;  if ind1[0] eq -1 then begin
  ;     k=where(junk2 eq 'rangegate')
  ;     if k[0] eq -1 then $
  ;       message,'Attribute pulse/rangegate for '+mnstr+' does not exist.' $
  ;     else ncdf_update,ncid,'rangegate', $
  ;               round(((abs(nswin)/1000.) mod 1)*1000.),mnstr,silent=silent
  ;  endif else begin
  ;    ncdf_update,ncid,'pulse',round(((abs(nswin)/1000.) mod 1)*1000.), $
  ;               mnstr,silent=silent
  ;    ncdf_rename,ncid,'pulse','rangegate',mnstr,silent=silent
  ;  endelse
  ;endif else begin
    k=abs(nswin)+(nswin mod 2 eq 0)
    if n_elements(noiseflag) eq 1 then junk2=noiseflag else begin
  ;    if nswin gt 0 then begin
        if coch eq 'h' then junk2=noiseflag[0] else junk2=noiseflag[1]
        if coch eq 'v' then junk2=noiseflag[1] else junk2=noiseflag[0]
  ;   endif
    endelse
    ncdf_update,ncid,'pulse',junk2,mnstr,silent=silent
  ;endelse
  ncdf_update,ncid,'smooth',k,mnstr,silent=silent
end

pro wcrwrite1nc,ncf,ncdir=ncdir,loadwcr=loadwcr,nswin=nswin,writestr=writestr, $
    misval=misval,revision=revision,big_endian=bgnd,close=close,silent=silent,help=help

  common wcrdata
  common wcrparams

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrwrite1nc'
    return
  endif

; Check inputs

  if n_params() lt 1 then begin
    message,"Usage: wcrwrite1nc,ncf,ncdir=ncdir,loadwcr=loadwcr, $",/info
    print,"                     nswin=nswin,misval=misval,revision=revision, $"
    print,"                     writestr=writestr,big_endian=bgnd, $"
    print,"                     silent=silent,help=help"
    return
  endif

  if n_elements(writestr) eq 0 then writestr='all' $
  else begin
    junk1=['all','time','z','tz','dv','zdv','tdv']
    writestr=strlowcase(writestr)
    ind1=where(writestr eq junk1)
    if ind1[0] eq -1 then begin
      message,'Invalid input for writestr'
    endif
  endelse

  if n_elements(revision) eq 0 then begin
    revstat='OK'
    revatt='Reflectivity and uncorrected Doppler velocity'
    revadd=1
  endif else begin
    if n_elements(revision) ge 1 then begin
      if writestr eq 'all' then begin
        revstat='OK' 
        revatt='Reflectivity and uncorrected Doppler velocity'
      endif else if revision[0] eq '' then begin 
        revstat=''
        revatt=''
      endif else begin
        revstat=''
        revadd=(strpos(revision[0],'$') ne 0)
        if revadd eq 1 then revatt=revision[0] $
        else begin
          if strlen(revision[0]) eq 1 then $
            revatt='Reflectivity and uncorrected Doppler velocity' $
          else revatt=strmid(revision[0],1)
        endelse
      endelse
    endif 
    if n_elements(revision) gt 1 then revstat=revision[1]
  endelse

  if n_elements(nswin) eq 0 then begin
    vnswin=99 & hnswin=99
  endif else if n_elements(nswin) eq 1 then begin
    hnswin=nswin & vnswin=hnswin
  endif else begin
    if keyword_set(loadwcr) then begin
      if n_elements(nswin) eq 2 then begin
        hnswin=nswin[0] & vnswin=hnswin & ppmnoav=nswin[1]+(nswin[1] gt 0)
      endif else begin
        hnswin=nswin[0] & vnswin=nswin[1] & ppmnoav=nswin[2]+(nswin[2] gt 0)
      endelse
    endif else begin
      hnswin=nswin[0] & vnswin=nswin[1]
    endelse
  endelse

  if n_elements(misval) eq 0 then novalue=-32767. $
  else begin
    if misval eq 1 then novalue=!values.f_nan $
    else if finite(misval) then novalue=(-abs(misval))<(-150.) $
    else novalue=!values.f_nan
  endelse

  if size(ncf,/type) eq 7 then begin
    ; Check ncf directory path

    if n_elements(ncdir) eq 0 then ncdir='./' $
    else if strpos(ncdir,'/',/reverse_search)+1 ne strlen(ncdir) then $
      ncdir=ncdir+'/'

    ; Open ncf in WRITE mode and load profile and range dimensions, all var
    ; names and some needed global attributes

    junk1=strpos(ncf,'.nc')
    if junk1 eq -1 then junk1='.nc' else junk1=''
    ncid=ncdf_open(ncdir+ncf+junk1,/write)
  endif else ncid=ncf

  ncnames=ncdf_varnames(ncid)

  ncdf_diminq,ncid,'range',junk1,rangedim
  ncdf_diminq,ncid,'profile',junk1,profdim

  ncdf_attget,ncid,'Aircraft',    aircraft, /global & aircraft=string(aircraft)

  ncdf_attget,ncid,'WCRprofav',   profav,   /global
  ncdf_attget,ncid,'WCRrangeav',  rangeav,  /global
  ncdf_attget,ncid,'WCRprofrange',profrange,/global
  ncdf_attget,ncid,'WCRdaqfile',  daqfile,  /global & daqfile=string(daqfile)

; Prepare HHcal and VVcal based on the beam configuration

  ncdf_attget,ncid,'WCRactivebeamsconfig',beams,/global

  ncdf_attget,ncid,'WCRcalHH1',HH1cal,/global
  ncdf_attget,ncid,'WCRcalVV1',VV1cal,/global

  if aircraft ne 'MARSF' then begin
    ncdf_attget,ncid,'WCRcalHH4',HH4cal,/global
    ncdf_attget,ncid,'WCRcalVV2',VV2cal,/global
    ncdf_attget,ncid,'WCRcalVV3',VV3cal,/global
  endif

  ; ncdf_attget,ncid,'WCRcalVH' ,VHcal, /global  ; currently not used
  ; ncdf_attget,ncid,'WCRcalHV',HVcal,/global    ; currently not used

  ncdf_attget,ncid,'WCRisoHV',HViso,/global
  ncdf_attget,ncid,'WCRisoVH',VHiso,/global
  ncdf_attget,ncid,'WCRcal100',cal100,/global
  ncdf_attget,ncid,'WCRcal500',cal500,/global

  if beams eq 1 then begin
    HHcal=HH1cal
    VVcal=VV1cal
  endif else if beams eq 2 then begin
    VVcal=VV2cal
  endif else if beams eq 3 then begin
    VVcal=VV3cal    
  endif else if beams eq 4 then begin
    HHcal=HH4cal
  endif else if beams eq 12 then begin
    HHcal=HH1cal
    VVcal=VV2cal
  endif else if beams eq 13 then begin
    HHcal=HH1cal
    VVcal=VV3cal
  endif else begin
    HHcal=HH4cal
    VVcal=VV2cal
  endelse

  ncdf_attget,ncid,'WCRpulsewidth',pulsewidth,/global
  if pulsewidth gt 300 then begin
    HHcal=HHcal+cal500
  ; VHcal=VHcal +cal500
    VVcal=VVcal+cal500
  ; HVcal=HVcal+cal500
  endif else if pulsewidth lt 200 then begin
    HHcal=HHcal+cal100
  ; VHcal =VHcal +cal100
    VVcal=VVcal+cal100
  ; HVcal=HVcal+cal100
  endif 

  ncdf_attget,ncid,'UTCcorrection',UTCcor,/global

;------------------------------------------------------------------------------
; Load WCR raw data if requested otherwise do some checking to verify that
; the data already loaded matches the ncf
;------------------------------------------------------------------------------

  if n_elements(loadwcr) eq 1 then begin   ; **** Load WCR data ****

    if strpos(loadwcr,'/',/reverse_search)+1 ne strlen(loadwcr) then $
      loadwcr=loadwcr+'/'
    openwcrf,loadwcr+daqfile,big_endian=bgnd

    if keyword_set(silent) eq 0 then $
      message,'Reading data from '+wcrfile+'...',/info
    if UTCcor eq tcorrection then $
      readwcrdata,profrange[0],profrange[1],timemode=3,ppmnoav=ppmnoav, $
                  big_endian=bgnd,/noleak $
    else begin
      if keyword_set(silent) eq 0 then $
        message,'No match between UTCcorrection and tcorrection. '+$
                'Use UTCcorrection',/info
      readwcrdata,profrange[0],profrange[1],timemode=3,timecor=UTCcor, $
                  ppmnoav=ppmnoav, /noleak,big_endian=bgnd
    endelse

    timeoffset=time

  endif else begin ; **** WCR data already loaded in the common blocks ****

  ; Compare raw wcr data file

    if daqfile ne wcrfile then begin
      ncdf_close,ncid
      message,'Raw WCR data files do not match.'
    endif

  ; Compare the loaded profile segment

    if profrange[0] ne prof[0]-fstblocknum or $
       profrange[1] ne prof[numrec-1]-fstblocknum  then begin
      ncdf_close,ncid
      message,'Profile segments do not match.'
    endif

  ; Compare profile and range dimension

    if profdim ne numrec/profav then begin
      ncdf_close,ncid
      message,"Profile dim("+num2str(profdim)+") doesn't match loaded profiles"$
              +'('+num2str(numrec/profav)+')'
    endif

    if rangedim ne rangegates/rangeav then begin
      ncdf_close,ncid
      message,'Range dim('+num2str(rangedim)+") doesn't match loaded rangegates"$
              +'('+num2str(rangegates/rangeav)+')'
    endif

   if UTCcor ne tcorrection then begin
     ncdf_close,ncid
     message,'No match between UTCcorrection and tcorrection.'
   endif

  ; If time is not in timemode=3 correct it

    if timemode ne 3 then begin
      if timemode eq 2 then begin
         timeoffset=time2sec(time)
         timeoffset=timeoffset-long(timeoffset)
      endif else timeoffset=time-long(time[0])
    endif else timeoffset=time

  endelse

  rdim=rangedim*rangeav-1
  pdim=profdim*profav-1

;------------------------------------------------------------------------------
; Write time and coordinates related variables
;------------------------------------------------------------------------------
  
if writestr eq 'all' or writestr eq 'time' or writestr eq 'tz' or $
writestr eq 'tdv' then begin

  ; Write basetime and time_offset depending on profav

  ncdf_update,ncid,'base_time',basetime,'var',silent=silent

  if profav gt 1 then timeoffset=rebin(timeoffset[0:pdim],profdim)
  ncdf_update,ncid,'time_offset',timeoffset,'var',silent=silent

  ; Prepare and write radar range coordinate and radar power range correction

  if rangeav gt 1 then junk1=rebin(radrng[0:rdim],rangedim) $
  else junk1=radrng
  ncdf_update,ncid,'range',junk1,'var',silent=silent

  junk1=20.*alog10(junk1)-60.
  ncdf_update,ncid,'range_correction',junk1,'var',silent=silent

  ; For FFT write psd_range

  if alg eq fftalg then ncdf_update,ncid,'psd_range',velo,'var',silent=silent

  ; Update AcquisitionDate, TimeInterval, WCRstattimeint, WCRDSPclockdiff, WCRppsstat

  ncdf_update,ncid,'AcquisitionDate',wcrdate,'global',silent=silent
  if profav le 1 then junk1=begtime+'-'+endtime $
  else junk1=sec2timestr(basetime mod 86400)+'-'+ $
             sec2timestr((basetime+long(timeoffset[profdim-1])) mod 86400)
  ncdf_update,ncid,'TimeInterval',junk1,'global',silent=silent

  junk1=histogram(time-long(time[0]),min=0.)
  k=n_elements(junk1)-2
  if k gt 2 then begin
    junk1=junk1[1:k]
    junk1=[float(num2str([mean(junk1),std(junk1)],fr=2)),max(junk1),min(junk1)]
  endif else junk1=[0.,0.,0.]
  ncdf_update,ncid,'WCRppsstat',junk1,'global',silent=silent
  ncdf_update,ncid,'WCRDSPclockstat',dspsync[0:2],'global',silent=silent
  ncdf_update,ncid,'WCRstattimeint',dspsync[3:5],'global',silent=silent

endif

;------------------------------------------------------------------------------
; Write uncorrected Doppler velocity
;------------------------------------------------------------------------------

if writestr eq 'all' or writestr eq 'dv' or writestr eq 'zdv' or $
writestr eq 'tdv' then begin

  if alg eq fftalg or alg eq sppalg then begin
    if alg eq fftalg then junk2=strlowcase(txstrpol[0]) $
    else junk2=ppstrpol[0]
    ind1=0
    junk1=wcrdopvel()
    if profav gt 1 or rangeav gt 1 then $
      junk1=rebin(junk1[0:rdim,0:pdim],rangedim,profdim)
    ncdf_update,ncid,'dv',junk1,'var',silent=silent
  endif else begin
    junk1=wcrdopvel()
    ind1=where(ppstrpol eq 'HH' or ppstrpol eq 'VV')
    if ind1[0] eq -1 then begin
      junk2=''
      if keyword_set(silent) eq 0 then $
        message,'There is no co-pol pulse pairs available. Ignore.", /info
    endif else begin
      if profav gt 1 or rangeav gt 1 then $
        junk1=rebin(junk1[0:rdim,0:pdim,n_elements(ind1)],rangedim,profdim, $
                    n_elements(ind1))
      if n_elements(ind1) gt 1 then junk1=transpose(junk1[*,*,ind1],[2,0,1])
      ncdf_update,ncid,'dv',junk1,'var',silent=silent
      junk2=ppstrpol[ind1[0]]
      if n_elements(ind1) gt 1 then $
        for k=1,n_elements(ind1)-1 do junk2=junk2+','+ppstrpol[ind1[k]]
    endelse
  endelse

  ; Update 'dv' attributes missing_value,prf,status

  ncdf_update,ncid,'missing_value',novalue,'dv',silent=silent
  ncdf_update,ncid,'prf',float(prf[ind1]),'dv',silent=silent
  ncdf_update,ncid,'status','uncorrected for platform motion,folded','dv', $
                   silent=silent

  ncdf_update,ncid,'pulsepair',junk2,'dv',silent=silent
  if alg eq fftalg then $
     ncdf_rename,ncid,'pulsepair','receiver','dv',silent=silent

  ;----------------------------------------------------------------------------
  ; Write uncorrected Doppler spectra (for FFT only)
  ;----------------------------------------------------------------------------

  if alg eq fftalg then begin
    if profav gt 1 or rangeav gt 1 then $
      junk1=rebin(fftps[fftbins,0:rdim,0:pdim],fftbins,rangedim,profdim) $
    else junk1=fftps
    ncdf_update,ncid,'fftpsd',junk1,'var',silent=silent

    if profav gt 1 then $
      junk1=rebin(fftnoise[fftbins,0:pdim],fftbins,profdim) $
    else junk1=fftnoise
    ncdf_update,ncid,'fftnoise',junk1,'var',silent=silent

    ; Update 'fftpsd' and 'fftnoise' attributes

    junk2=strlowcase(txstrpol[0])
    ncdf_update,ncid,'receiver',junk2,'fftpsd',silent=silent
    ncdf_update,ncid,'transmitter',junk2,'fftpsd',silent=silent
    ncdf_update,ncid,'prf',float(prf[0]),'fftpsd',silent=silent
    ncdf_update,ncid,'receiver',junk2,'fftnoise',silent=silent
    ncdf_update,ncid,'transmitter',junk2,'fftnoise',silent=silent
  endif

  ;----------------------------------------------------------------------------
  ; Write wcraspect and status attribute for aircraft platforms
  ;----------------------------------------------------------------------------

  if aircraft ne 'MARSF' then begin
    junk1=fltarr(profdim)+rangegatespacing/(timeinterval*kaspeed)
    ncdf_update,ncid,'wcraspect',junk1,'var',silent=silent
    junk2='constant, using '+aircraft+' '+num2str(kaspeed,fr=1)+' m/s airspeed'
    ncdf_update,ncid,'status',junk2,'wcraspect',silent=silent
  endif

endif

;------------------------------------------------------------------------------
; Write reflectivity and noise variables
;------------------------------------------------------------------------------

if writestr eq 'all' or writestr eq 'z' or writestr eq 'zdv' or writestr eq 'tz'$
then begin

  ; Update HH and VH channels

  if (where(txsequence eq polh))[0] ne -1 then begin

    ; Calibrate and write HH, missing_value, hmnoise and hsnoise

    junk1=wcrcalcoref([-999.,1.],HHcal,wcrstr='hh',mnoise=mnoise,nan=misval, $
          nswin=hnswin,nav=[profav,rangeav],/nothresh,silent=silent)
    ncdf_update,ncid,'hh',junk1,'var',silent=silent
    ncdf_update,ncid,'missing_value',novalue,'hh',silent=silent
    noise_update,ncid,hnswin,mnoise,'h',silent=silent

    ; Calibrate and write VH if exists

    if where(ncnames eq 'vh') ne -1 then begin
      junk1=wcrcalcrref([-999.,1.],[VVcal,HHcal,VHiso],wcrstr='vh',nan=misval,$
        mnoise=mnoise,nswin=vnswin,nav=[profav,rangeav],/nothresh,silent=silent)
      ncdf_update,ncid,'vh',junk1,'var',silent=silent
      ncdf_update,ncid,'missing_value',novalue,'vh',silent=silent

      ; If sppmag write vmnoise, vsnoise and attributes channel,pulse,smooth

      if alg eq sppalg then begin
        noise_update,ncid,vnswin,mnoise,'v',silent=silent
      endif
    endif

  endif

  ; Update VV and HV channels

  if (where(txsequence eq polv))[0] ne -1 then begin

    ; Calibrate and write VV, vmnoise and vsnoise

    junk1=wcrcalcoref([-999.,1.],VVcal,wcrstr='vv',mnoise=mnoise,nan=misval, $
          nswin=vnswin,nav=[profav,rangeav],/nothresh,silent=silent)
    ncdf_update,ncid,'vv',junk1,'var',silent=silent
    ncdf_update,ncid,'missing_value',novalue,'vv',silent=silent
    noise_update,ncid,vnswin,mnoise,'v',silent=silent

    ; Calibrate and write HV if exists

    if where(ncnames eq 'hv') ne -1 then begin
      junk1=wcrcalcrref([-999.,1.],[HHcal,VVcal,HViso],wcrstr='hv',nan=misval,$
        mnoise=mnoise,nswin=hnswin,nav=[profav,rangeav],/nothresh,silent=silent)
      ncdf_update,ncid,'hv',junk1,'var',silent=silent
      ncdf_update,ncid,'missing_value',novalue,'hv',silent=silent

      ; If sppmag write hmnoise, hsnoise and attributes channel,pulse,smooth

      if alg eq sppalg then begin
        noise_update,ncid,hnswin,mnoise,'h',silent=silent
      endif
    endif

  endif

endif

;------------------------------------------------------------------------------
; Update global attributes: ProcessAuthor,DateProcessed,RevisionDate,
;                           RevisionStatus,RevisionNumber,Revision_1
;------------------------------------------------------------------------------

  ncdf_update,ncid,'ProcessAuthor',getenv('USER')+'@'+getenv('HOST'),'global', $
                    silent=silent
  ncdf_update,ncid,'DateProcessed',systime(),'global',silent=silent
  ncdf_update,ncid,'RevisionDate',systime(),'global',silent=silent
  if revstat ne '' then $
    ncdf_update,ncid,'RevisionStatus',revstat,'global',silent=silent

  ; Check if the file had previous updates.  If so remove Revision attributes
  ; (only when writestr = 'all')

  ncdf_attget,ncid,'RevisionNumber',junk1,/global
  if writestr eq 'all' then begin
    if junk1 gt 1 then begin
      if keyword_set(silent) eq 0 then $
        message,'Revision_X (X>1) are deleted.',/info
      ncdf_control,ncid,/redef
      for k=2,junk1 do ncdf_attdel,ncid,'Revision_'+num2str(k),/global
      ncdf_control,ncid,/endef
      ncdf_control,ncid,/sync
    endif
    ncdf_update,ncid,'RevisionNumber',1,'global',silent=silent
    ncdf_update,ncid,'Revision_1',revatt,'global',silent=silent
  endif else begin
    if revatt ne '' then begin
      ncdf_attget,ncid,'RevisionNumber',junk1,/global
      if revadd eq 0 then begin
        if junk1 gt 1 then begin
          ncdf_attget,ncid,'Revision_'+num2str(junk1-1),junk2,/global
          if strtrim(strlowcase(string(junk2)),2) eq strtrim(strlowcase(revatt),$
          2) then begin
            ncdf_control,ncid,/redef
            ncdf_attdel,ncid,'Revision_'+num2str(junk1),/global
            ncdf_control,ncid,/endef
            ncdf_control,ncid,/sync
            ncdf_update,ncid,'RevisionNumber',junk1-1,'global',silent=silent
            if keyword_set(silent) eq 0 then $
              message,'Duplicate Revision_X. Last Revision_X deleted.',/info
          endif else $
            ncdf_update,ncid,'Revision_'+num2str(junk1),revatt,'global', $
                              silent=silent
        endif else $
          ncdf_update,ncid,'Revision_1',revatt,'global',silent=silent
      endif else begin
        ncdf_attget,ncid,'Revision_'+num2str(junk1),junk2,/global
        if strtrim(junk2) ne '' then junk1=junk1+1
        if strtrim(strlowcase(string(junk2)),2) eq strtrim(strlowcase(revatt),2)$
        then begin
          if keyword_set(silent) eq 0 then $
            message,'Duplicate Revision_X. New Revision_X ignored.',/info
        endif else begin
          if junk1 gt 1 then begin
            if keyword_set(silent) eq 0 then $
              message,'New RevisionNumber:'+num2str(junk1),/info
            ncdf_update,ncid,'RevisionNumber',junk1,'global',silent=silent
            ncdf_addnew,ncid,'Revision_'+num2str(junk1),revatt,'global', $
                             silent=silent
          endif else $
            ncdf_update,ncid,'Revision_1',revatt,'global',silent=silent
        endelse
      endelse
    endif
  endelse

;------------------------------------------------------------------------------
; Sync the file and close it if CLOSE is set
;------------------------------------------------------------------------------

  ncdf_control,ncid,/sync

  if keyword_set(close) then begin
    if keyword_set(silent) eq 0 then $
      message,'NetCDF file(ID='+num2str(ncid)+') closed.',/info
    ncdf_close, ncid
  endif else ncf=ncid

 if keyword_set(silent) eq 0 then message,'wcrwrite1nc is finished.',/info

end
