;$Id: wcrwrite2nc.pro,v 1.6 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRWRITE2NC
;
; PURPOSE:
;       Calculate and write corrected for platform motion, Doppler velocity and
;       related variables to the corresponding WCR netcdf file
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrwrite2nc,acfile,ncf,acnames=acnames,gpsc=gpsc,rmcor=rmcor, $
;                   timesync=timesync,unfold=unfold,misval=misval,ncdir=ncdir, $
;                   close=close,revision=revision,loadwcr=loadwcr,$
;                   big_endian=bgnd,silent=silent,help=help
;
; INPUTS:
;       Acfile:   string, path/name of the relevant aircraft data netcdf file
;       Ncf:      integer, id of netcdf file matching the WCR loaded data or
;                 string, name of the netcdf file matching the WCR loaded data
;                 (Acfile and Ncf are checked for matching using FlightDate
;                  and TimeInterval global attributes); (also see OUTPUTS)
; KEYWORD PARAMETERS:
;       ACNAMES:  input, string array, A/C netcdf varnames to be used
;                  (if not defined N2UW or N130AR default var names are used)
;                  acnames[0] - AC true air speed in m/s
;                  acnames[1] - AC roll [deg]
;                  acnames[2] - AC pitch [deg]
;                  acnames[3] - AC true heading [deg]
;                  acnames[4] - AC east-west(EW) ground velocity [m/s]
;                  acnames[5] - AC north-south(NS) ground velocity [m/s]
;                  acnames[6] - AC vertical velocity [m/s]
;                  acnames[7] - AC East component of the wind [m/s]
;                  acnames[8] - AC North component of the wind [m/s]
;                  acnames[9] - AC vertical wind [m/s]
;                  acnames[10]- AC roll rate [rad/s] (use '' if N/A)
;                  acnames[11]- AC pitch rate [rad/s](use '' if N/A)
;                  acnames[12]- AC heading rate [rad/s] (use '' if N/A)
;                  acnames[13]- AC EW ground vel. GPS correction(use '' if N/A)
;                  acnames[14]- AC NS ground vel. GPS correction(use '' if N/A)
;
;                  Note: define first 10 names only if the last 5 vars are
;                        not needed; see ACLOADINS.PRO help for default variable 
;                        names
;     BIG_ENDIAN: when set, assumes that the WCR file data are in big endian order.
;                 The default is LITTLE_ENDIAN. Ignored for data before 03/2002 
;       CLOSE:    when set netcdf file will be closed. Default is to keep the
;                 file open in WRITE mode. CAUTION: If a writable netcdf file
;                 is not closed before exiting IDL, the disk copy of the file
;                 may not reflect recent data changes or new definitions.
;       GPSC:     when set does NOT apply GPS correction to A/C E-W and N-S
;                 velocity for UWKA only (ignored for NCAR C130 or NRC CV580)
;                 NCAR C130 A/C E-W and N-S ground speed are GPS corrected.
;                 NRC CV580 does not use GPSC.
;       LOADWCR:  input,string; WCR raw data directory.  If this keyword is
;                 defined it is assumed that the raw WCR data matching ncf
;                 is not loaded. ATTEN: DO NOT DEFINE this keyword if the raw
;                 WCR data is already loaded in wcrparams and wcrdata common
;                 blocks (see the example below)
;       MISVAL    missing value for corrected Doppler field
;                 (default: same as for uncorrected Doppler velocity)
;       NCDIR:    input,string, directory path to ncf (default current dir)
;       RMCOR:    integer/fltarr(3), applies antenna moment correction
;                  0 - antenna moment correction NOT applied  
;                  1 - antenna moment correction applied
;                  [x,y,x] - ant. position in aircraft coord w.r.t IRS in meters
;                 Default: for UWKA or NRC Convair not applied (0) 
;                          for NCAR C130 applied(1)
;                 Default antenna position can be seen in wcracvelm.pro
;       REVISION: input,string or strarr(2) netcdf file revision
;                 revision[0] - update Revision_X attribute (default 'Corrected
;                               Doppler velocity'); If the string starts with $
;                               the last Revision_X attribute is overwritten
;                               otherwise a new Revision_X is created.  If only
;                               '$' the last Revision_X will be overwritten with
;                               the default string 'Corrected Doppler velocity'
;                               ATTN: You cannot overwrite Revision_1 attribute
;                 revision[1] - update RevisionStatus attribute(default is 'OK')
;                 If input a string, update Revision_X attribute only;
;                 If empty string(s) '' - do not update the attribute(s)
;       SILENT:   when set supress routine informational messages
;       TIMESYNC: input,float, WCR time correction to sync with A/C IRS(INS)
;                 data, in seconds  (default 0.0)
;       UNFOLD:   input, Doppler velocity unfolding procedure (see notes)
;                 0          - no unfolding is performed (default)
;                 1          - use AC measured winds
;                 fltarr(3)  - [u,v,w], use the given wind vector
;                              u,v,w are allowed the following values:
;                              float: actual wind value to be used in m/s
;                                888: use AC measured values for all 
;                                     profiles of this component
;                                777: use mean of AC measured values from 
;                                     all profiles of this component
;       HELP:     shows this text
;
; OUTPUTS:
;       Ncf:     when ClOSE is not set, WCR netcdf file id
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;
;       The following variables and attributes of the netcdf files are updated:
;       - time_offset (if timesync ne 0)
;       - dv and attributes dvshift, status and missing value (if misval given)
;       - wcraspect and status attribute
;       - wcrbeamvector and beamchannel attribute
;       - acvcbeam and beamchannel attribute
;       - acwcbeam and beamchannel attribute
;       - wcrbeams and beamchannel attribute (at this time no input from WCR
;         or aircraft data systems is available and global attribute
;         WCRactivebeamsconfig value is used)
;       - wcrmirror (if exist)
;
;       The global attributes, AircraftDataSource: the name of the file used
;       for INS/IRS aircraft data, and AircraftDateProcessed: processing date
;       for AircraftDatSource file are updated using input acfile name and
;       ProcessedDate from the netcdf acfile if exists.
;
;       The RevisionStatus and Revision_X global attributes are updated and
;       RevisionNumber attribute is increased by 1
;
;
; NOTES:
;
; EXAMPLES:
;
;       1) Example using CLEX-9 experiment data (single antenna config, N2UW)
;
;       .run openwcrm
;       readwcrdata,1200,2199,timemode=3, /noleak
;
;       ncfile=''
;       cdlfile='CLEX01.20020218.cdl'
;       wcrcdl2nc,cdlfile,[1200,2199],averages=2,ncfile=ncfile,savecdl=2,beams=1
;
;       ncid=ncfile
;       wcrwrite1nc,ncid,nswin=[99,99],rev=['$','Test']
;
;       acfile='/net/bobcat/disk2/kingair_data/20011101.c25.nc'
;       wcrwrite2nc,acfile,ncid,timesync=-0.2,rev='Corrected velocity'
;
;       2) Example using DYCOMS-II experiment data (2 down antennas, N130AR)
;
;       .run commonwcrset
;       openwcrf,'/sova4/dycoms01/jul10/Wpp01-07-10-10-33-12', $
;                timecor=0.,acspeed=115.
;       readwcrdata,timemode=3, /noleak, /nocross
;
;       cdlfile='DYCOMS01.06.20020305.cdl'
;       indir='~/wcr/wcrproc'
;       outdir=indir
;       wcrcdl2nc,cdlfile,indir=indir,outdir=outdir,ncfile=ncfile, $
;                 beams=24,savecdl=2
;
;       ncid=ncfile
;       wcrwrite1nc,ncid,rev=['$','Test']
;
;       acfile='/sova4/dycoms01/jul10/RF01.20010710.060111_151725.PNI.nc'
;       wcrwrite2nc,acfile,ncid,timesync=-0.125,rmcor=0
;       ; Note: Radar moment correction is not applied (rmcor=0) due to missing
;       ;       pitch/roll/heading rates in the C130 data files
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, March 2002
;       SH, 04/02 - added big_endian keyword
;       SH, 10/02 - added acwcbeam variable to be recorded in the nc file
;       SH, 05/03 - changed to work with the modified wcrdopcorm, added UNFOLD
;       SH, 09/03 - modified values for UNFOLD
;       SH, 11/03 - modified to work with NRC-CV580 idl restore data file
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrwrite2nc,acfile,ncf,acnames=acnames,gpsc=gpsc,rmcor=rmcor, $
                timesync=timesync,unfold=unfold,misval=misval,ncdir=ncdir, $
                revision=revision,big_endian=bgnd,loadwcr=loadwcr,close=close,$
                silent=silent,help=help

  @commonwcrinclude

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrwrite2nc'
    return
  endif

; Check inputs

  if n_params() lt 2 then begin
    message,"Usage: wcrwrite2nc,acfile,ncf,acnames=acnames,$",/info
    print,"                   gpsc=gpsc,rmcor=rmcor,timesync=timesync,$"
    print,"                   unfold=unfold,ncdir=ncdir,revision=revision,$"
    print,"                   loadwcr=loadwcr,misval=misval,close=close,$"
    print,"                   big_endian=bgnd,silent=silent,help=help"
    return
  endif

  if n_elements(timesync) eq 0 then timesync=0.

  if not(n_elements(acnames) eq 0 or n_elements(acnames) eq 10 or $
     n_elements(acnames) eq 13 or n_elements(acnames) eq 15) then $
     message,'Incorrect input for acnames'

  if size(ncf,/type) eq 7 then begin
    ; Check ncf directory path

    if n_elements(ncdir) eq 0 then ncdir='./' $
    else if strpos(ncdir,'/',/reverse_search)+1 ne strlen(ncdir) then $
      ncdir=ncdir+'/'

    ; Open ncf in WRITE mode and load profile and range dimensions, all var
    ; names and some needed global attributes

    junk1=strpos(ncf,'.nc')
    if junk1 eq -1 then junk1='.nc' else junk1=''
    ncid=ncdf_open(ncdir+ncf+junk1,/write)
  endif else ncid=ncf

  ncnames=ncdf_varnames(ncid)
  dimnames=ncdf_dimnames(ncid,dimsize=junk1)

  rangedim=junk1[where(dimnames eq 'range')]
  profdim=junk1[where(dimnames eq 'profile')]
  if where(dimnames eq 'beam') ne -1 then $
    beamdim=junk1[where(dimnames eq 'beam')] $
  else beamdim=0
  if where(dimnames eq 'copp') ne -1 then $
    coppdim=junk1[where(dimnames eq 'copp')] $
  else coppdim=0

  ncdf_attget,ncid,'WCRprofav',   profav,   /global
  ncdf_attget,ncid,'WCRrangeav',  rangeav,  /global
  ncdf_attget,ncid,'WCRprofrange',profrange,/global
  ncdf_attget,ncid,'WCRdaqfile',  daqfile,  /global & daqfile=string(daqfile)
  ncdf_attget,ncid,'UTCcorrection',utccor,/global

; Prepare beaminit(wcrdopcorm); (use WCRactivebeamsconfig global attribute);
; for NRC-CV580: beaminit=-1 will be redefined after AC netcdf file is open
  ncdf_attget,ncid,'WCRactivebeamsconfig',beams,/global

  if beams eq 1 then beaminit1=-1 $
  else if beams eq 2 then $
    ncdf_attget,ncid,'WCRbeam2UnitVector',beaminit1,/global $
  else if beams eq 3 then $
    ncdf_attget,ncid,'WCRbeam3UnitVector',beaminit1,/global $
  else if beams eq 4 then $
    ncdf_attget,ncid,'WCRbeam4UnitVector',beaminit1,/global $
  else if beams eq 12 then begin
    beaminit1=-1
   ;ncdf_attget,ncid,'WCRbeam1UnitVector',beaminit1,/global
    ncdf_attget,ncid,'WCRbeam2UnitVector',beaminit2,/global
  endif else if beams eq 13 then begin
    beaminit1=-1
   ;ncdf_attget,ncid,'WCRbeam1UnitVector',beaminit1,/global
    ncdf_attget,ncid,'WCRbeam3UnitVector',beaminit2,/global
  endif else begin
    ncdf_attget,ncid,'WCRbeam2UnitVector',beaminit2,/global
    ncdf_attget,ncid,'WCRbeam4UnitVector',beaminit1,/global
  endelse

;------------------------------------------------------------------------------
; Load WCR raw data if requested otherwise do some checking to verify that
; the data already loaded matches the ncfile
;------------------------------------------------------------------------------

  if n_elements(loadwcr) eq 1 then begin   ; **** Load WCR data ****

    if strpos(loadwcr,'/',/reverse_search)+1 ne strlen(loadwcr) then $
      loadwcr=loadwcr+'/'
    openwcrf,loadwcr+daqfile,timecor=UTCcor,big_endian=bgnd

    if keyword_set(silent) eq 0 then $
      message,'Reading data from '+wcrfile+'...',/info
    readwcrdata,profrange[0],profrange[1],timemode=3,/pponly,big_endian=bgnd

  endif else begin ; **** WCR data already loaded in the common blocks ****

  ; Compare raw wcr data file

    if daqfile ne wcrfile then begin
      message,'Raw WCR data files do not match.',/info
      ncdf_close,ncid
      return
    endif

  ; Compare the loaded profile segment

    if profrange[0] ne prof[0]-fstblocknum or $
       profrange[1] ne prof[numrec-1]-fstblocknum  then begin
      message,'Profile segments do not match.',/info
      ncdf_close,ncid
      return
    endif

  ; Compare profile and range dimension

    if profdim ne numrec/profav then begin
      message,"Profile dim("+num2str(profdim)+") doesn't match loaded profiles"$
              +'('+num2str(numrec/profav)+')',/info
      ncdf_close,ncid
      return
    endif

    if rangedim ne rangegates/rangeav then begin
      message,'Range dim('+num2str(rangedim)+") doesn't match loaded rangegates"$
              +'('+num2str(rangegates/rangeav)+')',/info
      ncdf_close,ncid
      return
    endif

   if utccor ne tcorrection then begin
     message,'No match between UTCcorrection and tcorrection.',/info
     ncdf_close,ncid
     return
   endif

  ; If time is not in timemode=3 correct it

    if timemode ne 3 then begin
      if timemode eq 2 then begin
         time=time2sec(time)
         time=time-long(time)
      endif else time=time-long(time[0])
    endif

  endelse

  rdim=rangegates
  pdim=numrec 

;------------------------------------------------------------------------------
; Prepare WCR time in UNIX seconds
;------------------------------------------------------------------------------

  wcrtime=time+double(basetime)

;------------------------------------------------------------------------------
; Open A/C file read Aircraft, update AircraftDataSource,AircraftDateProcessed
;------------------------------------------------------------------------------

  if strmatch(strupcase(acfile),'*NRC-CV580*') then begin
    restore,acfile
    acname=strupcase(acdata.info)

    ; update beaminit1 for the side beam
    if beaminit1 eq -1 then $
      ncdf_attget,ncid,'WCRbeam1sideUnitVector',beaminit1,/global

  endif else begin
    acid=ncdf_open(acfile)

    ncdf_attget,acid,'Aircraft',acname, /global 
    acname=strupcase(string(acname))

    junk1=ncdf_attload(acid,'allatts',/global)
    ind1=where(junk1[0,*] eq 'DateProcessed')
    if ind1[0] eq -1 then begin
      if keyword_set(silent) eq 0 then begin
        message,"Global attribute DateProcessed missing for "+junk2,/info
        print,'              Updating AircraftDateProcessed ignored.'
      endif
    endif else $
      ncdf_update,ncid,'AircraftDateProcessed',junk1[1,ind1],'global'
  endelse

  junk2=strpos(acfile,'/',/reverse_search)
  if junk2 eq -1 then junk2=acfile else junk2=strmid(acfile,junk2+1)
  ncdf_update,ncid,'AircraftDataSource',junk2,'global'

;------------------------------------------------------------------------------
; Calculate corrected Doppler velocity for every co-channel pulse pair
;------------------------------------------------------------------------------

  if beamdim le 1 then begin      ; One beam

    if alg ne fftalg then begin

      junk1=where(ppstrpol eq 'HH' or ppstrpol eq 'VV')
      if junk1[0] eq -1 then begin
        message,'There are no co-pulse pairs available',/info
        ncdf_close,ncid
        ncdf_close,acid
        return
      endif
      junk1=ppstrpol(junk1)

      if beams ne 1 and beams lt 10 then begin
        if beams eq 4 and junk1(n_elements(junk1)-1) ne 'HH' then $
           message,"No match between beamchannel:'h'and pp (must be HH)"
        if beams lt 4 and junk1(n_elements(junk1)-1) ne 'VV' then $
           message,"No match between beamchannel:'v'and pp (must be VV)"
      endif else if beams gt 10 then begin
        if keyword_set(silent) eq 0 then $
          message,"Two active beams but only 1st beam will be recorded",/info
        if beams/10 eq 1 and junk1[0] ne 'HH' then $
          message,"No match between beamchannel:'h'and pp (must be HH)"
        if beams/10 eq 2 and junk1[0] ne 'VV' then $
          message,"No match between beamchannel:'v'and pp (must be VV)"
      endif

    endif

    cdv=wcrdopcorm(wcrtime,acdata=acdata,acncf=acid,timehack=timesync,$
        ppselect='CO',beam=beaminit1,gpsc=gpsc,rmcor=rmcor,radbeam=beamvector,$
        unfold=unfold,wcracv=acvcbeam,wcracw=acwcbeam,ncnames=acnames)

    if n_elements(junk1) gt 1 then cdv=transpose(cdv,[2,0,1])

  endif else begin               ; Two beams

    junk1=where(ppstrpol eq 'HH')
    junk2=where(ppstrpol eq 'VV')

    if junk1[0] eq -1 and junk2[0] eq -1 then begin
      message,'There are no co-pulse pairs available',/info
      ncdf_close,ncid
      ncdf_close,acid
      return
    endif

    if junk1[0] eq -1 then begin
      if keyword_set(silent) eq 0 then $
        message,"Doppler for h_beamchannel is not recorded",/info
        if n_elements(acnames) gt 0 then ncnames1=acnames
    endif else begin
      cdv1=wcrdopcorm(wcrtime,acdata=acdata,acncf=acid,timehack=timesync,$
        ppselect='HH',beam=beaminit1,gpsc=gpsc,rmcor=rmcor,radbeam=beamvector1,$
        unfold=unfold,wcracv=acvcbeam1,wcracw=acwcbeam1,ncnames=acnames)

      if n_elements(junk1) gt 1 then cdv1=transpose(cdv1,[2,0,1])$
      else cdv1=reform(cdv1,1,rdim,pdim)

    endelse

    if junk2[0] eq -1 then begin
      if keyword_set(silent) eq 0 then $
        message,"Doppler for v_beamchannel is not recorded",/info
      cdv=reform(cdv1)   & beamvector=beamvector1 
      acvcbeam=acvcbeam1 & acwcbeam=acwcbeam1
    endif else begin
      cdv=wcrdopcorm(wcrtime,acdata=acdata,acncf=acid,timehack=timesync,$
        ppselect='VV',beam=beaminit2,gpsc=gpsc,rmcor=rmcor,radbeam=beamvector,$
        unfold=unfold,wcracv=acvcbeam,wcracw=acwcbeam)

      if n_elements(junk2) gt 1 then cdv=transpose(cdv,[2,0,1]) $
      else cdv=reform(cdv,1,rdim,pdim)
      if junk1[0] ne -1 then begin
        if junk1[0] lt junk2[0] then $
          cdv=[cdv1,cdv] $
        else cdv=[cdv,cdv1]
        k=n_elements(acdata.time)
        if beams eq 24 then begin
          beamvector=reform([beamvector,beamvector1],2,3,k)
          acvcbeam=transpose([[acvcbeam],[acvcbeam1]])
          acwcbeam=transpose([[acwcbeam],[acwcbeam1]])
        endif else begin
          beamvector=reform([beamvector1,beamvector],2,3,k)
          acvcbeam=transpose([[acvcbeam1],[acvcbeam]])
          acwcbeam=transpose([[acwcbeam1],[acwcbeam]])
        endelse
      endif
    endelse
  endelse

;------------------------------------------------------------------------------
; Close aircraft netcdf file
;------------------------------------------------------------------------------

  if acname ne 'NRC-CV580' then ncdf_close,acid

;------------------------------------------------------------------------------
; Prepare BEAMVECTOR, ACVCBEAM, ACWCBEAM, TAS and WCRMIRROR
;------------------------------------------------------------------------------

  beamvector= ac2wcr(beamvector,acdata.time,wcrtime,timehack=timesync,/errorm)
  acvcbeam  = ac2wcr(acvcbeam,acdata.time,wcrtime,timehack=timesync,/errorm)
  acwcbeam  = ac2wcr(acwcbeam,acdata.time,wcrtime,timehack=timesync,/errorm)
  tas       = ac2wcr(acdata.tas,acdata.time,wcrtime,timehack=timesync,/errorm)

  ; If wcrmirror is relevant find mirror changes and assign -1 for -1 sec
  ; to +3 sec around the mirror switch event ; then match to radar profiles

  if (beams eq 1 or fix(beams/10) eq 1 or round(beams/10. mod 1) eq 1) and $
  acname ne 'NRC-CV580' then begin
    junk2=n_elements(acdata.time)
    junk1=round(1./(acdata.time[1]-acdata.time[0])) ; sps for aircraft data

    ind1=where(abs((acdata.wcrmirror-shift(acdata.wcrmirror,1))[1:junk2-1]) eq 1)
    if ind1[0] ne -1 then begin
       ind1=ind1+1
       for k=0,n_elements(ind1)-1 do $
         acdata.wcrmirror[(ind1[k]-junk1)>0:(ind1[k]+3*junk1)<(junk2-1)]=-1
    endif

    wcrmirror=ac2wcr(acdata.wcrmirror,acdata.time,wcrtime,timehack=timesync,/errorm)

   ind1=where(wcrmirror ne 0 and wcrmirror ne 1 and wcrmirror ne -1)
   if ind1[0] ne -1 then wcrmirror[ind1]=-1
  endif

;------------------------------------------------------------------------------
; Determine CDVSTATUS attribute
;------------------------------------------------------------------------------

  if acname eq 'N2UW' then begin
    if keyword_set(gpsc) then junk1='=N' else junk1='=Y'
    if n_elements(rmcor) eq 0 then junk2='=N' $
    else begin
      if n_elements(rmcor) eq 3 then junk2='=Y' $
      else if n_elements(rmcor) eq 1 then $
        if rmcor eq 0 then junk2='=N' else junk2='=Y'
    endelse
  endif else begin
    if acname eq 'N130AR' or acname eq 'NRC-CV580' then junk1='=Y' $
    else if keyword_set(gpsc) then junk1='=N' else junk1='=Y'
    if n_elements(rmcor) eq 0 then begin
       if acname eq 'N130AR' then junk2='=Y' else junk2='=N'
    endif else begin
      if n_elements(rmcor) eq 3 then junk2='=Y' $
      else if n_elements(rmcor) eq 1 then $
        if rmcor eq 0 then junk2='=N' else junk2='=Y'
    endelse
  endelse

  cdvstatus='platform motion corr.=Y, radar moment corr.'+junk2+', '+$
            'GPS corr.'+junk1+'; noise thresh.=N, folded=Y'

;------------------------------------------------------------------------------
; Write DVSHIFT attribute according to UNFOLD keyword
;------------------------------------------------------------------------------

  if n_elements(unfold) eq 0 then junk1='none' $
  else begin
    if n_elements(unfold) eq 1 and unfold[0] eq 0 then junk1='none' $
    else junk1='wind component into the beam'
  endelse

  ncdf_update,ncid,'dvshift',junk1,'dv',silent=silent

;------------------------------------------------------------------------------
; Write the Doppler data in the ncf file and quit
;------------------------------------------------------------------------------

  wcrcdv2nc,ncid,cdv,beamvector,acvcbeam,acwcbeam,tas,wcrmirror, $
            timesync=timesync,misval=misval,cdvstatus=cdvstatus, $
            revision=revision,close=close,silent=silent

  if keyword_set(close) eq 0 then ncf=ncid

;------------------------------------------------------------------------------
; Done.
;------------------------------------------------------------------------------

 if keyword_set(silent) eq 0 then message,'wcrwrite2nc is finished.',/info

end
