;$Id: wcrwunfold.pro,v 1.2 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;      WCRWUNFOLD
;
; PURPOSE:
;      Returns shifted/unfolded WCR Doppler corrected for AC motion
;      velocity around given wind value component into the WCR beam
;
; CATEGORY:
;      wcrtools
;
; CALLING SEQUENCE:
;      res=wcrwunfold(wcrdvf,vmax,wcrwind)
;    or
;      res=wcrwunfold(wcrdvf,vmax,wind,wcrbeam)
;
; INPUTS:
;      Wcrdvf:  fltarr(rg,prof),WCR measured radial Doppler velocity field
;               corrected for AC motion in m/s; rg: range gates, pro: radar 
;               profiles; positive values means toward the radar
;      Vmax:    float, Doppler Nyquist velocity in m/s
;      -------
;      Wcrwind: fltarr(prof),wind contribution into the WCR beam;
;               Positive values means away from the radar
;
;      -------  or
;
;      Wind:    fltarr(3), Mean wind vector to be applied for all vel. points
;               or
;               fltarr(prof,3), Wind vector[EW,NS,vertical] for each 
;                               radar profile in m/s 
;      Wcrbeam: fltarr(3,prof), WCR beam in ground corrdinates
;      -------
;
; KEYWORD PARAMETERS:
;      HELP:     show this text
;
; SIDE EFFECTS:
;      The function returns an empty string if the usage is incorrect
;
; NOTES:
;      Attention is needed when AC measured wind component into the WCR
;      beam is to be used as wcrwind. When taken from wcrdopcorm or
;      wcracvelm (wcracw) it is synched with AC time and therefore it
;      has to be re-synched with wcrtime first.  Same is true for using Wind
;      and Wcrbeam inputs when they are taken/derived from the AC data 
;      (e.g., wcrdopcorm,wcracvelm,acloadins)            
;
; PROCEDURE:
;      This routine shifts the Doppler velocity data according to the
;      mean wind given as an input.  It is assumed that the input wind 
;      values are not changing along the range gate dimension of the
;      measured Doppler field.  It is also assumed that any folding
;      that takes place in the input Doppler field is caused by the
;      input wind.  In this case this procedure will unfold the folded
;      velocities even if the wind causes multiple folding. The unfolding
;      may not be successful if the folded velocities are affected by
;      other phenomena, e.g., perturbations to the velocity field exceeding
;      the unambiguous range, strong shear, etc.
; 
;
; EXAMPLES:
;      ; Use data from HBDD test experiment in September 2001
;      .run commonwcrset
;      openwcrf,'Wpp01-09-18-18-01-39'
;      readwcrdata,192830,192930,/timein,rawtime=wcrtime
;      acncf='/net/owl/data/kingair_data/20010918.c25.nc' & acid=acncf
;
;      ;Get folded velocities for the side beam
;      wcrdvf=wcrdopcorm(wcrtime,acncf=acid,timehack=-0.3,ppselect='VV')
;
;      ;Get unfolded velocities for the side beam
;      wcrdvu=wcrdopcorm(wcrtime,acncf=acid,timehack=-0.3,ppselect='VV',/unfold)
;
;      ;Get folded velocities for the side-slanted beam
;      wcrdvf=wcrdopcorm(wcrtime,acncf=acid,timehack=-0.3,ppselect='HH')
;
;      ;Get unfolded velocities for the side-slanted beam
;      wcrdvu=wcrdopcorm(wcrtime,acncf=acid,timehack=-0.3,ppselect='HH',/unfold)
;
;-
; MODIFICATION HISTORY:
;      Written by: Samuel Haimov and Rick Damiani, May 2003
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrwunfold,wcrdvf,vmax,wind,wcrbeam,help=help 

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrwunfold'
    return,''
  endif

; Check input and calculate wind component into wcr beam if needed

  if n_params() lt 3 then begin
    message,"Usage: res=wcrwunfold(wcrdvf,vmax,wcrwind)",/info
    print,"            res=wcrwunfold(wcrdvf,vmax,wind,wcrbeam)"
    return,''
  endif else if n_params() eq 3 then begin
    wcracw=wind
  endif else if n_params() eq 4 then begin
  ; Check wind input and calculate wind component into the radar beam
    if n_elements(wind) eq 3 then $ ; Use mean wind for all profiles
      wcracw=wcracwind(relicarr(wind,n_elements(wcrdvf[0,*]),wcrbeam)) $
    else wcracw=wcracwind(wind,wcrbeam)
  endif
    
; Process the shift/unfolding

  ; Form wind field to include all WCR range gates. Take care of sign convention    
    w=replicarr(-wcracw,n_elements(wcrdvf[*,0]),dim=1)

  ; Save the wind component directions (-1,0,1); 0 means zero wind, no shift
    sw=sign(w)

  ; Do some housework and calculate the foldings
    wcrdvu= wcrdvf
    vmax2 = 2.*vmax
    nvm   = fix(w/vmax2)*vmax2

  ; Calculate the folding border
    f=w-sw*(vmax+nvm)

  ; Find all velocity points left and right of the border and shift them
  ; accordingly
    w  = sw*wcrdvf
    f  = sw*f
    ind= where(w gt f)
    if ind[0] ne -1 then wcrdvu[ind]=wcrdvf[ind]+nvm 
    ind=where(w le f)
    if ind[0] ne -1 then wcrdvu[ind]=wcrdvf[ind]+nvm+sw*vmax2

; Done

  return,wcrdvu

end 
