;$Id: wcrzps.pro,v 1.7 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRZPS
;
; PURPOSE:
;       Creates a reflectivity PS file with maximum 3 plots per page
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       WCRZPS, Z,r,tsec
;
; INPUTS:
;    Z:     fltarr(rangeGates,profs), reflectivity in dBZ. If noread keyword is
;                 not used, Z is an output. HH reflectivity only.
;                 If Z is not given a widget to pick a WCR file will open.
;    R:     fltarr(rangeGates), range;
;                 mainly for non WCR files (see PRMS keyword)
;    Tsec:  fltarr(profs), time in sec.
;                 mainly for non WCR files (see PRMS keyword)
;
; KEYWORD PARAMETERS:
;    CALCOEF:     reflectivity (hh) calibration coefficient in dB
;                 (default, 31.)
;    COLORTB:     color table number (default is 0, black & white)
;                 Negative color numbers reverse the color tables.
;                 Use -99 for reverse black & white.
;    DIR:         string, when set, the directory for ps file(s) to be stored
;                 default is ps dir under current dir.
;    FNAME:       input, string array, [file,title,xtitle,ytitle,bar_title]
;                 if noread is not used fname(0) is a required wcr file name
;                 When prms is set fname(0) is also required and used for naming
;                 the output ps file. fname is also used for labeling the plots.
;                 Default labels: 'HH','UTC','Range [m]','dBZ'
;                 If fname[0]='' then it assumes that wcrfile exists and uses
;                 it to form the output ps file name
;    HCOR:        when set corrects h channel w.r.t. v channel using 0.95 slope
;    HELP:        show this text
;    IMGORDER:    when set reverse the image order and the x-axis
;    LANDSCAPE:   create PS file in landscape (default is portrait)
;    BIG_ENDIAN:  when set, assumes that the file data are in big endian order.
;                 The default is LITTLE_ENDIAN.(ignored if noread=1)
;    MAXLIM:      float, maximum reflectivity plotted (default  150 dB)
;    MINGATES:    integer, minimum number of range gates to be plotted
;                 If mingates is defined and greater than the actual number of
;                 range gates the image is plotted using mingates (default, 70)
;    MINLIM:      float, minimum reflectivity plotted (default -150 dB)
;    MINMAX:      when set (minmax=1), min and max for every subimage are used
;                 minmax < 0., minmax is used as a fixed min, max is calculated
;                              for every subimage
;                 minmax > 0. (but ne to 1), minmax is used as a fix max,
;                              min is calculated for every subimage
;                 minmax=0,(default, minmax not set), min and max are calculated
;                           for the whole image and used as fixed
;
;    NOREAD:      skip reading wcr file when it is already loadad; for wcr only;
;                 when not set (0), calculates and plots HH reflectivity or
;                 VV reflectivity (if HH is N/A)
;    PRMS:        [aspect,interrupts].  When exist won't use wcr common blocks
;                 and work with inputs Z, r and tsec.
;                 This keyword is intended for non WCR files.  If it is used
;                 for wcr files negative values for aspect will print wcr file
;                 info at the bottom of each page
;                 aspect      - aspect ratio for profile/range scaling
;                               0 will disable scaling.
;                               For negative values: abs(aspect) is used
;                 interrupts  - begin/end profiles for every data on/data off
;                               interrupt during the flight.
;                 Length of prms is 1+2*num_of_interrupts.  If the length
;                 is 1, it is assumed that there are no interrupts.  Zero
;                 as 2nd element of prms is not acceptable.
;    TIMECOR:     if defined gives UTC time correction of the wcr time
;                 After June 1998, IRIG card is used to set the time.
;                 Normal use of the card is to get the time from a master
;                 IRIG card with GPS time.  The default value is set to
;                 the leap seconds in openwcrf.
;    TLABEL:      string, adds title to the param/stat tables
;    VCOR:        when set corrects v channel w.r.t. v channel using default val
;
; OUTPUTS:
;    Z:     fltarr(rangeGates,profs), reflectivity. If NOREAD keyword is
;           not used, Z is an output. HH reflectivity only.
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;       Calling wcrzps without input parameters always must read and load a wcr
;       file.
;       wcr files with small number of range gates (< 50) may cause the pixel
;       size to be too big (because of aspect ratio applied to equally scale
;       the image in range/time axes).  To avoid this problem use mingates
;       keyword set to number of range gates higher than the actual number.
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, September 1996
;       S.H., Aug 99 -- modified help info and other minor changes
;                       added timecor keyword
;       S.H., 12/99  -- Use include statement for wcrdata common blocks
;                       Use readwcrdata to read data if requested
;       S.H., Jan 00 -- added help keyword
;       S.H., Apr 00 -- minor help,print changes
;       S.H., Nov 00 -- change prms to allow wcr info print when prms is used
;                       added mingates keyword
;       S.H., Jan 01 -- added default value for mingates, added fft reflectivity
;                       plot for noread=0, added antpos keyword, added
;                       little_endian keyword and made the routine
;                       independant on the data byteorder in the input file
;       S.H., Feb 01 -- changed routine name from vxizps to wcrzps
;       S.H., Sep 01 -- added antpos=2 for two sidelooking antennas
;       S.H., Apr 02 -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN,
;                       replaced antpos keyword with imgorder
;       S.H., May 02 -- fixed a problem with crossing midnight for timeaxis
;       S.H., Mar 03 -- modified irigflag use
;       S.H., Nov 03 -- replaced gapval with nosignal when padding the last
;                       plot 
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrZps, Z,r,tsec,fname=fname,minmax=minmax,minlim=minlim,maxlim=maxlim, $
            prms=prms,dir=dir,tlabel=tlabel,noread=noread,landscape=land, $
            colortb=colortb,calcoef=calcoef,hcor=hcor,timecor=timecor, $
            imgorder=imgorder,mingates=mingates,big_endian=bgnd,help=help

; WCR data common blocks

@commonwcrinclude

on_error,2

; Define a few constants
  pad=3000       ; for short records pad zeros to preserve aspect ratio
  minsignal=-70. ; dBZ, minimum valid signal (this can be changed if needed)
  maxsignal=150. ; default for maxlim
  nosignal=-150. ; expand the reflectivity image if needed; default for minlim

  if n_elements(mingates) eq 0 then mingates=70

if keyword_set(help) then begin
  doc_library,'wcrzps'
  return
endif

; Create postscript file directory if it does not exist. Use ./ps as default

  if n_elements(dir) eq 0 then dir='./ps/' $
  else if strmid(dir,strlen(dir)-1,1) ne '/' then dir=dir+'/'
  if dir ne '.' and dir ne './' then spawn,'mkdir '+dir+'>& /dev/null'

; Check if this is a request for a wcr file

if (n_elements(prms) eq 0) then begin

  ; Open interactively a new wcr file if requested

  if (n_elements(fname) eq 0 and KEYWORD_SET(noread) eq 0) then $
     fname=dialog_pickfile(/read)

  ; Load wcr file and calibrate hh reflectivity

  if (KEYWORD_SET(noread) eq 0) then begin
     fn=fname[0]
     openwcrf,fn,timecor=timecor,big_endian=bgnd
     fname[0]=wcrfile
     print,'Reading WCR file ...'
     readwcrdata,hcor=hcor,vcor=vcor,big_endian=bgnd,/fftp
     if keyword_set(calcoef) eq 0 then begin
        calcoef=48.
        print,'Using default calibration constant: ',num2str(calcoef)
     endif

   if (where(txsequence eq polh))[0] ne -1 then begin
       if (alg eq fftalg) then vvnoise=hhnoise
       Z=wcrcalcoref([-999,3.],calcoef,wcrstr='hh')
    endif else begin
       if (alg eq fftalg) then hhnoise=vvnoise
       Z=wcrcalcoref([-999,3.],calcoef,wcrstr='vv')
    endelse
  endif

  ; btime=timestr2sec(begtime)>0.01     ; convert Z beginning time to sec
  btime=1 ; aximage plots time axis in hhmmss, no need to add begtime anymore
          ; since the default time is in seconds after midnight
  tsec=time
  profs=numrec
  aspect=aspectratio
  if (n_elements(fname) eq 0) then fname=wcrfile
  printinfo=1

  rr=radrng
  gates=rangegates

endif else begin                      ; non wcr file

  gates=(size(Z))[1]
  profs=(size(Z))[2]
  if prms[0] lt 0 then printinfo=1 else printinfo=0
  aspect=abs(prms(0))

  n=n_elements(prms)
  if (n mod 2 ne 1) then begin
     print,'PRMS has wrong number of elements !'
     return
  endif
  if n gt 1 then begin
     interrupts=intarr((n-1)/2,2)
     l=1
     for k=0,(n-1)/2-1 do begin
         interrupts[k,0]=prms[l]
         interrupts[k,1]=prms[l+1]
         l=l+2
     endfor
  endif
  if (n_elements(fname) eq 0) then begin
     print, 'Keyword FNAME is undefined !'
     return
  endif
  if n_params() lt 3 then begin
     print,'Range (r) and time (tsec) vectors are undefined!'
     return
  endif else rr=r

  btime=0.01

endelse

; Check if tsec is crossing midnight

  tsc=tsec
  junk=abs(max(tsec)-min(tsec))
  if junk gt 70000. then begin
    junk=where(tsec ge max(tsec))
    tsc[junk+1:*]=tsec[junk+1:*]+86400d0
  endif 

; Check if by any chance gates=profs when Z is an input
; If the input image Z is [gates,profs] oriented transpose Zc.

  if n_params() gt 0 then begin
     if (gates eq profs) then begin
        print,' Range Gates = Num Profiles'
        print,' Assume (rangeGates, Profiles) input image!'
     endif
     if (gates eq (size(Z))[1] ) then Zc=transpose(Z) else Zc=Z
  endif else Zc=transpose(Z)

; If minimum number of range gates for plotting is given accommodate them

  if mingates gt gates then begin
     rr=[rr,(findgen(mingates-gates)+1)*(rr[1]-rr[0])+rr[gates-1]]
     Zc=[[Zc],[fltarr(profs,mingates-gates)+nosignal]]
     gates=mingates
  endif

; Determine the color preference

  if (KEYWORD_SET(colortb) eq 0) then begin
     color=0
     revcol=0
  endif else if (colortb eq -99) then begin
     revcol=1
     color=0
  endif else  begin
     if colortb lt 0 then revcol=1 else revcol=0
     color=abs(colortb)
  endelse

; Initialize

  ; Determine the order of the images

  if keyword_set(imgorder) then begin
      rr=reverse(rr)
      yorder=1
  endif else yorder=0

  resetps2x
  setxpswin,99,3,psfile=dir+fname(0)+'.ps',land=land,nocolor=(color eq 0)

  xpix=profs
  ypix=gates

  ; Assign labels

  if n_elements(fname) gt 0 then begin
     k=n_elements(fname)
     if fname[0] eq '' then fname[0]=wcrfile
     if k lt 5 then btitle='dBZ' else btitle=fname(4)
     if k lt 4 then ytitle='Range [m]' else ytitle=fname(3)
     if k lt 3 then begin
       if irigflag[0] eq -2 then xtitle='Count' $
       else if irigflag[0] lt 1 then xtitle='Time' $
       else xtitle='UTC'
     endif else xtitle=fname(2)
     if k lt 2 then title=':  HH' else title=fname(1)
  endif else begin
     title=':  HH'
     if irigflag[0] eq -2 then xtitle='Count' $
     else if irigflag[0] lt 1 then xtitle='Time' $
     else xtitle='UTC'
     ytitle='Range [m]'
     btitle='dBZ'
  endelse
  title=fname[0]+title
  if KEYWORD_SET(land) then begin
     title=[title,btitle]
     bar=2
  endif else begin
     title=[title,'                                   '+btitle]
     bar=1
  endelse
  charsize=1.4

  if n_elements(minlim) eq 0 then minlim= nosignal
  if n_elements(maxlim) eq 0 then maxlim= maxsignal

  nintrrp=(n_elements(interrupts)/2-1)>0

  first=0
  last=(lasti=(xpix-1))
  page=1B
  isize=[0,-1]

  if (n_elements(minmax) eq 0) then begin       ; fixed min/max for all plots
     minZ=min(Zc(where(Zc gt minsignal)))>minlim
     maxZ=max(Zc)<maxlim
  endif else if (n_elements(minmax) eq 2) then begin  ; Assign user's values
     minZ=minmax(0)>minlim
     maxZ=minmax(1)<maxlim
  endif else if (n_elements(minmax) eq 1) then $
     if (minmax(0) gt 0. and minmax(0) ne 1) then maxZ=minmax(0)<maxlim else $
     if (minmax(0) lt 0.) then minZ=minmax(0)>minlim

  gapval=2*max(Zc)*(revcol eq 0) + 2*min(Zc)*(revcol eq 1)


  print,'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
  print,fname(0),' processing ...'

; Loop over the number of interrupts

  print,'Number of interrupts:',nintrrp

for k=0,nintrrp do begin
   true=1B
   graph=0B

   ; Change last and lasti to the current interrupt if any

   if (nintrrp gt 0) then begin
      last=first+interrupts(k,1)-interrupts(k,0)-1
      lasti=last         ; save the last profile before the interrupt
   endif

;  Loop over the segments of the image

   while true do begin

      ; Find min/max for the current segment if requested

      if (n_elements(minmax) eq 1) then begin
         if (minmax(0) lt 0. or minmax(0) eq 1) then $
            maxZ=max(Zc[first:last,*])<maxlim
         if (minmax(0) gt 0.) then begin
            temp=Zc[first:last,*]
            minZ=min(temp(where(temp gt minsignal)))>minlim
         endif
      endif

      print,''
      print,format='("Plotting: Page ",I2,", Graph: ",I2)',page,graph+1
      print,''

      ; Optimize printwcrstat for more than one page

      if page eq 1 then stattext=''

      if isize(0) eq 0 and isize[1] eq -1 then begin
      ;  Determine the right plotting size for the image
      ;  Done once per image, in the beginning

         aximage,Zc[first:last,*],bar=bar,asp=aspect,ch=charsize,isize=isize, $
                /silent
         while (isize[1] lt ypix) do begin
            aximage,temporary([Zc[first:last,*],fltarr(pad,gates)]),$
                    bar=bar, asp=aspect, chars=charsize, isize=isize ,/silent
            pad=pad+3000
         endwhile

      ;  Assign the size of the subimage that preserves the aspect ratio

         last=isize(0)-1
         print,''
         print,'Total number of profiles: ',profs
         print,'Plotted image: profiles= ',num2str(isize[0]), $
               ', range gates= ',num2str(isize[1])
         print,''

      ;  After we determined the size reset the device and start plotting

         resetps2x
         setxpswin,99,3,psfile=dir+fname[0]+'.ps',land=land,color=color, $
                   nocolor=(color eq 0)

         if (last gt lasti) then begin

         ;  Plot the short image or short subimage between two interrupts

            nextplot
            aximage,temporary([Zc[first:lasti,*], $
                    gapval+fltarr(last-lasti,gates)]), $
                    temporary([tsc[first:lasti],tsc[lasti]+ $
                    (tsc[lasti]-tsc[first])/(lasti-first)*  $
                    (findgen(last-lasti)+1)]),rr,asp=aspect,order=yorder, $
                    timeax=btime,min=minZ,max=maxZ,title=title,xtitle=xtitle, $
                    ytitle=ytitle,chars=charsize,bar=bar,revc=revcol,/silent
            if printinfo then begin
               if n_elements(tlabel) gt 0 then $
                  xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
               printwcrprms,2,[0.08,-0.04,0.75]
               printwcrstat, 2,[0.52,-0.04,0.75]
            endif
            datestamp,/user,com='      Page '+num2str(fix(page))
            page=page+1B
            erase
            true=0B
         endif else begin

         ;  Plot first full size subimage

            aximage,Zc[first:last,*],tsc[first:last],rr,bar=bar, $
                    asp=aspect,timeax=btime,min=minZ,max=maxZ,title=title,$
                    xtitle=xtitle,ytitle=ytitle,chars=charsize,revc=revcol,$
                    order=yorder,/silent
         endelse

      endif else $
         if isize[0] le last-first then $

         ;  Plot the current full size subimage

            aximage,Zc[first:last,*],tsc[first:last],rr,bar=bar, $
                    asp=aspect,timeax=btime,min=minZ,max=maxZ,title=title,$
                    xtitle=xtitle,ytitle=ytitle,chars=charsize,revc=revcol,$
                    order=yorder,/silent        $
         else $

         ;  Plot last subimage when it is shorter than full subimage size

            aximage,temporary([Zc[first:last,*], $
                    nosignal+fltarr(isize[0]-last+first,gates)]), $
                    temporary([tsc[first:last],tsc[last]+(tsc[last]+86400d0 $
                    *(tsc[last] lt tsc[first])-tsc[first])/(last-first)* $
                    (findgen(isize[0]-last+first)+1)]),rr,bar=bar, $
                    asp=aspect,revc=revcol,timeax=btime,min=minZ,max=maxZ, $
                    title=title,xtitle=xtitle,ytitle=ytitle, $
                    chars=charsize,order=yorder,/silent

      ;  Make necessary assignments before next graph and/or page

      if true then begin
         if ((graph mod 3) lt 2) then begin
            graph=graph+1B
            first=last-10                     ; overlap 10 profiles
            last=(first+isize[0]) < min([lasti, xpix-1])

         ; If only 50 or less profs left do not bother to plot the rest of
         ; the image.  Print useful info.

            if (first+60 ge xpix-1) or (first+60 gt lasti) then begin
               true=0B
               if printinfo then begin
                  if n_elements(tlabel) gt 0 then $
                     xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
                  printwcrprms,2,[0.08,-0.04,0.75]
                  printwcrstat, 2,[0.52,-0.04,0.75],stattext=stattext
               endif
               datestamp,/user,com='      Page '+num2str(fix(page))
               page=page+1B
               erase
            endif
         endif else begin                             ; end of page (3 graphs)
            if printinfo then begin
               if n_elements(tlabel) gt 0 then $
                  xyouts,0.08,-0.01,tlabel,charsize=0.9,/normal,/noclip
               printwcrprms,2,[0.08,-0.04,0.75]
               printwcrstat, 2,[0.52,-0.04,0.75],stattext=stattext
            endif
            datestamp,/user,com='      Page '+num2str(fix(page))
            page=page+1B
            graph=graph+1
            erase
            first=last-10                     ; overlap 10 profiles
            last=(first+isize[0]) < min([lasti, xpix-1])
            if (first+20 ge xpix-1) or (first+20 gt lasti) then true=0B
         endelse
      endif

   endwhile

   !p.multi(0)=0              ; initialize multi plot @ the top of the page

   ; Assign next starting point to be at the beggining after current interrupt

   if (nintrrp gt 0) then $
      first=total(interrupts[0:k,1]-interrupts[0:k,0])+k+1
endfor

resetps2x

end




