;$Id: wcrzpsm.pro,v 1.8 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRZPSM
;
; PURPOSE:
;       Create reflectivity PS files from one day flight wcr data
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       WCRZPSM, dir
;
; INPUTS:
;       Dir:      string, wcr data file directory; undefined: use pickfile
;
; KEYWORD PARAMETERS:
;     ACONFIG:   input, integer/intarr(n); antenna configuration
;                0 - single side/up looking antenna; default
;                1 - two downlooking antennas (H-ch: down-fore, V-ch: down)
;                2 - two sidelooking antennas (V-ch: side-fore, H-ch: side/up)
;                3 - down and side/up antennas (V-ch: down, H-ch: side/up)
;                4 - two downlooking antennas (V-ch: down, H-ch: down-aft,
;                     this is for C130 DYCOMS01 experiment before H/V channels 
;                     were swapped in march 2002)
;                5 - down and side antennas for NRC-CV580 WCR installation
;
;                Note: dual-antenna configs have no polarimetric capability
;
;     ACSPEED:   input, float; average aircraft speed in m/s (default 90.)
;     CALCOEFF:  fltarr(2) or fltarr(2,n); n=n_elements(ACONFIG)
;                [hhcal,vvcal], cal. coeffs. in dB 
;     COLORTB:   color table number (default is -99, reversed black & white)
;                Negative color numbers reverse the color tables.
;                Use -99 for reverse black & white.
;     FILEID:    file identification string. Common substring from the data
;                file name starting from 2nd char (default 'pp0').
;     HCOR:      when set correct h channel w.r.t. v channel using 0.95 slope
;     HELP:      Show this text
;    BIG_ENDIAN: when set, assumes that the file data are big endian order.
;                The default is LITTLE_ENDIAN.
;     MINGATES:  integer, minimum number of range gates to be plotted
;                If mingates is defined and greater than the actual number of
;                range gates (rangegates) the image is plotted using mingates
;                (default, rangegates)
;     MINLIM:    float, minimum reflectivity plotted (default -150 dB)
;     MAXLIM:    float, maximum reflectivity plotted (default  150 dB)
;     PRINTER:   'lj4', 'lj4_gv', 'qms'
;                undefined: qms
;                '': suppress printing
;     PROC:      byte, define what to plot
;                  0 - produce plots for sppmag and fft (HH or VV); default
;                  1 - produce plots for  ppmag (HH and VV)
;                  2 - produce plots for HH only except in case of sppmag V
;                  3 - produce plots for VV only except in case of sppmag H
;     PSDIR:     string, directory for postscript file(s)
;                Default save ps files in subdir 'ps/', which is created
;                if non-existant.  When dir='select', ps directory is always
;                sub directory of the home directory, otherwise it is sub
;                of dir
;     SETNOISE:  sets noise subtraction; scalar or array
;                1) 0, uses the normal noise channels (averaged for ppmag
;                   mode), smoothing window size is 99, thresh=3*sigma_noise
;                2) 1, uses one noise pulse (matching the channel) with minimum
;                   mean only (for ppmag only, sppmag ignores it)
;                3) 99...999 - smoothing noise window (default is 99)
;                4) 10099 ... 10999 smoothing noise window using one noise 
;                   pulse,matching the channel, with minimum mean (ppmag only) 
;                5) [1)...4),#_sigma_noise], (e.g., [10200,2]-use one pulse
;                   noise channel, noise_window=200, thresh= 2*sigma_noise
;
;     TIMECOR:   if defined gives time correction of the wcr time
;                After June 1998, IRIG card is used to set the time.
;                Normal use of the card is to get the time from a master
;                IRIG card with GPS time.  Thus the default value is
;                the current leap seconds
;     TLABEL:    string, adds title to the param/stat tables; It always add
;                info about the cal const. ('Data (cal: ....)')
;     VCOR:      when set correct v channel w.r.t. v channel using default vals
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, October 1996
;       S.H., Aug 99 -- modified help info and other minor changes
;                       added timecor keyword
;       S.H., 12/99  -- Use include statement for wcrdata common blocks
;                       Use readwcrdata to read data
;       S.H., Jan 00 -- added help keyword
;       S.H., Apr 00 -- added setnoise keyword
;       S.H., Nov 00 -- added mingates keyword
;       S.H., Jan 01 -- minor change in the help text, added aconfig keyword,
;                       added little_endian keyword and made the routine
;                       independant on the data byteorder in the input file
;       S.H., Feb 01 -- minor changes, added colortb keyword
;                       changed routine name from vxizpsm to wcrzpsm
;       S.H., Sep 01 -- added antenna config 3, minor changes in the help text
;       S.H., Apr 02 -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN, changed
;                       aconfig and calcoef; modified TLABEL
;       S.H., Jun 03 -- modified the use of calcoeff to include fixed 
;                       corrections for 100ns and 500ns pulses
;       S.H., Nov 03 -- modified setnoise to match changed noise handling;
;                       added NRC-CV580 to aconfig configurations; added acspeed
;       S.H., Dec 03 -- modified setnoise
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrzpsm, dir,printer=printer,fileID=fileID,tlabel=tlabel,calcoeff=calcoeff,$
    proc=proc,minlim=minlim,maxlim=maxlim,psdir=psdir,timecor=timecor, $
    hcor=hcor,vcor=vcor,help=help,setnoise=setnoise,mingates=mingates, $
    aconfig=aconfig,acspeed=acspeed,colortb=colortb,big_endian=bgnd

@commonwcrinclude

on_error,2

; Check inputs

if keyword_set(help) then begin
  doc_library,'wcrzpsm'
  return
endif

if n_elements(fileID) eq 0 then fileID='200'
if n_elements(tlabel) eq 0 then tlbl='Data' else tlbl=tlabel+' Data'

antflag=0
if n_elements(aconfig) eq 0 then begin ; single antenna, dual position (up/side)
   aconfig=0
   antpos=0
   antstr=['','']
endif else if n_elements(aconfig) eq 1 then begin
   case aconfig[0] of
      0: begin            ; single side/up antenna
           antpos=0
           antstr=['',''] ; [0] - V-ch, [1] - H-ch
         end
      1: begin          ; two downlooking antennas (H-ch: down-fore, V-ch: down)
           antpos=1
           antstr=['down','down_fore']
         end
      2: begin          ; two sidelooking antennas (V-ch: side-fore, H-ch: side/up)
           antpos=0
           antstr=['side-fore','up.side']
         end
      3: begin          ; down and side/up antennas (V-ch: down, H-ch: side/up)
           antflag=1    ; antpos depends on Rx channel; 1 - V, 0 - H
           antstr=['down','up.side']
         end
      4: begin          ; two downlooking antennas (V-ch: down, H-ch: down-aft, C130, V/H swapped)
           antpos=1
           antstr=['down','down-aft']
         end
      5: begin          ; down and side antennas (V-ch: down, H-ch: side)
           antflag=1    ; antpos depends on Rx channel; 1 - V, 0 - H
           antstr=['down','side']
         end
   else: message,'Invalid value for ACONFIG.'
   endcase
endif else begin
   k=where(aconfig lt 0 or aconfig gt 5)
   if k[0] ne -1 then message,'Invalid input for aconfig'
endelse

if n_elements(printer) eq 0 then $
   printer=''   ;  supress printing

if n_elements(dir) eq 0 then $
   flst=dialog_pickfile(/read, filter='*',get_path=dir)

spawn, 'ls '+dir+'?'+fileID+'*',flst
flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,20)
nfiles=(size(flst))(1)-1

if n_elements(psdir) eq 0 then $
   if n_elements(dir) gt 1 then psdir='~/ps/' $
   else psdir=dir+'ps/'
if strmid(psdir,strlen(psdir)-1,1) ne '/' then psdir=psdir+'/'
if psdir ne '.' and psdir ne './' then spawn,'mkdir '+psdir+' >& /dev/null'

ppmnoav=0
nswin=99
nthresh=[-999,3]
if n_elements(setnoise) gt 0 then begin
   if n_elements(setnoise) eq 2 then nthresh=[-999,setnoise[1]]
   nswin=(setnoise[0] mod 10000) > 99
   if setnoise[0] gt 10000 or setnoise[0] eq 1 then ppmnoav=2
endif

if (n_elements(colortb) eq 0) then col=-99 else col=colortb

if n_elements(aconfig) gt 1 and n_elements(aconfig) ne nfiles+1 then $
   message,'Incorrect number of elements for aconfig'

; Process all files

for k=0,nfiles do begin
    fn=flst(k)

    openwcrf,dir+fn,timecor=timecor,acspeed=acspeed,big_endian=bgnd

    print,'+++++++++++++++++++++++++++++++++++++++'
    print,'Reading WCR file (',fn,') ...'

    if (alg eq pp4alg or alg eq pp6alg) then begin
       if n_elements(proc) eq 0 then procz=2 else procz=proc
       print,'PPMAG'
    endif else if (alg eq sppalg) then begin
       procz=2
       print,'SPPMAG'
    endif else if alg eq fftalg then begin
       print,'FFT'
       procz=0
    endif else if alg eq dftalg then begin
       print,'DPFFT'
       procz=0
    endif

    print,'+++++++++++++++++++++++++++++++++++++'

    ; Print noise settings

    if (alg eq fftalg or alg eq dftalg) then begin
       nswin=99
       nthresh=[-999,3.]
       print,'Noise setting (fft/dpfft):'
       print,'   Noise channel= fftnoise
       print,'   Noise window=',num2str(nswin)
       print,'   Noise threshold=',num2str(nthresh,fr=1)
    end else begin
       print,'Noise setting (sppmag/ppmag):'
       print,'   Noise channel=',num2str(ppmnoav,fr=0)
       print,'   Noise window=',num2str(nswin)
       print,'   Noise threshold=',num2str(nthresh,fr=1)
    endelse

    ; Determine the number of profiles to process;
    ; 164mm: plot xsize, 49mm: plot ysize; 3 plots per page, 3 pages

    if n_elements(mingates) eq 0 then gates=(mingates=rangegates) $
    else gates=max([rangegates,mingates])

    profpersec=fix(1./timeinterval)
    segd=long(3.*164./49.*gates*rangegatespacing/kaspeed*profpersec*3.)

    if tnumrec-segd lt 1000 then begin
       seg=tnumrec
       nseg=0
       print,'Total Segments: 1'
       print,''
    endif else begin
       seg=segd
       nseg=tnumrec/seg
       lastseg=tnumrec-nseg*seg
       print,'Total Segs: ',num2str(nseg+1)
       if lastseg le 200 then begin
          nseg=nseg-1
          print,'Last segment (size: ',num2str(lastseg),') ignored.'
       endif
       print,''
    endelse

    ; Prepare aconfig for this file

    if n_elements(aconfig) gt 1 then begin
      calcoef=calcoeff[*,k]
      antflag=0
      case aconfig[k] of
        0: begin            ; single side/up antenna
             antpos=0
             antstr=['',''] ; [0] - V-ch, [1] - H-ch
           end
        1: begin
             antpos=1
             antstr=['down','down_fore']
           end
        2: begin
             antpos=0
             antstr=['side-fore','up.side']
           end
        3: begin
             antflag=1
             antstr=['down','up.side']
           end
        4: begin
             antpos=1
             antstr=['down','down-aft']
           end
        5: begin
             antflag=1
             antstr=['down','side']
           end
      endcase      
    endif else calcoef=calcoeff

    tlab=tlbl+" (cal/250ns+2.5/100ns-3.5/500ns: "+num2str(calcoef[0],fr=1)+","+$
                 num2str(calcoef[1],fr=1)+")"

;   ************************  FFT or DPFFT mode ***************************

    if (alg eq fftalg or alg eq dftalg) then begin

       for ll=0,nseg do begin
          strt = ll*seg < (tnumrec-1)
          stp = ((ll+1)*seg-1) < (tnumrec-1)

          readwcrdata, strt, stp, big_endian=bgnd

          print,'----> Current segment: '+num2str(ll)
          print,'----> Begin Time: ',Begtime
          print,'----> Profiles: '+num2str(tnumrec)+', Processing: '+ $
                       num2str(strt)+' - '+num2str(stp)

          fname=strmid(begtime,0,2)+strmid(begtime,3,2)+strmid(begtime,6,2)

          ; Extract noise and signal power from the spectra and calibrate

          if txsequence[0] eq polv then begin
             if alg eq fftalg then begin
                vv=total(fftps,1)/fftbins^2
                vvnoise=total(fftnoise,1)/fftbins^2
                hhnoise=vvnoise
             endif else begin
                vv=total(reform(fftps[*,*,0,*]),1)/fftbins^2
                vvnoise=total(reform(fftnoise[*,0,*]),1)/fftbins^2
                hhnoise=total(reform(fftnoise[*,1,*]),1)/fftbins^2
             endelse
             junk=calcoef[1]+2.5*(iffilter eq 10)-3.5*(iffilter eq 2)
             Z=wcrcalcoref(nthresh,nswin=nswin,junk,wcrstr='vv')
             fname=[wcrfile+'.'+fname+'.FFT_VV'+antstr[0],' ']
             if antflag then antpos=1
          endif else begin
             if alg eq fftalg then begin
                hh=total(fftps,1)/fftbins^2
                hhnoise=total(fftnoise,1)/fftbins^2
                vvnoise=hhnoise
             endif else begin
                hh=total(reform(fftps[*,*,1,*]),1)/fftbins^2
                vvnoise=total(reform(fftnoise[*,0,*]),1)/fftbins^2
                hhnoise=total(reform(fftnoise[*,1,*]),1)/fftbins^2
             endelse
             junk=calcoef[0]+2.5*(iffilter eq 10)-3.5*(iffilter eq 2)
             Z=wcrcalcoref(nthresh,nswin=nswin,junk,wcrstr='hh')
             fname=[wcrfile+'.'+fname+'.FFT_HH'+antstr[1],' ']
             if antflag then antpos=0
          endelse

          wcrzps,Z,fname=fname,col=col,/noread,dir=psdir,tlab=tlab,$
            /minmax, minlim=minlim,maxlim=maxlim,mingates=mingates,imgord=antpos

          if printer ne '' then $
             spawn,'lp -o nobanner -d '+printer+' '+psdir+fname+'*.ps'
       endfor   ; end for ll

    endif else begin

;   *******************  SPPMAG, PPMAG4 or PPMAG6 mode ********************

       for ll=0,nseg do begin
          strt = ll*seg < (tnumrec-1)
          stp = ((ll+1)*seg-1) < (tnumrec-1)

          readwcrdata, strt, stp, ppmnoav=ppmnoav, big_endian=bgnd

          print,'----> Current segment: '+num2str(ll)
          print,'----> Begin Time: ',Begtime
          print,'----> Profiles: '+num2str(tnumrec)+', Processing: '+ $
                       num2str(strt)+' - '+num2str(stp)

          if keyword_set(hcor) then $
             wcrhcor,what=['hhnoise','hh'],/silent  ; h-correction
          if keyword_set(vcor) then $
             wcrvcor,what=['vvnoise','vv'],/silent  ; v-correction

          fname=strmid(begtime,0,2)+strmid(begtime,3,2)+strmid(begtime,6,2)

          if procz eq 2 then begin
            if alg eq sppalg and txsequence(0) eq polv then begin
               s='vv' &  ants=antstr[0] & if antflag then antpos=1
            endif else begin
               s='hh' & ants=antstr[1] & if antflag then antpos=0
            endelse
            junk=calcoef[0]*(s eq 'hh')+calcoef[1]*(s eq 'vv')+ $
                 2.5*(iffilter eq 10)-3.5*(iffilter eq 2)
            Z=wcrcalcoref(nthresh,nswin=nswin,junk,wcrstr=s)
            fname=[wcrfile+'.'+fname+'.'+strupcase(s)+ants,' ']
            wcrzps,Z,fname=fname,col=col,/noread,dir=psdir,tlab=tlab,$
             /minmax,minlim=minlim,maxlim=maxlim,mingates=mingates,imgord=antpos
          endif else if procz eq 3 then begin
            if alg eq sppalg and txsequence(0) eq polh then begin
               s='hh' &  ants=antstr[1] & if antflag eq 3 then antpos=0
            endif else begin
               s='vv' &  ants=antstr[0] & if antflag then antpos=1
            endelse
            junk=calcoef[0]*(s eq 'hh')+calcoef[1]*(s eq 'vv')+ $
                 2.5*(iffilter eq 10)-3.5*(iffilter eq 2)
            Z=wcrcalcoref(nthresh,nswin=nswin,junk,wcrstr=s)
            fname=[wcrfile+'.'+fname+'.'+strupcase(s)+ants,' ']
            wcrzps,Z,fname=fname,col=col,/noread,dir=psdir,tlab=tlab,$
             /minmax,minlim=minlim,maxlim=maxlim,mingates=mingates,imgord=antpos
          endif else begin
            for l=0,procz do begin
              if (l eq 0) then begin
                 if (where(txsequence eq polh))[0] ne -1 then begin
                    junk=calcoef[0]+2.5*(iffilter eq 10)-3.5*(iffilter eq 2)
                    Z=wcrcalcoref(nthresh,nswin=nswin,junk,wcrstr='hh')
                    fnameold=fname
                    fname=[wcrfile+'.'+fname+'.HH'+antstr[1],' ']
                    if antflag then antpos=0
                 endif else $
                 if (where(txsequence eq polv))[0] ne -1 then begin
                    junk=calcoef[1]+2.5*(iffilter eq 10)-3.5*(iffilter eq 2)
                    Z=wcrcalcoref(nthresh,nswin=nswin,junk,wcrstr='vv')
                    fnameold=fname
                    fname=[wcrfile+'.'+fname+'.VV'+antstr[0],' ']
                    if antflag then antpos=1
                 endif
              endif else begin
                 if (where(txsequence eq polv))[0] ne -1 then begin
                    junk=calcoef[1]+2.5*(iffilter eq 10)-3.5*(iffilter eq 2)
                    Z=wcrcalcoref(nthresh,nswin=nswin,junk,wcrstr='vv')
                    fname=[wcrfile+'.'+fnameold+'.VV'+antstr[0],' ']
                    if antflag then antpos=1
                 endif else $
                 if (where(txsequence eq polh))[0] ne -1 then begin
                    junk=calcoef[0]+2.5*(iffilter eq 10)-3.5*(iffilter eq 2)
                    Z=wcrcalcoref(nthresh,nswin=nswin,junk,wcrstr='hh')
                    fname=[wcrfile+'.'+fnameold+'.HH'+antstr[1],' ']
                    if antflag then antpos=0
                 endif
              endelse

              wcrzps,Z,fname=fname,col=col,/noread,dir=psdir,tlab=tlab,$
                 /minmax, minlim=minlim,maxlim=maxlim,mingates=mingates, $
                 imgord=antpos
            endfor   ;  for l
          endelse
          if printer ne '' then $
             spawn,'lp -o nobanner -d '+printer+' '+psdir+fname+'*.ps'
          hcorrection=intarr(8)
          vcorrection=intarr(8)
       endfor   ; end for ll

    endelse
    closewcrf

endfor   ; end for k

end
