;$Id: writewcrraw.pro,v 1.8 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WRITEWCRRAW
;
; PURPOSE:
;       Write a WCR file from raw data loaded into the structure 
;       Recraw(readwcrraw). If Recraw is not defined, first load the input data
;       file (FNIN) and then write the output file (FNOUT). 
;       
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       WRITEWCRRAW [, Recraw, Fileheader]
;
; INPUTS:
;       Recraw:  Nr-element array of structures with raw WCR data 
;                previously read with readwcrraw.  Structure format:
;                {header: lonarr(headervars),data:fltarr(Nch,Nn+Ntx+Nrg,Nmode)}
;                headervars= 25 after Oct 1996 and 40 after May 1999
;                Nch   = 2  ; real & imag for pulse pairs and V & H 
;                             for magnitudes ; SPPMAG and PPMAG modes
;                        fftbins ; for FFT mode
;                Nn    = noise samples (always 1 before May 1999)
;                Ntx   = 1, Tx pulse leakage; internal cal. pulse (before 1997)
;                Nrg   = # of range gates
;                Nmode =  2 - for SPPMAG
;                         7 - for PPMAG4
;                        11 - for PPMAG6
;                         1 - for FFT
;                Nr    = # of records read (max Nr=tnumrec)
;       Fileheader: structure with the file header fields 
;                   (output from readwcraw)
;                   For data files before April 2002 this input is ignored
;
; KEYWORD PARAMETERS:
;       FNIN:     string, input file name (default is wcrfile in the current
;                 directory). If recraw and fnout exists fnin is ignored.
;                 If recraw exists and fnout is not given, fnin is used to
;                 form fnout
;                 use 'load' to get a file selction;
;       FNOUT:    string, output file name (default fnin, where first char.
;                 is substituted with 'S'and saved in the current directory)    
;                 When TIMECOR is greater than 0 and there is no LSHIFT
;                 the default output file name first char is 'T'
;                 If TIMECOR and LSHIFT are set 1st char. of the default output 
;                 file name is 'R'
;       LSHIFT:   shifts all range gates one position to the left
;                 This resolves a bug in wcr data acquisition system,
;                 which happens intermittently
;                 0 - no shift (default)
;                 1 - shift all profiles
;                 2 - shift even profiles
;                 3 - shift odd profiles
; 
;       BIG_ENDIAN: when set, assumes big endian order for the input file and
;                   write output file in big endian order. The default is
;                   LITTLE_ENDIAN. Thus the byte order of the input file is 
;                   preserved in the output file regardless of the computer
;                   architecture running the current idl session.
;                      
;       STARTREC: long, start record (0 to tnumrec-2, default, 0)
;       STOPREC:  long, stop record (1 to tnumrec-1, default, Nr-1)
;       TIMECOR:  long/float, time correction 
;                   - long, abs(timecor) >= 1, in seconds 
;                   - float, 0. < timecor < 1., time will be raplaced with 
;                     a counter starting with 0 and step equal to timecor
;                     IRIG flag (irigflag/fileheader.irigstatus) is set to -2
;       HELP:     show this text
;
; OUTPUTS:
;       Recraw:  n-element array of structures, which was written into 
;                the output file; n=Nr or STOPREC-STARTREC+1
;
; SIDE EFFECTS:
;       Creates a new file containing data given in recraw or read from fnin. 
;       This routine will not work with WCR files before Oct. 1996.
;
; NOTES:
;       For very large files it is more efficient to let writewcrraw
;       read the data from the input file rather than reading it in 
;       using readwcrraw first.
;
; EXAMPLE:
;       Following example will read in Wpp00-03-29-21-35-38, shift the range 
;       gates one range gate to the left and write out Spp00-03-29-21-35-38
;
;       writewcrraw,fnin='Wpp00-03-29-21-35-38',/lshift
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Nov 1999
;       S.H., Jan 00 -- added help keyword
;       S.H., Mar 00 -- fixed a typo, added an example in the help
;       S.H., Apr 00 -- added TIMECOR keyword and status print for large files
;       S.H., Jan 01 -- added LITTLE_ENDIAN keyword
;       S.H., Feb 01 -- changed routine name from writevxiraw to writewcrraw
;       S.H., Jul 01 -- fixed a bug with input file directory path
;       S.H., Apr 02 -- modified to work with the new data format and replaced
;                       LITTLE_ENDIAN keyword with BIG_ENDIAN, added fileheader
;                       input parameter
;       S.H., May 02 -- modified TIMECOR to accomodate a complete replacement
;                       of the time vector with a counter
;       S.H., Jun 03 -- few minor changes
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro writewcrraw, recraw,fileheader, startrec=startrec, stoprec=stoprec, $
                 lshift=lshift, fnin=fnin, fnout=fnout, timecor=timecor,  $ 
                 big_endian=bgnd, help=help
				 
@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'writewcrraw'
  return
endif

recmax = 50000L  ; maximum number of records to be read and/or shifted 
                 ; simultaneously; must be even number (for simplicity) 

; Check the input

  if n_params() eq 0 then readflag=1 else readflag=0
  if n_elements(lshift) eq 0 then lshift=0
  lshift=fix(lshift)
  if lshift lt 0 or lshift gt 3 then begin
     message,'Invalid LSHIFT keyword value.',/info
     return
  endif

  if n_elements(timecor) eq 0 then timecor=0

  if abs(timecor) ge 1 then begin
    timecor=long(timecor)
    counter=0
  endif else if timecor gt 0 then begin
    timecor=abs(timecor)
    counter=1
  endif else counter=0
  
  if readflag then begin
     if n_elements(fnin) eq 0 then fnin=wcrfile $
     else if fnin eq 'load' then fnin=dialog_pickfile(/read, filter='?pp??-*') 
     if fnin eq '' then return
     fnin1=strmid(fnin,strlen(fnin)-20) 
  endif else begin
     if n_elements(fnout) eq 0 then begin
        if n_elements(fnin) eq 0 then fnin=wcrfile
	   fnin1=fnin
	endif   
  endelse 	     

  if n_elements(fnout) eq 0 then                $
     if lshift gt 0 and timecor ne 0 then       $
        fnout='R'+strmid(fnin1,1,strlen(fnin1)-1) $
     else if lshift eq 0 and timecor ne 0 then  $
        fnout='T'+strmid(fnin1,1,strlen(fnin1)-1) $
     else                                       $
        fnout='S'+strmid(fnin1,1,strlen(fnin1)-1)

; Check byte order for file data and this computer architecture

  swapbytes = 256L   ; 256 is default for no data swapping

  if keyword_set(bgnd) then byteorder,swapbytes,/swap_if_little_endian $
  else byteorder,swapbytes,/swap_if_big_endian

; Read/Write raw data

  if readflag eq 0 then begin             ; If recraw is given process it

    if recraw[0].header[1] gt 65537  then begin  ; file format after March 2002
      tind=[1,2]
      if n_params() ne 2 then message,'No input for fileheader.'
      if size(fileheader,/type) ne 8 then message,'Invalid input for fileheader.'

      ; If replacing time with a counter update IRIG status flag
      if counter eq 1 then fileheader.irigstatus=-2

    endif else tind=[3,4]

    n=n_elements(recraw)

    ; Determine which records to process

    if n_elements(startrec) eq 0 then startrec=0 $
    else if startrec lt 0 or startrec gt n-2 then begin
       message,'Invalid start record.',/info
       return
    endif 
    if n_elements(stoprec) eq 0 then stoprec=n-1 $
    else if stoprec le startrec or stoprec gt n-1 then begin
       message,'Invalid stop record.',/info
       return
    endif
	
    recraw=recraw[startrec:stoprec]
    n=n_elements(recraw)

    ; Shift data and correct time if requested

    if lshift gt 0 then begin
       if timecor ne 0 and counter eq 0 then $
          print,fnout+': time stamps corrected by '+num2str(timecor)+'s' $
       else if timecor ne 0 and counter gt 0 then $
          print,fnout+': time stamps replaced by counter w/ step: '+num2str(timecor)+'s'
       print,fnout+': shift range gates one gate to the left'    
       ind=lindgen(n/(lshift<2)+(n mod 2)*(lshift eq 2))*(lshift<2)+ $
           ((lshift>2) mod 2)
       if n gt recmax then begin
          nseg=n/recmax
          print,format='($,A)','Processing '+num2str(nseg+1+(seg gt 0))+ $
          ' segments: '
          for k=0L,nseg-1 do begin
            print,format='($,A)',num2str(k+1)+' '
            k1=k*recmax & k2=k1+recmax-1
            temp=recraw[ind[k1:k2]].data
            temp=shift(temp,0,-1,0,0)
            recraw[ind[k1:k2]].data=temp
            if timecor ne 0 then begin
              if counter eq 0 then $
                recraw.header[tind[0]]=recraw.header[tind[0]]+timecor $
              else begin
                junk=(double(k1)+dindgen(k2-k1+1))*double(timecor)
                recraw.header[tind[0]]=long(junk)
                recraw.header[tind[1]]=round((junk mod 1)*1.e6)
              endelse
            endif  
          endfor
          seg=n mod recmax
          if seg ne 0 then begin
            print,format='($,A)',num2str(nseg+1)+'.'
            k1=nseg*recmax
            k2=k1+seg-1
            temp=recraw[ind[k1:k2]].data
            temp=shift(temp,0,-1,0,0)
            recraw[ind[k1:k2]].data=temp
            if timecor ne 0 then begin
              if counter eq 0 then $
                recraw.header[tind[0]]=recraw.header[tind[0]]+timecor $
              else begin
                junk=(double(k1)+dindgen(k2-k1+1))*double(timecor)
                recraw.header[tind[0]]=long(junk)
                recraw.header[tind[1]]=round((junk mod 1)*1.e6)
              endelse
            endif  
          endif
          print,""		
       endif else begin
          recraw[ind].data=shift(recraw[ind].data,0,-1,0,0)
          if timecor ne 0 then begin
            if counter eq 0 then $
              recraw.header[tind[0]]=recraw.header[tind[0]]+timecor $
            else begin
              junk=dindgen(n)*double(timecor)
              recraw.header[tind[0]]=long(junk)
              recraw.header[tind[1]]=round((junk mod 1)*1.e6)
            endelse
          endif 
       endelse
    endif

    ; Swap bytes if needed
	
	if swapbytes eq 1 then recraw=swap_endian(recraw)

    ; Write data to the new file

    get_lun, lun
    openw, lun,fnout
    if n_elements(fileheader) ne 0 then writeu,lun,fileheader
    writeu,lun,recraw
 

  endif else begin               ; Read input file, process and write it out

    openwcrf,fnin,big_end=bgnd   ; Load main WCR file parameters

    if wcrdate gt 20020331 then tind=[1,2] else tind=[3,4]

    ; Dertermine the start/stop profile
	
    if n_elements(startrec) eq 0 then startrec=0 $
    else if startrec lt 0 or startrec gt tnumrec-2 then begin
       message,'Invalid start record.',/info
       return
    endif 

    if n_elements(stoprec) eq 0 then stoprec=tnumrec-1 $
    else if stoprec le startrec or stoprec gt tnumrec-1 then begin
       message,'Invalid stop record.',/info
       return
    endif

    if lshift gt 0 then $
      if lshift eq 1 then $
        print,fnout+': shift range gates one gate to the left for all profiles' $
      else if lshift eq 2 then $
        print,fnout+': shift range gates one gate to the left for even profiles' $
      else if lshift eq 3 then $
        print,fnout+': shift range gates one gate to the left for odd profiles'
    if timecor ne 0 and counter eq 0 then $
      print,fnout+': time corrected by '+num2str(timecor)+'s' $
    else if timecor lt 1 and counter eq 1 then $
      print,fnout+': time stamps replaced by counter w/ step: '+num2str(timecor)+'s'

    n   =stoprec-startrec+1  ; number of profiles to process 
    nseg=n/recmax            ; number of segments for read/write
    seg =n mod recmax        ; left over
	
    if nseg lt 1 or (nseg eq 1 and seg eq 0) then begin  ; one segment only

      readwcrraw,recraw,startrec,stoprec,fileh=fileheader,big_end=bgnd,/closewcr

      ; If replacing time with a counter update IRIG status flag

      if counter eq 1 and wcrdate gt 20020331 then fileheader.irigstatus=-2

      if lshift gt 0 then begin    
        ind=lindgen(n/(lshift<2)+(n mod 2)*(lshift eq 2))*(lshift<2)+ $
            ((lshift>2) mod 2)	  
        recraw[ind].data=shift(recraw[ind].data,0,-1,0,0)
      endif

      if timecor ne 0 then begin
        if counter eq 0 then $
          recraw.header[tind[0]]=recraw.header[tind[0]]+timecor $
        else begin
          junk=dindgen(n)*double(timecor)
          recraw.header[tind[0]]=long(junk)
          recraw.header[tind[1]]=round((junk mod 1)*1.e6)
        endelse
      endif 

    ; Swap bytes if needed and write the output file
	
      if swapbytes eq 1 then recraw=swap_endian(recraw)

      get_lun, lun
      openw, lun,fnout
      if n_elements(fileheader) ne 0 then writeu,lun,fileheader
      writeu,lun,recraw 

    endif else begin                     ; multiple segments	
      get_lun, lun                         
      openw, lun,fnout                   ; open output file
      print,format='($,A)','Processing '+num2str(nseg+(seg gt 0))+ ' segments: '

      for k=0L,nseg-1 do begin       ; read/write recmax data segments
        print,format='($,A)',num2str(k+1)+' '
        k1=startrec+k*recmax
        k2=(k1+recmax-1)

        readwcrraw,recraw,k1,k2,fileh=fileheader,big_end=bgnd ; read input file segment

        ; Write the file header if any
        if n_elements(fileheader) ne 0 and k eq 0 then begin
          ; If replacing time with a counter update IRIG status flag
          if counter then fileheader.irigstatus=-2

          writeu,lun,fileheader
        endif

        if lshift gt 0 then begin        ; shift the data in the segment
	    ind=lindgen(recmax/(lshift<2))*(lshift<2)+((lshift>2) mod 2)
            recraw[ind].data=shift(recraw[ind].data,0,-1,0,0)
        endif

        if timecor ne 0 then begin
          if counter eq 0 then $
            recraw.header[tind[0]]=recraw.header[tind[0]]+timecor $
          else begin
             junk=(double(k1)+dindgen(k2-k1+1))*double(timecor)
            recraw.header[tind[0]]=long(junk)
            recraw.header[tind[1]]=round((junk mod 1)*1.e6)
          endelse
        endif

        if swapbytes eq 1 then recraw=swap_endian(recraw)

        writeu,lun,recraw                ; append the segment to the output file
      endfor       

      if seg gt 0 then begin             ; process the left over if any
        print,format='($,A)',num2str(nseg+1)+'.'
        k1=k2+1 & k2=stoprec
        readwcrraw,recraw,k1,k2,fileh=fileheader,big_end=bgnd,/closewcr

        if lshift gt 0 then begin        ; shift the data in the segment
           ind=lindgen(seg/(lshift<2)+(seg mod 2)*(lshift eq 2))*(lshift<2)+ $ 
               ((lshift>2) mod 2)	  
           recraw[ind].data=shift(recraw[ind].data,0,-1,0,0)
        endif

        if timecor ne 0 then begin
          if counter eq 0 then $
            recraw.header[tind[0]]=recraw.header[tind[0]]+timecor $
          else begin
            junk=(double(k1)+dindgen(k2-k1+1))*double(timecor)
            recraw.header[tind[0]]=long(junk)
            recraw.header[tind[1]]=round((junk mod 1)*1.e6)
          endelse
        endif

        if swapbytes eq 1 then recraw=swap_endian(recraw)

        writeu,lun,recraw
      endif else closewcrf
      print,""		
    endelse
  endelse

  free_lun, lun                          ; close output file

end
