;$Id: wcrdisplay.pro,v 2.8 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;    WCRDISPLAY
;   (set of routines; initial compilation is NOT required)
;
; PURPOSE:
;
;    WCR Data Display (GUI is a blocking widget)
;
;    It uses a raw WCR data file (pre-recorded or currently being written) 
;    with a minimum size set to 100 KB. 
;
;    When DOPCOR keyword is set and/or DVC button is not OFF a netdcf
;    aircraft data file is also required
;
; CATEGORY:
;    wcrtools
;
; CALLING SEQUENCE:
;
;    wcrdisplay, aconfig,acspeed=acspeed,calval=calval,colortable=colortable,$
;                        dopcor=dopcor,minmax=minmax,start=strtp, $
;                        timesync=timesync,help=help
;
; INPUT:
;
;    aconfig - input,integer, WCR antenna configurations
;               0 - side/up antenna, dual pol (H and V channels); default
;               1 - two downlooking antennas(H-ch: 30 deg down-fore, V-ch: down)
;               2 - side, side-slant ant.(H-ch: side, V-ch: 35 deg side-fore)
;               3 - reserved (to be used for NCAR C130 configuration)
;               4 - side/up and down antennas (H-ch: side/up, V-ch: down)
;               5 - up and side-slant (H-ch: up, V-ch: side-slant)
;
; KEYWORDS:
;
;    ACSPEED:    input,float, average A/C(aircraft) speed in m/s (default 90 m/s)
;    CALVAL:     input, fltarr, [hhcal,vvcal,iso] (default [49,49,-18])
;                (defaults: hhcal value is for side/up antenna H-pol;
;                           vvcal value is for side/up V-pol or nadir antennas)
;    COLORTABLE: input,integer, color table number(default is 45-Wyo dual table;
;                when less than 64 colors are available the default is 41 Wyo
;                table); if less than 46 tables color table 13 - Rainbow is used
;                To leave the current color table enter negative number
;    DOPCOR:     when set calculate corrected Doppler velocity; requires A/C
;                data file with ins variables(currently, netcdf UW KingAir file)
;                Notes: When dopcor is set negative, a gps correction to the
;                       A/C ground velocities is applied
;    MINMAX:     input,fltarr[min_hh/vv,max_hh/vv,min_zdr,max_zdr,min_ldr,
;                max_ldr], default: [-45,10,-4,4,-25,-10]
;                if a pair has min>=max, default values are used
;    START:      input,integer, starting wcr record number
;    TIMESYNC:   input,float, positive (WCR time lags) or negative known time 
;                shift (in second) between WCR clock and A/C clock. This should 
;                not be confused with GPS leap seconds, which value is read from 
;                the A/C data file or the default value (currently -13 sec) is used.
;    HELP:       display this help text
;
; WIDGET CONTROLS:
;
;    ----------------- Top Buttons Control -----------------------
;
;    New:          Open a dialog_pickfile to select a new WCR file.
;                  Attn:  If DOPCOR keyword is set a 2nd dialog_pickfile
;                         will open to select the A/C file
;    DVC *:        Drop-list menu handling Doppler velocity correction for A/C
;                  motion
;        *          OFF - no correction
;                   ON  - correction applied
;                   GPS - correction applied including GPS correction to A/C
;                         ground velocity
;                  Note:  Positive Doppler velocity is toward the radar. There
;                         is one exception to this rule for down-up antenas
;                         configuration(4) simultaneously displaying velocity 
;                         for both nadir and up antennas on both windows.In this
;                         case nadir velocity is displayed in reversed color and 
;                         positive is away from the radar.  This is done to 
;                         maintain visual continuity between the two velocity 
;                         images.
;
;    Dsply 2 Img:  Pressing this button will toggle between 1 and 2 image widget
;
;    Pause/Cursor: Pressing this button will pause the display.  It also allows
;                  to extract image values by clicking the left button of the
;                  mouse at a desired pixel of the images.  In case of two
;                  images, clicking on either image will extract the
;                  corresponding value from the other image.  The values and
;                  their time stamps are printed in the status line.
;    Continue:     Continue label replaces Pause/Cursor label when Pause is
;                  active.  Pressing Continue will restore updating the display
;                  with new data.
;
;    Catch:        Pressing this button will force the display to position
;                  at the end of the currently available data in the wcr file.
;                  This feature is useful if the display get behind the
;                  incoming data (this could happen for various reasons,
;                  including using Pause/Cursor)
;
;    Color:        Allow change in the color table being used.  The default
;                  color table is a custom dual-table to allow different colors
;                  for, e.g., reflectivity and velocity images.  If the default
;                  color table is replaced the program checks if a narrow black
;                  strip exists in the middle of the new color table. If it does
;                  the color table splits in two otherwise the same color
;                  table is used for both images
;
;    Quit:         Terminates the program without destroying the widget.
;                  This could be used to preserve interesting images for
;                  later review and notes.
;
;    Destroy:      Terminates the program and destroys the widget
;
;    Help:         Shows Widget Control Help text
;
;    ----------------- Second row widget Control -----------------------
;
;    Antennae:     Drop-list menu for the antenna configuration deployed:
;                    U/S     - one up/side antenna, dual polarization
;                    D,D_f   - down-fore(H) and nadir(V) pointing antenae
;                    S,S_f   - up/side(H) and side-fore(V) antennae
;                    U/S,D   - up/side(H) and nadir(V) antennae
;                    U,S_f   - up(H) and side-fore(V) antennae
;                  When selecting one of the antenna configurations the image
;                  order is adjusted accordingly and if DOPCOR option is set
;                  applies the appropriate antenna beam coordinates
;
;    Doppler:      Drop-lits menu for Doppler velocity processing (dopcor
;                  keyword must be set):
;                    dc_H,dc_V - corrected for A/C motion (no gps correction)
;                    du_H,du_V - uncorrected for A/C motion
;                    dc_H,du_H - corr. and uncorr. Dopp. for H-beam only
;                    dc_V,du_V - corr. and uncorr. Dopp. for V-beam only
;                  'c' is replaced by 'g' if dopcor=-1 (applied gps correction)
;
;    Synch:        If DOPCOR keyword is used this editable field will show up.
;                  The default value is 0.0 sec and represent any mismatch
;                  between A/C and WCR clocks.  If such mismatch exists and is
;                  known it can be entered as a positive or negative value.
;                  The time synch correction is applied to the wcr clock.
;                  This should not be confused with GPS leap seconds, which
;                  are read from the A/C data file or the default value
;                  (currently -13 sec) and applied to wcr data independently.
;                  Entering 999 will restore the default value or the value
;                  defined by timesync keyword.
;
;    InitTime:     Editable field.  Contains the starting time of wcr data.
;                  Editing the field will cause the display to start again
;                  from time close to the entered time.  This field should
;                  not be used during data acquisition.  It is intended
;                  for browsing of already recorded files.  The new time must
;                  have one of the following formats: hhmm, hh:mm, hhmmss,
;                  hh:mm:ss, 9999 . Incorrect entries are ignored and the start
;                  time is redisplayed. If enter 9999 the field will show the
;                  end time of the file.
;
;    AvProf:       Editable field. Number of raw profiles averaged before
;                  display.  The default is aspect ratio which scales range:time
;                  to 1:1 based on acspeed (default 90 m/s).  Must be greater
;                  than 0.  If input is 0 the default (aspect ratio) is used.
;                  DispStep and AveProfs cannot be equal to 1 simultaneously.
;                  In this case the second one to be changed to one will be
;                  replaced automatically by 2.
;
;    DspStp:       Editable field. Number of profiles to be calibrated and
;                  displayed simultaneously.  Must be greater than 0. Non-valid
;                  entries are ignored and the default value is used.
;
;    Sleep:        Editable field. Sleep time[s] between disk access attempts 
;                  for 'Empty/short file' or 'End of file. Waiting for data...'
;                  conditions. Default is calculated based on DspStp, AvProf
;                  and an estimate of the time interval between profiles.
;                  This field may be used in real-time display mode to reduce 
;                  hard disk access and SCSI load 
;
;    hhcal:        Editable field. Calibration constant (dB) for H-channel
;                  (default value is for side/up antenna H-pol)
;    vvcal:        Editable field. Calibration constant (dB) for V-channel
;                  (default value is for side/up V-pol or nadir antennas)
;    isol:         Editable field. Isolation (dB) between H- and V-channel
;
;    ----------------- Image Selection Control -----------------------
;
;    Radar products:  availability depends on the wcr operational mode and
;    antenna configuration.
;
;       hh:   co-reflectivity for H-ch/beam (dBZ)
;       vv:   co-reflectivity for V-ch/beam (dBZ)
;       vh:   cross-reflectivity; H-ch transmitted, V-ch received
;       ldr:  vh/hh or hv/vv in dB
;       zdr:  hh/vv in dB
;       d*_h: mean Doppler velocity in m/s for H-channel
;       d*_v: mean Doppler velocity in m/s for V-channel
;       '*'   is 'u' for uncorrected velocity; 'c' for A/C motion corrected
;             velocity and 'g' for A/C motion corrected using GPS corrections
;       sv:   spectral width in m/s (for wcr FFT mode only)
;
;    Color bar limits: these are editable fields and the default values
;                      can be replaced.  To activate the new entry 'Enter'
;                      key must be hit after typing the new number.  There is
;                      no check for the validity of the new limits.
;                      Non-numerical entries are replaced with 0 and a
;                      non-fatal error message is issued
;
;    ----------------- Status Display/Control -----------------------
;
;    Status:  Non editable infomation message
;    <Hn>:    Non editable information indicator for the averaged H-ch noise.
;    mxHn:    Resetable indicator for the max <Hn> noise.
;    <Vn>:    Non editable information indicator for the averaged V-ch noise.
;    mxVn:    Resetable indicator for the max <Vn> noise.
;    Hleak:   Non editable information indicator for the leaked H-ch Tx power
;             WCR Tx leaked power (in dBm) at 0 range.  This indicator is used
;             to help detect data acquisition bug which causes a shift in the
;             range gates.  In this case the leak power will be close to the
;             noise level (around -65 dBm)
;    Vleak:   Non editable information indicator for the leaked V-ch Tx power.
;             WCR Tx leaked power (in dBm) at 0 range. This indicator can be
;             used together with Hleak to help detect a problem with Tx power
;             for H or V channel.
;    mxdT:    Resetable maximum average time interval (in ms) between profiles;
;             If 9999 is displayed a negative time jump in the time stamps was
;             encountered.
;
;    -----------------------------------------------------------------
;
; WIDGET CONTROL NOTE: 'Enter' from the keyboard must be used after modifying an
;                       editable field for the new value to take effect
;
; NOTES: 1) There is a positive side effect of using PICKFILE CANCEL button.
;           It can restore an interrupted display of a file if a new file could
;           not be open due to no data.
;        2) Pause/Cursor button pauses the display and gives control to the
;           mouse for image cursor trace. Clicking left mouse button while over
;           image1 or image2 will show image time and value in the status line.
;        3) The displayed time is UTC time (after May 1999)
;        4) External (custom) routines used by wcrdisplay: readse,mean,wyoctind,
;           sec2timestr,timestr2sec,num2str,stradj,ncdf_varload,ncdf_attload
;           from idltools library, spc from jhuapl library, and ac2gt from
;           wcrtools library
;
; ATTN: This browser is tailored to UW KingAir netcdf data files and thus
;       the names of the needed A/C variables are hardwired into the program
;
;-
;
; MODIFICATION HISTORY:
;
;    Written by Samuel Haimov, March 2001 (first release November 1996, upgrade
;                                          from wcrdisp.pro)
;    SH, Mar 01   --  added help button to GUI; fixed a bug in wcrdispread
;                     when initilizing recraw; added update avestep control
;                     display field; fixed a bug in fft mode when loopstep
;                     is changed; added input check for nimage (invalid input
;                     defaults to 2); modified colorbar min value position;
;                     changed help texts; workaround for a bug in draw_widget
;                     under MS Windows; minor bug fixed
;    SH, Jun 01   --  aspectratio min value set to 1; added acspeed and
;                     imgorder keywords
;    SH, Jul 01   --  fixed cursor readings when imgorder is set
;    SH, Jul 01   --  added skipprof keyword and skipprof widget field
;    SH, Aug 01   --  fixed y-axis for rangegates<minimgsz and imgorder set to 1
;    SH, Oct 01   --  replaced atan(pp) with atan(imaginary(pp),float(pp)),
;                     added ncfdir, undocumented use of dopcor(velcori,gpsacc,
;                     uerrid,verrid,uerr,verr); in common wcrdispp replaced
;                     lambda with ppind
;    SH, Nov 01   --  removed partly (idl5.5 or later) the workaround for the
;                     cursor display during pause and replaced it with 'empty'
;                     command; added work around for a memory leak problem in
;                     the 'pause/cursor' section under Linux
;    SH, Apr 02   --  modified to work with the new data format; took care of
;                     averaging multiple noise samples (noisesamples >1); added
;                     ImgOrd and Img buttons, added noise monitor, and a few
;                     other changes
;    SH, May 02   --  added profile time interval monitor; removed
;                     IMGORD keyword, added Order buttons to image windows,
;                     fixed limits sync between the 2 images; removed skipprof
;                     and added timeout editable field; increased visible
;                     windows to 450 pixels, added Leak monitor fields for H and
;                     V channels
;    SH, June 02  --  major revision (2.0); modified Doppler correction,
;                     aconfig and dopcor to work with the new antenna
;                     configurations,removed TIMEOUT keyword, removed nimage
;                     input param and changed ACONFIG from keyword to input
;                     param.; added display for H- and V-Doppler velocities;
;                     added range value in status display for pause/cursor;
;                     renamed all common blocks to avoid duplicates with
;                     wcrdisp; removed dispctrl keyword
;    SH, July 02  --  added DVC drop-list; dopcor is applied to H and V pol;
;                     modify Doppler velocity sign handling for the single case
;                     of up-down antenna configuration,added keyword order to  
;                     wcrdispbar, many minor changes(ver 2.2-2.3)
;    SH, Nov 02   --  Fixed Synch filed sensitivity when turning DVC to ON;
;                     some other minor modifications; changed default value
;                     for sleep button (it now depends on AvProf and DspStp
;    SH, Feb 03   --  Renamed routine kaaggat to ac2gt
;    SH, Jun 03   -- (v2.6) Fixed error due undefined u/verrid when first dial DVC GPS
;    SH, Jul 03   --  Modified image order for dual side; added 5th antenna
;                     configuration(up,side-fore); fixed a bug in y-axis update
;                     when rangegatedelay changes  
;    SH, Aug 03   --  modified mxdT field(timeint) to show 9999 for any
;                     occurance of negative time interval
;
; -------------------------------------------------------------------------
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrdisp_common_blocks

; Common blocks used by wcrdisplay routines

; WCR and A/C parameters related to the WCR Doppler correction

common wcrdspacd, velcor,ncf,ncfdir,tcor,thack,thacki,acbasetime,sps,deg2rad, $
                  rollid,pitchid,theadid,mirid,nsvelid,ewvelid,wvelid,uerrid, $
                  verrid,upbeam,side1beam,side2beam,down1beam,down2beam,      $
                  antconf,wcrbasetime,gpsacc

; WCR parameters from WCR data header and some derived parameters
; (not all of them are used by the real time display but are included
; for completeness; does not affect the efficiency)

common wcrdspp,  alg,aspectratio,averages,dftalg,fftalg,fftbins,fftwindow,   $
                 headersize,kaspeed,ppind,maxvel,noisesamples,npulse,numrec, $
                 polh,polv,pp4alg,pp6alg,ppstrpol,prf,pulselength,rangegates,$
                 rangegatedelay,rangegatespacing,rxsequence,sppalg,starttime,$
                 stoptime,timeinterval,tnumrec,tind,txsequence,txstrpol,     $
                 wcrdate,wcrfile,wcrflag,filehdrsz

; WCR raw data arrays and some time/range parameters
; (leakage arrays are not used; an integral parameter for the leakage is
;  calculated in the data reading routine and passed to the display for
;  monitoring purposes)

common wcrdspd,  vvnoise, hhnoise, ppnoise, fftnoise,               $
                 hh, vv, vh, hv, pp, fftps,                         $
                 time, prof, begtime, endtime, radrng, velos, velo

; Calibration parameters and arrays with calibrated data used in the real-time
; display (hhz,vvz,zdr,ldr,duh,duv,dch,dcv,vhz; vhz is used for vh or hv as
; well as for doppler width under fft mode))

common wcrdspcal,hhcal,hhcald,vvcal,vvcald,h_slope,v_slope,ldrcal,            $
                 ldrcallin,zdrcal,minfsize,minimgsz,maximgsz,nosignal,avestep,$
                 zero,msz1,loopstep,loopstepd,dml,rngcor,hhz,vvz,zdr,ldr,duh, $
                 duv,dch,dcv,vhz,velfactor,blocksize,isol,isoldb,isoldbd,     $
                 alphaiso,fftcor,Nch,Nmode,Nm2,indh,indv,swapbytes,sleept,    $
                 sleepti

; Widget arrays and parameters

common wcrdspwdg,image1,image2,axsize,bxsize,wxsize,ysize,ct2ind,        $
                 zzmin,zzmax,zzmins,zzmaxs,ldmin,ldmax,ldmins,ldmaxs,    $
                 zdmin,zdmax,zdmins,zdmaxs,dvhmin,dvhmax,dvhmins,dvhmaxs,$
                 dvvmin,dvvmax,dvvmins,dvvmaxs,czmin,czmax,czmins,czmaxs,$
                 svmin,svmax,svmins,svmaxs,mxhn,mxvn,timeint,timeintm,vleak

; Widget ids

common wcrdspids,baseid,axis1id,scroll1id,bar1id,b1maxid,b1labelid,b1minid,   $
                 axis2id,scroll2id,bar2id,b2maxid,b2labelid,b2minid,          $
                 scrollw1,scrollw2,barw1,barw2,helpid,quitid,openid,pauseid,  $
                 colorid,destroyid,catchid,inittid,hhcalid,vvcalid,isolid,    $
                 startid,stepid,aveid,nimgid,thackid,sleepid,hh1id,vv1id,     $
                 ldr1id,zdr1id,dvh1id,dvv1id,vh1id,hh2id,vv2id,ldr2id,zdr2id, $
                 dvh2id,dvv2id,vh2id,antid,dvid,btlabelid,etlabelid,fileid,   $
                 msgbaseid,cancelid,statusmsgid,statmnhnid,statmxhnid,        $
                 statmnvnid,statmxvnid,stathleakid,statvleakid,statdtid,dvcid,$
                 dvlabel,dvlabels

; Real-time widget control parameters

common wcrdspctrl,lun1,wcrdir,wcrfileold,openwf,quitw,destroyw,destroym,      $
                  catche,rgold,rgsold,rgdold,starte,dispc,inittime,scrollw1e, $
                  scrollw2e,itime,ivalue,nimg,img1order,img2order

end ; wcrdisp_common_blocks
; -------------------------------------------------------------------------
; Widget display color bar

pro wcrdispbar,tbln,tbli,rg,bxsz,vmin=vmin,vmax=vmax,order=order

   common wcrdspcal

   y=rg>minimgsz
   r=findgen(y)*(vmax-vmin)/y+vmin
   r=r##(fltarr(bxsz)+1)
   r=bytscl(r,min=vmin,max=vmax,top=tbli-1)+tbli*(tbln-1)
   tv,r,order=order
end ; wcrdispbar

; -------------------------------------------------------------------------
; Simple widget message box

pro wcrdispmsg, txt

   common wcrdspids

   msgbaseid=widget_base(title='WCR DISPLAY MESSAGE',/column)
   msgid=widget_label(msgbaseid,value=txt)
   cancelid=widget_button(msgbaseid,value='CANCEL',event_pro='wcrdispmsg_ev')

   widget_control,msgbaseid,/realize

end ; wcrdispmsg

; -------------------------------------------------------------------------

pro wcrdispmsg_ev, ev

   common wcrdspids
   common wcrdspctrl

   if ev.id eq cancelid then destroym=1B

end ; wcrdispmsg_ev

; -------------------------------------------------------------------------
; Select and open a WCR raw file.
; Check the existance of any data; if not wait for data.
; Select and open A/C file (netcdf): use for acquiring time,roll,pitch,heading,
; wcr antenna pointing, East-West A/C vel, North-South A/C vel, A/C vertical vel

pro wcrdispopen,fn,data,fsize,firstopen=fo

  common wcrdspp
  common wcrdspd
  common wcrdspacd
  common wcrdspcal
  common wcrdspwdg
  common wcrdspids
  common wcrdspctrl

; Initilize

  if keyword_set(fo) then fo=1B else fo=0B

; Select a file.

  OPENF: fsize=0L
         close,lun1
         fn=dialog_pickfile(/read,filter='?????-??-*', $
                   title='Select WCR File',path=wcrdir,get_path=dir)

; Take care if no file is selected

  if fn eq '' then begin
     if n_elements(wcrdir) ne 0 and n_elements(wcrfileold) ne 0 then $
       fn=wcrdir+wcrfileold
     destroym=2B & openwf=0B
     return
  endif

; Open the file and check if there is any data in it.  Allow to wait for data.

  openr,lun1,fn
  fstatus = fstat(lun1)
  fsize   = fstatus.size
  close,lun1
  if fsize le minfsize then begin

  ;  No data, wait sleept secs

     if fo eq 0B then $
        widget_control,statusmsgid,set_value='Empty/short file. Waiting for data...' $
     else wcrdispmsg,'Empty/short file. Waiting for data...'
     wait,sleept

  ;  Check for data again

     OPENFST:
         openr,lun1,fn
         fstatus = fstat(lun1)
         fsize   = fstatus.size
         close,lun1

         ;  If there is no data, wait. Allow to terminate waiting using Cancel.
         ;  For first time openning (no main widget yet) use message widget.

         if fsize le minfsize then $
            if fo eq 0B then begin
               widget_control,destroyid,set_value='Cancel'
               widget_control,openid,sensitive=0
               widget_control,quitid,sensitive=0
             ; widget_control,pauseid,sensitive=0
               widget_control,catchid,sensitive=0
               junk=widget_event(baseid,/nowait)

               ;  If wait cancel is requested give control to main widget

               if destroyw eq 1B then begin
                  destroyw=0B & destroym=1B & openwf=0B
                  widget_control,openid,sensitive=1
                  widget_control,quitid,sensitive=1
                  widget_control,destroyid,set_value='Destroy'
                  widget_control,statusmsgid,set_value='Waiting for an event...'
                  return

               ;  otherwise wait for data

               endif else begin wait,sleept & goto,OPENFST & endelse
            endif else begin

            ;  For first time openning, check if wait termination is requested

               junk=widget_event(msgbaseid,/nowait)
               if destroym eq 1B then begin
                  widget_control,msgbaseid,/destroy
                  return
               endif else begin wait,sleept & goto,OPENFST & endelse
            endelse $

  ; Data is available. Do some widget work and open the new wcr file

         else if fo eq 0B then begin
                 widget_control,destroyid,set_value='Destroy'
                 widget_control,openid,sensitive=1
                 widget_control,quitid,sensitive=1
              endif else widget_control,msgbaseid,/destroy
  endif

; Open wcr file and save wcr file name (moved after wcrdispacopen_nc) and location

  if fo eq 0B then $
     widget_control,statusmsgid,set_value='Opening a new WCR file ...'

  wcrdispopenf,fn,data,lun1
  wcrdir=dir

; Determine some params that will be handy later and initilize image arrays

  fstatus    = fstat(lun1)
  fsize      = fstatus.size         ; current WCR file size
  avestep    = ceil(aspectratio)>1  ; # profs averaged before display
  sleepti    = float(ceil(loopstep*avestep*timeinterval))
  sleept     = sleepti              ; sleep time between data disk acesses

  hhz        = fltarr(maximgsz,rangegates)-150.
  vvz        = fltarr(maximgsz,rangegates)-150.
  zdr        = fltarr(maximgsz,rangegates)-150.
  duh        = fltarr(maximgsz,rangegates)- 50.
  duv        = fltarr(maximgsz,rangegates)- 50.
  dch        = fltarr(maximgsz,rangegates)- 50.
  dcv        = fltarr(maximgsz,rangegates)- 50.
  ldr        = fltarr(maximgsz,rangegates)-150.
  vhz        = fltarr(maximgsz,rangegates)-150. ; used for hv or sv=std(fft)

  ; For fftalg speed up processing by making a 3-D velocity range array

    if alg eq fftalg then velo=reform(velos#(fltarr(rangegates*loopstep)+1), $
                                      fftbins,rangegates,loopstep)

  ; Modify img2order, dvlabel, dvlabels and assign default image2 to be Doppler
  ; vel if sppmag

  if alg eq fftalg or alg eq sppalg then begin

    ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    if velcor then begin
      if gpsacc then dvlabel=['dg_H','dg_V'] else dvlabel=['dc_H','dc_V']
      if txsequence[0] eq polh then begin
        image1='hh'
        dvlabels=[dvlabel[0]+',du_H']
        dvlabel=[dvlabel[0],'du_H']
        if antconf eq 4 then img2order=0
      endif else begin
        image1='vv'
        dvlabels=[dvlabel[1]+',du_V']
        dvlabel=[dvlabel[1],'du_V']
        if antconf eq 4 then img2order=1
      endelse
      image2=strlowcase(dvlabel[0])
    ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    endif else begin
      dvlabel=['du_H','du_V']
      if txsequence[0] eq polh then begin
        image1='hh'
        dvlabels='du_H'
        image2=strlowcase(dvlabel[0])
        if antconf eq 4 then img2order=0
      endif else begin
        image1='vv'
        dvlabels='du_V'
        image2=strlowcase(dvlabel[1])
        if antconf eq 4 then img2order=1
      endelse
    endelse

  endif else begin   ; PPMAG

    if velcor then begin
      if gpsacc then dvlabel=['dg_H','dg_V'] else dvlabel=['dc_H','dc_V']
      if ppind[0] ge 0 and ppind[1] ge 0 then  $
        dvlabels=[dvlabel[0]+','+dvlabel[1],   $
                  'du_H,du_V'              ,   $
                  dvlabel[0]+',du_H'       ,   $
                  dvlabel[1]+',du_V'         ] $
      else if ppind[0] ge 0 then begin
        if ppind[1] ne -2 then begin
          dvlabels=[dvlabel[0]+',du_H']
          dvlabel=[dvlabel[0],'du_H']
        endif else begin ; this case will happen only for operator wrong selection
          dvlabels=['du_'+ppstrpol[0]]
          ; dvlabel - have to think what to do; for now leave it wrong
        endelse
      endif else begin
        dvlabels=[dvlabel[1]+',du_V']
        dvlabel=[dvlabel[1],'du_V']
      endelse
    endif else begin 
       dvlabel=['du_H','du_V']
       dvlabels='du_H,du_V'
    endelse

  endelse

  ; Range correction for distributed targets and range in km

    rngcor=(fltarr(loopstep)+1.)#(20.*alog10(radrng)-60.)

  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  ; Open A/C data file (A/C time and ins variables checked)

    if velcor then wcrdispacopen_nc
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

  ; After all the necessary files are open remove the old wcr file name

    wcrfileold=wcrfile

end ; wcrdispopen

; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

; Select and open A/C data file (A/C time and attitude variables checked)
; Assume NCAR-RAF Nimbus convention compatible netcdf file

pro wcrdispacopen_nc

     common wcrdspp
     common wcrdspacd
     common wcrdspctrl

     ; Select and open A/C file if not already open

     if ncf lt 0 then begin
        ncf=dialog_pickfile(/read,title='Select A/C File',path=ncfdir,$
                             get_path=ncfdir)
        if ncf eq '' then begin
          message,'No A/C file was selected. Velocity correction ignored.',/info
          ncf=-2
          velcor=0
          return
        endif
        ncf=ncdf_open(ncf)
     endif else if strmid(strtrim(string(wcrdate),1),2) ne $
        strmid(wcrfileold,3,2)+strmid(wcrfileold,6,2)+strmid(wcrfileold,9,2)  $
        then begin
          ncdf_close,ncf
          ncf=dialog_pickfile(/read,title='Select A/C File',path=ncfdir,$
                             get_path=ncfdir)
        if ncf eq '' then begin
          message,'No A/C file was selected. Velocity correction ignored.',/info
          ncf=-2
          velcor=0
          return
        endif
        ncf=ncdf_open(ncf)
     endif

     ; Read leap second and base_time vars if available. The following vars and
     ; attributes are assumed. Attributes: units ; Variable names: leap_sec,
     ; base_time, hroll,hpitch,hthead,hnsvel,hewvel,hwp3,uerr,verr,lo_events;
     ; Global attributes: FlightDate; Dimensions: time

     junk=ncdf_varload(ncf,'leap_sec')
     if junk ne -1 and junk ne -tcor then tcor=-junk

     junk=ncdf_varload(ncf,'base_time')
     if junk ne -1 and junk ne acbasetime then acbasetime=junk

     ; Make sure wcr/ac dates are the same and wcr start time is within ac time

     caldat,floor(double(acbasetime)/86400d0)+2440588L,rollid,pitchid,theadid
     junk=theadid*10000L+rollid*100+pitchid
     if junk ne wcrdate then begin
       message,'WCR and A/C dates are different. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     ncdf_diminq,ncf,ncdf_dimid(ncf,'time'),dummy,junk

     if wcrbasetime lt acbasetime or wcrbasetime gt acbasetime+junk then begin
       message,'WCR base time is not within A/C time. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     ; Load netcdf var id for relevant ins and other variables

     rollid=ncdf_varid(ncf,'hroll')
     if rollid eq -1 then begin
       message,'hroll var does not exists. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     pitchid=ncdf_varid(ncf,'hpitch')
     if pitchid eq -1 then begin
       message,'hpitch var does not exists. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     theadid=ncdf_varid(ncf,'hthead')
     if theadid eq -1 then begin
       message,'hthead var does not exists. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     nsvelid=ncdf_varid(ncf,'hnsvel')
     if nsvelid eq -1 then begin
       message,'hnsvel var does not exists. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     ewvelid=ncdf_varid(ncf,'hewvel')
     if ewvelid eq -1 then begin
       message,'hewvel var does not exists. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     wvelid=ncdf_varid(ncf,'hwp3')
     if wvelid eq -1 then begin
       message,'hwp3 var does not exists. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     mirid=ncdf_varid(ncf,'lo_events')
     if mirid eq -1 then begin
       message,'lo_events var does not exists. Ignore DVC',/info
       velcor=0
       ncf=-2
       return
     endif

     ; Check a/c vel. error vars

     uerrid=ncdf_varid(ncf,'uerr')
     verrid=ncdf_varid(ncf,'verr')
     if uerrid eq -1 or verrid eq -1 then begin
       message,'u/verr var(s) do/does not exist. Ignore GPS correction',/info
       gpsacc=0
     endif

     ; Check units for attitude angles

     junk=ncdf_attload(ncf,'hroll','units')
     if num2str(junk) eq 'deg' then deg2rad=!dtor $
     else deg2rad=1

     ; Get sampling rate (sps) for ins data

     junk=ncdf_varinq1(ncf,'hroll')
     if junk.ndims eq 1 then sps=1 else sps=junk.dimsize[0]

end ; wcrdispopen
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

; -------------------------------------------------------------------------
; Read in main WCR parameters for a selected file.
; Create file association (assoc) which is later used for
; extracting the time stamps from the headers in real-time.
; (all information is taken from a data header; some of
; the derived parameters are not needed but are left for
; completeness and compatibility with wcrtools)

pro wcrdispopenf, fn,data,lun1

common wcrdspp
common wcrdspd
common wcrdspacd
common wcrdspcal

openr,lun1,fn

; Some WCR file constant parameters
  badf        = -999.    ; float bad value
  badi        = -999     ; integer bad value
  filenamel   =  20      ; this is fixed and used to find fileIDstr
  filehdrsz   =  206L    ; bytes; after March 2002
  headersize  =  20L     ; bytes; after March 2002
  headervars  =  5       ; number of 4 byte fields
  bind        =  0       ; blocksize var position in data header, after 3/02
  tind        = [1,2]    ; timestamp var positions in data header, after 3/02
  noisesamples=  1       ; # noise samples (always 1 before May 1999)
                         ; it is updated from the header info
  sppalg      =   9
  pp4alg      =  14
  pp6alg      =  15
  fftalg      =  11
  dftalg      =  13
  polh        =  104
  polv        =  118
  sppvars     =   4
  ppmvars     =  14
  npulse      =   4      ; Number of pulses in a burst it is updated if 1 or 6
  timemode    =   0      ; initialize timemode of the time vector
  wcrflag     =  [0,0,0] ; wcrflag=# bad profiles at the end of the file
                         ; wcrflag[1]=1, data shift is suspected
                         ; wcrflag[2]=1, WCR computer date and timestamps date
                         ; don't match or IRIG connection/data was lost/corrupted

; Get WCR file name and WCR date

  fileidstr=strmid(fn,strlen(fn)-filenameL,3)        ; allows renaming

  wcrfile=strmid(fn,strpos(fn,fileidstr,/reverse_search),filenamel)
  if strmid(wcrfile,3,2) lt '90' then wcrdate='20'+strmid(wcrfile,3,2) $
  else wcrdate='19'+strmid(wcrfile,3,2)

  wcrdate=long(wcrdate+strmid(wcrfile,6,2)+strmid(wcrfile,9,2))
  if wcrdate lt 19900101 or wcrdate gt 20200101 then begin
     message,'WCR file naming problem warning: '+wcrfile,/info
  endif

  if wcrdate gt 20020331 then begin
    ; Read file header
    version         =long(1)     & readse,lun1,big_end=bgnd,swap=swapbytes,version
    prf             =lonarr(6)   & readse,lun1,big_end=bgnd, prf
    junk            =lonarr(6)   & readse,lun1,big_end=bgnd, junk
    txsequence      =bytarr(6)   & readse,lun1,big_end=bgnd, txsequence
    rxsequence      =bytarr(6,2) & readse,lun1,big_end=bgnd, rxsequence
    pulselength     =long(1)     & readse,lun1,big_end=bgnd, pulselength
    txsamples       =long(1)     & readse,lun1,big_end=bgnd, txsamples
    noisesamples    =long(1)     & readse,lun1,big_end=bgnd, noisesamples
    rangegates      =long(1)     & readse,lun1,big_end=bgnd, rangegates
    rangegatedelay  =long(1)     & readse,lun1,big_end=bgnd, rangegatedelay
    rangegatespacing=long(1)     & readse,lun1,big_end=bgnd, rangegatespacing
    junk            =lonarr(2)   & readse,lun1,big_end=bgnd, junk
    iffilter        =long(1)     & readse,lun1,big_end=bgnd, iffilter
    averages        =long(1)     & readse,lun1,big_end=bgnd, averages
    alg             =long(1)     & readse,lun1,big_end=bgnd, alg
    fftbins         =long(1)     & readse,lun1,big_end=bgnd, fftbins
    fftwindow       =long(1)     & readse,lun1,big_end=bgnd, fftwindow
    junk            =long(1)     & readse,lun1,big_end=bgnd, junk
    irigflag        =long(1)     & readse,lun1,big_end=bgnd, irigflag
    junk            =lonarr(18)  & readse,lun1,big_end=bgnd, junk
    datablocksize   =long(1)     & readse,lun1,big_end=bgnd, datablocksize

    ; Read first data header

    fstblocknum     =long(1) & readse,lun1,big_end=bgnd, fstblocknum
    startt          =lonarr(2) & readse,lun1,big_end=bgnd, startt

    ; Tide up some of the header variables

    if alg eq pp6alg then npulse=6 $                        ; PPMAG6
    else if alg eq pp4alg or alg eq sppalg then npulse=4 $  ; SPPMAG/PPMAG
    else npulse=1                                           ; FFT, DFFT
    prf             =prf(0:npulse-1)

    irigflag        =(irigflag and 1)  ; 0 - host computer time, 1 - IRIG time
    if irigflag eq 0 then tcor=0.d0
    pulselength     =pulselength*25
    rangegatedelay  =float(rangegatedelay)*3.75
    rangegatespacing=float(rangegatespacing)*3.75
    txsequence      =txsequence[0:npulse-1]
    if alg eq sppalg or alg eq fftalg or alg eq dftalg then begin
      rxsequence    =txsequence[0]
      prf           =prf[0]
    endif else rxsequence =rxsequence[0:npulse-2>0,*]
    case iffilter of
      1: iffilter=5.01 ; MHz (old 5 MHz Chebychev filter,5.01 for identif.)
      2: iffilter=2.   ; MHz (2 MHz Bessel filter)
      3: iffilter=5.   ; MHz (5 MHz Bessel filter)
      5: iffilter=10.  ; MHz (10 MHz Bessel filter)
      else: iffilter=badf   ; unknown or not allowed
    endcase

    ; Check if time stamps match wcrdate

     caldat, startt[0]/86400d0+2440587.5d0, mm, dd, yyyy
     k=yyyy*10000L+mm*100+dd

    if wcrdate ne k then begin
       wcrflag[2]=1
       if wcrdate lt k-1 then begin
          message,'Using time stamps to define data date: '+num2str(k),/info
          wcrdate=k
       endif else $
          message,'Using file name to define data date: '+num2str(wcrdate),/info
    endif

  endif else begin
  ; --------------- header variables from old data formats --------------------
    bgnd=1
    filehdrsz  =  0
    headervars =  23
    headersize =  92
    polh       =   7
    polv       =  15
    bind       =  1
    tind       = [3,4]

    ; Read header data

    version         =long(1) & readse,lun1,big_end=bgnd, version,swap=swapbytes; 0; file type
    fstblocknum     =long(1) & readse,lun1,big_end=bgnd, fstblocknum   ; 1; data block #
    datablocksize   =long(1) & readse,lun1,big_end=bgnd, datablocksize ; 2; in bytes
    startt          =lonarr(2) & readse,lun1,big_end=bgnd, startt      ; 3; unix time [s, us]
    spare           =long(1) & readse,lun1,big_end=bgnd, spare         ; 5; radar azimuth
    spare           =long(1) & readse,lun1,big_end=bgnd, spare         ; 6; radar elevation
    spare           =long(1) & readse,lun1,big_end=bgnd, spare         ; 7; proc.alg.flag ?

  ; Check if time stamps match wcrdate

    caldat, floor(startt[0]/86400d0)+2440588L, mm, dd, yyyy
    k=yyyy*10000L+mm*100+dd

    if wcrdate lt k-1 then wcrdate=k  ; use time stamp to extract date

    ; Check if before May 18 1999 and process accordingly

  if (wcrdate lt 19990518) then begin
    pp4alg=7
    prf             =lonarr(4) & readse,lun1,big_end=bgnd, prf         ; 8; [n,n,n,n] Hz
    pulselength     =long(1)   & readse,lun1,big_end=bgnd, pulselength ; 12; n
    pulselength     =pulselength*25                         ;     n*50 [ns], N/A
    txsequence      =bytarr(4) & readse,lun1,big_end=bgnd, txsequence  ; 13;[n,n,n,n],H/V:7/15
    txsequence      =reverse(txsequence)
    pulsesampledelay=long(1) & readse,lun1,big_end=bgnd, pulsesampledelay; 14; n
    pulsesampledelay=(pulsesampledelay+2)*100               ;     (2+n)*100 [ns]
    rangegates      =long(1) & readse,lun1,big_end=bgnd, rangegates      ; 15;
    rangegatedelay  =long(1) & readse,lun1,big_end=bgnd, rangegatedelay  ; 16; n
    rangegatedelay  =float(rangegatedelay)*3.75             ;     3.75*n [m]
    rangegatespacing=long(1) & readse,lun1,big_end=bgnd, rangegatespacing; 17; n
    rangegatespacing=float(rangegatespacing)*3.75           ;     3.75*n [m]
    spare           =long(1) & readse,lun1,big_end=bgnd, spare           ; 18; n (n+1)*2 [us]
    spare           =long(1) & readse,lun1,big_end=bgnd, spare           ; 19; 1:rx atten. on
    spare           =long(1) & readse,lun1,big_end=bgnd, spare           ; 20; rx IF filt., N/A
    averages        =long(1) & readse,lun1,big_end=bgnd, averages        ; 21;
    alg             =long(1) & readse,lun1,big_end=bgnd, alg             ; 22;    9,7,11

    ; Add fftbins and fftwindow for data taken after Oct 22, 1996

    if (wcrdate gt 19961022) then begin

       headersize =   headersize+8
       headervars =   headervars+2

       fftbins  =long(1) & readse,lun1,big_end=bgnd,fftbins   ; 23; number of bins (FFT modes)
       fftwindow=long(1) & readse,lun1,big_end=bgnd,fftwindow ; 24; fft window: 0,1,2,3,4,5
                                                              ;     rect.,hann,taper,blackman,
                                                              ;     gauss,gaussian top
    endif else begin
       fftbins=averages*(alg eq fftalg)
       fftwindow=0
    endelse

    ; Pulse width was set manually; set it to -999
      pulselength=badi

  endif else begin   ; for data/header after May 18 1999

    headersize      =160       ; bytes
    headervars      =40        ; number of 4 byte fields

    prf             =lonarr(12) & readse,lun1,big_end=bgnd, prf          ; 8; [n,n,n,...] Hz

    txsequence   =bytarr(8) & readse,lun1,big_end=bgnd, txsequence ; 20;[s,s,s,s],'h','v'

    ; make it compatible with the previous versions

    for k=0,7 do $
        if txsequence[k] eq '104' then txsequence[k]=7 $
        else if txsequence[k] eq '118' then txsequence[k]=15 $
        else txsequence[k]=0   ; not used

    rxsequence =bytarr(16) & readse,lun1,big_end=bgnd, rxsequence        ; 22;[s,s,s,s],'h','v'

    ; follow the original convention for txsequence
    for k=0,15 do $
        if rxsequence[k] eq '104' then rxsequence[k]=7 $
        else if rxsequence[k] eq '118' then rxsequence[k]=15 $
        else rxsequence[k]=0   ; not used
    rxsequence=transpose(reform(rxsequence,8,2))

    pulselength     =long(1)   & readse,lun1,big_end=bgnd, pulselength  ; 26; n
    pulselength     =pulselength*25                        ;     n*50 [ns], 50
    spare           =lonarr(1) & readse,lun1,big_end=bgnd, spare        ; 27;
    spare           =lonarr(1) & readse,lun1,big_end=bgnd, spare        ; 28;
    noisesamples    =long(1) & readse,lun1,big_end=bgnd, noisesamples   ; 29; n
    rangegates      =long(1) & readse,lun1,big_end=bgnd, rangegates     ; 30;
    rangegatedelay  =long(1) & readse,lun1,big_end=bgnd, rangeGateDelay ; 31; n
    rangegatedelay  =float(rangeGateDelay)*3.75            ;     3.75*n [m]
    rangegatespacing=long(1) & readse,lun1,big_end=bgnd, rangegatespacing; 32; n
    rangegatespacing=float(rangegatespacing)*3.75          ;     3.75*n [m]
    spare           =fltarr(1) & readse,lun1,big_end=bgnd, spare        ; 33;
    spare           =fltarr(1) & readse,lun1,big_end=bgnd, spare        ; 34;
    spare           =long(1) & readse,lun1,big_end=bgnd,spare           ; 35; rx filt. code
    averages        =long(1) & readse,lun1,big_end=bgnd, averages       ; 36;
    ;alg.codes: SPPMAG: 9, FFT: 11, DPPMAG: 12, DPFFT: 13, PPMAG4: 14, PPMAG6: 15
    alg             =long(1) & readse,lun1,big_end=bgnd, alg            ; 37; 9,14(7),11,13,15

    fftbins  =long(1) & readse,lun1,big_end=bgnd,fftbins   ; 38; number of bins for FFT files
    fftwindow=long(1) & readse,lun1,big_end=bgnd,fftwindow ; 39; fft window: 0,1
                                                           ;     rectangular,hanning
    ; need to check the valid prfs and change npulse if needed;
    if alg eq pp6alg then begin                                 ; PPMAG6
       txsequence=txsequence[0:5] & rxsequence=transpose(rxsequence[*,0:4])
       prf=prf(0:5) & npulse=6
    endif else if alg eq pp4alg or alg eq sppalg then begin     ; SPPMAG/PPMAG
       txsequence=txsequence[0:3] &
       if alg eq pp4alg then begin
         rxsequence=transpose(rxsequence[*,0:2])
         prf=prf(0:3)
       endif else begin
         rxsequence=txsequence[0]
         prf=prf[0]
       endelse
    endif else begin                                            ; FFT, DFFT
      txsequence=txsequence[0] & rxsequence=txsequence[0]
      prf=prf[0] & npulse=1
    endelse

  endelse
endelse
; ---------------------------  End header data -------------------------------

; Some WCR file derivative parameters

  blocksize   = datablocksize+headersize                 ; record size in Bytes
  spare       = fstat(lun1)
  tnumrec     = long(strtrim(spare.size,2))/blocksize    ; total # of recs
  if spare.size ne tnumrec*blocksize+filehdrsz then wcrflag[0]=1
  numrec      = tnumrec                      ; the number of records to be read

; Create string for the transmitted pulse sequence

  txstrpol = strarr(npulse)
  indv   = where(txsequence eq polv)             ; Tx V-pol pulse indices
  if (indv[0] ne -1) then txstrpol[indv] = 'V'
  indh   = where(txsequence eq polH)             ; Tx H-pol pulse indices
  if (indh[0] ne -1) then txstrpol[indh] = 'H'

; Create string array for the received pulse pair sequence

  if alg eq sppalg then begin
    if rxsequence eq polV then ppstrpol='VV' else ppstrpol='HH'
    txstrpol=txstrpol[0]
  endif else begin
    if npulse gt 1 and wcrdate gt 19990518 then begin
      ppstrpol = strarr(npulse-1,2)
      ind      = where(rxsequence eq polV)
      if (ind[0] ne -1) then ppstrpol[ind] = 'V'
      ind      = where(rxsequence eq polH)
      if (ind(0) ne -1) then ppstrpol[ind] = 'H'
      ppstrpol=reform(ppstrpol[*,0]+ppstrpol[*,1])
    endif else ppstrpol=replicate(' ',npulse-1>1)
  endelse

; Define data association struture for random access to the header
; Access to the data is done differently

  Nch        = 2+(alg eq fftalg)*(fftbins-2) ; number of data channels
  Nmode      = 2*(alg eq sppalg) + $
               7*(alg eq pp4alg) + $
               11*(alg eq pp6alg)+ $
               1*(alg eq fftalg)
  Nm2        = Nmode/2

  data=assoc(lun1,{header: lonarr(headervars), $
                   dat: fltarr(Nch,noisesamples+1+rangegates,Nmode)},filehdrsz)

; Calculate start time and time interval (between profiles)
;
; starttime    - string, start recording date/time
; timeinterval - float, sampling interval in [sec]

  t=data[0]
  if swapbytes eq 1 then t=swap_endian(t)

; Save WCR corrected initial unix time stamp (whole seconds)
  wcrbasetime=t.header[tind[0]]+long(tcor)+long(thack)

; Prepare the accurate start time (to be used for time interval calc.)
  timeinterval=double(t.header[tind[0]])+double(t.header[tind[1]])/1.e6

  starttime=(timeinterval+double(tcor+thack)) mod 86400
  starttime=sec2timestr(starttime) ; $ ; use just hh:mm:ss
  ;         +'.'+ num2str(round(100*(starttime-long(starttime))),fr=0)

; Get last profile number

  k=tnumrec-1
  t=data[k]
  if swapbytes eq 1 then t=swap_endian(t)

  lstblocknum=t.header[bind]

; Calculate stop time, time interval (between profs), time length
;
;  stopTime     - string, stop recording date/time
;  timeLenght   - float, time duration in [sec] of all recorded profiles
;  timeInterval - float, sampling interval in [sec]
;  timeint      - integer, averaged sampling interval in ms for every display
;                 loop

  k=tnumrec-1
  t=data[k]
  if swapbytes eq 1 then t=swap_endian(t)

  timeinterval=float(abs(timeinterval-(double(t.header[tind[0]])+$
               double(t.header[tind[1]])/1.d6)))/(tnumrec-1.)
  timelength=timeinterval*(tnumrec-1L)

  timeint=long(timeinterval*1000.)   ; used for dT monitor

  stoptime=(double(t.header[tind[0]])+double(t.header[tind[1]]/1e6)+ $
            double(tcor+thack)) mod 86400
  stoptime=sec2timestr(stoptime) ; $                ; use just hh:mm:ss
  ;        +'.'+num2str(round(100*(stoptime-long(stoptime))),fr=0)

; Correct lstBlockNum if profiles went over 65536.

  lstBlockNum=fstBlockNum+tnumRec

; Estimate raw data images aspect ratio (used to determine the default number
; of profiles to be averaged

  aspectratio = (rangegatespacing/(timeinterval*kaspeed))>1

; Form range gate vector (geometric center of the gate)

  radrng = rangegatedelay+findgen(rangegates)*rangegatespacing

; Maximum Doppler velocity

  maxvel=prf/4.*0.00316

; Create velo for FFT and do some checking for maxvel

  if (alg eq fftalg or alg eq dftalg) then begin

    ; FFT spectra are caclulated with the first bin being DC (0 frequency) then
    ; +df to +Nyquist Freq.(Fn) and then from -Fn+df to -df (df - freq. step).
    ; In other words the fft hasn't been shifted.
    ; Positive Doppler velocity (negative freq) is toward the radar (radar
    ; convention).

    ; Form velocity axis for non-shifted spectra.
    maxvel=maxvel[0]
    velos=[-findgen(fftbins/2+1),reverse(findgen(fftbins/2-1)+1)]*2.*maxvel/fftbins
  endif else if alg eq sppalg then maxvel=maxvel[0]

; And finally determine which pulse pair(s) will be processed.
; ppind[0] - H-ch, ppind[1] - V-ch; Determine velfactor accordingly

  velfactor = maxvel/!pi
  if alg eq sppalg or alg eq fftalg then begin
    if txsequence[0] eq polh then ppind=[0,-1] else ppind=[-1,0]
    velfactor=velfactor[0]
  endif else if alg eq pp4alg or alg eq pp6alg then begin
    junk=(where(ppstrpol eq 'HH'))
    if junk[0] eq -1 then ppind=junk $
    else ppind=junk[(where(prf[junk] eq max(prf[junk])))[0]]

    junk=(where(ppstrpol eq 'VV'))
    if ppind+junk[0] eq -2 then begin
      message,'No co-pol pulse pair! Using 0 pulse pair',/info
      ppind=[0,-2]
      velfactor=velfactor[0]
      if velcor then begin
        velcor=0
        message,'Velocity correction(DVC) ignored.',/info
      endif
    endif else begin
      if junk[0] eq -1 then ppind=[ppind,junk] $
      else ppind=[ppind,junk[(where(prf[junk] eq max(prf[junk])))[0]]]
      if ppind[0] eq -1 then velfactor=[-1,velfactor[ppind[1]]] $
      else if ppind[1] eq -1 then velfactor=[velfactor[ppind[0]],-1] $
      else velfactor=velfactor[ppind]
    endelse
  endif

end ; wcrdispopenf

; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; Read A/C ins data and calculate A/C velocity contribution into the WCR beam
; for given time interval.

pro wcrdispacread, wcrtime,wcracv

  common wcrdspacd
  common wcrdspp

  startt=wcrtime[0]                         ; start time for the ac data
  stopt =wcrtime[n_elements(wcrtime)-1]     ; end time for the ac data
  offset=[0,long(startt)-acbasetime]        ; 0 offset for sps dim
  nsec  =ceil(stopt)-long(startt)+1L        ; # of secs to read
  count =[sps,nsec]                         ; read up to an extra second
  n=sps*nsec                                ; ins vars # of data points

; Form ac time vector

  actime=double(acbasetime+offset[1])+dindgen(sps*count[1])/double(sps)

; Get attitude angles

  ncdf_varget,ncf,rollid,roll,offset=offset,count=count      ; ac roll
  ncdf_varget,ncf,pitchid,pitch,offset=offset,count=count    ; ac pitch
  ncdf_varget,ncf,theadid,thead,offset=offset,count=count    ; ac heading

  if deg2rad ne 1 then begin
    roll =deg2rad*reform(roll,n)
    pitch=deg2rad*reform(pitch,n)
    thead=deg2rad*reform(thead,n)
  endif else begin
    roll =reform(roll,n)
    pitch=reform(pitch,n)
    thead=reform(thead,n)
  endelse

; Get ac velocities (east-west, north-south, vertical)

  ncdf_varget,ncf,nsvelid,nsvel,offset=offset,count=count    ; N-S ac vel
  ncdf_varget,ncf,ewvelid,ewvel,offset=offset,count=count    ; E-W ac vel
  ncdf_varget,ncf,wvelid,wvel,offset=offset,count=count      ; ac vertical vel

  nsvel=reform(nsvel,n)
  ewvel=reform(ewvel,n)
  wvel =reform(wvel,n)

; Get GPS ac vel corrections if requested and apply the correction

  if gpsacc eq 1 then begin
    ncdf_varget,ncf,uerrid,uerr,offset=offset,count=count    ; ac uerr
    ncdf_varget,ncf,verrid,verr,offset=offset,count=count    ; ac verr

    ewvel=ewvel-reform(uerr,n)
    nsvel=nsvel-reform(verr,n)
  endif

; Read radar mirror position if needed

  if antconf mod 2 eq 0 or antconf eq 5 then begin
    ncdf_varget,ncf,mirid,junk,offset=offset,count=count     ; low_events
    mir=intarr(nsec)                        ; 0, assign 'side' position
    ind=where((junk and 33554432) eq 0)     ; use 25 bit to find 'up' position
    if ind[0] ne -1 then mir[ind]=1         ; 1, assign up position
    mir=rebin(mir,sps*nsec)
  endif

; Determine WCR beam vector in A/C coordinates

  if alg eq sppalg or alg eq fftalg then begin
    if txsequence[0] eq polh then begin
      if antconf mod 2 eq 0  or antconf eq 5 then begin
        junk=upbeam#mir+side1beam#(mir eq 0)
      endif else if antconf eq 1 then begin
        junk=down2beam#(fltarr(n)+1.)
      endif else begin
        message,'Incorrect aconfig/dopcor selection for SPP-H. Use aconfig 0',/info
        junk=upbeam#fltarr(nsec*sps)+1.
        antconf=0   ; I'll ignore all that is done when changing antenna config
                    ; This is just to avoid stopping the program in case of
                    ; accidentally selected wrong antenna configuration
      endelse
    endif else begin
      if antconf eq 0 then begin
        junk=upbeam#mir+side1beam#(mir eq 0)
      endif else if (antconf eq 1 or antconf eq 4) then begin
        junk=down1beam#(fltarr(n)+1.)
      endif else if antconf eq 2 or antconf eq 5 then begin
        junk=sid2beam#(fltarr(n)+1.)
      endif else begin
        message,'Incorrect aconfig/dopcor selection SPP-V. Use aconfig 1',/info
        junk=down1beam#(fltarr(n)+1.)
        antconf=1   ; I'll ignore all that is done when changing antenna config
                    ; This is to avoid stopping the program in case of
                    ; accidentally selected wrong antenna configuration
      endelse
    endelse
  endif else if alg eq pp4alg or alg eq pp6alg then begin
    case antconf of
      0: begin
           junk=upbeam#mir+side1beam#(mir eq 0)
           if ppind[0] ge 0 and ppind[1] ge 0 then junk=[[[junk]],[[junk]]]
         end
      1: begin
           if ppind[0] ge 0 and ppind[1] ge 0 then begin
             junk=[[[down2beam#(fltarr(n)+1.)]], $
                   [[down1beam#(fltarr(n)+1.)]]]
           endif else begin
             if ppind[0] ne -1 then junk=down2beam#(fltarr(n)+1.) $
             else junk=down1beam#(fltarr(n)+1.)
           endelse
         end
      2: begin
           if ppind[0] ge 0 and ppind[1] ge 0 then begin
             junk=[[[upbeam#mir+side1beam#(mir eq 0)]], $
                   [[side2beam#(fltarr(n)+1.)]]]
           endif else begin
             if ppind[0] ne -1 then junk=upbeam#mir+side1beam#(mir eq 0) $
             else junk=side2beam#(fltarr(n)+1.)
           endelse
         end
      4: begin
           if ppind[0] ge 0 and ppind[1] ge 0 then begin
             junk=[[[upbeam#mir+side1beam#(mir eq 0)]], $
                   [[down1beam#(fltarr(n)+1.)]]]
           endif else begin
             if ppind[0] ne -1 then junk=upbeam#mir+side1beam#(mir eq 0) $
             else junk=down1beam#(fltarr(n)+1.)
           endelse
         end
      5: begin
           if ppind[0] ge 0 and ppind[1] ge 0 then begin
             junk=[[[upbeam#mir+side1beam#(mir eq 0)]], $
                   [[side2beam#(fltarr(n)+1.)]]]
           endif else begin
             if ppind[0] ne -1 then junk=upbeam#mir+side1beam#(mir eq 0) $
             else junk=side2beam#(fltarr(n)+1.)
           endelse
         end
      else:  begin
               closevxi
               ncdf_close,ncf
               ncf=-1
               message,'Incorrect aconfig selection. Should not happen here.'
             end
    endcase
  endif 

; Calculate ac vel component into the wcr beam

  ; Calculate coord. transformation matrix and wcr beam in A/C coordinates

  wcrt=ac2gt(roll,pitch,thead)               ; transformation matrix

  if alg eq sppalg or alg eq fftalg then begin

    ; Transform wcr beam into ground coords. and calc ac vel component into wcr

    wcracv=fltarr(n)
    for k=0L,n-1 do $
      wcracv[k]=transpose(junk[*,k]#wcrt[*,*,k])##([[ewvel],[nsvel],[wvel]])[k,*]

    ; Sync ac time-based data to wcr time samplings (not equally spaced)

    wcracv=interpol(wcracv,actime,wcrtime)

  endif else begin

    if ppind[0] lt 0 or ppind[1] lt 0 then begin
      wcracv=fltarr(n)
      for k=0L,n-1 do $
        wcracv[k]=transpose(junk[*,k]#wcrt[*,*,k])##([[ewvel],[nsvel],[wvel]])[k,*]
      wcracv=interpol(wcracv,actime,wcrtime)
    endif else begin
      wcracv=fltarr(n,2)
      for k=0L,n-1 do begin
        wcracv[k,0]=transpose((junk[*,k,0])#wcrt[*,*,k])##([[ewvel],[nsvel],[wvel]])[k,*]
        wcracv[k,1]=transpose((junk[*,k,1])#wcrt[*,*,k])##([[ewvel],[nsvel],[wvel]])[k,*]
      endfor

      wcracv = [ [interpol(reform(wcracv[*,0]),actime,wcrtime)], $
                 [interpol(reform(wcracv[*,1]),actime,wcrtime)]  ]
    endelse

  endelse

end ; wcrdispacread
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

; -------------------------------------------------------------------------
; WCR data reader and calibration procedure
; Data are read in portions as defined by the display step (loopstep) and
; # of profs averaged (avestep, default is aspect ratio).  The process is
; governed by a profile counter k. An integral leakage parameter calc is
; passed to the widget dispaly as a routine parameter rather than in a
; common block.

pro wcrdispread,k,calc,mnhn,mnvn

common wcrdspp
common wcrdspd
common wcrdspacd
common wcrdspcal
common wcrdspwdg
common wcrdspctrl
common recrawtemp, recraw,recraw1  ; serves only this routine

m   =loopstep*avestep              ; total number of profiles to read

; Initilize data arrays if necessary

if (size(recraw,/dim))[0] ne m or calc eq -99. then begin

   recraw1={header:lonarr(headersize/4), $
            data:fltarr(Nch,noisesamples+1+rangegates,Nmode)}
   recraw=replicate(recraw1,m)

   if alg eq fftalg then begin
     fftps=fltarr(fftbins,rangegates,loopstep)
     tmp1 =fltarr(fftbins,rangegates,m)
     vh   =fltarr(rangegates,loopstep)  ; used as a temporary work array
     velo =reform(velos#(fltarr(rangegates*loopstep)+1), $
                  fftbins,rangegates,loopstep)
   endif else begin
     tmp1 =fltarr(rangegates,m)
     vh   =fltarr(loopstep,rangegates)
     hv   =fltarr(loopstep,rangegates)
     pp   =complexarr(loopstep,rangegates)
   endelse
   tmp2   =fltarr(m)
   vvnoise=fltarr(loopstep)
   hhnoise=fltarr(loopstep)
   hh     =fltarr(loopstep,rangegates)
   vv     =fltarr(loopstep,rangegates)
   wcrtime=dblarr(m)
endif

calc=0. ; initialize leak after using calc=-99. as a flag in wcrdispread
        ; for starting new file

; .........................................................................
; Read data from wcr file

  point_lun,lun1,filehdrsz+k*blocksize
  readu,lun1,recraw

  if swapbytes eq 1 then recraw=swap_endian(recraw)

  ; Calculate average time interval for that data segment (in ms)

  ;junk1=(m-1)
  ;timeint=double((recraw.header[tind[0]])[0])+double((recraw.header[tind[1]])[0])/1.e6
  ;timeint=long(1000.*abs(timeint-(double((recraw.header[tind[0]])[junk1])+$
  ;             double((recraw.header[tind[1]])[junk1])/1.e6))/(junk1))

  junk1=m-1-(m mod 2)
  tmp2=1000.*((double((recraw.header[tind[0]])[1:junk1:2])+$
                          double((recraw.header[tind[1]])[1:junk1:2])/1.d6)-$
                         (double((recraw.header[tind[0]])[0:junk1:2])+$
                          double((recraw.header[tind[1]])[0:junk1:2])/1.d6))
  if min(tmp2) lt 0. then timeint=9999 else timeint=long(mean(tmp2))

  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  if velcor then begin
      ; Get WCR IRIG time (needed for the Doppler correction processing)

     tmp2=double(recraw.header[tind[0]])+double(recraw.header[tind[1]])/1d6
     wcrtime=rebin(tmp2,loopstep)    ; this will averaged the time.

     ; Change from GPS to UTC and shift if A/C clock and WCR clock do not sync

     wcrtime=wcrtime+double(tcor+thack)

     ; Calculate A/C velocity contribution into the wcr beam

     wcrdispacread,wcrtime,wcracv

  endif
  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

  ; Process the rest of the data according to the wcr mode

  if alg eq fftalg then begin                                ; FFT

    ; Extract spectra
    tmp1=reform(recraw.data[*,noisesamples+1:*,0])
    fftps=rebin(tmp1,fftbins,rangegates,loopstep)

    ; Extract leakage (use minimum to track a possible data shift)  ;; LEAK
    calc=10.*alog10(min(total(reform(recraw.data[*,noisesamples,0]),1)))+fftcor

    ; Extract noise and signal power from the spectra
    tmp2=total(reform(recraw.data[*,0:noisesamples-1,0]),1)
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    if txsequence[0] eq polh then begin
       hhnoise=rebin(tmp2,loopstep)
       hh=transpose(total(fftps,1))
    endif else begin
       vvnoise=rebin(tmp2,loopstep)
       vv=transpose(total(fftps,1))
    endelse

  endif else if alg eq sppalg then begin                     ; SPPMAG

    tmp2   =reform(recraw.data[0,0:noisesamples-1,Nm2:Nmode-1])
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    vvnoise=rebin(tmp2,loopstep)
    tmp2   =reform(recraw.data[1,0:noisesamples-1,Nm2:Nmode-1])
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    hhnoise=rebin(tmp2,loopstep)

    ;; LEAK, take min for shift identification
    ;calc=10.*alog10(total(reform(recraw.data[1,noisesamples,Nm2]))/m)
    calc=10.*alog10(min(reform(recraw.data[1,noisesamples,Nm2])))  ;; LEAK H ch
    vleak=10.*alog10(min(reform(recraw.data[0,noisesamples,Nm2]))) ;; LEAK V ch

    if txsequence[0] eq polv then begin
       tmp1=reform(recraw.data[0,noisesamples+1:*,1])
       vv  =transpose(rebin(tmp1,rangegates,loopstep))
       tmp1=reform(recraw.data[1,noisesamples+1:*,1])
       hv  =transpose(rebin(tmp1,rangegates,loopstep))
    endif else begin
       tmp1=reform(recraw.data[0,noisesamples+1:*,1])
       vh  =transpose(rebin(tmp1,rangegates,loopstep))
       tmp1=reform(recraw.data[1,noisesamples+1:*,1])
       hh  =transpose(rebin(tmp1,rangegates,loopstep))
    endelse

    pp=transpose(reform(complex(rebin(reform(recraw.data[0,noisesamples+1:*,0] $
       ),rangegates,loopstep),rebin(reform(recraw.data[1,noisesamples+1:*,0]), $
       rangegates,loopstep))))

  endif else if alg eq pp4alg or alg eq pp6alg then begin    ; PPMAG

    tmp2   =mean(reform(recraw.data[0,0:noisesamples-1,Nm2:Nmode-1]),1)
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    vvnoise=rebin(tmp2,loopstep)
    tmp2   =mean(reform(recraw.data[1,0:noisesamples-1,Nm2:Nmode-1]),1)
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    hhnoise=rebin(tmp2,loopstep)

    ;; LEAK, take min for shift identification
    ;calc=10.*alog10(total(reform(recraw.data[1,noisesamples,Nm2]))/m
    calc=10.*alog10(min(reform(recraw.data[1,noisesamples,Nm2])))  ;; LEAK H ch
    vleak=10.*alog10(min(reform(recraw.data[0,noisesamples,Nm2]))) ;; LEAK V ch

    tmp1=mean(reform(recraw.data[0,noisesamples+1:*,indv+Nm2]),2)
    vv  =transpose(rebin(tmp1,rangegates,loopstep))
    tmp1=mean(reform(recraw.data[1,noisesamples+1:*,indv+Nm2]),2)
    hv  =transpose(rebin(tmp1,rangegates,loopstep))
    tmp1=mean(reform(recraw.data[0,noisesamples+1:*,indh+Nm2]),2)
    vh  =transpose(rebin(tmp1,rangegates,loopstep))
    tmp1=mean(reform(recraw.data[1,noisesamples+1:*,indh+Nm2]),2)
    hh  =transpose(rebin(tmp1,rangegates,loopstep))

    ; get one pulse pair for HH and one for VV (select max prf)

    if ppind[0] ge 0 and ppind[1] ge 0 then $
      pp=[[[transpose(reform(complex(rebin(reform(recraw.data[0, $
            noisesamples+1:*,ppind[0]]),rangegates,loopstep),  $
            rebin(reform(recraw.data[1,noisesamples+1:*,ppind[0]]),$
            rangegates,loopstep))))]], $

          [[transpose(reform(complex(rebin(reform(recraw.data[0,noisesamples+1:*,$
            ppind[1]]),rangegates,loopstep),rebin(reform(recraw.data[1, $
            noisesamples+1:*,ppind[1]]),rangegates,loopstep))))]]] $
    else if ppind[0] ge 0 then $
      pp=transpose(reform(complex(rebin(reform(recraw.data[0, $
            noisesamples+1:*,ppind[0]]),rangegates,loopstep),  $
            rebin(reform(recraw.data[1,noisesamples+1:*,ppind[0]]),$
            rangegates,loopstep)))) $
    else $
      pp=transpose(reform(complex(rebin(reform(recraw.data[0,noisesamples+1:*,$
            ppind[1]]),rangegates,loopstep),rebin(reform(recraw.data[1, $
            noisesamples+1:*,ppind[1]]),rangegates,loopstep))))
  endif

; Calculate average noise and correct h/v noise (disabled)

  vvmn=total(vvnoise)/loopstep
; vvmn=hhmn^v_slope                          ; V-channel lookup table corr.
  mnvn=10.*alog10(vvmn)
  hhmn=total(hhnoise)/loopstep
; hhmn=hhmn^h_slope                          ; H-channel lookup table corr.
  mnhn=10.*alog10(hhmn)

; .........................................................................
; Calibrate data. H-channel is corrected (no cal. pulse or velocity bias
; correction is applied; no thresholding except for a minimum signal)

  if alg eq fftalg then begin
    ; avfac for fft is fftbin^2: dsp fft does not normilize by fftbins
      avfac=float(fftbins)*float(fftbins)

    if txstrpol[0] eq 'H' then begin
      hh =(hh-hhmn) > nosignal
;     hh =(hh^h_slope-hhmn) > nosignal                        ; H-channel corr.
      hhz=shift(hhz,-loopstep,0)
      hhz[dml:msz1,*]=10.*alog10(hh/avfac)+rngcor+hhcal+fftcor

    ; Calculate mean (duh) and standard deviation (vhz) Doppler velocities
      duh=shift(duh,-loopstep,0)
      vh=total(fftps,1)  ; temp use as a work array
      fftps=velo*fftps   ; fftps is not needed anymore so use it as a work array
      tmp1=transpose(total(fftps,1)/vh)
      duh[dml:msz1,*]=tmp1
      vhz=shift(vhz,-loopstep,0)
      vhz[dml:msz1,*]=sqrt(transpose(total(velo*fftps,1)/vh)-tmp1^2)
      ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      dch=shift(dch,-loopstep,0)
      if velcor then begin
        junk1=2.*maxvel
        tmp1=tmp1-wcracv#replicate(1,rangegates)
        tmp1=tmp1 mod junk1
        junk2=where(tmp1 gt maxvel)
        if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]-junk1
        junk2=where(tmp1 le -maxvel)
        if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]+junk1
        dch[dml:msz1,*]=tmp1
      endif else dch[dml:msz1,*]=-50.
      ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    endif else begin
      vv =(vv-vvmn) > nosignal
;     vv =(vv^v_slope-vvmn) > nosignal                        ; V-channel corr.
      vvz=shift(vvz,-loopstep,0)
      vvz[dml:msz1,*]=10.*alog10(vv/avfac)+rngcor+vvcal+fftcor

    ; Calculate mean (duv) and standard deviation (vhz) Doppler velocities
      duv=shift(duv,-loopstep,0)
      vh=total(fftps,1)  ; temp use as a work array
      fftps=velo*fftps   ; fftps is not needed anymore so use it as a work array
      tmp1=transpose(total(fftps,1)/vh)
      duv[dml:msz1,*]=tmp1
      vhz=shift(vhz,-loopstep,0)
      vhz[dml:msz1,*]=sqrt(transpose(total(velo*fftps,1)/vh)-tmp1^2)
      ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      dcv=shift(dcv,-loopstep,0)
      if velcor then begin
        junk1=2.*maxvel
        tmp1=tmp1-wcracv#replicate(1,rangegates)
        tmp1=tmp1 mod junk1
        junk2=where(tmp1 gt maxvel)
        if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]-junk1
        junk2=where(tmp1 le -maxvel)
        if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]+junk1
        dcv[dml:msz1,*]=tmp1
      endif else dcv[dml:msz1,*]=-50.
      ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    endelse

  endif else begin
    ldr=shift(ldr,-loopstep,0) ; get rid of the oldest/first loopstep points
    vhz=shift(vhz,-loopstep,0)

    if alg eq sppalg then begin
      if txstrpol[0] eq 'H' then begin
        duh=shift(duh,-loopstep,0)
        tmp1=velfactor*atan(imaginary(pp),float(pp))
        duh[dml:msz1,*]=tmp1
        ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        dch=shift(dch,-loopstep,0)
        if velcor then begin
          junk1=2.*maxvel
          tmp1=tmp1-wcracv#replicate(1,rangegates)
          tmp1=tmp1 mod junk1
          junk2=where(tmp1 gt maxvel)
          if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]-junk1
          junk2=where(tmp1 le -maxvel)
          if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]+junk1
          dch[dml:msz1,*]=tmp1
        endif else dch[dml:msz1,*]=-50.
        ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        hh =(hh-hhmn) > nosignal
;       hh =(hh^h_slope-hhmn) > nosignal                      ; H-channel corr.
        if antconf eq 0 then begin
          vh =10.*alog10((vh-vvmn-isol*ldrcallin*hh)>zero)    ; single iso corr.
;         vh =(vh^v_slope-vvmn) > nosignal                    ; H-channel corr.
        endif
        hh =10.*alog10(hh)
        hhz=shift(hhz,-loopstep,0)
        hhz[dml:msz1,*]=hh+rngcor+hhcal
        if antconf eq 0 then begin
          ldr[dml:msz1,*]=vh-hh-ldrcal
          vhz[dml:msz1,*]=vh+rngcor+vvcal
        endif
      endif else begin
        duv=shift(duv,-loopstep,0)
        tmp1=velfactor*atan(imaginary(pp),float(pp))
        duv[dml:msz1,*]=tmp1
        ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        dcv=shift(dcv,-loopstep,0)
        if velcor then begin
          junk1=2.*maxvel
          tmp1=tmp1-wcracv#replicate(1,rangegates)
          tmp1=tmp1 mod junk1
          junk2=where(tmp1 gt maxvel)
          if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]-junk1
          junk2=where(tmp1 le -maxvel)
          if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]+junk1
          dcv[dml:msz1,*]=tmp1
        endif else dcv[dml:msz1,*]=-50.
        ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        vv =(vv-vvmn) > nosignal
;       vv  =(vv^v_slope-vvmn) > nosignal                     ; H-channel corr.
        if antconf eq 0 then begin
          hv =(hv-hhmn-isol/ldrcallin*vv)>zero
;         hv =(hv^h_slope-hhmn-isol/ldrcallin*vv)>zero        ; single iso corr.
          hv =10.*alog10(hv)                                  ; H-ch. corr.
        endif
        vv =10.*alog10(vv)
        vvz=shift(vvz,-loopstep,0)
        vvz[dml:msz1,*]=vv+rngcor+vvcal
        if antconf eq 0 then begin
          ldr[dml:msz1,*]=hv-vv+ldrcal
          vhz[dml:msz1,*]=hv+rngcor+hhcal
        endif
      endelse
    endif else begin
      if size(pp,/n_dim) eq 3 then begin
        duh=shift(duh,-loopstep,0)
        duv=shift(duv,-loopstep,0)
        duh[dml:msz1,*]=velfactor[0]*atan(imaginary(pp[*,*,0]),float(pp[*,*,0]))
        duv[dml:msz1,*]=velfactor[1]*atan(imaginary(pp[*,*,1]),float(pp[*,*,1]))
        ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        dch=shift(dch,-loopstep,0)
        dcv=shift(dcv,-loopstep,0)
        if velcor then begin
          junk1=2.*maxvel[ppind[0]]
          tmp1=duh[dml:msz1,*]-reform(wcracv[*,0])#replicate(1,rangegates)
          tmp1=tmp1 mod junk1
          junk2=where(tmp1 gt maxvel[ppind[0]])
          if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]-junk1
          junk2=where(tmp1 le -maxvel[ppind[0]])
          if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]+junk1
          dch[dml:msz1,*]=tmp1

          junk1=2.*maxvel[ppind[1]]
          tmp1=duv[dml:msz1,*]-reform(wcracv[*,1])#replicate(1,rangegates)
          tmp1=tmp1 mod junk1
          junk2=where(tmp1 gt maxvel[ppind[1]])
          if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]-junk1
          junk2=where(tmp1 le -maxvel[ppind[1]])
          if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]+junk1
          dcv[dml:msz1,*]=tmp1
        endif else begin
          dch[dml:msz1,*]=-50.
          dcv[dml:msz1,*]=-50.
        endelse
        ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      endif else begin
        if ppind[0] ne -1 then begin
          duh=shift(duh,-loopstep,0)
          tmp1=velfactor[0]*atan(imaginary(pp),float(pp))
          duh[dml:msz1,*]=tmp1
          ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
          dch=shift(dch,-loopstep,0)
          if velcor then begin
            junk1=2.*maxvel[ppind[0]]
            tmp1=tmp1-wcracv#replicate(1,rangegates)
            tmp1=tmp1 mod junk1
            junk2=where(tmp1 gt maxvel[ppind[0]])
            if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]-junk1
            junk2=where(tmp1 le -maxvel[ppind[0]])
            if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]+junk1
            dch[dml:msz1,*]=tmp1
          endif else dch[dml:msz1,*]=-50.
          ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        endif else begin
          duv=shift(duv,-loopstep,0)
          tmp1=velfactor[1]*atan(imaginary(pp),float(pp))
          duv[dml:msz1,*]=tmp1
          ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
          dcv=shift(dcv,-loopstep,0)
          if velcor then begin
            junk1=2.*maxvel[ppind[1]]
            tmp1=tmp1-wcracv#replicate(1,rangegates)
            tmp1=tmp1 mod junk1
            junk2=where(tmp1 gt maxvel[ppind[1]])
            if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]-junk1
            junk2=where(tmp1 le -maxvel[ppind[1]])
            if junk2[0] ne -1 then tmp1[junk2]=tmp1[junk2]+junk1
            dcv[dml:msz1,*]=tmp1
          endif else dcv[dml:msz1,*]=-50.
          ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        endelse
      endelse
      vv  =(vv-vvmn) > zero
;     vv  =(vv^v_slope-vvmn) > nosignal                        ; V-channel corr.
      hh  =(hh-hhmn) > nosignal
;     hh  =(hh^h_slope-hhmn) > nosignal                        ; H-channel corr.
      if antconf eq 0 then begin
        vh  =10.*alog10((vh-vvmn-alphaiso*isol*ldrcallin*hh $
                         -(1-alphaiso)*isol*vv) > zero)
;       vh  =(vh^v_slope-vvmn) > nosignal                      ; V-channel corr.
      endif
      vv  =10.*alog10(vv)
      hh  =10.*alog10(hh)
      hhz=shift(hhz,-loopstep,0)
      vvz=shift(vvz,-loopstep,0)
      zdr=shift(zdr,-loopstep,0)
      hhz[dml:msz1,*]=hh+rngcor+hhcal
      vvz[dml:msz1,*]=vv+rngcor+vvcal
      if antconf eq 0 then begin
        ldr[dml:msz1,*]=vh-hh-ldrcal
        zdr[dml:msz1,*]=hh-vv+zdrcal
        vhz[dml:msz1,*]=vh+rngcor+vvcal
      endif else begin
        ldr[dml:msz1,*]=-150.
        zdr[dml:msz1,*]=-150.
        vhz[dml:msz1,*]=-150.
      endelse
    endelse
  endelse

end ; wcrdispread

; -------------------------------------------------------------------------
; Read time stamp for the current displayed block of profiles

function wcrdisptime,data,k,btime

   common wcrdspcal
   common wcrdspacd
   common wcrdspp

   if n_params() gt 2 then begin
      rec=k-dml*avestep > btime
   endif else rec=k

   t=data[rec]
   if swapbytes eq 1 then t=swap_endian(t)
   timelabel=sec2timestr((double(t.header[tind[0]])+ $
             double(t.header[tind[1]]/1e6+tcor+thack)) mod 86400)
   return,timelabel[0]

end ; wcrdisptime

; -------------------------------------------------------------------------
; Build real-time display widget

pro wcrdispwidget

common wcrdspacd
common wcrdspp
common wcrdspd
common wcrdspcal
common wcrdspwdg
common wcrdspids
common wcrdspctrl

  ; Assign colorbar labels

  case image1 of
    'hh'   : begin & b1min=zzmins & b1max=zzmaxs   & b1label=' hh(dBZ) ' & end
    'vv'   : begin & b1min=zzmins & b1max=zzmaxs   & b1label=' vv(dBZ) ' & end
    'du_h' : begin & b1min=dvhmins & b1max=dvhmaxs & b1label='du_h(m/s)' & end
    'du_v' : begin & b1min=dvvmins & b1max=dvvmaxs & b1label='du_v(m/s)' & end
    'dc_h' : begin & b1min=dvhmins & b1max=dvhmaxs & b1label='dc_h(m/s)' & end
    'dc_v' : begin & b1min=dvvmins & b1max=dvvmaxs & b1label='dc_v(m/s)' & end
    'dg_h' : begin & b1min=dvhmins & b1max=dvhmaxs & b1label='dg_h(m/s)' & end
    'dg_v' : begin & b1min=dvvmins & b1max=dvvmaxs & b1label='dg_v(m/s)' & end
    'zdr'  : begin & b1min=zdmins & b1max=zdmaxs   & b1label=' zdr(dB) ' & end
    'ldr'  : begin & b1min=ldmins & b1max=ldmaxs   & b1label=' ldr(dB) ' & end
    'vh'   : begin & b1min=czmins & b1max=czmaxs   & b1label=' vh(dBZ) ' & end
    'sv'   : begin & b1min=svmins & b1max=svmaxs   & b1label=' sv(m/s) ' & end
  endcase

  case image2 of
     'hh'  : begin & b2min=zzmins & b2max=zzmaxs   & b2label=' hh(dBZ) ' & end
     'vv'  : begin & b2min=zzmins & b2max=zzmaxs   & b2label=' vv(dBZ) ' & end
     'du_h': begin & b2min=dvhmins & b2max=dvhmaxs & b2label='du_h(m/s)' & end
     'du_v': begin & b2min=dvvmins & b2max=dvvmaxs & b2label='du_v(m/s)' & end
     'dc_h': begin & b2min=dvhmins & b2max=dvhmaxs & b2label='dc_h(m/s)' & end
     'dc_v': begin & b2min=dvvmins & b2max=dvvmaxs & b2label='dc_v(m/s)' & end
     'dg_h': begin & b2min=dvhmins & b2max=dvhmaxs & b2label='dg_h(m/s)' & end
     'dg_v': begin & b2min=dvvmins & b2max=dvvmaxs & b2label='dg_v(m/s)' & end
     'zdr' : begin & b2min=zdmins & b2max=zdmaxs   & b2label=' zdr(dB )' & end
     'ldr' : begin & b2min=ldmins & b2max=ldmaxs   & b2label=' ldr(dB) ' & end
     'vh'  : begin & b2min=czmins & b2max=czmaxs   & b2label=' vh(dBZ) ' & end
     'sv'  : begin & b2min=svmins & b2max=svmaxs   & b2label=' sv(m/s) ' & end
     else  :
  endcase

  ; Build widget structure

  baseid=widget_base(title='Wyoming Cloud Radar "Kestrel" DISPLAY',/column)

  ; Main button group

    bg1id       =widget_base(baseid,/row,/frame)
       openid   =widget_button(bg1id,value='New',xsize=50,event_pro='bg1_ev')
       dummyid  =widget_label(bg1id,value=' ',xsize=15)
       nimgid   =widget_button(bg1id,value='Dsply '+num2str(nimg)+' img',event_pro='bg1_ev')
       dvcid    =widget_droplist(bg1id,value=['DVC OFF','DVC ON','DVC GPS'], $
                                 event_pro='bg1_ev')
       dummyid  =widget_label(bg1id,value=' ')
       pauseid  =widget_button(bg1id,value='Pause/Cursor',event_pro='bg1_ev')
       catchid  =widget_button(bg1id,value='Catch',xsize=50,event_pro='bg1_ev')
       colorid  =widget_button(bg1id,value='Color',xsize=50,event_pro='bg1_ev')
       dummyid  =widget_label(bg1id,value=' ')
       quitid   =widget_button(bg1id,value='Quit',xsize=50,event_pro='bg1_ev')
       destroyid=widget_button(bg1id,value='Destroy',xsize=70, $
                               event_pro='bg1_ev')
       dummyid=widget_label(bg1id,value=' ',xsize=18)
       helpid   =widget_button(bg1id,value='Help',xsize=50,event_pro='bg1_ev')

  ; Display control group

    dispcid   =widget_base(baseid,column=3,/align_center)

       dispc1id =widget_base(dispcid,column=3,/frame)
         dummyid=widget_label(dispc1id,value='Antennae')
         antid  =widget_droplist(dispc1id,value=['U/S','D,D_f','S,S_f',      $
                                 'U/S,D','U,S_f'],event_pro='bg2_ev')
         dummyid=widget_label(dispc1id,value='Doppler')
         dvid   =widget_droplist(dispc1id,value=dvlabels,xsize=100,            $
                                 event_pro='bg2_ev')
         dispc11id =widget_base(dispc1id,/column)
           dummyid=widget_label(dispc11id,value='Synch')
           thackid=widget_text(dispc11id,/edit,value=num2str(thack,fr=2)+'s',  $
                               xsize=5,event_pro='dispc_ev')
       dispc2id=widget_base(dispcid,column=4,/frame)
         dispc21id =widget_base(dispc2id,/column)
         inittid=widget_label(dispc21id,value='InitTime')
         startid=widget_text(dispc21id,/edit,value=starttime,xsize=8,          $
                             event_pro='bg1_ev')
         dispc22id =widget_base(dispc2id,/column)
         dummyid=widget_label(dispc22id,value='AvProf')
         aveid  =widget_text(dispc22id,/edit,value=num2str(avestep),xsize=4,   $
                             event_pro='dispc_ev')
         dispc23id =widget_base(dispc2id,/column)
         dummyid=widget_label(dispc23id,value='DspStp')
         stepid =widget_text(dispc23id,/edit,value=num2str(loopstep),xsize=4,  $
                             event_pro='dispc_ev')
         dispc24id =widget_base(dispc2id,/column)
         dummyid=widget_label(dispc24id,value='Sleep')
         sleepid =widget_text(dispc24id,/edit,value=num2str(sleept,fr=1),      $
                              xsize=4,event_pro='dispc_ev')

       dispc3id=widget_base(dispcid,column=3,/frame)
         dispc31id =widget_base(dispc3id,/column)
         dummyid=widget_label(dispc31id,value='hhcal')
         hhcalid=widget_text(dispc31id,/edit,value=num2str(hhcal,fr=1),        $
                             xsize=4,event_pro='dispc_ev')
         dispc32id =widget_base(dispc3id,/column)
         dummyid=widget_label(dispc32id,value='vvcal')
         vvcalid=widget_text(dispc32id,/edit,value=num2str(vvcal,fr=1),        $
                             xsize=4,event_pro='dispc_ev')
         dispc33id =widget_base(dispc3id,/column)
         dummyid=widget_label(dispc33id,value='isol')
         isolid =widget_text(dispc33id,/edit,value=num2str(isoldb,fr=1),       $
                             xsize=4,event_pro='dispc_ev')

  ; Top image (image1) widget

    if alg eq fftalg then vhlabel='sv' else vhlabel='vh'

    image1id =widget_base(baseid,/column,/frame)
    bg2id=widget_base(image1id,/row)
      dummyid=widget_label(bg2id,value='     Range    |  Image:   ')
      bg2img1id=widget_base(bg2id,/row,/exclusive)
       hh1id =widget_button(bg2img1id,value='hh',xsize=50,event_pro='bg2_ev')
       vv1id =widget_button(bg2img1id,value='vv',xsize=50,event_pro='bg2_ev')
       vh1id =widget_button(bg2img1id,value=vhlabel,xsize=50,event_pro='bg2_ev')
       ldr1id=widget_button(bg2img1id,value='ldr',xsize=50,event_pro='bg2_ev')
       zdr1id=widget_button(bg2img1id,value='zdr',xsize=50,event_pro='bg2_ev')
       dvh1id=widget_button(bg2img1id,value=dvlabel[0],xsize=55,event_pro='bg2_ev')
       dvv1id=widget_button(bg2img1id,value=dvlabel[1],xsize=55,event_pro='bg2_ev')
    base1id       =widget_base(image1id,/row)
      base11id    =widget_base(base1id,/row)
        axisid    =widget_base(base11id,/column)
          axis1id =widget_draw(axisid,xsize=axsize,ysize=ysize)
          dummyid =widget_label(axisid,value='kft    km')
        ; for next line y_scroll=ysize is what is needed; +1 is a MS Windows
        ; bug workaround
        scroll1id =widget_draw(base11id,xsize=maximgsz,ysize=ysize, $
                               x_scroll=wxsize,y_scroll=ysize+1,retain=2)
        bar1id   =widget_draw(base11id,xsize=bxsize,ysize=ysize,/frame)
      base12id   =widget_base(base1id,/column)
        b1maxid  =widget_text(base12id,/align_left,value=b1max,xsize=5, $
                              /editable,event_pro='minmax_ev')
        dummyid   =widget_label(base12id,value=' ')
        if rangegates gt 100 then dummyid   =widget_label(base12id,value=' ')
        b1labelid =widget_label(base12id,value=b1label)
        dummyid   =widget_label(base12id,value=' ', $
                   scr_ysize=((rangegates>minimgsz)-100)>30)
        dummyid   =widget_label(base12id,value=' ')
        b1minid   =widget_text(base12id,/align_left,value=b1min,xsize=5, $
                              /editable,event_pro='minmax_ev')

  ; Time labels and wcr info text

    timeid     =widget_base(baseid,/row,/frame)
      btlabelid=widget_label(timeid,/frame,value=' ',xsize=75)
      fileid   =widget_label(timeid,value=' ',/align_center,xsize=450)
      etlabelid=widget_label(timeid,/frame,value=starttime,xsize=75)

  ;  Bottom image (image2) widget

  if nimg eq 2 then begin
     image2id =widget_base(baseid,/column,/frame)
     bg2id=widget_base(image2id,/row)
      dummyid=widget_label(bg2id,value='     Range    |  Image:   ')
      bg2img2id=widget_base(bg2id,/row,/exclusive)
       hh2id =widget_button(bg2img2id,value='hh',xsize=50,event_pro='bg2_ev')
       vv2id =widget_button(bg2img2id,value='vv',xsize=50,event_pro='bg2_ev')
       vh2id =widget_button(bg2img2id,value=vhlabel,xsize=50,event_pro='bg2_ev')
       ldr2id=widget_button(bg2img2id,value='ldr',xsize=50,event_pro='bg2_ev')
       zdr2id=widget_button(bg2img2id,value='zdr',xsize=50,event_pro='bg2_ev')
       dvh2id=widget_button(bg2img2id,value=dvlabel[0],xsize=55,event_pro='bg2_ev')
       dvv2id=widget_button(bg2img2id,value=dvlabel[1],xsize=55,event_pro='bg2_ev')
     base2id       =widget_base(image2id,/row)
       base21id    =widget_base(base2id,/row)
         axisid    =widget_base(base21id,/column)
           axis2id =widget_draw(axisid,xsize=axsize,ysize=ysize)
           dummyid =widget_label(axisid,value='kft    km')
         ; for next line y_scroll=ysize is what is needed; +1 is a MS Windows
         ; bug workaround
         scroll2id =widget_draw(base21id,xsize=maximgsz,ysize=ysize, $
                                x_scroll=wxsize,y_scroll=ysize+1,retain=2)
         bar2id    =widget_draw(base21id,xsize=bxsize,ysize=ysize,/frame)
       base22id    =widget_base(base2id,/column)
         b2maxid   =widget_text(base22id,/align_left,value=b2max,xsize=5, $
                               /editable,event_pro='minmax_ev')
         dummyid   =widget_label(base22id,value=' ')
         if rangegates gt 100 then dummyid   =widget_label(base22id,value=' ')
         b2labelid =widget_label(base22id,value=b2label)
         dummyid   =widget_label(base22id,value=' ', $
                    scr_ysize=((rangegates>minimgsz)-100)>30)
         b2minid   =widget_text(base22id,/align_left,value=b2min,xsize=5, $
                               /editable,event_pro='minmax_ev')
  endif

  ; Status message line

  statbaseid=widget_base(baseid,column=8)
    statbase1id =widget_base(statbaseid,/column)
      statlabelid=widget_label(statbase1id,value='Status:',/align_left)
      statusmsgid=widget_text(statbase1id,value=' ',xsize=43)
    statbase2id =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase2id,value='<Hn>')
      statmnhnid =widget_text(statbase2id,value='-99.9',xsize=5)
    statbase3id =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase3id,value='mxHn')
      statmxhnid =widget_text(statbase3id,value='-99.9',xsize=5, $
                              /editable,event_pro='minmax_ev')
    statbase4id =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase4id,value='<Vn>')
      statmnvnid =widget_text(statbase4id,value='-99.9',xsize=5)
    statbase5id =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase5id,value='mxVn')
      statmxvnid =widget_text(statbase5id,value='-99.9',xsize=5, $
                              /editable,event_pro='minmax_ev')
    statbase6id  =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase6id,value='Hleak')
      stathleakid=widget_text(statbase6id,value='-99.9',xsize=4)
    statbase8id  =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase8id,value='Vleak')
      statvleakid=widget_text(statbase8id,value='-99.9',xsize=4)
    statbase7id  =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase7id,value='mxdT')
      statdtid   =widget_text(statbase7id,value='0',xsize=4, $
                             /editable,event_pro='minmax_ev')
end ; wcrdispwidget

; -------------------------------------------------------------------------
; Control widget events from 'color' table button, 'quit', 'new', 'destroy',
; 'pause/cursor', 'init time', 'imgord'

pro bg1_ev,ev

   common wcrdspacd
   common wcrdspp
   common wcrdspd
   common wcrdspcal
   common wcrdspwdg
   common wcrdspids
   common wcrdspctrl

   case ev.id of
     colorid : begin
        widget_control,statusmsgid,set_value='Waiting for a new color table.'
        xloadct,group=baseid,/block
        ct2ind=wyoctind()
        if ct2ind eq -1 then ct2ind=!d.table_size
        wset,barw1
        if image1 eq 'du_h' or image1 eq 'dc_h' or image1 eq 'dg_h' then    $
           wcrdispbar,(ct2ind lt !d.table_size)+1,                          $
                       ct2ind,rangegates,bxsize,vmin=dvhmin,vmax=dvhmax     $
        else if image1 eq 'du_v' or image1 eq 'dc_v' or image1 eq 'dg_v'    $
           then wcrdispbar,(ct2ind lt !d.table_size)+1,                     $
                       ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax     $
        else if image1 eq 'sv' then                                         $
           wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax      $
        else if image1 eq 'zdr' then wcrdispbar,(ct2ind lt !d.table_size)+1,$
           ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax                   $
        else if image1 eq 'hh' or image1 eq 'vv' then                       $
           wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax      $
        else if image1 eq 'ldr' then                                        $
           wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax      $
        else if image1 eq 'vh' then                                         $
             wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax

        if image2 ne ' ' then begin
           wset,barw2
           if image2 eq 'du_h' or image2 eq 'dc_h' or image2 eq 'dg_h' then $
              wcrdispbar,(ct2ind lt !d.table_size)+1,                       $
                       ct2ind,rangegates,bxsize,vmin=dvhmin,vmax=dvhmax     $
           else if image2 eq 'du_v' or image2 eq 'dc_v' or image2 eq 'dg_v' $
              then wcrdispbar,(ct2ind lt !d.table_size)+1,                  $
                       ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax     $
           else if image1 eq 'sv' then                                         $
              wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax      $
           else if image2 eq 'zdr' then wcrdispbar,(ct2ind lt !d.table_size)+1,$
              ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax                   $
           else if image2 eq 'hh' or image2 eq 'vv' then                       $
              wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax      $
           else if image2 eq 'ldr' then                                        $
              wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax      $
           else if image2 eq 'vh' then                                         $
                wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
        endif
               end
     quitid    : quitw=1B
     openid    : begin
                  openwf=1B
                  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                  ; next lines are commented out to allow using the same ac file
                  ; when a new wcr file is from the same flight
                  ;if ncf ne -1 then begin
                  ;   ncdf_close,ncf
                  ;   ncf=-1 & thack=thacki
                  ;   widget_control,thackid,set_value=num2str(thack,fr=2)+'s'
                  ;endif
                  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                 end
     catchid   : catche=1B
     startid   : starte=1B
     destroyid : destroyw=1B
     pauseid   : begin
                   widget_control,pauseid,get_value=bl
                   if bl eq 'Continue' then begin
                      widget_control,pauseid,set_value='Pause/Cursor'
                    ; widget_control,pauseid,sensitive=1
                      widget_control,quitid,sensitive=1
                      widget_control,openid,sensitive=1
                      widget_control,destroyid,sensitive=1
                      return
                   endif
                   widget_control,pauseid,set_value='Continue'
                   widget_control,quitid,sensitive=0
                   widget_control,openid,sensitive=0
                   widget_control,destroyid,sensitive=0
                 end
     dvcid     : begin
                   ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                   junkid=velcor
                   velcor=0
                   if ev.index gt 0 then velcor=1 
                   if ev.index eq 2 then begin
                     if n_elements(uerrid) gt 0 then begin 
                       if uerrid ne -1 and verrid ne -1 then $
                         gpsacc=1 else gpsacc=0
                     endif else gpsacc=1
                   endif else gpsacc=0

                   if velcor then begin
                     if gpsacc then $
                       dvlabel=['dg_H','dg_V'] $
                     else dvlabel=['dc_H','dc_V']
                     if ppind[0] ge 0 and ppind[1] ge 0 then  $
                       dvlabels=[dvlabel[0]+','+dvlabel[1],   $
                                 'du_H,du_V'              ,   $
                                 dvlabel[0]+',du_H'       ,   $
                                 dvlabel[1]+',du_V'         ] $
                     else if ppind[0] ge 0 then begin
                       if ppind[1] ne -2 then begin
                         dvlabels=[dvlabel[0]+',du_H']
                         dvlabel=[dvlabel[0],'du_H']
                       endif else begin ; this case will happen only for operator wrong selection
                         dvlabels=['du_'+ppstrpol[0]]
                         ; dvlabel - have to think what to do; for now leave it wrong
                       endelse
                     endif else begin
                       dvlabels=[dvlabel[1]+',du_V']
                       dvlabel=[dvlabel[1],'du_V']
                     endelse

                     if junkid eq 0 then wcrdispacopen_nc ; can return velcor=0
                   endif

                   if velcor eq 0 then begin
                     dvlabel=['du_H','du_V']
                     dvlabels='du_H,du_V'
                     if alg eq sppalg or alg eq fftalg then begin
                       if txsequence[0] eq polh then begin
                         widget_control,dvv1id,sensitive=0
                         widget_control,dvv2id,sensitive=0
                         dvlabels='du_H'
                       endif else begin
                         widget_control,dvh1id,sensitive=0
                         widget_control,dvh2id,sensitive=0
                         dvlabels='du_V'
                       endelse
                     endif
                     widget_control,thackid,sensitive=0
                   endif else begin
                     widget_control,dvh1id,sensitive=1
                     widget_control,dvh2id,sensitive=1
                     widget_control,dvv1id,sensitive=1
                     widget_control,dvv2id,sensitive=1
                     widget_control,thackid,sensitive=1
                   endelse

                   widget_control,dvid,set_value=dvlabels
                   widget_control,dvcid,set_droplist_select=velcor+gpsacc*(velcor eq 1)

                   if n_elements(dvlabels) eq 1 then begin
                     widget_control,dvid,sensitive=0
                   endif else begin
                     widget_control,dvid,sensitive=1
                     widget_control,dvid,set_droplist_select=0
                   endelse

                   widget_control,dvh1id,set_value=dvlabel[0]
                   widget_control,dvv1id,set_value=dvlabel[1]
                   if strmid(image1,0,1) eq 'd' then begin
                     image1=strlowcase(dvlabel[0])
                     widget_control,dvh1id,set_button=1
                     if velcor eq 0 and (alg eq sppalg or alg eq fftalg) then begin
                       if txsequence[0] eq polv and strmid(image1,3,1) eq 'h' then begin
                         image1='du_v'
                         widget_control,dvv1id,set_button=1
                         if image2 eq image1 and nimg eq 2 then begin
                           image1='vv' 
                           widget_control,vv1id,set_button=1
                         endif
                       endif
                     endif
                   endif
                   if nimg eq 2 then begin
                     widget_control,dvh2id,set_value=dvlabel[0]
                     widget_control,dvv2id,set_value=dvlabel[1]
                     if strmid(image2,0,1) eq 'd' then begin
                       image2=strlowcase(dvlabel[1])
                       widget_control,dvv2id,set_button=1
                       if velcor eq 0 and (alg eq sppalg or alg eq fftalg) then begin
                         if txsequence[0] eq polh then begin
                           if strmid(image2,3,1) eq 'v' then begin
                             if strmid(image1,0,1) eq 'd' then begin
                               image2=image1 & image1='hh'
                               widget_control,hh1id,set_button=1
                               widget_control,dvh2id,set_button=1
                             endif else begin
                               image2=strlowcase(dvlabel[0])
                               widget_control,dvh2id,set_button=1
                             endelse
                           endif else if image1 eq image2 then begin
                             image2=image1 & image1='hh'
                             widget_control,hh1id,set_button=1
                             widget_control,dvh2id,set_button=1
                           endif
                         endif else begin 
                           if strmid(image2,3,1) eq 'h' then begin
                             if strmid(image1,0,1) eq 'd' then begin
                               image2=image1 & image1='vv'
                               widget_control,vv1id,set_button=1
                               widget_control,dvv2id,set_button=1
                             endif else begin
                               image2=strlowcase(dvlabel[1])
                               widget_control,dvh2id,set_button=1
                             endelse
                           endif else if image1 eq image2 then begin
                             image2=image1 & image1='vv'
                             widget_control,vv1id,set_button=1
                             widget_control,dvv2id,set_button=1
                           endif
                         endelse
                       endif
                     endif
                   endif
                   if antconf eq 4 then begin
                     img1order=(image1 eq 'vv' or image1 eq 'du_v' or image1 eq 'dc_v' or image1 eq 'dg_v')
                     img2order=(image2 eq 'vv' or image2 eq 'du_v' or image2 eq 'dc_v' or image2 eq 'dg_v')
                   endif
                 end
     nimgid    : begin
                  nimg=(nimg eq 2)+2*(nimg eq 1)
                  timeintm=(mxhn=(mxvn=-99.9))
                  widget_control,baseid,/destroy
                  wcrdispwidget
                  widget_control,dvcid,set_droplist_select=velcor+gpsacc*(velcor eq 1)
                          ; remove logical op when antconf 3 is available
                  widget_control,antid,set_droplist_select=antconf-(antconf gt 3)
                  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                  if n_elements(dvlabels) eq 1 then begin
                    widget_control,dvid,sensitive=0
                    if velcor then widget_control,thackid,sensitive=1 $
                    else widget_control,thackid,sensitive=0
                  endif
                  ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                  widget_control,baseid,/realize

                  if alg eq sppalg or alg eq fftalg then begin
                     widget_control,zdr1id,sensitive=0
                     if txstrpol[0] eq 'H' then begin
                       widget_control,vv1id,sensitive=0
                       if velcor eq 0 then widget_control,dvv1id,sensitive=0
                       widget_control,vh1id,set_value='vh'
                     endif else begin
                       widget_control,hh1id,sensitive=0
                       if velcor eq 0 then widget_control,dvh1id,sensitive=0
                       widget_control,vh1id,set_value='hv'
                     endelse
                     if alg eq fftalg then begin
                       widget_control,ldr1id,sensitive=0
                       widget_control,vh1id,set_value='sv'
                     endif
                     if nimg eq 2 then begin
                       widget_control,zdr2id,sensitive=0
                       if txstrpol[0] eq 'H' then begin
                         widget_control,vv2id,sensitive=0
                         widget_control,vh2id,set_value='vh'
                         if velcor eq 0 then widget_control,dvv2id,sensitive=0
                       endif else begin
                         widget_control,hh2id,sensitive=0
                         widget_control,vh2id,set_value='hv'
                         if velcor eq 0 then widget_control,dvh2id,sensitive=0
                       endelse
                       if alg eq fftalg then begin
                         widget_control,ldr2id,sensitive=0
                         widget_control,vh2id,set_value='sv'
                       endif
                     endif
                  endif

                  if antconf gt 0 then begin
                    widget_control,ldr1id,sensitive=0
                    widget_control,zdr1id,sensitive=0
                    if alg ne fftalg then widget_control,vh1id,sensitive=0
                    if nimg eq 2 then begin
                      widget_control,ldr2id,sensitive=0
                      widget_control,zdr2id,sensitive=0
                      if alg ne fftalg then widget_control,vh2id,sensitive=0
                    endif
                  endif

                  if alg eq fftalg then begin
                    prfs=num2str(prf[0]/1000,fr=0)
                    widget_control,fileid,set_value=wcrfile+'  /FFT: '+ $
                      txstrpol[0]+'/'+strmid(prfs,0,2)+'KHz/'+ $
                      num2str(maxvel[0],fr=1)+'m/s'
                  endif else begin
                    prfs=velfactor*!pi
                    strpols=string(reform(byte(txstrpol)))
                    prfs=string(reform(byte(num2str(prfs,fr=1,len=4)+','),   $
                                5*n_elements(prfs)))
                    widget_control,fileid, set_value=wcrfile+'  /'+strpols+  $
                      ' maxvel:'+ prfs+' #av:'+num2str(averages)+', pulse:'+ $
                      num2str(pulselength)+'/'
                  endelse

                  widget_control,axis1id,get_value=axisw1
                  widget_control,scroll1id,get_value=scrollw1
                  widget_control,bar1id,get_value=barw1
                  wset,barw1
                  if image1 eq 'du_h' or image1 eq 'dc_h' or image1 eq 'dg_h' $
                    then wcrdispbar,(ct2ind lt !d.table_size)+1,              $
                      ct2ind,rangegates,bxsize,vmin=dvhmin,vmax=dvhmax        $
                  else if image1 eq 'du_v' or image1 eq 'dc_v' or image1 eq   $
                  'dg_v' then wcrdispbar,(ct2ind lt !d.table_size)+1,         $
                      ct2ind,rangegates,bxsize,vmin=dvhmin,vmax=dvvmax        $
                  else if image1 eq 'sv' then                                 $
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,         $
                      vmax=svmax                                              $
                  else if image1 eq 'zdr' then                                $
                    wcrdispbar,(ct2ind lt !d.table_size)+1,                   $
                      ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax          $
                  else if image1 eq 'hh' or image1 eq 'vv' then               $
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,         $
                      vmax=zzmax                                              $
                  else if image1 eq 'ldr' then                                $
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,         $
                      vmax=ldmax                                              $
                  else if image1 eq 'vh' then                                 $
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,         $
                      vmax=czmax
                  widget_control,scroll1id,set_draw_view=[maximgsz,0]

                  wset, axisw1 & erase
                  junkid=((minimgsz-rangegates)*rangegatespacing)>0
                  ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                         junkid*(img1order eq 1))/1000.
                  plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                    xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                  xl_cm=!d.x_size/!d.x_px_cm
                  tkly=-1./xl_cm
                  axis,0.45,ax_yrange[0]/0.305,/nodat,ytickle=tkly, $
                    yaxis=0,ystyle=1, yrange=!y.crange*(img1order eq 0)+ $
                    reverse(!y.crange)*(img1order eq 1),chars=1.1,/normal
                  plot,ax_yrange,/nodata,/noerase,/normal, $
                    xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                  axis,1.0,ax_yrange[0],/nodata,yticklen=tkly,yaxis=0, $
                    ystyle=1,yrange=!y.crange*(img1order eq 0)+$
                    reverse(!y.crange)*(img1order eq 1),chars=1.1,/normal

                  case image1 of
                    'hh'  : widget_control,hh1id,set_button=1
                    'vv'  : widget_control,vv1id,set_button=1
                    'du_h': widget_control,dvh1id,set_button=1
                    'du_v': widget_control,dvv1id,set_button=1
                    'dc_h': widget_control,dvh1id,set_button=1
                    'dc_v': widget_control,dvv1id,set_button=1
                    'dg_h': widget_control,dvh1id,set_button=1
                    'dg_v': widget_control,dvv1id,set_button=1
                    'zdr' : widget_control,zdr1id,set_button=1
                    'ldr' : widget_control,ldr1id,set_button=1
                    'vh'  : widget_control,vh1id,set_button=1
                    'sv'  : widget_control,vh1id,set_button=1
                  endcase

                  if nimg eq 2 then begin
                    widget_control,axis2id,get_value=axisw2
                    widget_control,scroll2id,get_value=scrollw2
                    widget_control,bar2id,get_value=barw2
                    wset,barw2
                    if image2 eq 'du_h' or image2 eq 'dc_h' or image2 eq    $
                      'dg_h' then wcrdispbar,(ct2ind lt !d.table_size)+1,  $
                      ct2ind,rangegates,bxsize,vmin=dvhmin,vmax=dvhmax  $
                    else if image2 eq 'du_v' or image2 eq 'dc_v' or image2 $
                    eq 'dg_v' then wcrdispbar,(ct2ind lt !d.table_size)+1, $
                      ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvhmax  $
                    else if image2 eq 'sv' then                     $
                      wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin, $
                        vmax=svmax                              $
                    else if image2 eq 'zdr' then                    $
                      wcrdispbar,(ct2ind lt !d.table_size)+1,           $
                        ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax  $
                    else if image2 eq 'hh' or image2 eq 'vv' then       $
                      wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin, $
                        vmax=zzmax                              $
                    else if image2 eq 'ldr' then                    $
                      wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin, $
                        vmax=ldmax                              $
                    else if image2 eq 'vh' then                     $
                      wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin, $
                        vmax=czmax
                    widget_control,scroll2id,set_draw_view=[maximgsz,0]

                    wset, axisw2 & erase
                    ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                      junkid*(img2order eq 1))/1000.
                    axis,1.0,ax_yrange[0],/nodat,yticklen=tkly,yaxis=0,$
                      ystyle=1,yrange=!y.crange*(img2order eq 0)+ $
                      reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal
                    plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                      xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                    axis,0.45,ax_yrange[0]/0.305,/nodata,yticklen=tkly, $
                      yaxis=0,ystyle=1,yrange=!y.crange*(img2order eq 0)+ $
                      reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal

                    case image2 of
                     'hh'  : widget_control,hh2id,set_button=1
                     'vv'  : widget_control,vv2id,set_button=1
                     'du_h': widget_control,dvh2id,set_button=1
                     'du_v': widget_control,dvv2id,set_button=1
                     'dc_h': widget_control,dvh2id,set_button=1
                     'dc_v': widget_control,dvv2id,set_button=1
                     'dg_h': widget_control,dvh2id,set_button=1
                     'dg_v': widget_control,dvv2id,set_button=1
                     'zdr' : widget_control,zdr2id,set_button=1
                     'ldr' : widget_control,ldr2id,set_button=1
                     'vh'  : widget_control,vh2id,set_button=1
                     'sv'  : widget_control,vh2id,set_button=1
                    endcase
                 endif
                 end
     helpid    : begin

junk=["                   Description of WCR Display GUI                    ",$
"                                                                           ",$
"For more help on the use of wcrdisplay, exit the program and run           ",$
"wcrdisplay,/help from the IDL prompt. To exit this help press              ",$
"'Done with WCR Display Help'                                               ",$
"                                                                           ",$
"This is a blocking widget and only one copy per idl session is allowed.    ",$
"                                                                           ",$
"'Enter' from the keyboard must be used after modifying an editable field   ",$
" for the new value to take effect                                          ",$
"                                                                           ",$
"Widget user interface:                                                     ",$
"                                                                           ",$
"New:          Open a dialog_pickfile to select a new WCR file.             ",$
"                                                                           ",$
"DVC * :       Drop-list menu handling Doppler velocity correction for A/C  ",$
"              motion                                                       ",$
"              * OFF - no correction                                        ",$
"                ON  - correction applied                                   ",$
"                GPS - correction applied including GPS correction to A/C   ",$
"                      ground velocity                                      ",$
"              Note:  Positive Doppler velocity is toward the radar. There  ",$
"                     is one exception to this rule for down-up antenas     ",$
"                     configuration(4) simultaneously displaying velocity   ",$
"                     for both nadir and up antennas on both windows.In this",$
"                     case nadir velocity is displayed in reversed color and",$ 
"                     positive is away from the radar.  This is done to     ",$
"                     maintain visual continuity between the two velocity   ",$
"                     images.                                               ",$
"                                                                           ",$
"Dsply 2 img:  Pressing this button will toggle between 1 and 2 image widget",$
"                                                                           ",$
"Pause/Cursor: Pressing this button will pause the display.  It also allows ",$
"              to extract image values by clicking the left button of the   ",$
"              mouse at a desired pixel of the images.  In case of two      ",$
"              images, clicking on either image will also extract the       ",$
"              corresponding value from the other image.  The values and    ",$
"              their time stamps are printed in the status line.            ",$
"Continue:     Continue label replaces Pause/Cursor label when Pause is     ",$
"              active.  Pressing Continue will restore updating the display ",$
"              with new data.                                               ",$
"                                                                           ",$
"Catch:        Pressing this button will force the display to position      ",$
"              at the end of the currently available data in the wcr file.  ",$
"              This feature is useful if the display get behind the         ",$
"              incoming data (this could happen for various reasons,        ",$
"              including using Pause/Cursor)                                ",$
"                                                                           ",$
"Color:        Allow change in the color table being used.  The default     ",$
"              color table is a custom dual-table to allow different colors ",$
"              for, e.g., reflectivity and velocity images.  If the default ",$
"              color table is replaced the program checks if a narrow black ",$
"              strip exists in the middle of the new color table. If it does",$
"              the color table splits in two otherwise the same color       ",$
"              table is used for both images                                ",$
"                                                                           ",$
"Destroy:      Terminates the program and destroys the widget               ",$
"                                                                           ",$
"Quit:         Terminates the program without destroying the widget.        ",$
"              This could be used to preserve interesting images for        ",$
"              later review and notes.                                      ",$
"                                                                           ",$
"Antennae:     Drop-list menu for the antenna configuration deployed:       ",$
"                  U/S   - one up/side antenna, dual polarization           ",$
"                  D,D_f - down-fore(H) and nadir(V) pointing antenae       ",$
"                  S,S_f - up/side(H) and side-fore(V) antennae             ",$
"                  U/S,D - up/side(H) and nadir(V) antennae                 ",$
"                  U,S_f - up(H) and side-fore(V) antennae                  ",$
"              When selecting one of the antenna configurations the image   ",$
"              order is adjusted accordingly and if DOPCOR option is set    ",$
"              applies the appropriate antenna beam coordinates             ",$
"                                                                           ",$
"Doppler:      Drop-lits menu for Doppler velocity processing (dopcor       ",$
"              keyword must be set):                                        ",$
"                 dc_H,dc_V - corrected for A/C motion (no gps correction)  ",$
"                 du_H,du_V - uncorrected for A/C motion                    ",$
"                 dc_H,du_H - corr. and uncorr. Dopp. for H-beam only       ",$
"                 dc_V,du_V - corr. and uncorr. Dopp. for V-beam only       ",$
"              'c' is replaced by 'g' if dopcor=-1 (applied gps correction) ",$
"                                                                           ",$
"Synch:        If DOPCOR keyword is used this editable field will show up.  ",$
"              The default values is 0.0 sec and represent any mismatch     ",$
"              between A/C and WCR clocks.  If such mismatch exists and is  ",$
"              known it can be entered as a positive or negative value.     ",$
"              The time synch correction is applied to the wcr clock.       ",$
"              This should not be confused with GPS leap seconds, which     ",$
"              are read from the A/C data file or the default value         ",$
"              (currently -13 sec) and applied to wcr data independantly.   ",$
"              Entering 999 will restore the default value or the value     ",$
"              defined by timesync keyword.                                 ",$
"                                                                           ",$
"InitTime:     Editable field.  Contains the starting time of wcr data.     ",$
"              Editing the field will cause the display to start again      ",$
"              from time close to the entered time.  This field should      ",$
"              not be used during data acquisition.  It is intended         ",$
"              for browsing of already recorded files.  The new time must   ",$
"              have one of the following formats: hhmm, hh:mm, hhmmss,      ",$
"              hh:mm:ss, 9999 . Incorrect entries are ignored and the start ",$
"              time is redisplayed. If enter 9999 the field will show the   ",$
"              end time of the file.                                        ",$
"                                                                           ",$
"DspStp:       Editable field. Number of profiles to be calibrated and      ",$
"              displayed simultaneously.  Must be greater than 0. Non-valid ",$
"              entries are ignored and the default value is used.           ",$
"                                                                           ",$
"AvProf:       Editable field. Number of raw profiles averaged before       ",$
"              display.  The default is aspect ratio which scales range:time",$
"              to 1:1 based on acspeed (default 90 m/s).  Must be greater   ",$
"              than 0.  If input is 0 the default (aspect ratio) is used.   ",$
"              DispStep and AveProfs cannot be equal to 1 simultaneously.   ",$
"              In this case the second one to be changed to one will be     ",$
"              replaced automatically by 2.                                 ",$
"                                                                           ",$
"Sleep:        Editable field. Sleep time[s] between disk access attempts   ",$
"              for 'Empty/short file' or 'End of file. Waiting for data...' ",$
"              conditions. Default is calculated based on DspStp, AvPorf    ",$
"              and an estimate of the time interval between profiles.       ",$
"              This field may be used in real-time display mode to reduce   ",$
"              hard disk access and SCSI load                               ",$
"                                                                           ",$
"hhcal:        Editable field. Calibration constant (dB) for H-channel      ",$
"              (default value is for side/up antenna H-pol)                 ",$
"vvcal:        Editable field. Calibration constant (dB) for V-channel      ",$
"              (default value is for side/up V-pol or nadir antennas)       ",$
"isol:         Editable field. Isolation (dB) between H- and V-channel      ",$
"                                                                           ",$
"Radar products (image selection):                                          ",$
"   Availability depends on wcr operational mode and antenna configuration  ",$
"   hh:   co-reflectivity for H-ch/beam (dBZ)                               ",$
"   vv:   co-reflectivity for V-ch/beam (dBZ)                               ",$
"   vh:   cross-reflectivity; H-ch transmitted, V-ch received               ",$
"   ldr:  vh/hh or hv/vv in dB                                              ",$
"   zdr:  hh/vv in dB                                                       ",$
"   d*_h: uncorrected/corrected for the platform motion mean Doppler        ",$
"         velocity in m/s for H channel(antenna)                            ",$
"   d*_v: uncorrected/corrected for the platform motion mean Doppler        ",$
"         velocity in m/s for V channel(antenna)                            ",$
"   '*'   is 'u' for uncorrected velocity; 'c' for A/C motion corrected     ",$
"         velocity and 'g' for A/C motion corrected using GPS corrections   ",$
"   sv:   spectral width in m/s (for wcr FFT mode only)                     ",$
"                                                                           ",$
"Color bar limits: these are editable fields and the default values         ",$
"                  can be replaced.  To activate the new entry 'Enter'      ",$
"                  key must be hit after typing the new limit.  There is    ",$
"                  no check for the validity of the entries.                ",$
"                  Non-numerical entries are replaced with 0 and a non-fatal",$
"                  error message is issued.                                 ",$
"Status:    Non editable infomation message                                 ",$
"  <Hn>:    Non editable information indicator for the averaged H-ch noise. ",$
"  mxHn:    Resetable indicator for the max <Hn> (click on the field and    ",$
"           press Enter)                                                    ",$
"  <Vn>:    Non editable information indicator for the averaged V-ch noise. ",$
"  mxVn:    Resetable indicator for the max <Vn>  (click on the field and   ",$
"           press Enter)                                                    ",$
"  Hleak:   Non editable information indicator for the leaked H-ch Tx power.",$
"           WCR Tx leaked power (in dBm) at 0 range. This indicator is used ",$
"           to help detect data acquisition bug which causes a shift in the ",$
"           range gates.  In this case the leak power will be close to the  ",$
"           noise level (around -65 dBm)                                    ",$
"  Vleak:   Non editable information indicator for the leaked V-ch Tx power.",$
"           WCR Tx leaked power (in dBm) at 0 range. This indicator can be  ",$
"           used together with Hleak to help detect a problem with Tx power ",$
"           for H or V channel.                                             ",$
"  mxdT:    Resetable maximum average time interval(in ms) between profiles;",$
"           If 9999 is displayed a negative time jump in the time stamps was",$
"           encountered."]

                 xdisplayfile,title='WCR Display Help', text=junk, $
                              group=baseid, /modal
                 end
     else      :
   endcase

end ; bg1_ev

; -------------------------------------------------------------------------
; Control widget events from image ranges (velocity, reflectivity data)

pro bg2_ev,ev

   common wcrdspacd
   common wcrdspp
   common wcrdspwdg
   common wcrdspids
   common wcrdspd       ; used by imgordid only
   common wcrdspcal     ; used by imgordid only
   common wcrdspctrl    ; used by imgordid only

   case ev.id of
      hh1id : begin
                  image1='hh' &  wset,barw1
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
                  widget_control,b1maxid,set_value=zzmaxs
                  widget_control,b1minid,set_value=zzmins
                  widget_control,b1labelid,set_value=' hh(dBZ) '
                  if antconf eq 4 then img1order=0
              end
      vv1id : begin
                  image1='vv' &  wset,barw1
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
                  widget_control,b1maxid,set_value=zzmaxs
                  widget_control,b1minid,set_value=zzmins
                  widget_control,b1labelid,set_value=' vv(dBZ) '
                  if antconf eq 4 then img1order=1
              end
     ldr1id : begin
                  image1='ldr' &  wset,barw1
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax
                  widget_control,b1maxid,set_value=ldmaxs
                  widget_control,b1minid,set_value=ldmins
                  widget_control,b1labelid,set_value=' ldr(dB) '
              end
     zdr1id : begin
                  image1='zdr' &  wset,barw1
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=zdmin,vmax=zdmax
                  widget_control,b1maxid,set_value=zdmaxs
                  widget_control,b1minid,set_value=zdmins
                  widget_control,b1labelid,set_value=' zdr(dB) '
              end
     dvh1id : begin
                  if velcor eq 0 then image1='du_h' $
                  else image1=strlowcase(dvlabel[0])
                  wset,barw1
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=dvhmin,vmax=dvhmax
                  widget_control,b1maxid,set_value=dvhmaxs
                  widget_control,b1minid,set_value=dvhmins
                  widget_control,b1labelid,set_value=image1+'(m/s)'
                  if antconf eq 4 and image1 ne 'dc_v' then img1order=0
              end
     dvv1id : begin
                  if velcor eq 0 then image1='du_v' $
                  else image1=strlowcase(dvlabel[1])
                  wset,barw1
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
                  widget_control,b1maxid,set_value=dvvmaxs
                  widget_control,b1minid,set_value=dvvmins
                  widget_control,b1labelid,set_value=image1+'(m/s)'
                  if antconf eq 4 and image1 ne 'du_h' then img1order=1
              end
      vh1id : begin
                  wset,barw1
                  widget_control,vh1id,get_value=vhlabel
                  if alg eq fftalg then begin
                    image1='sv'
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax
                    widget_control,b1labelid,set_value=' '+vhlabel+'(m/s) '
                    widget_control,b1maxid,set_value=svmaxs
                    widget_control,b1minid,set_value=svmins
                  endif else begin
                    image1='vh'
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
                    widget_control,b1labelid,set_value=' '+vhlabel+'(dBZ) '
                    widget_control,b1maxid,set_value=czmaxs
                    widget_control,b1minid,set_value=czmins
                  endelse
              end
      hh2id : begin
                  image2='hh' &  wset,barw2
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
                  widget_control,b2maxid,set_value=zzmaxs
                  widget_control,b2minid,set_value=zzmins
                  widget_control,b2labelid,set_value=' hh(dBZ) '
                  if antconf eq 4 then img2order=0
              end
      vv2id : begin
                  image2='vv' &  wset,barw2
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
                  widget_control,b2maxid,set_value=zzmaxs
                  widget_control,b2minid,set_value=zzmins
                  widget_control,b2labelid,set_value=' vv(dBZ) '
                  if antconf eq 4 then img2order=1
              end
     ldr2id : begin
                  image2='ldr' &  wset,barw2
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax
                  widget_control,b2maxid,set_value=ldmaxs
                  widget_control,b2minid,set_value=ldmins
                  widget_control,b2labelid,set_value=' ldr(dB) '
              end
     zdr2id : begin
                  image2='zdr' &  wset,barw2
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=zdmin,vmax=zdmax
                  widget_control,b2maxid,set_value=zdmaxs
                  widget_control,b2minid,set_value=zdmins
                  widget_control,b2labelid,set_value=' zdr(dB) '
              end
     dvh2id : begin
                  if velcor eq 0 then image2='du_h' $
                  else image2=strlowcase(dvlabel[0])
                  wset,barw2
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=dvhmin,vmax=dvhmax
                  widget_control,b2maxid,set_value=dvhmaxs
                  widget_control,b2minid,set_value=dvhmins
                  widget_control,b2labelid,set_value=image2+'(m/s)'
                  if antconf eq 4 and image2 ne 'dc_v' then img2order=0
              end
     dvv2id : begin
                  if velcor eq 0 then image2='du_v' $
                  else image2=strlowcase(dvlabel[1])
                  wset,barw2
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
                  widget_control,b2maxid,set_value=dvvmaxs
                  widget_control,b2minid,set_value=dvvmins
                  widget_control,b2labelid,set_value=image2+'(m/s)'
                  if antconf eq 4 and image2 ne 'du_h' then img2order=1
              end
      vh2id : begin
                  wset,barw2
                  widget_control,vh2id,get_value=vhlabel
                  if alg eq fftalg then begin
                    image2='sv'
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax
                    widget_control,b2labelid,set_value=' '+vhlabel+'(m/s) '
                    widget_control,b2maxid,set_value=svmaxs
                    widget_control,b2minid,set_value=svmins
                  endif else begin
                    image2='vh'
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
                    widget_control,b2labelid,set_value=' '+vhlabel+'(dBZ) '
                    widget_control,b2maxid,set_value=czmaxs
                    widget_control,b2minid,set_value=czmins
                  endelse
              end
       dvid : begin ; This is active(sensitive=1, velcor=1) for ppmag modes only
                ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                case ev.index of
                  0: begin
                       if gpsacc then dvlabel=['dg_H','dg_V'] $
                       else dvlabel=['dc_H','dc_V']
                       if strmid(image1,0,1) eq 'd' then begin
                         image1=strlowcase(dvlabel[0])
                         widget_control,dvh1id,set_button=1
                       endif
                       if nimg eq 2 and strmid(image2,0,1) eq 'd' then begin
                         image2=strlowcase(dvlabel[1])
                         widget_control,dvv2id,set_button=1
                       endif
                     end
                  1: begin
                       dvlabel=['du_H','du_V']
                       if strmid(image1,0,1) eq 'd' then begin
                         image1=strlowcase(dvlabel[0])
                         widget_control,dvh1id,set_button=1
                       endif
                       if nimg eq 2 and strmid(image2,0,1) eq 'd' then begin
                         image2=strlowcase(dvlabel[1])
                         widget_control,dvv2id,set_button=1
                       endif
                     end
                  2: begin
                       if gpsacc then dvlabel=['dg_H','du_H'] $
                       else dvlabel=['dc_H','du_H']
                       if strmid(image1,0,1) eq 'd' then begin
                         image1=strlowcase(dvlabel[0])
                         widget_control,dvh1id,set_button=1
                       endif
                       if nimg eq 2 and strmid(image2,0,1) eq 'd' then begin
                         image2=strlowcase(dvlabel[1])
                         widget_control,dvv2id,set_button=1
                       endif
                     end
                  3: begin
                       if gpsacc then dvlabel=['dg_V','du_V'] $
                       else dvlabel=['dc_V','du_V']
                       if strmid(image1,0,1) eq 'd' then begin
                         image1=strlowcase(dvlabel[0])
                         widget_control,dvh1id,set_button=1
                       endif
                       if nimg eq 2 and strmid(image2,0,1) eq 'd' then begin
                         image2=strlowcase(dvlabel[1])
                         widget_control,dvv2id,set_button=1
                       endif
                    end
                endcase
                widget_control,dvh1id,set_value=dvlabel[0]
                widget_control,dvv1id,set_value=dvlabel[1]
                if nimg eq 2 then begin
                  widget_control,dvh2id,set_value=dvlabel[0]
                  widget_control,dvv2id,set_value=dvlabel[1]
                endif
                if antconf eq 4 then begin
                  img1order=(image1 eq 'vv' or image1 eq 'du_v' or image1 eq 'dc_v' or image1 eq 'dg_v')
                  img2order=(image2 eq 'vv' or image2 eq 'du_v' or image2 eq 'dc_v' or image2 eq 'dg_v')
                endif
                ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
              end
      antid : begin
                 antconf=ev.index + (ev.index eq 3) ; remove logical op when
                 case antconf of                    ; antconf 3 become available
                   0: img1order=(img2order=0)
                   1: img1order=(img2order=1)
                   2: img1order=(img2order=1)
                   3: img1order=(img2order=1)
                   4: begin
                       img1order=(image1 eq 'vv' or image1 eq 'du_v' or image1 eq 'dc_v' or image1 eq 'dg_v')
                       img2order=(image2 eq 'vv' or image2 eq 'du_v' or image2 eq 'dc_v' or image2 eq 'dg_v')
                      end
                   5: img1order=(img2order=0)
                 endcase

                 ; Update y-axes for image1

                 widget_control,axis1id,get_value=axisw1
                 wset, axisw1 & erase    ; go to axis top window

                 junkid=((minimgsz-rangegates)*rangegatespacing)>0
                 ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                             junkid*(img1order eq 1))/1000.
                 plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                      xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                 xl_cm=!d.x_size/!d.x_px_cm
                 tkly=-1./xl_cm            ; reasonable tick length

                 axis,0.45,ax_yrange[0]/0.305,/nodat,ytickle=tkly,yaxis=0, $
                 yrange=!y.crange*(img1order eq 0)+ $
                   reverse(!y.crange)*(img1order eq 1),ystyle=1,chars=1.1,/normal
                 plot,ax_yrange,/nodata,/noerase,/normal, $
                   xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                 axis,1.0,ax_yrange[0],/nodata,yticklen=tkly,yaxis=0,ystyle=1, $
                 yrange=!y.crange*(img1order eq 0)+ $
                   reverse(!y.crange)*(img1order eq 1),chars=1.1,/normal

                 if nimg eq 2 then begin

                   ; Update y-axes for image2 axis

                   widget_control,axis2id,get_value=axisw2
                   wset, axisw2 & erase

                   junkid=((minimgsz-rangegates)*rangegatespacing)>0
                   ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                               junkid*(img2order eq 1))/1000.
                   plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                        xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                   xl_cm=!d.x_size/!d.x_px_cm
                   tkly=-1./xl_cm            ; reasonable tick length

                   axis,0.45,ax_yrange[0]/0.305,/nodat,ytickle=tkly,yaxis=0, $
                    yrange=!y.crange*(img2order eq 0)+reverse(!y.crange)* $
                    (img2order eq 1),ystyle=1,chars=1.1,/normal
                   plot,ax_yrange,/nodata,/noerase,/normal, $
                     xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                   axis,1.0,ax_yrange[0],/nodata,yticklen=tkly,yaxis=0, $
                   ystyle=1,yrange=!y.crange*(img2order eq 0)+ $
                     reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal
                endif

                 ; For dual antenna configurations dissable all cross channels products

                 if antconf gt 0 then begin
                   widget_control,ldr1id,sensitive=0
                   widget_control,zdr1id,sensitive=0
                   if alg ne fftalg then widget_control,vh1id,sensitive=0
                   if nimg eq 2 then begin
                     widget_control,ldr2id,sensitive=0
                     widget_control,zdr2id,sensitive=0
                     if alg ne fftalg then widget_control,vh2id,sensitive=0
                   endif
                 endif else begin
                   widget_control,hh1id,sensitive=1
                   widget_control,vv1id,sensitive=1
                   widget_control,zdr1id,sensitive=1
                   widget_control,ldr1id,sensitive=1
                   widget_control,vh1id,sensitive=1
                   widget_control,dvh1id,sensitive=1
                   widget_control,dvv1id,sensitive=1
                   if nimg eq 2 then begin
                      widget_control,hh2id,sensitive=1
                      widget_control,vv2id,sensitive=1
                      widget_control,zdr2id,sensitive=1
                      widget_control,ldr2id,sensitive=1
                      widget_control,vh2id,sensitive=1
                      widget_control,dvh2id,sensitive=1
                      widget_control,dvv2id,sensitive=1
                   endif

                   if alg eq sppalg or alg eq fftalg then begin
                      widget_control,zdr1id,sensitive=0
                      if txstrpol[0] eq 'H' then begin
                        widget_control,vv1id,sensitive=0
                        if velcor eq 0 then widget_control,dvv1id,sensitive=0
                      endif else begin
                        widget_control,hh1id,sensitive=0
                        if velcor eq 0 then widget_control,dvh1id,sensitive=0
                      endelse
                      if alg eq fftalg then begin
                        widget_control,ldr1id,sensitive=0
                        widget_control,vh1id,set_value='sv'
                      endif
                      if nimg eq 2 then begin
                        widget_control,zdr2id,sensitive=0
                        if txstrpol[0] eq 'H' then begin
                          widget_control,vv2id,sensitive=0
                          if velcor eq 0 then widget_control,dvv2id,sensitive=0
                        endif else begin
                          widget_control,hh2id,sensitive=0
                          if velcor eq 0 then widget_control,dvh2id,sensitive=0
                        endelse
                        if alg eq fftalg then begin
                          widget_control,ldr2id,sensitive=0
                        endif
                      endif
                   endif

                 endelse
              end
       else : message,'Cannot be happening.'
   endcase

end ; bg2_ev

; -------------------------------------------------------------------------
; Provide widget change activities depending on an event request:
; 'destroy', 'quit', 'open', 'catch', 'inittime'

pro check_ev,fn,data,fsize,strtp

   common wcrdspp
   common wcrdspcal
   common wcrdspids
   common wcrdspctrl

   if destroyw then begin                         ;  Destroy widget
      close, lun1
      widget_control, baseid, /destroy
      return
   endif else if quitw then begin                 ;  Quit widget
      close, lun1
      return
   endif else if openwf then begin                ;  Open new wcr file
      close,lun1
      strtp=0
      wcrdispopen,fn,data,fsize
      return
   endif else if catche then begin                ; Go to the end of current
      hhz[dml:msz1,*]=-150.                       ; data and draw a black strip
      vvz[dml:msz1,*]=-150.
      ldr[dml:msz1,*]=-150.
      zdr[dml:msz1,*]=-150.
      duh[dml:msz1,*]=-50.
      duv[dml:msz1,*]=-50.
      dch[dml:msz1,*]=-50.
      dcv[dml:msz1,*]=-50.
      vhz[dml:msz1,*]=-150.

   endif else if starte then begin  ; Handle request for restart at new time

   ; If stoptime = starttime ignore the request (do nothing)

     if stoptime eq starttime then begin
        starte=0B
        widget_control,startid,set_value=starttime
     endif else begin

       ; Check if the new start time is correct; if not do nothing
       ; If new clear data arrays and start form the new time (approximately)

       widget_control,startid,get_value=newinittime  ; get the new time
       newinittime=newinittime[0]

       ; Check the format of the input (9999,hhmm,hhmmss,hh:mm,h:mm:ss,hh:mm:ss)

       if newinittime eq '9999' then newinittime=stoptime
       temp=strlen(newinittime)
       if temp gt 8 or temp lt 4 then newinittime='0' $
       else if temp eq 4 then $
         newinittime=strmid(newinittime,0,2)+':'+strmid(newinittime,2,2)+':00' $
       else if temp eq 5 then newinittime=newinittime+':00' $
       else if temp eq 6 then $
         newinittime=strmid(newinittime,0,2)+':'+strmid(newinittime,2,2)+':'+ $
                     strmid(newinittime,4,2)

       ; Check the validity of the new time and find the starting profile

       temp=timestr2sec(newinittime)
       if temp eq -1 or newinittime eq stoptime then begin
         starte=0B
         if temp eq -1 then $
            ; incorrect time string; don't do anything and restore starttime
            inittime=starttime $
         else inittime=stoptime
       endif else begin          ; check the new time and find the start profile
         junk1=timestr2sec(starttime)
         junk2=timestr2sec(stoptime)
         if junk1 gt junk2 then begin                    ; midnight was crossed
           if temp ge junk1 or temp lt junk2 then begin
           ; The new time is ok; find the starting profile
             if temp lt junk1 then temp=temp+86400.
             strtp = long((temp-junk1)/timeinterval)>0
             inittime=newinittime
             hhz[*]=-150.
             vvz[*]=-150.
             ldr[*]=-150.
             zdr[*]=-150.
             duh[*]= -50.
             duv[*]= -50.
             dch[*]= -50.
             dcv[*]= -50.
             vhz[*]=-150.
           endif else begin
           ; New time is beyond the available time range. Ignore it.
             starte = 0B
             inittime=starttime
           endelse
         endif else begin                           ; midnight was not crossed
           if temp ge junk1 and temp lt junk2 then begin
           ; new time is ok
             strtp = long((temp-junk1)/timeinterval)>0
             inittime=newinittime
             hhz[*]=-150.
             vvz[*]=-150.
             ldr[*]=-150.
             zdr[*]=-150.
             duh[*]= -50.
             duv[*]= -50.
             dch[*]= -50.
             dcv[*]= -50.
             vhz[*]=-150.
           endif else begin
           ; New time is beyond the available time range. Ignore it.
             starte = 0B
             inittime=starttime
           endelse
         endelse
       endelse
       widget_control,startid,set_value=inittime
       if inittime eq starttime then $
         widget_control,inittid,set_value='InitTime' $
       else if inittime eq stoptime then $
         widget_control,inittid,set_value='EndTime' $
       else widget_control,inittid,set_value='NewTime'
     endelse
   endif

end ; check_ev

; -------------------------------------------------------------------------
; Provides widget control for real-time display parameters:
; calibration constants,isolation,display step(loopstep), profile
; averaging (avestep), time shift (thack), I/O sleep time (sleept)

pro dispc_ev, ev

  common wcrdspacd
  common wcrdspp
  common wcrdspd
  common wcrdspcal
  common wcrdspwdg
  common wcrdspids
  common wcrdspctrl

   widget_control, ev.id, get_value=temp

   temp=float(temp[0])

   dispc=1B
   case ev.id of
      hhcalid : begin
                  if temp eq 999. or temp le 0. then begin
                     hhcal=hhcald
                     widget_control,hhcalid,set_value=num2str(hhcal,fr=1)
                  endif else hhcal=temp
                  ldrcal=hhcal-vvcal
                  zdrcal=ldrcal
                end
      vvcalid : begin
                  if temp eq 999. or temp le 0. then begin
                     vvcal=vvcald
                     widget_control,vvcalid,set_value=num2str(vvcal,fr=1)
                  endif else vvcal=temp
                  ldrcal=hhcal-vvcal
                  zdrcal=ldrcal
                end
       isolid : begin
                  if temp eq 999. or temp le 0. then begin
                     isoldb=isoldbd
                     widget_control,isolid,set_value=num2str(isoldb,fr=1)
                  endif else isoldb = temp
                  isol   = 10.^(0.1*(-isoldb))
                end
       stepid : begin
                  if temp eq 999. or temp le 0. then loopstep=loopstepd $
                  else loopstep=(fix(temp)>(1+(avestep eq 1)))<maximgsz
                  widget_control,stepid,set_value=num2str(loopstep)
                  sleept = float(ceil(loopstep*avestep*timeinterval))
                  widget_control,sleepid,set_value=num2str(sleept,fr=1)
                  dml     =maximgsz-loopstep
                  rngcor  =(fltarr(loopstep)+1.)#(20.*alog10(radrng)-60.)
                end
        aveid : begin
                  if temp eq 999. or temp le 0. then $
                  avestep=ceil(aspectratio)>(1+(loopstep eq 1)) $
                  else avestep=(fix(temp)>(1+(loopstep eq 1)))<minimgsz
                  widget_control,aveid,set_value=num2str(avestep)
                  sleept = float(ceil(loopstep*avestep*timeinterval))
                  widget_control,sleepid,set_value=num2str(sleept,fr=1)
                end
      sleepid : begin
                  if temp eq 999. or temp le 0. then sleept=sleepti $
                  else sleept=temp<60.
                  widget_control,sleepid,set_value=num2str(sleept,fr=1)
                  loopstep=(floor(sleept/(avestep*timeinterval))>loopstepd)<maximgsz
                  widget_control,stepid,set_value=num2str(loopstep)
                  dml     =maximgsz-loopstep
                  rngcor  =(fltarr(loopstep)+1.)#(20.*alog10(radrng)-60.)
                end
      thackid:  begin
                  if temp eq 999. then thack=thacki $
                  else thack=temp
                  widget_control,thackid,set_value=num2str(thack,fr=2)+'s'
                end
         else : dispc=0B
   endcase

end ; dispc_ev

; -------------------------------------------------------------------------
; Provides widget control for real-time display parameters:
; min/max values for image ranges

pro minmax_ev, ev

   common wcrdspp
   common wcrdspwdg
   common wcrdspids

   widget_control, ev.id, get_value=minmaxs
   minmax=(float(minmaxs))[0]

   case ev.id of

      b1minid : begin
                  case image1 of
                    'hh'   : begin zzmin=minmax & zzmins=minmaxs & end
                    'vv'   : begin zzmin=minmax & zzmins=minmaxs & end
                    'du_h' : begin dvhmin=minmax & dvhmans=minmaxs & end
                    'du_v' : begin dvvmin=minmax & dvvmins=minmaxs & end
                    'dc_h' : begin dvhmin=minmax & dvhmins=minmaxs & end
                    'dc_v' : begin dvvmin=minmax & dvvmins=minmaxs & end
                    'dg_h' : begin dvhmin=minmax & dvhmins=minmaxs & end
                    'dg_v' : begin dvvmin=minmax & dvvmins=minmaxs & end
                    'zdr'  : begin zdmin=minmax & zdmins=minmaxs & end
                    'ldr'  : begin ldmin=minmax & ldmins=minmaxs & end
                    'vh'   : begin czmin=minmax & czmins=minmaxs & end
                    'sv'   : begin svmin=minmax & svmins=minmaxs & end
                    else   :
                  endcase
                  widget_control,b1minid,set_value=num2str(minmax,fr=0)
                  if image1 eq image2 or (image1 eq 'hh' and image2 eq 'vv') $
                     or (image1 eq 'vv' and image2 eq 'hh') or $
                     (image1 eq 'du_h' and image2 eq 'dc_h') or $
                     (image1 eq 'dc_h' and image2 eq 'du_h') or $
                     (image1 eq 'du_h' and image2 eq 'dg_h') or $
                     (image1 eq 'dg_h' and image2 eq 'du_h') or $
                     (image1 eq 'du_v' and image2 eq 'dc_v') or $
                     (image1 eq 'dc_v' and image2 eq 'du_v') or $
                     (image1 eq 'du_v' and image2 eq 'dg_v') or $
                     (image1 eq 'dg_v' and image2 eq 'du_v') then $
                     widget_control,b2minid,set_value=num2str(minmax,fr=0)
                end

      b1maxid : begin
                  case image1 of
                    'hh'   : begin zzmax=minmax & zzmaxs=minmaxs & end
                    'vv'   : begin zzmax=minmax & zzmaxs=minmaxs & end
                    'du_h' : begin dvhmax=minmax & dvhmaxs=minmaxs & end
                    'du_v' : begin dvvmax=minmax & dvvmaxs=minmaxs & end
                    'dc_h' : begin dvhmax=minmax & dvhmaxs=minmaxs & end
                    'dc_v' : begin dvvmax=minmax & dvvmaxs=minmaxs & end
                    'dg_h' : begin dvhmax=minmax & dvhmaxs=minmaxs & end
                    'dg_v' : begin dvvmax=minmax & dvvmaxs=minmaxs & end
                    'zdr'  : begin zdmax=minmax & zdmaxs=minmaxs & end
                    'ldr'  : begin ldmax=minmax & ldmaxs=minmaxs & end
                    'vh'   : begin czmax=minmax & czmaxs=minmaxs & end
                    'sv'   : begin svmax=minmax & svmaxs=minmaxs & end
                    else   :
                  endcase
                  widget_control,b1maxid,set_value=num2str(minmax,fr=0)
                  if image1 eq image2 or (image1 eq 'hh' and image2 eq 'vv') $
                     or (image1 eq 'vv' and image2 eq 'hh') or $
                     (image1 eq 'du_h' and image2 eq 'dc_h') or $
                     (image1 eq 'dc_h' and image2 eq 'du_h') or $
                     (image1 eq 'du_h' and image2 eq 'dg_h') or $
                     (image1 eq 'dg_h' and image2 eq 'du_h') or $
                     (image1 eq 'du_v' and image2 eq 'dc_v') or $
                     (image1 eq 'dc_v' and image2 eq 'du_v') or $
                     (image1 eq 'du_v' and image2 eq 'dg_v') or $
                     (image1 eq 'dg_v' and image2 eq 'du_v') then $
                     widget_control,b2maxid,set_value=num2str(minmax,fr=0)
                end

      b2minid : begin
                  case image2 of
                    'hh'   : begin zzmin=minmax & zzmins=minmaxs & end
                    'vv'   : begin zzmin=minmax & zzmins=minmaxs & end
                    'du_h' : begin dvhmin=minmax & dvhmins=minmaxs & end
                    'du_v' : begin dvvmin=minmax & dvvmins=minmaxs & end
                    'dc_h' : begin dvhmin=minmax & dvhmins=minmaxs & end
                    'dc_v' : begin dvvmin=minmax & dvvmins=minmaxs & end
                    'dg_h' : begin dvhmin=minmax & dvhmins=minmaxs & end
                    'dg_v' : begin dvvmin=minmax & dvvmins=minmaxs & end
                    'zdr'  : begin zdmin=minmax & zdmins=minmaxs & end
                    'ldr'  : begin ldmin=minmax & ldmins=minmaxs & end
                    'vh'   : begin czmin=minmax & czmins=minmaxs & end
                    'sv'   : begin svmin=minmax & svmins=minmaxs & end
                    else   :
                  endcase
                  widget_control,b2minid,set_value=num2str(minmax,fr=0)
                  if image1 eq image2 or (image1 eq 'hh' and image2 eq 'vv') $
                     or (image1 eq 'vv' and image2 eq 'hh') or $
                     (image1 eq 'du_h' and image2 eq 'dc_h') or $
                     (image1 eq 'dc_h' and image2 eq 'du_h') or $
                     (image1 eq 'du_h' and image2 eq 'dg_h') or $
                     (image1 eq 'dg_h' and image2 eq 'du_h') or $
                     (image1 eq 'du_v' and image2 eq 'dc_v') or $
                     (image1 eq 'dc_v' and image2 eq 'du_v') or $
                     (image1 eq 'du_v' and image2 eq 'dg_v') or $
                     (image1 eq 'dg_v' and image2 eq 'du_v') then $
                     widget_control,b1minid,set_value=num2str(minmax,fr=0)
                end

      b2maxid : begin
                  case image2 of
                    'hh'   : begin zzmax=minmax & zzmaxs=minmaxs & end
                    'vv'   : begin zzmax=minmax & zzmaxs=minmaxs & end
                    'du_h' : begin dvhmax=minmax & dvhmaxs=minmaxs & end
                    'du_v' : begin dvvmax=minmax & dvvmaxs=minmaxs & end
                    'dc_h' : begin dvhmax=minmax & dvhmaxs=minmaxs & end
                    'dc_v' : begin dvvmax=minmax & dvvmaxs=minmaxs & end
                    'dg_h' : begin dvhmax=minmax & dvhmaxs=minmaxs & end
                    'dg_v' : begin dvvmax=minmax & dvvmaxs=minmaxs & end
                    'zdr'  : begin zdmax=minmax & zdmaxs=minmaxs & end
                    'ldr'  : begin ldmax=minmax & ldmaxs=minmaxs & end
                    'vh'   : begin czmax=minmax & czmaxs=minmaxs & end
                    'sv'   : begin svmax=minmax & svmaxs=minmaxs & end
                    else   :
                  endcase
                  widget_control,b2maxid,set_value=num2str(minmax,fr=0)
                  if image1 eq image2 or (image1 eq 'hh' and image2 eq 'vv') $
                     or (image1 eq 'vv' and image2 eq 'hh') or $
                     (image1 eq 'du_h' and image2 eq 'dc_h') or $
                     (image1 eq 'dc_h' and image2 eq 'du_h') or $
                     (image1 eq 'du_h' and image2 eq 'dg_h') or $
                     (image1 eq 'dg_h' and image2 eq 'du_h') or $
                     (image1 eq 'du_v' and image2 eq 'dc_v') or $
                     (image1 eq 'dc_v' and image2 eq 'du_v') or $
                     (image1 eq 'du_v' and image2 eq 'dg_v') or $
                     (image1 eq 'dg_v' and image2 eq 'du_v') then $
                     widget_control,b1maxid,set_value=num2str(minmax,fr=0)
                end
      statmxhnid: begin
                  mxhn=-99.
                  widget_control,statmxhnid,set_value=num2str(mxhn)
                end
      statmxvnid: begin
                  mxvn=-99.
                  widget_control,statmxvnid,set_value=num2str(mxvn)
                end
      statdtid: begin
                  timeintm=0L
                  widget_control,statdtid,set_value=num2str(timeintm)
                end
      else :
   endcase

end ; minmax_ev

; -------------------------------------------------------------------------
; Widget control for top image scroll bar

pro scrollw1_ev,ev

   common wcrdspids
   common wcrdspctrl

   if ev.press then begin
      scrollw1e=1B
      itime=ev.x
      ivalue=ev.y
   endif

end ; scrollw1_ev

; -------------------------------------------------------------------------
; Widget control for bottom image scroll bar

pro scrollw2_ev,ev

   common wcrdspids
   common wcrdspctrl

   if ev.press then begin
      scrollw2e=1B
      itime=ev.x
      ivalue=ev.y
   endif

end ; scrollw2_ev

; -------------------------------------------------------------------------
; Main real-time display routine.  Real time display is controlled by a main
; loop which monitors the availability of data/new data in a raw WCR file.
; Thus it does not utilize widget high level xmanager function but the widget
; low level function widget_event.

pro wcrdisplay, aconfig, acspeed=acspeed,calval=calval,colortable=colortable,$
                         dopcor=dopcor,minmax=minmax,start=strtp, $
                         timesync=timesync,help=help

  common wcrdspacd
  common wcrdspp
  common wcrdspd
  common wcrdspcal
  common wcrdspwdg
  common wcrdspids
  common wcrdspctrl

; .........................................................................

if keyword_set(help) then begin
  doc_library,'wcrdisplay'
  return
endif

; Initilize

  on_error,2

  ; Check Input

  if n_params() eq 0 then antconf=0 $
  else begin
    antconf=fix(aconfig)
    if antconf lt 0 or antconf gt 5 then begin
      message,'Invalid aconfig. Use aconfig=0', /info
      aconfig=0
    endif
    if antconf eq 3 then begin
      message,'aconfig 3 is not a valid configuration yet. Ignore; use 0',/info
      aconfig=0
    endif
  endelse

  ; Check if Doppler and GPS A/C velocity corrections are requested.
  ; dvlabels is dependant on the wcr alg and will be updated

  dvlabels='du_H,du_V'
  thack=(thacki=0.d0)
  if n_elements(dopcor) eq 0 then begin
     velcor=0
     gpsacc=0
     dvlabel=['du_H','du_V']
  endif else begin
     ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     if dopcor lt 0 then gpsacc=1 else gpsacc=0
     velcor=fix(abs(dopcor))
     if velcor eq 0 then dvlabel=['du_H','du_V'] $
     else begin
       if gpsacc then dvlabel=['dg_H','dg_V'] $
       else dvlabel=['dc_H','dc_V']
       dvlabels=[dvlabel[0]+','+dvlabel[1], $
                 'du_H,du_V'              , $
                 dvlabel[0]+',du_H'       , $
                 dvlabel[1]+',du_V'         ]
     endelse

     ; Assign the time shift between WCR and A/C clocks.  Default is 0.

     if n_elements(timesync) eq 1 then thack=double(timesync)
     thacki=thack  ; save initial WCR-A/C time sync hack in seconds

     ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  endelse

  ; Aircraft average air speed in m/s; used for determining approximate aspectratio
  ; of radar range resolution vs. radar profile resolution in meters

  if n_elements(acspeed) eq 0 then begin
     if antconf ne 3 then kaspeed=90. else kaspeed=115.
  endif else kaspeed=acspeed

  ; Initilize image orders depending on antconf; 0 bottom-top, 1 top-bottom

  nimg        = 2             ; default display of two images
  image1      ='hh'           ; default image displayed in the top window
  image2      ='vv'           ; default image displayed in the bottom window
                              ; (for sppmag this is changed to Dopp velocity)

  case antconf of
    0: img1order=(img2order=0)
    1: img1order=(img2order=1)
    2: img1order=(img2order=1)
    3: img1order=(img2order=1)
    4: begin
         img1order=(image1 eq 'vv' or image1 eq 'du_v' or image1 eq 'dc_v' or image1 eq 'dg_v')
         img2order=(image2 eq 'vv' or image2 eq 'du_v' or image2 eq 'dc_v' or image2 eq 'dg_v')
       end
    5: img1order=(img2order=0)
  endcase

  ; Load color table

  if n_elements(colortable) eq 0 then begin
     loadct,0,get_names=junkid
     if n_elements(junkid) lt 46 then colortable=15 $
     else colortable=45  ; WYO12 dual color table
     loadct,colortable
  endif else if colortable ge 0 then loadct,colortable

  if (colortable eq 45 or colortable eq 46) and !D.N_COLORS lt 64 then $
     colortable=41

  ; If dual color table find starting index for the second subcolor table

  ct2ind=wyoctind()
  if ct2ind eq -1 then ct2ind=!d.table_size

  ; Start from a given profile if requested

  if n_elements(strtp) eq 0 then strtp=0

  ; Calibration coefficients

  if n_elements(calval) eq 0 then begin
     hhcal     = 49.        ; dB ; side/up antenna H-pol
     vvcal     = 49.        ; dB ; side/up antenna V-pol or nadir ant. H-pol   
     iso       =-18.        ; dB
     vvcald=vvcal           ; dB
     hhcald=hhcal           ; dB
  endif else begin
     ncal=n_elements(calval)
     if (ncal lt 2 or ncal gt 3) then $
        message,"Wrong input for keyword CALVAL" $
     else begin
       hhcal   = calval[0]
       vvcal   = calval[1]
     endelse
     if ncal eq 2 then iso  = -18.             $
     else              iso  = -abs(calval[2])
  endelse

 ; Assign some other initial values

  maximgsz    =600            ; image (scroll window) max x-size
  minimgsz    =100            ; image (scroll window) min y-size
  minfsize    =100000L        ; minimum file size to open or read update
  maxrg       =250            ; maximum range gates when nimg=2
  axsize      =75B            ; axis windows x_size
  wxsize      =450            ; scroll windows x_size
  bxsize      =15B            ; color bar windows x_size
  ysize       =minimgsz       ; all windows y_size
  destroyw    =0B             ; close wcr and destroy widget
  destroym    =0B             ; destroy message widget
  quitw       =0B             ; close wcr file and quit
  openwf      =0B             ; open new wcr file
  catche      =0B             ; go to the end of current data
  scrollw1e   =0B             ; image1 cursor control variable
  scrollw2e   =0B             ; image2 cursor control variable
  rgold       =0              ; # of range gates
  rgsold      =0              ; range gate spacing in m
  rgdold      =0              ; range gate delay in m
  wcrfileold  =''             ; WCR file name
  loopstep=5                  ; default display loop step
  loopstepd =loopstep         ; remember default loop step
  dml         =maximgsz-loopstep
  msz1        =maximgsz-1
  startid     =999            ; work around for a bug in HP IDL 4.0
  ivalue      =0              ; initialize cursor range position

  tcor        =-13.d0         ; GPS leap seconds correction
                              ; (updated from A/C file if read)
  sleept      =2.             ; initial disk I/O sleep time (wait time for
                              ; the data file to reach minimum size; after
                              ; that sleept is updated to match disp. data rate 

  acbasetime  =0L             ; UNIX start time for A/C data file
  ncf         =-1             ; A/C file identificator
  upbeam      =[0.052336,0.,-0.99862951]    ; upbeam unit vector in A/C coords
  side1beam   =[-0.01128,0.999933,0.002743] ; sidebeam unit vector in A/C coords
  side2beam   =[0.581156,0.812945,0.0371314]; 35 deg side-fore beam
 ;down1beam   =[-0.0565,0.0245,0.9981]      ; down beam
  down1beam   =[-0.05624,0.0026,0.9984]     ; down beam after May 26 2002
  down2beam   =[ 0.439, 0.005, 0.898]       ; 30 deg down-fore beam

  get_lun,lun1                ; get WCR file lun

  nosignal  = 1.e-15          ; min signal level in mW
  zero      = 1.e-25          ; 0 level for data
  fftcor    = 30.             ; dB; error in fft wcr cal (in W instead of mW)
; h_slope   = 1./0.95         ; error in H-channel lookup table before 1999
  h_slope   = 1.              ; no correction after May 1999
  v_slope   = 1.              ; no correction
  alphaiso  = 0.5             ; split 2-way measured isolation to Tx and Rx iso.

  ldrcal    = hhcal-vvcal
  ldrcallin = 10.^(0.1*ldrcal)
  zdrcal    = hhcal-vvcal
  isoldb    = -iso            ; H/V channels isolation
  isoldbd   = isoldb
  isol      = 10.^(0.1*iso)

  ; Default min/max values for received power quantities

  zzmax =  10.                    ; co-pol,    dBZ
  zzmin = -45.                    ; co-pol,    dBZ
  zdmax =   4.                    ; zdr,       dBZ
  zdmin =  -4.                    ; zdr,       dBZ
  ldmax = -10.                    ; ldr,       dBZ
  ldmin = -25.                    ; ldr,       dBZ
  czmax = -10.                    ; cross-pol, dBZ
  czmin = -45.                    ; cros-pol,  dBZ

  ; Check for minmax input

  if n_elements(minmax) ge 2 then $
     if minmax[0] lt minmax[1] then begin
        zzmin=minmax[0] & zzmax=minmax[1]
     endif
  if n_elements(minmax) ge 4 then $
     if minmax[2] lt minmax[3] then begin
        zdmin=minmax[2] & zdmax=minmax[3]
     endif
  if n_elements(minmax) eq 6 then $
     if minmax(4) lt minmax[5] then begin
        ldmin=minmax[4] & ldmax=minmax[5]
  endif

  ; Convert min/max to strings for widget use

  zzmaxs=num2str(round(zzmax))
  if zzmin lt 0. and zzmax gt 0. then zzmaxs='+'+zzmaxs
  zzmins=num2str(round(zzmin))
  zdmaxs=num2str(round(zdmax))
  if zdmin lt 0. and zdmax gt 0. then zdmaxs='+'+zdmaxs
  zdmins=num2str(round(zdmin))
  ldmaxs=num2str(round(ldmax))
  if ldmin lt 0. and ldmax gt 0. then ldmaxs='+'+ldmaxs
  ldmins=num2str(round(ldmin))
  czmaxs=num2str(round(czmax))
  if czmin lt 0. and czmax gt 0. then czmaxs='+'+czmaxs
  czmins=num2str(round(czmin))

  ; Open WCR file

  wcrdispopen,fn,data,fsize,/firstopen
  if destroym ne 0B then begin
     return
     free_lun,lun1
  endif

; .........................................................................

NEWFILE:     ; start new wcr file here (if requested by check_ev)

  ysize=rangegates>minimgsz
  calc =-99. ; current H leak, also flags new file for wcrdispread
  vleak=-99. ; current V leak
  mnhn =-99. ; initilize mean H noise
  mxhn =-99. ; initilize max H noise
  mnvn =-99. ; initilize mean V noise
  mxvn =-99. ; initilize max V noise
  timeint = 0L ; initilize dT (mean time interval for every display loop)
  timeintm= 0L ; initilize max dT

  ; Default min/max Doppler velocity and Doppler width

    if ppind[0] ge 0 then dvhmax = maxvel[ppind[0]] $
    else dvhmax=maxvel[0]                          ; mean vel, m/s
    if ppind[1] ge 0 then dvvmax = maxvel[ppind[1]] $
    else dvvmax=maxvel[0]
    dvhmin = -dvhmax                ; mean vel, m/s
    dvvmin = -dvvmax                ; mean vel, m/s
    svmax = maxvel[0]               ; vel std,  m/s
    svmin = 0                       ; vel std,  m/s

  ; Convert min/max Doppler to strings for widget use

    dvhmaxs=num2str(round(dvhmax))
    dvvmaxs=num2str(round(dvvmax))
    if dvhmin lt 0. and dvhmax gt 0. then dvhmaxs='+'+dvhmaxs
    if dvvmin lt 0. and dvvmax gt 0. then dvvmaxs='+'+dvvmaxs
    dvhmins=num2str(round(dvhmin))
    dvvmins=num2str(round(dvvmin))

    svmaxs=num2str(round(svmax))
    if (dvhmin lt 0. or dvvmin lt 0.) and svmax gt 0. then svmaxs='+'+svmaxs
    svmins=num2str(round(svmin))

; Set up or update 1/2 scrolling window widget

  if openwf eq 0B then begin             ; first open: build and realize widget
     if rangegates gt maxrg then nimg=1
     wcrdispwidget
     widget_control,baseid,/realize
                         ; remove logical op when antconf 3 is available
     widget_control,antid,set_droplist_select=antconf-(antconf gt 3)
  endif else begin
     if (rgold ne rangegates) then begin ; change in range gates: rebuild widget
        widget_control,baseid,/destroy
        if rangegates gt maxrg then nimg=1
        wcrdispwidget
        widget_control,baseid,/realize
                          ; remove logical op when antconf 3 is available
        widget_control,antid,set_droplist_select=antconf-(antconf gt 3)
     endif else begin                    ; use existing widget: initilize
        widget_control,hh1id,sensitive=1
        widget_control,vv1id,sensitive=1
        widget_control,zdr1id,sensitive=1
        widget_control,ldr1id,sensitive=1
        widget_control,vh1id,sensitive=1
        widget_control,dvh1id,set_value=dvlabel[0]
        widget_control,dvv1id,set_value=dvlabel[1]
        widget_control,dvh1id,sensitive=1
        widget_control,dvv1id,sensitive=1
        widget_control,aveid,set_value=num2str(avestep)
        widget_control,statmnhnid,set_value='-99.9'
        widget_control,statmxhnid,set_value='-99.9'
        widget_control,statmnvnid,set_value='-99.9'
        widget_control,statmxvnid,set_value='-99.9'
        widget_control,stathleakid,set_value='-99.9'
        widget_control,statvleakid,set_value='-99.9'
        widget_control,statdtid,set_value='0'
        widget_control,startid,set_value=starttime
        if nimg eq 2 then begin
           widget_control,hh2id,sensitive=1
           widget_control,vv2id,sensitive=1
           widget_control,zdr2id,sensitive=1
           widget_control,ldr2id,sensitive=1
           widget_control,vh2id,sensitive=1
           widget_control,dvh2id,set_value=dvlabel[0]
           widget_control,dvv2id,set_value=dvlabel[1]
           widget_control,dvh2id,sensitive=1
           widget_control,dvv2id,sensitive=1
        endif
    endelse
  endelse

  ; Set Doppler velocity correction menu according to velcor and gpsacc

  widget_control,dvcid,set_droplist_select=velcor+gpsacc*(velcor eq 1)

  ; Now that the widget is created set appropriate button sensitivity

  if rangegates gt maxrg then widget_control,nimgid,sensitive=0

  widget_control,dvid,set_value=dvlabels
  if n_elements(dvlabels) eq 1 then begin
    widget_control,dvid,sensitive=0
    if velcor eq 0 then widget_control,thackid,sensitive=0 $
    else widget_control,thackid,sensitive=1
  endif

  if alg eq sppalg or alg eq fftalg then begin  ; SPPMAG or FFT
     widget_control,zdr1id,sensitive=0
     if txstrpol[0] eq 'H' then begin           ; HHHH
        widget_control,vv1id,sensitive=0
        if velcor eq 0 then widget_control,dvv1id,sensitive=0
        widget_control,vh1id,set_value='vh'
     endif else begin                           ; VVVV
        widget_control,hh1id,sensitive=0
        if velcor eq 0 then widget_control,dvh1id,sensitive=0
        widget_control,vh1id,set_value='hv'
     endelse
     if alg eq fftalg then begin            ; for FFT remove ldr, use vh for sv
        widget_control,ldr1id,sensitive=0
        widget_control,vh1id,set_value='sv'
     endif
     if nimg eq 2 then begin                ; repeat above for image2 (bottom)
        widget_control,zdr2id,sensitive=0
        if txstrpol[0] eq 'H' then begin
           widget_control,vv2id,sensitive=0
           if velcor eq 0 then widget_control,dvv2id,sensitive=0
           widget_control,vh2id,set_value='vh'
        endif else begin
           widget_control,hh2id,sensitive=0
           if velcor eq 0 then widget_control,dvh2id,sensitive=0
           widget_control,vh2id,set_value='hv'
        endelse
        if alg eq fftalg then begin
           widget_control,ldr2id,sensitive=0
           widget_control,vh2id,set_value='sv'
        endif
     endif
  endif

  ; For dual antenna configurations dissable all cross channels products

    if antconf gt 0 then begin
      widget_control,ldr1id,sensitive=0
      widget_control,zdr1id,sensitive=0
      if alg ne fftalg then widget_control,vh1id,sensitive=0
      if nimg eq 2 then begin
        widget_control,ldr2id,sensitive=0
        widget_control,zdr2id,sensitive=0
        if alg ne fftalg then widget_control,vh2id,sensitive=0
      endif
    endif

  ; Setup wcr info text

    if alg eq fftalg then begin
      prfs=num2str(prf[0]/1000,fr=0)
      widget_control,fileid,set_value=wcrfile+'  /FFT: '+txstrpol[0]+'/'+ $
             strmid(prfs,0,2)+'KHz/'+num2str(maxvel[0],fr=1)+'m/s'
    endif else begin
      prfs=velfactor*!pi
      strpols=string(reform(byte(txstrpol)))
      prfs=string(reform(byte(num2str(prfs,fr=1,len=4)+','),5*n_elements(prfs)))
      widget_control,fileid, set_value=wcrfile+'  /'+strpols+', maxvel:'+$
        prfs+' #av:'+num2str(averages)+', pulse:'+num2str(pulselength)+'/'
    endelse

  ; Get window ids and draw color bars

    widget_control,axis1id,get_value=axisw1
    widget_control,scroll1id,get_value=scrollw1
    widget_control,bar1id,get_value=barw1
    wset,barw1
    if image1 eq 'du_h' or image1 eq 'dc_h' or image1 eq 'dg_h' then begin
            wcrdispbar,(ct2ind lt !d.table_size)+1,                        $
                        ct2ind,rangegates,bxsize,vmin=dvhmin,vmax=dvhmax
            widget_control,b1minid,set_value=dvhmins
            widget_control,b1maxid,set_value=dvhmaxs
    endif else if image1 eq 'du_v' or image1 eq 'dc_v' or image1 eq 'dg_v' $
    then begin
            wcrdispbar,(ct2ind lt !d.table_size)+1,                        $
                     ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
            widget_control,b1minid,set_value=dvvmins
            widget_control,b1maxid,set_value=dvvmaxs
    endif else if image1 eq 'sv' then begin
            wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax
            widget_control,b1minid,set_value=svmins
            widget_control,b1maxid,set_value=svmaxs
    endif else if image1 eq 'zdr' then begin                                    $
            wcrdispbar,(ct2ind lt !d.table_size)+1,                        $
                        ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax
            widget_control,b1minid,set_value=zdmins
            widget_control,b1maxid,set_value=zdmaxs
    endif else if image1 eq 'hh' or image1 eq 'vv' then begin
            wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
            widget_control,b1minid,set_value=zzmins
            widget_control,b1maxid,set_value=zzmaxs
    endif else if image1 eq 'ldr' then begin
            wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax
            widget_control,b1minid,set_value=ldmins
            widget_control,b1maxid,set_value=ldmaxs
    endif else if image1 eq 'vh' then begin
            wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
            widget_control,b1minid,set_value=czmins
            widget_control,b1maxid,set_value=cmmaxs
    endif

    if nimg eq 2 then begin
      widget_control,axis2id,get_value=axisw2
      widget_control,scroll2id,get_value=scrollw2
      widget_control,bar2id,get_value=barw2
      wset,barw2
      if image2 eq 'du_h' or image2 eq 'dc_h' or image2 eq 'dg_h' then begin
            wcrdispbar,(ct2ind lt !d.table_size)+1,                         $
                        ct2ind,rangegates,bxsize,vmin=dvhmin,vmax=dvhmax
            widget_control,b2minid,set_value=dvhmins
            widget_control,b2maxid,set_value=dvhmaxs
      endif else if image2 eq 'du_v' or image2 eq 'dc_v' or image2 eq 'dg_v'$
      then begin
            wcrdispbar,(ct2ind lt !d.table_size)+1,                         $
                        ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
            widget_control,b2minid,set_value=dvvmins
            widget_control,b2maxid,set_value=dvvmaxs
      endif else if image2 eq 'sv' then begin
            wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax
            widget_control,b2minid,set_value=svmins
            widget_control,b2maxid,set_value=svmaxs
      endif else if image2 eq 'zdr' then begin
            wcrdispbar,(ct2ind lt !d.table_size)+1,                         $
                        ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax
            widget_control,b2minid,set_value=zdmins
            widget_control,b2maxid,set_value=zdmaxs
      endif else if image2 eq 'hh' or image2 eq 'vv' then begin
            wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
            widget_control,b2minid,set_value=zzmins
            widget_control,b2maxid,set_value=zzmaxs
      endif else if image2 eq 'ldr' then begin
            wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax
            widget_control,b2minid,set_value=ldmins
            widget_control,b2maxid,set_value=ldmaxs
      endif else if image2 eq 'vh' then begin
            wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
            widget_control,b2minid,set_value=czmins
            widget_control,b2maxid,set_value=cmmaxs
      endif
    endif

  ; Set the scroll bars to the right

    widget_control,scroll1id,set_draw_view=[maximgsz,0]
    if nimg eq 2 then widget_control,scroll2id,set_draw_view=[maximgsz,0]

  ; Plot y-axes in image1/image2 axis windows

    if (openwf eq 0B) or (openwf eq 1B and rgold ne rangegates) or $
       (rgsold ne rangegatespacing) or (rgdold ne rangegatedelay) then begin

       widget_control,axis1id,get_value=axisw1
       wset, axisw1 & erase    ; go to axis top window

       ;   Establish plot parameters; y-range is in kft

       junkid=((minimgsz-rangegates)*rangegatespacing)>0
       ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
               junkid*(img1order eq 1))/1000.
       plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
          xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
       xl_cm=!d.x_size/!d.x_px_cm
       tkly=-1./xl_cm          ; reasonable tick length

       ;   Plot axis in kft and in km after establishing new plot parameters

       axis,0.45,ax_yrange[0]/0.305,/nodat,ytickle=tkly,yaxis=0,ystyle=1, $
         yrange=!y.crange*(img1order eq 0)+reverse(!y.crange)*(img1order eq 1),$
        chars=1.1,/normal
       plot,ax_yrange,/nodata,/noerase,/normal, $
          xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
       axis,1.0,ax_yrange[0],/nodata,yticklen=tkly,yaxis=0,ystyle=1, $
         yrange=!y.crange*(img1order eq 0)+reverse(!y.crange)*(img1order eq 1),$
        chars=1.1,/normal

       if nimg eq 2 then begin  ; for image 2 repeat the above
          widget_control,axis2id,get_value=axisw2
        wset, axisw2 & erase
        ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
           junkid*(img2order eq 1))/1000.
        axis,1.0,ax_yrange[0],/nodat,yticklen=tkly,yaxis=0,ystyle=1, $
            yrange=!y.crange*(img2order eq 0)+ $
           reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal
         plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
            xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
        axis,0.45,ax_yrange[0]/0.305,/nodata,yticklen=tkly,yaxis=0,ystyle=1,$
            yrange=!y.crange*(img2order eq 0)+ $
           reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal
       endif
    endif

  ; Set exclusive buttons

    if velcor then junkid=widget_info(dvid,/droplist_select) else junkid=0

    case image1 of
      'hh'  : widget_control,hh1id,set_button=1
      'vv'  : widget_control,vv1id,set_button=1
      'du_h': if junkid eq 2 or $
                (junkid eq 0 and (alg eq sppalg or alg eq fftalg)) then $
                widget_control,dvv1id,set_button=1 $
              else widget_control,dvh1id,set_button=1
      'du_v': widget_control,dvv1id,set_button=1
      'dc_h': widget_control,dvh1id,set_button=1
      'dc_v': if junkid eq 3 or $
                (junkid eq 0 and (alg eq sppalg or alg eq fftalg)) then $
                widget_control,dvh1id,set_button=1 $
              else widget_control,dvv1id,set_button=1
      'dg_h': widget_control,dvh1id,set_button=1
      'dg_v': if junkid eq 3 or $
                (junkid eq 0 and (alg eq sppalg or alg eq fftalg)) then $
                widget_control,dvh1id,set_button=1 $
              else widget_control,dvv1id,set_button=1
      'zdr' : widget_control,zdr1id,set_button=1
      'ldr' : widget_control,ldr1id,set_button=1
      'vh'  : widget_control,vh1id,set_button=1
      'sv'  : widget_control,vh1id,set_button=1
    endcase

    if nimg eq 2 then $
      case image2 of
        'hh'  : widget_control,hh2id,set_button=1
        'vv'  : widget_control,vv2id,set_button=1
        'du_h': if junkid eq 2 or $
                (velcor and junkid eq 0 and (alg eq sppalg or alg eq fftalg)) $
                then widget_control,dvv2id,set_button=1 $
                else widget_control,dvh2id,set_button=1
        'du_v': widget_control,dvv2id,set_button=1
        'dc_h': widget_control,dvh2id,set_button=1
        'dc_v': if junkid eq 3 or $
                (velcor and junkid eq 0 and (alg eq sppalg or alg eq fftalg)) $
                then widget_control,dvh2id,set_button=1 $
                else widget_control,dvv2id,set_button=1
        'dg_h': widget_control,dvh2id,set_button=1
        'dg_v': if junkid eq 3 or $
                (velcor and junkid eq 0 and (alg eq sppalg or alg eq fftalg)) $
                then widget_control,dvh2id,set_button=1 $
                else widget_control,dvv2id,set_button=1
        'zdr' : widget_control,zdr2id,set_button=1
        'ldr' : widget_control,ldr2id,set_button=1
        'vh'  : widget_control,vh2id,set_button=1
        'sv'  : widget_control,vh2id,set_button=1
      endcase

if openwf eq 0 then print,'wcrdisplay in progress ...'
; .........................................................................

; Start the main loop

  ; Initilize

  openwf  =0B
  destroyw=0B
  destroym=0B
  catche  =0B
  dispc   =0B
  rgold   =rangegates
  rgsold  =rangegatespacing
  rgdold  =rangegatedelay
  img1    =bytarr(maximgsz,rangegates)
  img2    =bytarr(maximgsz,rangegates)

  LOOP1: starte=0B                ; restart main loop here if new InitTime
         kstrt   =long(0 + strtp)
         kstep=loopstep*avestep
       ; If start profile exceeds existing profiles zero it.  KSTRT is NOT
       ; initialized in order to allow widget openning and position at the end
       ; of the file waiting for data

       if strtp ge (numrec/kstep-2)*kstep-1 then strtp=0L

  LOOP2: ;restart main loop here if new data or display control action requested

       kstep=loopstep*avestep
       kstop=(numrec/kstep-1)*kstep-1

       ; If kstrt > kstop in idl 'for' loop won't execute the body
       ; If kstrt+kstep > kstop skip the loop and wait for more data

         if (kstop lt kstrt+kstep) then begin     ; avoid insufficient data
            k=kstrt
            goto, CATCH
         endif

     ; widget_control,pauseid,sensitive=1       ; restore pause button
       widget_control,catchid,sensitive=1       ; restore catch button

  for k=kstrt,kstop,kstep do begin

  ; ##########################################################################
  ; Update time and current profiles being processed

    dispitime=wcrdisptime(data,k,strtp)
    widget_control,btlabelid,set_value=dispitime
    widget_control,statusmsgid,set_value='Processing profiles '+ $
            num2str(k)+' to '+num2str(k+kstep)

  ; ##########################################################################
  ; Read and calibrate current loopstep*avestep profiles of wcr data

    wcrdispread,k,calc,mnhn,mnvn

  ; ##########################################################################
  ; Scale requested image1 and image2

    i2s=1.
    if antconf eq 4 then begin
      if ((image1 eq 'dc_h' or image1 eq 'dg_h' or image1 eq 'du_h') and $
         (image2 eq 'dc_v' or image2 eq 'dg_v' or image2 eq 'du_v')) or  $
         ((image1 eq 'dc_v' or image1 eq 'dg_v' or image1 eq 'du_v') and $
         (image2 eq 'dc_h' or image2 eq 'dg_h' or image2 eq 'du_h'))     $
      then i2s=-1.
    endif

    case image1 of
       'hh'  : img1=bytscl(hhz,max=zzmax,min=zzmin,top=ct2ind-1)
       'vv'  : img1=bytscl(vvz,max=zzmax,min=zzmin,top=ct2ind-1)
       'du_h': img1=bytscl(duh,max=dvhmax,min=dvhmin,top=ct2ind-1)+         $
                    ct2ind*(ct2ind lt !d.table_size)
       'du_v': begin
                 img1=bytscl(i2s*duv,max=dvvmax,min=dvvmin,top=ct2ind-1)+   $
                      ct2ind*(ct2ind lt !d.table_size)
                 if antconf eq 4 then begin
                   wset,barw1
                   wcrdispbar,(ct2ind lt !d.table_size)+1,order=(i2s eq -1),$
                      ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
                 endif
               end
       'dc_h': img1=bytscl(dch,max=dvhmax,min=dvhmin,top=ct2ind-1)+ $
                    ct2ind*(ct2ind lt !d.table_size)
       'dc_v': begin
                 img1=bytscl(i2s*dcv,max=dvvmax,min=dvvmin,top=ct2ind-1)+   $
                      ct2ind*(ct2ind lt !d.table_size)
                 if antconf eq 4 then  begin
                   wset,barw1
                   wcrdispbar,(ct2ind lt !d.table_size)+1,order=(i2s eq -1),$
                      ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
                 endif
               end
       'dg_h': img1=bytscl(dch,max=dvhmax,min=dvhmin,top=ct2ind-1)+         $
                    ct2ind*(ct2ind lt !d.table_size)
       'dg_v': begin
                 img1=bytscl(i2s*dcv,max=dvvmax,min=dvvmin,top=ct2ind-1)+   $
                      ct2ind*(ct2ind lt !d.table_size)
                 if antconf eq 4 then begin
                   wset,barw1
                   wcrdispbar,(ct2ind lt !d.table_size)+1,order=(i2s eq -1),$
                      ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
                 endif
               end
       'sv'  : img1=bytscl(vhz,max=svmax,min=svmin,top=ct2ind-1)
       'zdr' : img1=bytscl(zdr,max=zdmax,min=zdmin,top=ct2ind- $
                    (ct2ind eq !d.table_size))+ct2ind*(ct2ind lt !d.table_size)
       'ldr' : img1=bytscl(ldr,max=ldmax,min=ldmin,top=ct2ind- $
                    (ct2ind eq !d.table_size))
       'vh'  : img1=bytscl(vhz,max=czmax,min=czmin,top=ct2ind-1)
    endcase

    if nimg eq 2 then begin
      case image2 of
       'hh'  : img2=bytscl(hhz,max=zzmax,min=zzmin,top=ct2ind-1)
       'vv'  : img2=bytscl(vvz,max=zzmax,min=zzmin,top=ct2ind-1)
       'du_h': img2=bytscl(duh,max=dvhmax,min=dvhmin,top=ct2ind-1)+         $
                    ct2ind*(ct2ind lt !d.table_size)
       'du_v': begin
                 img2=bytscl(i2s*duv,max=dvvmax,min=dvvmin,top=ct2ind-1)+   $
                      ct2ind*(ct2ind lt !d.table_size)
                 if antconf eq 4 then begin
                   wset,barw2
                   wcrdispbar,(ct2ind lt !d.table_size)+1,order=(i2s eq -1),$
                      ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
                 endif
               end
       'dc_h': img2=bytscl(dch,max=dvhmax,min=dvhmin,top=ct2ind-1)+ $
                    ct2ind*(ct2ind lt !d.table_size)
       'dc_v': begin
                 img2=bytscl(i2s*dcv,max=dvvmax,min=dvvmin,top=ct2ind-1)+   $
                      ct2ind*(ct2ind lt !d.table_size)
                 if antconf eq 4 then begin
                   wset,barw2
                   wcrdispbar,(ct2ind lt !d.table_size)+1,order=(i2s eq -1),$
                      ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
                 endif
               end
       'dg_h': img2=bytscl(dch,max=dvhmax,min=dvhmin,top=ct2ind-1)+ $
                    ct2ind*(ct2ind lt !d.table_size)
       'dg_v': begin
                 img2=bytscl(i2s*dcv,max=dvvmax,min=dvvmin,top=ct2ind-1)+   $
                      ct2ind*(ct2ind lt !d.table_size)
                 if antconf eq 4 then begin
                   wset,barw2
                   wcrdispbar,(ct2ind lt !d.table_size)+1,order=(i2s eq -1),$
                      ct2ind,rangegates,bxsize,vmin=dvvmin,vmax=dvvmax
                 endif
               end
       'sv'  : img2=bytscl(vhz,max=svmax,min=svmin,top=ct2ind-1)
       'zdr' : img2=bytscl(zdr,max=zdmax,min=zdmin,top=ct2ind- $
                    (ct2ind eq !d.table_size))+ct2ind*(ct2ind lt !d.table_size)
       'ldr' : img2=bytscl(ldr,max=ldmax,min=ldmin,top=ct2ind- $
                    (ct2ind eq !d.table_size))
       'vh'  : img2=bytscl(vhz,max=czmax,min=czmin,top=ct2ind-1)
       else  :
      endcase
    endif

  ; ##########################################################################
  ; Plot image1 and image2

    wset, scrollw1
    tv, img1,order=img1order
    if nimg eq 2 then begin
       wset, scrollw2
       tv, img2,order=img2order
    endif

  ; ##########################################################################
  ; Update end-time and NOISE/LEAK display
  ; Internal calibration was disabled (ver. 7.2) and then restored(RCS v.3.0)
  ; Currently status display is modified to display noise,leak and time
  ; interval. calc, mnhn,mxhn, mnvn,mxvn, timeint,timeintm are initialized with
  ; a new file.

    dispetime=wcrdisptime(data,(k+kstep)<(kstop-1))
    widget_control,etlabelid,set_value=dispetime

    widget_control,stathleakid,set_value=strcompress(calc,/remove_all)
    widget_control,statvleakid,set_value=strcompress(vleak,/remove_all)
    widget_control,statmnhnid,set_value=strcompress(mnhn,/remove_all)
    widget_control,statmnvnid,set_value=strcompress(mnvn,/remove_all)
    if mnhn gt mxhn then begin
      mxhn=mnhn
      widget_control,statmxhnid,set_value=strcompress(mxhn,/remove_all)
    endif
    if mnvn gt mxvn then begin
      mxvn=mnvn
      widget_control,statmxvnid,set_value=strcompress(mxvn,/remove_all)
    endif
    if timeint gt timeintm then begin
      timeintm=timeint
      widget_control,statdtid,set_value=strcompress(timeintm,/remove_all)
    endif

  ; ##########################################################################
  ; Register coming events

    junkid=widget_event(baseid,/nowait)

  ; ##########################################################################
  ; If Pause is requested stall the images and wait for Continue button
  ; Give control to the mouse for image cursor trace

    widget_control,pauseid,get_value=bl
    if bl eq 'Continue' then begin
       widget_control,statusmsgid,set_value='Last Prof:'+num2str(k+kstep)+ $
       '-> Pause ...',/no_copy

    ;  Initiate mouse events

       widget_control,scroll1id,/draw_button_events,event_pro='scrollw1_ev',/no_copy
       if nimg eq 2 then $
       widget_control,scroll2id,/draw_button_events,event_pro='scrollw2_ev',/no_copy

    ;  Establish plot parameters for cursor marks

       plot,[0,0],/nodata,/noerase,yrange=[0,(rangegates-1)>(minimgsz-1)], $
            xrange=[0,maximgsz-1],xstyle=5,ystyle=5,xmargin=[0,0],ymargin=[0,0]

    ;  Wait for a mouse event from draw widgets or Continue button

       while bl eq 'Continue' do begin
         junkid=widget_event(baseid,/nowait)
         widget_control,pauseid,get_value=bl,/no_copy
         wait,0.1  ; sleep 100ms minimizes a memory leak problem under Linux
                   ; widget_control,..,get_value is the source (reported to
                   ; RSI, Nov 2001

         if scrollw1e ne 0 and ivalue lt rangegates then begin
           if img1order eq 1 then $
             junkid=rangegates-ivalue-1 else junkid=ivalue

           case image1 of
             'hh'  : ivalue1=hhz(itime,junkid)
             'vv'  : ivalue1=vvz(itime,junkid)
             'du_h': ivalue1=duh(itime,junkid)
             'du_v': ivalue1=duv(itime,junkid)
             'dc_h': ivalue1=dch(itime,junkid)
             'dc_v': ivalue1=dcv(itime,junkid)
             'dg_h': ivalue1=dch(itime,junkid)
             'dg_v': ivalue1=dcv(itime,junkid)
             'zdr' : ivalue1=zdr(itime,junkid)
             'ldr' : ivalue1=ldr(itime,junkid)
             'vh'  : ivalue1=vhz(itime,junkid)
             'sv'  : ivalue1=vhz(itime,junkid)
           endcase
           ivaluec='Img1:'+num2str(ivalue1,fr=1)

        ;  Determine the color for the cross (black: color table upper half;
        ;  white: color table lower half) and plot it

           widget_control,b1maxid,get_value=temp & bmaxc=(float(temp))(0)
           widget_control,b1minid,get_value=temp & bminc=(float(temp))(0)
           if ivalue1 lt bminc+(bmaxc-bminc)/2. then xcol=255B $
           else xcol=0B
           wset, scrollw1
           oplot,[itime],[ivalue], psym=1,thick=2.,color=xcol
           ; It appears that print or empty are not needed for idl5.5 but
           ; is needed for the second image (nimg eq 2)
           ; The following line was replaced with empty command
           ; PRINT,format='($,"")' ; this line solves an idl problem that
                                   ; causes oplot activity to be finished
                                   ; after next I/O operation (for single
                                   ; image mode)
           ; empty

           ; If image2 is present mark matching location and get the value

           if nimg eq 2 and ivalue lt rangegates and img1order eq img2order $
           then begin

             case image2 of
               'hh'  : ivalue2=hhz(itime,junkid)
               'vv'  : ivalue2=vvz(itime,junkid)
               'du_h': ivalue2=duh(itime,junkid)
               'du_v': ivalue2=i2s*duv(itime,junkid)
               'dc_h': ivalue2=dch(itime,junkid)
               'dc_v': ivalue2=i2s*dcv(itime,junkid)
               'dg_h': ivalue2=dch(itime,junkid)
               'dg_v': ivalue2=i2s*dcv(itime,junkid)
               'zdr' : ivalue2=zdr(itime,junkid)
               'ldr' : ivalue2=ldr(itime,junkid)
               'vh'  : ivalue2=vhz(itime,junkid)
               'sv'  : ivalue2=vhz(itime,junkid)
             endcase
             ivaluec=ivaluec+' Img2:'+num2str(ivalue2,fr=1)
             widget_control,b2maxid,get_value=temp & bmaxc=(float(temp))(0)
             widget_control,b2minid,get_value=temp & bminc=(float(temp))(0)
             if ivalue2 lt bminc+(bmaxc-bminc)/2. then xcol=255B $
             else xcol=0B
             wset, scrollw2
             oplot,[itime],[ivalue],psym=5,thick=2,col=xcol
             ; The following line was repalced with empty command
             ; PRINT,format='($,"")' ; this line solves an idl problem that
                                     ; causes oplot activity to be finished
                                     ; after next I/O operation
             empty
           endif

        ;  Get time in hh:mm:ss
        ;  To reduce the effect of the rounding error in timeinterval
        ;  use dispitime for 1st half of the image and dispetime for 2nd half

           if (itime gt maximgsz/2) then $
             itime=timestr2sec(dispetime)-$
                   (maximgsz-1-itime)*avestep*timeinterval $
           else itime=timestr2sec(dispitime)+itime*avestep*timeinterval

        ;  Take care if itime crosses midnight

           if itime lt 0. then itime=itime+86400. $
           else if itime gt 86400. then itime=itime-86400.
           itime=sec2timestr(itime)

        ;  Put the information on the status line

           widget_control,statusmsgid,set_value=ivaluec+'; T/R: '+itime+'/'+$
            num2str((junkid*rangegatespacing+rangegatedelay)/1000.,fr=2),/no_copy
           scrollw1e=0B

         endif else if scrollw2e and ivalue lt rangegates then begin
           ; mouse request from 2nd image
           if img2order eq 1 then $
             junkid=rangegates-ivalue-1 else junkid=ivalue

           case image2 of
            'hh'  : ivalue2=hhz(itime,junkid)
            'vv'  : ivalue2=vvz(itime,junkid)
            'du_h': ivalue2=duh(itime,junkid)
            'du_v': ivalue2=duv(itime,junkid)
            'dc_h': ivalue2=dch(itime,junkid)
            'dc_v': ivalue2=dcv(itime,junkid)
            'dg_h': ivalue2=dch(itime,junkid)
            'dg_v': ivalue2=dcv(itime,junkid)
            'zdr' : ivalue2=zdr(itime,junkid)
            'ldr' : ivalue2=ldr(itime,junkid)
            'vh'  : ivalue2=vhz(itime,junkid)
            'sv'  : ivalue2=vhz(itime,junkid)
           endcase
           ivaluec='Img2:'+num2str(ivalue2,fr=1)

        ;  Determine the color for the cross (black: color table upper half;
        ;  white: color table lower half) and plot it

           widget_control,b2maxid,get_value=temp & bmaxc=(float(temp))[0]
           widget_control,b2minid,get_value=temp & bminc=(float(temp))[0]
           if ivalue2 lt bminc+(bmaxc-bminc)/2. then xcol=255B $
           else xcol=0B
           wset, scrollw2
           oplot,[itime],[ivalue], psym=1,thick=2.,color=xcol

           ; If img1order eq img2order mark the corresponding spot on image1

           if img1order eq img2order then begin 
             case image1 of
              'hh'  : ivalue1=hhz(itime,junkid)
              'vv'  : ivalue1=vvz(itime,junkid)
              'du_h': ivalue1=duh(itime,junkid)
              'du_v': ivalue1=duv(itime,junkid)
              'dc_h': ivalue1=dch(itime,junkid)
              'dc_v': ivalue1=dcv(itime,junkid)
              'dg_h': ivalue1=dch(itime,junkid)
              'dg_v': ivalue1=dcv(itime,junkid)
              'zdr' : ivalue1=zdr(itime,junkid)
              'ldr' : ivalue1=ldr(itime,junkid)
              'vh'  : ivalue1=vhz(itime,junkid)
              'sv'  : ivalue1=vhz(itime,junkid)
             endcase

             ivaluec='Img1:'+num2str(ivalue1,fr=1)+' '+ivaluec

             widget_control,b1maxid,get_value=temp,/no_copy
             bmaxc=(float(temp))[0]
             widget_control,b1minid,get_value=temp,/no_copy
             bminc=(float(temp))[0]
             if ivalue1 lt bminc+(bmaxc-bminc)/2. then xcol=255B $
             else xcol=0B
             wset, scrollw1
             oplot,[itime],[ivalue],psym=5,thick=2.,color=xcol
             empty

           endif

           if (itime gt maximgsz/2) then $
             itime=timestr2sec(dispetime)- $
                   (maximgsz-1-itime)*avestep*timeinterval $
           else itime=timestr2sec(dispitime)+itime*avestep*timeinterval
           if itime lt 0. then itime=itime+86400. $
           else if itime gt 86400. then itime=itime-86400.
           itime=sec2timestr(itime)

           widget_control,statusmsgid,set_value=ivaluec+'; T/R: '+itime+'/'+$
            num2str((junkid*rangegatespacing+rangegatedelay)/1000.,fr=2),/no_copy
           scrollw2e=0B
         endif
       endwhile
    endif

  ; ##########################################################################
  ; Check for events from the main buttons

    CHECKEVENT1: check_ev,fn,data,fsize,strtp

    ; If cancel from pickfile just continue

    if destroym eq 2B then begin openr,lun1,fn & destroym=0B

    ; Process events from main widget

    endif else if destroyw eq 1B or quitw eq 1B then return $
    else if openwf eq 1B then goto, NEWFILE $
    else if catche eq 1B then goto, CATCH $
    else if starte eq 1B then goto, LOOP1 $

    ; If a display control parameter is changed restart the main loop

    else if dispc eq 1B then begin
       kstrt=k+kstep>0L & dispc=0B & goto, LOOP2

    ; If waiting for data give control to main widget

    endif else if destroym eq 1B then begin
       junkid=widget_event(baseid,/nowait) & goto,CHECKEVENT1 & end

  endfor  ; end of main LOOP
; .........................................................................

  ; Save data pointer for possible next portion of data. If CATCH is requested
  ; go to the end of the currently available data. Note that in IDL the loop
  ; index variable after the end of the loop is equal to kstrt+kstep

CATCH:  if catche eq 1B then kstrt=numrec else kstrt = k > 0L
        fsizeold=fsize
        destroyw=0B
        catche=0B
        widget_control,catchid,sensitive=0

  ; Check for new data and process events from appropriate main buttons

    close,lun1
    openr,lun1,fn
    fstatus = fstat(lun1)
    fsize   = fstatus.size

    if fsize le fsizeold+minfsize then begin

    ;  No new data, wait sleept secs

     ; widget_control,pauseid,sensitive=0
       close,lun1
       widget_control,statusmsgid,set_value='End of file. Waiting for data ...'
       wait,sleept

    ;  Check for new data again

       NEWDATASTATUS:
         openr,lun1,fn
         fstatus = fstat(lun1)
         fsize   = fstatus.size
         close,lun1

         ; If no new data check for other events

         if fsize le fsizeold+minfsize then begin
         CHECKEVENT2: junkid=widget_event(baseid,/nowait)
            check_ev,fn,data,fsize,strtp
            if quitw eq 1B or destroyw eq 1B then return $
            else if openwf eq 1B then goto, NEWFILE $
            else if destroym eq 1B then goto, CHECKEVENT2 $
            else if destroym eq 2B then destroym=0B $
            else if starte eq 1B then begin openr,lun1,fn & goto, LOOP1 & end
            wait,sleept & goto,NEWDATASTATUS
         endif else openr,lun1,fn
    endif

    ; If new data exist go get them

    numrec=long(strtrim(fsize,2))/blocksize
    goto, LOOP2

end ; wcrdisplay
